\\ Copyright 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("interv.gp");
read("interv-wip.gp");
read("a346913-rows.gp");
kill(levels); \\  \ globals in a346913-rows.gp
kill(vpar);   \\  /

nearly_equal_epsilon = 1e-15;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) = {
  \\ print(type(x)," ",type(y)," ",x," ",y);
  my(d=x-y); \\ ensure compatible types
  if(type(x)=="t_MAT",x=Vec(x);y=Vec(y));
  if(type(x)=="t_VEC" || type(x)=="t_COL",
       sum(i=1,max(#x,#y), !nearly_equal(x[i],y[i],epsilon)) == 0,
     type(x)=="t_QUAD" || type(x)=="t_COMPLEX",
          nearly_equal(real(x-y),0,epsilon)
       && nearly_equal(imag(x-y),0,epsilon),
     abs(d) < epsilon);
}


\\-----------------------------------------------------------------------------
\\ A087803 cumulative num rooted trees

A087803(n) =
{
  n>=1 || error();
  vecsum(vpar_count_unlabelled_vector(n));
}
{
  my(v=OEIS_data("A087803"));
  check_equal(vector(#v,n,A087803(n)), v,
              "A087803 cumualtive rooted trees");
}
{
  levels=[1]; \\ global variable
  my(N=1,target=A087803(N));
  for(n=1,2000,
     #levels==N || error();
     my(prev_levels=levels);
     A346913_next_levels();
     if(n==target,
        check_equal(#levels,N+1,
                    "last of length at A087803");
        check_equal(prev_levels,vector(N,i,if(i==1,1,2)),
                    "last of this length");
        check_equal(levels,[1..N+1],
                    "first of next length");
        N++; target=A087803(N)));
}

\\-----------------------------------------------------------------------------
\\ levels_validate_siblings_are_descending()

is_levels_validate_siblings_are_descending(levels) =
{
  my(ename=0);
  iferr(levels_validate_siblings_are_descending(levels),e,
        ename=errname(e));
  if(ename==0, 1,
     ename=="e_USER", 0,
     error("oops"));
}
check_equal(is_levels_validate_siblings_are_descending([1,2,3,2,3,4]), 0);
check_equal(is_levels_validate_siblings_are_descending([1,2,2,3]), 0);
check_equal(is_levels_validate_siblings_are_descending([1,2,3,2]), 1);
check_equal(is_levels_validate_siblings_are_descending([1,2,3,3,4]), 0);

\\-----------------------------------------------------------------------------
\\ levels_next_sibling()

{
  for(rep=1,100,
     my(vpar=vpar_make_random_preorder(random(50),1), \\ forest
        levels=vpar_forest_to_tree_levels(vpar),
        vpar_tree=vpar_add_root_above_forest(vpar));
     check_equal(vpar_seq_preorder(vpar_tree), 
                 Vecsmall(concat(#vpar_tree,[1..#vpar_tree-1])));
     vpar_tree=vpar_relabel_preorder(vpar_tree);

     my(by_vpar=vpar_next_sibling_vector(vpar_tree),
        by_levels=vector(#levels,v,levels_next_sibling(levels,v)));
     check_equal(by_levels,by_vpar,
                 "levels_next_sibling() vs vpar_next_sibling_vector()"));
}


\\-----------------------------------------------------------------------------
\\ A346913  levels
\\ A346913_next_levels() iteration

\\ {
\\   my(got=[]);
\\   levels=[1];
\\   while(#got<85,
\\         got=concat(got,levels);
\\         A346913_next_levels());
\\   for(i=1,#got,print1(got[i],",");if(i==30||i==60,print()));
\\   print();
\\   quit;
\\ }
\\ with 0-based depth levels
\\ not in OEIS: 1,2,0,1,1,0,1,2,3,0,1,2,2,0,1,2,1,0,1,1,1,0,1,2,3,4

{
  my(want=OEIS_data("A346913"));
  my(got=[]);
  levels=[1];
  while(#got<#want,
        got=concat(got,levels);
        A346913_next_levels());
  check_equal(got,want);     
}
{
  \\ triangle samples
  my(want=[[ 1             ],
           [ 1, 2          ],
           [ 1, 2, 3       ],
           [ 1, 2, 2       ],
           [ 1, 2, 3, 4    ],
           [ 1, 2, 3, 3    ],
           [ 1, 2, 3, 2    ],
           [ 1, 2, 2, 2    ],
           [ 1, 2, 3, 4, 5 ],
           [ 1, 2, 3, 4, 4 ] ]);
  levels=[1];
  for(n=1,10,
     check_equal(levels,want[n], "triangle begins");
     A346913_next_levels());
}

level_sequence_of_vpar(vpar) = \
  apply(depth->depth+1, vpar_depths_vector(vpar));

{
  \\ versus vpar_unlabelled_next()
  my(bad=0);
  for(n=0,12,
     my(vpar=vpar_unlabelled_first(n));
     levels=[1..n]; \\ global variable
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           if(levels != level_sequence_of_vpar(vpar),
              bad++);
           A346913_next_levels()));
  check_equal(bad, 0, 
              "A346913_next_levels() vs vpar_unlabelled_next() trees");
}
{
  if(0,
     system("rm -f /tmp/b346913.txt");
     levels=[];
     my(idx=1);
     while(1,
           A346913_next_levels();
           if(!(#levels<=10), break);
           for(i=1,#levels, 
              write("/tmp/b346913.txt", Str(idx" "levels[i]));
              idx++));
     system("ls -l /tmp/b346913.txt");
     quit);

  \\ ~/OEIS/b346913.txt
  if(0,
     my(want=OEIS_data("A346913",'bfile));
     print("b346913.txt length ",#want);
     my(got=[],
        row_n=0);
     levels=[];
     while(#got<#want,
           A346913_next_levels(); row_n++;
           got=concat(got,levels);
           if(row_n==1, check_equal(levels,[1], "row n=1 singleton")));
     check_equal(got,want, "b346913.txt");     
    \\ check_equal(row_n,  1205, "b346913.txt size");
    \\ check_equal(#levels, 10, "b346913.txt num vertices");
    \\ A346913_next_levels();
    \\ check_equal(#levels, 11, "b346913.txt next num vertices");
     );
}

kill(levels);

\\-----------------------------------------------------------------------------
\\ A346914  vpar
\\ A346914_next_vpar() iteration

\\ {
\\   my(got=[]);
\\   vpar=[0];
\\   while(#got<85,
\\         got=concat(got,vpar);
\\         A346914_next_vpar());
\\   for(i=1,#got,print1(got[i],",");if(i==30||i==60,print()));
\\   print();
\\   quit;
\\ }
{
  my(want=OEIS_data("A346914"));
  my(got=[]);
  vpar=[0];
  while(#got<#want,
        got=concat(got,vpar);
        A346914_next_vpar());
  check_equal(got,want);     
}

{
  \\ same as vpar_unlabelled_next()
  my(bad=0);
  for(n=0,12,
     my(my_vpar=vpar_unlabelled_first(n));
     vpar=[0..n-1];  \\ global
     until((my_vpar=vpar_unlabelled_next(my_vpar,1)) == 'none,  \\ forests
           if(vpar != my_vpar,
              bad++);
           A346914_next_vpar());
     check_equal(#vpar,n+1, "vpar first of next length");
     check_equal(vpar,[0..n], "vpar first of next length");
     );
  check_equal(bad, 0, 
              "A346914_next_vpar() vs vpar_unlabelled_next() forests");
}

{
  if(0,
     system("rm -f /tmp/b346914.txt");
     vpar=[];
     my(idx=2);
     while(1,
           A346914_next_vpar();
           if(!(#vpar<=9), break);
           for(i=1,#vpar, 
              write("/tmp/b346914.txt", Str(idx" "vpar[i]));
              idx++));
     system("ls -l /tmp/b346914.txt");
     quit);

  \\ ~/OEIS/b346914.txt
  if(0,
     my(want=OEIS_data("A346914",'bfile));
     print("b346914.txt length ",#want);
     my(got=[],
        last_length='none,
        row_n=1);
     vpar=[];
     while(#got<#want,
           A346914_next_vpar(); row_n++;
           got=concat(got,vpar);
           if(row_n==2, check_equal(vpar,[0], "row n=2 singleton")));
     check_equal(got,want, "b346914.txt");     
     \\ check_equal(#vpar,  9,    "b346914.txt num vertices");
     \\ check_equal(row_n,  1205, "b346914.txt size");
     \\ check_equal(row_n,  vecsum(vpar_count_unlabelled_vector(9,1))
     \\                     + 1);  \\ because first forest is n=2
     \\ A346914_next_vpar();
     \\ check_equal(#vpar, 10, "b346914.txt next num vertices");
    );
}

kill(vpar);


\\-----------------------------------------------------------------------------
\\ Various Not in OEIS

\\ A000081
\\ vector(12,n, vecsum(apply(vpar_num_components,vpar_unlabelled_all(n,1))))
\\ A005197  1, 3, 7, 17, 39, 96, 232, 583, 1474, 3797, 9864, 25947

\\ {
\\   \\ row widths
\\   \\ not in OEIS: 1,2,3,3,4,4,4,4,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7
\\   \\ not in OEIS: 2,2,3,3,3,3,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,6
\\   levels=[1];
\\   for(n=1,50,
\\      print1(#levels-1,",");
\\      A346913_next_levels());
\\   print();
\\   quit;
\\ }

\\ {
\\   \\ not in OEIS: 3,2,4,3,2,2,5,4,3,2,3,2,3,2,2,6,5,4,3,2,4,3,2,4
\\   \\ not in OEIS: 2,1,3,2,1,1,4,3,2,1,2,1,2,1,1,5,4,3,2,1,3
\\   levels=[1..3];
\\   for(n=1,50,
\\      print1(levels[#levels]-1,",");
\\      A346913_next_levels());
\\   print();
\\ }

\\ {
\\   \\ trees or forests preorder vpar
\\   \\ not in OEIS: 0, 1, 0, 1, 2, 0, 1, 1, 0, 1, 2, 3, 0, 1, 2, 2, 0, 1, 2, 1, 0, 1, 1, 3, 0, 1, 1, 1
\\   \\ not in OEIS: 0, 1, 0, 0, 0, 1, 2, 0, 1, 1, 0, 1, 0, 0, 0, 2, 0, 0, 0
\\   \\ as depths
\\   \\ not in OEIS: 0, 1, 0, 1, 2, 0, 1, 1, 0, 1, 2, 3, 0, 1, 2, 2, 0, 1, 2, 1, 0, 1, 1, 2, 0, 1, 1, 1
\\   \\ not in OEIS: 0, 1, 0, 0, 0, 1, 2, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0
\\   \\ as levels
\\   \\ not in OEIS: 1, 2, 1, 2, 3, 1, 2, 2, 1, 2, 3, 4, 1, 2, 3, 3, 1, 2, 3, 2, 1, 2, 2, 3, 1, 2, 2, 2
\\   \\ not in OEIS: 1, 2, 1, 1, 1, 2, 3, 1, 2, 2, 1, 2, 1, 1, 1, 2, 1, 1, 1
\\   for(forest=0,1,
\\      my(got=[]);
\\      for(n=2,4-forest,
\\         my(vpar=vpar_preorder_first(n));
\\         until((vpar=vpar_preorder_next(vpar,forest))=='none,
\\               got=concat(got,apply(n->n+1,vpar_depths_vector(vpar)))));
\\      print(got));
\\   quit;
\\ }

\\ {
\\   \\ trees or forests preorder vpar, ascending
\\   \\ not in OEIS: 0, 1, 0, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 1, 1, 3, 0, 1, 2, 1, 0, 1, 2, 2, 0, 1, 2, 3
\\   \\ not in OEIS: 0, 0, 0, 1, 0, 0, 0, 0, 0, 2, 0, 1, 0, 0, 1, 1, 0, 1, 2
\\   \\ as depths
\\   \\ not in OEIS: 0, 1, 0, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 1, 1, 2, 0, 1, 2, 1, 0, 1, 2, 2, 0, 1, 2, 3
\\   \\ not in OEIS: 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 2
\\   \\ as levels
\\   \\ not in OEIS: 1, 2, 1, 2, 2, 1, 2, 3, 1, 2, 2, 2, 1, 2, 2, 3, 1, 2, 3, 2, 1, 2, 3, 3, 1, 2, 3, 4
\\   \\ not in OEIS: 1, 1, 1, 2, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 2, 2, 1, 2, 3
\\   \\ not A120698 initially same but differ at a(51)
\\   for(forest=0,1,
\\      my(got=[]);
\\      for(n=2,4-forest,
\\         my(vpar=vpar_preorder_last(n,forest));
\\         until((vpar=vpar_preorder_prev(vpar,forest))=='none,
\\               \\ got=concat(got,vpar);
\\               got=concat(got,apply(n->n+1,vpar_depths_vector(vpar)));
\\              ));
\\      print(got));
\\   quit;
\\ }

\\ A120698 restricted growth strings (set partitions)
\\ initially same as pre-order levels in ascending order,
\\ but then different
{
  my(want=OEIS_data("A120698"),
     forest=1,
     got=[]);
  for(n=1,#want,
     my(vpar=vpar_preorder_last(n,forest));
     until((vpar=vpar_preorder_prev(vpar,forest))=='none,
           if(#got>=#want,break(2));
           got=concat(got,apply(n->n+1,vpar_depths_vector(vpar)))));
  got=got[1..#want];  \\ truncate
  check_equal(got[1..50],want[1..50],
              "A120698 restricted growth strings start like pre-order reverse");
  check_equal(got[51] == want[51], 0,
              "A120698 restricted growth strings start then differ");

  \\ print(got-want);
  \\ print(select(n->got[n]!=want[n],[1..#want]));
}


\\-----------------------------------------------------------------------------

\\ Simon Plouffe
\\ ~/OEIS/treegrowth.txt in A051491
\\
{
  treegrowth_d_integer =
29557652856519949747148175241231945883754923046635965953504724789059647331395
749510866682836765813525306643713553062216140804716398086805528130252344795910
656450830094857724237733381960025839469497653106826655718270860682985427979012
044927318264877923165543441886049809053386951478126324251699809842476454117022
914498182207731305326691488822870879884674225079006972041559262826724346603865
511395549974595716724047293015669814373335868696232253972389009814080091539603
396562978537734246881433388503598376598762533529229147960470508680388449072017
852231661330812159293100901747448705408544599945314487758953479948204037497159
912479440501273250666044628064269870751284766131175952434919430350883603512578
989841518484797085223412953014018440496271680305050463592540513181033634182650
672066046380962799286758918257087669461349069339268320138893997266074178959247
586625205304377695494726802194622430816490355172525009444586418370221450429689
312742148243751820531421791396400211513114551580559206012994389002996432708915
094736117842404681882631543798228701299640222254254171936454319388908744845163
971249113539277459762993560177200108997444778629789111107961642283445260740253
854188508679691395461632459569732978287298742382144611656482195846528648947556
777368528640152334661600050980078324740613894003120675551874335356255280134247
326076167392305893027999116890124478263211972769332046786910115199877285353282
485397365366918738443264829773210413827145948737380235150426049008766459808935
445371969044928033412737087505186653313369634847150425719535240970539214056462
298983378711892653117304583435970188646592828031256172924819550419573417519525
259109945997412651726668095720265040399971791357211649473217196215499807167208
556195963320654326682124480373471898209405271292572318272871372593299040607263
2203163


  ;
}
check_equal(#digits(treegrowth_d_integer), 1800);

treegrowth_d_interval = \
  interv_frac_fromdigits(digits(treegrowth_d_integer));
print("treegrowth.txt     ",treegrowth_d_interval*1.0);
A051491_inexact = treegrowth_d_interval[1];

{
  my(v=OEIS_data("A051491"),
     t=interv_frac_fromdigits(v));
  print("A051491            ",t*1.0);
  check_equal(interv_contains(t,treegrowth_d_interval), 1);
}
{
  if(0,
     my(v=OEIS_data("A051491",'bfile),
        t=interv_frac_fromdigits(v));
     print("  b051491.txt ",#v,"  ",t*1.0);
     check_equal(interv_contains(t,treegrowth_d_interval), 1));
}

\\-----------------------------------------------------------------------------
\\ A346915 mems

{
  my(want=OEIS_data("A346915"));
  check_equal(#want,105);
  A346915_sample_interval = interv_frac_fromdigits(want);
  A346915_inexact = A346915_sample_interval[1];
}
{
  \\ 2 + 3/(A051491 - 1)
  my(t=interv_add_constant(treegrowth_d_interval, -1));
  t = 3*interv_recip(t);
  t=interv_add_constant(t, 2);
  print("mems calc            ",t*1.0);

  print("A346915 mems samples ",A346915_sample_interval*1.0);
  check_equal(interv_contains(A346915_sample_interval,t), 1);

  my(v=interv_frac_digits(t));
  check_equal(#v, 1799, "mems by treegrowth.txt num digits");
  \\ print(interv_frac_digits(t,,105)); quit;
  
  if(0,
     system("rm -f /tmp/b346915.txt");
     write("/tmp/b346915.txt",
           "# derived from Simon Plouffe's treegrowth.txt in A051491");
     for(n=1,#v,
             write("/tmp/b346915.txt", Str(n" "v[n])));
     system("ls -l /tmp/b346915.txt");
     quit);

  if(0,
     my(bfile_samps=OEIS_data("A346915",'bfile),
        A346915_bfile_interval=interv_frac_fromdigits(bfile_samps));
     print("b346915.txt length ",#bfile_samps);
     check_equal(interv_contains(A346915_bfile_interval,t), 1);

     \\ d back-calculated
     my(d2 = interv_add_constant(A346915_bfile_interval,-2));
     d2=3*interv_recip(d2);
     d2=interv_add_constant(d2,1);
     check_equal(interv_contains(d2,treegrowth_d_interval), 1);
     );
}

\\-----------------------------------------------------------------------------
\\ A346916 forest singletons limit
\\ 0.51130879934124

{
  my(want=OEIS_data("A346916"));
  check_equal(#want,105);
  A346916_samps_interval = interv_frac_fromdigits(want,,0);
  A346916_inexact = A346916_samps_interval[1];
}
{
  \\ 1/(A051491 - 1)
  my(t = interv_add_constant(treegrowth_d_interval, -1));
  t = interv_recip(t);
  print("S calc     ",t*1.0);
  print("S samples  ",A346916_samps_interval*1.0);
  check_equal(interv_contains(A346916_samps_interval,t), 1);

  my(v=interv_frac_digits(t));
  check_equal(v[1],0);  \\ integer part 0 for 0.xxx
  v=v[^1];
  check_equal(#v, 1798, "S mean singletons by treegrowth.txt, num digits");

  if(0,
     system("rm -f /tmp/b346916.txt");
     write("/tmp/b346916.txt",
           "# derived from Simon Plouffe's treegrowth.txt in A051491");
     for(i=1,#v,
        my(n=i-1);
        write("/tmp/b346916.txt", Str(n" "v[i])));
     system("ls -l /tmp/b346916.txt");
     quit);

  if(0,
     my(bfile_samps=OEIS_data("A346916",'bfile),
        A346916_bfile_interval=interv_frac_fromdigits(bfile_samps,,0));
     print("b346916.txt length ",#bfile_samps);
     check_equal(interv_contains(A346916_bfile_interval,t), 1);

     \\ d back-calculated
     my(d2=interv_recip(A346916_bfile_interval));
     d2=interv_add_constant(d2,1);
     check_equal(interv_contains(d2,treegrowth_d_interval), 1);
     );
}

\\ A000081 number of rooted unlabelled,
\\ but it recons none of n=0
A000081(n) =
{
  n>=0 || error();
  if(n==0,0, vpar_count_unlabelled(n));
}
OEIS_check_func("A000081");
{
  \\ more in ../examples/mean-unlabelled-singletons.gp
  check_equal(vector(20,n, A000081(n+1)),
              vector(20,n, vpar_count_unlabelled(n,1)),  \\ forests
              "A087803 count forests");
  
  my(N=29);
  print("S at N="N" is "A087803(N) / A000081(N+1) *1.0);
}

{
  \\ formulas mems
  check_equal(nearly_equal(A346915_inexact,
                           2 + 3/(A051491_inexact - 1)), 1);
  check_equal(nearly_equal(A346915_inexact,
                           3*A346916_inexact + 2), 1);

  \\ formulas S mean singletons
  check_equal(nearly_equal(A346916_inexact,
                           1/(A051491_inexact - 1)), 1);
  check_equal(nearly_equal(A346916_inexact,
                           (A346915_inexact - 2)/3), 1);
}


\\-----------------------------------------------------------------------------
\\ A346913_next_levels() examples

{
  levels=[1]; \\ global variable
  my(BH_before= [1,2,3,4,3,2,2,2,2,2,2,2],
     BH_after=  [1,2,3,4,2,3,4,2,3,4,2,3],
     my_example_levels=[ 1, 2,3,4,4,  2,3,4,3,  2,3,4,3,  2,3 ]);
  check_equal(#BH_before, 12);
  check_equal(#BH_before, #BH_after);

  my(seen=0);
  for(n=1,140193+1,
     if(n==1, check_equal(levels, [1]);   seen++);
     if(n==2, check_equal(levels, [1,2]); seen++);

     \\ example table
     if(n==1,  check_equal(levels, [ 1             ]));
     if(n==2,  check_equal(levels, [ 1, 2          ]));
     if(n==3,  check_equal(levels, [ 1, 2, 3       ]));
     if(n==4,  check_equal(levels, [ 1, 2, 2       ]));
     if(n==5,  check_equal(levels, [ 1, 2, 3, 4    ]));
     if(n==6,  check_equal(levels, [ 1, 2, 3, 3    ]));
     if(n==7,  check_equal(levels, [ 1, 2, 3, 2    ]));
     if(n==8,  check_equal(levels, [ 1, 2, 2, 2    ]));
     if(n==9,  check_equal(levels, [ 1, 2, 3, 4, 5 ]));
     if(n==10, check_equal(levels, [ 1, 2, 3, 4, 4 ]));
     if(n==11, check_equal(levels, [ 1, 2, 3, 4, 3 ]));

     \\ examples
     if(n==35,     check_equal(levels, [1, 2, 3, 2, 3, 2]); seen++);
     if(n==7726,   check_equal(levels, BH_before);          seen++);
     if(n==7726+1, check_equal(levels, BH_after);           seen++);

     \\ if(levels==my_example_levels, print(n); quit);
     if(n==140193, check_equal(levels, my_example_levels);  seen++);

     if(n==33, check_equal(levels,[1,2,3,3,2,3]); seen++);

     A346913_next_levels();
     );
  check_equal(seen,7);

  my(depths=apply(d->d-1,my_example_levels));
  check_equal(vpar_is_depths_vector(depths),1,
              "my example depths");
  my(vpar=vpar_make_preorder_depths(depths));
  vpar_validate(vpar);
}
{
  \\ Knuth example levels
  my(end = digits(23443433000000000));
  levels = concat([1,2], apply(t->t+1, end));
  A346913_next_levels();
  my(want_end=digits(23443432344343234));
  check_equal(levels,
              concat([1,2],apply(t->t+1, want_end)),
              "Knuth example next levels");
}
kill(levels);


\\-----------------------------------------------------------------------------
\\ A346914_next_vpar() examples

{
  \\ empty vpar allowed as if an empty n=1 row, per note with
  \\ vpar_forest_to_tree_levels() checking
  vpar=[];
  A346914_next_vpar();
  check_equal(vpar,[0]);
}
{
  \\ triangle samples
  my(want=[[],            \\ empty row n=1
           [ 0          ],
           [ 0, 1       ],
           [ 0, 0       ],
           [ 0, 1, 2    ],
           [ 0, 1, 1    ],
           [ 0, 1, 0    ],
           [ 0, 0, 0    ],
           [ 0, 1, 2, 3 ],
           [ 0, 1, 2, 2 ] ]);
  vpar=[];
  for(n=1,10,
     check_equal(vpar,want[n], "vpar triangle begins");
     A346914_next_vpar());
}
{
  vpar=[0]; \\ global variable, row n=2
  my(seen=0);
  for(n=2,1200,
     vpar_validate(vpar);
     \\ triangle begins
     if(n==2,  check_equal(vpar, [ 0          ]); seen++);
     if(n==3,  check_equal(vpar, [ 0, 1       ]); seen++);
     if(n==4,  check_equal(vpar, [ 0, 0       ]); seen++);
     if(n==5,  check_equal(vpar, [ 0, 1, 2    ]); seen++);
     if(n==6,  check_equal(vpar, [ 0, 1, 1    ]); seen++);
     if(n==7,  check_equal(vpar, [ 0, 1, 0    ]); seen++);
     if(n==8,  check_equal(vpar, [ 0, 0, 0    ]); seen++);
     if(n==9,  check_equal(vpar, [ 0, 1, 2, 3 ]); seen++);
     if(n==10, check_equal(vpar, [ 0, 1, 2, 2 ]); seen++);
     if(n==11, check_equal(vpar, [ 0, 1, 2, 1 ]); seen++);   \\ 10 of

     \\ examples
     if(n==35,   check_equal(vpar, [0, 1, 0, 3, 0]); seen++); \\ too small
     if(n==1156, check_equal(vpar, [0,1,2,1,0,5,5,0,8]); seen++);
     \\ if(vpar==[0,1,2,1,0,5,5,0,8],print(n);quit);

     A346914_next_vpar());
  check_equal(seen, 12);
}
kill(vpar);



\\-----------------------------------------------------------------------------
\\ A346914_next_vpar() adapted to trees-only, per final comment

\\ acting on global variable vpar[]
A346914_next_vpar_of_trees_only() =
{
  forstep(p=#vpar,2,-1,
     my(q=vpar[p]);     \\ q = parent of p
     q>1 || next;
     
     \\ replace p..#vpar with copies of block vpar[q .. p-1],
     \\ with suitable offset added for their new location in vpar,
     \\ and except each copy of vpar[q] is unchanged (stays value g)
     my(g=vpar[q],            \\ parent of q
        offset=p-q);
     while(p<=#vpar,
           vpar[p] = vpar[q] + if(vpar[q]!=g, offset);
           p++; q++);
        return(1));  \\ have stepped to a new forest of same num vertices

  \\ no vertex p which has a parent, no more of this #vpar length,
  \\ step to the first of the next length
  vpar = [0..#vpar];
  0;  \\ new num vertices
}

{
  my(bad=0);
  for(n=0,12,
     my(my_vpar=vpar_unlabelled_first(n));
     vpar=[0..n-1];  \\ global
     until((my_vpar=vpar_unlabelled_next(my_vpar)) == 'none,  \\ trees
           if(vpar != my_vpar,
              bad++);
           A346914_next_vpar_of_trees_only()));
  check_equal(bad, 0, 
           "A346914_next_vpar_of_trees_only() vs vpar_unlabelled_next() trees");
}
kill(vpar);


\\-----------------------------------------------------------------------------

\\ levels_to_next_sibling_vector(row) =
\\ {
\\   my(prev_at_level=vector(#row),
\\      next_sibling=vector(#row));
\\   for(v=1,#row,
\\      my(level=row[v]);
\\      if(prev_at_level[level],
\\         next_sibling[prev_at_level[level]] = v);
\\      prev_at_level[level] = v);
\\   next_sibling;
\\ }

\\ No, wrong w_end.
\\   my(next_sibling=levels_to_next_sibling_vector(row));
\\   for(v=1,#row,
\\      my(w=next_sibling[v]);
\\      if(w,
\\         my(v_end=w-1,
\\            w_end=if(next_sibling[w], next_sibling[w]-1, #row),
\\            v_subtree_levels=row[v..v_end],
\\            w_subtree_levels=row[w..w_end]);
\\         \\ descending order v >= w
\\ print(v_subtree_levels,w_subtree_levels);
\\         lex(v_subtree_levels,w_subtree_levels) >= 0 || error()));

\\-----------------------------------------------------------------------------
check_equal(prev_levels,'prev_levels);
check_equal(prev_vpar,'prev_vpar);
check_equal(seen,'seen);
check_equal(levels,'levels);
check_equal(vpar,'vpar);
print("end a346913-rows-test.gp");
