\\ Copyright 2017, 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");
\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=0;
progress_prev=-1;



\\------------------------------------------------------------------------------
\\ rooted forests, uniquely occurring partitions,
\\ determined by orbits sizes

\\ 1, 2, 2, 3, 2, 4, 3, 5, 6, 5, 9, 9, 15, 10, 17, 17, 30, 20

A347414_partitions_count_map(n) =
{
  my(m=Map(), mv=Map(),
     vpar=vpar_unlabelled_first(n));
  until(!vpar_unlabelled_nextR(~vpar,1),  \\ forests
        my(p=Vecsmall(vecsort(vpar_orbit_sizes(vpar))),c);
        mapisdefined(m,p,&c); c++;
        mapput(m,p,c);
        if(c==1, mapput(mv,p,vpar),
           c==2, mapdelete(mv,p)));
  [m,mv];
}
{
  for(n=1,15,
     print("n="n);
     my(m,mv); [m,mv]=A347414_partitions_count_map(n);
     mv=Mat(mv);
     my(unique=#mv);
     for(i=1,matsize(mv)[1],
        my(part=mv[i,1],
           vpar=mv[i,2],
           num_components=vpar_num_components(vpar));        
        print("  unique partition ",Vec(part)," forest ",vpar" components "num_components));
     print("  ",unique," unique");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Partitions Coded Into Bits

partition_to_code(v) =
{
  v==vecsort(v) || error();
  my(P=0);
  for(i=1,#v,
     v[i] >= 1 || error();
     P = ((P<<1)+1) << (v[i]-1));
  P;
}
check_equal(to_binary(partition_to_code([1,1,2])), 1110);

make_partitions_recurse(P,pos,t) =
{
  \\ print("recurse_partitions P=",to_binary(P)," pos="pos" t="t);
  t>=1 || error();
  if(pos==-1,
     listput(partitions_out,P);
     return);
  if(pos-t < -1,
     return); 
  make_partitions_recurse(P,pos,t+1);  \\ none of t

  \\ print("recurse_partitions P=",to_binary(P)," pos="pos" t="t"  one or more");
  while(pos-t >= -1,
        P += 1<<pos;
        pos -= t;
        make_partitions_recurse(P,pos,t+1));  \\ one or more of t
}
make_partitions(n) =
{
  partitions_out=List([]);
  make_partitions_recurse(0,n-1,1);
  partitions_out = Vec(partitions_out);
}
all=[]
{
  for(n=1,32,
     make_partitions(n);
     by_partitions=vecsort(apply(partition_to_code,partitions(n)));
     if(n<=6,
        print(apply(to_binary,partitions_out));
        print(apply(to_binary,by_partitions));
        );
     \\ by_partitions-partitions_out
     by_partitions==partitions_out || error("different");
     all=concat(all,partitions_out);
     check_equal(#partitions_out, numbpart(n));
     check_equal(partitions_out, Set(partitions_out));
  );
}
check_equal(all,Set(all));
check_equal(numbpart(32), 8349);
{
  my(want=OEIS_samples("A225620"));
  check_equal(all[1..#want], want, "make_partitions() = A225620");
}
quit
\\ A225620 Indices of partitions in the table of compositions of A228351.


\\------------------------------------------------------------------------------

n_to_partitions = List([ [ Vecsmall([1]) ] ]);
n_to_partitions = List([ [ 1             ] ]);

new_m = Map();
n_to_map = List([ Map(Mat([ Vecsmall([1]), 1 ])) ]);

p_to_count = Map();
{
  for(n=4,20,
     my(m=Map(),
        vpar=vpar_unlabelled_first(n));
     until(!vpar_unlabelled_nextR(vpar,1),  \\ forests
           my(orbitsizes=vpar_orbit_sizes(vpar),
              key=concat(orbitsizes[1],vecsort(orbitsizes[2..#orbitsizes])),
              c=0);
           \\ print(key" from "orbitsizes);
           mapisdefined(m,key,&c);
           mapput(m,key,c+1));
     \\ print("n="n" count "#m"  cf ",numbpart_one_distinguished(n)
     \\       " forests "vpar_count_unlabelled(n,1));
     print1(#m,",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Num Orbit Partitions of Rooted Forests - with distinguished root=1 orbit

\\ not in OEIS: 5,7,11,16,23,34,45,65,87,123,161
\\
\\ vector(20,n,numbpart_one_distinguished(n))
numbpart_one_distinguished(n) = sum(i=1,n, numbpart(i));
{
  for(n=4,20,
     my(m=Map(),
        vpar=vpar_unlabelled_first(n));
     until(!vpar_unlabelled_nextR(vpar,1),  \\ forests
           my(orbitsizes=vpar_orbit_sizes(vpar),
              key=concat(orbitsizes[1],vecsort(orbitsizes[2..#orbitsizes])),
              c=0);
           \\ print(key" from "orbitsizes);
           mapisdefined(m,key,&c);
           mapput(m,key,c+1));
     \\ print("n="n" count "#m"  cf ",numbpart_one_distinguished(n)
     \\       " forests "vpar_count_unlabelled(n,1));
     print1(#m,",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Num Orbit Partitions of Rooted Forests

{
  my(n=4);
  my(m=Map(),
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_next(vpar,1))=='none,  \\ forests
        print("vpar "vpar);
        my(p=vecsort(vpar_orbit_sizes(vpar)));
        if(!mapisdefined(m,p),
           print("  new "p);
           mapput(m,p,1)));
  print("count "#m"  cf "numbpart(n));
  quit;
}

{
  \\ rooted forests num partitions occurring
  \\ 1,1,2,3,5,6,11,13,21,28,38,51,73,
  \\ not in OEIS: 2,3,5,6,11,13,21,28,38,51,73,
  \\ forests same by omitting the root
  \\
  \\ rooted forests num partitions not occurring
  \\ 0, 0, 0, 0, 1, 0, 2, 1, 2, 4, 5, 4, 8, 11, 13, 19, 19, 33, 31
  \\
  my(forest=1);
  for(n=1,20,
     my(m=Map(),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           mapput(m,Vecsmall(vecsort(vpar_orbit_sizes(vpar))),1));
     \\ print1(#m",");
     print("n="n"  "#m"  cf "numbpart(n));
     forpart(p=n,
        if(!mapisdefined(m,p), print("  "Vec(p))));
  );
  quit;

  \\ rooted: 1,1,2,3,5,6,11,13,21,28,38,51,73,
  \\ free:   1,1,1,2,3,6,9,15,19,32,36,56
}

\\ n=1  a(n) = 1                             
\\ n=2  a(n) = 2                             
\\ n=3  a(n) = 3                             
\\ n=4  a(n) = 5                             
\\ n=5  a(n) = 6                             
\\ n=6  a(n) = 11                             
\\ n=7  a(n) = 13                             
\\ n=8  a(n) = 21                             
\\ n=9  a(n) = 28                             
\\ n=10  a(n) = 38                             
\\ n=11  a(n) = 51                             
\\ n=12  a(n) = 73                             
\\ n=13  a(n) = 93                             
\\ n=14  a(n) = 124                             
\\ n=15  a(n) = 163                             
\\ n=16  a(n) = 212                             
\\ n=17  a(n) = 278                             
\\ n=18  a(n) = 352                             
\\ n=19  a(n) = 459                             
\\ n=20  a(20) = 572
\\ n=21  a(21) = 736
\\ n=22  a(22) = 914

\\ numbpart(21) == 792
\\ numbpart(22) == 1002

\\ 0, 0, 0, 0, 1, 0, 2, 1, 2, 4, 5, 4, 8, 11, 13, 19, 19, 33,  31



\\------------------------------------------------------------------------------
\\ Rooted by Subtrees
\\ 1,1,2,3,5,6,11,13,21,28,38,51,73,

partition_to_count_map = Map();
partition_to_count(p) = my(ret=0); mapisdefined(partition_to_count_map,ret); ret;
partition_to_count_add(p,n) = \
  mapput(partition_to_count_map,p, partition_to_count(p) + c)
partition_to_count_add(Vecsmall([1]),1)
{
  for(n=2,2,
     forpart(p=n-1,
        
  
}
quit

\\------------------------------------------------------------------------------

\\ unused initial partition for rooted forests
\\
\\ #if 0
\\   int histogram[MAXN];
\\   int v, max_depth = -1;
\\   int d, dpos[MAXN+1];
\\ 
\\   memset(histogram, '\0', n*sizeof(histogram[0]));
\\   for (v = 0; v < n; v++) {
\\     int d = depths[v];
\\     assert (d >= 0);
\\     assert (d < n);
\\     if (d > max_depth) {
\\       assert (d == max_depth+1);
\\       max_depth = d;
\\       histogram[d] = 1;
\\     } else {
\\       histogram[d]++;
\\     }
\\   }
\\ 
\\   /* ptn[] is all 1s except a 0 marker at the last of each colour class. */
\\   for (v = 0; v < n; v++) ptn[v] = 1;
\\   dpos[0] = 0;
\\   for (d = 0; d <= max_depth; d++) {
\\     int after = dpos[d] + histogram[d];
\\     dpos[d+1] = after;
\\ 
\\     assert (after-1 >= 0 && after-1 < n);
\\     ptn[after-1] = 0;
\\   }
\\   assert (dpos[max_depth+1] == n);  /* total vertices at all depths */
\\ 
\\   /* lab[] is a list of the vertices 0..n-1 with those of each colour
\\      contiguous.  Here this is the vertices of successive depth levels.
\\   */
\\   for (v = 0; v < n; v++) {
\\     int d = depths[v];
\\     lab[dpos[d]++] = v;
\\   }
\\   assert (dpos[max_depth] == n);  /* advances to after all of n */
\\ #endif
