\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(parisizemax,500*10^6);
default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("a347967-maximum-matchings.gp");


\\-----------------------------------------------------------------------------
\\ Helpers

A206483_matchnum(n) = vpar_matchnum(vpar_from_primecode(n));
{
  my(want=OEIS_data("A206483"));
  check_equal(vector(#want,n, A206483_matchnum(n)),
              want,
              "A206483_matchnum() vs samples");
}

A202853_matchpoly_row(n) = Vecrev(vpar_matchpoly(vpar_from_primecode(n)));
{
  my(want=OEIS_data("A202853"));  \\ OFFSET=1
  my(got=[], n=1);
  for(n=1,oo,
     if(#got>=#want, break);
     got=concat(got, A202853_matchpoly_row(n)));
  check_equal(got[1..#want], want,
              "A202853 matchpoly rows, lowest term first");
}
A202853_matchpoly_term(n,k) = A202853_matchpoly_row(n)[k+1];

\\ formula A347967 by triangle A202853
{
  my(a=A347967,
     A202853=A202853_matchpoly_term,
     A206483=A206483_matchnum);
  for(n=1,500,
     check_equal(a(n),
                 A202853(n, A206483(n))));
}


\\-----------------------------------------------------------------------------
\\ Quantities Combinations

quantities_singleton = [0,1,1];

quantities_validate(v) =
{
  #v==3 || error();
  my(with,without,without_is_same);
  [with,without,without_is_same] = v;

  with    >= 0 || error();
  without >= 1 || error();  \\ always have at least one without
  without_is_same==0 || without_is_same==1 || error();
  if(with==0, without_is_same==1 || error());
}

quantities_total(v) =
{
  my(with,without,without_is_same);
  [with,without,without_is_same] = v;

  if(without_is_same==1, with+without,
     without_is_same==0, with,
     error("without_is_same is ",without_is_same));
}
{
  check_equal(quantities_total(quantities_singleton), 1,
              "quantities_total() of singleton");
}

quantities_accumulate(v,c) =
{
  \\ print("quantities_accumulate() "v" + "c);
  quantities_validate(v);
  quantities_validate(c);

  my(c_with,c_without,c_without_is_same);
  [c_with,c_without,c_without_is_same] = c;
  my(c_with_size    = 0,
     c_without_size = if(c_without_is_same, 0, -1),
     c_total      = quantities_total(c),
     c_total_size = c_with_size);

  my(v_with,v_without,v_without_is_same);
  [v_with,v_without,v_without_is_same] = v;
  my(v_with_size    = 0,
     v_without_size = if(v_without_is_same, 0, -1));

  my(new_without = v_without * c_total,
     new_without_size = v_without_size + c_total_size);
  \\ print("  without = "new_without," size "new_without_size);

  my(W0 = v_with    * c_with,
     W1 = v_with    * c_without,
     W2 = v_without * c_without,
     W0_size = v_with_size    + c_with_size,
     W1_size = v_with_size    + c_without_size,
     W2_size = v_without_size + c_without_size + 1);

  \\ print("  W0 = ",W0," size ",W0_size);
  \\ print("  W1 = ",W1," size ",W1_size);
  \\ print("  W2 = ",W2," size ",W2_size);

  my(s = v_without_is_same + c_without_is_same);
  \\ print("  s = ",s);

  my(new_with      = 'none,
     new_with_size = 'none,
     new_W_list    = 'none);
  if(!v_without_is_same && !c_without_is_same,
     \\ print("  with = W0");
     new_with = W0;
     new_with_size = W0_size;
     new_W_list = [0];
     W1==0 || W1_size < W0_size  || error();
     W2==0 || W2_size < W0_size  || error();
     ,
     !v_without_is_same && c_without_is_same,
     new_with = W0+W1+W2;
     new_with_size = W2_size;
     new_W_list = [0,1,2];
     \\ print("  with = W0+W1+W2 = "new_with"  size ",new_with_size);
     W2!=0 || error();
     W0==0 || W0_size == W2_size || error();
     W1==0 || W1_size == W2_size || error();
     ,
     v_without_is_same && !c_without_is_same,
     new_with = W0   +W2;
     new_with_size = W2_size;
     new_W_list = [0,2];
     \\ print("  with = W0 + W2 = "new_with"  size ",new_with_size);
     W2!=0 || error();
     W0==0 || W0_size == W2_size || error();
     W1==0 || W1_size < W2_size || error();
     ,
     v_without_is_same && c_without_is_same,
     \\ print("  with = W2");
     new_with = W2;
     new_with_size = W2_size;
     new_W_list = [2];
     W0_size < W2_size || error();
     W1_size < W2_size || error();
     ,
     error()) ;

  my(new_without_is_same = (new_with == 0
                            || new_with_size == new_without_size));

  if(s!=2, setsearch(new_W_list,0) || error("s!=2 ="s" "new_W_list));
  if(s!=0, setsearch(new_W_list,2) || error());

  if(s==0, setsearch(new_W_list,0) || error());
  if(s==1, setsearch(new_W_list,2) || error());
  if(s==2, setsearch(new_W_list,2) || error());

  if(v_without_is_same && c_without_is_same,
     new_without_is_same == 0  || error();
     ,
     \\ unchanged
     new_without_is_same == v_without_is_same  || error();
    );

  \\ print("  new with    ",new_with,   " size ",new_with_size);
  \\ print("  new without ",new_without," size ",new_without_size);

  my(ret = [new_with, new_without, new_without_is_same]);
  \\ print("  ret ",ret);
  quantities_validate(ret);
  ret;
}
{
  \\ 1 <-- 2
  \\ path-2, 1 matching with,without but without_is_same=0
  quantities_path2 = [1,1,0];
  check_equal(quantities_accumulate(quantities_singleton,
                                    quantities_singleton),
              quantities_path2);

  \\ 1 <-- 2 <-- 3
  check_equal(quantities_accumulate(quantities_singleton,
                                    quantities_path2),
              [1,1,1],
              "quantities_accumulate() path-3");
  \\   v
  \\  / \
  \\ 2   c
  check_equal(quantities_accumulate(quantities_path2,
                                    quantities_singleton),
              [2,1,0],
              "quantities_accumulate() star-3");

  \\    v
  \\  / | \
  \\ 2  3  c
  check_equal(quantities_accumulate([2,1,0],
                                    quantities_singleton),
              [3,1,0],
              "quantities_accumulate() star-4");
}

quantities_of_vpar_tree(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     qvec=vector(#vpar,v, quantities_singleton));
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v]);
     if(p,
        qvec[p] = quantities_accumulate(qvec[p], qvec[v]);
        ,
        return(qvec[v])));
  error();
}
check_equal(quantities_of_vpar_tree([0]), [0,1,1]);

test_maximum_matchings_count_by_quantities(vpar) = \
  quantities_total(quantities_of_vpar_tree(vpar));

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     #vpar || next;
     \\ print(vpar);
     check_equal(test_maximum_matchings_count_by_quantities(vpar),
                 vpar_matchnum_and_count(vpar)[2],
                 "test_maximum_matchings_count_by_quantities() sample");
  );
}
{
  for(rep=1,1000,
     my(len=random(100),
        vpar=vpar_make_random(len));
     #vpar || next;
     check_equal(test_maximum_matchings_count_by_quantities(vpar),
                 vpar_matchnum_and_count(vpar)[2],
                 "test_maximum_matchings_count_by_quantities() random"));
}


\\-----------------------------------------------------------------------------
\\ A347967()

{
  my(want=OEIS_data("A347967"));
  check_equal(vector(#want,n, A347967(n)),
              want,
              "A347967() vs samples");

  if(0,
     system("rm -f /tmp/b347967.txt");
     for(n=1,10000, 
        write("/tmp/b347967.txt", Str(n," ",A347967(n))));
     system("ls -l /tmp/b347967.txt");
     quit);

  \\ ~/OEIS/b347967.txt
  if(0,
     my(want=OEIS_data("A347967",'bfile));
     print("b347967.txt length ",#want);
     check_equal(vector(#want,n, A347967(n)),
                 want,
                 "A347967() vs ~/OEIS/b347967.txt"));
}

A347967_by_vpar(n) = vpar_matchnum_and_count(vpar_from_primecode(n))[2];
{
  check_equal(vector(100,n, A347967(n)),
              vector(100,n, A347967_by_vpar(n)));

  for(rep=1,5000,
     my(len=random(10),
        n=random(10^len)+1);
     n>=1 || error();
     \\ print(n);
     check_equal(A347967(n), A347967_by_vpar(n),
                 "A347967() random"));
}


\\-----------------------------------------------------------------------------
check_equal(b,'b);
check_equal(c,'c);
check_equal(f,'f);
check_equal(n,'n);
check_equal(idx,'idx);
check_equal(got,'got);
check_equal(ret,'ret);
check_equal(qvec,'qvec);
check_equal(quantities,'quantities);
check_equal(with,'with);
check_equal(without,'without);
check_equal(without_is_same,'without_is_same);
check_equal(upto,'upto);
check_equal(vec,'vec);
check_equal(vpar,'vpar);
check_equal(want,'want);
check_equal(with,'with);
print("end a347967-maximum-matchings--test.gp");
