\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Tests for a354322-Matula-subtree-rows.gp, calculating fows of A354322,
\\ the distinct subtrees occurring in Matula-Goebel tree n.


default(recover,0);
default(strictargs,1);
default(parisizemax,1500*10^6);  \\ for big search

read("a354322-Matula-subtree-rows.gp");
print("------------\n");

option_verbose=0;
option_progress=1;

read("memoize.gp");
read("OEIS-data");
read("OEIS-data-wip");
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");


\\-----------------------------------------------------------------------------

is_pow2(n) = n>=1 && logint(n,2)==valuation(n,2);


\\-----------------------------------------------------------------------------
\\ Num Vertices
\\ A061775

A061775_num_vertices(n) = #vpar_from_primecode(n);
OEIS_check_func("A061775_num_vertices");

\\-----------------------------------------------------------------------------
\\ Largest Prime Factor
\\ A061395

largest_prime_factor(n) =
{
  n>=1 || error();
  if(n==1,1, my(f=factor(n)); f[matsize(f)[1],1]);
}
alias(A006530_largest_prime_factor,largest_prime_factor);
OEIS_check_func("A006530_largest_prime_factor");

largest_prime_factor_exponent(n) =
{
  n>=1 || error();
  if(n==1,0, my(f=factor(n)); f[matsize(f)[1],2]);
}
alias(A071178_largest_prime_factor_exponent,largest_prime_factor_exponent);
OEIS_check_func("A071178_largest_prime_factor_exponent",,['offset,1]);

A061395_largest_prime_factor_index(n) =
{
  n>=1 || error();
  if(n==1,0, primepi(largest_prime_factor(n)));
}
OEIS_check_func("A061395_largest_prime_factor_index",,['offset,1]);


\\-----------------------------------------------------------------------------
\\ Prime'th Recurrence

\\ return a vector of len many terms (n=0..len-1)
A007097_primeth_vector(len) =
{
  my(v=vector(len,i,1));
  for(i=2,#v, v[i]=prime(v[i-1]));
  v;
}
A007097_primeth(n) = A007097_primeth_vector(n+1)[n+1];
{
  check_equal(A007097_primeth_vector(8),
              OEIS_data("A007097")[1..8]);
}

is_A007097_primeth(n) =
{
  my(p=1);
  until((p=prime(p))>n,
        if(p==n,return(1)));
  0;
}
{
  my(v=A007097_primeth_vector(9),
     hi=v[#v]);
  check_equal(hi, 5381);
  check_equal(vector(hi+1,n,n--; is_A007097_primeth(n)),
              vector(hi+1,n,n--; setsearch(v,n)!=0));
}


\\-----------------------------------------------------------------------------

\\ old code with separate queue and return
row_by_q(n) =
{
  my(q=[n],
     ret=List([]));
  while(#q,
        listput(ret,q[#q]);
        q=setunion(q[^#q],
                   apply(primepi,factor(q[#q])[,1]~)));
  Vecrev(ret);
}
{
  check_equal(vector(500,n,row(n)),
              vector(500,n,row_by_q(n)));
}

\\-----------------------------------------------------------------------------
\\ rows_vector()

{
  my(by_indiv=vector(100,n,row(n)));
  for(maxn=1,100,
     my(rows=rows_vector(maxn));
     #rows == maxn  || error();
     rows == by_indiv[1..maxn] || error();
     );
}


\\-----------------------------------------------------------------------------
\\ DATA

{
  my(want=[
1, 1, 2, 1, 2, 3, 1, 4, 1, 2, 3, 5,
              1, 2, 6, 1, 4, 7, 1, 8, 1, 2, 9, 1,
              2, 3, 10, 1, 2, 3, 5, 11, 1, 2, 12,
              1, 2, 6, 13, 1, 4, 14, 1, 2, 3, 15,
              1, 16, 1, 4, 7, 17, 1, 2, 18, 1, 8,
              19, 1, 2, 3, 20, 1, 2, 4, 21, 1, 2,
              3, 5, 22, 1, 2, 9, 23, 1, 2, 24, 1,
              2, 3, 25
 ]);
  check_equal(concat(vector(25,n, row(n))),
              want);

  if(0,
     my(v=concat(vector(25,n,row(n))));
     for(i=1,#v,
        print1(v[i],",");
        if(i==29||i==55,print()));
     quit);
}


\\-----------------------------------------------------------------------------
\\ b-File
{
  if(0,
     system("mkdir -p /tmp/new");
     system("rm -f /tmp/new/b354322.txt");
     my(v=concat(vector(2000,n,row(n))));
     for(n=1,#v, 
        write("/tmp/new/b354322.txt", Str(n," ",v[n])));
     system("ls -l /tmp/new/b354322.txt");
     quit);

  if(0,
     \\ ~/OEIS/b354322.txt
     my(want=OEIS_data("A354322",'bfile));
     my(got=[]);
     for(n=1,oo,
        if(#got>=#want,break);
        got=concat(got,row(n)));
     print("b354322.txt length ",#want);
     check_equal(got,want,
                 "rows_vector() vs b-file"));
}

\\-----------------------------------------------------------------------------
\\ row()
{
  check_equal(row(6), [1,2,6]);

  \\ row widths
  \\ A317713 number of distinct subtrees
  OEIS_check_func("A317713", (n)->#row(n), ['offset,1]);

  for(n=1,5000,
     my(v=row(n));
     v[1] == 1   || error();  \\ first
     v[#v] == n  || error();  \\ last
     if(#v>=2,               
        is_pow2(v[2]) || error()); \\ second

     (#v==2) == (n>=2 && is_pow2(n)) || error();

     \\ second-last colunm
     \\ A061395_largest_prime_factor_index()
     if(#v>=2,
        v[#v-1] == A061395_largest_prime_factor_index(n) || error());

     my(num_vertices=A061775_num_vertices(n));
     #v <= num_vertices   || error();
     (#v == num_vertices) == is_A007097_primeth(n)  || error();

     );
}

\\-----------------------------------------------------------------------------
\\ Star

{
  \\ star of k children is n=2^k
  check_equal(vector(20,k, vpar_to_primecode(vpar_make_star(k+1))),
              vector(20,k, 2^k));
  \\ row is n,1
  check_equal(vector(20,k,my(n=2^k); row(n)),
              vector(20,k,my(n=2^k); [1,n]));
}

\\-----------------------------------------------------------------------------
\\ path down
\\ A007097_primeth() recurence

{
  check_equal(vector(8,k,vpar_to_primecode(vpar_make_path(k))),
              vector(8,k, A007097_primeth(k-1)),
              "path k vertices");

  \\ row of path k down
  check_equal(vector(8,k,my(n=A007097_primeth(k-1)); row(n)),
              vector(8,k, apply(A007097_primeth,[0..k-1])));
}

\\-----------------------------------------------------------------------------
\\ Examples

{
  \\ example triangle begins
  my(want=[
                 1,
                 1, 2,
                 1, 2, 3,
                 1, 4,
                 1, 2, 3, 5,
                 1, 2, 6,
                 1, 4, 7
          ]);
  check_equal(concat(vector(7,n, row(n))),
              want);
  if(0,
     for(n=1,7,
        printf("%%e   n=%d: ", n);
        my(v=row(n));
        for(i=1,#v,
           printf(" %d,", v[i]));
        print());
     quit);     
}

{
  \\ example
  \\       78              1
  \\     / | \           / | \    
  \\    1  2  6         2  3  5   
  \\       |  | \          |  | \ 
  \\       1  1  2         4  6  7
  \\             |               |
  \\             1               8
  my(vpar=[0,1,1,3,1,5,5,7]);
  vpar_validate(vpar);
  check_equal(vpar_to_primecode(vpar), 78);
  check_equal(vector(#vpar,v, vpar_to_primecode(vpar_subtree(vpar,v))),
              [78,1,2,1,6,1,2,1]);
  check_equal(row(78), [1, 2, 6, 78]);
}

\\-----------------------------------------------------------------------------
{
  \\ column k=2
  \\ print(vector(20,n,n++; row(n)[2]));quit;
  \\ not in OEIS: 2, 2, 4, 2, 2, 4, 8, 2, 2, 2, 2, 2, 4, 2, 16, 4, 2, 8, 2, 2
  \\ print(vector(50,n,n++; logint(row(n)[2],2)));quit;
  \\ not in OEIS: 1, 1, 2, 1, 1, 2, 3, 1, 1, 1, 1, 1, 2, 1, 4
  check_equal(Set(vector(500,n,n++; row(n)[2])),
              vector(8,k, 2^k));

  check_equal(prime(2)*prime(2^2), 21);
  check_equal(prime(2^2)*prime(2^3), 133);
  check_equal(row(21), [1, 2, 4, 21]);

  my(row_compact(n) = 
     if(n==1,[1], concat(fold(setunion,[self()(primepi(p)) |p<-factor(n)[,1]~]), n));
     );
  check_equal(row(6),         [1,2,6]);
  check_equal(row_compact(6), [1,2,6]);
  check_equal(vector(500,n, row_compact(n)),
              vector(500,n, row(n)));
}

\\-----------------------------------------------------------------------------
\\ Other

{
  \\ distint in forward or reverse order
  if(0,
     print(concat(vector(10,n,n++;A354322_Matula_distinct_subtree_row(n))));
     print(concat(vector(10,n,n++;Vecrev(A354322_Matula_distinct_subtree_row(n)))));
     print(concat(vector(10,n,n++;Vecrev(A354322_Matula_distinct_subtree_row(n)[^1]))));
     quit);

  \\ not in OEIS: 2, 1, 3, 2, 1, 4, 1, 5, 3, 2, 1, 6, 2, 1, 7, 4, 1, 8, 1, 9, 2, 1, 10, 3, 2, 1, 11, 5, 3, 2, 1
  \\ not in OEIS: 1, 2, 1, 2, 3, 1, 4, 1, 2, 3, 5, 1, 2, 6, 1, 4, 7, 1, 8, 1, 2, 9, 1, 2, 3, 10, 1, 2, 3, 5, 11
  \\ not in OEIS: 1, 1, 2, 1, 1, 2, 3, 1, 2, 1, 4, 1, 1, 2, 1, 2, 3, 1, 2, 3, 5
}

{
  \\ largest rooted star within tree
  \\ = largest power of 2 in row
  if(0,
     print(vector(21,n, vecmax(select(test_is_pow2,row(n)))));
     quit);
  \\ not in OEIS: 2, 2, 4, 2, 2, 4, 8, 2, 2, 2, 2, 2, 4, 2, 16, 4, 2, 8, 2, 4
}


\\-----------------------------------------------------------------------------
{
  \\ speed row() vs rows_vector()
  \\ about 4* faster whole vector
  if(0,
     my(maxn=100000);
     gettime();
     \\ my(u=vector(maxn,n,row(n)));
     for(n=1,maxn, row(n));
     my(t1=gettime());
     gettime();
     \\ my(v=rows_vector(maxn));
     rows_vector(maxn);
     my(t2=gettime());
     print("by row()          ",t1);
     print("by rows_vector()  ",t2);
     \\ check_equal(u,v);
     quit;
     );
}

\\-----------------------------------------------------------------------------
\\ largest_prime_to_primepi_multiplicity()

largest_prime_to_primepi_multiplicity(n) =
{
  n>=1 || error();
  if(n==1,1,
     my(f=factor(n),
        r=matsize(f)[1]);
     f[r,1] = primepi(f[r,1]);
     factorback(f));
}

{
  my(a=largest_prime_to_primepi_multiplicity);
  check_equal(vector(2000,n,n++; #row(a(n))),
              vector(2000,n,n++; #row(n) - 1),
              "distinct n to n-1");

  check_equal(vector(2000,n,n++; a(n)),
              vector(2000,n,n++;
    my(p=A006530_largest_prime_factor(n),
       e=A071178_largest_prime_factor_exponent(n));
    n * primepi(p)^e / p^e),
              "by n * primepi(p)^e / p^e");
}



\\-----------------------------------------------------------------------------
\\ No Global Variables

check_equal(d,'d);
check_equal(k,'k);
check_equal(l,'l);
check_equal(m,'m);
check_equal(n,'n);
check_equal(p,'p);
check_equal(q,'q);
check_equal(r,'r);
check_equal(ret,'ret);
check_equal(prev,'prev);

\\-----------------------------------------------------------------------------
print("end");
