\\ Copyright 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");
\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=1;
progress_prev=-1;
progress(str) =
{
  my(t=getwalltime()\2000);
  if(t!=progress_prev,
     progress_prev=t;
     print(str));
}

\\default(realprecision,200);
nearly_equal_epsilon = 1e-10;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) = {
  \\ print(type(x)," ",type(y)," ",x," ",y);
  my(d=x-y); \\ ensure compatible types
  if(type(x)=="t_MAT",x=Vec(x);y=Vec(y));
  if(type(x)=="t_VEC" || type(x)=="t_COL",
       sum(i=1,max(#x,#y), !nearly_equal(x[i],y[i],epsilon)) == 0,
     type(x)=="t_QUAD" || type(x)=="t_COMPLEX",
          nearly_equal(real(x-y),0,epsilon)
       && nearly_equal(imag(x-y),0,epsilon),
     abs(d) < epsilon);
}
check_nearly_equal(got,want,name="") =
{
  my(bad=0);
  check_count++;
  if(!nearly_equal(got,want),
     print("wrong at "check_count": "name);
     print("got  "got);
     print("want "want);
     bad=1);

  if(bad,
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}


read("memoize.gp");
a(n) =
{
  n>=0 || error("a(n) is for n>=0");
  my(x=1,y=1); for(i=2,n, [x,y] = [x^2 + 2*x*y^2, x]); x;
}
a=memoize(a);

\\ a(n) = a(n-1)^2 + 2*a(n-1)*a(n-2)^2

c(n) = a(n)^(1/2^n);
c(n) = (2*a(n))^(1/2^n);

\\ a(n) = 1/2*c(n)^(2^n)

\\ Jacobsthal numbers A001045
\\ for(n=3,20,print(J(n)" "factor(J(n))))
\\
\\ biggest prime factor
\\ vector(20,n,n+=2; my(m=factor(J(n))[,1]); m[#m])
\\ 3, 5, 11, 7, 43, 17, 19, 31, 683, 13, 2731, 127, 331, 257, 43691, 73, 174763, 41, 5419, 683
\\ A271314
\\
J(n) = (2^n - (-1)^n)/3;
{
  check_equal(vector(#A001045_samples,n,n--; J(n)),
              A001045_samples,  \\ OFFSET=0
              "J() = A001045_samples");

  \\ A014113 = 2*J(n)
  check_equal(vector(#A014113_samples,n,n--; 2*J(n)),
              A014113_samples,  \\ OFFSET=0
              "J() = A014113_samples");

  \\ A139818 = J(n)^2
  check_equal(vector(#A139818_samples,n,n--; J(n)^2),
              A139818_samples,  \\ OFFSET=0
              "J()^2 = A139818_samples");
}

\\ A062510
H(n) = 2^n - (-1)^n;
{
  check_equal(vector(#A062510_samples,n,n--; H(n)),
              A062510_samples,  \\ OFFSET=0
              "H() = A062510_samples");
}

run_length_transform_of_func(n,func) =
{
  my(ret=1);
  for(k=0,if(n,logint(n,2)),
     if(bittest(n,k), my(s=k);
        while(bittest(n,k++),); ret*=func(k-s)));
  ret;
}

n_to_run_lengths(n) =
{
  my(ret=List([]));
  for(k=0,if(n,logint(n,2)),
     if(bittest(n,k),             \\ start of run of 1s
        my(start=k);
        while(bittest(n,k++),);   \\ step up through those 1s
        listput(ret,k-start)));   \\ len = k-start
  Vec(ret);
}
check_equal(n_to_run_lengths(5), [1,1]);
check_equal(n_to_run_lengths(13), [1,2]);

log2(n) = log(n)/log(2);

delete_second_lowest_bit(n) = ((n+1)>>1) - bittest(n,1);
delete_second_lowest_bit(n) = (bitnegimply(n,2)+1)>>1;
delete_second_lowest_bit(n) = ceil(bitnegimply(n,2)/2);
delete_second_lowest_bit(n) = (n>>1) - bittest(n,1) + bittest(n,0);
{
  check_equal(delete_second_lowest_bit(10), 4);

  \\ vector(20,n, delete_second_lowest_bit(n))
  \\ A106249  delete from n+1
  check_equal(vector(#A106249_samples,n,n--; delete_second_lowest_bit(n+1)),
              A106249_samples,  \\ OFFSET=0
              "delete_second_lowest_bit() vs A106249_samples");

  \\ A083219  delete from n+2
  check_equal(vector(#A083219_samples,n,n--; delete_second_lowest_bit(n+2)),
              A083219_samples,  \\ OFFSET=0
              "delete_second_lowest_bit() vs A106249_samples");
}

from_run_lengths(v) =
{
  my(bits=vector(vecsum(v)),upto=0,i=1);
  for(i=1,#v,
        for(j=1,v[i], bits[upto++]=1);
        upto += v[i++]);
  upto==vecsum(v) || error();
  fromdigits(bits,2);
}
to_run_lengths(n) =
{
  my(l=List([]));
  if(n,
     my(k=logint(n,2));
     while(k>=0,
           my(x=k);
           while(bittest(n,k), k--);
           listput(l,x-k);
           x=k;
           while(k>=0 && !bittest(n,k), k--);
           listput(l,x-k)));
  Vec(l);
}


\\-----------------------------------------------------------------------------

duplicate_runs_vector(v) = \
  vector(2*#v,i, v[delete_second_lowest_bit(i-1)+1]);
check_equal(duplicate_runs_vector([4,1]), [4,1, 4,1]);
check_equal(duplicate_runs_vector([1,2,3,4,5,6]), \
            [1,2,1,2, 3,4,3,4, 5,6,5,6]);

duplicate_runs(n) = \
  from_run_lengths(duplicate_runs_vector(to_run_lengths(n)));
{
  check_equal(duplicate_runs(0), 0);
  check_equal(duplicate_runs(test_from_binary(10)),
                             test_from_binary(1010));
  check_equal(duplicate_runs(test_from_binary(101100)),
                             test_from_binary(101011001100));
  check_equal(duplicate_runs(test_from_binary(101100)),
                             test_from_binary(101011001100));
  check_equal(duplicate_runs(test_from_binary(10111)),
                             test_from_binary(1010111111));
  check_equal(duplicate_runs(test_from_binary(1110010)),
                             test_from_binary(11100111001010));
  check_equal(vector(256,n, test_bitlength(duplicate_runs(n))),
              vector(256,n, 2*test_bitlength(n)),
              "duplicate_runs() doubles bit length");
}
vector(50,n, duplicate_runs(n))
\\ not in OEIS: 3, 10, 15, 36, 43, 54, 63, 136, 147, 170, 175, 204, 219, 238, 255, 528
OEIS_samples("A330940")
\\ maybe?:
\\ A330940 smallest by shuffling two copies of n
\\ A330940(n) =
\\ {
\\   my(v=binary(n),
\\      BL=List([]));
\\   if(#v,
\\      my(elem=[0,0],i=1);
\\      while(1,
\\            while(i<=#v&&v[i]==1,elem[1]++;i++);
\\            while(i<=#v&&v[i]==0,elem[2]++;i++);
\\            listput(BL,elem)));
\\ 
\\         if(v[i]!=b,
\\            listput(BL([b,len]));
\\            b=v[i]; len=1));
\\      listput(BL([b,len])));
\\   my(i=1,j=1, ret=0);
\\   while(i<=#BL && j<=#BL,
\\         if(BL[i][1]==0, ret<<=BL[i][2]; i++; next);
\\         if(BL[j][1]==0, ret<<=BL[j][2]; j++; next);
\\         if(BL[i][2] < BL[j][2]==1,
\\ }
quit


\\-----------------------------------------------------------------------------
{
  \\ vector(40,n,n+=5; J(n) % 11)
  \\ vector(40,n,n=2*n; valuation(3^n\2,2))
  for(n=3,15,
     printf("n=%2d %5d  %40s\n",
            n, J(n), test_to_binary((J(n)*3)^if(n%2==0,1,1))));
  print(test_to_binary(3*J(8) * 3*J(10) ));

  for(n=3,15,
     printf("n=%2d  %40s\n", n, test_to_binary(3^n)));
}

\\ J(4)
\\ m=J(7);
\\ vector(20,i, (J(4)^i)%m)
\\ for(i=3,12, for(j=3,i-1, print1(J(i)%J(j),"  ")); print())
\\ for(i=3,12, for(j=3,i-1, print1(gcd(J(i),J(j)),"  ")); print())
\\ for(i=3,20, for(j=3,i-1, my(g=gcd(J(i),J(j))); \
\\   printf(" %6d",znorder(Mod(J(j)/g,J(i)/g)),"  ")); print())

{
  \\ new prime factor in each J
  my(seen=[]);
  for(n=3,10000,
      print1("n="n" seen ",#seen,Strchr(13));
     my(t=factor(J(n))[,1]~);
     t==Set(t) || error(t);
     \\ print1(t[#t],",");    \\ biggest
     \\ print1(#t,",");    \\ num distinct
     if(setsearch(seen,t[#t]),
        \\ print("n="n" biggest "t[#t]" previously seen ",t);
        \\ print1(n,",");
       );
     \\ for(i=1,#t,print1(t[i],","));
     my(r = setminus(t,seen));
     if(#r==0,
        print("n="n" subset of previous primes"));
     \\ print1(#r,",");    \\ num new
     seen=setunion(seen,r);
     seen==Set(seen) || error();
  );
  quit;

  \\ how many new primes in J(n)
  \\ 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,2,1,1,2,2,1,1,1,2,1,2,2,2,1,1,1,2,1,1,2,1,2,2,2,1,2,3,3,2,1,2,1,2,3,3
  \\
  \\ biggest prime
  \\ A271314     cf 2^+/-1 A005420 and A002587
  \\ 3,5,11,7,43,17,19,31,683,13,2731,127,331,257,43691,73,174763,41,5419,683,2796203
  \\
  \\ n where biggest is already seen before
  \\ not in OEIS: 22,28,30,42,46,50,51,52,58,68,78,82,86,90,92,94,102,106,110,114,124,126,134,141,142,146,148,150,
  \\
  \\ list distinct prime factors
  \\ not in OEIS: 3,5,11,3,7,43,5,17,3,19,11,31,683,3,5,7,13,2731,43,127,3,11,331
  \\
  \\ num distinct prime factors
  \\ not in OEIS: 1,1,1,2,1,2,2,2,1,4,1,2,3,3,1,4,1,4,3,3,1,6,3,2,3,5,2,6

  \\ 2^n - (-1)^n
  \\ 4^n - 1     = (2^n + 1)*(2^n - 1)
  \\ 2*4^n + 1

  \\ J(10) == (4^5 - 1)/3
  \\ J(10) == (2^5 + 1) * (2^5 - 1) / 3
  \\ J(10) == (2^10 - 1) /3
  \\ J(10) == (2^5 - 1) * J(5)
  \\ J(11) == (2^11 + 1)/3

  \\ for(n=1,50, print(factor(2^n+1)[,1]~));
}

\\ vector(12,n, sum(i=0,n-1,2^i*fibonacci(i+2)))
\\ not in OEIS: 1, 5, 17, 57, 185, 601, 1945, 6297, 20377, 65945, 213401, 690585
\\
\\ vector(12,n, sum(i=0,n-1,2^i*fibonacci(i)))
\\ A014334 exponential convolution
\\ 0,0,2,6,22,70,230,742,2406
\\ recurrence_guess(vector(12,n, sum(i=0,n-1,2^i*fibonacci(i))))
\\ -2/5 + gf (2/5 + 8/5*x)/(1 - 2*x - 4*x^2)
\\ g = 2*x/(1 - 3*x - 2*x^2 + 4*x^3)
\\ g + g/2/x
\\
\\ recurrence_guess(vector(12,n, sum(i=0,n-1,2^i*fibonacci(i+2))))
\\ -3/5 + gf (8/5 + 12/5*x)/(1 - 2*x - 4*x^2)
\\ (1 + 2*x)/(1 - 3*x - 2*x^2 + 4*x^3)
\\
\\ polroots(polrecip(1 - 2*x - 4*x^2))
\\ phi*2.0
\\ 2.0/phi
\\ sqrt5 = quadgen(20)
\\ phi = (1+sqrt5)/2
\\ recurrence_guess(vector(10,n, 2^n * (phi^n - (1/phi)^n) / (phi - 1/phi)))
\\ (1 + (1 - sqrt5)*x) * (1 + (1 - sqrt5)*x)

{
  check_equal(lift((1+'x)*Mod('a+'b*'x,'x^2-2*'x-4)),
              ('a + 4*'b) + ('a + 3*'b)*'x);
  check_equal(1    % ('x^2-2*'x-4),  1);
  check_equal('x   % ('x^2-2*'x-4),  'x);
  check_equal('x^2 % ('x^2-2*'x-4),  2*'x + 4);
  check_equal('x^3 % ('x^2-2*'x-4),  8*'x + 8);
  check_equal(1/'x % ('x^2-2*'x-4),  1/4*('x - 2));
  check_equal(((4+4*'x)/'x) % ('x^2-2*'x-4),  ('x+2));

  check_equal(vector(10,k,k--; 2^k*fibonacci(k+2)),
              vector(10,k,k--; polcoeff(lift((4+4*'x)*Mod('x,'x^2-2*'x-4)^(k-1)),1)));
  check_equal(vector(10,k,k--; 2^k*fibonacci(k+2)),
              vector(10,k,k--; polcoeff(lift(Mod('x,'x^2-2*'x-4)^(k+2))/2,1)));

  check_equal(vector(10,k,k--; 2^k*fibonacci(k+2)),
              vector(10,k,k--; polcoeff(lift(('x+2)*Mod('x,'x^2-2*'x-4)^k),1)));
  \\
  \\             vector(10,k,k--; subst(lift((1+x)*Mod('x,'x^2-2*'x-4)^k),'x,'x))
  \\ vector(10,k,k--; lift(Mod(1/'x,1-2*'x-4*'x^2)^k))
  \\ vector(10,k,k--; lift(1*Mod('x,'x^2-2*'x-4)^k))
  \\             vector(10,k,k--; 2^k*fibonacci(k+2))
  \\ check_equal(vector(2^14,n,n--; A134659_by_polmod(n)), \
  \\             vector(2^14,n,n--; A134659_by_blocks(n)));
}

\\ 2^CountOneBits
\\ vector(24,n, run_length_transform_of_func(n,(n)->2^n))

\\ vector(24,n, run_length_transform_of_func(n,(n)->J(n+2)*3))
\\ not in OEIS: 9, 9, 15, 9, 81, 15, 33, 9, 81, 81, 135, 15, 135, 33, 63, 9, 81, 81, 135, 81, 729

\\ Thue-Morse
\\ vector(24,n, run_length_transform_of_func(n,(n)->(-1)^n))

\\ product run lengths A227349
\\ vector(24,n, run_length_transform_of_func(n,(n)->n))
A227349_compact(n) =
{
  my(ret=1); for(k=0,if(n,logint(n,2)),
     if(bittest(n,k), my(s=k);while(bittest(n,k++),); ret*=k-s));
  ret;
}
{
  check_equal(vector(#A227349_samples,n,n--; A227349_compact(n)),
              A227349_samples,  \\ OFFSET=0
              "A227349_compact() vs A227349_samples");
}

\\ A071053
\\ number of odd entries in trinomial (1+x+x^2)^n
\\ run lengths transform of Jacobsthal(n+2)
A071053_trinomials(n) =
{
  \\ my(ret=1);
  \\ while(n,
  \\       n >>= valuation(n,2);
  \\       n++;
  \\       my(k=valuation(n,2));
  \\       ret *= J(k+2);
  \\       n >>= k+1);
  \\ ret;

  \\ run lengths transform of J(n+2), simplifying Jorg's code
  my(ret=1); for(k=0,if(n,logint(n,2)),
    if(bittest(n,k), my(s=k);
       while(bittest(n,k++),); ret*=round((4/3)<<(k-s))));
  ret;
}
\\ A071053_trinomials(10)
{
  check_equal(vector(#A071053_samples,n,n--; A071053_trinomials(n)),
              A071053_samples,  \\ OFFSET=0
              "A071053_trinomials() vs A071053_samples");
}
{
  \\ ~/OEIS/b071053.txt
  my(g=OEIS_data_gf("A071053"));
  print("A071053 poldegree ",poldegree(g));
  check_equal(g,
              Polrev(vector(poldegree(g)+1,n,n--;A071053_trinomials(n))), \\ OFFSET=0
              "b071053.txt");
  \\ poldegree(OEIS_bfile_gf("A071053"))
}

check_equal(vector(1024,n, A071053_trinomials(n)), \
            vector(1024,n, run_length_transform_of_func(n,(n)->J(n+2))));

\\ A134659 cumulative to n inclusive
A134659(n) =
{
  n>=0 || error();
  A071053_trinomials(n) + if(n>0, A134659(n-1));
}
A134659=memoize(A134659);
{
  check_equal(vector(64,n,n--; A134659(n)),
              vector(64,n,n--; sum(i=0,n, A071053_trinomials(i))));
  check_equal(vector(#A134659_samples,n,n--; A134659(n)),
              A134659_samples,  \\ OFFSET=0
              "A134659() vs A134659_samples");
}

\\ 2^k many terms
\\ vector(8,k, A134659((1<<k)-1))
\\ 4, 12, 40, 128, 416, 1344, 4352, 14080
\\ = A087206
A134659_block_k(k) = 2^k*fibonacci(k+2);
A087206(n) = A134659_block_k(n);
{
  check_equal(vector(#A087206_samples,n,n--; A087206(n)),
              A087206_samples,  \\ OFFSET=0
              "A087206() vs A087206_samples");

  check_equal(vector(12,k, A134659_block_k(k)),
              vector(12,k, A134659(2^k-1)));
}
\\ Armend Shabani, count words letters 1..4 with no adjacent odds
\\ a(n) = 2*a(n-1) + 4*a(n-2); with a(0)=1, a(1)=4
\\ new even 2 ways, any a(n-1)
\\ new odd 2 ways, must previous even 2 ways, then any a(n-2)
A087206_pred_vector(v) = for(i=1,#v-1, if(v[i]%2 && v[i+1]%2, return(0))); 1;
{
  check_equal(vector(8,k,k--; sum(n=0,4^k-1,
                                  my(v=digits(n,4));
                                  while(#v<k, v=concat(0,v));
                                  v=apply(d->d+1,v);
                                  A087206_pred_vector(v))),
              vector(8,k,k--; A087206(k)));

  \\ the words
  \\ not in OEIS: 12,14,21,22,23,24,32,34,41,42,43,44    \\ no adjacent odd
  \\ not in OEIS: 11,12,13,14,21,23,31,32,33,34,41,43    \\ no adjacent even
  \\ my(k=2);
  \\ for(n=0,4^k-1,
  \\    my(v=digits(n,4));
  \\    while(#v<k, v=concat(0,v));
  \\    v=apply(d->d+1,v);
  \\    if(A087206_pred_vector(v),
  \\                   print1(concat(apply(d->Str(d),v))",")));
  \\ print();
  \\ quit;
}

A134659_by_blocks(n) =
{
  n++;
  my(ret=0);
  for(k=0,logint(n,2),
     if(bittest(n,k),
        ret += A071053_trinomials(n>>(k+1)) * A134659_block_k(k)));
  ret;
}
{
  check_equal(vector(1024,n, A134659_by_blocks(n)),
              vector(1024,n, A134659(n)));
  my(g=OEIS_bfile_gf("A134659"));
  print("A134659 poldegree ",poldegree(g));
  check_equal(g,
              Polrev(vector(poldegree(g)+1,n,n--; A134659_by_blocks(n))), \\ OFFSET=0
              "b134659.txt");
}

from_Zeck2_vector(v) = \
  sum(i=1,#v, my(k=#v-i); 2^k*fibonacci(k+2) * v[i]);
\\ vector(20,n, from_Zeck2_vector(binary(n)))
\\ not in OEIS: 1, 4, 5, 12, 13, 16, 17, 40, 41, 44, 45, 52, 53, 56, 57, 128, 129, 132, 133, 140

A134659_by_Zeck2(n) =
{
  my(v=binary(n+1),high=1);
  for(i=1,#v,
     if(v[i], my(len=0);
        until(i++>#v || !v[i],
              v[i] = high * round((2/3)<<len++));
        high *= round((4/3)<<len)));
  from_Zeck2_vector(v);
}
{
  check_equal(vector(2^14,n,n--; A134659_by_Zeck2(n)),
              vector(2^14,n,n--; A134659_by_blocks(n)));
}

A134659_by_polmod(n) =
{
  my(v=binary(n+1),high=1);
  for(i=1,#v, if(v[i], my(len=0);
    until(i++>#v || !v[i], v[i]=high*round((2/3)<<len++));
    high *= round((4/3)<<len)));
  polcoeff(lift(('x+2)*subst(Pol(v),'x,Mod('x,'x^2-2*'x-4))), 1);
}
{
  check_equal(vector(2^14,n,n--; A134659_by_polmod(n)),
              vector(2^14,n,n--; A134659_by_blocks(n)));
}
A134659_compact(n) =
{
  my(v=binary(n+1),high=1); for(i=1,#v, if(v[i], my(len=0);
    until(i++>#v || !v[i],
      v[i] = high * round((2/3)<<len++) * fibonacci(#v-i+2));
    high *= round((4/3)<<len)));
  fromdigits(v,2);

              \\ print(i" "high" "J(len+2)" "fibonacci(#v-i+2)" "A134659_block_k(#v-i));
  \\ print(v);
}
A134659_compact(1)
A134659(1)
[A071053_trinomials(0),A071053_trinomials(1)]
{
  check_equal(vector(2^14,n,n--; A134659_compact(n)),
              vector(2^14,n,n--; A134659_by_blocks(n)));
}

\\ to 2^k inclusive, is +3 for the 2^k term
\\ vector(8,k, A134659(1<<k))
\\ not in OEIS: 7, 15, 43, 131, 419, 1347, 4355, 14083


\\ vector(10,k, sum(i=0,2^k-1, A071053_trinomials(i)))
\\ vector(4,k, sum(i=0,2^(2^k)-1, A071053_trinomials(i)))
\\ vector(6,k, a(k))

{
  check_equal(A071053_trinomials(from_run_lengths([1,1, 2,0])),
              a(3),
              "a(3) trinomials");

  check_equal(A071053_trinomials(from_run_lengths([1,1, 1,1, 2,1, 3,0])),
              a(4),
              "a(4) trinomials");

  check_equal(A071053_trinomials(from_run_lengths([1,1, 1,1, 1,1, 1,1,
                                                   2,1, 2,1,
                                                   3,1,
                                                   4,0])),
              a(5),
              "a(5) trinomials");
}
first_trinomial_index_vector(n) =
{
  n>=0 || error();
  my(l=List([]));
  if(n>=2,
     for(i=1,n-1,
        for(j=1,2^(n-2-i),
           listput(l,i);
           listput(l,1)));
     listput(l,n-1);
     listput(l,0));
  Vec(l);
}
{
  check_equal(first_trinomial_index_vector(1), []);
  check_equal(first_trinomial_index_vector(2), [1,0]);
  check_equal(first_trinomial_index_vector(3), [1,1, 2,0]);
  check_equal(first_trinomial_index_vector(4), [1,1, 1,1, 2,1, 3,0]);

  \\ num terms
  check_equal(vector(10,n,n--; #first_trinomial_index_vector(n)),
              vector(10,n,n--; 2*if(n<2,0, 2^(n-2))));

  \\ count 1s
  check_equal(vector(10,n,n--; my(v=first_trinomial_index_vector(n),
                                  ones=vector(#v/2,i, v[2*i-1]));
                     vecsum(ones)),
              vector(10,n,n--; if(n<2,0, 2^(n-1)-1)));
}

first_trinomial_index(n) = from_run_lengths(first_trinomial_index_vector(n));
{
  for(n=0,4,
     my(target=a(n));
     print("target a("n") = "target);
     for(idx=0,oo,
        if(A071053_trinomials(idx)==target,
           check_equal(first_trinomial_index(n),
                       idx,
                       Str("first_trinomial_index() vs search n="n));
           break)));
  check_equal(first_trinomial_index(5),
              2799343);
  check_equal(test_to_binary(first_trinomial_index(5)),
              1010101011011011101111);

  check_equal(first_trinomial_index(6),
              46912700660703);
  check_equal(test_to_binary(first_trinomial_index(6)),
              1010101010101010110110110110111011101111011111);
}
\\ vector(9,n,n--; first_trinomial_index(n))
\\ not in OEIS: 11, 695, 2799343, 46912700660703, 13204693753255806004329574335
\\
\\ vector(8,n,n--; my(m=first_trinomial_index(n)); bitneg(m,test_bitlength(m)))
\\ not in OEIS: 4, 328, 1394960, 23456043516960, 6602346875310278394056413248
\\ vector(5,n,n+=2; my(m=first_trinomial_index(n)); m=bitneg(m,test_bitlength(m)); m>>=valuation(m,2))
\\ not in OEIS: 1, 41, 87185, 733001359905, 103161669926723099907131457

first_trinomial_index_compact(n) =
{
  if(n<2,0, my(v=vector(3*2^(n-2)-2),upto=0);
     for(i=1,n-1, for(j=1,2^max(0,n-2-i), for(k=1,i, v[upto++]=1); upto++));
     fromdigits(v,2));
}
{
  check_equal(vector(6,n,n--; first_trinomial_index_compact(n)),
              vector(6,n,n--; first_trinomial_index(n)),
              "first_trinomial_index_compact()");
}

{
  check_equal(vector(8,n, A071053_trinomials(first_trinomial_index(n))),
              vector(8,n, a(n)),
              "a() by first_trinomial_index()");

  check_equal(vector(8,n,n++; duplicate_runs(2*first_trinomial_index(n-1)) + 1),
              vector(8,n,n++; first_trinomial_index(n)),
              "first_trinomial_index() recurrence duplicate_runs()");

  \\ bitlength 3*2^(n-2) - 2
  check_equal(vector(10,n,n--; test_bitlength(first_trinomial_index(n))),
              vector(10,n,n--; if(n<2,0, 3*2^(n-2) - 2)));
}

\\ immediately above each run
\\ vector(20,n, n=run_lengths_minimum(n); bitnegimply(n<<1,n))
\\ not in OEIS: 2, 2, 4, 2, 10, 4, 8, 2, 10, 10, 20, 4, 20, 8, 16, 2, 10, 10, 20, 10

\\ runs of 1-bits longest most significant
\\ vector(20,n, fromdigits(Vecrev(binary(run_lengths_minimum(n))),2))
\\ not in OEIS: 1, 1, 3, 1, 5, 3, 7, 1, 5, 5, 13, 3, 13, 7, 15, 1, 5, 5, 13, 5

quit

\\-----------------------------------------------------------------------------
\\ C Compact A338294

\\ x^k / k!
exp(x+O(x^8))

num_digits = 100;
num_terms_logC = 4*num_digits;
num_terms_exp  = num_digits;

this = prev = 0;
for(t=1,num_terms_logC, \
   prev=this; this += (-1/(-3)^t + 1/((-2)^(t+1) + 1))/t);
[lo,hi] = vecsort([prev,this]);

exp(lo)
exp(hi)
s = exp(x+O(x^num_terms_exp))
lo = subst(Pol(s),x,lo);
s_bound = serchop(s,num_terms_exp-1);  \\ highest term
hi = subst(Pol(s + s_bound),x,hi);
f = 10^num_digits;
lo=floor(f*lo);
hi=floor(f*hi);
lo == hi || error("oops, different up to num_digits");
digits(lo)

quit
\\ exp_range(lo,hi) =
\\ {
\\ }
\\ 
\\ A338294_vector(len) =
\\ {
\\   my(scale=10^len,
\\      prev_logC=0, 
\\      logC=0,      
\\      t=1);        
\\   while(1,
\\         sign(log43_term)    == -(-1)^t           || error();  \\ alternating
\\         s += scale/t * (log43_term + logCrest_term);
\\ 
\\         my(this=floor(s));
\\         if(this==prev, return(digits(this)));
\\         prev=this;
\\         t++);
\\ }


\\-----------------------------------------------------------------------------
\\ b-file A338293 Matchings

system("rm -f /tmp/b338293.txt");
{
  for(n=0,oo,
     my(s=Str(a(n)));
     print("n="n" s length ",length(s));
     if(length(s)>999,
        print("  too big");
        break);
     write("/tmp/b338293.txt", Str(n" "s)));
}
system("ls -l /tmp/b338293.txt");
system("wc -l /tmp/b338293.txt");
system("cat /tmp/b338293.txt");

\\ s = OEIS_bfile_gf("A338293");
\\ F = sum(n=1,4095, a(n)*x^n);
\\ check_equal(F, s, "b338293.txt vs F");

quit


\\-----------------------------------------------------------------------------
\\ Other Starts (none)

{
  my(m=Map(),
    verbose=0);
  for(x=-2,8,
     for(y=-2,8,
        if(x==0 && y==0, next);
        if(verbose, print("\nx="x" y="y" gives"));
        my(x=x,y=y,l=List([]));
        for(i=1,10,
           if(#digits(y)>60,break);
           listput(l,y);
           [x,y] = [x^2 + 2*x*y^2, x]);
        l=Vec(l);
        if(verbose, print("  "l));
        if(l[#l]==0 || l[#l]==-1, next);
        my(s=concat(apply(t -> Str(","t), l[4..#l])));
        if(mapisdefined(m,s),next);
        mapput(m,s,1);
        s=Str("perl -MMath::OEIS::Grep=-search"s);
        if(verbose, print("  ",s));
        system(s);
     ));
  quit;
}

\\-----------------------------------------------------------------------------
\\ Kropf and Wagner

s(n) = 1/3*(2^(n+2) - (-1)^n); \\  = Chi(n)
T(n) = run_length_transform_of_func(n,s);

\\ log(t_n) = sum numruns_i(n) * log(s_i)
log_of_t(n) =
{
  my(v=n_to_run_lengths(n));
  sum(i=1,#v, log(s(v[i])));
}
{
  for(n=1,100,
     nearly_equal(log_of_t(n), log(T(n))) || error());
}

mu = suminf(i=1, log(s(i))*2^(-i-2))
my(n=20000); sum(i=1,n,log_of_t(i)) / n / log2(n)

\\ N_k uniform in 0 .. q^k-1 which for q=2 is n=2^k terms
\\ k = log2(n)
\\ mean mu*k + O(1)
\\      mu*log2(n)
\\     

quit





\\-----------------------------------------------------------------------------
\\ complete binary tree sequences and their offsets
\\
\\ A158681   Wiener index, height h>=1
\\ A076725   indsets count, 2^(n-1)-1 vertices, n-1 levels, height n-2
\\ 
\\ A220837  Normalized position numbers of distant parents in complete binary trees.
\\ A157679  Number of subtrees of a complete binary tree   n nodes
\\ A286778  Sum of the common path length over all 2-tuples of nodes in a complete binary tree of height n.


\\-----------------------------------------------------------------------------
\\ Wynn's Epsilon Method

\\ not in OEIS: 0.14226536089
logCpart_approx = suminf(k=1, log(1 - 1/(-2)^k) / 2^k);
print("log(4/3)         ", log(4/3));
print("log(C)           ", logCpart_approx + log(4/3));
print();

logCpart_term(t) =
{
  t>=1 || error();
  1/t * 1/((-2)^(t+1) + 1);
}

s(n) = sum(t=1,n, logCpart_term(t));
s=memoize(s);
for(n=1,12,print(s(n)*1.0))

\\ iteration j, term k
\\                                   1
\\ e(j)[k+1] = e(j+1)[k-1] + -------------------
\\                           e(j+1)[k] - e(j)[k]
\\
\\ series S_n = sum_{k=0}^{n} a_k
\\                                             1
\\ e_(r+1)(S_n) = e_(r-1)(S_(n+1)) + ----------------------
\\                                   e_r(S_(n+1)) - e_r(S_n)

Wynn(func,r,n) =
{
  r>=-1 || error();
  if(r==-1,return(0));

  n>=0 || error();
  n>=r || return(0);
  if(r==0,return(func(n)));

  Wynn(func,r-2,n+1) + 1/( Wynn(func,r-1,n+1) - Wynn(func,r-1,n) );
}
vector(6,n, Wynn(s,-1,n))
vector(6,n, s(n))
vector(6,n, Wynn(s,0,n))
vector(6,n, Wynn(s,1,n))
{
  \\ my(n_list=[1,2,3,4,10,20,50,100,200,500]);
  \\ for(i=1,#n_list,
  \\    my(n=n_list[i]);
  for(n=1,10,
     forstep(r=0,4,2,
        my(fmt=if(r==0, " %8.22f",  \\ s
                  r==1, " %12.2f",
                  r==2, " %9.22f",  \\ again
                  r==3, " %16.2f",
                        " %.22f"));
        printf(fmt,Wynn(s,r,n)));
     print());
}
print("logCpart_approx  ", logCpart_approx);

vector(10,n,Wynn(s,1,n)-Wynn(s,1,n-1))
v=vector(100,n,Wynn(s,1,n+1)-Wynn(s,1,n));
\\ v=vector(100,n,n--; my(t=Wynn(s,0,n)); if(t,1/t,0));
recurrence_guess(v)
\\ v=vector(50,n,Wynn(s,3,n)-Wynn(s,3,n-1));
\\ recurrence_guess(v)

{
  check_equal(vector(10,n,Wynn(s,0,n)),
              vector(10,n, s(n)),
              "Wynn r=0 is s");
  check_equal(vector(100,n,Wynn(s,1,n)),
              vector(100,n, 4*(n+1)*(-2)^n + n + 1),
              "Wynn r=1");
  check_equal(vector(100,n, Wynn(s,1,n+1) - Wynn(s,1,n)),
              vector(100,n, - (12*n+20) *(-2)^n + 1),
              "Wynn delta r=1");

  check_equal(vector(100,n,n++; Wynn(s,2,n)),
              vector(100,n,n++; 
                     s(n+1) - 1/((12*n+20)*(-2)^n - 1)
                    ),
              "Wynn r=2");
  check_equal(vector(10,n,n++; Wynn(s,2,n+1) - Wynn(s,2,n)),
              vector(10,n,n++; 
                     \\ s(n+2) - s(n+1)
                     logCpart_term(n+2)
                     + 1/((24*n+64)*(-2)^n + 1)
                     + 1/((12*n+20)*(-2)^n - 1)
                    ),
              "Wynn incr r=2 S");
}
quit


\\-----------------------------------------------------------------------------
\\ log(C) logs to fracs

\\ log(1-x) = - x - 1/2*x^2 - 1/3*x^3 - 1/4*x^4 - 1/5*x^5

log(4/3) + sum(k=1,40, log(1 - 1/(-2)^k) /2^k)

\\ log(1 - 1/(-2)^k) /2^k
\\ x = 1/(-2)^k
\\
\\          -1        -1/2*     -1/3*
\\ k=1   1/(-2)^1   1/(-2)^2   1/(-2)^3      /2^1
\\ k=2   1/(-2)^2   1/(-2)^4   1/(-2)^6      /2^2
\\ k=3   1/(-2)^3   1/(-2)^6   1/(-2)^9      /2^3
\\ k=4   1/(-2)^4   1/(-2)^8   1/(-2)^12     /2^4
\\
\\          -1        -1/2*     -1/3*       -1/4*   
\\ k=1  -1/(-2)^2  -1/(-2)^3  -1/(-2)^4   -1/(-2)^5 
\\ k=2   1/(-2)^4   1/(-2)^6   1/(-2)^8    1/(-2)^10
\\ k=3  -1/(-2)^6  -1/(-2)^9  -1/(-2)^12  -1/(-2)^15
\\ k=4   1/(-2)^8   1/(-2)^12  1/(-2)^16   1/(-2)^20
\\
\\          1/3      -1/9        1/15

\\ row
my(k=3); sum(c=1,10, -1/c * 1/((-2)^k)^c  *1.0) /2^k
my(k=3); sum(c=1,10, -1/c * 1/(-2)^(k*c)  *1.0) /2^k
my(k=3); sum(c=1,10, -1/c * 1/(-2)^(k*c) /2^k  *1.0) 
my(k=3); sum(c=1,10, -1/c * 1/(-2)^(k*c) /(-2^k) * (-1)^k  *1.0) 
my(k=3); sum(c=1,10, -(-1)^k/c * 1/(-2)^(k*(c+1))  *1.0) 
my(k=3); log(1 - 1/(-2)^k) /2^k

my(k=4); sum(c=1,10, -(-1)^k/c * 1/(-2)^(k*(c+1))  *1.0) 
my(k=4); log(1 - 1/(-2)^k) /2^k

\\ columns   
\\ recurrence_guess([0,5,-14,51,-124,325,-762,1799,-4088,9225,-20470,45067,-98292,213005,-458738,983055,-2097136,4456465,-9437166,19922963,-41943020])
for(c=1,6, my(s=sum(k=1,50, -(-1)^k/c * 1/(-2)^(k*(c+1))  *1.0)); \
   print(s"  "bestappr(s,100000000000))); 
   print1(1/bestappr(s,100000000000)",")); 
for(t=1,6, print(1/(t * ((-2)^(t+1) + 1))))

sum(k=1,100, log(1 - 1/(-2)^k) /2^k)    + log(4/3)
sum(c=1,100, 1/(c * ((-2)^(c+1) + 1)))  + log(4/3)
sum(c=1,100, (-1)^c/(c * ((-1)^c*(-2)^(c+1) + (-1)^c)))  + log(4/3)
sum(c=1,100, 1/c * (-1)^c /((-1)^c*(-2)^(c+1) + (-1)^c))  + log(4/3)
sum(c=1,100, 1/c * (-1)^c /((-1)^c*(-2)^(c+1) - (-1)^(c+1)))  + log(4/3)
sum(c=1,100, 1/c * (-1)^c /((-1)^c*(-2)^(c+1) - (-1)^(c+1)))  + log(4/3)

\\ log(C) =
sum(c=1,100, (-1)^(c+1) * (1/c) / (2^(c+1) + (-1)^(c+1)))  + log(4/3)
sum(c=1,100, 1/c * (-1)^(c+1) / (2^(c+1) + (-1)^(c+1))) + \
sum(c=1,100, 1/c * (-1)^(c+1) / 3^c * 1.0)

sum(c=1,100, (-1)^(c+1) * (1/c) / (2^(c+1) + (-1)^(c+1))  *1.0)
sum(c=1,100, (1/c) / ((-2)^(c+1) + 1)  *1.0)

\\ cf Lambert
sum(c=1,1000, 1/((-2)^c - 1) *1.0)
my(x=-2);   sum(k=1,100, 1/(x^k - 1) * 1.0) 
my(x=1/-2); sum(k=1,100, x^(k^2)*(1+x^k)/(1-x^k) * 1.0)   \\ Lambert series

quit


\\------------------------
\\ sums without the 1/c factor

sum(c=1,1000, (-1)^(c+1) /(2^(c+1) + (-1)^(c+1)) *1.0)
\\ not in OEIS: 0.093802796024
sum(c=0,1000, (-1)^(c+1) /(2^(c+1) + (-1)^(c+1)) *1.0)
\\ not in OEIS: 0.9061972039759
sum(c=0,1000, (-1)^c /(2^c + (-1)^c) *1.0)
\\ not in OEIS: 0.4061972039759

sum(c=0,1000, 1/(2^c + 1) *1.0)
\\ 1.26449978034844
\\ A323482  sum 1/(2^k + 1)
\\ A179951  sum 1/(2^k + 1/2)
\\
sum(c=1,1000, 1/(2^c - 1) *1.0)
\\ 1.60669515241529
\\ A065442
\\ Erdos-Borwein constant
\\ Sum_{n>=1} x^(n^2) * (1+x^n)/(1-x^n)      x=1/2
\\
\\ A160502 sum with one 0 bit 
\\
sum(c=0,1000, 1/(4^c + 1) *1.0)
\\ not in OEIS: 0.7794002624059
\\
sum(c=1,1000, 1/(4^c - 1) *1.0)
my(x=1/4); sum(k=1,100, x^(k^2)*(1+x^k)/(1-x^k) * 1.0)   \\ Lambert series
\\ 0.42109768603342
\\ A248721
\\
sum(c=1,1000, 1/(10^c - 1) *1.0)
\\ 0.122324243426244
\\ A073668
\\ Some Interesting Results On d(N) the Number of Divisors of a Natural Number
\\ https://dl.acm.org/doi/abs/10.5555/362085.362129    cookies demanded
\\ Volume 8, Issue 2, Oct. 2000
\\ https://doi.org/10.5555/362085.362129



\\-----------------------------------------------------------------------------
\\ C limit by product

\\ just under 1 bit per term, ascending
\\ so 2^100'th roots ...

c_prod_part(n) = prod(i=1,n, J(i)^(1/2^i))
{
  my(prev=1);
  for(n=1,40,
     my(this=c_prod_part(n));
     print(this"   ",if(this!=prev,log(this-prev)/log(2)));
     prev=this);
  quit;
}


\\------------------------------------------------------------------------------
\\ A242049
\\ cf 0.6202830299260946960737425

\\ vector(20,n, vecsum((Vec((1+x+x^2)^n)%2)))
\\ 3, 3, 5, 3, 9, 5, 11, 3, 9, 9, 15, 5, 15, 11, 21, 3, 9, 9, 15, 9
\\ A071053
\\ count of odd coeffs

\\ Y. Moshe, The Distribution of Elements in Automatic Double Sequences,
\\    Discrete Math. volume 297, 2005, 91103


f(n) = 1 + (-1)^n * (3/2)/(2^n - (-1)^n);


A242049_term(k) = ((1/3)*(2^(k+2) - (-1)^k));
check_equal(vector(20,k, A242049_term(k)), \
            vector(20,k, J(k+2)));


\\------------------------------------------------------------------------------

log_1_plus_x_poly_for_bits(x_bits,upper, target_bits) =
{
  my(bigpow = ceil(target_bits / x_bits));
  if(bigpow%2 != upper, bigpow++);
  log(1 + 'x + O(x^(bigpow+1)));
}
log_1_plus_x_poly_for_bits(3,1, 20)
log_1_plus_x_poly_for_bits(3,0, 20)
subst(Pol(log_1_plus_x_poly_for_bits(3,1, 20)),'x,1/8.0)
log(1 + 1/8)
subst(Pol(log_1_plus_x_poly_for_bits(3,0, 20)),'x,1/8.0)

log_1_minus_x_poly_for_bits(x_bits,upper, target_bits) =
{
  my(bigpow = ceil(target_bits / x_bits),
     p=log(1 - 'x + O(x^(bigpow+1))));
  if(!upper,
     p += polcoeff(lift(p),bigpow)*'x^bigpow);
  p;
}
log_1_minus_x_poly_for_bits(3,1, 20)
log_1_minus_x_poly_for_bits(3,0, 20)
subst(Pol(log_1_minus_x_poly_for_bits(3,1, 20)),'x,1/8.0)
log(1 - 1/8)
subst(Pol(log_1_minus_x_poly_for_bits(3,0, 20)),'x,1/8.0)


logsum(n_limit,upper, target_bits) =
{
  sum(n=1,n_limit,
      \\ 1/2^n * log(1 +  (-1)^n * (3/2)/(2^n - (-1)^n)) ))),
      my(x = (3/2)/(2^n - (-1)^n),
         p = if(n%2==0,
                log_1_plus_x_poly_for_bits (n,upper, target_bits-n),
                log_1_minus_x_poly_for_bits(n,upper, target_bits-n)));
      f(n) == 1 + (-1)^n * x || error();
      \\ print("n="n" "p);
      subst(Pol(p),'x,x) / 2^n);
}

logsum_by_logs(n) = \
  sum(i=1,n, log(1 + (3/2)*(-1)^i / (2^i - (-1)^i)) /2^i);

f(1)
logsum(1,1, 30) * 1.0
log(f(1))/2.0
logsum(1,0, 30) * 1.0
logsum_by_logs(1)

f(2)
logsum(2,1, 30) * 1.0
log(f(1))/2.0 + log(f(2))/4
logsum(2,0, 30) * 1.0
logsum_by_logs(2)

print("20");
u=ceil (logsum(800,1, 800) * 10^120)
v=floor(logsum(800,0, 800) * 10^120)
logsum_by_logs(20)
logsum_by_logs(20)^(1/2)
2*exp(logsum_by_logs(20))
log(2*exp(logsum_by_logs(20)))
log(2) + logsum_by_logs(20)

(0 <= u-v && u-v <= 1) || error();

{
  localprec(500);
  log2 = log(2);
  v += floor(10^120 * log2);
}
vec=OEIS_samples("A242049")
digits(v)[1..#vec] == vec

(1/4)*sum(k=1,10, log((1/3)*(2^(k+2) - (-1)^k))/2^k)



\\------------------------------------------------------------------------------
print("a(0) = ",a(0));
print("a(1) = ",a(1));
print("c(0) = ",c(0));
d(n) = (-1)^n * (log(c(n)) - log(c(n-1)));

my(n=6); a(n) / a(n-1)^2.0
{
  check_equal(vector(8,n,n++; a(n) / a(n-1)^2 ),
              vector(8,n,n++; 1 + 2*a(n-1)*a(n-2)^2 / a(n-1)^2));
  check_equal(vector(8,n,n++; a(n) / a(n-1)^2 ),
              vector(8,n,n++; 1 + 2*a(n-2)^2 / a(n-1)));
  check_equal(vector(8,n,n++; a(n) / a(n-1)^2 ),
              vector(8,n,n++; 1 + 2 / (a(n-1)/a(n-2)^2)));

  check_equal(vector(8,n,n++; 1/2 * a(n) / a(n-1)^2 ),
              vector(8,n,n++; 1/2 + a(n-1)*a(n-2)^2 / a(n-1)^2));
  check_equal(vector(8,n,n++; 1/2 * a(n) / a(n-1)^2 ),
              vector(8,n,n++; 1/2*(1 + 1 / (1/2 * a(n-1) / a(n-2)^2))));

  check_equal(1 + 2, 3);
  check_equal(1 + 2/(1 + 2), 5/3);
  check_equal(1 + 2/(1 + 2/(1 + 2)), 11/5);
  check_equal(1 + 2/(1 + 2/(1 + 2/(1 + 2))), 21/11);
  check_equal(1 + 2/(1 + 2/(1 + 2/(1 + 2/(1 + 2)))), 43/21);
  check_equal(1 + 2/(1 + 2/(1 + 2/(1 + 2/(1 + 2/(1 + 2))))), 85/43);
  check_equal(1 + 2/(1 + 2/(1 + 2/(1 + 2/(1 + 2/(1 + 2/(1 + 2)))))), 171/85);
  \\ 5,11,21,43,85,171
  \\ A001045 Jacobsthal   like (2^n)/3
}
\\ 1/b[0] * (a0 + b[1]/(a1 + ...
\\ contfrac(1/2,[1,2,2,2,2,2],5)

{
  check_equal(a(1), J(2),
              "a(1) by J products");
  check_equal(a(2), J(3) * J(2),
              "a(2) by J products");
  check_equal(a(3), J(4) * J(3) * J(2)^2,
              "a(3) by J products");
  check_equal(a(6), J(7) * J(6) * J(5)^2 * J(4)^4 * J(3)^8 * J(2)^16,
              "a(6) by J products");
}


\\ for(n=1,6, print(n"  "d(n)"  ", c(n+1)-c(n)));
{
  for(n=2,12,
     printf("%d  %13.10f  %15.10f %13.10f\n  %31.10f    %13.10f\n",
            n, d(n), d(n)-1/4*d(n-1), (-1/2)^n*log(cosh(d(n-1)*2^(n-2))),
           1/4*d(n-1),
           1/4*d(n-1) + (-1/2)^n*log(cosh(d(n-1)*2^(n-2)))
           ));
}
{
  for(n=2,15,
     printf("%d  %13.10f  %15.10f\n",
            n, d(n), d(n)*4^n
           ));
}
check_nearly_equal(d(1), log(c(0)) - log(c(1)) );
check_nearly_equal(c(0), 2);
check_nearly_equal(c(1), sqrt(2));
check_nearly_equal(d(1), log(2) - log(sqrt(2)) );
check_nearly_equal(d(1), log(2/sqrt(2)) );
check_nearly_equal(d(1), log(sqrt(2)) );
check_nearly_equal(log(c(1)), log(sqrt(2)) );
check_equal(log(c(1)) < 1/2, 1 );
\\ 0.3465735902
\\ A016655 = log(32) = 5*log(2)


{
  \\ log(c(n)) - log(c(n-1)) = (-1)^n * d(n)
  check_nearly_equal(vector(8,n,n++; log(c(n)) - log(c(n-1)) ),
                     vector(8,n,n++; (-1)^n * d(n)),
                     "log(c(n)) - log(c(n-1))");

  check_nearly_equal(vector(8,n,n++; log(c(n)/c(n-1)) ),
                     vector(8,n,n++; (-1)^n * d(n)),
                     "log(c(n)/c(n-1))");
}
\\ log(c(n)) - log(c(n-1)) = (-1)^n * d(n)
\\ log(c(n)/c(n-1)) = (-1)^n * d(n)
\\ c(n)/c(n-1) = exp((-1)^n * d(n))


{
  \\ log(c(n)) = d(1) - d(2) + d(3) - ... +- d(n)
  check_nearly_equal(log(c(1)), log(c(1)));
  check_nearly_equal(log(c(2)), log(c(1)) + (log(c(2)) - log(c(1))), "log(c(2))");
  check_nearly_equal(log(c(3)),
                     log(c(1))
                     + (log(c(2)) - log(c(1)))
                     + (log(c(3)) - log(c(2))), "log(c(3))");
  check_nearly_equal(log(c(4)),
                     log(c(1))
                     + (log(c(2)) - log(c(1)))
                     + (log(c(3)) - log(c(2)))
                     + (log(c(4)) - log(c(3))), "log(c(4))");
  check_nearly_equal(log(c(2)),
                     log(c(1)) + d(2), "log(c(2)) by d series");
  check_nearly_equal(log(c(3)),
                     log(c(1)) + d(2) - d(3),
                     "log(c(3)) by d series");
  check_nearly_equal(log(c(4)),
                     log(c(1)) + d(2) - d(3) + d(4),
                     "log(c(4)) by d series");
  check_nearly_equal(log(c(5)),
                     log(c(1)) + d(2) - d(3) + d(4) - d(5),
                     "log(c(5)) by d series");
  check_nearly_equal(vector(15,n,n++; log(c(n))),
                     vector(15,n,n++; log(c(1)) + sum(i=2,n, if(i%2==0,1,-1) * d(i))),
                     "log(c(n)) by d series");
}

\\ 1/2*c(n)^(2^n) = (1/2*c(n-1)^(2^(n-1)))^2 + 2*(1/2*c(n-1)^(2^(n-1)))*(1/2*c(n-2)^(2^(n-2)))^2
\\ 1/2*c(n)^(2^n) = 1/4*c(n-1)^(2^n) + 1/4*c(n-1)^(2^(n-1))*c(n-2)^(2^(n-1))

{
  \\ c(n)^(2^n) = 1/2*c(n-1)^(2^n) + 1/2*c(n-1)^(2^(n-1))*c(n-2)^(2^(n-1))
  check_nearly_equal(vector(5,n,n++; c(n)^(2^n)),
                     vector(5,n,n++; 1/2*c(n-1)^(2^n) + 1/2*c(n-1)^(2^(n-1))*c(n-2)^(2^(n-1))),
                     "c(n)^(2^n)");
}


{
  \\ (c(n)/c(n-1))^(2^n) = 1/2 + 1/2*(c(n-1)/c(n-2))^(-2^(n-1))
  check_nearly_equal(vector(8,n,n++; (c(n)/c(n-1))^(2^n) ),
                     vector(8,n,n++; 1/2 + 1/2*(c(n-1)/c(n-2))^(-2^(n-1))),
                     "(c(n)/c(n-1))^(2^n)");
}
{
  \\ log( (c(n)/c(n-1))^(2^n) ) = log( 1/2 + 1/2*(c(n-1)/c(n-2))^(-2^(n-1)) )
  check_nearly_equal(vector(8,n,n++; log( (c(n)/c(n-1))^(2^n) ) ),
                     vector(8,n,n++; log( 1/2 + 1/2*(c(n-1)/c(n-2))^(-2^(n-1)) )),
                     "(c(n)/c(n-1))^(2^n)");
}

{
  \\ 2^n * (-1)^n * d(n) = log(1/2 + 1/2*(c(n-1)/c(n-2))^(-2^(n-1)) )
  check_nearly_equal(vector(8,n,n++; 2^n * (-1)^n * d(n) ),
                     vector(8,n,n++; log( 1/2 + 1/2*(c(n-1)/c(n-2))^(-2^(n-1)) )),
                     "2^n * (-1)^n * d(n)");
}
{
  \\ 2^n * (-1)^n * d(n) = log(1/2 + 1/2* exp((-1)^(n-1) * d(n-1))^(-2^(n-1)) )
  check_nearly_equal(vector(8,n,n++; 2^n * (-1)^n * d(n) ),
                     vector(8,n,n++; log(1/2 + 1/2* exp((-1)^(n-1) * d(n-1))^(-2^(n-1)) )),
                     "2^n * (-1)^n * d(n)");
}
{
  \\ d(n) = (-2)^-n * log(1/2 + 1/2* exp((-1)^(n-1) * d(n-1) * -2^(n-1)) )
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log(1/2 + 1/2* exp((-1)^(n-1) * d(n-1) * -2^(n-1)) )),
                     "d(n)");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log(1/2 + 1/2* exp(- (-2)^(n-1) * d(n-1)) )),
                     "d(n)");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log( ( 1/2*exp(-(-2)^(n-2) * d(n-1))
                                                     + 1/2* exp(- (-2)^(n-1) * d(n-1)) * exp(-(-2)^(n-2) * d(n-1)) )
                                                   / exp(-(-2)^(n-2) * d(n-1)) )),
                     "d(n) exp div");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log( ( 1/2 *exp( - (-2)^(n-2) * d(n-1))
                                                     + 1/2*exp(((-1)^n * (2^(n-1) - 2^(n-2))) * d(n-1)) )
                                                   / exp(- (-2)^(n-2) * d(n-1)) )),
                     "d(n) exp div 2");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log( ( 1/2 *exp( - (-2)^(n-2) * d(n-1))
                                                     + 1/2*exp(((-1)^n * (2^(n-1) - 2^(n-2))) * d(n-1)) )
                                                   / exp(- (-2)^(n-2) * d(n-1)) )),
                     "d(n) exp div 2");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log( (  1/2 *exp( - (-2)^(n-2) * d(n-1))
                                                     + 1/2*exp(    (-2)^(n-2) * d(n-1)) )
                                                   / exp(- (-2)^(n-2) * d(n-1)) )),
                     "d(n) exp div 2");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log( cosh((-2)^(n-2) * d(n-1))
                                                   / exp(- (-2)^(n-2) * d(n-1)) )),
                     "d(n) exp div 2");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * ( (-2)^(n-2) * d(n-1) +
                                                log( cosh((-2)^(n-2) * d(n-1))
                                                   ))),
                     "d(n) exp div 2");
}
{
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (1/4)*d(n-1) + (-1/2)^n*log(cosh(d(n-1)*2^(n-2)))),
                     "d(n) 1/4");
}
\\ plot(x=-3,3,x^2 - log(cosh(x)) )
\\ plot(x=0,3,x - log(cosh(x)) )
\\ plot(x=0,3, exp(x) - cosh(x) )
\\ log(cosh(x)) < x^2
check_equal(cosh(x), 1/2*(exp(x) + exp(-x)), "cosh");

{
  \\ 1/K * log( cosh('d * K/4 ))
  check_equal(vector(8,n,n++; (1/2)^n * log( cosh(d(n-1) * 2^(n-2) ))
                            < (1/2)^n * (d(n-1) * 2^(n-2) )^2 ),
              vector(8,n,n++; 1),
              "using cosh <= x^2");
}
{
  check_nearly_equal(vector(8,n,n++; (1/2)^n * (d(n-1) * 2^(n-2) )^2),
                     vector(8,n,n++; (1/2)^n * d(n-1)^2 * 2^(2*n-4) ),
                     "d(n-1)^2");
}
{
  check_nearly_equal(vector(8,n,n++; (1/2)^n * (d(n-1) * 2^(n-2) )^2),
                     vector(8,n,n++; 1/16 * 2^n * d(n-1)^2 ),
                     "d(n-1)^2");
}
{
  check_equal(vector(10,n,n++; d(n) < 1/2*d(n-1)),
              vector(10,n,n++; 1),
              "d(n) < 1/2*d(n-1)");
}
{
  check_equal(vector(10,n,n++; d(n) < 1/2^n),
              vector(10,n,n++; 1),
              "d(n) < 1/2^n");
}

{
  \\ log(cosh(x)) < x
  forstep(x=1/100,10,1/100,
     log(cosh(x)) < x || error());

  check_equal(x - log(cosh(x)) + O(x^12),
              x - 1/2*x^2 + 1/12*x^4 - 1/45*x^6 + 17/2520*x^8 - 31/14175*x^10 + O(x^12));
}

{
  \\ using log(cosh(x)) < x
  check_equal(vector(8,n,n++; (1/2)^n * log( cosh(d(n-1) * 2^(n-2) ))
                            < (1/2)^n * d(n-1) * 2^(n-2) ),
              vector(8,n,n++; 1),
              "using cosh <= x");
}
{
  \\ which goes to 0 < d(n) < 2*1/4*d(n-1)
  check_equal(vector(10,n,n++; (1/2)^n * d(n-1) * 2^(n-2) ),
              vector(10,n,n++; 1/4 * d(n-1)),
              "d(n) < ... ");
}


{
  my(n=11,
     decimals=6,
     tenfac=10^decimals,
     l=List([]));
  for(n=n,n+1,
     my(r=2*a(n),
        s=sqrtnint(r * tenfac^(2^n), 2^n));
     ( s   /tenfac)^(2^n) <= r || error();
     ((s+1)/tenfac)^(2^n) > r || error();
     print(s"    "s/tenfac*1.0);
     listput(l,s));
  if(l[1]!=l[2], error());
}

\\ default(realprecision,200);
\\ my(n=25); print(c(n)); print(c(n+1));
\\ 1.537176717
\\ A338294 (mine)

\\ 1.537176717 / sqrt(2)
\\ not in OEIS: 1.086948
\\ not A201943 middle match
\\
\\ log(1.537176717182)
\\ 0.4299474333
\\ A242049



t(n) = 2^n * d(n);
{
  check_nearly_equal(vector(8,n,n++; t(n)),
                     vector(8,n,n++; (-1)^n * 2^n * (log(c(n)) - log(c(n-1)))),
                     "t(n) and c");

  check_nearly_equal(vector(6,n,n++; t(n)),
                     vector(6,n,n++; (-1)^n * 2^n * log( c(n)/c(n-1) )),
                     "t(n) ratio");
  check_nearly_equal(vector(6,n,n++; t(n)),
                     vector(6,n,n++; (-1)^n * log( c(n)^(2^n) / c(n-1)^(2^n) )),
                     "t(n) ratio power");

  check_nearly_equal(vector(6,n,n++; t(n)),
                     vector(6,n,n++; (-1)^n * log( 2*a(n) / c(n-1)^(2^n) )),
                     "t(n) ratio power");

  check_nearly_equal(vector(6,n,n++; t(n)),
                     vector(6,n,n++; (-1)^n * log( 1/2 * a(n)/ a(n-1)^2 )),
                     "t(n) a");
}

f(n) = 1/2 * a(n) / a(n-1)^2;
{
  check_equal(vector(10,n,n++; f(n)),
              vector(10,n,n++; 1/2 + 1/2 /f(n-1)),
              "f(n) recurrence");
}

A001045(n) = (2^n - (-1)^n) / 3
{
  check_equal(vector(#A001045_samples,n,n--; A001045(n)),
              A001045_samples,  \\ OFFSET=0
              "A001045(n)");
}

A014113(n) = (2^(n+1) - 2*(-1)^n)/3;
{
  check_equal(vector(#A014113_samples,n,n--; A014113(n)),
              A014113_samples,  \\ OFFSET=0
              "A014113(n)");
}

{
  check_equal(vector(10,n,n; numerator(f(n))),
              vector(10,n,n; J(n+1)),
              "f(n) numerator J");
  check_equal(vector(10,n,n; denominator(f(n))),
              vector(10,n,n; 2*J(n)),
              "f(n) denominator J");

  check_equal(vector(10,n,n; f(n)),
              vector(10,n,n; 1/2 * J(n+1)/J(n)),
              "f(n) by J");

  check_equal(vector(10,n,n; f(n)),
              vector(10,n,n; (2*2^n + (-1)^n)/3 / ((2*2^n - 2*(-1)^n)/3)),
              "f(n) ratio");
  check_equal(vector(10,n,n; f(n)),
              vector(10,n,n; (2*2^n + (-1)^n) / ((2*2^n - 2*(-1)^n))),
              "f(n) ratio");
  check_equal(vector(10,n,n; f(n)),
              vector(10,n,n; (2^(n+1) + (-1)^n) / (2^(n+1) - 2*(-1)^n)),
              "f(n) ratio");
  check_equal(vector(10,n,n; f(n)),
              vector(10,n,n; 1 + (3/2)*(-1)^n / (2^n - (-1)^n)),
              "f(n) ratio");

  check_equal(vector(10,n,n; f(n)),
              vector(10,n,n; A001045(n+1) / A014113(n)),
              "f(n) A-numbers");
}
{
  check_equal(f(1), 1/2);
  check_equal(f(2), 3/2);
  check_equal(f(3), 5/6);
  check_equal(f(10), 683/682);   \\ about 2

  check_equal(a(2), 2*f(2),
              "a(2)");
  check_equal(a(3), 2^3* f(3) * f(2)^2,
              "a(3)");
  check_equal(a(4), 2^7 * f(4) * f(3)^2 * f(2)^4,
              "a(4)");
  check_equal(a(5), 2^15 * f(5) * f(4)^2 * f(3)^4 * f(2)^8,
              "a(5) by f products");
  check_equal(a(6), 2^(2^6-1) * f(6) * f(5)^2 * f(4)^4 * f(3)^8 * f(2)^16 * f(1)^32,
              "a(6) by f products");

  check_equal(vector(12,n,n; a(n)),
              vector(12,n,n; 2^(2^n-1) * prod(i=0,n-1, f(n-i)^(2^i))),
              "a(n) prod f(i)");
}
t(n) = (-1)^n * log( f(n) );
{
  for(n=2,15,
     printf("%d  %13.10f\n",
            n, t(n)
           ));
}

{
  check_nearly_equal(vector(6,n,n++; f(n)),
                     vector(6,n,n++; exp((-1)^n * t(n))),
                     "f by t");

  check_nearly_equal(a(2), 2*exp((-1)^2 * t(2)),
                     "a(2) by t");
  check_nearly_equal(2*a(5),
                    2* (2*exp((-1)^5 * t(5)))
                     * (2*exp((-1)^4 * t(4)))^2
                     * (2*exp((-1)^3 * t(3)))^4
                     * (2*exp((-1)^2 * t(2)))^8,
                     "a(5) by t");
  check_nearly_equal(c(5),
                     (2*2*exp((-1)^5 * t(5))
                     * (2*exp((-1)^4 * t(4)))^2
                     * (2*exp((-1)^3 * t(3)))^4
                     * (2*exp((-1)^2 * t(2)))^8)^(1/2^5)
                      ,
                     "c(5) by t");
  check_nearly_equal(c(5),
                     (2*
                       2   * exp((-1)^5 * t(5))
                     * 2^2 * exp((-1)^4 * t(4))^2
                     * 2^4 * exp((-1)^3 * t(3))^4
                     * 2^8 * exp((-1)^2 * t(2))^8 )^(1/32)
                      ,
                     "c(5) by t");

  check_nearly_equal(c(5),
                     ( 2^(1/32)
                     * 2^(1/32) * exp((-1)^5 * t(5))^(1/32)
                     * 2^(1/16) * exp((-1)^4 * t(4))^(1/16)
                     * 2^(1/8)  * exp((-1)^3 * t(3))^(1/8)
                     * 2^(1/4)  * exp((-1)^2 * t(2))^(1/4) )
                      ,
                     "c(5) by t");

  check_nearly_equal(c(5),
                       2^(1/4)  * exp((-1)^2 * t(2))^(1/4)
                     * 2^(1/8)  * exp((-1)^3 * t(3))^(1/8)
                     * 2^(1/16) * exp((-1)^4 * t(4))^(1/16)
                     * 2^(1/32) * exp((-1)^5 * t(5))^(1/32)
                     * 2^(1/32)
                     ,
                     "c(5) by t");

  check_nearly_equal(c(4),
                       2^(1/4)  * exp((-1)^2 * t(2))^(1/4)
                     * 2^(1/8)  * exp((-1)^3 * t(3))^(1/8)
                     * 2^(1/16) * exp((-1)^4 * t(4))^(1/16)
                     * 2^(1/16)
                     ,
                     "c(4) by t");

  check_nearly_equal(c(4),
                     2^(1/4 + 1/8 + 1/16 + 1/16)
                     * exp((-1)^2 * t(2) / 4)
                     * exp((-1)^3 * t(3) / 8)
                     * exp((-1)^4 * t(4) / 16)
                     ,
                     "c(4) by t");

  check_nearly_equal(c(4),
                     2^(1/4 + 1/8 + 1/16 + 1/16)
                     * exp((-1)^2 * t(2) / 4
                         + (-1)^3 * t(3) / 8
                         + (-1)^4 * t(4) / 16)
                     ,
                     "c(4) by t");

  \\ t(n) = (-1)^n * log( 1/2 * f(n) )
  check_nearly_equal(c(4),
                     2^(1/4 + 1/8 + 1/16 + 1/16)
                     * exp(log(f(2)) / 4
                         + log(f(3)) / 8
                         + log(f(4)) / 16)
                     ,
                     "c(4) by f 1");

  check_nearly_equal(exp(1/2*log(1/2)),
                     exp(log(sqrt(1/2))));

  check_nearly_equal(c(4),
                     2^(1/2 + 1/4 + 1/8 + 1/16 + 1/16)
                     * exp(log(f(1)) / 2
                         + log(f(2)) / 4
                         + log(f(3)) / 8
                         + log(f(4)) / 16)
                     ,
                     "c(4) by f 2");


  check_nearly_equal(c(4),
                     2^(1/2 + 1/4 + 1/8 + 1/16 + 1/16)
                     * exp(log(f(1)) / 2
                         + log(f(2)) / 4
                         + log(f(3)) / 8
                         + log(f(4)) / 16)
                     ,
                     "c(4) by t 1");

  check_nearly_equal(c(4),
                     2*
                     exp(log(f(1)) / 2
                       + log(f(2)) / 4
                       + log(f(3)) / 8
                       + log(f(4)) / 16)
                     ,
                     "c(4) by f 3");
}
{
  check_nearly_equal(vector(6,n,n++; c(n)),
                     vector(6,n,n++;
                            2^(sum(i=1,n, 1/2^i) + 1/2^n)
                            * prod(i=1,n, f(i)^(1/2^i))),
                     "prods f");
}
{
  check_nearly_equal(vector(6,n,n++; c(n)),
                     vector(6,n,n++;
                            2^(sum(i=1,n, 1/2^i) + 1/2^n)
                            * prod(i=1,n, f(i)^(1/2^i))),
                     "prods f");
  check_nearly_equal(vector(6,n,n++; c(n)),
                     vector(6,n,n++;
                            2^(sum(i=1,n, 1/2^i) + 1/2^n)
                            * prod(i=1,n,
                                   (1 + (3/2)*(-1)^i / (2^i - (-1)^i)) ^(1/2^i))),
                     "prods 1+");

  check_nearly_equal(vector(6,n,n++; c(n)),
                     vector(6,n,n++;
                            2^(sum(i=1,n, 1/2^i) + 1/2^n)
                            * exp(sum(i=1,n,
                                      log(1 + (3/2)*(-1)^i / (2^i - (-1)^i)) /2^i))),
                     "prods exp");

  check_nearly_equal(vector(6,n,n++; c(n)),
                     vector(6,n,n++;
                            2
                            * exp(sum(i=1,n,
                                      log(1 + (3/2)*(-1)^i / (2^i - (-1)^i)) /2^i))),
                     "prods exp");
}

c_by_prod_f(n) = 2 * prod(i=1,n, f(i)^(1/2^i));
{
  check_nearly_equal(vector(6,n,n++; c(n)),
                     vector(6,n,n++; c_by_prod_f(n)),
                     "c_by_prod_f");

}
{
  check_nearly_equal(c(0), 2);
  check_nearly_equal(c(1), sqrt(2));
  check_nearly_equal(c(2), sqrt(2) * (3/2)^(1/4));
  check_nearly_equal(c(2), (12/2)^(1/4));
  check_equal(vector(10,n, c(2*n) > c(2*n+1)),
              vector(10,n, 1),
              "c(2n) bigger");
  check_equal(vector(10,n, c(2*n+1) < c(2*n+2)),
              vector(10,n, 1),
              "c(2n+1) smaller");

  check_equal(vector(10,n, f(2*n) > 1),
              vector(10,n, 1),
              "f(2n) > 1");
  check_equal(vector(10,n, f(2*n+1) < 1),
              vector(10,n, 1),
              "f(2n+1) < 1");
}

{
  check_nearly_equal(c(4),
                     2^(1/4 + 1/8 + 1/16 + 1/16)
                     * exp(log(1/2 * J(3)/J(2)) / 4
                         + log(1/2 * J(4)/J(3)) / 8
                         + log(1/2 * J(5)/J(4)) / 16)
                     ,
                     "c(4) towards J");
  check_nearly_equal(c(4),
                     2^(1/16)
                     * exp(log(J(3)/J(2)) / 4
                         + log(J(4)/J(3)) / 8
                         + log(J(5)/J(4)) / 16)
                     ,
                     "c(4) towards J");

  check_nearly_equal(log(c(4)),
                     1/16 * log(2)
                     + log(J(3)/J(2)) / 4
                     + log(J(4)/J(3)) / 8
                     + log(J(5)/J(4)) / 16
                     ,
                     "log(c(4)) by J");

  check_nearly_equal(log(c(4)),
                     1/16 * log(2)
                     + log(J(3)) / 4
                     - log(J(2)) / 4
                     + log(J(4)) / 8
                     - log(J(3)) / 8
                     + log(J(5)) / 16
                     - log(J(4)) / 16
                     ,
                     "log(c(4)) by J");


  check_nearly_equal(log(c(4)),
                     1/16 * log(2)
                     - log(J(2)) / 4
                     + log(J(3)) / 8
                     + log(J(4)) / 16
                     + log(J(5)) / 16
                     ,
                     "log(c(4)) by J");

  for(n=2,8, printf("%12.10f  %14.10f\n", log(J(n))/2^n, log(f(n))/2^n));
}

quit
vector(8,n,n++;  (-2)^-n *  log( cosh(d(n-1) * 2^(n-2) )))
plot(x=-3,3,log(cosh(x)) - abs(x))
exp(1/2)

{
  \\ d(n) = (-2)^-n * log(1/2 + 1/2* exp((-1)^(n-1) * d(n-1) * -2^(n-1)) )
  check_nearly_equal(vector(8,n,n++; d(n)),
                     vector(8,n,n++; (-2)^-n * log(1/2 + 1/2* exp((-1)^(n-1) * d(n-1) * -2^(n-1)) )),
                     "d(n)");
}

for(n=2,6, print(n"  "d(n)"\n   ", 1/4*d(n-1) + (-2)^-n * log(cosh(d(n-1) * (-2)^(-(n-2))))));

quit




\\------------------------------------------------------------------------------
\\ Complete Binary Tree Matchings Count

\\ a(n) = B * C^2^n
\\ a(n+1)/a(n) = C^2^n
\\ a(n+1)/a(n)^2 = 1/B
\\ c = log_{2^(n-1)-1} a(n)
\\ B = 1/2
\\ C = 1.53717671
\\ C*B = 0.768588
\\ C^2*B = 1.181456
\\
\\ a(n)/B = C^2^n
\\ a(n) = a(n-1)^2 + 2*a(n-1)*a(n-2)^2
\\ B * C^2^n = B^2 * C^2^n + 2*B * C^2^(n-1) * B^2 * C^2^(n-1)
\\ C^2^n = B * C^2^n + 2* C^2^(n-1) * B^2 * C^2^(n-1)
\\ C^2^n = B * C^2^n + 2 * B^2 * C^2^n
\\ 1 = B + 2*B^2
\\ polroots(2*B^2 + B - 1)    \\ 1/2 and -1
\\ C = (a(n)/B)^(2^n)
\\ log(a(n)/B) = 2^n*log(C)
\\ log(1+x)
\\ exp(x)

\\ g = log(1/2 + 1/2*exp(x + O(x^40)))
\\ 1/2*x + 1/8*x^2 - 1/192*x^4 + 1/2880*x^6 - 17/645120*x^8 + 31/14515200*x^10 - 691/3832012800*x^12
\\ v=abs(apply(numerator,Vec(g)))
\\ v=apply(denominator,Vec(g))
\\ vector(#v\2,i,v[2*i])
\\ numerators
\\ 17, 31, 691, 5461, 929569, 3202291, 221930581, 4722116521, 56963745931, 14717667114151, 2093660879252671, 86125672563201181, 129848163681107301953, 868320396104950823611, 209390615747646519456961, 14129659550745551130667441
\\ A276592    positives
\\ or A089171 alternating, numerators 1/(1 + cosh(sqrt(x)))
\\            aerated, numerators 2 * tanh(x/2)
\\ or A279370 negatives
\\
\\ not in OEIS: 8, 192, 2880, 645120, 14515200, 3832012800
\\ forstep(i=2,30,2, print1(denominator(polcoeff(lift(g),i)*2^i),", "))
\\ A046991 denominators of Taylor series log(1/cos(x)), also log(cos(x))

\\ g = 2*tanh(x/2)
\\ v=abs(apply(numerator,Vec(g)))
\\ v=apply(denominator,Vec(g))
\\ vector(#v\2,i,v[2*i-1])

\\ log(1 + exp(x))
\\ v=apply(denominator,Vec(log(1 + exp(x + O(x^40)))))
\\ vector(#v\2,i,v[2*i+1])
\\ not in OEIS: 8, 192, 2880, 645120, 14515200, 3832012800
\\ factor(3832012800)
\\ vector(#v\2,i,v[2*i-1]  * 8/4^i)
\\ not in OEIS: 24, 90, 5040, 28350, 1871100, 42567525

\\ 1/2/K * log(1/2 + 1/2*exp(K * -d))
\\ L[n]-L[n-1] = log(1/2 + 1/2*exp(2^(n-1)*(-L[n-1]+L[n-2]))) / 2^n
\\             =   1/2*2^(n-1)*(-L[n-1]+L[n-2])  / 2^n
\\               + 1/8*(2^(n-1)*(-L[n-1]+L[n-2]))^2  / 2^n - ...
\\             = - 1/4*(L[n-1]-L[n-2])
\\               + 1/32*(L[n-1]-L[n-2])^2 / 2^n

\\ 1/4*x + log(cosh(1/4*x))
\\ 1/2/K * log(1/2 + 1/2*exp(K * -d))
\\ x = K * -d
\\ 1/2/K * ( -1/2*x - 1/2*log(cosh(x)) )
\\ 1/4*d - 1/4/K * log(cosh(x))
\\ y = 1024
\\ log(cosh(1024))
\\ 1/4/K * log(cosh(x)) > 1/4/K*x
\\ 1/4/K*x == -1/4*d

\\ g=log(cosh(x + O(x^30)))
\\ v=apply(numerator,Vec(g))
\\ vector(#v\2-3,i,v[2*i+5])
\\ -17, 31, -691, 10922, -929569, 3202291, -221930581, 9444233042
\\ abs values A046990 numerators Taylor series for log(1/cos(x))
\\ v=apply(denominator,Vec(g))
\\ vector(#v\2,i,v[2*i-1])
\\ 2, 12, 45, 2520, 14175, 935550, 42567525, 10216206000, 97692469875
\\ A046991 Denominators of Taylor series for log(1/cos(x))



r(n) = sqrtn(2*a(n), 2^n);
r=memoize(r);
f(n) = r(n)/r(n-1);
f=memoize(f);
L(n) = log(r(n));
f=memoize(f);

\\ b*r(n)^(2^n) = b^2*r(n)^(2^n) + 2*b^3 * r(n-1)^(2^(n-1)) * r(n-2)^(2^(n-1))
{
  my(b=1/2);
  for(n=0,6,
     check_nearly_equal(a(n), b*r(n)^(2^n), Str("a and r n="n)));
  for(n=2,6,
     check_nearly_equal(b*r(n)^(2^n),
                        a(n-1)^2 + 2*a(n-1)*a(n-2)^2,
                        Str("formula n="n));
     \\ print(lex(r(n+1), r(n)));
  );
  for(n=2,6,
     check_nearly_equal(b*r(n)^(2^n),
                        b^2* r(n-1)^(2^n) + 2* b^3 * r(n-1)^(2^(n-1)) * r(n-2)^(2^(n-1)),
                        Str("formula n="n)));
  for(n=2,6,
     check_nearly_equal(r(n)^(2^n),
                        1/2 * r(n-1)^(2^n) + 1/2 * r(n-1)^(2^(n-1)) * r(n-2)^(2^(n-1)),
                        Str("formula n="n)));
  for(n=2,6,
     check_nearly_equal(r(n),
                        sqrtn(1/2 * r(n-1)^(2^n) + 1/2 * r(n-1)^(2^(n-1)) * r(n-2)^(2^(n-1)),
                              2^n),
                        Str("formula n="n)));
  for(n=2,6,
     check_nearly_equal(r(n),
                        r(n-1) * sqrtn(1/2 + 1/2 * (r(n-2)/r(n-1))^(2^(n-1)),
                                       2^n),
                        Str("formula n="n)));
  for(n=2,6,
     check_nearly_equal(r(n) / r(n-1),
                         sqrtn(1/2 + 1/2 * (r(n-2)/r(n-1))^(2^(n-1)),
                               2^n),
                        Str("formula n="n)));
  for(n=2,6,
     check_nearly_equal(f(n),
                         sqrtn(1/2 + 1/2 * 1/f(n-1)^(2^(n-1)),
                               2^n),
                        Str("formula n="n)));

  for(n=2,6,
     check_nearly_equal(2^n * log(r(n)),
                        log(1/2)
                        + 2^n*log(r(n-1))
                        + log(1 + r(n-1)^(-2^(n-1)) * r(n-2)^(2^(n-1))),
                        Str("formula n="n)));

  for(n=2,6,
     check_nearly_equal(2^n * log(r(n)),
                        2^n * log(r(n-1))
                        + log(1/2)
                        + log(1 + r(n-1)^(-2^(n-1)) * r(n-2)^(2^(n-1))),
                        Str("formula n="n)));
}

for(n=2,20, print(r(n)"  ", r(n)-r(n-1)))
\\ recurrence_guess(vector(15,n, a(n+1)/a(n)))
quit


 r(n-1)*sqrtn(1/2 + 1/2*r(n-2)/r(n-1), 2^(n-1))))
my(len=20); plothraw([1..len], vector(len,n, sqrtn(a(n+1)/a(n), 2^n)), 1);
my(n=20); print( sqrtn(a(n+1)/a(n), 2^n) )
my(n=25); print( sqrtn(a(n+1)/a(n), 2^n) )

quit
my(len=20); plothraw([1..len], vector(len,n, a(n+1)/a(n)^2), 1);
my(n=25); print( a(n+1)/a(n)^2.0 )
for(n=3,15, print(log(test_complete_binary_tree_matchings_count(n)) / log(2^n-1)));
my(len=20); plothraw(log(test_complete_binary_tree_matchings_count_vector(len)), log(vector(len,n, 2^n-1)), 1);

my(len=20); plothraw([1..len], vector(len,n,vpar_matchings_count(vpar_make_path(n))/phi^n), 1);


\\------------------------------------------------------------------------------

\\ sqrt(x)
\\
\\ sqrt(1+x)  = 1 + 1/2*x - 1/8*x^2 + 1/16*x^3 - 5/128*x^4 + 7/256*x^5 - 21/1024*x^6
\\ apply(numerator,Vec(sqrt(1+x)))
\\ A002596  a(n+2) = C(n+1)/2^k(n+1)  Catalans and k(n)= A048881(n) = CountOneBits
\\
\\ sqrt(1-x)    all negatives

\\ sqrt(sqrt(1+x))
\\ apply(numerator,Vec(sqrt(sqrt(1+x))))
\\ apply(numerator,Vec(sqrt(sqrt(1-x))))   \\ all negatives

\\ sqrt(1+x + O(x^8))*sqrt(sqrt(1+y + O(y^8)))


\\------------------------------------------------------------------------------
\\ Cf
\\
\\     Jens Christian Claussen, "Time-Evolution of the Rule 150 Cellular
\\     Automaton Activity From a Fibonacci Iteration", October 2004
\\

\\ Claussen gives this in the form of a floor rather than the (-1)^n used in
\\ J above.
for(n=1,20, Chi(n) == floor( (2^(n+2) + 1)/3 ) || error());

\\ Claussen identifies a run length l at bit position i by a product
\\
\\   (bit(i-1)==0) * bit(i) * bit(i+1) * ... * bit(i+l-1) * (bit(i+l)==0)
\\   matching:  0      1        1        ...     1             0
\\
bitlength(n) = if(n,logint(n,2)+1,0);

T_by_run_products(n) =
{
  my(k=bitlength(n));
  prod(l=1,k,
       Chi(l) ^ sum(i=0,k-l, !bittest(n,i-1)
                           * prod(j=i,i+l-1, bittest(n,j))
                           * !bittest(n,i+l)));
}

{
  print1("T(n) = ");
  for(n=0,8, print1(T_by_run_products(n),", "));
  print("...          n>=0  (A071053)");
}
{
  my(A071053_samples=[1,3,3,5,3,9,5,11,3,9,9,15,5,15,11,21,3,9,9,15,9]);
  vector(#A071053_samples,n,n--; T_by_run_products(n)) == A071053_samples
    || error();  \\ A071053_samples OFFSET=0
}

\\------------------------------------------------------------------------------








\\   \\ for(n=2,10,
\\   \\    nearly_equal(ELT(n) / log(n), log(C)));
\\ quit
\\ 0.62028
\\ for(n=0,100, T(4*n+1) == 3*T(n)            || error());
\\ for(n=0,100, T(4*n+3) == T(2*n+1) + 2*T(n) || error());
\\ 
\\ ET(n) = sum(i=0,n-1, T(i)) / n;
\\ EL(n) = sum(i=0,n-1, log(T(i))) / n;
\\ {
\\   for(n=2,20,
\\       print( 
\\             EL(n)
\\             / log(n) 
\\             );
\\      );
\\   print(A242049_sample / log(2));
\\   \\ 0.62028302992609
\\ }
\\ 
\\ Tprod_runs(n) =
\\ {
\\   if(n==0,[],
\\      fold(concat, vector(n,i,i--; n_to_run_lengths(i))));
\\ }
\\ Tprod_runs(0) == []               || error();
\\ Tprod_runs(1) == []               || error();
\\ Tprod_runs(2) == [1]              || error();
\\ Tprod_runs(3) == [1,1]            || error();
\\ Tprod_runs(4) == [1,1,2]          || error();
\\ Tprod_runs(5) == [1,1,2,1]        || error();
\\ Tprod_runs(6) == [1,1,2,1, 1,1]   || error();
\\ 
\\ histogram(v) = my(h=vector(vecmax(v))); for(i=1,#v,h[v[i]]++); h;
\\ vecsort(Tprod_runs(32))
\\ for(n=2,25,print("n="n"  "histogram(Tprod_runs(2^n))))
\\ 
\\ 
\\ quit
\\ {
\\   for(n=500,500,
\\       \\print(log(prod(i=0,n-1, T(i))) / n);
\\       print( 
\\             log(ET(n))
\\             / log(n)
\\             );
\\      );
\\   print(log2(1+sqrt(5)) - 1);
\\   print(log2(phi));     \\ A242208
\\ \\ 0.6942419136306
\\ }


\\ Mercator for log(4/3)
\\ log(1+x)
\\ a_vector(len) = my(p=10^len,s=0,prev=-1); \
\\   for(i=1,oo, print(i" "s); p/=-3; s-=p/i; my(this=floor(s)); \
\\     if(this==prev,return(digits(this))); prev=this);
\\ a_vector(5)
\\ my(v=OEIS_samples("A083679")); a_vector(#v) == v


\\ Benoit Cloitre in A083679
\\ log(4/3) = sum(k>=1, sum(i=1, k, 1/i /2^i)    /2^(k+1))

\\ sum(k=1,100, sum(i=1,k, 1/i /2^i)    /2^(k+1))*1.0



\\ Return a vector [a(1), ... a(len)].
\\ Starting y=0 means the first step gives x=y=1 resdy for a(1).
a_vector(len) = my(x=1,y=0); vector(len,n, [x,y] = [x^2 + 2*x*y^2, x]; x)
\\
for(len=0,5, a_vector(len) == vector(len,n, a(n)) || error());
{
  my(v=a_vector(10));
  for(i=1,#v, for(j=1,i-1, v[i] % v[j] == 0  || error()))
}
