\\ Copyright 2017, 2018, 2019, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");

option_verbose = 1;


\\------------------------------------------------------------------------------

\\ 1 = only singletons
\\ 2 = 1 with 1 under
\\     2 with 1 under
\\     4 with 1 under
\\     2 with 3 under
\\ 3 = 1 with 2 under
\\     2 with 3 under
\\ 4 = 1 or 4 with 3 under      so 1 with >=1 3s
\\ 5 = 1 with 4 under



\\------------------------------------------------------------------------------
\\ equimatchable - isomorphic halves
\\ none, since none of the equimatchableness states allows another of itself
\\ as a child

{
  for(n=0,8,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(joined=vpar_join(vpar,1,vpar));
           if(vpar_is_equimatchable(joined),
              print(vpar);
              count++));
     print(count", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ equimatchable - count by search
\\
\\ equimatchable and any edge removal stays equimatchable

\\ labelled rooted trees edge stable equimatchable
\\          n = 1  2  3   4   5   6     7      8
\\ not in OEIS: 1, 2, 9, 16, 25, 36, 4459, 

\\ unlabelled rooted trees edge stable equimatchable
\\              1, 1, 1, 2, 2, 2, 2, 5, 7, 10, 15, 25, 40, 64, 99, 164, 
\\ not in OEIS:                      5, 7, 10, 15, 25, 40, 64, 99, 164, 

\\ free trees edge stable equimatchable
\\          n = 1  2  3  4  5  6  7  8  9 10
\\              1, 1, 1, 1, 1, 1, 2, 2, 3, 4, 6, 8, 13, 17, 27, 40, 62, 92, 147, 
\\ not in OEIS:                         3, 4, 6, 8, 13, 17, 27, 40, 62, 92, 147, 
\\ always star-n
\\ n=7  [0, 1, 2, 2, 1, 5, 5]       complete binary
\\ n=8  [0, 1, 2, 2, 2, 1, 6, 6]    complete binary, extra leaf
\\          *
\\        /   \
\\      *       *
\\     / \     / \ \
\\    *   *   *   * *
\\ n=9  [0, 1, 2, 2, 2, 2, 1, 7, 7] complete binary, 2 extra leaf
\\ n=9  [0, 1, 2, 2, 2, 1, 6, 6, 6]    on same or different
\\          *                      *             
\\        /   \                  /    \           
\\      *       *              *        *          n=9
\\   / / \     / \ \          / \    / / \ \      
\\  * *   *   *   * *        *   *  * *   * *     
\\ 
\\ n=10  [0, 1, 2, 2, 2, 2, 2, 1, 8, 8]
\\          *
\\        /   \          max deg 6
\\      *       *        n=10
\\     / \     /...\
\\    *   *   * 5of *
\\ n=10  [0, 1, 2, 2, 2, 2, 1, 7, 7, 7]
\\          *
\\        /   \          max deg 5
\\      *       *        n=10
\\     / \     /  \
\\   * 3of *  * 4of *
\\ n=10  [0, 1, 2, 2, 1, 5, 5, 1, 8, 8]
\\           *
\\        /  |  \        complete tree 3,2
\\      *    *    *       max deg 3
\\     / \  / \  / \
\\     * *  * *  * *
\\ n=10  [0, 1, 1, 1, 1, 1, 1, 1, 1, 1]
\\   star-5 max deg 11

try(vpar) =
{
  \\ vpar_is_degrees_le(vpar,3) || return;
  \\ vpar_is_equimatchable(vpar) || return;

  vpar_is_stable_equimatchable(vpar) || return;
  count++;
  if(#vpar<=10,
     print("\nn="#vpar"  "vpar);
     print(vpar_to_sparse6(vpar)));
}
{
  for(n=1,22,
     count=0;
     read(Str("/tmp/trees-"n".gp"));
     print1(count", ");
  );
  quit;
}

{
  my(forest=0);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_stable_equimatchable(vpar));
     print1(count", ");
  );
  quit;
}
{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(got=vpar_is_stable_equimatchable(vpar));
           count+=got);
     print1(count", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Ordered Counts - Newton Iteration per Bill Allombert

P=substvec((1+x)*g^3-2*g^2+(1-x+2*x^2)*g-x^2,[x,g],[X,y])
P'
g=subst(x^2 + O(x^3),x,X)
g=subst(1 - x + O(x^2),x,X)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)

quit


P=substvec((1+x)*g^3 - 2*g^2 + (1-x+2*x^2)*g - x^2,[x,g],[X,y])
P'
g=subst(x^2+O(x^3),x,X)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
quit

P=substvec((1+x)*g^3 - 2*g^2 + (1-x+2*x^2)*g - x^2,[x,g],[X,y])
\\ g=subst(x^2 + x^3 + x^4 + 3*x^5 + 6*x^6 + 12*x^7 + 29*x^8 + 67*x^9 + O(x^10),x,X)
g=subst(x^2 + O(x^3),x,X)
\\ truncate(g)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
g=truncate(g)-subst(P,y,truncate(g))/subst(P',y,g)
quit
y;
X;
g;


\\------------------------------------------------------------------------------
\\ Ordered Counts - GF Solution

to_squarefree(x) =
{
  my(m=factor(x));
  for(i=1,matsize(m)[1], m[i,2]=1);
  factorback(m);
}

g2 = x^2 + x^3 + x^4 + 3*x^5 + 6*x^6 + 12*x^7 + 29*x^8 + 67*x^9 + O(x^10)
sqrt(g2^2)
(g2^3)^(1/3)

p = (1+x)*g^3 - 2*g^2 + (1-x+2*x^2)*g - x^2
s = 1;

print("zero term ", subst(p,'g, s + 't + O(x^1)));
factor(t^3 - 2*t^2 + t)
{
  for(n=1,10,
     my(q = subst(p,'g, s + t*x^n));
     print("n="n"  "q);
     my(lo=valuation(q,'x),
        term = polcoeff(q,lo,'x));
     print(" term="term);
     term = to_squarefree(term);
     print(" squarefree "term"  poldisc "poldisc(term));

     poldegree(term)==1 || error("not linear");
     my(new_s = - polcoeff(term,0,'t) / polcoeff(term,1,'t));
     print("  new "new_s);
     s += new_s*x^n;
     );
}
print(s + O(x^(poldegree(s)+1)));

quit


\\------------------------------------------------------------------------------
\\ Ordered Counts - GF Solution Try serreverse()

g2 = x^2 + x^3 + x^4 + 3*x^5 + 6*x^6 + 12*x^7 + 29*x^8 + 67*x^9 + O(x^10);

print();
p = (1+x)*'g^3 - 2*'g^2 + (1-x+2*x^2)*'g - x^2;
print("p = "p);

p = (1+z)*'x^3 - 2*'x^2 + (1-z+2*z^2)*'x - z^2;
print("p = "p);
print("subst ",subst(p,'x,'x-'t));

\\ low term (-2*t - 1)*z^2 + (-t^3 + t)*z + (-t^3 - 2*t^2 - t)
factor(-t^3 - 2*t^2 - t)
\\ z^2   factor t=-1/2
\\ z     factor t=0 or 1
\\ const factor t=0 or 1
\\ doesn't get to zero constant term in p

print("subst ",subst(p,'x,'x*z^2 + z + 0));
q=subst(p,'x,'x*z^2 + 0)
print("subst q = ",q);
print("subst ",subst(q,'x,'x-'t));
quit

L = O(z^10);
p = (1+z+L)*'x^3 - (2+L)*'x^2 + (1-z+2*z^2+L)*'x - z^2+L;
print("p = "p);
g2z = subst(g2,'x,'z);
print("g2z = ",g2z);

q = subst(p,'x,'x+1)
print("q = "q);
quit


r = serreverse(p   + O(x^10))
subst(Pol(r),'x,1)
quit


\\------------------------------------------------------------------------------
\\ Ordered Counts

read("../examples/equimatchable-count.gp");

L = O(x^(limit+1));
g1 = Polrev(ordered_count[1,]) * x  + L;
g2 = Polrev(ordered_count[2,]) * x  + L;
g3 = Polrev(ordered_count[3,]) * x  + L;
g4 = Polrev(ordered_count[4,]) * x  + L;
g5 = Polrev(ordered_count[5,]) * x  + L;
gT = Polrev(ordered_count_total) * x  + L;

g1 == x                                || error();
g5 == g1*g4                            || error();
g4 == g1*g3 + g4*g3                    || error();
g3 == g1*g2 + g2*g3                    || error();
g2 == g1*g1 + g2*g1 + g2*g3 + g4*g1    || error();
gT == g1 + g2 + g3 + g5                || error();

g4 == x*g3 + g4*g3 
g3 == x*g2 + g2*g3                 
g2 == x^2 + g2*x + g2*g3 + x*g4 
gT == x + g2 + g3 + x*g4

g2 + g3 + x*g4 == 2*g2 - x^2

{
  + x*'g2 + 'g2*'g3                       \\ g3        
  - ( x^2 + 'g2*x + 'g2*'g3 + x*'g4 )     \\ -g2
  == -x^2 - x*'g4    || error()
}

g3 - g2 == -x^2 - x*g4  || error()
g2 == x^2 + g3 + x*g4   || error()
quit

g2 == gT - g3 - x*g4 - x

g4 == x*g3 + g4*g3 
g3 == x*(gT - g3 - x*g4 - x) + (gT - g3 - x*g4 - x)*g3                 
gT - g3 - x*g4 - x == x^2 + (gT - g3 - x*g4 - x)*x + (gT - g3 - x*g4 - x)*g3 + x*g4 

g4 == x*g3/(1 - g3)

g3 == x*(gT - g3 - x*x*g3/(1 - g3) - x) + (gT - g3 - x*x*g3/(1 - g3) - x)*g3                 
gT - g3 - x*x*g3/(1 - g3) - x == x^2 + (gT - g3 - x*x*g3/(1 - g3) - x)*x + (gT - g3 - x*x*g3/(1 - g3) - x)*g3 + x*x*g3/(1 - g3) 

gT - g3 - 2*x^2*g3/(1 - g3) == (gT - g3 - x*x*g3/(1 - g3) - x)*x + (gT - g3 - x*x*g3/(1 - g3) - x)*g3 + x + x^2
g3 == -x^2 + x*gT - x*g3 - x^3*g3/(1 - g3) - g3*x + g3*gT - g3*g3 - x^2*g3*g3/(1 - g3)
g3 == -x^2 + x*gT + g3*gT - 2*x*g3 - x^3*g3/(1 - g3) - g3^2 - x^2*g3^2/(1 - g3)

g4 == x*g3/(1 - g3)
g3 == x*g2/(1 - g2)
\\ cubic
(1+x)*g2^3 - 2*g2^2 + (1-x+2*x^2)*g2 - x^2 == 0
g2^3 == 2/(1+x)*g2^2 - (1-x+2*x^2)/(1+x)*g2 + x^2/(1+x)

g2v = [0,1,0];
g2sqv = [1,0,0];
g2cubv = [2/(1+x), -(1-x+2*x^2)/(1+x), x^2/(1+x)];
g2^3 == g2cubv[1]*g2^2 + g2cubv[2]*g2 + g2cubv[3] || error()

p = subst((1+x)*'g2^3 - 2*'g2^2 + (1-x+2*x^2)*'g2 - x^2, 'g2, 1-'h)
for(i=0,3, print("i="i"  "polcoeff(p,i,'h)))

(x + 1)*(1-g2)^3 - (3*x + 1)*(1-g2)^2 + 2*x*(x + 1)*(1-g2) - x^2 == 0
(x + 1)/x^2*(1-g2)^3 - (3*x + 1)/x^2*(1-g2)^2 + 2/x*(x + 1)*(1-g2) == 1
(x + 1)/x^2*(1-g2)^2 - (3*x + 1)/x^2*(1-g2) + 2/x*(x + 1) == 1/(1-g2)

p = (x + 1)/x^2*(1-'g2)^2 - (3*x + 1)/x^2*(1-'g2) + 2/x*(x + 1)
for(i=0,3, print("i="i"  "polcoeff(p,i,'g2)))

1/(1-g2) == (x + 1)/x^2 * g2^2 + (x - 1)/x^2 * g2 + 2

g3 == x*g2*((x + 1)/x^2 * g2^2 + (x - 1)/x^2 * g2 + 2)
g3 == g2*((x + 1)/x * g2^2 + (x - 1)/x * g2 + 2*x)
g3 == (x + 1)/x * g2^3 + (x - 1)/x * g2^2 + 2*x*g2

p = (x + 1)/x * 'g2^3 + (x - 1)/x * 'g2^2 + 2*x*'g2
p = substpol(p,'g2^3, 2/(1+x)*'g2^2 - (1-x+2*x^2)/(1+x)*'g2 + x^2/(1+x) )
for(i=0,3, print("i="i"  "polcoeff(p,i,'g2)))

g3 == (x + 1)/x * g2^2 + (x - 1)/x * g2 + x
g3v = [(x + 1)/x, (x - 1)/x, x];
print("g3v = "g3v);
g3 == (x + 1)/x*g2^2 + (x - 1)/x*g2 + x   || error()
1-g3 == -(x + 1)/x * g2^2 - (x - 1)/x * g2 + 1-x

reduce(v,red) =
{
  \\ print(v);
  for(i=1,#v-#red,
     my(sub=v[i]*red);
     for(j=1,#sub,
        v[i+j] += sub[j]));
  \\ print(v);
  simplify(v[#v-#red+1..#v]);
}
g2red = g2cubv;
check_equal(reduce([0,0,0],g2red), [0,0,0], "g2red zero");
check_equal(reduce([1,0,0,0],g2red), g2cubv, "g2cubv from reduction");

mul_unred(u,v) =
{
  my(p=vector(#u+#v-1));
  for(i=1,#u,
     for(j=1,#v,
        p[i+j-1] += u[i]*v[j]));
  p;
}
mul(u,v,red) = reduce(mul_unred(u,v),red);
check_equal(mul([0,0,1],[0,0,1], g2red), [0,0,1]);
check_equal(mul(g2v,g2v, g2red), g2sqv);
check_equal(mul([0,1,0],[1,0,0], g2red), g2cubv, "g2cubv by product");

\\ (A*g^2 + B*g + C)*(v2*g^2 + v1*g + v0) == 1
\\ A*g^2*v + B*g*v + C*v == 1
\\ / v0 v1 v2 \ / A \   / 0 \
\\ |    v0 v1 | | B | = | 0 |
\\ \       v0 / \ C /   \ 1 /
recip(v,red) = 
{
  v = reduce(v,red);
  print("v="v);
  my(m = vector(#red,i,
                my(s=concat(v,vector(#red-i)),
                  sr=reduce(s,red));
                print("shift "s"  = "sr);
                sr~));
  print(m);
  m=Mat(m);
  print("m = "m);
  Vec(matsolve(m,[0,0,1]~));
}
check_equal(recip([0,0,1], g2red), [0,0,1], "recip 1");

g2inv = recip(g2v, g2red);
print("g2inv = "g2inv);

check_equal(g2cubv, [2/(x + 1), (-2*x^2 + x - 1)/(x + 1), x^2/(x + 1)], "g2cubv");

{ m = [-2/(x + 1),              1, 0;
       (2*x^2 - x + 1)/(x + 1), 0, 1;
       -x^2/(x + 1),            0, 0]; }
m= [2/(x + 1), 1, 0; (-2*x^2 + x - 1)/(x + 1), 0, 1; x^2/(x + 1), 0, 0]
m*[0,1,0]~
m*['A,0,0]~
m*[0,'B,0]~
m*[0,0,'C]~

\\ g2inv = [(x + 1)/x^2,   -2/x^2,   (2*x^2 - x + 1)/x^2]
A = (x + 1)/x^2
B = - 2/x^2
C =  - (-2*x^2 + x - 1)/x^2
m*[A,0,0]~
m*[0,B,0]~
m*[0,0,C]~
m*[A,B,C]~

mul_unred(['A,'B,'C],g2v)
p = mul(['A,'B,'C],g2v, g2red)
p = subst(p,'A, g2inv[1])
p = subst(p,'B, g2inv[2])
p = subst(p,'C, g2inv[3])

mul_unred(g2inv, g2v)
check_equal(mul(g2inv,g2v, g2red), [0,0,1]);


onesubg3v = [0,0,1] - g3v;

one_over_onesubg3v = recip(onesubg3v, g2red);
g4v = mul(x*g3v, one_over_onesubg3v, g2red);
print("g4v = "g4v);
g4 == (-x - 1)/x^2*g2^2 + 1/x^2*g2 -x - 1 || error();

g5v = x*g4v;
print("g5v = "g5v);
g5 == x*g4  || error();
g5 == (-x - 1)/x*g2^2 + 1/x*g2 -x^2 - x   || error();

g1v = x*[0,0,1];
gTv = g1v + g2v + g3v + g5v;
print("gTv = "gTv);
gT == g1 + g2 + g3 + g5  || error();

gT == 2*g2 - x^2 + x     || error();
g2 == (gT + x^2 - x)/2     || error();

p = (1+x)*'g2^3 - 2*'g2^2 + (1-x+2*x^2)*'g2 - x^2;
p = substpol(p, 'g2, ('gT + x^2 - x)/2) * 8
for(i=0,4, print("+ ("polcoeff(p,i,'gT)" )*gT^"i));

{
+ (1+x)                             * gT^3
+ (-4 - 3*x + 3*x^3)                * gT^2
+ (1+x)*(4 + 3*x^2 - 6*x^3 + 3*x^4) * gT
- x * (4 + 4*x + 5*x^2 - 6*x^3 + 2*x^5 - x^6)
== 0 || error()
}

factor((4 + 3*x^2 - 6*x^3 + 3*x^4)*I)

quit


\\------------------------------------------------------------------------------
\\ Labelled Counts

read("../examples/equimatchable-count.gp");

gT == g1 + x*(1-1/exp(x)) * exp(x*exp(g2)) -x + x*exp(g2) + x*-x + x^2*exp(-x + x*exp(g2))    || error();
gT == x + x*exp(g2) + x*(1-1/exp(x)) * exp(x*exp(g2)) + x^2*exp(-x + x*exp(g2)) - x - x^2    || error();
gT == x*exp(g2) + x*(1-1/exp(x)) * exp(x*exp(g2)) + x^2*exp(-x)*exp(x*exp(g2)) - x^2    || error();
gT == x*exp(g2) + (x*(1-1/exp(x)) + x^2*exp(-x))*exp(x*exp(g2)) - x^2    || error();

gT == - x^2 + x*exp(g2) + ( 1 + (x-1)/exp(x) )*x*exp(x*exp(g2))    || error();

gT - g3 - x*g4 - x == x*(1-1/exp(x)) * exp(x*exp(gT - g3 - x*g4 - x))      || error();

g3 == -x + x*exp(x*(exp(g3+x) - exp(g3)))                       || error();

g4 == -x + x*exp(g3)                       || error();
g4/x+1 == exp(g3)                       || error();
g3 == log(g4/x+1)                          || error();

log(g4/x+1) == -x + x*exp((exp(x)-1)*x*exp(log(g4/x+1)))     || error();
log(g4/x+1) == -x + x*exp((exp(x)-1)*x*exp(log((g4+x)/x)))     || error();
\\ log(g4/x+1) == -x + x*exp((exp(x)-1)*x*exp(log(g4+x) - log(x)))
\\ log(g4/x+1) == -x + x*exp((exp(x)-1)*exp(log(x+g4)))
log(g4/x+1) == -x + x*exp((exp(x)-1)*(x+g4))   || error();
g4/x+1 == exp(-x + x*exp((exp(x)-1)*(x+g4)))   || error();
g4 == -x + x*exp(-x + x*exp((exp(x)-1)*(x+g4)))   || error();
g4 == -x + x*exp(-x + x*exp(-x - g4 + exp(x)*x + exp(x)*g4))   || error();
g4 == -x + x*exp(-x + x*exp(- g4 + exp(x)*x + exp(x)*g4)/exp(x))   || error();

g5/x == -x + x*exp(-x + x*exp((exp(x)-1)*(x+g5/x)))   || error();
g5 == -x^2 + x^2*exp(-x + x*exp((exp(x)-1)*(x+g5/x)))   || error();
g5 == -x^2 + x^2*exp(-x + x*exp((exp(x)-1)*((x^2+g5)/x)))   || error();
g5 == -x^2 + x^2*exp(-x + x*exp((exp(x)-1)/x * (x^2+g5)))   || error();

quit


\\------------------------------------------------------------------------------
\\ equimatchable - count by search states

\\ n=1 count 1  [1, 0, 0, 0, 0]
\\ [0, 1]
\\ [2, 0]
\\ n=2 count 2  [0, 2, 0, 0, 0]
\\ [0, 1, 1]
\\ [2, 0, 2]
\\ [3, 3, 0]
\\ n=3 count 9  [0, 3, 6, 0, 0]
\\ [0, 1, 1, 1]
\\ [2, 0, 2, 2]
\\ [3, 3, 0, 3]
\\ [4, 4, 4, 0]
\\ n=4 count 16  [0, 4, 12, 24, 0]
\\ n=5 count 325  [0, 125, 80, 60, 120]

{
  my(forest=0);
  for(n=1,5,
     my(count=0,
        count_by_state=vector(5));
     forvec(vec=vpar_forvec_from(n,forest), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_equimatchable(vpar);
           my(state=test_equimatchable_get_state(vpar));
           if(state, count_by_state[state]++);
           if(state==2 && n <= 4,
              print(vpar)));
     print("n="n" count "count"  "count_by_state);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ equimatchable unlabelled

vector_cumulative(v) = for(i=2,#v,v[i]+=v[i-1]); v;
vector_shift_up(v,n=1) = concat(vector(n),v[1..#v-n]);
check_equal(vector_shift_up([1,2,3]), [0,1,2]);

\\ b[n] = 1/n * (c[n] + sum(k=1, n-1, c[k] * b[n-k]))
\\ c[n] = sumdiv(d|n, d*a[d])
\\
Euler_transform_term(a,b,n,offset=0) =
{
  my(c = (n)->sumdiv(n,d, d*a[d]));
  (c(n) + sum(k=1,n-1, c(k) * b[n-k+offset])) / n;
}

\\ 1 sole
\\ 2 = 1,1,1     >=1 of
\\     3,3,3     >=0 of
\\ 3 = 2,2,2     >=1 of
\\ 4 = 3,3,3     >=1 of
\\ 5 = 4         =1 only

\\ 3 = forest of 2s
\\ 4 = forest of 3s
\\ 2of1s = forest of 1s = 1 way
\\ 2 = cumulative forest of 3s = cumulative 4

v1 = [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
v2 = [0, 1, 1, 1, 2, 3, 5, 9, 16, 28, 52, 96, 178, 336]
v3 = [0, 0, 1, 1, 2, 3, 6, 9, 18, 31, 58, 106, 202, 375]
v4 = [0, 0, 0, 1, 1, 2, 4, 7, 12, 24, 44, 82, 158, 302]
v5 = [0, 0, 0, 0, 1, 1, 2, 4, 7, 12, 24, 44, 82, 158]
\\ total
vT = [1, 1, 2, 2, 5, 7, 13, 22, 41, 71, 134, 246, 462, 869]
print();

check_equal(v3, vector_shift_up(vpar_INTERNAL_Euler_transform(v2)));
check_equal(v4, vector_shift_up(vpar_INTERNAL_Euler_transform(v3)));
check_equal(v2, vector_shift_up(vector_cumulative(v4) + vector_cumulative(v1)))
{
  my(v2_sub=v2); v2_sub[2]=0;
  check_equal(v2_sub,
              vector_shift_up(vpar_INTERNAL_Euler_transform(v3+v1), 2),
              "v2 Euler transform v1+v3");
}

check_equal(v3, vector(#v3,n, if(n==1,0, Euler_transform_term(v2,v3,n-1,1))))


a = vector(8,i, random(10));
b = vpar_INTERNAL_Euler_transform(a);
check_equal(vector(#a,i,Euler_transform_term(a,b,i)), b);

limit = 13;
unlabelled_count = vector(5,state, vector(limit));
unlabelled_count[1][1] = 1;
unlabelled_count[2][2] = 1;
{
  for(n=3,limit,
     unlabelled_count[5][n] = unlabelled_count[4][n-1];
     unlabelled_count[3][n] = Euler_transform_term(unlabelled_count[2],
                                                   unlabelled_count[3],
                                                   n-1, 1);
     unlabelled_count[4][n] = Euler_transform_term(unlabelled_count[3],
                                                   unlabelled_count[4],
                                                   n-1, 1);
     unlabelled_count[2][n] = unlabelled_count[2][n-1] + unlabelled_count[4][n-1]);
}
{
  for(state=1,5,
     print(unlabelled_count[state]));
}
print("total");
unlabelled_count_total = unlabelled_count[1] + unlabelled_count[2] \
                       + unlabelled_count[3] + unlabelled_count[5]

{
  for(n=1,min(8,#unlabelled_count_total),
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           count+=vpar_is_equimatchable(vpar));
     check_equal(count,unlabelled_count_total[n]));
}


L = O(x^(limit+1));
g(vec,x) = subst(Polrev(vec)*x + L,'x,x);
g1(x) = g(unlabelled_count[1],x);
g2(x) = g(unlabelled_count[2],x);
g3(x) = g(unlabelled_count[3],x);
g4(x) = g(unlabelled_count[4],x);
g5(x) = g(unlabelled_count[5],x);
print("g1(x) = "g1(x));
print("g2(x) = "g2(x));
print("g3(x) = "g3(x));
print("g4(x) = "g4(x));
print("g5(x) = "g5(x));


check_equal(g3(x), -x + x*exp(sum(k=1,limit, 1/k*g2(x^k))), "g3");
check_equal(g4(x), -x + x*exp(sum(k=1,limit, 1/k*g3(x^k))), "g4");
check_equal(g2(x), x/(1-x)*(x + g4(x)), "g2");
check_equal(g5(x), x*g4(x));

quit

\\------------------------------------------------------------------------------
\\ equimatchable with limited degree

\\ combinations of children to make a new root with an equimatchable state
\\ (if no state then cannot be part of anything bigger)

\\ edge stable degree <= 3
\\ 1 n=1  [0]
\\ 1 n=2  [0, 1]
\\ 1 n=3  [0, 1, 1]
\\ 1 n=4  [0, 1, 1, 1]
\\ 1 n=7  [0, 1, 1, 1, 4, 5, 5]
\\ 1 n=10  [0, 1, 1, 1, 4, 5, 5, 4, 8, 8]
\\
\\ vertex stable degree <= 3 
\\ 1 n=1  [0]
\\ 1 n=2  [0, 1]
\\ 1 n=3  [0, 1, 1]
\\ 1 n=4  [0, 1, 1, 1]
\\ 1 n=7  [0, 1, 1, 1, 4, 5, 5]
\\ 1 n=10  [0, 1, 1, 1, 4, 5, 5, 4, 8, 8]
\\
\\ n=10
\\       *
\\    /  |  \
\\  *    *    *
\\ / \  / \  / \
\\ * *  * *  * *

\\ almost equimatchable infinite with degree <= 3

pred = vpar_is_equimatchable;
statefunc = test_equimatchable_get_state;

pred = vpar_is_stable_equimatchable;
statefunc = test_stable_equimatchable_get_state;

pred = vpar_is_almost_equimatchable;
statefunc = test_almost_equimatchable_get_state;

g(d) =
{
  my(l=List([ [0] ]),
     more=1);
  while(more,
        more=0;
        print("at l "#l);
        forvec(vec=vector(d,i,[0,#l]),  \\ 0 to d children from l[]
              my(vpar=[0]);
              for(i=1,#vec,
                 if(!vec[i], next());
                 my(child=l[vec[i]]);
                 \\ print("  child "child);
                 if(vpar_degree_of_vertex(child,1)==d,next());
                 vpar=vpar_join(vpar,1,child));

              vpar_validate(vpar);
              statefunc(vpar) || next();
              for(i=1,#l, if(vpar_is_isomorphic(vpar,l[i]),next(2)));
              if(vecmax(vpar_degrees_vector(vpar))>d, error());

              listput(l,vpar);
              more=1;
              print(" new "vpar" height "vpar_height(vpar)" degs "Set(vpar_degrees_vector(vpar)));
        );
        );
  l=Vec(l);
  print("count "#l);
  l=vecsort(l,(x,y)->lex([length(x),x],[length(y),y]));
  for(i=1,#l,
     my(vpar=l[i]);
     print(pred(vpar)" n="#vpar"  "vpar));

  print();
  my(f=List([]));
  for(i=1,#l,
     my(vpar=l[i]);
     for(j=1,#f, if(vpar_is_free_isomorphic(vpar,f[j]),next(2)));
     listput(f,vpar));  
  f=Vec(f);
  f=select(pred,f);
  f=vecsort(f,(x,y)->lex([length(x),x],[length(y),y]));
  print("free count "#f);

  for(i=1,#f,
     my(vpar=f[i]);
     print(pred(vpar)" n="#vpar"  "vpar));
  print("max n = "vecmax(apply(length,f)));
  for(i=1,#f,
     my(vpar=f[i]);
     print(vpar_to_sparse6(vpar)));
}

g(3);
quit

\\ infinite edge stable equimatchable of degree <= 4 (and made in test.gp)
\\
\\     *  3         *  6        *  7         *  4        
\\     |            |         / |         // |           
\\     *  2         *  5     *  *  5     **  *  5        
\\     |            |           |            |           
\\     *  1         *  4        *  4         *  4        
\\                / |         / |          / |           
\\               *  *  1     *  *  1      *  *  1        
\\ 


\\------------------------------------------------------------------------------
\\ almost equimatchable

\\ unlabelled rooted trees almost equimatchable
\\              0, 0, 0, 0, 2, 4, 13, 35, 81, 200, 456, 1035, 2309, 5103, 11139, 24221, 
\\ not in OEIS:             2, 4, 13, 35, 81, 200, 456, 1035, 2309, 5103, 11139, 24221, 

\\ free trees almost equimatchable
\\              0, 0, 0, 1, 1, 4, 7, 16, 32, 68, 137, 282, 572, 1164, 2354, 4773, 9646, 19524, 
\\ not in OEIS:                4, 7, 16, 32, 68, 137, 282, 572, 1164, 2354, 4773, 9646, 19524, 
\\          n = 1  2  3  4  5  6  7
\\
\\ n=6 free
\\ [0, 1, 2, 3, 1, 5] height 3    path-6
\\ [0, 1, 2, 2, 2, 1] height 2
\\ [0, 1, 2, 2, 1, 1] height 2
\\ [0, 1, 2, 1, 4, 1] height 2

try(vpar) =
{
  \\ vpar_is_degrees_le(vpar,3) || return;
  \\ vpar_is_equimatchable(vpar) || return;
  \\ vpar_is_stable_equimatchable(vpar) || return;

  vpar_is_almost_equimatchable(vpar) || return;

  count++;
  my(h=vpar_height(vpar));
  max_height=max(max_height,h);
  if(#vpar<=6, print("  "vpar" height "h));
  0;
}
{
  for(n=1,22,
     count=0;
     max_height=0;
     read(Str("/tmp/trees-"n".gp"));
     print(count"   max_height "max_height);
     \\ print1(count", ");
  );
  quit;
}

{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(got=is_almost_equimatchable(vpar));
           count+=got;
           if(#vpar<=10, print("  "vpar" height "vpar_height(vpar)));
          );
     print(count", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ edge critical equimatchable
\\
\\ In trees there is always an edge cut which remains equimatchable.

is_edge_critical(vpar) =
{
  if(! vpar_is_equimatchable(vpar), return(0));
  for(v=1,#vpar,
     if(vpar[v],
        my(vpar_cut=vpar);
        vpar_cut[v]=0;
        if(vpar_is_equimatchable(vpar_cut),
           return(0))));
  1;
}
num_edges_critical(vpar) =
{
  if(! vpar_is_equimatchable(vpar), return(0));
  my(count=0);
  for(v=1,#vpar,
     if(vpar[v],
        my(vpar_cut=vpar);
        vpar_cut[v]=0;
        if(vpar_is_equimatchable(vpar_cut),
           count++)));
  1;
}
{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        total=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           total += num_edges_critical(vpar);
           \\ total += #vpar-num_edges_critical(vpar);
          );
     print1(total", ");
     );
  quit;
}

{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(got=is_edge_critical(vpar));
           count+=got);
     print1(count", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable() counts by gfs, ordered trees

limit = 14;
L = O(x^limit);
\\             1  2  3  4  5  6
g1 = x*Polrev([1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]) + L
g2 = x*Polrev([0, 1, 1, 1, 3, 6, 12, 29, 67, 157, 383, 936, 2312, 5786]) + L
g3 = x*Polrev([0, 0, 1, 1, 2, 5, 10, 23, 55, 129, 313, 770, 1904, 4766]) + L
g4 = x*Polrev([0, 0, 0, 1, 1, 2, 6, 12, 28, 70, 166, 408, 1020, 2546]) + L
g5 = x*Polrev([0, 0, 0, 0, 1, 1, 2, 6, 12, 28, 70, 166, 408, 1020]) + L

check_equal(g1, x + L)
check_equal(g5, g1*g4, "g5");                        \\ sole g5 out
check_equal(g4, g1*g3 + g4*g3 + L, "g4");
check_equal(g3, g1*g2 + g2*g3 + L, "g3");
check_equal(g2, g1*g1 + g2*g1 + g2*g3 + g4*g1 + L, "g2");

check_equal(g5,  x*g4         + L, "g5");
check_equal(g4,  x*g3 + g4*g3 + L, "g4");
check_equal(g3,  x*g2 + g2*g3 + L, "g3");
check_equal(g2,  x^2 + x*g2 + g2*g3 + x*g4 + L, "g2");

check_equal(g4,  x*g3/(1-g3) + L, "g4");
check_equal(g3,  x*g2/(1-g2) + L, "g3");
check_equal(g2,  x^2 + x*g2 + g2*g3 + x*g4 + L, "g2");

check_equal(g4,  x*x*g2/(1-g2)/(1-x*g2/(1-g2)) + L, "g4");
check_equal(g4,  x^2*g2/(1-(1+x)*g2)  + L, "g4");
g=g2;
check_equal(g,  x^2 + x*g + g*x*g/(1-g) + x*x*x*g/(1-g)/(1-x*g/(1-g)) + L, "g2");
check_equal(-g + x^2 + x*g + g*x*g/(1-g) + x^3*g/(1-g)/(1-x*g/(1-g)) + L, 0, "g2");
check_equal((-g + x^2 + x*g)*(1-g)*(1-x*g/(1-g)) + g*x*g*(1-x*g/(1-g)) + x^3*g + L, 0, "g2");
check_equal((-g + x^2 + x*g)*(1-g) - x*g*(-g + x^2 + x*g)*(1-g)/(1-g) + g*x*g*(1-x*g/(1-g)) + x^3*g + L, 0, "g2");
check_equal((-g + x^2 + x*g)*(1-g) + x^3*g - x*g*(-g + x^2 + x*g) + g*x*g*(1-x*g/(1-g)) + L, 0, "g2");
check_equal((-2*g + 1)*x^2 + (-g^3 + g)*x + (-g^3 + 2*g^2 - g) + L, 0, "g2");

check_equal((-x - 1)*g^3 + 2*g^2 + (-2*x^2 + x - 1)*g + x^2   + L, 0, "g2");
\\
check_equal((1+x)*g^3 - 2*g^2 + (2*x^2 - x + 1)*g - x^2   + L, 0, "g2");

check_equal(g^3 + 2/(-x - 1)*g^2 + ((-2*x^2 + x - 1)/(-x - 1))*g + x^2/(-x - 1)   + L, 0, "g2");

p = 'g^3 + 2/(-x - 1)*'g^2 + ((-2*x^2 + x - 1)/(-x - 1))*'g + x^2/(-x - 1);
subst(p,'g,'h - 2/(-x - 1)/3);
h = g - 2/3/(1+x);

check_equal(h^3 + ((6*x^3 + 3*x^2 - 1)/(3*x^2 + 6*x + 3))*h \
        + ((-27*x^4 - 18*x^3 - 9*x^2 + 2)/(27*x^3 + 81*x^2 + 81*x + 27))   + L, 0, "h2");

check_equal(h^3  + 1/3*(6*x^3 + 3*x^2 - 1)/(x+1)^2*h \
        - 1/27*(3*x-1)*(9*x^3 + 9*x^2 + 6*x + 2)/(x+1)^3   + L, 0, "h2");

\\ x^3 + p*x + q == 0

T = g1+g2+g3+g5;
check_equal(T, g1+g2+g3+g5 + L,  "T");
check_equal(T, x + g2 + x*g2/(1-g2) + x^3*g2/(1-g2)/(1-x*g2/(1-g2)) + L,  "T");
check_equal(T, x + g2 + x*g2/(1-g2) + x^3*g2/(1-(1+x)*g2)   + L,  "T");

quit


\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable() counts of states, ordered trees by recurrence

\\ check_equal(g2, g1*g1 + g2*g1 + g2*g3 + g4*g1 + L, "g2");

{
  my(limit=40,
     m=matrix(5,limit));
  m[1,1]=1;
  for(n=2,limit,
     m[5,n] = sum(i=1,n-1, m[1,i]*m[4,n-i]);

     m[4,n] = sum(i=1,n-1, m[1,i]*m[3,n-i] + m[4,i]*m[3,n-i]);

     m[3,n] = sum(i=1,n-1, m[1,i]*m[2,n-i] + m[2,i]*m[3,n-i]);

     m[2,n] = sum(i=1,n-1, m[1,i]*m[1,n-i] + m[2,i]*m[1,n-i]
                         + m[2,i]*m[3,n-i] + m[4,i]*m[1,n-i]);
     );

  for(t=1,5,
     print(m[t,]));
  print();
  print(m[1,]+m[2,]+m[3,]+m[5,]);
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable() counts of states, ordered

\\ [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
\\ [0, 1, 1, 1, 3, 6, 12, 29, 67, 157, 383, 936, 2312, 5786]
\\ [0, 0, 1, 1, 2, 5, 10, 23, 55, 129, 313, 770, 1904, 4766]
\\ [0, 0, 0, 1, 1, 2, 6, 12, 28, 70, 166, 408, 1020, 2546]
\\ [0, 0, 0, 0, 1, 1, 2, 6, 12, 28, 70, 166, 408, 1020]
\\ 
\\ [1, 1, 2, 2, 6, 12, 24, 58, 134, 314, 766, 1872, 4624, 11572]
\\ superseeker nothing on whole

{
  my(limit=16,
     m=matrix(5,limit));
  for(n=1,limit,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar)) == 'none,
           my(t=test_equimatchable_get_state(vpar));
           if(t, m[t,n]++);
           \\ if(n==3&&t==2,print(vpar));
          );
     );

  for(t=1,5,
     print(m[t,]));
  print();
  print(m[1,]+m[2,]+m[3,]+m[5,]);
 quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable() counts of states, rooted unlabelled

\\ [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
\\ [0, 1, 1, 1, 2, 3, 5, 9, 16, 28, 52, 96, 178, 336]
\\ [0, 0, 1, 1, 2, 3, 6, 9, 18, 31, 58, 106, 202, 375]
\\ [0, 0, 0, 1, 1, 2, 4, 7, 12, 24, 44, 82, 158, 302]
\\ [0, 0, 0, 0, 1, 1, 2, 4, 7, 12, 24, 44, 82, 158]
\\ 
\\ total
\\ not in OEIS: 1, 1, 2, 2, 5, 7, 13, 22, 41, 71, 134, 246, 462, 869


{
  my(limit=10,
     m=matrix(5,limit));
  for(n=1,limit,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(t=test_equimatchable_get_state(vpar));
           if(t, m[t,n]++);

           if(n==3&&t==2,print(vpar));
          );
     );

  for(t=1,5,
     print(m[t,]));
  print();
  print(m[1,]+m[2,]+m[3,]+m[5,]);
\\  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable() counts by gfs, rooted unlabelled

\\ WRONG plain products are an order on the children

limit = 8;
L = O(x^limit);
\\             1  2  3  4  5  6
g1 = x*Polrev([1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]) + L
g2 = x*Polrev([0, 1, 1, 1, 2, 3, 5, 9, 16, 28, 52, 96, 178, 336]) + L
g3 = x*Polrev([0, 0, 1, 1, 2, 3, 6, 9, 18, 31, 58, 106, 202, 375]) + L
g4 = x*Polrev([0, 0, 0, 1, 1, 2, 4, 7, 12, 24, 44, 82, 158, 302]) + L
g5 = x*Polrev([0, 0, 0, 0, 1, 1, 2, 4, 7, 12, 24, 44, 82, 158]) + L
\\
\\ g2 n=5    2 of
\\    *                         *
\\   / \   path-5            / / \ \    star-5
\\  *   *  rerooted         * *   * *
\\  |
\\  *
\\  |
\\  *
\\
\\ g3 n=6
\\    *    p=1               *                   3  *
\\    |    above g2 n=5      |                      |    \
\\    *                      *                      *      * 2
\\   / \                  / / \ \    star-5        / \     |
\\  *   *                * *   * *                *   *    *
\\  |
\\  *                                             
\\  |
\\  *
\\
\\ cf g3 n=3  *     g2 n=3               p=1 getting 2 x g2 children
\\            |                          in different order
\\            *           *
\\            |          / \
\\            *         *   *


check_equal(g1+g2+g3+g5, \
            x*Polrev([1, 1, 2, 2, 5, 7, 13, 22, 41, 71, 134, 246, 462, 869])+L);

check_equal(g1*g2, x*g2 + L);
g2*g3+L
check_equal(g1, x + L)
check_equal(g5, g1*g4, "g5");                        \\ sole g5 out
check_equal(g4, g1*g3 + g4*g3 + L, "g4");
check_equal(g3, g1*g2 + g2*g3 + L, "g3");
quit
check_equal(g2, g1*g1 + g2*g1 + g2*g3         + L, "g2");
check_equal(g2, g1*g1 + g2*g1 + g2*g3 + g4*g1 + L, "g2");

\\        1  2  3  4  5  6
\\ g2   = 0, 1, 1, 1, 2, 3
\\ x*g2 =    0, 1, 1, 1, 2    = 2 of g3 from p=2,v=1

\\          1  2  3  4  5  6
\\ g2     = 0, 1, 1, 1, 2, 3
\\ g3     = 0, 0, 1, 1, 2, 3
\\
\\ x^2*g3 =       0, 0, 1, 1
\\ x^3*g3 =          0, 0, 1
\\


quit
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

g1 = x
g2 = g1*g1 + g1*g2 + g2*g3 + g1*g4
g3 =         g1*g2 + g2*g3
g4 = g1*g3 + g3*g4
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

g1 = x
g2 = x^2 + x*g2 + g2*g3 + x*g4
g3 =       x*g2 + g2*g3
g4 = x*g3 + g3*g4
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

g1 = x
g2 = x^2 + x*g2 + g2*g3 + x*g4
g3 = x*g2/(1 - g2)
g4 = x*g3 + g3*g4
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

g1 = x
g2 = x^2 + x*g2 + x*g2^2/(1 - g2) + x*g4
g4 = x^2*g2/(1 - g2) + x*g4*g2/(1 - g2)
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

g1 = x
g2 = x^2 + x*g2 + x*g2^2/(1 - g2) + x*g4
g4 = x^2*g2/(1 - g2) / (1- x*g2/(1 - g2))
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

g2 = x^2 + x*g2 + x*g2^2/(1 - g2) + x*x^2*g2/(1 - g2) / (1- x*g2/(1 - g2))
print();
kill(g1);kill(g2);kill(g3);kill(g4);kill(g5);

x^2 - g + x*g + x*g^2/(1 - g) + x*x^2*g/(1 - g) / (1- x*g/(1 - g))
(x^2 - g + x*g)*(1 - g) + x*g^2 + x*x^2*g / (1- x*g/(1 - g))
x^2 + (-1 + x - x^2)*g + g^2 + x^3*g / (1- x*g/(1 - g))
x^2 + (-1 + x - x^2)*g + g^2 + x^3*(g - g^2)/(1 - (1+x)*g)
(x^2 + (-1 + x - x^2)*g + g^2)*(1 - (1+x)*g) + x^3*(g - g^2)
-2*x^2*g + 1*x^2 + -x*g^3 + x*g - g^3 + 2*g^2 - g
x^2 - (1+x + 2*x^2)*g + 2*g^2 - (1+x)*g^3

-(1+x)*g^3 + 2*g^2 - (1+x + 2*x^2)*g + x^2
(-(1+x)*g^3 + 2*g^2 - (1+x + 2*x^2)*g + x^2)/-(1+x)

g^3 - 2/(1+x)*g^2 + (1+x + 2*x^2)/(1+x)*g - x^2/(1+x)

l = - 2/(1+x)/3

p=subst(g^3 - 2/(1+x)*g^2 + (1+x + 2*x^2)/(1+x)*g - x^2/(1+x), \
      g, g-l)
subst(p,x,m)

print();
g^3 + ((6*x^3 + 9*x^2 + 6*x - 1)/(3*x^2 + 6*x + 3))*g \
    + ((-27*x^4 - 18*x^3 + 27*x^2 + 36*x + 2)/(27*x^3 + 81*x^2 + 81*x + 27))

g^3 + (6*x^3 + 9*x^2 + 6*x - 1)/(3*x^2 + 6*x + 3) * g \
    + 1/27*(-27*x^4 - 18*x^3 + 27*x^2 + 36*x + 2)/(x+1)^3

g=Polrev([1, 1, 2, 2, 5, 7, 13, 22, 41, 71, 134, 246, 462, 869]) + O(x^15)
g^3 - 2/(1+x)*g^2 + (1+x + 2*x^2)/(1+x)*g - x^2/(1+x)

quit

{
  \\ t=2  p=1 v=1
  \\ t=2  p=2 v=1
  \\ t=2  p=2 v=3
  \\ t=2  p=4 v=1

  \\ t=3  p=1 v=2
  \\ t=3  p=3 v=2

  \\ t=4  p=1 v=3
  \\ t=4  p=4 v=3

  \\ t=5  p=1 v=4

  for(t=1,5,
     for(p=1,5,
        for(v=1,5,
           if(test_is_equimatchable_table_lookup(p,v) == t,
              print("t="t"  p="p" v="v)))));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable() states

rel_validate(rel) =
{
  my(with,without,without_max,matched);
  [with,without,without_max,matched] = rel;
  
  if(without_max!='no_without_max,
     without_max >= without || error());
}

equimatchable_rel_add(p_rel,v_rel) =
{
  \\ print("equimatchable_rel_add() "p_rel"  "v_rel);
  rel_validate(p_rel);
  rel_validate(v_rel);

  my(v_with,v_without,v_without_max,v_matched);
  [v_with,v_without,v_without_max,v_matched] = v_rel;
  my(p_with,p_without,p_without_max,p_matched);
  [p_with,p_without,p_without_max,p_matched] = p_rel;

  my(v_max = min(if(v_with=='no_with,               99, v_with),
                 if(v_without_max=='no_without_max, 99, v_without_max)));
  \\ print(" v_max  "v_max);
  if(v_max==99, error());

  \\ with[p] = min(if(with_exists[p], with[p] + v_max,  #vpar),
  \\               without[p] + without[v] + 1);
  my(ret_with = min(if(p_with=='no_with, 99,  p_with + v_max),
                    p_without + v_without  + 1));
  \\ print(" ret_with  "ret_with);
  if(ret_with==99, error());

  ret_without = p_without + v_max;

  \\ without_max[p] += with[v];
  \\ if(!with_exists[v], without_max_exists[p] = 0);
  my(ret_without_max = if(p_without_max=='no_without_max || v_with=='no_with,
                          'no_without_max,
                          p_without_max + v_with));
  if(ret_without_max==99, error());
  \\ print(" ret_without_max  "ret_without_max);

  my(ret_matched=p_matched,
     matchnum_offset=0);
  if(p_matched=='un && v_matched=='un,
     ret_matched='m; matchnum_offset=1);     

  \\ without[p] += v_max;
  [if(ret_with=='no_with, ret_with,  
                          ret_with        - matchnum_offset),
   ret_without                            - matchnum_offset,
   if(ret_without_max=='no_without_max, ret_without_max,
                          ret_without_max - matchnum_offset),
   ret_matched];
}
is_dead_end(rel) =
{
  my(with,without,without_max,matched);
  [with,without,without_max,matched] = rel;

  (with!='no_with && with <= -1 && matched=='m)
  || (with!='no_with && with <= -4)
  \\|| (without_max!='no_without_max && without_max <= -3 && matched=='un)
  ;  
}
states=List([ [no_with,0,0,'un] ]);
vpars =List([ [0] ]);
{
  my(more=1);
  while(more,
        more=0;
        my(this_len=#states);
        for(i=1,this_len,
           for(j=1,this_len,
              my(p_rel = states[i],
                 v_rel = states[j]);
              if(is_dead_end(p_rel) || is_dead_end(v_rel), next());

              my(out = equimatchable_rel_add(p_rel,v_rel));
              if(!test_vector_search(states,out),
                 my(out_vpar=vpar_join(vpars[i],1,vpars[j]));
                 printf("%-22s + %-22s -> %s  tree %s\n", p_rel, v_rel, out, out_vpar);
                 if(is_dead_end(out),
                    print("  dead end");
                    ,
                    listput(states,out);
                    listput(vpars,out_vpar);
                    more=1;
                    next(3));;
                ))));
}

rel_v_max(rel) =
{
  my(with,without,without_max,matched);
  [with,without,without_max,matched] = rel;

 min(if(with=='no_with,               99, with),
     if(without_max=='no_without_max, 99, without_max));
}

print(#states," states");
for(i=1,#states, print(states[i]"      "rel_v_max(states[i])"   "vpars[i]));

print("one based");
table=matrix(#states+1,#states);
for(i=1,#states, table[#states+1,i]=#states);
{
  for(from=1,#states,
     for(digit=1,#states,
        my(p_rel = states[from],
           v_rel = states[digit]);
        if(is_dead_end(p_rel) || is_dead_end(v_rel), next());
        my(out = equimatchable_rel_add(p_rel,v_rel),
           to = test_vector_search(states,out));
        table[from,digit] = if(to,to,#states+1)));
}
print1("[undef, "); for(digit=1,#states, print1(undef",")); print("],");
for(from=1,#states+1, print1("[undef, "); for(digit=1,#states, print1(table[from,digit]",")); print("],"));
for(i=1,#states, if(rel_v_max(states[i])<0, print1(i","))); print(#states+1);

print("one based");
{
  for(from=1,5,
     print1("  ");
     for(digit=1,5,
        my(to=table[from,digit]);
        if(to>5,to=0);
        print1(to,if(digit!=5,",","")));
     print(if(from==5,"",";")));
}
for(i=1,5, if(rel_v_max(states[i])<0, print1(i","))); print();
quit


\\------------------------------------------------------------------------------
\\ vpar_is_equimatchable()

\\ free tree equimatchable
\\          n = 1  2  3  4  5  6
\\ not in OEIS: 1, 1, 1, 1, 2, 2, 4, 5, 9, 13, 23, 36, 64, 106, 190, 330, 598,
\\
\\ n=4 star
\\ n=5 star and path
\\ n=6 star and split *--*--*--*--*
\\                             |
\\                             *    
\\ n=7 star
\\     claw subdivided
\\     complete binary height=2
\\     tail  *--*--*--*--*
\\                    |\
\\                    * *
\\     all same diameter
\\
\\ n=14 diameter max
\\      *--*--*--*--*--*--*--*--*       pattern of max diameter equimatchable
\\      |     |     |     |     |    
\\      *     *     *     *     *
\\ 4+5+5==14
\\
\\ n=1 count 1 max_diameter 0 reps 1
\\ n=2 count 1 max_diameter 1 reps 1
\\ n=3 count 1 max_diameter 2 reps 1
\\ n=4 count 1 max_diameter 2 reps 1
\\ n=5 count 2 max_diameter 4 reps 1
\\ n=6 count 2 max_diameter 4 reps 1
\\ [0, 1, 2, 2, 2, 1, 6]     [2, 7] 7
\\ [0, 1, 2, 2, 1, 5, 5]     [2, 8] 8
\\ [0, 1, 2, 1, 4, 1, 6]     [3, 4] 4
\\ [0, 1, 1, 1, 1, 1, 1]     [1, 6] 6
\\ n=7 count 4 max_diameter 4 reps 3
\\ n=8 count 5 max_diameter 6 reps 1
\\ n=9 count 9 max_diameter 6 reps 2
\\ n=10 count 13 max_diameter 6 reps 5
\\ n=11 count 23 max_diameter 8 reps 1
\\ n=12 count 36 max_diameter 8 reps 2
\\ n=13 count 64 max_diameter 8 reps 8
\\ n=14 count 106 max_diameter 10 reps 1
\\ n=15 count 190 max_diameter 10 reps 3
\\ n=16 count 330 max_diameter 10 reps 11
\\ n=17 count 598 max_diameter 12 reps 1
\\ n=18 count 1069 max_diameter 12 reps 3
\\ n=19 count 1964 max_diameter 12 reps 15
\\ n=20 count 3590 max_diameter 14 reps 1


try(vpar) =
{
  if(vpar_is_equimatchable(vpar),
     count++;
     if(7 <= #vpar && #vpar<=7,
        print(vpar,"     ",vpar_matchnum_and_count(vpar)," ",vpar_maximal_matchings_count(vpar));
        \\ vpar_view(vpar)
       );

     my(diameter=vpar_diameter(vpar));
     if(diameter>max_diameter,
        max_diameter=diameter;
        max_diameter_reps=0;
        max_diameter_list=List([]));
     if(diameter==max_diameter,
        max_diameter_reps++;
        listput(max_diameter_list,vpar));
    );
}
{
  for(n=1,22,
     count=0;
     max_diameter=0;
     max_diameter_reps=0;
     max_diameter_list=List([]);
     read(Str("/tmp/trees-"n".gp"));
     print("n="n" count "count" max_diameter "max_diameter" reps "max_diameter_reps);
     if(max_diameter_reps==1,
        print(max_diameter_list[1]));
     \\ print(count",");
  );
  quit;
}
