\\ Copyright 2017, 2018, 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Count trees or forests where different types of seq vertex sequences come
\\ out the same.


default(recover,0)
read("../vpar.gp");
read("../vpar-wip.gp");
setrand(getwalltime()%2^32);


elems =
{[
  ["vpar_seq_diagpost", vpar_seq_diagpost ],
  ["vpar_seq_uplexqueue", vpar_seq_uplexqueue ],
  ["vpar_automorphism_ids_vector", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_automorphism_ids_vector(vpar))],

  ["vpar_seq_successive_paths", vpar_seq_successive_paths ],
  ["vpar_seq_successive_paths,down=1", vpar->vpar_seq_successive_paths(vpar,1) ],
  ["vpar_seq_childless_paths",        vpar_seq_childless_paths ],
  ["vpar_seq_childless_paths,down=1", vpar->vpar_seq_childless_paths(vpar,1) ],

  ["vpar_seq_num_children",        vpar_seq_num_children],
  ["vpar_seq_num_children,down=1", vpar->vpar_seq_num_children(vpar,1)],

  ["min descendant then height",
   vpar->
   my(heights    = vpar_subtree_heights_vector(vpar),
      descendant = vpar_minmax_descendants_vector(vpar,min),
      keys       = vector(#vpar,v, [descendant[v], heights[v]]));
   vecsort(keys,,1) \\ flag=1 for seq
  ],
  
  ["vpar_seq_upascend",         vpar_seq_upascend ],
  ["vpar_seq_upascend,ahead=1", vpar->vpar_seq_upascend(vpar,1)],
  ["vpar_seq_subtree_height",        vpar_seq_subtree_height ],
  ["vpar_seq_subtree_height,down=1", vpar->vpar_seq_subtree_height(vpar,1)],
  ["vpar_seq_subtree_height",        vpar_seq_subtree_height ],
  ["vpar_seq_subtree_height,down=1", vpar->vpar_seq_subtree_height(vpar,1)],
  ["vpar_seq_upqueue", vpar_seq_upqueue ],

  ["vpar_seq_blob",      vpar_seq_blob ],
  ["vpar_seq_dandelion", vpar_seq_dandelion ],
  \\ ["happy", vpar_seq_happy ],
  
  \\-------
  \\ these not reversible

  ["vpar_seq_diagqueue", vpar_seq_diagqueue ],
  ["vpar_seq_diagqueue,down=1", (vpar)->vpar_seq_diagqueue(vpar,1)],
  ["vpar_subtree_num_childless_vector", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_subtree_num_childless_vector(vpar))],
  ["vpar_seq_siblings", vpar_seq_siblings ],
  ["vpar_seq_downqueue", vpar_seq_downqueue ],

  ["vpar_seq_lexmin",            vpar_seq_lexmin ],
  ["vpar_eccentricities_vector", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_eccentricities_vector(vpar))],

  ["premax", vpar_seq_premax ],
  ["siblingpos", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_siblingpos_vector(vpar))],
  ["siblingpos,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_siblingpos_vector(vpar,1))],
  ["next_sibling", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_next_sibling_vector(vpar))],
  ["next_sibling,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_next_sibling_vector(vpar,1))],
  ["num_siblings", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_num_siblings_vector(vpar))],

  ["upqueue by parent", 
   vpar->
   my(num_children=vpar_INTERNAL_num_children_vecsmall(vpar),
      seq=vectorsmall(#vpar),
      qhead=0,
      qtail=0);

   for(v=1,#vpar,
      if(num_children[v]==0,
         seq[qhead++]=v));

   my(s=vecsort(Vec(seq[1..qhead]), (x,y)->cmp([vpar[x],x],[vpar[y],y])));
   for(i=1,#s, seq[i]=s[i]);

   while(qtail<qhead,
         my(v=seq[qtail++], p=vpar[v]);
         if(p,
            num_children[p]--;
            if(num_children[p]==0,
              \\ print("child "v" enqueue parent "p);
               seq[qhead++]=p)));
   seq;
  ],

  ["min descendant", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_descendants_vector(vpar,min))],
  ["max descendant", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_descendants_vector(vpar,max))],
  ["min descendant,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_descendants_vector(vpar,min),1)],
  ["max descendant,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_descendants_vector(vpar,max),1)],
  ["min childless descendant", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_childless_descendants_vector(vpar,min))],
  ["max childless descendant", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_childless_descendants_vector(vpar,max))],
  ["min at_height", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_height_descendants_vector(vpar,min))],
  ["max at_height", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_height_descendants_vector(vpar,max))],

  \\ min ancestor reversible for root=1 trees
  ["min ancestor", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_ancestors_vector(vpar,min))],
  ["max ancestor", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_ancestors_vector(vpar,max))],
  ["min ancestor,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_ancestors_vector(vpar,min),1)],
  ["max ancestor,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_ancestors_vector(vpar,max),1)],

  ["rows",vpar_seq_rows],
  ["rows,up=1",vpar->vpar_seq_rows(vpar,1)],
  ["preorder", vpar_seq_preorder ],
  ["postorder", vpar_seq_postorder ],

  ["weights", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_weights_vector(vpar))],
  ["row_widths", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_row_widths_vector(vpar))],

  \\ degrees reversible for root=1 trees
  ["degrees", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_degrees_vector(vpar))],
  ["depths", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_depths_vector(vpar))],
  ["parent", vpar_INTERNAL_seq_bucket_sort],
  ["parent,descend", vpar->vpar_INTERNAL_seq_bucket_sort(vpar,1)],
  ["max child", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_child_vector(vpar,max))],
  ["min child", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_child_vector(vpar,min))],
  ["max child,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_child_vector(vpar,max), 1)],
  ["min child,down=1", vpar->vpar_INTERNAL_seq_bucket_sort(vpar_minmax_child_vector(vpar,min), 1)]
  
  \\ [ "min descendant then depth", \\ = downsuccessive
  \\  vpar->
  \\  my(min_descendant = vpar_minmax_descendants_vector(vpar,min),
  \\     depths         = vpar_depths_vector(vpar),
  \\     keys = vector(#vpar,v, [min_descendant[v],depths[v]]));
  \\ vecsort(keys,,1) \\ flag=1 for seq
  \\  ],

 ];
}

{
  print("among ordered trees");
  \\ NOT row_widths, everything else reversible
  my(n=8,
     forest=1);
  for(i=1,#elems,
     my(elem=elems[i],
        name=elem[1],
        seq_func=elem[2],
        seen=Map(),
        reversible=1,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(seq=seq_func(vpar),
              vec=vecextract(vpar,seq));
           if(mapisdefined(seen,vec),
              reversible=0; break());
           mapput(seen,vec,1);
           );
     print(if(reversible,"    ","NOT "),name);
     );
  quit
}
{
  print("among labelled trees");
  my(n=6,
     forest=1);
  for(i=1,#elems,
     my(elem=elems[i],
        name=elem[1],
        seq_func=elem[2],
        seen=Map(),
        reversible=1);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));

           \\ #vpar_centres(vpar)==1 || next();
           \\ #vpar_centres(vpar)==2 || next();
 
           my(seq=seq_func(vpar),
              vec=vecextract(vpar,seq));
           if(mapisdefined(seen,vec),
              reversible=0; break());
           mapput(seen,vec,1);
           );
     print(if(reversible,"    ","NOT "),name);
     );
  quit;
}
{
  print("among unlabelled trees");
  \\ NOT row_widths, everything else reversible
  my(n=8,
     forest=1);
  for(i=1,#elems,
     my(elem=elems[i],
        name=elem[1],
        seq_func=elem[2],
        seen=Map(),
        reversible=1,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(seq=seq_func(vpar),
              vec=vecextract(vpar,seq));
           if(mapisdefined(seen,vec),
              reversible=0; break());
           mapput(seen,vec,1);
           );
     print(if(reversible,"    ","NOT "),name);
     );
  quit;
}
