\\ Copyright 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");

\\ A000262 "sets of lists", partition 1..n into ordered lists
\\ 1, 3, 13, 73, 501, 4051, 37633, 394353,
\\
F(n) =
{
  if(n<0, error(),
     n<=1, 1,
     n==2, 3,
     (2*n-1)*F(n-1) - (n-1)*(n-2)*F(n-2));
}
F=memoize(F);
check_equal(vector(#A000262_samples,n,n--; F(n)), A000262_samples);

\\ how many made with exactly k chains
Fk(n,k) =
{
  n>=0 || error();
  k>=0 || error();
  if(k==0 && n==0, return(1));
  if(k==0, return(0));
  if(k>n, return(0));
  if(k==1, return(n!));
  sum(l=1,n, binomial(n-1,l-1)*l! * Fk(n-l,k-1));
}
Fk=memoize(Fk);


\\-----------------------------------------------------------------------------

chains_n(vec) = sum(i=1,#vec, #vec[i]);

\\ "chains" is a vector of vectors.
\\ Return the index into vec of the element containing n, or 0 if none do.
chain_containing(vec,n) =
{
  for(i=1,#vec,
     if(test_vector_search(vec[i],n),
        return(i)));
  0;
}

\\ chains is a vector of vectors
chains_to_forest(chains) =
{
  \\ print("chains_to_forest()  "chains);
  my(n=chains_n(chains),
     vpar=vector(n));
  \\ print(" n="n);
  if(n,
     my(seen=vectorsmall(n));
     while(#chains,
           \\ print(" at chains = "chains);
           my(v=0,c);
           for(i=1,#vpar-1,
              if(seen[i] && (c=chain_containing(chains,v+1)),
                 v=i;break));
           if(v,
              p=vpar[v],
              c=1; p=0);
           my(chain=chains[c]);
           \\ print("  v="v" p="p" c="c" chain "chain);

           for(i=1,#chain-1,
              vpar[chain[i]] = chain[i+1];
              seen[chain[i]] = 1);
           my(e=chain[#chain]);
           vpar[e] = p;
           seen[e] = 1;
           chains=chains[^c];
           \\ print("  vpar "vpar"  seen "Vec(seen));
          ));
  vpar;
}

forest_to_chains(vpar) =
{
  \\ print("forest_to_chains()  "vpar);
  my(vec=vpar_childless_vertices(vpar),
     min_sibling=vpar_minmax_sibling_vector(vpar,min),
     ret=List([]));
  for(i=1,#vec,
     my(v=vec[i],  \\ childless vertex
        l=List([]));
     while(1,
           listput(l,v);
           min_sibling[v]==v || break;
           (v=vpar[v]) || break;
          );
     \\ print(" v="vec[i]" chain "Vec(l));
     listput(ret,Vec(l)));
  vecsort(Vec(ret), (x,y)->lex(vecmin(x),vecmin(y)));
}

diff(v1,v2) =
{
  vector(#v1,i,
         if(v1[i]==v2[i], 0,
            v1[i]==0 || v2[i]==0, 'Z,
            v1[i]-v2[i]));
}

{
  \\ duplications occurring

  \\ [3, 3, 0]    [3, 0, 0]    diff [0, Z, 0]
  \\     3          2  3
  \\    / \            |          [[1], [2, 3]] missing
  \\   1   2           1          [[1, 3], [2]] dup

  \\ n=4 childless 2,3
  \\
  \\ [4,1,0,0]       [4,1,1,0]      [4,4,1,0]
  \\   3  4             4               4
  \\      |             |               | \      
  \\      1             1               1  2
  \\      |             | \             |
  \\      2             2  3    d       3
  \\ [[2,1,4],[3]]  [[2,1,4],[3]]  [[3,1,4],[2]]
  \\                [[3,1,4],[2]]  [[3,1],[2,4]]
  \\
  \\ [0,0,4,1]       [0,4,4,1]      [0,4,1,1]
  \\   1  2            1                 1
  \\   |               |               / |          missing [[2,1], [3,4]]
  \\   4               4              3  4          missing [[1], [2], [3, 4]]
  \\   |               | \               |
  \\   3               2  3              2
  \\ [[3,4,1],[2]]  [[2,4,1],[3]]   [[2,4],[3,1]]
  \\                [[3,4,1],[2]]   [[2,4,1],[3]]
  \\
  \\ [4, 1, 0, 0]  [[2, 1, 4], [3]]  DUPLICATE of [4, 1, 1, 0]
  \\ [4, 1, 1, 0]  [[2, 1, 4], [3]]
  \\ [4, 4, 1, 0]  [[3, 1, 4], [2]]
  \\ [0, 4, 1, 1]  [[3, 1], [2, 4]]
  \\
  \\ [0, 0, 4, 1]  [[3, 4, 1], [2]]
  \\ [0, 4, 4, 1]  [[2, 4, 1], [3]]
  \\ missing [[2, 1], [3, 4]]

  my(n=4,
     func=min,
     forest=1,
     m=Map());
  forvec(vec=vpar_forvec_from(n,forest),
        my(vpar=vpar_from_upascend(vec),
           min_sibling=vpar_minmax_sibling_vector(vpar,min));
        if(test_vector_is_increasing(min_sibling),
           my(chains=forest_to_chains(vpar),
              old_vpar);
           if(1, \\ vpar_childless_vertices(vpar)==[2,3],
           print(vpar"  "chains,
                 if(mapisdefined(m,chains,&old_vpar),
                    Str("  DUPLICATE of "old_vpar" diff "diff(vpar,old_vpar)),  "")));
           mapput(m,chains, vpar)));
  my(vec=test_make_ordered_partitions(n));
  vec=vecsort(vec);
  for(i=1,#vec,
     if(!mapisdefined(m,vec[i]),
        print("missing "vec[i])));

  quit;
}
{
  \\ Number of different chains down occurring.
  \\ Is fewer than all possible ordered lists (A000262), other than for
  \\ root=1.
  \\
  \\ forests
  \\          n = 1  2   3
  \\ not in OEIS: 1, 3, 12, 63, 411, 3195, 28748,
  \\ trees
  \\ not in OEIS: 1, 2, 8, 43, 288, 2293, 21070,
  \\ root=1
  \\ A000262   1, 1, 3, 13, 73, 501, 4051, 37633,

  my(forest=1,
     func=min);
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec = vpar_minmax_sibling_vector(vpar,func));
           test_vector_is_increasing(vec) || next;
           my(chains=forest_to_chains(vpar));
           test_ordered_partition_validate(chains);
           mapput(m,chains, vpar));
     print1(matsize(Mat(m))[1]", ");
  );
  quit;
}

{
  for(n=0,10,
     my(vec=test_make_ordered_partitions(n));
     for(i=1,#vec,
        my(chains=vec[i]);
        my(vpar=chains_to_forest(chains));
        vpar_validate(vpar);

        my(min_sibling=vpar_minmax_sibling_vector(vpar,min));
        if(! test_vector_is_increasing(min_sibling),
           print("chains "chains" vpar "vpar"  min_sibling "min_sibling);
           quit);


        my(chains_again=forest_to_chains(vpar));
        \\ print("chains "chains" vpar "vpar"  chains again "chains_again);
        test_ordered_partition_validate(chains_again);
        check_equal(chains_n(chains_again), n);
        check_equal(chains_again, chains);
     ))
}
quit


\\-----------------------------------------------------------------------------
\\ min sibling increasing,
\\ count by num sets of siblings or num childless
\\
\\ by num childless matches ordered partitions k many parts

{
  for(n=1,7,
     print(vector(n+1,k,k--; test_A105278(n,k))));
}
{
  my(forest=1,
     func=min);
  for(n=1,8,
     my(count=vector(n));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           if(test_vector_is_increasing(vpar_minmax_sibling_vector(vpar,func)),
              \\ count[vpar_num_siblings_sets(vpar)]++;
              count[vpar_num_childless(vpar)]++;
             ));
     print(count);
  );
  quit;
}


\\-----------------------------------------------------------------------------

\\ A000262 Dennis P. Walsh
\\ Subset D comprising k many elements of 1..n
\\ Acyclic injective functions D -> 1..n count (n-1)!/(n-k-1)!.
\\ So A000262 = sum(k=0,n-1, binomial(n,k) * (n-1)!/(n-k-1)! )


for(n=0,8, printf("n=%2d ",n); for(k=0,n+1, printf("%7d", Fk(n,k))); print());

F_by_Fk(n) = sum(k=0,n, Fk(n,k));
check_equal(vector(10,n,n--; F(n)), vector(10,n,n--; F_by_Fk(n)), "F vs F_by_Fk");7


T(n) =
{
  n>=0 || error();
  if(n<=1, return(1));
  sum(k=0,n-1, (n-k+1)*Fk(n-1,k));
}
print("T    ",vector(10,n,n--; T(n)));

print("want ",[1, 1, 2, 8, 44, 302, 2462, 23132]);

{
  my(n=3);
  for(k=0,n-1, print("k="k"  ",n-k+1," * ",Fk(n-1,k)));
}

{
  my(forest=0,
     func=min);
  for(n=3,3,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              sibling_vector = vpar_minmax_sibling_vector(vpar,func),
              is_monotonic = test_vector_is_increasing(sibling_vector));
           count += is_monotonic;
           if(is_monotonic,
              print(vpar"  "sibling_vector));
     );
     print("count "count);
  );
}
quit

\\-----------------------------------------------------------------------------

\\ k..n = n-k+1 many
\\
\\ top t>=k and b>=k
Tsame(n,k) =
{
  (n>=0 && k>=1) || error("Tsame n="n" k="k);
  if(k>n, return(0));
  if(k<1, k=1);
  my(len=n-k+1);
  (len>=0 && len<=n) || error();
  sum(l=1,len, binomial(len,l)*l! * Tk(n-l,k));
}

\\ n-b-1 many >b excluding t>b
Tnewlow(n,k) =
{
  (n>=0 && k>=1) || error("Tnewlow n="n" k="k);
  sum(l=2,n,
      sum(b=1,k-1,
          sum(t=k,n,
              binomial(n-b,l-2)*(l-2)!
              * Tk(n-l,b))));
}

\\ all tops t>=k
Tk(n,k) =
{
  if(n==0, 1,
     k>n, 0,
     Tsame(n,k) + Tnewlow(n,k));
}

\\ b..n = l many is n-b+1=l so b=n-l+1
\\ t..n = l-1 many so t=n-l+2
\\ 1..b..n
\\ n-b+1 many >= b
\\ n-b-1 many >b excluding t>b
TfirstBsmall(n) =
{
  sum(l=2,n,
      sum(b=1,n-l+1,
          sum(t=b+1,n-l+2,
              binomial(n-b,l-2)*(l-2)!
              * Tk(n-l,b))));
}

\\ t..n = l many is n-t+1=l so t=n-l+1
\\ b..n = l-1 many so b=n-l+2
\\ b+1..n = n-(b+1)+1 many = n-b
TfirstTsmall(n) =
{
  sum(l=2,n,
      sum(t=1,n-l+1,
          sum(b=t+1,n-l+2,
              print("TfirstTsmall n="n" l="l" t="t" b="b" binomial "n-b","l-2" = "binomial(n-b,l-2)*(l-2)!" recurse "n-l","b-1" = "Tk(n-l,b-1));
              binomial(n-b,l-2)*(l-2)!
              * Tk(n-l,b-1))));
}

T(n) =
{
  if(n<0, error(),
     TfirstBsmall(n) + TfirstTsmall(n));
}

print("Tsame 1,1 = ",Tsame(1,1));


TfirstTsmall(3)
TfirstBsmall(3)
quit
vector(10,n,n--; T(n))