\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <non-crossing-partitions.gp
\\
\\ Ordered forests are in one-to-one correspondence with non-crossing
\\ partitions.  This is one of the many correspondences between "Catalan
\\ objects".
\\
\\ A non-crossing partition of elements 1..n is a partition of those
\\ integers into one or more sets, and where two sets either don't overlap
\\ or the entire range of one set is in a gap in elements of the other.  For
\\ example,
\\
\\         [1,4,8], [5,7], [2,3], [6]
\\
\\ Set 5,7 overlaps 1,2,8 but is entirely in the gap between 4 and 8.
\\
\\ A crossing example would instead be [1,4,6,8] and [5,7].  The 5,7 range
\\ is not in a gap of 1,4,6,8 since that latter has 6 within the 5 to 7
\\ range.
\\
\\ This sort of "sub-range in a gap" is exactly what occurs for child
\\ subtrees of a pre-order labelled forest.  The sets of siblings of a
\\ pre-order forest (vpar_siblings_sets()) are a non-crossing partition of
\\ vertex numbers 1..n.
\\


\\ vec is a vector of sets like [ [1], [2,4], [3] ] 
print_partition(vec) =
{
  for(i=1,#vec,
     if(i!=1, print1(", "));
     print1("[");
     for(j=1,#vec[i],
        if(j!=1, print1(","));
        print1(vec[i][j]));
     print1("]"));
  print();
}

n = 4;   \\ vertices

print("Pre-order forests siblings sets are non-crossing partitions:");
{
  my(vpar=vpar_preorder_first(n));
  until((vpar=vpar_preorder_next(vpar,1))=='none,  \\ forests
        my(siblings_sets=vpar_siblings_sets(vpar));
        print_partition(siblings_sets));
}

print();
print("Post-order labelled likewise:");
{
  my(vpar=vpar_preorder_first(n));
  until((vpar=vpar_preorder_next(vpar,1))=='none,  \\ forests
        my(postorder_vpar=vpar_relabel_postorder(vpar),
           siblings_sets=vpar_siblings_sets(postorder_vpar));
        print_partition(siblings_sets));
}

\\-----------------------------------------------------------------------------
