\\ Copyright 2017, 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Tests requiring GP 2.13 or higher.

default(recover,0);
default(strictargs,1);

option_verbose=0;
option_randomize=1;
option_progress=1;

read("vpar.gp");
read("vpar-wip.gp");  \\ multiple reads ok to the various install() or alias()
read("vpar-wip.gp");
read("vpar-wip.gp");
read("test-helpers.gp");

\\ informational diagnostics
print("default(secure) is "default(secure));
print("have call-by-reference = ",iferr(eval("(~x)->0");1,e,0));


\\-----------------------------------------------------------------------------
progress("vpar_preorder_nextR()");

{
  my(vpar=[0,1]);
  vpar_preorder_nextR(~vpar,1); \\ forest
  check_equal(vpar, [0,0]);
  vpar=[0,1];
  check_equal(iferr(vpar_preorder_nextR(~vpar,'forest);0, e, 1),
              1,
  "vpar_preorder_nextR() error on forest=symbol (whereas it should be 0,1)");
}

{
  my(vpar=[0,1,2,3]);
  check_equal(vpar_preorder_nextR(~vpar), 1);
  check_equal(vpar, [0,1,2,2], "changed");
}
{
  my(vpar=[0,1,1,1]);
  check_equal(vpar_preorder_nextR(~vpar), 0,
              "vpar_preorder_nextR() no next");
  check_equal(vpar, [0,1,1,1],
              "vpar_preorder_nextR() no next, unchanged");
}
{
  my(vpar=[0,1,2,3],
     new_vpar=vpar_preorder_next(vpar));
  check_equal(new_vpar, [0,1,2,2], "vpar_preorder_next() new");
  check_equal(vpar,     [0,1,2,3], "vpar_preorder_next() old unchanged");
}

{
  for(forest=0,1,
     for(n=0,8-forest,
        my(vpar=vpar_preorder_first(n),
           prev_vpar='none,
           count=0);
        while(1,
              vpar_validate(vpar);
              if(!forest, check_equal(vpar_is_tree(vpar), 1));
              check_equal(vpar_is_preorder(vpar), 1);

              if(prev_vpar!='none,
                 check_equal(lex(prev_vpar, vpar), 1,
                             "vpar_preorder_nextR() lex decreasing vpar"));
              count++;

              prev_vpar = vpar;
              my(ret=vpar_preorder_nextR(~vpar,forest));
              check_equal(ret==0 || ret==1, 1,
                          "vpar_preorder_nextR() return flag");
              if(!ret, break));

        check_equal(prev_vpar, vpar_preorder_last(n,forest));
        check_equal(count, vpar_count_ordered(n,forest));
     ));
}

\\-----------------------------------------------------------------------------
progress("vpar_unlabelled_nextR()");

{
  my(vpar=[0,1,2,3]);
  check_equal(vpar_unlabelled_nextR(~vpar), 1);
  check_equal(vpar, [0,1,2,2], "changed");
}
{
  my(vpar=[0,1,1,1]);
  check_equal(vpar_unlabelled_nextR(~vpar), 0,
              "vpar_unlabelled_nextR() no next");
  check_equal(vpar, [0,1,1,1],
              "vpar_unlabelled_nextR() no next, unchanged");
}
{
  my(vpar=[0,1,2,3],
     new_vpar=vpar_unlabelled_next(vpar));
  check_equal(new_vpar, [0,1,2,2], "vpar_unlabelled_next() new");
  check_equal(vpar,     [0,1,2,3], "vpar_unlabelled_next() old unchanged");
}

{
  for(forest=0,1,
     for(n=0,8-forest,
        my(vpar=vpar_unlabelled_first(n),
           prev_vpar='none,
           count=0);
        while(1,
              vpar_validate(vpar);
              if(!forest, check_equal(vpar_is_tree(vpar), 1));
              check_equal(vpar_is_unlabelled(vpar), 1);

              if(prev_vpar!='none,
                 check_equal(lex(prev_vpar, vpar), 1,
                             "vpar_unlabelled_nextR() lex decreasing vpar"));
              count++;

              prev_vpar = vpar;
              my(ret=vpar_unlabelled_nextR(~vpar,forest));
              check_equal(ret==0 || ret==1, 1,
                          "vpar_unlabelled_nextR() return flag");
              if(!ret, break));

        check_equal(prev_vpar, vpar_unlabelled_last(n,forest));
        check_equal(count, vpar_count_unlabelled(n,forest));
     ));
}

\\==============================================================================
progress("No globals left over");
\\ test-symbols.txt is all symbols appearing in vpar.gp.

check_equal(vpar,'vpar, "variable \"vpar\" should be unset");
{
  my(test_symbols=readstr("test-symbols.txt"));
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     check_equal(eval(test_symbol),
                 eval(Str("'"test_symbol)),
                 Str("leftover global variable: "test_symbol)));
}
\\ {
\\   my(test_symbols=readstr("test-funcs.txt"));
\\   for(test_i=1,#test_symbols,
\\      my(test_symbol=test_symbols[test_i]);
\\      my(value=eval(test_symbol));
\\      check_equal(value == eval(Str("'"test_symbol))
\\                  || type(value) == "t_CLOSURE", 1,
\\                  Str("leftover non-function: "test_symbol)));
\\ }


\\-----------------------------------------------------------------------------
{
  printf("total %d tests in %.1f seconds\n",
         check_count,
         if(option_progress,progress_total_time,gettime()) /1000);
}

\\-----------------------------------------------------------------------------
