\\ Copyright 2017, 2018, 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");
\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=1;
progress_prev=-1;
progress(str) =
{
  my(t=getwalltime()\2000);
  if(t!=progress_prev,
     progress_prev=t;
     print(str));
}


\\ A114567 balanced binary trailing substring ?



\\------------------------------------------------------------------------------
\\ vpar_edgecover_num()


vpar_INTERNAL_edgecover_num_table = {Vecsmall([
     3, 0,  2,  \\ p state 0 must cover
     3, 2,  2,  \\ p state 1 uncovered same
     4, 4, -2,  \\ p state 2 uncovered smaller
     1, 0,  0   \\ no p, v is root
\\   --------
\\ v=0  1   2 state,  new child under p
])};
vpar_edgecover_num(vpar) = \
  vpar_INTERNAL_xxxnum(vpar,vpar_INTERNAL_edgecover_num_table);

{
  \\ optional cover of root is at most 1 smaller
  for(n=4,18,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(covered = edgecover_num(vpar),
              plant = vpar_join([0],1,vpar),
              optional = edgecover_num(plant) - 1,
              delta = covered - optional);
           if(delta!=0 && delta!=1,
              print(covered" "optional"  "vpar))));
  quit;
}


\\------------------------------------------------------------------------------
\\ A102379  height-balanced binary trees min vertices
\\   0,1,2,4,6,9,12,17,22,29,36,46,56
\\ n=0 1 2 3 4 5  6  7  8  9 10 11 12
A102379(n) =
{
  if(n==0,0,
     A102379(n-1) + A102379(floor((n-1)/2)) + 1);
}
A102379=memoize(A102379);

vector(10,n,A102379(n))
OEIS_check_func("A102379",,['offset,0])

{
  my(n=17, v=vector(n), s=0);
  v[n]=1;
  forstep(i=n,2,-1,
     print(v"  ",s+sum(j=1,#v,A102379(j)*v[j]));
     my(lo=floor((i-1)/2));
     if(lo==0,break);
     my(t=v[i]); v[i]=0;
     v[i-1] += t;
     v[lo] += t;
     s+=t);
  print("final ",v);
  print(v[1] + 2*v[2] + s);
}
quit


\\------------------------------------------------------------------------------
{
  my(n=6,
     vpar=vpar_free_first(n));
  until((vpar=vpar_free_next(vpar))=='none,
        my(poly=vpar_subfree_poly(vpar,,1));
        print(Vec(poly)"     "vpar);
       );
  quit;
}


\\------------------------------------------------------------------------------
\\ A352885
\\ Values never edge covers count

test_edgecovers_count(vpar) =
{
  my(ret=0);
  forvec(flags=vector(#vpar,i,[0,1]),
     ret+=vpar_is_edgecover_flags(vpar,flags));
  ret;
}

seen=Map();
{
  for(n=1,14,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(num,count); 
           \\ [num,count] = vpar_matchnum_and_count(vpar);
           \\ count=vpar_maximal_matchings_count(vpar);
           \\ count=vpar_matchings_count(vpar);
           count=vpar_edgecovers_count(vpar);
           if(mapisdefined(seen,count),next);
           mapput(seen,count,vpar)));
}
\\ mapget(seen,19)
\\ #mapget(seen,19)
select(n->!mapisdefined(seen,n),[1..100])

\\ how many vertices needed for n edge covers
apply(n->my(vpar); if(mapisdefined(seen,n,&vpar), #vpar-1, -1), [2..100])
\\ not in OEIS: 4, 5, 6, 6, 7, 7, 7, 8, 8, 8, 9, 8, 9, 9, 9, 9, 9
\\ how many edges
\\ not in OEIS: 3, 4, 5, 5, 6, 6, 6, 7, 7, 7, 8, 7, 8, 8, 8, 8, 8

\\ cf
\\ matchings counts
\\    3, 4, 5, 6, 7, 7, 7, 8, 9, 8    vertices
\\    2, 3, 4, 5, 6, 6, 6, 7, 8, 7, 8     num edges
\\ n= 2  3  4  5  6  7  8  9   num matchings
quit;


\\------------------------------------------------------------------------------
\\ Complete Binary Tree Distance from Leaf

a(n) =
{
  my(vpar=vpar_make_complete_tree(2,n),
     v=vpar_childless_vertices(vpar)[1],
     dists=vpar_path_lengths_vector(vpar,v));
  sum(i=1,#dists, dists[i]<=n);
  sum(i=1,#dists, dists[i]<=n && i!=v);
}

\\ exactly distance n
\\ 1, 2, 2, 4, 4, 8, 8, 16, 16, 32, 32, 64, 64, 128, 128, 256
\\ A016116 a(n) = 2^floor(n/2).
\\ A016116 ,1,1,2,2,4,4,8,8,16,16,32

\\ distance <= n
\\ 2, 4, 6, 10, 14, 22, 30, 46, 62, 94, 126, 190, 254, 382
\\ A027383 Number of balanced strings of length n

\\ A136252       1, 3, 5, 9, 13, 21, 29, 45, 61, 93, 125, 189, 253, 381, 509,
\\   0, 1, 3, 5, 9, 13, 21, 29, 45, 61, 93, 125, 189, 253
A136252(n) = vector_modulo([4, 6],n) <<(n\2) - 3;
{
  my(want=OEIS_samples("A136252")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A136252(n)), want,
              "A136252()");

  check_equal(vector(16,n,n--; a(n+1)),
              vector(16,n,n--; A136252(n)));
}

quit




\\------------------------------------------------------------------------------
\\ Variation on Chung and Graham - Tree with Subtrees all n Vertices
\\
\\ 1,2,3,5,7,10,14

all_subtrees(vpar) =
{
  my(m=Map());
  mapput(m,vpar,1);
  forstep(s=#vpar,2,-1,
     foreach(Mat(m)[,1],sub,
        #sub==s || next;
        foreach(vpar_leaves(sub),l,
           mapput(m, vpar_delete_vertex(sub,l), 1))));
  Set(Mat(m)[,1]);
}
all_subtrees([0,1,1,1])

{
  my(ret=List([]));
  for(n=1,8,
     my(want=Set(vpar_free_all(n)));
     print("n="n"  want ",want);
     for(t=n,oo,
        print("  try t="t);
        my(vpar=vpar_free_first(t),
           count=0,
           found=0);
        until((vpar=vpar_free_next(vpar))=='none,
              my(subs=all_subtrees(vpar));
              subs=select(s->#s==n,subs);
              subs=Set(apply(vpar_to_free,subs));
              \\ print("subs ",subs);
              \\ print("want ",want);
              if(subs==want,
                 print("found "vpar);
                 found++);
              count++);
        if(found,
           listput(ret,t);
           print("result ",t," num found ",found);
           break);
        print("  nothing among "count)));

  print(ret=Vec(ret));
  quit;
}

\\------------------------------------------------------------------------------

{
  for(n=1,4,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(count=0);
           forsubset(#vpar,set,
              ret += is_irredundant_set(vpar,set));
           print(vpar" "count));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_unlabelled_nextR() speed

vpar_unlabelled_nextV(vpar,forest=0) =
{
  /* flip sense of forest variable so 1=tree, 0=forest,
     and also provoke an error if forest is not a number */
  forest=(forest<1);
  my(q);
  forstep(p=#vpar,2,-1,
     (q=vpar[p]) > forest || next;  /* q>0 for forest, q>1 for a tree */
     my(g=vpar[q],
        offset=p-q);
     until(q++; p++>#vpar,
           vpar[p] = vpar[q] + if(vpar[q]!=g, offset));
     return(vpar));
  'none;
}
{
  my(n=5000,
     limit=10000);
  print("n="n);
  gettime();
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_next(vpar)) == 'none,
        count++;
        if(count>limit,break));
  print("vpar_unlabelled_next()  ",gettime());

  gettime();
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_nextV(vpar)) == 'none,
        count++;
        if(count>limit,break));
  print("vpar_unlabelled_next()  ",gettime());

  gettime();
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until(!vpar_unlabelled_nextR(~vpar),
        count++;
        if(count>limit,break));
  print("vpar_unlabelled_nextR() ",gettime());
  quit;
}

{
  my(n=16);
  gettime();
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_next(vpar)) == 'none,
        count++);
  count==vpar_count_unlabelled(n)|| error();
  print("vpar_unlabelled_next()  ",gettime());

  gettime();
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_nextV(vpar)) == 'none,
        count++);
  count==vpar_count_unlabelled(n)|| error();
  print("vpar_unlabelled_next()  ",gettime());

  gettime();
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until(!vpar_unlabelled_nextR(~vpar),
        count++);
  count==vpar_count_unlabelled(n)|| error();
  print("vpar_unlabelled_nextR() ",gettime());
  quit;
}


\\------------------------------------------------------------------------------
\\ Call by Reference Speed

next_return(~v) = v[1]++; v;
next_noret(~v) = v[1]++; 0;
{
  my(v=vector(10^5));
  gettime();
  for(rep=1,500, next_noret(v));
  print(gettime()" now "v[1]"  noret");

  gettime();
  for(rep=1,500, next_return(v));
  print(gettime()" now "v[1]);

  gettime();
  for(rep=1,500, next_noret(~v));
  print(gettime()" now "v[1]"  noret");

  gettime();
  for(rep=1,500, next_return(~v));
  print(gettime()" now "v[1]);

  quit;
}



\\------------------------------------------------------------------------------
\\ free trees determined by degrees    unigraphical
\\ n=1                  11     14       17
\\   2,3,4,4,6,5,6,7,7,7,9,8,9,10,10,10,12,

\\ 5L 1 -- 1 4L -- 1 5L  
\\ 5+1 + 1+4 + 1+5 == 17
\\ vector(17,n, if(n==1,2, n==2,3, 2+n\/2+(n%3==2))) - \
\\ [2,3,4,4,6,5,6,7,7,7,9,8,9,10,10,10,12]

{
  for(n=17,17,
     my(m=Map(),
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(degrees=vecsort(vpar_degrees_vector(vpar)));
           \\ for(i=1,#degrees,if(degrees[i]==2,next(2)));
           my(c=0); mapisdefined(m,degrees,&c);
           mapput(m, degrees, c+1));
     m=Mat(m);
     my(degs=m[,1]~,
        counts=m[,2]~,
        determined=0);
     for(i=1,#counts,
        if(counts[i]==1,
           my(vpar=vpar_make_degrees(degs[i]));
           vpar=vpar_reroot(vpar,#vpar);
           print(degs[i]" tree "vpar);
           if(1,vpar_view(vpar));
           determined++;
          ));
     print1(determined,","));
  quit;
}

\\-----------------------------------------------------------------------------
\\ total fixed vertices of automorphisms

{
  \\ free trees
  \\ 2, 4, 6, 13, 26, 56, 122, 278, 634, 1494, 3540, 8542,

  for(n=1,20,
     my(vpar=vpar_free_first(n),
        total=0);
     until(!vpar_free_nextR(~vpar),
           total+=vecsum(apply(s->s==1,vpar_orbit_sizes(vpar))));
     print1(total,",");
     );
  quit;
}


\\------------------------------------------------------------------------------

L=List([1,2,3])
apply(x->x+1, L)
L[1..2]
lex(L[1..2], L[2..3])
lex([1,2], [2,3])

vecextract(L,[1,3])
List([1,2,3]) == [1,2,3]
quit

\\------------------------------------------------------------------------------
\\ Call by Reference Nexts

foo(~vec) = vec[1]=5; vec=[1,2,3,4];
my(u=[9,8,7]); foo(~u); u

vpar_preorder_nextR(~vpar,forest=0) =
{
  \\    g         v raise to under its grandparent g, decreasing depth.
  \\    |         If no grandparent then for forest set v as new root.
  \\    p         If cannot raise v then look to raise v-1 and set v
  \\   / \        and onwards as a path down which is maximum depths
  \\ v-1  v       for them so smallest depths decrease.
  \\
  \\ bitor() is used for testing forest so as to catch a mistake like
  \\ passing symbol 'forest instead of flag 0,1.

  forstep(v=#vpar,1,-1,
         my(p=vpar[v]);
         if(p && bitor(vpar[p],forest),
            vpar[v] = vpar[p];   \\ raise v to be under its grandparent
            return(1));
         vpar[v]=v-1);   \\ ending path down from v-1 if can raise v-1 etc
  0;
}

{
  my(n=500,
     M=vpar_preorder_first(n),c=0);
  gettime();
  for(i=1,10000,
     vpar_preorder_nextR(~M) || break; c++);
  print("M ",gettime());

  my(V=vpar_preorder_first(n),c=0);
  gettime();
  for(i=1,10000,
     (V=vpar_preorder_next(V))!='none || break; c++);
  print("V ",gettime());
  print(M==V);
  quit;
}
{
  my(M=vpar_preorder_first(8),
     V=vpar_preorder_first(8));
  for(i=1,10,
     vpar_preorder_nextR(~M);
     V=vpar_preorder_next(V);
     print(M" "V" "M==V));
  quit;
}


\\------------------------------------------------------------------------------
\\ A345971 Series Reduced Trees Count by Degree Sequence

want1={[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 
       2, 3, 1, 4, 1, 1, 1, 1, 1, 2, 3, 2, 2, 4, 6, 2, 1, 1, 1, 1, 1, 2, 3, 3, 2, 
       2, 4, 9, 4, 8, 1, 1, 1, 1, 1, 1, 2, 3, 3, 2, 2, 3, 1, 4, 9, 6, 9, 2, 8, 
       14, 4, 1, 1, 1, 1, 1, 1, 2, 3, 3, 3, 2, 3, 2, 2, 4, 9, 9, 9, 9, 4, 8, 25, 
       14, 15, 1, 1, 1, 1, 1, 1, 1
]};

want={[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1,
       2, 3, 1, 4, 1, 1, 1, 1, 1, 2, 3, 2, 2, 4, 6, 2, 1, 1, 1, 1, 1, 2, 3, 3, 2,
       2, 4, 9, 4, 8, 1, 1, 1, 1, 1, 1, 2, 3, 3, 2, 2, 3, 1, 4, 9, 6, 9, 2, 8,
       14, 4, 1, 1, 1, 1, 1, 1, 2, 3, 3, 3, 2, 3, 2, 2, 4, 9, 9, 9, 9, 4, 8, 25,
       14, 15, 1, 1, 1, 1, 1, 1
]};
#want
#want1
want==want1[1..106] || error();

row(n) =
{
  my(m=Map(),
     vpar=vpar_free_first(n));
  until((vpar=vpar_free_next(vpar))=='none,
        my(degrees=vecsort(vpar_degrees_vector(vpar)));
        for(i=1,#degrees,if(degrees[i]==2,next(2)));
        my(c=0); mapisdefined(m,degrees,&c);
        mapput(m, degrees, c+1));
  m=Mat(m);
  my(degs=m[,1]~,
     counts=m[,2]~);
  degs==Set(degs) || error();
  degs==vecsort(degs,lex) || error();

  my(codes=apply(dv->prod(i=1,#dv,prime(dv[i])), degs));
  for(i=1,#degs, print(degs[i]," ",codes[i]," ",m[i,2]));

  my(p=vecsort(codes,,1));
  vecextract(codes,p) == Set(codes) || error();

  if(vecextract(counts,p) != counts,
     print("different n="n);
     print("extract ",vecextract(counts,p));
     print("counts  ",counts);
     quit);

  counts;
}

{
  for(n=8,12,
     print(row(n));
     print());
  quit
}

v=[]
for(n=4,20, v=concat(v,row(n)); if(#v>=#want,break))
print("v    ",v)
print("want ",want)
v[1..#want] == want
quit



\\------------------------------------------------------------------------------
\\ vpar_degrees_vector() occurring

{
  \\ num sorted degrees vectors of trees
  \\ is number of partitions of n
  \\ vecsum(parts) = 2*(#parts-1)
  \\      1,1,1,2,3,5,7,11,15,22,30,42,56,77,101,135,
  \\ n= 0 1 2 3 4 5 6 7  8  9
  \\
  \\ forests
  \\ not in OEIS: 1,2,3,6,10,18,29,48,74,115,171,254,366,526,739,1034,
  \\         n= 0 1 2 3 4 5 6 7  8  9
  \\ total to n
  \\ not in OEIS: 1,3,6,12,22,40,69,117,191,306,477,731,
  \\ diffs
  \\ 1,1,1,3,4,8,11,19,26,41,56,83,112,160,213,295,389,526,686,911,
  \\ A024786 Number of 2s in all partitions of n
  \\         Number of singletons in all partitions of n-1 (part occurring once)

  my(forest=1,
     prev=0);
  for(n=1,20,
     my(m=Map());
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(degrees=vecsort(vpar_degrees_vector(vpar)));
           if(vecmin(degrees)==0,next);
           mapput(m, degrees, 1));
     \\ print1(#m-prev,",");
     print1(#m,",");
     prev=#m;
     );
  quit;

  my(forest=1,
     total=0,
     prev=0);
  for(n=1,16,
     my(m=Map());
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(degrees=vecsort(vpar_degrees_vector(vpar)));
           mapput(m, degrees, 1));
     \\ total+=#m;
     print1(#m-prev,",");
     \\ print1(#m,",");
     \\ print1(total,",");
     \\if(n<=4,print(m));
     prev=#m;
     );
  quit;

}


\\ trees binomial(2*n-2,n-1) same as num_children
\\              1 1 1 3 10 35 126
\\          n = 0 1 2 3  4  5   6
\\ n=3 [1, 1, 2], [1, 2, 1], [2, 1, 1]

\\ vpar_INTERNAL_exponential_transform([1, 1, 1, 3, 10, 35, 126])

\\ forests degrees vectors
\\              1, 1, 2, 7, 30, 136, 629
\\ not in OEIS:          7, 30, 136, 629
\\
\\ n=3 [0, 0, 0],
\\     [0, 1, 1], [1, 0, 1], [1, 1, 0],
\\     [1, 1, 2], [1, 2, 1], [2, 1, 1]
\\
\\ n+1 quantities >=0, sum 2*n-2, last quantity even,
\\ 2*n-2+n = 3*n-2 positions,
vector(7,n, binomial(3*n-2,n))

my(n=4); binomial(3*n-2,n-1)
vector(7,n, sum(i=0,n, if(2*n-2-2*i>=0, binomial(3*n-2-2*i,n))))

flags_to_runs(v) =
{
  my(l=List([]), pos=0);
  while(pos<=#v,
        \\ print("at pos="pos);
        pos++;
        my(r=0);
        while(pos<=#v && v[pos]==0, pos++;r++);
        listput(l,r);
        \\ print(" push r="r);
       );
  Vec(l);
}
check_equal(flags_to_runs([1,1]), [0,0,0]);
check_equal(flags_to_runs([1,0,0,1]), [0,2,0]);
check_equal(flags_to_runs([1,0,1,0,1]), [0,1,1,0]);

check_equal(vpar_is_degrees_vector([1,1,1,1],1), 1);
check_equal(vpar_is_degrees_vector([6,0,0,0],1), 0);

check_equal(vpar_num_children_vectors_count(5,-1), 35);
{
  my(n=4, count=0, count_odd=0);
  print("sum <= ",2*(n-1));
  forvec(v=vector(3*n-2,i,[0,1]),
        if(vecsum(v) != n, next());  \\ separators
        my(runs=flags_to_runs(v),
           degrees=runs[^#runs],
           is_degrees=vpar_is_degrees_vector(degrees,1));
        if(runs[#runs]%2, count_odd++;next());
        print(v"   "runs" = "vecsum(degrees)"  "is_degrees);
        count++;
        total_degrees += is_degrees);
  print(count" odd "count_odd" total "count+count_odd"  degrees "total_degrees);
  quit
}


check_equal(vpar_num_children_vectors_count(3,-1), 3);
check_equal(vpar_num_children_vectors_count(2,-1), 1);

\\ select(v->v[4]%2==0, test_compositions(4,[0,4]))
\\ v= test_compositions(4);



\\ 10 + 1*1*7 + 1*3*2 + 3*3*1
check_equal(vpar_num_children_vectors_count(4,-1), 10);
my(n=4,t=1); check_equal(vpar_num_children_vectors_count(t,-1), 1);
my(n=4,t=1); check_equal(binomial(n-1,t-1), 1);
my(n=4,t=1); check_equal(F(n-t), 7);
my(n=4,t=2); check_equal(vpar_num_children_vectors_count(t,-1), 1);
my(n=4,t=2); check_equal(binomial(n-1,t-1), 3);
my(n=4,t=2); check_equal(F(n-t), 2);
my(n=4,t=3); check_equal(vpar_num_children_vectors_count(t,-1), 3);
my(n=4,t=3); check_equal(binomial(n-1,t-1), 3);
my(n=4,t=3); check_equal(F(n-t), 1);

{
  my(forest=1); \\ trees or forests
  for(n=0,6,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              degrees=vpar_degrees_vector(vpar));
           vpar_is_degrees_vector(degrees,forest) || error(degrees);
           if(mapisdefined(seen,degrees),next());
           mapput(seen,degrees,1);
           count++);
     my(l=Mat(seen)[,1]);
     l=vecsort(l);
     \\ if(n==3,print(l));
     \\ for(i=1,#l, print(l[i]));
     print(count"  "vpar_num_children_vectors_count(n,-1)"  "vpar_num_children_vectors_count(n)"  "vpar_num_children_vectors_count(n,1));
     );
  quit;
}


\\------------------------------------------------------------------------------

partition_to_tree_degrees(p) =
{
  p=Vec(p);
  my(t=vecsum(p));
  concat(vector(t+2-#p,i,1),apply(e->e+1,p));
}
primeprod(vec) = vecprod(apply(prime,vec));
{
  my(l=List([]));
  for(n=4,20,
     print("n="n);
     my(m=0,md);
     forpart(p=n-2,
        my(degrees=partition_to_tree_degrees(p),
           P=primeprod(degrees));
        if(P%3==0,next);
        \\ print(degrees" "P);
        vpar_is_degrees_vector(degrees) || error();
        if(P>m,m=P;md=degrees);
     );
     print("max "m" at "md);
     listput(l,m);
     );
  print(Vec(l));
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_degrees_vector() repeats
\\
\\ [1, 1, 1, 1, 1, 1, 3, 3, 4]  from  [[0, 1, 2, 2, 2, 1, 6, 6, 1],
\\                                     [0, 1, 2, 2, 1, 5, 5, 1, 1]]
\\
\\ [0, 1, 2, 2, 2, 1, 6, 6, 1]      [0, 1, 2, 2, 1, 5, 5, 1, 1]
\\       8   9   5                  4   9   6
\\       |   |   |                  |   |   |
\\       6---1---2---3              2---1---5---7
\\       |       |                  |   |
\\       7       4                  3   8

\\       8   9   5                  4   9   6
\\       |   |   |                  |   |   |
\\   *---6---1---2              *---2---1---5
\\       |       |                      |   |
\\       7       4                      8   *

{
  my(vpar1=[0, 1, 2, 2, 2, 1, 6, 6, 1]);
  check_equal(vpar_is_free_isomorphic(vpar_make_caterpillar([2,1,2,0]),vpar1), 
              1);
  check_equal(vpar_is_free_isomorphic(vpar_make_caterpillar([0,2,1,2]),vpar1), 
              1);

  my(vpar2=[0, 1, 2, 2, 1, 5, 5, 1, 1]);
  check_equal(vpar_is_free_isomorphic(vpar_make_caterpillar([2,2,1,0]), vpar2),
              1);
  check_equal(vpar_is_free_isomorphic(vpar_make_caterpillar([0,1,2,2]), vpar2),
              1);

  check_equal(vpar_is_free_isomorphic(vpar1,vpar2),  0);
  check_equal(vecsort(vpar_degrees_vector(vpar1)),
              vecsort(vpar_degrees_vector(vpar2)));
  check_equal(vecsort(vpar_degrees_vector(vpar1)),
              [1,1,1,1,1,1,3,3,4]);
  quit;
}

{
  for(n=1,10,
     print("n="n);
     my(m=Map());
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(degrees=vecsort(vpar_degrees_vector(vpar)));
           if(setsearch(degrees,2),next);  \\ series reduced
           my(v=[]); mapisdefined(m,degrees,&v);
           mapput(m,degrees, concat(v,[vpar])));
     m=Mat(m);
     for(i=1,matsize(m)[1],
        if(#m[i,2]>=2,
           print(m[i,1],"  from  ",m[i,2])));
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_degrees_vector()

  \\ print("vpar_is_degrees_vector() "vec" forest="forest);
  \\ print(" init ",#vec==0 || vecmin(vec) >= (forest<1 && #vec>1));
  \\ print(" even ",!bittest(s=vecsum(vec),0));
  \\ print(" sum  "vecsum(vec));
  \\ print(" cmp  ",(#vec-1)<<1);
  \\ print(" if   ",if(forest==1,
  \\                   !bittest(s=vecsum(vec),0)        \\ even
  \\                     && s <= (#vec-1)<<1,
  \\                   s == (#vec-1)<<1));


{
  my(forest=0); \\ trees or forests
  for(n=7,8,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              roots=vpar_num_roots(vpar),
              singletons=vpar_num_singletons(vpar));

           my(degrees=vpar_degrees_vector(vpar));
           degrees=vecsort(degrees);
           if(mapisdefined(seen,degrees),next());
           mapput(seen,degrees,1);

           my(L=sum(v=1,#degrees, degrees[v]==1),
              D=sum(v=1,#degrees, degrees[v]>=2),
              M=sum(v=1,#degrees, degrees[v]!=0),
              sum_d = sum(v=1,#degrees, if(degrees[v]>=2,degrees[v])));

           sum_d <= L || error(degrees" sum_d="sum_d" L="L);

           if(singletons<1,next());
           if(roots-singletons<2,next());
           if(D==0,next());
           print(degrees"  L="L" K="K" D="D" M="M"  L+K="L+K"  sum(d)="sum_d" vecsum="vecsum(degrees));
     ));
  quit;
}
{
  my(forest=1); \\ trees or forests
  for(n=7,8,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              roots=vpar_num_roots(vpar),
              singletons=vpar_num_singletons(vpar));
           if(singletons<1,next());
           if(roots-singletons<2,next());

           my(degrees=vpar_degrees_vector(vpar));
           degrees=vecsort(degrees);
           if(mapisdefined(seen,degrees),next());
           mapput(seen,degrees,1);

           my(L=sum(v=1,#degrees, degrees[v]==1),
              K=sum(v=1,#degrees, degrees[v]==2),
              D=sum(v=1,#degrees, degrees[v]>=3),
              M=hammingweight(degrees),
              sum_d = sum(v=1,#degrees, if(degrees[v]>=3,degrees[v])));
           if(D==0,next());
           print(degrees"  L="L" K="K" D="D" M="M"  L+K="L+K"  sum(d)="sum_d" vecsum="vecsum(degrees));
     ));
  quit;
}


\\------------------------------------------------------------------------------

\\ A194439  = partitions A000041(n-2)
\\ "regions" A206437 maybe
\\
\\ degree sequences is partitions per Geoffrey Critzer in A000041
\\ vector(20,n,numbpart(n))    \\ A000041
\\                  1,2,3,5
\\

degs(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar));
  vecsum(degrees) == 2*(#vpar-1) || error();
  abs(sum(v=1,#vpar, sqr(degrees[v])));
}
{
  for(n=2,20,
     my(vpar=vpar_free_first(n),
        m=Map(),
        max_vpar,
        max_t);
     until((vpar=vpar_free_next(vpar))=='none,
           my(t=degs(vpar));
           \\ print(t);
           if(t>=0,
              my(c); mapisdefined(m,t,&c);
              mapput(m,t,c+1));
           if(t>max_t,
              max_t=t;max_vpar=vpar);
           );
     my(row=Mat(m)[,1]~);
     \\ print1(count",");
     \\ print("n="n"  "m[,1]~);
     \\ print1(#m,",");
     print("n="n"  "row);
     \\ print("  max "max_t" ",(n-1)*(n-2),"   ",max_vpar" rep "mapget(m,max_t));
     \\ check_equal(max_t, );

     m=Map();
     forpart(p=2*n-2, mapput(m,sum(i=1,#p,sqr(p[i])),1) ,,[n,n]);
     \\ print("      "Mat(m)[,1]~);
     row == Mat(m)[,1]~ || error();

     \\ print(apply(s->s-(4*n-6),row));
  );
  quit
}


\\------------------------------------------------------------------------------
\\ tree containing all small trees

verbose=0;
n_limit = 10;
tree_to_index = Map();
index_to_tree = List([]);
all_to_n = vector(n_limit);
xx=[0, 1, 2, 3, 2, 2, 1, 7, 7]
vpar_to_free([0, 1, 2, 3, 3, 1, 6, 1, 1])
vpar_to_free(xx)
quit
\\
{
  for(n=1,n_limit,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           listput(index_to_tree,vpar);
           if(vpar_is_free_isomorphic(vpar,xx),
              print("found xx");
              print(vpar);
              print(xx);
              quit);
           \\ print("n="n" "vpar);
          );
     all_to_n[n] = bitneg(0,#index_to_tree)<<1);
  tree_to_index = Map(Mat([Col(index_to_tree), [1..#index_to_tree]~]));
}
\\ index_to_tree
\\ tree_to_index
print("all_to_n ",apply(test_to_binary,all_to_n));

index_to_cover = vector(#index_to_tree,i, 1<<i);
found_all = vector(#index_to_tree);

target=1;
{
  for(i=1,#index_to_tree,
     my(vpar=index_to_tree[i]);
     if(verbose,
        print("at i="i" cover="test_to_binary(index_to_cover[i])," size="#vpar," ",vpar));
     if(bitand(index_to_cover[i], all_to_n[target]) == all_to_n[target],
        print("  found all ",test_to_binary(all_to_n[target]),
              " at size "#vpar" "vpar);
        found_all[target] = #vpar;
        target++;
       );
     if(#vpar>=n_limit,break);
     for(v=1,#vpar,
        my(bigger=concat(vpar,v));
        bigger=vpar_to_free(bigger);
        \\ vpar_is_free(bigger) || error(bigger);
        my(to_i);
        if(!mapisdefined(tree_to_index,bigger,&to_i),
           print("oops, not found");
           print("  size="#bigger" "bigger);
           vpar_validate(bigger);
           quit);
        my(new_cover=bitor(index_to_cover[to_i], index_to_cover[i]));
        if(new_cover != index_to_cover[to_i],
           if(verbose,
              print("  new cover "test_to_binary(new_cover)," for "bigger));
           index_to_cover[to_i] = new_cover)));
}
\\ print("index_to_cover ",index_to_cover);
print("found_all ",found_all[1..target]);
quit


\\------------------------------------------------------------------------------
\\ A102417  egf ???
\\ needs spread for (n/2)!

limit=14;
L = O(x^limit);
f(x) = 1/(1-x) + L;  \\ of n!
check_equal(test_egf_terms(f(x),limit), vector(limit,n,n--; n!), "f");

h(x) = f(x^2) + L;
h(x)
check_equal(vector(limit,n,n--; 1), \
            test_egf_terms(exp(x+L),limit), "exp");
check_equal(vector(limit,n,n--; if(n==0,0, (-1)^(n-1) * (n-1)!)), \
            test_egf_terms(log(1+x+L),limit), "log");
check_equal(vector(limit,n,n--; 1), \
            test_egf_terms(cos(x),limit), "log x^2");

\\ check_equal(test_egf_terms(h(x),limit), \
\\             vector(limit,n,n--; if(n%2==0,n!,0)), "h");

\\ d(x) = f(x) - f(x^2) + L;  \\ of n! - (n/2)!

vector(30,n,if(n%2==0,(n/2)!,0))

quit


\\------------------------------------------------------------------------------


A057058_row(n) = my(v=vector(n), pos=1, i=1); \
  while(pos<=#v, v[pos]=i; i++; pos+=2); \
  pos--; if(pos>n,pos-=2); \
  while(i<=n, v[pos]=i; i++; pos-=2); \
  v;

A294673(n)=znorder(Mod(if(n%2, 2, -2), 4*n+3))
{
  my(want=OEIS_samples("A294673")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A294673(n)), want,
              "A294673()");
  check_equal(vector(50,n, test_permorder(Vecsmall(A057058_row(2*n)))),
              vector(50,n, A294673(n-1)),
              "row 2n order");
}

\\ order of the "milk shuffle" (1,n,2,n-1,3,n-2,...)
A003558(n) =
{
  my(m=Mod(2,2*n+1), k=znorder(m));
  if(k%2==0 && m^(k/2) == -1, k/2, k);
}
{
  my(want=OEIS_samples("A003558")); \\ OFFSET=0
  \\ print(want);
  \\ print(vector(#want,n,n--; A003558(n)) -  want);
  \\ print(vector(#want,n,n--; A003558(n) ==2*want[n+1]));
  check_equal(vector(#want,n,n--; A003558(n)), want,
              "A003558()");
}
{
  check_equal(vector(50,n, test_permorder(Vecsmall(A057058_row(n)))),
              vector(50,n, A003558(n-1)));
}

vector(50,n, #test_perm_to_cycles(Vecsmall(A057058_row(n))))
\\ not in OEIS: 1, 2, 2, 2, 3, 2, 2, 4, 3, 2, 4, 2, 3, 4, 2, 4, 5, 4, 2, 4, 3, 4, 6, 2, 3, 6, 2, 4, 5, 2, 2, 8, 7, 2, 4, 2, 5, 6, 4, 2, 5, 2, 8, 4, 5, 6, 8, 4, 3, 8

\\  Least number m > 0 such that 2^m == +- 1 (mod 2n + 1).
\\ A003558 ,1,1,2,3,3,5,6,4,4,9,6,11,10,9,14,5,5,12,18,12,10,7,12,23,21,8,26,20,9,29,30,6,6,33,22,35,9,20,30,39,27,41,8,28,11,12,10,36,24,15,50,51,12,53,18,36,14,44,12,24,55,20,50,7,7,65,18,36,34,69,46,
\\ A216066 a(n) = card {cos((2^k)*Pi/(2*n-1)): k in N}.
\\ A216066 ,1,1,2,3,3,5,6,4,4,9,6,11,10,9,14,5,5,12,18,12,10,7,12,23,21,8,26,20,9,29,30,6,6,33,22,35,9,20,30,39,27,41,8,28,11,12,10,36,24,15,50,51,12,53,18,36,14,44,12,24,55,20,50,7,7,65,18,36,34,69,46,

quit




\\------------------------------------------------------------------------------
\\ rooted balanced

is_balanced(vpar) =
{
  my(depths=vpar_depths_vector(vpar),
     num_children=vpar_num_children_vector(vpar),
     height=vecmax(depths),
     depth_to_num_children=vector(height+1,i,'none));
  for(v=1,#vpar,
     if(depth_to_num_children[depths[v]+1]=='none,
        depth_to_num_children[depths[v]+1]=num_children[v],
        depth_to_num_children[depths[v]+1]!=num_children[v],
        return(0)));
  1;
}
{
  \\ unlabelled balanced
  \\ 1,1,2,3,5,6,10,11,16,19,26,27,40,
  \\ A003238
  for(n=1,15,
    my(vpar=vpar_unlabelled_first(n),
      count=0);
    until((vpar=vpar_unlabelled_next(vpar))=='none,
          count += is_balanced(vpar));
     print1(count",");
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ distance from centre is equivalent to eccentricity
\\
\\ ecc
\\   bicentral,  to near centre then far centre out to its ecc
\\   unicentral, to centre then out to its ecc other than itself

centre_distances_vector(vpar) =
{
  my(centres=vpar_centres(vpar),
     d1=vpar_path_lengths_vector(vpar,centres[1]));
  if(#centres==2,
     my(d2=vpar_path_lengths_vector(vpar,centres[2]));
     for(v=1,#d1, d1[v]=min(d1[v],d2[v])));
  d1;
}

is_free_balanced(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar),
     ecc=centre_distances_vector(vpar),
     radius=vecmax(ecc),
     ecc_to_degree=vector(radius+1,i,'none));
  for(v=1,#vpar,
     if(ecc_to_degree[ecc[v]+1]=='none,
        ecc_to_degree[ecc[v]+1]=degrees[v],
        ecc_to_degree[ecc[v]+1]!=degrees[v],
        return(0)));
  1;
}
{
  \\ free balanced
  \\ not in OEIS: 1,1,1,2,2,3,4,4,5,8,7,7,13,11,12,19,16,17
  for(n=1,27,
    my(vpar=vpar_free_first(n),
      count=0);
    until((vpar=vpar_free_next(vpar))=='none,
          count += is_free_balanced(vpar));
     print1(count",");
  );
  quit;
}

{
  for(n=1,7,
    my(vpar=vpar_free_first(n));
    until((vpar=vpar_free_next(vpar))=='none,
          my(ecc=vpar_eccentricities_vector(vpar),
             dist=centre_distances_vector(vpar),
             diff=ecc-dist);
          print(vpar);
          print(ecc"  "vpar_diameter(vpar)" "#vpar_centres(vpar)
               " radius "vpar_radius(vpar));
          print(dist);
          print(diff" "vecmin(diff)" "vecmax(diff));
          print();
         );
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ binary trees count by height

\\ A328349 binary trees of min height
\\ A073429
\\ A073345

[1, 1, 2, 1, 6, 6, 6, 1, 94, 114, 116, 94, 60, 16, 8, 1]

\\ how many binary trees of n vertices with height <= h
count_binary_trees_height_le(n,h) =
{
  n>=0 || error("count_binary_trees_height_le() is for n>=0");
  if(n==0, h>=-1,  \\ height of empty tree is -1
     h<-1, 0,
     n--;h--; \\
     sum(x=0,n,   count_binary_trees_height_le(x,  h)
                * count_binary_trees_height_le(n-x,h)));
}
count_binary_trees_height_le=memoize(count_binary_trees_height_le);
count_binary_trees_height(n,h) = \
  count_binary_trees_height_le(n,h) - count_binary_trees_height_le(n,h-1);

vector(20,n,n+=3; count_binary_trees_height(n,3))
{
  for(n=1,17,
     printf("n=%2d:",n);
     for(h=0,n+5,
        \\ printf(" %5d,",count_binary_trees_height(n,h)));
        printf(" %5d,",count_binary_trees_height_le(n,h)));
     print());
}
vector(20,n, count_binary_trees_height_le(n,logint(n,2)))

{
  check_equal(vector(20,n,count_binary_trees_height   (n,logint(n,2))),
              vector(20,n,count_binary_trees_height_le(n,logint(n,2))));

  check_equal(vector(20,n, count_binary_trees_height_le(n,n)),
              vector(20,n, vpar_count_ordered(n,1)),
              "count_binary_trees_height_le() last");

  check_equal(vector(20,n,sum(h=-1,n,count_binary_trees_height(n,h))),
              vector(20,n,vpar_count_ordered(n,1)),
              "count_binary_trees_height() total row");

  check_equal(vector(#A328349_samples,n, count_binary_trees_height(n,logint(n,2))),
              A328349_samples,
              "A328349 count binary trees of minimal height");

  \\ transposed:
  \\    n= 0 1 2
  \\ h=-1  1 0 0 0 0 0 0 0 0 ...
  \\ h=0   0 1 0 0 0 0 0 0 0 ...
  \\ h=1   0 0 2 1 0 0 0 0 0 ...
  \\ h=0   0 0 0 4 6 6 4 1 0 ...
  \\ h=0   0 0 0 0 8 20 40 68 94 ...
  check_equal(test_antidiagonals_of_func(#A073345_samples,count_binary_trees_height,0,-1),
              A073345_samples,
              "A073345 table count binary trees n vertices height k");
}
{
  \\ A073429 triangle n vertices height k
  my(l=List([]));
  for(n=0,#A073429_samples,
     for(h=-1,n-1,
        if(#l>=#A073429_samples,break(2));
        listput(l,count_binary_trees_height(n,h))));
  check_equal(Vec(l), A073429_samples,
              "A073429 triangle rows count binary trees n vertices height k");
}
{
  \\ A137560 binary trees J childless and height <= N
  \\ columns h with n ranging 0 <= n <= 2*2^h
  \\
  \\ ~/OEIS/b137560.txt   2.7mb, 8204 terms
  A137560_samples = Vecrev(OEIS_bfile_gf("A137560"));
  A137560_samples=A137560_samples[1..1500];
  my(l=List([1,0]));
  for(h=-1,#A137560_samples,
     for(n=0,2*2^h,
        if(#l>=#A137560_samples,break(2));
        listput(l,count_binary_trees_height_le(n,h))));
  check_equal(Vec(l), A137560_samples,
              "A137560");
}
{
  \\ A202019
  \\ columns h with n ranging n=2*2^h-1 down to 0
  \\ http://oeis.org/A202019/b202019.txt  2.7mb
  my(l=List([]));
  for(h=-1,#A202019_samples,
     forstep(n=2*2^h-1,0,-1,
        if(#l>=#A202019_samples,break(2));
        listput(l,count_binary_trees_height_le(n,h))));
  check_equal(Vec(l), A202019_samples,
              "A202019");
}

\\ {
\\   \\ A137867 columns, but also something else between
\\   \\
\\   my(l=List([]));
\\   for(h=-1,#A137867_samples,
\\      for(n=0,2^h,
\\         if(#l>=#A137867_samples,break(2));
\\         listput(l,count_binary_trees_height(n,h))));
\\   check_equal(Vec(l), A137867_samples,
\\               "A137867 triangle rows count binary trees height k and across n vertices");
\\ }

vector(10,n,n+=3; vpar_count_ordered(n,1) - count_binary_trees_height(n,logint(n,2)))
\\ not in OEIS: 8, 36, 128, 428, 1336, 4748, 16680, 58692, 207952, 742872


{
  my(forest=1);
  for(n=1,6,
     my(vpar=vpar_preorder_first(n),
        minh=logint(n,2),
        seen=vector(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(dd=vpar_diagdepths_vector(vpar),
              h=vecmax(dd));
           seen[h+1]++);
     while(seen[1]==0,seen=seen[^1]);
     for(i=2,#seen,seen[i]+=seen[i-1]);
     print(seen);
  );
  quit
}

\\------------------------------------------------------------------------------
\\ binary trees minimal height

\\ 2 and 3 -> h=1
\\ 4 .. 7  -> h=2

[1, 1, 2, 1, 6, 6, 6, 1, 94, 114, 116, 94, 60, 16, 8, 1]
{
  print1("[   ");
  my(forest=1);
  for(n=1,16,
     my(vpar=vpar_preorder_first(n),
        minh=logint(n,2),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(dd=vpar_diagdepths_vector(vpar),
              h=vecmax(dd));
           \\ print(vpar" "dd" "h);
           h>=minh || error(vpar" h="h" "minh);
           if(h == minh,
              count++));
     \\ print("n="n" minh="minh"  "count);
     print1(count", ");
  );
  quit
}

\\------------------------------------------------------------------------------
\\ bipartite degrees as two equal sides

{
  for(n=9,20,
     my(m=Map());
     forpart(p=n-1,
        my(p=Vec(p));
        my(np=#p,
           nq=n-#p);
        forpart(q=n-1,
           my(q=Vec(q));
           #p+#q == n || error(n," ",p," ",q);
           sum(i=1,#p,p[i]==1) + sum(i=1,#q,q[i]==1) >= 2 || next;
           vecsum(p)+vecsum(q)==2*n-2 || error();
           my(d=vecsort([p,q]));
           if(mapisdefined(m,d),next);
           mapput(m,d,1);
           \\ print(d);
           ,
           ,
           [nq,nq] \\ num parts
       ));
    my(m1=m);
    m=Map();
    my(vpar=vpar_free_first(n));
    until((vpar=vpar_free_next(vpar))=='none,
          my(b=vpar_bipartition(vpar),
             degrees=vpar_degrees_vector(vpar),
             d=apply(s->vecsort(vecextract(degrees,s)), b));
          d=vecsort(d);
          vecsum(d[1])+vecsum(d[2])==2*n-2 || error();
          \\ printf("%-25s %-25s %-25s\n", d[1],d[2],
          \\        vecsort(concat(d)));
          mapput(m,d,1);
         );
     print(m==m1);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ bipartite degrees
\\
\\ distinct pairs of degrees vectors each half
\\              1,1,1,2,3,6,10,18,30,52,84,137,218,343,529,810,1221,1823,
\\          n = 1 2 3 4 5 6
\\ not in OEIS:       2,3,6,10,18,30,52,84,137,218,343,529,810,1221
\\ two ways [1,1,1,2,2,3]
{

  for(n=1,6,
     my(vpar=vpar_free_first(n),
        m=Map());
     until((vpar=vpar_free_next(vpar))=='none,
           my(b=vpar_bipartition(vpar),
              degrees=vpar_degrees_vector(vpar),
              d=apply(s->vecsort(vecextract(degrees,s)), b));
           d=vecsort(d);
           printf("%-25s %-25s %-25s\n", d[1],d[2],
                  vecsort(concat(d)));
           mapput(m,d,1);
           );
     \\ print(#m"  cf ",vpar_count_free(n));
     print1(#m",");
  );
  quit
}

degs(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar),
     flags = vpar_bipartition_flags(vpar));
  vecsum(degrees) == 2*(#vpar-1) || error();
  abs(sum(v=1,#vpar, sqr(degrees[v])*(-1)^flags[v]));
}

\\------------------------------------------------------------------------------
\\ alternating sum degrees
\\ A335790

\\  1 - 2 + 1 - 2 = -2
\\  1,2,1,2

{
  want=[
 0,
 2,
 0, 6,
 2, 4, 12,
 0, 2, 6, 10, 20,
 0, 2, 4, 6, 8, 10, 12, 18, 30,
 0, 2, 4, 6, 8, 10, 14, 16, 20, 28, 42,
 0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 22, 24, 26, 30, 40, 56,
 0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 34, 36, 38, 42, 54, 72
];
}
seq_around(vpar) =
{
  \\ my(r=vpar_root(vpar),
  \\    first_child=vpar_minmax_child_vector(vpar,min),
  \\    next_sibling=vpar_next_sibling_vector(vpar),
  \\    l=List([]),
  \\    v=r);
  \\ while(v,
  \\       print("at "v);
  \\       listput(l,v);
  \\       if(first_child[v],
  \\          v=first_child[v];
  \\          print("  first child "v);
  \\          ,
  \\          while(1,
  \\                v=vpar[v];
  \\                print("  up to "v);
  \\                if(!v, return(Vec(l)));
  \\                listput(l,v);
  \\                if(next_sibling[v],
  \\                   v=next_sibling[v];
  \\                   print("  next sibling "v);
  \\                   break))));
  \\ Vec(l);
  my(seq=vpar_seq_preorder(vpar),
     l=List([]),
     prev=0);
  for(i=1,#seq,
     my(v=seq[i]);
     while(vpar[v]!=prev,
           prev=vpar[prev];
           listput(l,prev));
     listput(l,v);
     prev=v);
  while((prev=vpar[prev]),
        listput(l,prev));
  Vec(l);
}
\\         1
\\       /    \
\\      2       5
\\     / \     / \
\\    3   4   6   7
check_equal(seq_around([0,1]), [1,2,1]);
check_equal(seq_around([0,1,2,3]), [1,2,3,4,3,2,1]);
check_equal(seq_around([0,1,2,2, 1,5,5]), [1,2,3,2,4,2, 1, 5,6,5,7,5,1]);

trav(vpar) =
{
  \\ my(vpar=vpar_reroot(vpar,vpar_leaves(vpar)[1]));
  \\ print("vpar ",vpar);
  my(degrees=vpar_degrees_vector(vpar),
     seq=seq_around(vpar),
     t=sum(i=1,#seq-1, degrees[seq[i]]*(-1)^(i-1)));
  abs(t);
}
degs(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar),
     flags = vpar_bipartition_flags(vpar));
  vecsum(degrees) == 2*(#vpar-1) || error();
  abs(sum(v=1,#vpar, sqr(degrees[v])*(-1)^flags[v]));
}
{
  for(n=2,20,
     my(vpar=vpar_free_first(n),
        m=Map(),
        max_vpar,
        max_t);
     until((vpar=vpar_free_next(vpar))=='none,
           my(t=trav(vpar));
           t == degs(vpar) || error();
           \\ print(t);
           if(t>=0,
              my(c); mapisdefined(m,t,&c);
              mapput(m,t,c+1));
           if(t>max_t,
              max_t=t;max_vpar=vpar);
           );
     \\ print1(count",");
     \\ print("n="n"  "m[,1]~);
     \\ print1(#m,",");
     print("n="n"  "Mat(m)[,1]~);
     \\ print("  max "max_t" ",(n-1)*(n-2),"   ",max_vpar" rep "mapget(m,max_t));
     \\ check_equal(max_t, );
  );
  quit
}
\\ vector(50,n, n*(n-1))
\\ A002378  n*(n+1))
\\ cf offset A279019

\\ row widths
\\ 1,1,2,3,5,9,11,16,22,27,33,41,49,59,68,
\\ v=[1,1,2,3,5,9,11,16,22,27,33,41,49,59,68,80,92]
\\ vector(#v-1,i,v[i+1]-v[i])

\\------------------------------------------------------------------------------
\\ A075374 eigensequence of a triangle
\\ 1, 4, 19, 110, ...
\\
\\ lower triangle matrix M
\\ http://qntmpkt.blogspot.com/2012/01/eigensequence-of-triangle-eigentriangle.html
\\ eigensequence-of-triangle-eigentriangle.html

{
  Q = [1,     0,0,0,0,0;
       1,     0,0,0,0,0;
       1, 1,    0,0,0,0;
       1, 2, 1,   0,0,0;
       1, 3, 3, 1,  0,0;
       1, 4, 6, 4, 1, 0 ];
}
Q
Q^2
Q^3
Q^4
Q^5

print("-------------");
{
  Q = [1,     0,0,0,0,0;
       1,     0,0,0,0,0;
       3, 1,    0,0,0,0;
       5, 3, 2,   0,0,0;
       7, 8, 9, 10,  0,0;
       11,12,13,14,15, 0 ];
}
Q
Q^2
Q^3
Q^4
Q^5

print("-------------");
{
  \\ a(n+2) = n*a(n+1)-a(n).

  M = [1,      0,0,0,0,0;
      -1, 4,     0,0,0,0;
       0,-1, 5,    0,0,0;
       0, 0,-1, 6,   0,0;
       0, 0, 0,-1, 7,  0;
       0, 0, 0, 0,-1, 8 ];
}
v=[1, 4, 19, 110, 751]~
e=concat([1]~,v)
e==[1, 1, 4, 19, 110, 751]~
M*e
{
[ -1, 4,
       0,-1, 5,
       0, 0,-1, 6,
       0, 0, 0,-1, 7,
       0, 0, 0, 0,-1, 8]
}

print("-------------");
{
  \\ b[n] = a[n] + sum(t=1,i-1, b[n-t]*(n-t)*a[t])

  M = [1*1,                0,0,0,0,0;
       1*2, 2*1,             0,0,0,0;
       1*3, 2*2, 3*1,          0,0,0;
       1*4, 2*3, 3*2, 4*1,       0,0;
       1*5, 2*4, 3*3, 4*2, 5*1,    0;
       1*6, 2*5, 3*4, 4*3, 5*2, 6*1 ];
}
v=[1, 4, 19, 110, 751]~
e=concat([1]~,v)
e==[1, 1, 4, 19, 110, 751]~
M*e

\\ A003991 multiplication table
{ [1*1,
   1*2, 2*1,
   1*3, 2*2, 3*1,
   1*4, 2*3, 3*2, 4*1,
   1*5, 2*4, 3*3, 4*2, 5*1,
   1*6, 2*5, 3*4, 4*3, 5*2, 6*1 ]
}

quit


\\------------------------------------------------------------------------------
\\ Ordered Trees, <= 2 children, and nowhere equal size sibling subtrees
\\ to be A335562

want = {[
 1, 1, 1,      \\ n = 1,2,3
 3, 5, 13,
 29, 71, 165, 421, 1029, 2647, 6593,
              17241, 43801, 115555, 297513, 791337, 2062737, 5516755,
              14497373, 38994597, 103269053, 278921927, 742950845,
              2014648253, 1101243061, 1792404615, 823127081, 327227265
];}
any_samesize(vpar) =
{
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min),
     subtree_sizes=vpar_subtree_sizes_vector(vpar),
     num_children=vpar_num_children_vector(vpar));
  for(p=1,#vpar,
     if(num_children[p] > 2, return(0));
     if(next_sibling[p]
        && subtree_sizes[p] == subtree_sizes[next_sibling[p]],
        return(0));

     \\ if(next_sibling[p] && first_child[p]
     \\    && subtree_sizes[next_sibling[p]] == subtree_sizes[first_child[p]],
     \\    return(0));
  );
  1;
}
{
  my(forest=0);
  for(n=1,20,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           if(any_samesize(vpar),
              count++));
     print1(count","));
  quit
}

 \\ ("https://oeis.org/draft/A335562"
 \\  "A335562 - OEIS"
 \\  (24294 31887)
 \\  (24295 2423 984192 987000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A335560"
 \\  "A335560 - OEIS"
 \\  (24294 32211)
 \\  (24295 2343 762296 261000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A334908"
 \\  "A334908 - OEIS"
 \\  (24294 45292)
 \\  (24295 1309 814219 544000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A334909"
 \\  "A334909 - OEIS"
 \\  (24294 33441)
 \\  (24295 1222 726432 598000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A335597"
 \\  "A335597 - OEIS"
 \\  (24294 48011)
 \\  (24295 491 897456 485000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A335078"
 \\  "A335078 - OEIS"
 \\  (24294 61860)
 \\  (24295 475 315776 841000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A014715"
 \\  "A014715 - OEIS"
 \\  (24294 62169)
 \\  (24295 448 384672 617000)
 \\  nil
 \\  nil)
 \\ ("https://oeis.org/draft/A335598"
 \\  "A335598 - OEIS"
 \\  (24294 64097)
 \\  (24295 130 279254 846000)
 \\  nil
 \\  nil)

\\------------------------------------------------------------------------------
\\ Preorder All Childless Same Depth
\\ Dyck words all peaks at same height
\\ Table of m-naccis
\\
\\   *
\\   |        attach new leg of k
\\   *        under existing of n vertices,
\\   | \      for 1 <= k <= height
\\   *  @
\\   |  |     ordered all childless same height
\\   *  @     k-Fibonacci

\\            bx + cx^2 + dx^3    mod 10 + 20x + 30x^2 + x^3 = M
\\  -  10d  20dx  30dx^2  dx^3
\\
\\ b,c,d -> -10d,  b - 20d, c - 30d

A000073
tribonacci(n) = A000073_samples[n+1];

\\ 0, 1, 1, 2, 3, 5, 8, 13, 21, 34, 55
\\ 0, 0, 1, 1, 2, 4, 7, 13, 24, 44, 81, 149, 274, 504
\\ 0, 0, 0, 1, 1, 2, 4
\\ vector(30,n,n--; tribonacci(n)-fibonacci(n))

\\ A007059
\\ A038495  duplicate
\\ A079500  with offset=0
\\ A048888  1-  is 0, 1, 2, 4, 7, 13, 23, 42, 76, 139, 255, 471, 873, 1627, 3044, 5718,
my(k=0); check_equal(x^(k+1)*(1-x^k)/(1-2*x+x^(k+1)), Pol(0));
my(k=1); check_equal(x^(k+1)*(1-x^k)/(1-2*x+x^(k+1)) == x^2/(1-x), 1);

\\ x^3 - x^2 - x - 1
P(n) = x^n-sum(i=0,n-1,x^i);
{
  check_equal(vector(20,n, x^n - (1-x^n)/(1-x)),
              vector(20,n, P(n)));
  check_equal(vector(20,n, ((1-x)*x^n - (1-x^n))/(1-x)),
              vector(20,n, P(n)));
  check_equal(vector(20,n, ((2-x)*x^n - 1)/(1-x)),
              vector(20,n, P(n)));
  check_equal(vector(20,n, ((x-2)*x^n + 1)/(x-1)),
              vector(20,n, P(n)));
  check_equal(vector(20,n, 1 - x*(1-x^n)/(1-x)),
              vector(20,n, polrecip(P(n))));
  check_equal(vector(20,n, (1 - 2*x + x^(n+1)) / (1-x)),
              vector(20,n, polrecip(P(n))));
  check_equal(vector(20,n, (1-x) / (1 - 2*x + x^(n+1))) ==
              vector(20,n, 1/polrecip(P(n))),  1);
}
\\gcd(P(5),P(4))
\\quit

Fn(n,m) =
{
  \\ used for sum(m=2,n+1,Fn(m,n));   so small n, small p
  \\ x^n power unchanged by p, then bigger m does some reducing

  \\ my(p=P(n));
  \\ \\ print((p));
  \\ Vec(lift(Mod(x,p)^m));
  \\ polcoeff(lift(Mod(x,p)^m), n-1);
  \\ \\ polcoeff(lift(Mod(x,p)^(m+1)), n-2);  \\ wrong

  \\ if(n<1,0,
  \\    my(p=polrecip(P(n)));
  \\    polcoeff(lift(Mod(x^-1,p)^(m-n+1)),0));

  if(m<n-1,return(0));
  my(v=vector(n),p=1); v[1]=1;
  for(i=n,m, p--||p=n; v[p]=vecsum(v); );
  \\ print(v);
  v[p];
}
Fn(3,7)
tribonacci(7)
Fn(2,7)
fibonacci(7)

\\ A048887
print("Fn table");
for(n=1,10,for(m=1,20,my(m=m+n-1); print1(Fn(n,m)" ")); print())
print("diags");
for(d=1,10,for(m=1,d, my(n=d-(m-1)); my(m=m+n-1); print1(Fn(n,m)," ")); print())


is_consecutive_1s_lt_m(m,n) =
{
  my(len=0);
  for(i=0,if(n,logint(n,2)),
     if(bittest(n,i),
        len++; if(len>=m,return(0)),
        len=0));
  1;
}
check_equal(is_consecutive_1s_lt_m(1,3), 0);
check_equal(is_consecutive_1s_lt_m(2,3), 0);
check_equal(is_consecutive_1s_lt_m(3,3), 1);

\\ A048887 T(m,n) compositions of n into parts all <= m
A048887_by_consecutive_1s(m,n) =
{
  \\ each 0-bit is a separator in the composition
  sum(i=0,2^(n-1)-1, is_consecutive_1s_lt_m(m,i));
}
print("A048887_by_consecutive_1s table");
for(n=1,8,for(m=1,10, print1(A048887_by_consecutive_1s(n,m)" ")); print())

{
  check_equal(test_antidiagonals_of_func(#A048887_samples,A048887_by_consecutive_1s),
              A048887_samples,
              "A048887_by_consecutive_1s() vs A048887_samples");
}

A048887_compact(m,n) =
{
  my(v=vector(m),p=1); v[1]=1; for(i=1,n, p--||p=m; v[p]=vecsum(v)); v[p];
}
A048887_by_polmod_inv(m,n) =
{
  \\ inverse ^-n makes small n worse than plain powering,
  \\ but that reduces as proportion of total time when big n
  polcoeff(lift(Mod('x,Pol(vector(m+1,i,if(i>m,-1,1))))^-n),0);
}
A048887_by_polmod(m,n) =
{
  \\ much faster than vecsum() for big n
  polcoeff(lift(Mod('x,Pol(vector(m+1,i,if(i==1,1,-1))))^(n+m-1)), m-1);
}
print("A048887_compact table");
for(n=1,8,for(m=1,15, print1(A048887_compact(n,m)" ")); print())
print("A048887_by_polmod table");
for(n=1,8,for(m=1,15, print1(A048887_by_polmod(n,m)" ")); print())
{
  check_equal(matrix(10,10,m,n, A048887_compact(m,n)),
              matrix(10,10,m,n, A048887_by_consecutive_1s(m,n)));

  check_equal(matrix(10,10,m,n, A048887_compact(m,n)),
              matrix(10,10,m,n, A048887_by_polmod(m,n)));

  check_equal(test_antidiagonals_of_func(#A048887_samples,A048887_compact),
              A048887_samples,
              "A048887_compact() vs A048887_samples");

  \\ ~/OEIS/b048887.txt
  my(g=OEIS_bfile_gf("A048887"));
  print("b048887.txt poldegree ",poldegree(g));
  \\ g=Pol(g + O(x^20));     \\ truncate for testing
  check_equal(x*Polrev(test_antidiagonals_of_func(poldegree(g),
                                           A048887_by_polmod)),
              g,
              "A048887_by_polmod() vs bA048887.txt");
}
{
  gettime();
  my(m=100,n=100);
  \\ A048887_compact(m,n); print("A048887_compact "gettime());
  my(x=A048887_by_polmod_inv(m,n)); print("A048887_by_polmod_inv "gettime());
  my(y=A048887_by_polmod(m,n));     print("A048887_by_polmod     "gettime());
  print(logint(x,10));
  x==y || error();
}

\\ A092921 row k>=1 column n>=0
\\ read by downwards anti-diagonals
\\ example table printed transposed,
\\         top row is column n=0
\\         k=2 Fibonaccis is second column

\\ Ralf Stephan in A092921
A092921(k,n) =
{
  k>=1 || error();
  \\ n>=0 || error();    \\ negatives used for the recursion
  if(n<2, if(n<1, 0, 1), sum(i=1, k, A092921(k,n-i)));
}

\\ this code by Max Hasler in A092921
A092921_by_matrix(m,n) =
{
  m>=1 || error();
  n>=0 || error();
  \\ m>=1 || error("m="m);
\\  if(m==0,'z,
     !!n*(matrix(m, m, i, j, j==i+1||i==m)^(n+m-2))[1,m]
\\    );
}
A092921_by_polmod(k,n) =
{
  k>=1 || error();
  n>=0 || error();
  if(n==0,0, polcoeff(lift(Mod('x,Pol(vector(k+1,i,if(i==1,1,-1))))^(n+k-2)), k-1));
}
{
  \\ Max Hasler 1 < n <= k+1
  for(k=1,10,
     for(n=2,k+1,
        check_equal(A092921(k,n), 2^(n-2),
                    "A092921 pow 2")));
}
print("A092921 table, n>=0");
for(k=1,8, print1("k="k": "); for(n=0,15, print1(A092921(k,n)" ")); print())
print("A092921_by_matrix table");
for(k=1,8, print1("k="k": "); for(n=0,15, print1(A092921_by_matrix(k,n)" ")); print())
print("A092921_by_polmod table");
for(k=1,8, print1("k="k": "); for(n=0,15, print1(A092921_by_polmod(k,n)" ")); print())

print("want");
matrix(5,8,k,n,n--; A092921_by_matrix(k,n))
print("got");
matrix(5,8,k,n,n--; A092921_by_polmod(k,n))
{
  \\ any k=1 don't want the coeff 1 which results
  my(k=1,n=0);
  my(p=Pol(vector(k+1,i,if(i==1,1,-1))));
  check_equal(p,'x-1);
  check_equal(n+k-2,-1);
  check_equal(Mod('x,p)^-1 == Mod(1,p), 1);
  check_equal(lift(Mod(1,p)), 1);
  check_equal(polcoeff(1,k-1), 1);
}
{ my(k=1,n=7);
  my(p=Pol(vector(k+1,i,if(i==1,1,-1))));
  check_equal(p,'x-1);
  check_equal(n+k-2, 6);
  check_equal(Mod('x,p)^6 == Mod(1,p), 1);
  check_equal(lift(Mod(1,p)), 1);
  check_equal(polcoeff(1,k-1), 1);
}
{
  \\ k>=1 n>=0
  check_equal(matrix(10,10,k,n,n--; A092921_by_matrix(k,n)),
              matrix(10,10,k,n,n--; A092921(k,n)),
              "A092921_by_matrix() vs A092921()");

  \\ k>=1 n>=0
  check_equal(test_antidiagonals_of_func(#A092921_samples,A092921_by_matrix,
                                    1,0,  \\ k>=1 n>=0
                                    1),   \\ taken downwards
              A092921_samples,
              "A092921_by_matrix() vs A092921_samples");

  check_equal(matrix(5,7,k,n,n--; A092921_by_polmod(k,n)),
              matrix(5,7,k,n,n--; A092921_by_matrix(k,n)),
              "A092921_by_polmod() vs A092921_by_matrix()");

  \\ ~/OEIS/b092921.txt     \\ OFFSET=0
  \\ 0 0
  \\ 1 1
  \\ 2 0
  \\ 3 1
  \\ 4 1
  \\ 5 0
  \\ 6 1
  \\ 7 1
  \\ 8 1
  \\ 9 0
  \\ 10 1
  \\ 11 2
  \\ 12 1
  \\ 13 1
  \\ 14 0
  my(g=OEIS_bfile_gf("A092921"));
  print("b092921.txt poldegree ",poldegree(g));
  \\ g=Pol(g + O(x^20));     \\ truncate for testing
  check_equal(Polrev(test_antidiagonals_of_func(poldegree(g)+1,
                                           \\ (r,c)->[r,c],
                                           A092921_by_polmod,
                                           1,0,  \\ k>=1 n>=0
                                           1)),  \\ taken downwards
              g,
              "A092921_by_polmod() vs bA092921.txt");
}

{
  gettime();
  my(m=300,n=30);
  my(y=A092921_by_polmod(m,n)); print("A092921_by_polmod "gettime());
  my(x=A092921_by_matrix(m,n)); print("A092921_by_matrix "gettime());
  print("size ",logint(x,10));
  x==y || error();
}

\\ for(m=1,10,print(Fn(3,m)))
\\ quit
{
  \\ Fibonaccis A000045
  check_equal(vector(#A000045_samples,m,m--; Fn(2,m)),
              A000045_samples,
              "Fibonaccis"); \\ OFFSET=0

  \\ Tribonaccis A000073   0, 0, 1, 1, 2, 4, 7, 13, 24, 44, 81, 149, 274, 504
  check_equal(vector(#A000073_samples,m,m--; Fn(3,m)),
              A000073_samples, \\ OFFSET=0
              "Tribonaccis");

  \\ Tetranaccis A000078
  check_equal(vector(#A000078_samples,m,m--; Fn(4,m)),
              A000078_samples, \\ OFFSET=0
              "Tetranaccis");

  check_equal(vector(100,n,n++; Fn(n,n)),
              vector(100,n,n++; 1),
              "Fn(n,n)");
  check_equal(vector(100,n,n++; Fn(n-1,n)),
              vector(100,n,n++; min(2,n-1)),
              "Fn(n-1,n)");
  check_equal(vector(100,n,n+=2; Fn(n-2,n)),
              vector(100,n,n+=2; if(n==2,0, n==3,1, n==4,3, 4)),
              "Fn(n-2,n)");
}
A048888_by_Fn(n) = sum(m=2,n+1,Fn(m,n));
A048888_by_polmods(n) =
{
  \\ sum(m=2,n+1, polcoeff(lift(Mod(x,P(m))^n), m-1));
  \\ \\ p==P(m) || error(p);
  \\ my(p=x-1);
  \\ min(n,2) + sum(m=2,n-1, p=p*x-1; polcoeff(lift(Mod(x,p)^n), m-1));
  \\
  \\ \\     1 - x - x^2
  \\ \\ 1 - x - x^2 - x^3
  \\ my(p=1-x);
  \\ min(1,n)
  \\ + sum(m=2,n,
  \\       p=1+x*(p-2);
  \\       p==polrecip(P(m)) || error(p" "polrecip(P(m)));
  \\       polcoeff(lift(Mod(x^-1,p)^(n-m+2)), m-1));
  \\
  \\ my(p=1-x);
  \\ sum(m=1,n,
  \\       p=1+x*(p-2);
  \\       p==polrecip(P(m+1)) || error(p" "polrecip(P(m+1)));
  \\       polcoeff(lift(Mod(x,p)^(m-n-1)), m));

  \\ my(q=1);
  \\ sum(m=1,n,
  \\       q=x*q+1;
  \\       1-x*q==polrecip(P(m+1)) || error(p" "polrecip(P(m+1)));
  \\       polcoeff(lift(Mod(q,1-x*q)^(n-m+1)), m));

  \\ q^2 div 1-x*q
  \\ quot -q/x
  \\ q^2 - (-q/x)*(1-x*q) == 1/x*q

  min(3,n) + sum(m=1,n-2, my(v=vector(m),p=m); v[m]=1;
                 for(i=m,n, p--||p=m; v[p]=vecsum(v)); v[p]);
}
{
  my(n=7);
  my(p=1-x);
  for(m=1,n,
     p=1+x*(p-2);
     p==polrecip(P(m+1)) || error(p" "polrecip(P(m+1)));
     my(e=lift(Mod(x,p)^(m-n-1)));
     print(m-n-1" "Vec(e),"  ",e));
  \\ quit;
}
print(Mod(x^-1,polrecip(P(7))));
gettime();
A048888_by_Fn(100); print("Fn "gettime());
A048888_by_polmods(100); print("polmods "gettime());
\\quit
{

  \\ A048888
  check_equal(vector(#A048888_samples,n,n--; A048888_by_polmods(n)),
              A048888_samples, \\ OFFSET=0
              "A048888_by_polmods()");
  for(n=1,100,
     my(want=A048888_by_Fn(n));
     check_equal(A048888_by_polmods(n),
                 want,
     Str("A048888_by_polmods() vs A048888_by_Fn() n="n" len "logint(want,10))));
}
{
  print("table");
  for(n=2,10,
     print1("n="n" is ");
     for(m=1,15,
        print1(Fn(n,m)," "));
     print());

  print("sum");
  for(m=1,15,
     print1(sum(n=2,m+1,Fn(n,m))," "));
  print();
  quit;
}

{
  my(limit=10,
     g=sum(k=1,limit+1, x^k * (1 - 2*x+x^2 + (1-x)*x^(k+1)) /(1 - 2*x + x^(k+1)) + O(x^limit)));
  print(Vec(g));
  g= 1/(1-x) + sum(k=1,limit+1, x^(k+1)*(1-x^k)/(1-2*x+x^(k+1) + O(x^limit)));
  print(Vec(g));
  print();
  for(k=1,7,
     g= x^(k+1) * (1-x^k)/(1 - 2*x + x^(k+1)) + O(x^20);
     print(Vec(g)));
  print(Vec(g));
  k=3;
  for(n=1,10,
     my(p=(1-x^k)*Mod(x, 1-2*x^k+x^(k+1))^n);
     print(polcoeff(lift(p),0)," ",p));
  quit;
}
{
  for(n=1,20,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,1))=='none,
           \\ print(vpar);
           my(h=vpar_height(vpar),
              depths=vpar_depths_vector(vpar),
              childless=vpar_childless_vertices(vpar));

           if(vecextract(depths,childless) == vector(#childless,i,h),
              count++));
     print1(count","));
  quit
}


\\------------------------------------------------------------------------------
\\ Binomial Tree Centres
\\ runs 2^n 0s, 2^n 1s,  second highest bit
\\ 0,1,0,1,1,0,0,1,1,1,1,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0
\\ A086694
\\ 1,0,1,0,0,1,1,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1
\\ A079944

\\ biggest centre
\\ 1,2,1,3,3,1,1,5,5,5,5,1,1,1,1,9,9,9,9,9,9,9,9,1,1,1,1,1,1,1

{
  for(n=1,30,
     my(vpar=vpar_make_binomial_tree(n),
        centres=vpar_centres(vpar));
     centres=vecsort(centres);
     print1(vecmax(centres),",");
     );
  print();
  for(n=1,30,
     print1(bittest(n,logint(n,2)-1),",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_unlabelled() compares

\\ n=10 [0, 1, 2, 1, 4, 0, 6, 7, 6, 9]
\\    1      6
\\    |\     | \          6 compares
\\    2  4   7  9         unique this many
\\    |  |   |  |
\\    3  5   8  10
\\
\\ n=14 [0, 1, 2, 3, 1, 5, 6, 0, 8, 9, 10, 8, 12, 13]
\\    1      8
\\    |\     | \          10 compares
\\    2  5   9  12        (total 5 this many)
\\    |  |   |  |
\\    3  6  10  13
\\    |  |   |  |
\\    4  7  11  14
\\
\\ n=15 max 11 1 [0, 1, 2, 1, 4, 0, 6, 7, 6, 9, 0, 11, 12, 11, 14]
\\    1      6      11
\\    | \    | \     | \       11 compares
\\    2  4   7  9   12  14     unique this many
\\    |  |   |  |    |   |
\\    3  5   8  10  13  15


test_is_unlabelled_by_depths(vpar) =
{
  vpar_is_preorder(vpar) || return(0);
  my(depths=vpar_depths_vector(vpar),
     subtree_sizes=vpar_subtree_sizes_vector(vpar),
     siblings_sets=vpar_siblings_sets(vpar));
  for(i=1,#siblings_sets,
     for(j=2,#siblings_sets[i],
        my(u=siblings_sets[i][j-1],
           v=siblings_sets[i][j]);
        lex(depths[u..u+subtree_sizes[u]-1],
            depths[v..v+subtree_sizes[v]-1]) >= 0 || return(0)));
  1;
}

vpar=[0, 1, 1, 3, 0, 5, 6]
vpar_to_unlabelled(vpar)
v=vpar_unlabelled_all(7);
v=Set(v);
print("setsearch ",setsearch(v,vpar));
print("func      ",vpar_is_unlabelled(vpar));
print("func      ",test_is_unlabelled_by_depths(vpar));
c=vpar_component_trees(vpar);
dd=apply(vpar_depths_vector,c)
vecsort(dd)
\\ quit

is_unlabelled_compares(vpar) =
{
  \\ print("is_unlabelled_compares() ",vpar);
  my(next_sibling=vpar_next_sibling_vector(vpar),
     v,
     count=0,
     result=1);
  for(u=1,#vpar-1,
     (v=next_sibling[u]) || next;
     \\ print(" siblings "u" "v);
     for(i=v+1,#vpar,
        \\ print("  at i="i" parent "vpar[i]);
        my(j=i-v+u);
        if(vpar[i]<v,  \\ i is outside v subtree, so same or shorter
           if(j<v,
              \\ print("  i="i" u longer, good");
             );
           break);
        if(j>=v,       \\ j is outside u subtree, so shorter than v
           result=0;
           \\ print("  i="i" u shorter");
           break(2));

        count++;
        if(vpar[j] > vpar[i]-v+u,
           \\ print("  different u bigger, good");
           break);
        if(vpar[j] < vpar[i]-v+u,
           result=0;
           \\ print("  u smaller, vi="i" parent "vpar[i]"("vpar[i]-v+u,")  uj="j" parent "vpar[j]);
           break(2))));
\\  result==test_is_unlabelled_by_depths(vpar) || error("oops got ",result," ",vpar," depths ",vpar_depths_vector(vpar));
  count;
}
is_unlabelled_compares(vpar) =
{
  \\ print("vpar_is_unlabelled() ",vpar);
  my(v=1,
     count=0);
  while(v<=#vpar,
        \\ print(" at v="v" parent q="vpar[v]", preorder check above preceding p="v-1);
        my(p=v-1, q=vpar[v]);
        count++;
        if(p==q,
           v++;
           ,
           my(u);
           until(p==q,
                 count++;
                 p || return(count);  \\ at root, vpar[v] not above v-1
                 u=p; p=vpar[p]);
           \\ print("  prev sibling u="u" common parent p="p" q="q);
           u==vpar_next_sibling_of_vertex(vpar,v,1) || error();

           p=v-u;    \\ offset v part back to u part
           q=v;      \\ first in the v part
           \\ print("  comparison loop start v=q="q", offset p="p);
           while(1,
                 u++;
                 v++;
                 \\ print("  compare u="u" v="v"  ",if(u>=q,"END",vpar[u])," cmp ",if(v>#vpar || vpar[v]<q,"END",vpar[v]-p));
                 my(c);
                 if(v>#vpar,break); \\ v end of vector, good
                 count++;
                 if(
                     vpar[v]<q                  \\ v end of subtree, good
                    || (c=lex(vpar[u],
                              vpar[v]-p)) > 0,    \\ cmp u>v, good
                    \\ print("  u same or bigger, good");
                    break);
                 if(c<0,
                    \\ print("  different, v bigger, bad");
                    return(count)));
          ));
  count;
}
\\ is_unlabelled_compares([0,0,0,0,0,0,0])
\\ quit

{
  \\ with preorder, with advance past same parts
  \\ n=   5           10      13 14
  \\ cmps 7,10,13,16,19,22,25,28,31
  \\ reps 1                             num vpars of this many cmps


  \\ without preorder, and without advance past same parts
  \\ n=   5           10      13 14
  \\ reps 6,5,35,19,9, 1,3,6, 6,  5     num vpars of this many cmps
  \\ cmps 1,2, 2, 3,4, 6,6,7, 8  10
  for(n=1,20,
     my(vpar=vpar_preorder_first(n),
        m=0,
        m_reps,
        m_vpar);
     until((vpar=vpar_preorder_next(vpar,1))=='none,
           \\ print(vpar);
           my(c=is_unlabelled_compares(vpar));
           if(c>m, m=c;m_reps=1;m_vpar=vpar,
              c==m, m_reps++));
     print("n="n" max "m" "m_reps" "m_vpar));
  quit
}

\\ vpar_is_unlabelled([0,1,2,3])
vpar_is_unlabelled([2,0])
quit

\\------------------------------------------------------------------------------
\\ vpar_free_next forest

vpar_free_last(5)
vpar_free_first(5)
\\ vpar_free_next_ff([0,1,1,1,1,1, 0,0,0])
vpar_free_next_ff([0,1,2,1,4, 0,0,0,0,0,0,0,0,0,0,0,0,0,0],  1)
quit


\\------------------------------------------------------------------------------
\\ binomial tree permute siblings
\\
\\ 1,1,2,12,576,1658880,16511297126400,
\\ A052129 a(0)=1; thereafter a(n) = n*a(n-1)^2.
\\ vpar_num_ordered_automorphisms -- but that func probably wrong just yet

\\ (2^n)! / ways
\\ not in OEIS: 1,2,12,3360,36324288000,158619572804358079045632000000
\\ vpar_num_ordered_isomorphic -- but that func probably wrong just yet

{
  for(k=0,6,
     my(vpar=vpar_make_binomial_tree(2^k),
        sets=vpar_siblings_sets(vpar));
     \\ print1(prod(i=1,#sets, (#sets[i])!),",");
     \\ print1((#vpar)! /  prod(i=1,#sets, (#sets[i])!),",");
     \\ print1(vpar_num_ordered_automorphisms(vpar),",");
     print1(vpar_num_ordered_isomorphic(vpar),",");
  );
  quit
}


\\------------------------------------------------------------------------------
\\ binomial tree relabel reverse
{
  for(k=0,6,
     my(vpar=vpar_make_binomial_tree(2^k),
        rev=vpar_relabel_reverse(vpar));
     print(vpar_to_balanced_binary(rev)));
  quit
}


\\------------------------------------------------------------------------------
\\ binomial postorder
\\
\\ locations of odd, 1-based
\\ 1,2,4,5,8,9,11,12,16,17,19,20,23,24,26,27
\\ A101925
\\
\\ locations of odd, 0-based
\\ 0,1,3,4,7,8,10,11,15,16,18,19,22,23,25,26
\\ A005187   2n - count1s(n)
\\
\\ locations of even 1-based
\\ 3,6,7,10,13,14,15,18,21,22,25,28,29,30,31
\\ A045412  already in the sequence ...
\\
\\ locations of even, 0-based
\\ 2,5,6,9,12,13,14,17,20,21,24,27,28,29,30
\\ A055938   not a 2n-count1s(n)
\\ maybe A080578(n+1) - 2
\\ maybe A080468(n+1) + 2*n
\\
\\ inverse
\\ A108918,A118319

len=32
vpar=vpar_make_binomial_tree(len);
by_vpar=apply(n->n-1, Vec(vpar_seq_postorder(vpar),len-1))

inv=Vec(Vecsmall(by_vpar)^-1)

for(i=1,#by_vpar,if(by_vpar[i]%2==0, print1(i,",")));
quit




\\------------------------------------------------------------------------------
\\ A331951 binary trees with superleaf

\\     *
\\    / \
\\   *
\\  / \

A331951_values = {[
0, 0, 0, 1, 2, 6, 20, 69, 246, 894, 3292, 12242, 45868,
              172884, 654792, 2489981, 9500774, 36356214, 139471404,
              536217814, 2065543012, 7970227084, 30801517624,
              119198827218, 461863265660, 1791626278060, 6957151415832,
              27041349974436, 105197526148312, 409575623758440,
              1595836895778320
]};

A331951(n) =
{
  my(ret=0);
  for(i=1,oo,
     my(c=n-7*i);
     if(c<0,break);
     ret += (-1)^(i-1) * vpar_INTERNAL_Catalan_number(c) * binomial(c+1,i));
  ret;
}
vector(16,n, A331951(n))
quit




\\------------------------------------------------------------------------------
\\ vpar_is_oddeven() of functional digraphs

{
  \\ functional digraphs, vpar any 0 to n
  \\              1  2  3   4   5    6     7     8
  \\ not in OEIS: 1, 4, 12, 81, 432, 4096, 32000,
  \\
  \\ n=1  1     to 0
  \\ n=3  1,3 to 0,2     ceil to ceil
  \\      2   to 0,1,3   floor to ceil
  \\ n=5  1,3,5 to 0,2,4     ceil to ceil
  \\      2,4   to 0,1,3,5   floor to ceil
  \\
  \\ n=2  1  to 0,2     /2 to +1
  \\      2  to 0,1     /2 to +1
  \\ n=4  1,3  to 0,2,4     /2 to +1
  \\      2,4  to 0,1,3     /2 to +1
  \\
  \\ GP-Test  2^2 * 3^1 == 12
  \\
  \\ without /2^n
  \\ not in OEIS: 2, 16, 96, 1296, 13824, 262144, 4096000

  print(vector(10,n,
               if(n%2==0, (n+2)^n,
                          (n+1)^ceil(n/2) * (n+3)^floor(n/2))
               / 1^n));
  print(vector(10,n,
               if(n%2==0,
                  ((n+2)/2)^n,
                  ((n+1)/2)^ceil(n/2) * ((n+3)/2)^floor(n/2))));
  print1(" ");

  my(forest=1);
  for(n=1,9,
     my(count=0);
     forvec(vpar=vector(n,i,[0,n]),
           \\ if(vpar_is_oddeven(vpar),
           \\    print(vpar,"  ",vpar_is_oddeven(vpar)));
           count += vpar_is_oddeven(vpar));
     print1(count,", "));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_oddeven()

\\ oddeven trees
\\              1  2  3   4   5    6     7     8
\\ not in OEIS: 1, 2, 3, 16, 60, 486, 3024
\\
\\ oddeven trees, odd root (is half)
\\              1  2  3  4   5    6     7      8
\\ not in OEIS: 1, 1, 2, 8, 36, 243, 1728, 16384
\\ A340234 maybe ?
\\
\\ oddeven trees, root=1
\\              1  2  3  4   5   6    7     8
\\ not in OEIS: 1, 1, 1, 4, 12, 81, 432, 4096
\\
\\ oddeven forests
\\              1  2  3   4   5    6     7     8
\\ not in OEIS: 1, 3, 8, 45, 216, 1792, 12800, 140625,


check_equal(vpar_is_oddeven([]), 1);
check_equal(vpar_is_oddeven([0,1]), 1);
check_equal(bittest(2+1,0), 1);
check_equal(vpar_is_oddeven([2,0]), 1);
{
  my(forest=1);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_oddeven(vpar));
     print1(count,", "));
  quit;
}


\\------------------------------------------------------------------------------
\\ Bileaves
\\ all leaves in same bipartite side
\\ distance to leaves all even or all odd

\\ 5,6,13,20,41,70,144,266,545
\\ A304867 hypertrees weight n
\\ connected antichain of sets
\\ hyperedge = set of vertices
\\ hypertree acyclic, so only cycles are within one hyperedge
\\ weight = sum size each hyperedge
\\
\\ 5 of  weight=6  free n=7
\\ correspondence of weight n=6 hypertrees to bipartition free trees 7 vertices
\\
\\   {1,2,3,4,5,6}         3   2                          Eroot 1   Codd
\\                          \ /                           Oroot 1
\\                         4-*-1     star 7
\\                          / \
\\                         5   6
\\
\\   {1,2,3},              2-\       /-4                  Eroot 2   Ceven
\\   {1,4,5}                  *--1--*                     Oroot 1
\\                         3-/       \-5
\\
\\   {1,2}                 3-\                            Eroot 3   Codd
\\   {1,3,4,5}             4--*--1--*--2                  Oroot 2
\\                         5-/
\\
\\   {1,2},                 /-*--2                        Eroot 2   Ceven
\\   {1,3},                1--*--3                        Oroot 1
\\   {1,4}                  \-*--4
\\
\\   {1,2}, {2,4}          3--*--1--*--2--*--4   path 7   Eroot 2   Codd
\\   {1,3}                          C                     Oroot 2
\\
\\                                                total Eroot 1+2+3+2+2 == 10
\\                                                total Oroot 1+1+2+1+2 == 7

\\ weight=7
\\   {1,2,3,4,5,6,7}       4 3 2
\\                          \|/
\\                         5-*-1   star 8
\\                          / \
\\                         6   7
\\
\\   {1,2,3}               2-\       /-4
\\   {1,4,5,6}                *--1--*--5
\\                         3-/       \-6
\\
\\
\\   {1,2}                 3--\
\\   {1,3,4,5,6}           4--*--1--*--2
\\                         5--|
\\                         6--/
\\
\\                               C
\\   {1,2}                 4--*--1--*--2
\\   {1,3}                    |  |
\\   {1,4,5}                  5  *
\\                               |
\\                               3
\\
\\                               C  C
\\   {1,2}  {2,4,5}        4--*--2--*--1--*--3
\\   {1,3}                    |
\\                            5
\\                                  C
\\   {1,2,5} {2,4}         3--*--1--*--2--*--4
\\   {1,3}                          |
\\                                  5
\\
\\

\\ Andrew Howroyd in A304867
EulerT(v)=
{
  Vec(exp(x*Ser(dirmul(v, vector(#v, n, 1/n))))-1, -#v);
}
c(n) =
{
  my(v=[1]);
  for(i=1, ceil(n/2),
     v=concat(1, EulerT(concat(0, EulerT(v)))));
  v;
}
print(c(40)); quit;
\\ A337089 = 3, 4, 10, 17, 38, 73, 158, 324, 700, 1483, 3224, 6979, 15300

seq(n) =
{
  my(u=c(n));
  Vec(x
      + x*Ser(EulerT(u))*(1-x*Ser(u))
      + (1-x)*Ser(u)
      + O(x*x^n));
}
print(seq(18));

{
  print1(" ");
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           vpar_is_bileaves(vpar) || next;
           if(n==7,print(vpar));
           count++);
     print1(count", ");
  );
  quit;
}

\\ Return a vector of two sets [ set1, set2 ] which are the bipartite
\\ decomposition on the vertices in vpar.
\\ Vertex 1, or the given vertex v, is in set1.
bipartition(vpar,v=1) =
{
  my(dist=vpar_path_lengths_vector(vpar,v));
  [ select(v->dist[v]%2==0, [1..#vpar]),
    select(v->dist[v]%2==1, [1..#vpar]) ];
}


\\------------------------------------------------------------------------------
\\ Binomial Tree Num Maximum Indsets
\\
\\ num maximum indsets
\\            n=1
\\ not in OEIS: 1,2,1,3,2,5,3,8,6,14,8,22,16,38,22,60,
\\
\\         k=1     n=2^k vertices
\\ A347289 = 2,3,8,60,3456,11612160,132090377011200,17175244766164688547348480000,

num_maximum_indsets(k) =
{
  if(k<2, return(k+1));
  \\ my(ret=2);
  \\ for(i=2,k,
  \\    ret = ret^2 * (i^2-1)/i^2);
  \\ ret;

  \\ 2^2 * (2^2-1)/2^2;
  \\ 2^4 * (2^2-1)^2/(2^2)^2 * (3^2-1)/3^2;

  2^(2^(k-1)) * prod(i=2,k, ((i^2-1)/i^2)^(2^(k-i)));
}

{
  for(k=0,8,
     my(vpar=vpar_make_binomial_tree(2^k),
        ic=vpar_indnum_and_count(vpar));
     \\ print1(ic[2]",");
     print("k="k"  "ic[2]" "num_maximum_indsets(k));
     );
  quit;
}
{
  \\ ratios
  for(k=1,16,
     my(vpar=vpar_make_binomial_tree(2^k),
        ic=vpar_indnum_and_count(vpar),
        vpar2=vpar_make_binomial_tree(2^(k-1)),
        ic2=vpar_indnum_and_count(vpar2));
     \\ print(ic[2]/ic2[2]^2);
     \\ print(denominator(ic[2]/ic2[2]^2));
     );
  quit;
}
{
  for(n=1,16,
     my(vpar=vpar_make_binomial_tree(n),
        ic=vpar_indnum_and_count(vpar));
     print1(ic[2]",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Binomial Tree Perfect Matching

{
  for(n=1,16,
     my(vpar=vpar_make_binomial_tree(n));
     \\ print(Vec(vpar_seq_preorder(vpar)));
     \\ print(vpar_is_equimatchable(vpar)" "vpar);
     print(vpar_has_perfect_matching(vpar)" "vpar);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Adjacency Poly Roots vs Max Degree
\\
\\ *--*--*--* path-4
\\ x^2 + 3*x + 1
\\ -2.618, -0.3820
\\ 2*sqrt(2-1) == 2
\\ adjpoly root <= 2*sqrt(d-1)
\\ matchpoly root = 1/sqrt(a)
\\ vpar = vpar_make_path(4)
\\ vpar_adjacency_poly(vpar)
\\ vpar_matchpoly(vpar)
\\ polrootsreal(vpar_adjacency_poly(vpar))
\\ polrootsreal(vpar_matchpoly(vpar))
\\ apply(a->1/sqrt(a),polrootsreal(vpar_adjacency_poly(vpar)))
\\ apply(a->1/sqrt(a),polrootsreal(vpar_matchpoly(vpar)))
\\ apply(a->1/sqr(a),polrootsreal(vpar_adjacency_poly(vpar)))
\\ apply(a->1/sqr(a),polrootsreal(vpar_matchpoly(vpar)))


{
  default(realprecision,4);
  for(n=2,5,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           my(\\ p=vpar_matchpoly(vpar),
              p=vpar_adjacency_poly(vpar),
              r=polrootsreal(p),
              maxroot=vecmax(apply(abs,r)),
              degrees=vpar_degrees_vector(vpar),
              d=vecmax(degrees),
              s=2*sqrt(d-1));
           print(d" "s" "maxroot" "vpar" "p" "r);
          );
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ num free tree perfect or near-perfect matchings
\\ 1,1,1,1,2,2,6,5,20,15,76,49,313,180,1361,701,
\\ not in OEIS: 6,5,20,15,76,49,313,180,1361,701,
\\
\\ num free trees perfect matching
\\ 1,1,1,2,5,15,49,180,701,
\\ A005751
\\
\\ num free trees near perfect matching
\\            n=1 3 5 7  9
\\ not in OEIS: 1,1,2,6,20,76,313,1361,6161,

{
   forstep(n=1,20,2,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           if(vpar_has_perfect_matching(vpar,n%2), count++));
     print1(count",");
  );
  quit;
}
{
  \\ total orbits in free trees of n vertices with perfect matchings
  \\ n=2    *---*
  \\  n=0 2 4 6  8
  \\    0,1,2,7,26,107,458,2058,9498,44947,
  \\ cf 1,1,2,6,20, 76,313,1361,6161, free trees near perfect matching
  \\    adding a vertex to different orbits sometimes gives same n+1 tree
  check_equal(vpar_free_automorphisms([0,1]),
              [Vecsmall([1,2]), Vecsmall([2,1])]);
  forstep(n=0,20,2,
     my(vpar=vpar_free_first(n),
        total=0);
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           vpar_has_perfect_matching(vpar) || next;
           total += vpar_num_free_orbits(vpar);
          );
     print1(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_has_perfect_matching()

{
  \\ perfect matching among labelled rooted
  \\ trees, is n*root1
  \\ not in OEIS: 2, 48, 4320, 860160
  \\ vector(6,h,my(n=2*h); h!*binomial(n,h)*n^(h-1))
  \\
  \\ forests
  \\ 2, 60, 5880,
  \\ 2* A036363 = line-labelled
  \\ vector(6,n,n+=2; (2*n-3)!/(2*(n-2)!) * (2*n-3)^(n-4) )
  \\ vector(6,n,n+=2; 1/2*(2*n-3)!/(n-2)! * (2*n-3)^(n-4) )
  \\ vector(6,n,n+=2; 1/2*(n-1)!*binomial(2*n-3, n-2) * (2*n-3)^(n-4) )
  \\ vector(6,n,n++; 1/2*n!*binomial(2*n-1, n-1) * (2*n-1)^(n-3) )*2
  \\ F(n) = if(n%2==1,0, h!*binomial(n-1,h-1) * (n-1)^(h-3))
  \\ forest (n+1)^(n-1)
  \\ but h of them lower of pair so (n+1)^(h-1)
  \\ binomial(n,h) for the top each pair, h! permute the lower each
  \\ vector(6,h,my(n=2*h); h!*binomial(n,h)*(n+1)^(h-1))
  \\
  \\ root=1
  \\ not in OEIS: 1, 12, 720, 107520,
  \\ 107520 == 448/3*720
  \\
  \\ pair underneath existing perfect matching  -- WRONG
  \\ F(n) = if(n%2==1,0, n<=2,1, sum(t=2,n, if(t%2==0, (t/2)!*(n-t)*F(n-t))))
  \\ vector(6,n,F(2*n))

  my(forest=-1);  \\ trees or forests
  forstep(n=2,8,2,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count += vpar_has_perfect_matching(vpar));
     print1(count", ");
     );
  quit;
}
{
  \\ perfect matching among rooted unlabelled, is all trees n/2 by corona singletons
  \\ 1, 2, 7, 26, 107, 458, 2058, 9498,
  \\ A000151, rooted labelled of n/2

  forstep(n=2,20,2,
     my(count=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           count += vpar_has_perfect_matching(vpar));
     print1(count", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ 0,1 matrices real symmetric polys
\\
\\ Distinct adjacency spectra of graphs of n vertices with self-loops allowed.
\\
\\ distinct, with self-loops
\\ 1,2,6,20,87,507,4607,
\\ not in OEIS: 6,20,87,507,4607
\\
\\ duplicates
\\ 0,0,2,16,83,503,

\\ extras due to self-loops
\\ [1,2,6,20,87,507,4607]-[1,1,2,4,11,33,151]
\\ not in OEIS: 0, 1, 4, 16, 76, 474, 4456
\\
\\ ------
\\ distinct, no self-loops
\\ 1,1,2,4,11,33,151,
\\ A082104 graphs n vertex distinct charpoly
\\
\\ duplicates, no self-loops
\\ 0,0,0,2,9,31,149,
\\ A089475 starts 1,9,31 instead

\\ up to isomorphism:
\\ A006608 graphs n vertices not determined by spectrum
\\ A178925 graphs n vertices determined by spectrum

\\ my(n=6); n*(n+1)/2 == 21
{
  my(loops=0);
  for(n=0,7,
     my(count=0,
        t=test_triangular(n-1+loops),
        distinct=Map());
     forvec(vec=vector(t,i,[0,1]),
           my(m=matrix(n,n),upto=0);
           for(i=1,n,
              for(j=i+!loops, n,
                 m[i,j] = m[j,i] = vec[upto++]));
           upto==t || error("n=",n," upto=",upto," t=",t);
           my(p=charpoly(m));
           polcoeff(p,n) == 1 || error();

           my(c=0);
           mapisdefined(distinct,p,&c);
           mapput(distinct,p,c+1));
     my(v=Mat(distinct)[,2],
        count_distinct=#v);
     #distinct == count_distinct || error();

     \\ print1(count_distinct",");

     \\ my(count_unique=sum(i=1,#v,v[i]==1));
     \\ print1(count_unique,",");

     my(count_not_unique=sum(i=1,#v,v[i]>=2));
     print1(count_not_unique,",");

     \\ print(#distinct);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ x^3 + x^2 - 2*x - 1
{
  for(n=3,6,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           print(factor(vpar_adjacency_poly(vpar)));
          );
     print();
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Bileaves vs Maximum Matchings
\\
\\ Every maximum matching has all the odd (type B) vertices since work
\\ inwards from a vertex with >=1 leaves and 1 non-leaf neighbour.

\\ putting v in the first half
\\ what to do in a forest?
\\ maybe first half each component smallest vertex
bipartition(vpar,v=1) =
{
  my(dist=vpar_path_lengths_vector(vpar,v));
  [ select(v->dist[v]%2==0, [1..#vpar]),
    select(v->dist[v]%2==1, [1..#vpar]) ];
}
bileaves(vpar) = bipartition(vpar, vpar_leaves(vpar)[1]);

{
  for(n=4,8,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           vpar_is_bileaves(vpar) || next;
           my(b=bileaves(vpar),
              odd=b[2],
              matchings=vpar_matchnumsets(vpar));
           print("vpar "vpar"  "b"  "#b[2]);
           for(i=1,#matchings,
              my(set=matchings[i]);
              print(vpar" "set" "#set))));
  quit;
}


\\------------------------------------------------------------------------------
\\ Rooted Unlabelled Childless at Even Depths
\\
\\ trees childless even depths = forests childless odd depths
\\ 1,0,1,1,3,4,10,17,38,73,158,324,700,1483,3224,6979,     n>=1 trees
\\ 0,1,1,3,4,10,17,38,73,158,324,700,          n>=1 forests
\\ not in OEIS: 3,4,10,17,38,73,158,324,700,
\\
\\ trees childless odd depths = forests childless even depths
\\ 0,1,1,2,3,6,11,22,43,89,183,384,812,1738,3742,8125,     n>=1 trees
\\ A337090 = 3,6,11,22,43,89,183,384,812
\\
\\ forests even depths
\\ 1,1,1,2,3,6,11,22,43,89,183,384,812,1738,3742,8125,

\\ trees even -> forests even  by collecting together
vpar_INTERNAL_Euler_transform([1,0,1,1,3,4,10,17,38,73,158,324,700])
\\ [1, 1, 2, 3, 6, 11, 22, 43, 89, 183, 384, 812, 1738]
\\
\\ trees odd -> forests odd
vpar_INTERNAL_Euler_transform([0,1,1,2,3,6,11,22,43,89,183,384,812])
\\ 0, 1, 1, 3, 4, 10, 17, 38, 73, 158, 324, 700, 1483
\\
\\ Mutual Euler transforms.

\\ Teven + Todd = childless depths all same parity
\\ 1, 1, 2, 3, 6, 10, 21, 39, 81, 162, 341, 708, 1512, 3221, 6966, 15104
\\ not in OEIS: 3, 6, 10, 21, 39, 81, 162, 341, 708, 1512, 3221, 6966, 15104
\\ vpar_INTERNAL_Euler_transform(Teven+Todd)
\\
\\ forests childless depths all same parity
\\ 1, 2, 4, 8, 16, 33, 68, 142, 300, 638, 1368, 2955, 6421, 14032, 30825, 68021
\\ not in OEIS: 4,8,16,33,68,142,300,638,1368,2955,6421,14032,30825,68021

\\ odd depth children are not even leaf-to-leaf if root is leaf
\\    *  O
\\    |
\\    *  E
\\    |
\\    *  O
\\    |
\\    *  E
\\  / | \
\\ *  *  *   odd leaves

\\ g(x) = 1 + T(x) - T^2(x)/2
Teven = [1,0,1,1,3,4,10,17,38,73,158,324,700,1483,3224,6979];
Todd  = [0,1,1,2,3,6,11,22,43,89,183,384,812,1738,3742,8125];
Teven+Todd
T = x*Polrev(T) + O(x^14)
Vec(T)
U = 1 + T - T^2/2
Vec(U)

ToddNon = vector(#Todd,n, Todd[n] - if(n>=2,Teven[n-1]))
\\ vector(13,n, Teven[n] - sum(i=1,floor(n/2), Todd[i]*Teven[n-i])) + \

\\ Even root, less its unbalanced.
\\ Odd root, less its unbalanced, including remove root is a leaf.
\\ Bicentroidal even one side odd the other.
\\ n=1  2  3  4  5  6  7
\\   1, 1, 1, 2, 3, 5, 9, 16, 30, 58, 114, 228, 468
vector(16,n, Teven[n]   - sum(i=1,floor(n/2), Teven[i]*Todd[n-i]) ) + \
vector(16,n, ToddNon[n] - sum(i=1,floor(n/2), Todd[i]*Teven[n-i]) ) + \
vector(16,n, if(n%2==0, Teven[n/2]*Todd[n/2]))

vector(16,n, Teven[n]+Todd[n] - sum(i=1,floor(n/2), Teven[i]*Todd[n-i] \
                                        + if(i==1,1,Todd[i])*Teven[n-i])) + \
vector(16,n, if(n%2==0, Teven[n/2]*Todd[n/2]))

vector(13,n, vpar_INTERNAL_rooted_vec_to_free(Todd,n,0)) + \
vector(13,n, vpar_INTERNAL_rooted_vec_to_free(Teven,n,0))

\\       unicentroial                bicentroidal, same size halves
\\    centE       centO            centE -- centO
\\     / \          |  \             |        | \
\\    o   o         E   E            o        E  E
\\    |   |                          |
\\    E   E                          E

\\ Return 1 if all childless vertices in vpar are at depth = want mod 2.
\\ "want" must be 0 or 1.
is_childless_depths_parity(vpar,want) =
{
  my(depths=vpar_depths_vector(vpar),
     childless=vpar_childless_vertices(vpar));
  for(i=1,#childless,
     if(depths[childless[i]]%2 != want, return(0)));
  1;
}
{
  my(forest=0);
  for(n=1,18,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(got=is_childless_depths_parity(vpar,0));
           \\ if(got && n==7,
           \\    print(vpar);
           \\    vpar_view(vpar));
           count+=got);
     print1(count",");
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ Free Trees Adjacent Eccentricity Opposite Parity
\\
\\ 1,0,1,1,2,3,7,12,27,55,127,284,682,1618,3979,
\\ A000676 unicentrals

is_eccentricity_opposites(vpar) =
{
  my(ecc=vpar_eccentricities_vector(vpar));
  for(v=1,#vpar,
     my(p=vpar[v]);
     if(p && ecc[v]%2==ecc[p]%2, return(0)));
  1;
}
{
  print1(" ");
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           my(got=is_eccentricity_opposites(vpar),
              uni=(#vpar_centres(vpar)==1));
           uni==got || error();
           \\ if(n==7,print(vpar));
           count+=got);
     print1(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Prufer Code One Change

free_edge_set(vpar) =
{
  my(vec=vector(vpar_num_edges(vpar)),upto=0);
  for(v=1,#vpar,
     if(vpar[v],
        vec[upto++]=vecsort([v,vpar[v]])));
  upto==#vec || error();
  Set(vec);
}
check_equal(free_edge_set([0,1,1]), [ [1,2],[1,3] ]);
{
  \\ Root=1 trees with Prufer codes differing in one element.
  \\ How many of trees (one way around) with edge set differing by one element,
  \\ not in OEIS: 3, 40, 510, 7500, 129521,
  \\
  \\ Craig Lennon, "On the Locality of the Prufer Code"

  for(n=3,10,
     my(total=0,
        diff1=0);
     forvec(vec=vpar_forvec_from(n,-1),   \\ root=1
           my(vpar=vpar_from_upascend(vec),
              set=free_edge_set(vpar));
           for(i=1,#vec-2,
              for(j=vec[i]+1,n,
                 total++;
                 my(vec2=vec); vec2[i]=j;
                 my(vpar2=vpar_from_upascend(vec2),
                    set2=free_edge_set(vpar2),
                    diffs=n-1-#setintersect(set,set2));
                                        \\ sum(i=1,n, vpar[i]!=vpar2[i]));
                 \\ print(vec" "vpar" "set"     "vec2" "vpar2" "set2);
                 vpar_validate(vpar2);
                 vpar_roots(vpar2)==[1] || error();
                 if(diffs==1,
                    diff1++))));
     print("n="n" total "total" diff1 = "diff1"  "diff1/total*1.0);
     total == 1/2*(n-2)*(n-1)*n^(n-2)  || error();
  );
  quit;
}
\\ vector(10,n, (n-2)*(n-1)*n^(n-2))
\\ not in OEIS: 6, 96, 1500, 25920, 504210, 11010048, 267846264, 7200000000
\\ vector(10,n, 1/2*(n-2)*(n-1)*n^(n-2))
\\ not in OEIS: 3, 48, 750, 12960, 252105, 5505024, 133923132, 3600000000


\\------------------------------------------------------------------------------
\\ mean number of singletons in an unlabelled forest

mean(n) =
{
  if(n<2, return(1));
  my(vec=vpar_count_unlabelled_vector(n,1)); \\ forests
  (vecsum(vec[1..n-1])+1) / vec[n];
}

mean(10)*1.0
mean(100)*1.0
mean(1000)*1.0
mean(5000)*1.0

\\ A000081 Polya in Knuth
\\ c * d^n * n^(-3/2), where c = A187770 = 0.439924... and
\\ d = A051491 = 2.955765...  [Polya; Knuth, section 7.2.1.6].

d=2.95576528565199497471481752412319458837549230466359659535
S=1/(d-1)
A346916 = 0.51130879934124
sum(i=0,5,7^i) == (7^6-1)/(7-1)

\\ Knuth 7.6.1.2 exercise 88
2+3/(d-1)
2+3*S
\\ A346915 = 3.5339263980237

\\ A087803
\\ vector(10,n, vecsum(vpar_count_unlabelled_vector(n,1)) + 1)

{
  my(forest=1);
  for(n=3,20,
     my(vpar=vpar_unlabelled_first(n),
        total=0,
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count++;
           total += vpar_num_singletons(vpar));
     my(vec=vpar_count_unlabelled_vector(n,1));
     print("n="n" count="count"  "total"  "vecsum(vec[1..n-1])+1"  "vec);
     total == vecsum(vec[1..n-1])+1 || error();
     count == vec[n] || error();
     print("n="n" "total/count*1.0"   "mean(n)*1.0));
}

quit


\\-----------------------------------------------------------------------------


from_adjacency_matrix(m) =
{
  \\ vpar[p+1..n] are decided.
  \\ The loops are over parent p and child v, with p < v.
  \\ Edges are in m both ways so it's enough to look at the p<v triangular
  \\ half of m as m[p,v].
  \\ Edge p to v gets v underneath p simply by setting vpar[v]=p.
  \\ If vpar[v]!=0 then it already has another such parent.  In that case
  \\ re-root so v is the root of that existing component, then set vpar[v].
  \\ The rerooting is like vpar_reroot(), but starting p=p which will store
  \\ into vpar[v] which is the desired new parent.
  \\ Looping p downwards from #vpar to 1 leaves root as minimum vertex each
  \\ component.
  \\ See test.gp test_from_adjacency_matrix_by_rows() on not rewriting.

  my(n=matsize(m)[1],
     vpar=vector(n));
  forstep(p=n,1,-1,
     forstep(v=n,p,-1,
        if(m[p,v]==1,
           print("edge p="p" v="v"  into vpar="vpar);
           my(p=p, v=v);
           until(!v,
                 print("at v="v" for p="p);
                 my(new_v=vpar[v]); vpar[v]=p; p=v; v=new_v))));
  vpar;
}
vpar=test_make_cycle(5)
vpar=vpar_relabel_reverse(vpar)
m=vpar_adjacency_matrix(vpar)
from_adjacency_matrix(m)
quit


\\-----------------------------------------------------------------------------
\\ Free Tree Lattice Polynomials

{
  for(n=0,11,
     print1("[");
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           print(abs(Vec(vpar_Laplacian_poly(vpar)))","));
     print("],");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_diameter_num_endpoints() == vpar_num_leaves()
\\
\\ All diameter endpoints are leaves, so have <= num_leaves.
\\ Sometimes all leaves are ends.
\\ Equivalent to all leaves having same eccentricity.

{
  \\ free periphery=leaves vs unlabelled childless=height
  \\
  \\ rooted need >=2 paths root to height to attain the diameter,
  \\ too root num children >= 2

  for(h=0,14,
     my(d=2*h);
     for(n=1,h,
        my(vpar=vpar_free_first(n),
           l_free=List([]));
        until((vpar=vpar_free_next(vpar))=='none,    \\ trees
              if(vpar_diameter_num_endpoints(vpar) == vpar_num_leaves(vpar)
                 && vpar_diameter(vpar)==d,
                 listput(l_free,vpar)));

        vpar=vpar_unlabelled_first(n);
        my(l_unlabelled=List([]));
        until((vpar=vpar_unlabelled_next(vpar))=='none,    \\ trees
              if(vpar_height_num_vertices(vpar) == vpar_num_childless(vpar)
                 && vpar_height(vpar)==h
                 && vpar_num_children_of_vertex(vpar,vpar_root(vpar))>=2
                 ,
                 listput(l_unlabelled,vpar)));

        print("h="h" d="d"  n="n);
        \\ print("  free  "l_free);
        \\ print("  un    "l_unlabelled);
        check_equal(#l_unlabelled, #l_free,
                    Str("h="h" d="d"  n="n));
        ));
  quit;
}

{
  \\ free periphery==leaves
  \\ counted by diameter
  \\ not in OEIS: 1, 4, 7, 9, 5, 3, 1, 1
  \\
  \\ diameter=4 leaves=periphery
  \\ 1,1,3,3,6,7,11,13,20,23,33
  \\ A083751 partitions >=2 parts, minimum part >=2
  \\     L---*---C---*---L
  \\ vector(12,n,n+=2; numbpart(n)-numbpart(n-1)-1)
  \\ A002865 - 1,   not containing 1 as a part
  \\
  \\ diameter=5 leaves=periphery
  \\ not in OEIS: 1,1,3,4,9,12,22,31,52,72,113,
  \\
  \\ diameter=6 leaves=periphery
  \\ not in OEIS: 1,1,3,5,10,15,28,43,72,112,178,
  \\
  \\ cf A034853 all free trees n>=3 count by diameter >= 2  (cf. Riorden)
  \\    A000094 num free trees diameter=4
  \\      = partitions of n-1 at least 2 parts >=2
  for(n=7,20,
     my(vpar=vpar_free_first(n),
       counts=vector(n-1));
     until((vpar=vpar_free_next(vpar))=='none,    \\ trees
           if(vpar_diameter_num_endpoints(vpar) == vpar_num_leaves(vpar),
              counts[vpar_diameter(vpar)]++);
          );
     \\ print(counts[^1]);
     print1(counts[6],",");
  );
  quit;
}


TK(n,k) =
{
  if(n==1, 1, \\ (k==0), \\ && k==-1, 1,
     k==0, 0,
     k<0, error("oops n=",n," k=",k),
     n<0, error("oops n=",n," k=",k),
     k>=n, 0,
     n==1, 1,
     sum(j=1,n-1,
         sumdiv(j,d, if(d>=k, TK(d,k-1)*d)) * TK(n-j,k))
     / (n-1));
}
for(n=0,5, for(k=0,10, printf(" %4d",TK(n,k))); print());
T(n) = sum(k=0,n,TK(n,k));
vector(10,n,T(n))

{
  \\ unicentral rooted at that is an unlabelled equal heights
  \\ bicentral
  \\   diameters all through centre
  \\   so same height in both halves
  \\   1/2 * T[k] * T[n-k]  + 1/2*T[n/2] when even
  \\

  for(n=1,20,
     my(count = T(n) + sum(k=1,n\2, sum(p=k+1,n\2, TK(p,k)*TK(n-p,k))));
     print(count));
  quit;
}

{
  \\ free vpar_diameter_num_endpoints() == vpar_num_leaves()
  \\ n=1     4 5
  \\   1,1,1,2,3,5,8,12,19,31,48,77,125,201,323,527,849,
  \\ not in OEIS: 2,3,5,8,12,19,31,48,77,125,201,323,527,849,
  \\
  \\ n=5
  \\ [0, 1, 2, 1, 4]    path-5
  \\ [0, 1, 2, 1, 1]    star arms 1,1,2
  \\ [0, 1, 1, 1, 1]    5-star
  \\
  for(n=1,20,
     my(vpar=vpar_free_first(n),
       count=0);
     until((vpar=vpar_free_next(vpar))=='none,    \\ trees
           if(vpar_diameter_num_endpoints(vpar) == vpar_num_leaves(vpar),
              count++;
              if(n==5,print(vpar));
              );
          );
     print1(count","));
  quit;
}
{
  \\ labelled unrooted vpar_diameter_num_endpoints() == vpar_num_leaves()
  \\ 1,1,1,3,16,125,936,7987,89104,
  \\ not in OEIS: 3,16,125,936,7987,89104,
  for(n=0,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,-1),   \\ root=1
           my(vpar=vpar_from_upascend(vec));
           count += (vpar_diameter_num_endpoints(vpar)
                     == vpar_num_leaves(vpar)));
     print1(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_height_and_count() all childless at height

\\ unlabelled forests
\\ 1,2,3,5,7,12,17,28,42,68,103,168,
\\ same as trees 1 earlier
\\
\\ unlabelled trees
\\ 1,1,2,3,5,7,12,17,28,42,68,103,168,260,
\\ A048816 every childless same height
\\

{
  my(forest=1);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(depths=vpar_depths_vector(vpar),
              childless=vpar_childless_vertices(vpar),
              height=vecmax(depths));
           for(i=1,#childless,
              if(depths[childless[i]]!=height,
                 next(2)));
           count++);
     print1(count",");
  );
}
quit;


\\-----------------------------------------------------------------------------
\\ vpar_Laplacian_poly() coeffs between star and path

\\ Ivan Gutman and Ljiljana Pavlovic, "On the Coefficients of the
\\ Laplacian Characteristic Polynomial of Trees", Bulletin Academie
\\ Serbe Des Sciences Et Des Arts. Classe Des Sciences Mathematiques Et
\\ Naturelles. Sciences Mathematiques, number 28, 2003, pages 31-40.
\\ https://www.jstor.org/stable/44095650
\\
\\ Section 4 conjecturing that star and path are extremes of all Laplacian
\\ coefficients.
\\ Zhou and Gutman later prove this.

{
  for(n=5,20,
     my(star_poly=vpar_Laplacian_poly(vpar_make_star(n)));
     my(path_poly=vpar_Laplacian_poly(vpar_make_path(n)));
     my(vpar=vpar_free_first(n));
     print("n="n);
     print("  path "Vec(path_poly));
     print("  star "Vec(star_poly));
     until((vpar=vpar_free_next(vpar))=='none,
           my(poly=vpar_Laplacian_poly(vpar));
           for(i=2,n-2,
              if(abs(polcoeff(poly,i)) <= abs(polcoeff(star_poly,i)),
                 if(test_is_free_star(vpar),next);
                 print(vpar" smaller than star at i="i);
                 print("  "star_poly);
                 print("  "poly);
                 break));
           for(i=2,n-2,
              if(abs(polcoeff(poly,i)) >= abs(polcoeff(path_poly,i)),
                 if(test_is_free_path(vpar),next);
                 print(vpar" bigger than path at i="i);
                 print("  "poly);
                 print("  "path_poly);
                 break));
          );
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ test_is_gnil()

\\ install("test_is_gnil","G","test_is_gnil","../test_is_gnil.so");

test_is_gnil
test_is_gnil(0)
test_is_gnil(if(0,1))
quit


\\------------------------------------------------------------------------------
\\ Knuth Unlabelled Next Mems
\\
\\ Limit mean singletons .511308...
\\
\\ vpar = [ x, y, z, 0, 0, 0 ]
\\                ^  ^  ^       read to find z position
\\                ^  ^  ^       fetch from earlier and re-write these
\\
\\ Knuth A(n) = num unlabelled trees
\\       A(n+1) = num unlabelled forests
\\
\\ k = vertex to change
\\ j = parent p[k]
\\ d = k - j offset
\\ copy p[k-d] + d to p[k]                        with offset
\\   or p[k-d]     to p[k] when p[k-d]=p[j]       without offset
\\ first d=k-j is copy of p[j]
\\
\\ p[n] in a register
\\ stored back?  so +1
\\ p[j] fetched sep? so +1

{
  my(len=12,
     vec=vpar_count_unlabelled_vector(len,1));
  my(m=(1+vecsum(vec[1..len-1]))/vec[len]*1.0);
  print(m);
  print(2+3*m);
  print();
}
{
  my(n=12);
  my(vpar=vpar_unlabelled_first(n),
     count=0,
     mems=0,
     singletons=0);
  until((vpar=vpar_unlabelled_next(vpar,1))=='none,  \\ forests
        my(s=vpar_num_singletons(vpar));
        singletons+=s;
        mems+=2+3*s;
        count++);
  my(s=singletons/count*1.0);
  print(s);
  print(2+3*s);
  my(m=mems/count*1.0);
  print(m);
  quit;
}




\\------------------------------------------------------------------------------
\\ A079559

A079559(n) =
{
  forstep(k=if(n,3+logint(n+1,2)),1,-1,
     print("n="n" cmp "2^k-2);
     if(n==2^k-2,return(0));
     k--;
     if(n>=2^k-1,
        print("n="n" sub "2^k-1);
        n-=2^k-1);
  );
  1;
}
A079559(5)
\\vector(16,n,n--; A079559(n))
A079559_samples
quit


\\------------------------------------------------------------------------------
\\ balanced binary

to_unlabelled_index(want_vpar) =
{
  my(i=0);
  for(n=1,oo,
     if(n>#want_vpar, return('none));
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,1))=='none,  \\ forests
           \\ print(vpar" "want_vpar);
           i++;
           if(vpar==want_vpar, return(i))));
}
b_to_unlabelled_index(b) = \
  to_unlabelled_index(vpar_from_balanced_binary(b));

want_b = test_from_binary(1100)
vpar_is_unlabelled([])
{
  for(n=1,4,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,1))=='none,  \\ forests
           my(b=vpar_to_balanced_binary(vpar),
              i=to_unlabelled_index(vpar));
           print(vpar" "test_to_binary(b)" "i)));
}

b_to_unlabelled_index(test_from_binary(1 111000 11010100 10 0 1100))
quit


\\------------------------------------------------------------------------------
\\ vpar_to_balanced_binary() sequence of unlabelled BH

{
  \\ in ascending order

  \\ forests
  \\ not in OEIS: 2,10,12,42,50,52,56,170,202,204,210,
  \\ not in OEIS: 10,1010,1100,101010,110010,110100,111000,10101010,11001010,11001100,11010010,
  \\
  \\ trees
  \\ not in OEIS: 2,12,52,56,212,228,232,240,852,916,920,
  \\ not in OEIS: 10,1100,110100,111000,11010100,11100100,11101000,11110000,1101010100,1110010100,1110011000

  my(forest=1,
     count=0);
  for(n=1,20,
     my(vpar=vpar_unlabelled_last(n,forest));
     until((vpar=vpar_unlabelled_prev(vpar,forest))=='none,
           \\ test_to_binary
           print1((vpar_to_balanced_binary(vpar)),",");
           if(count++>10,break(2))));
  quit;
}
{
  \\ in BH descending order

  \\ forests
  \\ not in OEIS: 2,12,10,56,52,50,42,240,232,228,226,212,210,204,202,170,992
  \\ not in OEIS: 10,1100,1010,111000,110100,110010,101010,11110000,11101000,11100100,11100010,
  \\
  \\ trees
  \\ not in OEIS: 2,12,56,52,240,232,228,212,992,976,968,964,936,932,920,916,852,4032,4000,3984,3976
  \\ not in OEIS: 10,1100,111000,110100,11110000,11101000,11100100,11010100,1111100000,1111010000,1111001000,

  my(forest=1,
     count=0);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           print1(test_to_binary(vpar_to_balanced_binary(vpar)),",");
           if(count++>10,break(2))));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_to_balanced_binary() total all preorder

\\ trees
\\ not in OEIS: 2, 12, 108, 1128, 12876, 156024
\\ forests
\\ not in OEIS: 2, 22, 244, 2854, 35004, 445692

forest=1;
vector(6,n,vecsum(apply(vpar_to_balanced_binary,vpar_preorder_all(n,forest))))
quit


\\-----------------------------------------------------------------------------
\\ Tree Distinct 2-Groups by Order

\\ not quite sure this is right
\\         n=1  2  3  4  5  6  7  8  9  10  11
\\           2  4  8 16                            order 2^n
\\           1, 1, 2, 2, 3, 4, 5, 6, 9, 11, 15,
\\ not in OEIS: 1, 2, 2, 3, 4, 5, 6, 9, 11, 15,
\\
\\ n=1 2^n=2   S2 only
\\ n=2 2^n=4   S2xS2 only
\\ n=3 2^n=8   S2xS2xS2, S2wrS2
\\ n=4 2^n=16  S2xS2xS2xS2, S2 x S2wrS2
\\ n=5 2^n=32  S2^5,  S2wrS2 x S2^2,  S2^2 wr S2
\\
\\ n=6         n=5 x S2          i=1,2,3
\\             S2wrS2 x S2wrS2   i=4
\\
\\ n=7         S2^7                    1       1+1+1+1+1+1+1
\\             S2^3 x S2wrS2           1       1+1+1+1+3
\\             S2^2 x S2^2wrS2         1       1+1+5
\\             n=3 wr S2               2       7
\\             S2wrS2 x S2wrS2 x S2            3+3+1
\\   so  n=6 x S2     i=1,2,3,4
\\   and S2wrS2wrS2   i=5
\\   S2^3 wr S2  same as  i=4 S2 x S2wrS2 x S2wrS2

{
  my(groups=vector(7),
     verbose=1);
  my(S2=test_perms_Sn(2));
  print("S2 = "S2);

  groups[1] = [ S2 ];  \\ S2 only size 2^1
  print1(#groups[1]", ");
  if(verbose, print());

  for(n=2,#groups,
     if(verbose, print("n="n));
     my(l=List([]));

     for(a=1,n-1,
        my(b=n-a);
        if(b<a,break);
        if(verbose, print(" direct a="a" b="b));
        a<n && b<n || error();
        for(i=1,#groups[a],
           for(j=1,#groups[b],
              if(verbose, print("  i="i" j="j));
              \\ print("direct "groups[a][i]);
              \\ print("       "groups[b][j]);
              my(new=test_perms_direct_product(groups[a][i], groups[b][j]));
              #new == 2^n || error("n="n" direct a="a" b="b" got size "#new);
              my(found=0);
              for(i=1,#l,
                 if(test_perms_is_isomorphic(new,l[i]),
                    if(verbose, print("  same as i="i));
                    found=1;break));
              if(!found,
                 if(verbose, print("  new"));
                 listput(l,new)))));

     if(n%2==1,
        my(a=n\2);
        if(verbose, print(" wreath a="a));
        for(i=1,#groups[a],
           if(verbose, print("  i="i));
           my(new=test_perms_wreath_product(groups[a][i], S2));
           #new == 2^n || error();
           my(found=0);
           for(i=1,#l,
              if(test_perms_is_isomorphic(new,l[i]),
                 if(verbose, print("  same as i="i));
                 found=1;break));
           if(!found,
              if(verbose, print("  new"));
              listput(l,new))));

     groups[n] = Vec(l);
     if(verbose, print(" sizes "apply(length,groups[n])));

     print1(#l,", ");
     if(verbose, print());
     );
  print();

  my(G=test_perms_wreath_product(S2,S2));
  G = test_perms_direct_product(G,G);
  G = test_perms_direct_product(G,S2);
  my(n=7);
  #G==2^n || error();
  for(i=1,#groups[n],
     if(test_perms_is_isomorphic(G,groups[n][i]),
        print("G iso to i="i)));
  quit;
}

{
  \\ wrong,
  my(a=vector(20));
  a[1]=1;
  my(b=vpar_INTERNAL_Euler_transform(a));
  print(b);
  for(k=1,#a,
     if(2*k+1>#a, break);
     a[2*k+1] = b[k];
     b=vpar_INTERNAL_Euler_transform(a);
     print(b));
  print("final a = "a);
  print("      b = "b);
\\  quit;
}


\\-----------------------------------------------------------------------------
\\ count free trees whose automorphisms are 2-groups

{
  \\ free trees
  \\ 2, 4, 6, 13, 26, 56, 122, 278, 634, 1494, 3540, 8542,
  \\ A309352 (mine)
  \\ vector(15,n,n--; vpar_count_free(n))

  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += (
                     \\ vpar_is_automorphisms_le(vpar_INTERNAL_for_free_automorphisms(vpar),2)
                     \\ &&
                     #vpar_centroids(vpar) <= 1      \\ unicentroidal
                    ));
     print1(count,", ");
     );

}
T = Ser(vector(10,n,n--;vpar_count_unlabelled(n))) - 1;
T = Ser([1, 1, 1, 2, 3, 7, 15, 34, 79, 190, 459, 1136, 2833, 7154, 18206]) - 1
Vec(T - 1/2*T^2 + 1/2*subst(T,'x,'x^2))      \\ all
rooted_vec_to_unicentroidal_free(a,n) =
{
  a[n]                              \\ unicentroidal root=centroid
  - sum(i=1,n>>1, a[i]*a[n-i]);     \\ less unbalanced combinations in n

  \\ print("- "i" * "n-i" = "a[i]" * "a[n-i]);
  \\ print("binomial a["n/2"]="a[n/2]"  = "binomial(a[n/2]+1,2));
}
vector(12,n,vpar_INTERNAL_rooted_vec_to_free([1, 1, 2, 3, 7, 15, 34, 79, 190, 459, 1136, 2833, 7154, 18206],n,1))
vector(14,n,rooted_vec_to_unicentroidal_free([1, 1, 2, 3, 7, 15, 34, 79, 190, 459, 1136, 2833, 7154, 18206],n))
Vec(T - 1/2*T^2 - 1/2*subst(T,'x,'x^2))  \\ odd unicentroidals
quit;

{
  \\ trees rooted unlabelled
  \\ 1, 2, 3, 7, 15, 34, 79, 190, 459, 1136, 2833, 7154, 18206,
  \\ A248869 Satisfies Sum_{n>=0} a(n)*x^n = x * Product_{n>=0} (1 + x^n + x^(2*n))^a(n).
  \\
  \\ cf
  \\ vector(10,n,vpar_count_unlabelled(n))
  \\ n=4 first without, being star-4 three children
  \\ all 1, 1, 2, 4, 9,  20, 48, 115, 286, 719
  \\
  \\ forests rooted unlabelled, one earlier
  \\ root=1 trees same
  \\ not in OEIS: 1, 2, 3, 7, 15, 33, 77, 184, 446, 1098, 2737, 6893,

  my(forest=-1);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count += vpar_is_automorphisms_le(vpar,2);
           \\ count += vpar_is_automorphisms_le(vpar,2,1) != vpar_is_automorphisms_le(vpar,2);
          );
     print1(count,", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_transpose()

{
  \\ forests
  \\ root=1
  \\ trees
  \\
  my(forest=1);
  for(n=0,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              depths=vpar_depths_vector(vpar));
           count += (depths==Vecrev(vpar_depths_vector(vpar_transpose(vpar)))));
     print1(count",");
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ Oliveira and Vergnas
\\ parking to forest

\\ T tree   k below i, so k < i
\\ phi(l) = m min descendant incl i, with m>l    if k <= l < i
\\          k            if l==i
\\          unchanged    if l not below i or l<k or l>i

\\ parking function f, sigma = P(f)
\\ descendants of i a[1] > ... > a[dp(sigma[i])] decreasing
\\ T[i] = phi(T[i-1]; i, a[sigma[i]-f[i]]
\\ phi(f) = T[n]
\\
\\ p = sigma[i] - f[i]
\\ T[i] from T[i-1] cycle (a[p], a[p-1], ..., a[2], a[1], i)

\\ 1 2 3 4 5 6 7 8 9
\\ 1,8,5,2,7,4,4,8,1 in PF9
\\ sigma = P(f)
\\ 1,8,5,2,7,4         six unchanged, park at those place
\\             6       4,5 occupied
\\               9     8 occupied
\\                 3   9,1,2 occupied, cycling around

quit


\\------------------------------------------------------------------------------

vpar_INTERNAL_seq_porder([0,1,1,0,4],0,0)
vpar_INTERNAL_seq_porder([0,1,1,0,4],1,0)
vpar_INTERNAL_seq_porder([0,1,1,0,4],0,1)
vpar_INTERNAL_seq_porder([0,1,1,0,4],1,1)
print("want");
print("1,2,3,4,5");
print("4,5,1,3,2");
print("2,3,1,5,4");
print("5,4,3,2,1");
vpar_seq_preorder([0,1,1,0,4])
vpar_seq_preorder([0,1,1,0,4],1)
vpar_seq_postorder([0,1,1,0,4])
vpar_seq_postorder([0,1,1,0,4],1)
quit

\\ 1  3        1    5         3,4,6 -> 1,2,3
\\ |  |        |    |
\\ 2  4        2    6
\\    |\       |\
\\    5 6      3 4

vpar=[0,1, 0,3,4,4]
vpar_relabel_preorder(vpar_relabel_reverse(vpar))
perm=vpar_preorder_reverse(vpar)
vpar_relabel_perm(vpar,perm)
quit

\\ 1    4       1  3
\\ |\   |       |  |\
\\ 2 3  5       2  4 5

vpar_relabel_preorder(vpar_relabel_reverse([0,1,1, 0,4]))
vpar_preorder_reverse([0,1,1, 0,4])
quit

{
  my(n=4,
     vpar=vpar_preorder_first(n));
  until((vpar=vpar_preorder_next(vpar,1))=='none,
        my(depths=vpar_depths_vector(vpar),
           rev=vpar_relabel_preorder(vpar_relabel_reverse(vpar)),
           revdepths=vpar_depths_vector(rev),
           revpost=vpar_relabel_postorder(rev),
           revpostdepths=vpar_depths_vector(revpost));
        print(vpar"  "depths" "revdepths
              "  "revpostdepths" "depths==Vecrev(revpostdepths));
       );
  quit
}


\\------------------------------------------------------------------------------

f(pqvec) =
{
  my(k=#pqvec);
  if(k==0, return(1));
  if(k==1, return(binomial(pqvec[1][1]+pqvec[1][2], pqvec[1][1])));
  if(pqvec[1][1]>0,
     my(p1sub1=pqvec); p1sub1[1][1]--;
     f(p1sub1))
  +
  sum(j=1,k-1,  f(pqvec[1..j])*f(pqvec[j+1..k]))
  +
  if(pqvec[k][2]>0,
     my(qksub1=pqvec); qksub1[k][2]--;
     f(qksub1));
}
f=memoize(f);
f([[1,0],[0,0]])
f([[1,0],[0,0],[1,0],[5,3],[0,0],[0,0],[0,2],[0,0],[1,2]])
quit


\\------------------------------------------------------------------------------
\\ postorder specification

\\ R N B N
\\ [3, 3, 0, 0]
\\
\\        3 B             1 R
\\       / \               \             R N B N
\\    R 1   4 N             3 B          in-order
\\       \                 / \
\\        2 N           N 2   4 N
vpar=[3, 3, 0, 0]
vpar=vpar_relabel_reverse(vpar);
vpar=vpar_transpose(vpar);
vpar=vpar_relabel_reverse(vpar);
vpar=vpar_relabel_postorder(vpar)


spec_of_vpar(vpar) =
{
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min));
  vector(#vpar,v,
         if(next_sibling[v],
            if(first_child[v], 'B, 'R),
            if(first_child[v], 'L, 'N)));
}
spec_to_pqvecs(spec) =
{
  my(pvec=List([]),
     qvec=List([]),
     i=1);
  while(i<=#spec,
        my(count=0);
        while(i<=#spec && spec[i]=='R, count++;i++);
        listput(pvec,count);

        spec[i]=='N || error();
        i++;

        count=0;
        while(i<=#spec && spec[i]=='L, count++;i++);
        listput(qvec,count);

        if(i<=#spec,
           spec[i]=='B || error();
           i++));

  [Vec(pvec),Vec(qvec)];
}
pqvecs_count(pvec,qvec) =
{
  \\ Knuth's formula
  prod(i=1,#pvec, binomial(pvec[i]+qvec[i], qvec[i]))
    * vpar_INTERNAL_Catalan_number(#pvec-1);
}


n=8;
{
  my(vpar=vpar_preorder_first(n),
     count=0,
     m=Map());
  until((vpar=vpar_preorder_next(vpar,1))=='none,
        my(vpar=vpar_relabel_postorder(vpar));
        my(spec=spec_of_vpar(vpar));
        print(vpar"  ",concat(apply(s->Str(s),spec)));

        my(prev_sibling=vpar_next_sibling_vector(vpar,1));
        for(v=1,#vpar,
           if(spec[v]=='B && prev_sibling[v] && spec[prev_sibling[v]]!='B,
              next(2));
           if(spec[v]=='B && vpar[v] && spec[vpar[v]]!='B,
              next(2)));

        \\ spec[vpar_root(vpar)]=='B || next;

        my(count=0);
        mapisdefined(m,spec,&count);
        mapput(m,spec,count+1);
        \\ spec==['R,'N,'B,'N] || next;
       );
  my(M=Mat(m));
  print("matsize "matsize(M));
  for(i=1,matsize(M)[1],
     my(spec=M[i,1], pqvecs=spec_to_pqvecs(spec), pvec,qvec);
     \\ spec==['R,'N,'B,'N] || next;
     [pvec,qvec]=pqvecs;
     my(c=M[i,2],
        f=pqvecs_count(pvec,qvec));
     print("spec "spec" count "c"  "pqvecs
           " formula "f,if(c!=f,"   ***","")));
  print("total ",vecsum(M[,2]));
  quit;
}


\\------------------------------------------------------------------------------
\\ footprint specification matching

matches_spec(vpar,spec) =
{
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min));
  for(v=1,#vpar,
     if(spec[v]=='R,  next_sibling[v] || return(0));
     if(spec[v]=='R,  !first_child[v] || return(0));
     if(spec[v]=='L,  first_child[v] || return(0));
     if(spec[v]=='L,  !next_sibling[v] || return(0));
     if(spec[v]=='B,  (first_child[v] && next_sibling[v]) || return(0));
      if(spec[v]=='N,  (!first_child[v] && !next_sibling[v]) || return(0));
     );
  1;
}

\\ [3, 3, 0, 0]  1              3 B         1 R
\\ [0, 3, 0, 0]  1             / \           \
\\                          R 1   4 N         3 B
\\ 3   4     1 3 4             \             / \
\\ |\          |              N 2         N 2   4 N
\\ 1 2         2
\\
\\ p1=1; q1=0; p2=0; q2=0; k=2;
\\ spec = ['R, 'N, 'B, 'N];


\\ [2, 4, 4, 0]  1            4 L        B 2
\\ [2, 0, 4, 0]  1           /            / \
\\                        B 2          N 1   4 L
\\    4       2  4         / \              /
\\    |\      |  |      N 1   3 N          3 N
\\    2 3     1  3
\\    |
\\    1
\\ footprint [0, 1, 0, 0]
\\ p1=0; q1=0; p2=0; q2=1; k=2;
\\ spec = ['N, 'B, 'N, 'L];

\\       7             1       1  7          4,5,6,7
\\      /               \         |\         N L L L
\\     6                 7        2 6
\\    /                 /           |
\\   5                 2            5
\\  /                   \           |\
\\ 1                     6          3 4
\\  \                   /
\\   2                 5
\\    \               /
\\     3             3
\\      \             \
\\       4             4
\\ count 20
\\ p1=3; q1=3; binomial(p1+q1,q1) == 20
\\ spec = ['R,'R,'R, 'N, 'L,'L,'L];


\\ p1=2;q1=2; p2=2;q2=2; binomial(p1+q1,q1)*binomial(p2+q2,q2)
spec = ['R,'R, 'N, 'L,'L, 'B, 'R,'R, 'N, 'L,'L];


\\ pqs_to_spec(pvec,qvec) =
\\ {
\\   my(l=List([]));
\\   for(i=1,#pvec,
\\      if(i!=1, listput(l,'B));
\\      for(j=1,pvec[i], listput(l,'R));
\\      listput(l,'N);
\\      for(j=1,qvec[i], listput(l,'L)));
\\   Vec(l);
\\ }
\\ spec = pqs_to_spec([0,0],[0,1])


n = #spec;
{
  my(vpar=vpar_preorder_first(n),
     count=0);
  until((vpar=vpar_preorder_next(vpar,1))=='none,
        my(vpar=vpar_relabel_postorder(vpar));
        my(m=matches_spec(vpar,spec));
        m || next;
        print(vpar"  "m);
        count++;
        );
  print("count "count);
  quit;
}






\\------------------------------------------------------------------------------
\\ vpar_lower_matchnum() states

lower_matchnum_rel_add(p_rel,v_rel) =
{
  \\ print("lower_matchnum_rel_add() "p_rel"  "v_rel);

  my(v_max = min(if(v_rel[1]=='no_with,        99, v_rel[1]),
                 if(v_rel[2]=='no_without_max, 99, v_rel[2])));
  \\ print(" v_max  "v_max);

  \\ with[p] = min(if(with_exists[p], with[p] + v_max,  #vpar),
  \\               without[p] + without[v] + 1);
  my(ret_with = min(if(p_rel[1]=='no_with, 99,  p_rel[1] + v_max),
                    0         +     0      + 1));
  \\ print(" ret_with  "ret_with);
  if(ret_with==99, error());

  \\ without_max[p] += with[v];
  \\ if(!with_exists[v], without_max_exists[p] = 0);
  my(ret_without_max = if(p_rel[2]=='no_without_max || v_rel[1]=='no_with,
                          'no_without_max,
                          p_rel[2] + v_rel[1]));
  if(ret_without_max==99, error());
  \\ print(" ret_without_max  "ret_without_max);

  \\ without[p] += v_max;
  [if(ret_with=='no_with,               ret_with,        ret_with - v_max),
   if(ret_without_max=='no_without_max, ret_without_max, ret_without_max - v_max)];
}

check_equal(lower_matchnum_rel_add([no_with,0], [no_with,0]), [1,no_without_max]);
check_equal(lower_matchnum_rel_add([1,no_without_max], [no_with,0]), [1,no_without_max]);
check_equal(lower_matchnum_rel_add([no_with,0], [1,no_without_max]), [0, 0]);

states=List([ [no_with,0] ]);
vpars =List([ [0] ]);
{
  my(more=1);
  while(more,
        more=0;
        for(i=1,#states,
           for(j=1,#states,
              my(p_rel = states[i],
                 v_rel = states[j]);

              my(out = lower_matchnum_rel_add(p_rel,v_rel));
              if(!test_vector_search(states,out),
                 my(out_vpar=vpar_join(vpars[i],1,vpars[j]));
                 printf("%-20s + %-20s -> %s  tree %s\n", p_rel, v_rel, out, out_vpar);
                 listput(states,out);
                 listput(vpars,out_vpar);
                 more=1;
                \\ next(3);
                ))));
}
quit


\\------------------------------------------------------------------------------
\\ Bit Runs Rearrange

\\ high 1 and low 0 can move too:
\\ not in OEIS: 2,3,3,4,2,6,4,5,3,2,2,10,3,10,5,6,4,3,3,2,2,2,2,15,6,3,3,20,4,15,6,7,5,4,4,3,3,3,3,2,2,2,2,2,2,2,2,21,10,6,6,3,3,3,3,35,10,4,4,35,5,21,7,
\\
\\ high 1 and low 0 fixed
\\ not in OEIS: 1,1,1,1,1,2,1,1,1,1,1,3,1,3,1,1,1,1,1,1,1,1,1,4,2,1,1,6,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,3,2,2,1,1,1,1,10,3,1,1,10,1,5,1,

rearrange_blocks(n) =
{
  n>=1 || error();
  my(b=binary(2*n));
  b=test_vector_run_lengths(b);
  #b%2==0 || error();
  my(ret=1);
  forstep(i=1,#vec,2,
     ret *= binomial(b[i]+b[i+1], b[i]));
  ret;
}
\\check_equal(rearrange_blocks(1), 1);
print("x");
{
  my(k=6);
  for(n=1,2^k-1, print1(rearrange_blocks(n)","));
  print();
  quit;
}

{
  \\ run lengths 1s and 0s of 2n
  \\ not in OEIS: 1,1,1,2,2,1,1,3,1,1,1,1,2,2,3,1,1,4,1,2,1,1,1,1,1,2,1,1,2,1,2,3,2,1,1,1,3,2,4,1,

  \\ 1,1,1,2,1,2,1,1,1,2,1,3,1,3,1,2,1,1,1,1,1,1,1,2,2,2,2,1,1,3,1,4,
  \\ A101211 run lengths of 1s and 0s of n

  my(k=4);
  for(n=1,2^k-1,
     my(b=test_vector_run_lengths(binary(n)));
     for(j=1,#b, print1(b[j]",")));
  print();
}
quit


\\------------------------------------------------------------------------------
\\ Footprint Catalan
\\
\\ p1=1 q1=1        1
\\  1,0,0           |\
\\                  2 3
\\                   \
\\                    3
\\ binomial(1+1,1) == 2
\\
\\ p1=0 q1=0  p2=0 q2=1
\\     0,       1,0,0       1  2        1  2  4
\\                             |\          |
\\                             3 4         3
\\
\\ binomial(0+0,0) == 1
\\ binomial(0+1,1) == 1
\\ vpar_INTERNAL_Catalan_number(2-1) == 1
\\
\\ k things in m buckets
\\ binomial(m+k-1,m-1) = binomial(m+k-1,k)
\\ qk things
\\ pk+1 places
\\ binomial(pk+1+qk-1, qk) = binomial(pk+qk, qk)
\\
\\ p1=1 q1=0  p2=0 q2=0
\\    1,0,       1,0           1          1  3
\\                             | \        |  |
\\                             2  3       2  4
\\                                |
\\                                4
\\ binomial(1+0,0) == 1
\\ binomial(0+0,0) == 1
\\ vpar_INTERNAL_Catalan_number(2-1) == 1
\\ [0, 1, 2, 2, 4] [1, 0, 1, 0]   1     1
\\ [0, 1, 2, 1, 4] [1, 0, 1, 0]   |     | \
\\                                2     2  4
\\                                |\    |  |
\\                                3 4   3  5
\\                                  |
\\                                  5
\\
\\ p1=1 q1=1  p2=0 q2=0         1 3
\\  1, 0,0,     1, 0            |\   \
\\                              2 3   4
\\                                    |
\\                                    5
\\ binomial(1+1,1) == 2
\\ binomial(0+0,0) == 1
\\ vpar_INTERNAL_Catalan_number(2-1) == 1
\\ [0, 1, 1, 3, 3] [0, 1, 0, 0]   1      1
\\ [0, 1, 1, 3, 1] [0, 1, 0, 0]   |\     |\ \
\\                                2 3    2 3 5
\\                                  |\     |
\\                                  4 5    4
\\
\\
\\ p1=0 q1=0  p2=0 q2=1
\\       0,    1,0,0
\\ binomial(0+0,0) == 1
\\ binomial(0+1,1) == 1
\\ vpar_INTERNAL_Catalan_number(2-1) == 1

is_FC(vpar) =
{
  my(is_childful=vpar_is_childful_vector(vpar),
     is_top=vector(#vpar,v,
                   v==1 || (is_childful[v] && !is_childful[v-1])),
     depths=vpar_depths_vector(vpar));

  my(to_top=vector(#vpar));
  to_top[1]=1;
  for(v=2,#vpar,
     to_top[v] = if(is_top[v], v,
                    v==1, 0,
                    to_top[v-1]));

  if(#vpar>=2,
     \\ vpar[2]==1 || return(0);
     \\ is_childful[2]==1 || return(0);
    );

  for(v=2,#vpar,
     if(is_top[v],
        my(p=vpar[v]);
        if(!p, return(0));
        is_top[p] || return(0)));

  for(v=2,#vpar,
     if(!is_childful[v],
        my(p=vpar[v]);
        p>=to_top[v]  || return(0)));
\\ || to_top[v]==1
  1;
}

footprint_runs(vec) =
{
  \\ vec[1]==1 || error();
  vec[#vec]==0 || error();
  my(l=List([]),
     i=1);
  while(i<=#vec,
        my(p=-1);
        while(i<=#vec && vec[i]==1, p++; i++);
        listput(l,p);
        my(q=-1);
        while(i<=#vec && vec[i]==0, q++; i++);
        listput(l,q);
        );
  l=Vec(l);
  \\ if(#l, l[1]++);
  l;
}

footprint_count(vec) =
{
  \\ Knuth's formula
  my(runs=footprint_runs(vec),
     k=#runs/2);
  if(k,
     prod(i=1,k,
          my(p=runs[2*i-1], q=runs[2*i]);
          binomial(p+q, q))
     * vpar_INTERNAL_Catalan_number(k-1));
}

{
  my(forest=0);
  for(n=1,12,
     print("n="n);
     my(vpar=vpar_preorder_first(n),
        total=0,
        m=Map());
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vec=vpar_is_childful_vector(vpar),
              isf=is_FC(vpar));
           print("vpar="vpar"  "vec"  "isf);
           if(isf,
              total++;
              my(count=0);
              mapisdefined(m,vec,&count);
              mapput(m,vec,count+1);
             ));
     print("distincts "#m);
     my(M=Mat(m));
     print("matsize "matsize(M));
     my(bad=0);
     for(i=1,matsize(M)[1],
        my(vec=M[i,1],
           runs=footprint_runs(vec),
           f=footprint_count(vec));
        print("vec "vec" runs "runs" count ",M[i,2]" formula "f,
              if(f==M[i,2],"",bad++;"  ***")));
     print("total "total);
     print();
     if(bad,break);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_depths_vector() occurring on functional digraphs

\\ function digraph forests
\\ not in OEIS: 1,1,4,17,103,747,6523,66019,
\\ (nothing from superseeker, with 20 or so values)
\\
\\ function digraph single component
\\ 1,1,3,13,75,541,4683,47293,
\\ A000670 ordered Bell
\\
\\ cf
\\ trees   1,1,2,9,52,375,3246,
\\ forests 1,1,3,13,75,541,4683,
\\
\\ Multi-component functional digraphs.
\\ With no depth gaps is ordered Bell.
\\ At depths gap the next value d must occur d+1 times cycle.
\\ Or multiple of d+1 multiple component cycles.
\\ Below the gap is functional digraphs of height <= d-2.
\\
\\ n=4  103

OBell_max_buckets(n,k) =
{
  \\ all in first bucket,
  \\ or 1 to n-1 in first bucket, rest n-1 in k-1 buckets
  if(n<0 || k<0, error());
  if(k==0, return(n==0));
  1 + sum(i=1,n-1, binomial(n,i)*OBell_max_buckets(n-i,k-1));
}
OBell_max_buckets=memoize(OBell_max_buckets);
check_equal(OBell_max_buckets(0,0), 1);
check_equal(OBell_max_buckets(1,1), 1);
check_equal(OBell_max_buckets(2,1), 1);
check_equal(OBell_max_buckets(2,2), 3);  \\ 0,0  0,1  1,0
vector(10,n,n--; test_ordered_Bell(n))
\\ not in OEIS: 63, 603, 2163, 3963, 4683
for(n=1,8, for(k=1,n, print1(OBell_max_buckets(n,k), ", ")); print());
print()

\\ A019538 assign 1..n to ordered list of exactly k buckets = k!*Stirling2(n,k)
for(n=1,8, for(k=1,n, print1(OBell_max_buckets(n,k)-OBell_max_buckets(n,k-1), ", ")); print());
print()

\\ FF(n) =
\\ {
\\   \\ top
\\   sum(c=2,n,
\\       sum(m=1,n\c,
\\           my(C=m*c);
\\           sum(T=C,n,
\\               binomial(n,T) * binomial(T,C) * Fle(T-C,c-3))));
\\ }
Fle(n,k) =
{
  \\ no gaps, into k+1 buckets
  OBell_max_buckets(n,k+1)
  +
  \\ top gap at cycle length c, is depth c-1
  \\ must c-1<=k so c<=k+1
  \\ rest depths c to k inclusive, is <= k-c+1
  \\ smalle depth c-2 none, must limit to c-3
  sum(c=2,k+1,
      sum(m=1,n\c,      \\ m many cycles
          my(C=m*c);    \\ C many vertices in cycles
          sum(T=C,n,    \\ T many vertices in cycles plus deeper
              binomial(n,T)
              * binomial(T,C)
              * OBell_max_buckets(T-C, k-c+1)
              * Fle(n-T, c-3)
          )));
}
Fle=memoize(Fle);
FF(n) = Fle(n,n-1);


\\ n=2
\\ [0, 0]  \
\\ [0, 1]  |
\\ [1, 0]  /
\\ [1, 1]
check_equal(OBell_max_buckets(2,2), 3);

\\
check_equal(vector(8,n,n--; FF(n)), [1,1,4,17,103,747,6523,66019]);

vector(20,n,n--; FF(n))

{
  my(forest=1);
  for(n=0,8,
     my(m=Map());
     forvec(vpar=vector(n,i,[0,n]),
           forest || vpar_num_components(vpar)<=1 || next;
           my(depths=vpar_depths_vector(vpar));
           mapput(m,depths,1));
     print1(#m,",");
     my(l=Mat(m)[,1]);
     if(n==2,
        \\ for(i=1,#l, print(l[i]));
        \\ for(i=1,#l, if(hammingweight(l[i])==4,print(l[i])));
        \\ for(i=1,#l, if(hammingweight(l[i])!=4,
        \\                print(l[i],
        \\                      if(vpar_is_depths_vector(l[i],1),""," ***"))));
       );
  );
  quit;
}

\\ {
\\   \\ Number of different depths vectors occurring in functional digraphs,
\\   \\ same as forests.
\\   \\ Single-component or multi-component functional digraphs are the same,
\\   \\ since all of their cyclic vertices are the same depth=0.
\\   for(n=0,6,
\\      for(forest=0,1,
\\         my(l=List([]));
\\         forvec(vpar=vector(n,i,[0,n]),
\\               forest || vpar_num_components(vpar)<=1 || next;
\\               my(depths=vpar_depths_vector(vpar));
\\               my(pos=setsearch(l,depths,1));
\\               if(pos, listinsert(l,depths,pos)));
\\         l=Vec(l);
\\         check_equal(setisset(l),1);
\\         check_equal(#l, vpar_depths_vectors_count(n,1),
\\           "vpar_depths_vectors_count() vs distinct vpar_depths_vector()")));
\\ }


\\------------------------------------------------------------------------------
\\ Motzkin Forests by Footprint
\\
\\ Motzkin forests distinct is_childless_vector occurring
\\ 1,2,3,6,10,20,35,70,126,252,462,924,
\\ A001405 a(n) = binomial(n, floor(n/2))

footprint_runs(vec) =
{
  vec[#vec]==0 || error();
  my(l=List([]),
     i=1);
  while(i<=#vec,
        my(p=-1);
        while(i<=#vec && vec[i]==1, p++; i++);
        listput(l,p);
        my(q=-1);
        while(i<=#vec && vec[i]==0, q++; i++);
        listput(l,q);
        );
  l=Vec(l);
  if(#l, l[1]++);
  l;
}

footprint_count(vec) =
{
  \\ Knuth's formula
  my(runs=footprint_runs(vec),
     k=#runs/2);
  if(k,
     prod(i=1,k,
          my(p=runs[2*i-1], q=runs[2*i]);
          binomial(p+q, q))
     * vpar_INTERNAL_Catalan_number(k-1));
}
{
  my(footprint
     =[1,0, 1,0, 1,1,0, 1,1,1,1,1,1,0,0,0,0, 1,0, 1,0, 1,0,0,0, 1,0, 1,1,0,0,0],
    runs
    = [1,0, 0,0,   1,0,           5,3,       0,0, 0,0, 0,2,     0,0,   1,2    ]);
  check_equal(vecsum(footprint), 16);
  check_equal(#footprint, 32);
  check_equal(footprint_runs(footprint), runs);
  check_equal(footprint_count(footprint), 240240);
}
{
  my(forest=1,
     prev_counts=[]);
  for(n=2,7,
     my(vpar=vpar_preorder_first(n),
        m=Map(),
        count_all=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           \\ my(vpar=vpar_relabel_postorder(vpar));
           \\ test_is_Motzkin_forest(vpar) || next;
           count_all++;
           my(vec=vpar_is_childful_vector(vpar),
              count=0);
           \\ if(vec==[0, 1, 0, 1], print(vpar" "vec));
           \\ if(vec==[0,0,0,1, 0,0,0,1], print(vpar" "vec));
           mapisdefined(m,vec,&count);
           mapput(m,vec,count+1));
     \\ print1(#m,",");
     my(M=Mat(m),
       l=List([]),
       fcount_total=0);
     if(1,
        print("n="n);
        print("matsize "matsize(M));
        for(i=1,matsize(M)[1],
           my(vec=M[i,1]);
           \\ f=f[^1];
           my(fruns=footprint_runs(vec),
              fk=#fruns\2,
              fcount=footprint_count(vec));
           print("vec ",vec,
                 "="vecsum(vec),
                 " runs ",fruns,
                 " formula=",footprint_count(vec),
                 " M=",M[i,2]
                 "   k="fk
                );
           fcount_total += fcount;
        );
        \\ count_all==A001006_samples[n] || error();  \\ Motzkin numbers
        print("fcount_total "fcount_total" count_all "count_all));
     \\ print1(#m",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_diameter() Speed of Cyclic

\\ vpar=vpar_relabel_reverse(vpar);

{
  for(i=1,1000,
     my(t1=getabstime());
     if(getabstime()<t1, error()));
}
{
  print("time");
  for(k=8,16,
     my(n=2^k,
        vpar=test_make_cycle(n),
        \\ vpar=vpar_make_path(n),
        reps=max(1,ceil(500000000/n^3)),
        t1=getabstime());
     for(i=1,reps, vpar_diameter(vpar));
     my(t2=getabstime());
     rtime=(t2-t1)/reps*1.0;

     print(k"  "rtime"  "log(rtime)" reps="reps);
     if(t2<t1, print(t1" "t2));
  );
}
quit


\\------------------------------------------------------------------------------
\\ Ecc Pairwise

{
  \\ 1 2 3 4 5 6 7 8 9 10
  \\   1 2 3 4 5 4 3 2  1
  my(vec=vector(10),
     i=1);
  check_equal(vector(#vec\2,j, ((i+j-1)%#vec)+1), [2, 3, 4, 5, 6]);
  i=8;
  check_equal(vector(#vec\2,j, ((i+j-1)%#vec)+1), [9,10, 1,2,3]);
}
{
  \\ 1 2 3 4 5 6 7 8 9
  \\   1 2 3 4 4 3 2  1
  my(vec=vector(9),
     i=1);
  check_equal(vector(#vec\2,j, ((i+j-1)%#vec)+1), [2, 3, 4, 5]);
  i=7;
  check_equal(vector(#vec\2,j, ((i+j-1)%#vec)+1), [8,9, 1,2]);
}

{
  \\ 1 2 3 4 5 6 7 8 9 10
  \\   1 2 3 4 5 4 3 2 1
  my(vec=vector(10),
     i=1, l=List([]));
  for(j=i+1,#vec, listput(l, abs(test_mod_nearest(i-j,#vec))));
  check_equal(l, List([1,2,3,4,5,4,3,2,1]));
  i=7; l=List([]);
  for(j=i+1,#vec, listput(l, abs(test_mod_nearest(i-j,#vec))));
  check_equal(l, List([1,2,3]));
}
{
  \\ 1 2 3 4 5 6 7 8 9
  \\   1 2 3 4 4 3 2 1
  my(vec=vector(9),
     i=1, l=List([]));
  for(j=i+1,#vec, listput(l, abs(test_mod_nearest(i-j,#vec))));
  check_equal(l, List([1,2,3,4,4,3,2,1]));
  i=7; l=List([]);
  for(j=i+1,#vec, listput(l, abs(test_mod_nearest(i-j,#vec))));
  check_equal(l, List([1,2]));
}

test_eccycle_to_diameter(vec) =
{
  my(m=if(#vec,vecmax(vec)));
  for(i=1,#vec,
     for(j=i+1,#vec,
        m=max(m,vec[i] + abs(test_mod_nearest(i-j,#vec)) + vec[j])));
  m;
}
{
  check_equal(test_eccycle_to_diameter([1,1,1]), 3);
  check_equal(test_eccycle_to_diameter([1,1,1,1]), 4);
  check_equal(test_eccycle_to_diameter([1,1,1,1,1]), 4);
  check_equal(test_eccycle_to_diameter([1,1,1,1,1,1]), 5);
  check_equal(test_eccycle_to_diameter([10,10,1,1,1,1]), 21);
  check_equal(test_eccycle_to_diameter([10,1,1,1,1,10]), 21);
  check_equal(test_eccycle_to_diameter([10,1,1,1,10,1]), 22);
}

\\ start i=1
\\ offset = ecc[1]
\\        #ecc       #ecc-1     #ecc-2     #ecc-3
\\ d = [ delta -1,  delta -2,  delta -3,  delta -4 ]
\\
\\ step to i=2
\\ discard delta -4 now too far
\\ replace with i=1 ecc
\\ others offset +1
\\ step offset ecc[2] - ecc[1]
\\ new i=1 term want diameter = ecc[2]+ecc[1]+1
\\ have offset = ecc[1] + ecc[2] - ecc[1] + 1 = ecc[2]+1
\\ value + offset = ecc[2]+ecc[1]+1
\\ value = ecc[2]+ecc[1]+1 - offset;
\\
test_eccycle_to_diameter_by_vecmax(vec) =
{
  \\ print("test_eccycle_to_diameter_by_vecmax() "vec);
  if(#vec==1,  vec[1],
     #vec>1,
     my(m=0,
        h=#vec>>1,
        d = vectorsmall(h,i, vec[#vec-h+i] + h - i),
        p=1);
     \\ print("  distance ",vector(h,i, h-i));
     \\ print("  index    ",vector(h,i, #vec-h+i));
     for(i=1,#vec,
        my(offset = );
        \\ print("  at i="i" p="p" offset="offset" d="Vec(d)" implies "apply(v->v+offset,Vec(d)));
        m=max(m, vecmax(d) + vec[i] + i);
        d[p] = vec[i] - i;
        if(p++>h, p=1);
     );
     m);


        \\ offset = vec[1],
        \\ print( offset"  " vec[i] + i-1);
        \\ d[p] = (vec[i]<<1) - offset;
        \\ offset += vec[i+1] - vec[i] + 1;
        \\ offset = vec[i+1] + i;
        \\ print( offset"  " vec[i+1] + i);
        \\ d[p] = vec[i+1]+vec[i]+1 - offset;
     \\ max(m,vecmax(d)+offset)
}

{
  check_equal(test_eccycle_to_diameter_by_vecmax([100,1000,10]), 1101);
  check_equal(test_eccycle_to_diameter_by_vecmax([1,1,1]), 3);
  check_equal(test_eccycle_to_diameter_by_vecmax([1,1,1,1]), 4);
  check_equal(test_eccycle_to_diameter_by_vecmax([1,1,1,1,1,1,1]), 5);
  check_equal(test_eccycle_to_diameter_by_vecmax([1,1,1,1,1]), 4);
  check_equal(test_eccycle_to_diameter_by_vecmax([10,10,1,1,1,1]), 21);
  check_equal(test_eccycle_to_diameter_by_vecmax([10,1,1,1,1,10]), 21);
  check_equal(test_eccycle_to_diameter_by_vecmax([10,1,1,1,10,1]), 22);
}


{
  for(rep=1,1000000,
     my(len=random(20),
        eccs=vector(len,i, random(20)));
     test_eccycle_to_diameter_by_vecmax(eccs)
     == test_eccycle_to_diameter(eccs)
       || error(eccs));

  \\ 8+13+1
}

quit




\\------------------------------------------------------------------------------
\\ vpar_is_childless_vector() distinct occurring, footprints
\\
\\ All 2^(n-1) combinations occur.

\\ Preorder distinct products count * count(complement) occurring
\\ not in OEIS: 1,6,10,12,15,20,25,28,30,36,38,40,45,48,49,60,63,70

\\ Preorder counts of forests by their leaf footprints occurring,
\\ not in OEIS: 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 20, 21, 22, 23, 25, 28, 30, 31, 34, 35
\\
\\ Knuth fasc4a 7.2.1.6 exercise 30(b) answer counting by footprint.
\\ Footprint 1xp1, 0*q1+1, 1*p2+1, 0*q2+1, ..., 1*pk+1, 0*qk+1
\\
\\ Exercise 27 answer: forest represented by right-child and left-sibling.
\\ Preorder forest = preorder binary tree.
\\ Forest subtree size = binary tree right size.

footprint_runs(vec) =
{
  vec[#vec]==0 || error();
  my(l=List([]),
     i=1);
  while(i<=#vec,
        my(p=-1);
        while(i<=#vec && vec[i]==1, p++; i++);
        listput(l,p);
        my(q=-1);
        while(i<=#vec && vec[i]==0, q++; i++);
        listput(l,q);
        );
  l=Vec(l);
  if(#l, l[1]++);
  l;
}
footprint_count(vec) =
{
  my(runs=footprint_runs(vec),
     k=#runs/2);
  if(k,
     prod(i=1,k, binomial(runs[2*i-1]+runs[2*i], runs[2*i-1]))
     * vpar_INTERNAL_Catalan_number(k-1));
}
footprint_binomials(vec) =
{
  my(runs=footprint_runs(vec),
     k=#runs/2);
  prod(i=1,k, binomial(runs[2*i-1]+runs[2*i], runs[2*i-1]));
}
{
  \\ with high 1 always
  \\ not in OEIS: 1,2,1,3,1,3,1,4,2,1,1,6,1,4,1,5,3,2,2,1,2,2,1,10,3,1,1,10
  \\ sans high 1
  \\ not in OEIS: 1,1,1,1,1,2,1,1,1,1,1,3,1,3,1,1,1,1,1,1,2,2,1,4,2,1,1,6,1,4
  my(k=18,
     total_by_len=vector(k+1));
  forstep(n=2,2^(k+1)-2,2,
     my(footprint=binary(n));
     \\ footprint=footprint[^1];
     my(runs=footprint_runs(footprint),
        c = footprint_count(footprint));
     #runs % 2 == 0 || error();
     \\ print(footprint" runs "runs" count "c);
     \\ print1(footprint_count(footprint)",");
     total_by_len[#footprint] += c;
  );
  print();
  print(total_by_len);

  \\ totals, no initial 1
  \\ A086622 A(x) = 1/(1-2*x) + x^2*A(x)^2
  \\ Motzkin paths of length n having no (1,0)-steps at levels 1,3,5,...
  \\ (1,0)-steps of two colours at levels 2,4,6,...

  \\ totals, with initial 1
  \\ not in OEIS: 1, 3, 8, 20, 50, 126, 323, 843, 2241, 6061, 16655, 46423, 131043, 374043, 1078088, 3133848, 9177598, 27052618
  \\ cumulative of A086622
  quit;
}
{
  my(footprint
     =[1,0, 1,0, 1,1,0, 1,1,1,1,1,1,0,0,0,0, 1,0, 1,0, 1,0,0,0, 1,0, 1,1,0,0,0],
    runs
    = [1,0, 0,0,   1,0,           5,3,       0,0, 0,0, 0,2,     0,0,   1,2    ]);
  check_equal(vecsum(footprint), 16);
  check_equal(#footprint, 32);
  check_equal(footprint_runs(footprint), runs);
  check_equal(footprint_count(footprint), 240240);

  \\ my(n=16,
  \\    vpar=vpar_preorder_first(n),
  \\    want_childless=apply(b->1-b, footprint),
  \\    count=0);
  \\ until((vpar=vpar_preorder_next(vpar,1))=='none,
  \\       if(vpar_is_childless_vector(vpar)==want_childless,
  \\          \\ print(vpar);
  \\          count++));
  \\ print(count);
}

{
  my(footprint=[1, 0, 1, 0]);
  print(footprint_count(footprint));
  check_equal(footprint_runs(footprint), [1,0,0,0]);
  check_equal(binomial(1+0,0), 1);
  check_equal(binomial(0+0,0), 1);
  check_equal(binomial(1+0,0)*binomial(0,0), 1);
  check_equal(binomial(1+0,0)*binomial(0,0)*vpar_INTERNAL_Catalan_number(2-1), 1);
}
\\ footprint 11111000
\\
\\ 1010                        1100
\\    1       1  3            1  4
\\    | \     |  |            | \
\\    2  3    2  4            2  4
\\       |                    | \
\\       4                    3  4
\\
\\ run 1s each under the previous to force it nonleaf
\\ first 0 under last 1 to force it nonleaf
\\ other 0s under any combination of all the 1s
\\ next run 1s go under any of the open preceding 1s

\\ 1,1,1,0, 1,1,1,0       3,0,2,0
\\ RRR N B RR
\\
\\       / \
\\ binomial(3+0,3) == 1
\\ binomial(2+0,2) == 1
\\ vpar_INTERNAL_Catalan_triangle(1) == 1
\\ [0, 1, 2, 3, 3, 5, 6, 7] [0, 0, 0, 1, 0, 0, 0, 1]
\\ [0, 1, 2, 3, 2, 5, 6, 7] [0, 0, 0, 1, 0, 0, 0, 1]
\\ [0, 1, 2, 3, 1, 5, 6, 7] [0, 0, 0, 1, 0, 0, 0, 1]
\\ [0, 1, 2, 3, 0, 5, 6, 7] [0, 0, 0, 1, 0, 0, 0, 1]
\\  1   .
\\  | \
\\  2
\\  | \
\\  3
\\  | \
\\  4  5
\\     |
\\     6
\\     |
\\     7
\\     |
\\     8


Ccount(vec,must=0,k=1) =
{
  \\ print("Ccount "vec" must="must" k="k);
  if(#vec==0,
     return(if(must, 0, 1)));

  my(rest=vec[2..#vec]);
  if(vec[1],
     if(must, Ccount(rest,1,k+1),
        sum(i=1,k, Ccount(rest,1,i+1)));
     ,
     \\ if(must, Ccount(rest,0,k),
     \\    sum(i=1,k, Ccount(rest,0,i)));

     my(i=1,zeros=0); while(i<=#vec && vec[i]==0, zeros++;i++);
     rest=vec[i..#vec];
     \\ print("zeros "zeros" rest "rest);
     if(must,
        zeros--;
        \\ print("decr for must to zeros "zeros);
       );
     \\ print("zeros "zeros);
     if(zeros==0, return(Ccount(rest,0,k)));
     sum(i=1,k,  \\ i many left open, k-i closed
         \\ print(" bin "k-i+zeros-1," ",zeros-1" to "rest);
         binomial(k-i+zeros-1,zeros-1) * Ccount(rest,0,i));
    );
}
check_equal(Ccount([1,0]), 1, "1,0");
check_equal(Ccount([1,1,1,0]), 1, "[1,1,1,0]");
check_equal(Ccount([1,1,1,0,0]), 4, "00");
\\ check_equal(Ccount([1,1,1,0,0,0]), 4);

Ccount=memoize(Ccount);
Ccount([1,0,1,0,1,1,0,1,1,1,1,1,1, 0,0,0,0, 1,0,1,0,1,0,0,0, 1,0,1,1,0,0,0])
Ccount([1,0,1,1,0,1,1,1,1,1,1, 0,0,0,0, 1,0,1,0,1,0,0,0, 1,0,1,1,0,0])



\\ binomial(7,2) == 21
check_equal(footprint_count([1,1,1,1,1, 0,0,0]), 21);
check_equal(footprint_count([1,1,1,1,1, 0,0,0, 1,0]), 21);

comp(vec) = vector(#vec,i, if(i==#vec,vec[i], 1-vec[i]));
{
  my(forest=1,
     prev_counts=[]);
  for(n=2,7,
     my(vpar=vpar_preorder_first(n),
        m=Map());
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vec=vpar_is_childless_vector(vpar),
              count=0);
           if(vec==[0, 1, 0, 1], print(vpar" "vec));
           if(vec==[0,0,0,1, 0,0,0,1], print(vpar" "vec));
           mapisdefined(m,vec,&count);
           mapput(m,vec,count+1));
     \\ print1(#m,",");
     my(M=Mat(m),
       l=List([]));
     for(i=1,matsize(M)[1],
        my(c=comp(M[i,1]),
           cnum=if(mapisdefined(m,c), mapget(m,c)),
           p=cnum*M[i,2]);
        listput(l,p);
        \\ print(M[i,1]"  "M[i,2]"  comp "c"  "cnum" prod "p);

        my(f=M[i,1]); f=apply(b->1-b,f);
        my(fruns=footprint_runs(f),
           fk=#fruns\2);
        print("foot ",f,
              " runs ",fruns,
              " ",footprint_count(f),
              " ",M[i,2]
              " ",Ccount(f)
              "   k="fk" div ",M[i,2]/footprint_binomials(f)
             );
     );
     \\ print(Set(l));

     my(counts=Set(M[,2]));
     \\ print(counts);
     \\ counts==[1..#counts] || error();
     setminus(prev_counts,counts)==[] || error();
     prev_counts=counts;
  );
quit;
}


\\------------------------------------------------------------------------------
\\ Kreweras Lattice Num Edges

\\ 1 ..... c ... n         first partition k terms
\\   |---| |-----|
\\ 1  c-2   n-c+1
\\    any  firsts k-1

\\ n-c+1 >= k-1
\\ c <= n-k+2

S(n) =
{
  n>=0 || error("oops S() n="n);
  sum(k=1,n, SS(n,k));
}
S=memoize(S);

\\ num ordered forests of n vertices of which k many are roots
Ck(n,k) =
{
  n >= 0 || error();
  vpar_INTERNAL_Catalan_triangle(n-1,n-k);
}
check_equal(Ck(1,1), 1);
check_equal(Ck(2,1), 1);
check_equal(Ck(2,2), 1);
check_equal(Ck(3,1), 2);
check_equal(Ck(3,2), 2);
check_equal(Ck(3,3), 1);

SS(n,k) =
{
  n>=0 || error("oops SS() n="n);
  (1 <= k && k <= n) || error("oops SS() n="n" k="k);

  if(k==1, S(n-1),
     (k-1) * Ck(n,k)
     +
     sum(c=2,n-k+2,
         S(c-2) * Ck(n-c+1,k-1)
         + SS(n-c+1,k-1) * vpar_INTERNAL_Catalan_number(c-2)));
}
SS=memoize(SS);

print("S(1) = ",S(1));
print("S(2) = ",S(2));
print("S(3) = ",S(3));
SS(3,1)
SS(3,2)
\\check_equal(SS(3,3), 3*2/2);
my(n=3,k=3); check_equal(n-k+2, 2);
my(n=3,k=2); check_equal(n-k+2, 3);
print("S   ",vector(10,n,S(n)));
print("bin ",vector(10,n, binomial(2*n, n-2)));
my(n=4); vector(n,k,SS(n,k))

\\ n=3 k=2
\\ 12,3 13,2
\\  ^    ^
\\

siblings_sets_sizes(vpar) = apply(length,vpar_siblings_sets(vpar));
setsize_to_num_split(n) = n*(n-1)/2;
setsizes_to_num_split(setsizes) = \
  sum(i=1,#setsizes, setsize_to_num_split(setsizes[i]));
{
  for(n=4,4,
     my(vpar=vpar_preorder_first(n),
        count_by_firstsize=vector(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none,
           my(setsizes=siblings_sets_sizes(vpar),
              splits=setsizes_to_num_split(setsizes));
           if(setsizes[1]==2,
              print(vpar" splits "splits"  sizes "setsizes" sets "vpar_siblings_sets(vpar)));
           count_by_firstsize[setsizes[1]] += splits;
          );
     print(count_by_firstsize);
     );
}

\\ n=4 k=2
\\   c=2       c=3       c=4
\\ 12         1  3     1    4
\\    3..4     2  4     2..3
\\
S(2)
SS(3,1)
SS(4,2)
my(n=4,k=2,c=2); check_equal(k-1 + S(c-2) + SS(n-c+1,k-1),  2);
my(n=4,k=2,c=2); check_equal(k-1 + S(c-2) + SS(n-c+1,k-1),  2);
my(n=4,k=2,c=3); check_equal(k-1 + S(c-2) + SS(n-c+1,k-1),  1);
my(n=4,k=2,c=4); check_equal(k-1 + S(c-2) + SS(n-c+1,k-1),  2);

\\ recurrence_guess(vector(50,n,my(k=n-4); if(k>=1,SS(n,k))));


recurrence_guess(vector(50,n,n--;my(k=n-4); if(k>=1&&k<=n,SS(n,k))));
vector(12,n,n--;my(k=n-4); if(k>=1&&k<=n,SS(n,k)))

\\ factor(  1/4 * n^4 + -1/2 * n^3 + -3/4 * n^2   + 3 * n   + -2)

quit


\\------------------------------------------------------------------------------
\\ Tamari Complementary Pairs

\\ 1,3,11,46,211,1037,5377,29101,
\\ A046996 Baxter permutations: A001181/2.
{
  for(n=2,10,
     my(vpars=vpar_postorder_all(n,1),
        is_childlesses=apply(vpar_is_childless_vector,vpars),
        keys=apply(vec->vec[2..n], is_childlesses),
        comps=apply(vec->apply(s->1-s, vec), keys));
     \\ print(keys);
     my(count=0);
     for(i=1,#vpars,
        for(j=i+1,#vpars,
           count += (keys[i]==comps[j])));
     print1(count,",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Kreweras Lattice by Out Degree

\\ splits sizes occurring, sums of triangulars of partition terms
\\ 0
\\ 0, 1
\\ 0, 1, 3
\\ 0, 1, 2, 3, 6
\\ 0, 1, 2, 3, 4, 6, 10
\\ 0, 1, 2, 3, 4, 6, 7, 10, 15
\\ 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 15, 21
\\ not in OEIS: 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 15, 21

siblings_sets_sizes(vpar) = apply(length,vpar_siblings_sets(vpar));
setsize_to_num_split(n) = n*(n-1)/2;
setsizes_to_num_split(setsizes) = \
  sum(i=1,#setsizes, setsize_to_num_split(setsizes[i]));

{
  for(n=1,7,
     my(vpar=vpar_preorder_first(n),
        count_by_splits=vector(100));
     until((vpar=vpar_preorder_next(vpar,1))=='none,
           my(setsizes=siblings_sets_sizes(vpar),
              splits=setsizes_to_num_split(setsizes));
           print(vpar"  "setsizes"   "splits);
           count_by_splits[splits+1]++;
          );
     while(#count_by_splits && count_by_splits[#count_by_splits]==0,
           count_by_splits=Vec(count_by_splits,#count_by_splits-1));
     print(count_by_splits);

     my(splits_occurring=apply(s->s-1,Vec(select(c->c!=0, count_by_splits, 1))));
     print(splits_occurring);
     print();
     );
  quit;
}


\\------------------------------------------------------------------------------

is_forest(vpar) =
{
  \\ both or neither left,right
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min));
  for(v=1,#vpar,
     if((next_sibling[v] && !first_child[v])
        || (!next_sibling[v] && first_child[v]),
        return(0)));
  1;
}
is_forest(vpar) =
{
  \\ left leaf must right leaf
  \\ not in OEIS: 1,2,5,12,33,98,302,954,3076,10086,33525
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min),
     is_binary_leaf=vector(#vpar,v,
                           !next_sibling[v] && !first_child[v]));
  for(v=1,#vpar,
     if(next_sibling[v]
        && first_child[v]
        && is_binary_leaf[first_child[v]]
        && ! is_binary_leaf[next_sibling[v]],
        return(0)));
  1;
}
is_forest(vpar) =
{
  \\ both or neither left leaf, right leaf
  \\ not in OEIS: 1,2,5,10,24,68,201,598,1808,5580,
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min),
     is_binary_leaf=vector(#vpar,v,
                           !next_sibling[v] && !first_child[v]));
  for(v=1,#vpar,
     if(next_sibling[v]
        && first_child[v]
        && is_binary_leaf[first_child[v]] != is_binary_leaf[next_sibling[v]],
        return(0)));
  1;
}

{
  my(forest=1);
  for(n=1,12,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(got=test_is_Motzkin_forest(vpar));
           count += got;
           if(got && n==7,
              my(b=vpar_to_balanced_binary(vpar));
              print(" "test_to_binary(b));
             );
          );
     print1(count",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_seq_downqueue() == vpar_seq_preorder()
\\ 2^n

{
  my(forest=1);
  for(n=1,8,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(seq1=vpar_seq_downqueue(vpar));
           my(seq2=vpar_seq_preorder(vpar));
           my(same=(seq1==seq2));
           count += same;
           if(same && n==5,
              print(vpar));
          );
     print1(count",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_horizpos_vector()

{
  my(forest=1);
  for(n=1,4,
     my(vpar=vpar_preorder_first(n),
        seen=Map(),
        distinct=1);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vpar=vpar_relabel_postorder(vpar));
           my(vpar=vpar_relabel_seq(vpar,vpar_seq_diagpost(vpar)));

           my(vec=vpar_depths_vector(vpar));
           my(vec=vpar_diagdepths_vector(vpar));
           my(vec=vpar_horizpos_vector(vpar));

           print(vpar"  "vec);
           if(mapisdefined(seen,vec), distinct=0);
           mapput(seen,vec,1);
          );
     print("distinct "distinct);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_horizpos_vector() occurring

preorder_horizpos_possible_nexts(vec,forest=0) =
{
  vec=Vec(vec,#vec+1);
  Vec(select(n->vec[#vec]=n; vpar_is_preorder_horizpos_vector(vec),
             [0..#vec]));
}
preorder_horizpos_opens(vec,forest=0) =
{
  \\ print("vpar_is_preorder_horizpos_vector() "vec" forest="forest);
  vec[1]==0 || return(0);

  my(l=vecmax(vec));
     \\ print(" l="l);

     \\ maximum pos range 0 to n-1 forest, 0 to n-2 tree
  l < max(1,#vec-!forest) || return(0);

     \\ pos[k+2] = next allowed pos of a child of the vertex at depth k
  my(pos=vectorsmall(#vec+1),
     d=1);
  for(i=2-forest,#vec,
        \\ print(" i="i" pos "vec[i]" onto "Vec(pos[1..d]));
     vec[i]>=0 || return(0);
     forstep(j=d,1,-1,
           \\ print("  cf j="j" pos "pos[j]);
        if(vec[i]==pos[j],
              \\ print("  add sibling");
           pos[d=j]=vec[i]+1;
           pos[d++]=vec[i];
           next(2)));
        \\ print(" no predecessor");
     return(0));

  Vec(pos,d);
}

{
  \\ Preorder, vpar_is_preorder_horizpos_vector()
  \\
  \\ forests
  \\ not in OEIS: 1, 2, 4, 8, 17, 39, 95, 241, 629, 1677,
  \\ with last entry >=2
  \\ not in OEIS: 1, 4, 11, 28, 72, 189, 504, 1362,
  \\ 0, 0, 1, 4, 11, 28, 72, 189, 504, 1362,
  \\
  \\ trees, same 1 later
  \\ 1, 1, 2, 4, 8, 17, 39, 95, 241, 629, 1677,
  \\
  \\ [0, 0, 0, 0]
  \\ [0, 0, 0, 1]
  \\ [0, 0, 1, 1]
  \\ [0, 0, 1, 2]
  \\ [0, 1, 1, 1]
  \\ [0, 1, 1, 2]
  \\ [0, 1, 2, 2]
  \\ [0, 1, 2, 3]
  \\
  \\ [0, 0, 0, 0, 0]         1...5      0,0,1,2,1
  \\ [0, 0, 0, 0, 1]         |
  \\ [0, 0, 0, 1, 1]         2...3...4
  \\ [0, 0, 0, 1, 2]
  \\ [0, 0, 1, 1, 1]
  \\ [0, 0, 1, 1, 2]
  \\ [0, 0, 1, 2, 1]
  \\ [0, 0, 1, 2, 2]
  \\ [0, 0, 1, 2, 3]
  \\ [0, 1, 1, 1, 1]
  \\ [0, 1, 1, 1, 2]
  \\ [0, 1, 1, 2, 2]
  \\ [0, 1, 1, 2, 3]
  \\ [0, 1, 2, 2, 2]
  \\ [0, 1, 2, 2, 3]
  \\ [0, 1, 2, 3, 3]
  \\ [0, 1, 2, 3, 4]

  my(forest=1);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vec=vpar_horizpos_vector(vpar));
           \\ vec[#vec]>=2 || next;
           mapput(m,vec,1));
     \\ print1(#m", ");
     m=Mat(m);
     if(1 && n==7,
        for(i=1,matsize(m)[1],
           print(m[i,1]"  -> open "preorder_horizpos_opens(m[i,1],forest))));
     );
  quit;
}
{
  \\ trees
  \\ 1, 1, 3, 12, 60, 360, 2520,
  \\ n!/2
  \\ forests
  \\ 1, 2, 6, 24, 120, 720,
  \\ n!
  \\ trees root=1, same as forests 1 previous
  \\ 1, 1, 2, 6, 24, 120, 720,

  \\ forests n=2
  \\ [0, 0]  [0, 1]
  \\   1      1..2
  \\   |
  \\   2
  \\ 0 before, +1 each  011 012
  \\ 0 lowest           000 010
  \\                    001 101
  \\
  \\ 0 before, +1 each  011 012
  \\ 0 above, shift     000 001
  \\                    010 101
  \\ 01 after 010 011 012
  \\          000 001 101
  \\
  \\ forests n=3
  \\ [0, 0, 0]  [0, 0, 1]  [0, 1, 0]  [0, 1, 1]  [0, 1, 2]  [1, 0, 1]
  \\    1          1..3       1..2       1..2     1..2..3      2..3
  \\    |          |          |             |                     |
  \\    2          2          3             3                     1
  \\    |
  \\    3          1
  \\               |
  \\               2..3
  \\ 0s root and chain down
  \\ 1s chain down from some 0 smaller than some 1
  \\ 2s chain down from some 1 smaller than some 2
  \\
  \\ forests n=4
  \\ [0,0,0,0]  [0,0,1,0]  [0,1,0,0]  [0,1,1,0]  [0,1,2,0]  [1,0,1,0]
  \\ [0,0,0,1]  [0,0,1,1]  [0,1,0,1]  [0,1,1,1]  [0,1,2,1]  [1,0,1,1]
  \\            [0,0,1,2]  [0,1,0,2]  [0,1,1,2]  [0,1,2,2]  [1,0,1,2]
  \\                                             [0,1,2,3]
  \\
  \\ [1,1,0,1]  [1,0,0,1]  [1,0,2,1]             [2,0,1,2]  [0,2,1,2]
  \\                       [1,2,0,1]
  \\
  \\
  \\ [0,0,0,0]  [0,0,1,1]  [0,1,0,2]  [0,1,1,0]  [0,1,2,0]  [1,0,1,0]
  \\ [0,0,0,1]  [0,0,1,2]  [0,1,0,1]  [0,1,1,1]  [0,1,2,1]  [1,0,1,1]
  \\ [0,0,1,0]  [1,0,0,1]
  \\ [0,1,1,2]  [0,1,2,2]  [1,0,1,2]
  \\ [0,1,0,0]                                             [0,1,2,3]
  \\ [0,2,1,2]
  \\ [1,0,2,1]
  \\ [1,1,0,1]
  \\ [1,2,0,1]
  \\ [2,0,1,2]
  \\
  \\ forests counts of pos
  \\ [1]
  \\ [5, 1]
  \\ [34, 13, 1]
  \\ [307, 165, 27, 1]
  \\ [3506, 2341, 581, 51, 1]
  \\ [48729, 37883, 12373, 1763, 93, 1]
  \\ [800738, 697089, 277397, 54622, 4992, 169, 1]
  \\ not in OEIS: 1,5,34,307,3506,48729,800738,
  \\ not in OEIS: 5,13,27,51,93,169,

  my(forest=1); \\ trees or forests
  for(n=1,10,
     my(m=Map(),
        hist=vector(n));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           my(vec=vpar_horizpos_vector(vpar));
           mapput(m,vec,1);
           for(i=1,#vec, hist[vec[i]+1]++);
     );
     \\ print1(hist[1]",");
     \\ if(n>=2,print1(hist[#hist-1]","));
     \\ print(hist);
     \\ print1(#m", ");
     m=Mat(m);
     if(n==4, for(i=1,matsize(m)[1], print(m[i,1])));
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Binary Tree Layout Fits

\\ fit
\\ not in OEIS: 1,2,5,14,41,124,382,1194,3770,
\\
\\ no fit
\\ n = 1 2 3 4 5
\\     0,0,0,0,1,8,47,236,1092,4796,
\\ not in OEIS: 1,8,47,236,1092,4796,
\\      1... 4
\\      |    |
\\      2..3 5

is_fit(vpar) =
{
  \\ all distinct coordinate pairs depths[v],horizpos[v]
  my(depths=vpar_depths_vector(vpar),
     horizpos=vpar_horizpos_vector(vpar),
     m=matrix(#vpar,#vpar));
  for(v=1,#vpar,
     if(m[depths[v]+1,horizpos[v]+1]++ == 2,
        return(0)));
  1;
}
{
  my(forest=1);
  for(n=1,12,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           count += ! is_fit(vpar);
           if(n==5 && ! is_fit(vpar), print(vpar));
          );
     print1(count,",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Isomorphic Halves -- HOG

count = 0;
l=List([]);
try(vpar) =
{
  if(test_is_isomorphic_halves(vpar),
     count++;
     print(vpar);
     listput(l,vpar_to_free(vpar));
    );
  0;
}
read("/tmp/all.g6");
print("count "count);
print();
l=vecsort(l, (x,y)->lex([#x,x],[#y,y]));
for(i=1,#l, print(l[i]));
quit


\\------------------------------------------------------------------------------
\\ vpar_preorder_is_parsed(vpar,vec)

num_parsed(vpar) =
{
  my(ret=0);
  forvec(vec=vector(#vpar+1,i,[1,3]),
        ret += vpar_preorder_is_parsed(vpar,vec));
  ret;
}
num_both_parsed(vpar1,vpar2,m) =
{
  my(ret=0);
  forvec(vec=vector(#vpar1+1,i, if(i==1,[1,1],[1,3])),
        ret += (vpar_preorder_is_parsed(vpar1,vec)
                && vpar_preorder_is_parsed(vpar2,vec));
        if(ret>m, break));
  ret;
}

{
  \\ pairwise
  \\
  \\ num pairs attaining the minimum both parsable
  \\ not in OEIS: 1,4,24,188,1616
  \\ is not A001506 which differs after 188

  my(forest=1);
  for(n=1,6,
     my(m=oo,
        mreps=0);
     my(vpar1=vpar_preorder_first(n));
     until((vpar1=vpar_preorder_next(vpar1,forest))=='none,
           my(vpar2=vpar1);
           while((vpar2=vpar_preorder_next(vpar2,forest))!='none,
                 my(both=num_both_parsed(vpar1,vpar2,m));
                 if(both<m, m=both;mreps=1,
                    both==m, mreps++);
                 \\ print(vpar1" "vpar2"  "both);
                ));
     print("n="n" min "m" reps "mreps);
     );
  quit;
}
{
  my(forest=1);
  for(n=1,6,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           print(vpar"  "num_parsed(vpar)));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_Seidel_adjacency_poly()

\\ Mirko Lepovic, "On Formal Products and the Seidel Spectrum of Graphs"
\\
\\ S = vertex subset
\\ T = V\S = other vertices
\\ Overgraph G_S = new vertex connected to each of S
\\ adjacency: P(G_S) = x*P(G) - <S,S>
\\ Seidel:    P(G_S) = x*P(G) - <S,S> - <T,T> + 2<S,T>
\\ <X,Y> = sum i in X, j in Y  A(i,j)
\\ A(i,j) = matrix xI-A

matsum(m) = sum(i=1,matsize(m)[1], sum(j=1,matsize(m)[2], m[i,j]));
{
  \\ x^(n-2) = triangular numbers
  \\
  \\ x^(n-3)
  \\ -1/3 * n^3 + 4 * n^2 - 35/3 * n + 8 == -1/3 * (n-8)*(n-3)*(n-1)
  \\
  \\ constant term
  \\ not in OEIS: -1,-2,5,-12,-5,42,209,-320,-1081,-330,7261,
  \\
  for(n=1,10,
     my(m=vpar_Seidel_adjacency_matrix(vpar_make_path(n)),
        p=charpoly(m));
     \\ print1(polcoeff(p,n-3),",");
     \\ print(factor(p));

     my(A=x*matid(n)-m,
        sm=vpar_Seidel_adjacency_matrix(vpar_make_path(1)),
        sA=x*matid(1)-sm,   \\ ['x]
        TT=sum(i=1,n-1, sum(j=1,n-1, A[i,j])),
        SS=sum(i=n,n, sum(j=n,n, A[i,j])),
        ST=sum(i=n,n, sum(j=1,n-1, A[i,j])),
        pnext=x*p - SS - TT + 2*ST);
     \\ print(pnext);
     \\ print(" diff "pnext-vpar_Seidel_adjacency_poly(vpar_make_path(n+1)));

     ;
  );
}

{
  for(n=1,6,
     my(m=vpar_adjacency_matrix(vpar_make_path(n)),
        p=charpoly(m));
     my(A=x*matid(n)-m,
        sm=vpar_adjacency_matrix(vpar_make_path(1)),
        SS=sum(i=n,n, sum(j=n,n, A[i,j])),
        pnext=x*p - SS,
        want=vpar_adjacency_poly(vpar_make_path(n+1)));
     print("p = "p"   SS = "SS);
     print("pnext = "pnext);
     print(" want = "want);
     print(" diff "want-pnext);
     print("  sub ", want - p*x);

     ;
  );
}
quit


\\-----------------------------------------------------------------------------

from_adjacency_matrix_rewriting(m) =
{
  my(n=matsize(m)[1],
     vpar=vector(n),
     ret=0);
  forstep(p=n,1,-1,
     forstep(v=n,p+1,-1,
        if(m[p,v]==1,
           my(p=p, v=v);
           until(!v, ret++; my(new_v=vpar[v]); vpar[v]=p; p=v; v=new_v))));
  ret;
}
{
  \\ labelled worst rewriting, triangular number on star reverse
  my(forest=0);
  for(n=0,6,
     my(count=0,
        worst=0,
        worst_vpar);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              m=vpar_adjacency_matrix(vpar),
              r=from_adjacency_matrix_rewriting(m));
           if(r > worst,
              worst=r; worst_vpar=vpar));
     print("worst n="n" "worst" "worst_vpar
           " ",from_adjacency_matrix_rewriting(vpar_adjacency_matrix(worst_vpar)));
     \\ print1(worst",");
  );
}
\\ {
\\   for(n=1,50,
\\      my(total=0,reps=10);
\\      for(rep=1,reps,
\\         vpar=vpar_make_random(n);
\\         m=vpar_adjacency_matrix(vpar);
\\         total += from_adjacency_matrix_rewriting(m));
\\      print("n="n"  "total/reps*1.0/n);
\\   );
\\   quit;
\\ }

test_from_adjacency_matrix_by_neighbours(m) =
{
  my(n=matsize(m)[1],
     vpar=vector(n,i,-1),
     pending=vectorsmall(#vpar),
     upto);
  for(i=1,n,
     if(vpar[i]<0,
        vpar[i]=0;
        pending[upto=1]=i;
        while(upto,
              my(v=pending[upto]); upto--;
              for(c=1,n,
                 if(vpar[c]<0 && m[c,v],     \\ c = child of v
                    vpar[c]=v; pending[upto++]=c)))));
  vpar;
}

n=2000;
reps=1;
\\ vpar=vpar_make_random(n);
\\ vpar=vpar_make_path(n);
vpar=vpar_make_star(n);
vpar=vpar_relabel_reverse(vpar);
m=vpar_adjacency_matrix(vpar);
from_adjacency_matrix_rewriting(m)

print("time");
t1=getabstime();
for(i=1,reps, test_from_adjacency_matrix_by_neighbours(m));
rtime=getabstime()-t1;

t1=getabstime();
for(i=1,reps, vpar_from_adjacency_matrix(m));
ptime=getabstime()-t1;

print(rtime);
print(ptime);
quit


\\-----------------------------------------------------------------------------
\\ Graham and Lovasz, "Distance Matrix Polynomials of Trees", Bell Systems
\\ Technical Journal.

a(vpar) =
{
  if(#vpar==0, return(0));
  if(vpar_is_tree(vpar),
     my(m=vpar_path_lengths_matrix(vpar));
     return(1/#vpar * sum(i=1,#vpar, sum(j=i+1,#vpar, m[i,j]*(2-m[i,j])))));
  my(trees=vpar_component_trees(vpar));
  sum(i=1,#trees, a(trees[i]));
}
b(vpar) =
{
  if(#vpar==0, return(0));
  if(vpar_is_tree(vpar),
     my(m=vpar_path_lengths_matrix(vpar));
     return(4/#vpar * sum(i=1,#vpar, sum(j=i+1,#vpar, 2-m[i,j]))));
  my(trees=vpar_component_trees(vpar));
  sum(i=1,#trees, b(trees[i]));
}
c(vpar) =
{
  if(#vpar==0, return(-4));
  if(vpar_is_tree(vpar),
     return(-4 / #vpar));
  my(trees=vpar_component_trees(vpar));
  sum(i=1,#trees, c(trees[i]) + if(i!=1, 4));
}
check_equal(a([0,1]), 1/2);
check_equal(b([0,1]), 2);
check_equal(c([0,1]), -2);
check_equal(c([]), -4);
check_equal(c([0]), -4);

\\ 2-1 + 2-2 + 2-3 + 2-4 \
\\     + 2-1 + 2-2 + 2-3 \
\\           + 2-1 + 2-2 \
\\                 + 2-1 == 0
{
  check_equal(b([0,1,2,3,4]), 0);
  check_equal(vpar_path_lengths_matrix([0,1,2,3,4]),
              [0, 1, 2, 3, 4;
               1, 0, 1, 2, 3;
               2, 1, 0, 1, 2;
               3, 2, 1, 0, 1;
               4, 3, 2, 1, 0]);
  check_equal(1+2+3+4 + 1+2+3 + 1+2 + 1,  20);
  check_equal(10*2,                       20);

  check_equal(a([0,1,2,3,4]), -2);
}

pi(vpar) =
{
  my(ns=vpar_component_sizes(vpar));
  prod(i=1,#ns, ns[i]);
}
check_equal(pi([]), 1, "pi() empty");
check_equal(pi([0,1]), 2);

nonroot_vetices(vpar) = Vec(select(p->p, vpar, 1));
check_equal(nonroot_vetices([0,1,2]), [2,3]);

N_free_subedges(vpar,sub) =
{
  #vpar==#sub || error();
  my(d=vpar_num_edges(vpar)-vpar_num_edges(sub));
  if(d<0, return(0));
  my(nonroots=nonroot_vetices(vpar),
     ret=0);
  forsubset([#nonroots,d],del_indices,
            if(vpar_is_free_isomorphic
               (vpar_delete_edges(vpar,vecextract(nonroots,del_indices)),
                sub),
               ret++));
  ret;
}

\\ number of subgraphs of vpar which are isomorphic to F
N(vpar,F) =
{
  vpar_num_singletons(F)==0 || error();
  my(ret=0);
  forsubset([#vpar,#F],keep,
            ret += N_free_subedges(vpar_keep_vertices(vpar,keep), F));
  ret;
}
check_equal(N([0,1,2],[]), 1, "N() empty");
check_equal(N([0,1,2],[0,1]), 2);
check_equal(N([0,1,2],[0,1,1]), 1);
check_equal(N([0,1],[0,1]), 1);

\\ free forests exactly k edges, <=n vertices, and no singletons
F_list(k,n_limit=2*k) =
{
  my(ret=List([]));
  for(n=0,min(2*k,n_limit),
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,1))=='none,
           if(vpar_num_edges(vpar)==k && vpar_num_singletons(vpar)==0,
              listput(ret,vpar))));
  Vec(ret);
}
F_list=memoize(F_list);
check_equal(F_list(0), [ [] ]);
check_equal(F_list(1), [ [0,1] ]);
check_equal(F_list(2), [[0,1,1], [0,1,0,3]]);
check_equal(#F_list(4), 8);
\\ vector(8,k,#F_list(2*k,k))
\\ [1, 2, 4, 8, 16, 34, 71, 154]
\\ A215930 free forests n edges no singletons

a_sum(vpar,k) =
{
  my(l=F_list(k+1,#vpar));
  sum(i=1,#l, a(l[i]) * pi(l[i]) * N(vpar,l[i]));
}
check_equal(a_sum([0,1],2), 0, "a_sum() path-2");

b_sum(vpar,k) =
{
  my(l=F_list(k,#vpar));
  sum(i=1,#l, b(l[i]) * pi(l[i]) * N(vpar,l[i]));
}
check_equal(b_sum([0,1],2), 0, "b_sum() path-2");

c_sum(vpar,k) =
{
  my(l=F_list(k-1,#vpar));
  sum(i=1,#l, c(l[i]) * pi(l[i]) * N(vpar,l[i]));
}
check_equal(c_sum([0,1],2), -4, "c_sum() path-2");

path_lengths_poly_coeff(vpar,k) =
{
  \\ not (-1)^(n-1) with charpoly() style matdet(xI-M)

  my(n=#vpar);
  - 2^(n-k-2)
  * (a_sum(vpar,k) + b_sum(vpar,k) + c_sum(vpar,k));
}
path_lengths_poly(vpar) =
{
  sum(k=0,#vpar, path_lengths_poly_coeff(vpar,k) * x^k);
}
check_equal(vpar_path_lengths_poly([0]), 'x, "vpar_path_lengths_poly()");
check_equal(path_lengths_poly     ([0]), 'x);
check_equal(vpar_path_lengths_poly([0,1]), x^2 - 1, "vpar_path_lengths_poly()");
check_equal(path_lengths_poly     ([0,1]), x^2 - 1);
{
  my(vpar=[0, 1, 2, 1, 4]);  \\ path-5
  \\ x^5 + 2*x^4 - 22*x^3 - 126*x^2 - 120*x - 32
  \\ x^5         - 50*x^3 - 140*x^2 - 120*x - 32
  my(n=#vpar,
     k=2);
  check_equal(a_sum(vpar,k), 14);
  check_equal(b_sum(vpar,k), 72);
  check_equal(c_sum(vpar,k), -16);
  check_equal(7 + 72 - 16, 63);
  check_equal(2^(n-k-2), 2);
  check_equal(7 + 72 - 16, 63);
  check_equal(path_lengths_poly_coeff(vpar,k), -140);

  my(k=4);
  check_equal(a_sum(vpar,k), 0, "a_sum k=4");
  check_equal(b_sum(vpar,k), 0);
  check_equal(c_sum(vpar,k), 0, "c_sum k=4");
  check_equal(a_sum(vpar,k) + b_sum(vpar,k) + c_sum(vpar,k), 0);
  check_equal(2^(n-k-2), 1/2);
  check_equal(path_lengths_poly_coeff(vpar,k), 0);

  \\ c: k=3 edges, omit one
  check_equal(apply(F->c(F),     F_list(k-1)), [-1, -1, 2/3,  2]);
  check_equal(apply(F->N(vpar,F), F_list(k-1)), [2,  0,   2,  0]);
  check_equal(apply(F->pi(F),     F_list(k-1)), [4,  4,   6,  8]);
  check_equal(F_list(k-1)[1], [0, 1, 2, 1]);  \\ path-4
  check_equal(F_list(k-1)[3], [0, 1, 1, 0, 4]);  \\ path-3 and path-2
  check_equal(c([0, 1, 2, 1]), -1);
  check_equal(c([0, 1, 1]), -4/3);
  check_equal(c([0, 1]),    -4/2, "c path-2");
  check_equal(c([0, 1, 1, 0, 4]), -4/3 + -4/2 + 4);
  check_equal(-4/3 + -4/2 + 4, 2/3);
  check_equal(apply(length, F_list(k-1)), [4, 4, 5, 6]);
  check_equal(4*-1, -4);
  check_equal(6*2/3, 4);
  check_equal(N([0,1,2,3,4],[0,1,2,3]), 2);
  check_equal(N([0,1,2,3,4],[0,1,2,0,4]), 2);
  \\ 1---2---3---4---5
  \\ 1---2---3   4---5
  \\

  \\ b: k=4 edges, only self as subgraph
  check_equal(apply(F->N(vpar,F), F_list(k)),   [1, 0,0,0,0,0,0,0]);
  check_equal(F_list(k)[1],                     vpar, "F_list(4)[1] path-5");
  check_equal(b(F_list(k)[1]),                  0);

  \\ a: none with k+1=5 edges
  check_equal(apply(F->N(vpar,F), F_list(k+1)), vector(16));
}

{
  for(n=1,10,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar))=='none,
           \\ print(vpar);
           \\ print("want "vpar_path_lengths_poly(vpar));
           \\ print("got  "path_lengths_poly(vpar));
           check_equal(path_lengths_poly(vpar),
                       vpar_path_lengths_poly(vpar));
          ));
}
quit


check_equal(vector(10,n, c(vpar_make_path(n))), []);
check_equal(vector(10,n, b(vpar_make_path(n))), []);
check_equal(vector(10,n, a(vpar_make_path(n))), []);


\\-----------------------------------------------------------------------------
\\ real roots

l=List([]);
{
  forvec(vec=vector(6,i,[0,2]),
        my(p=Pol(vec));
        if(test_poly_is_all_real_roots(p), listput(l,p)));
}
#l
{
  for(i=1,#l,
     for(j=1,#l,
        my(p=l[i]+l[j]);
        if(!test_poly_is_all_real_roots(p),
           print(p))));
}
quit


\\-----------------------------------------------------------------------------
\\ uniqueness of polys / cospectral

\\ free trees distinct charpolys,
\\ same as distinct matchpolys
\\ 1, 1, 1, 2, 3, 6, 11, 22, 42, 102, 204, 488, 1078, 2723,
\\ = A226594
\\
\\ free trees distinct indpoly
\\              1, 1, 1, 2, 3, 6, 11, 22, 46, 93, 213, 475, 1102, 2583, 6283, 15443,
\\ not in OEIS:          2, 3, 6, 11, 22, 46, 93, 213, 475, 1102, 2583, 6283, 15443,
\\
\\ free trees distinct dompoly
\\              1, 1, 1, 2, 3, 6, 10, 18, 35, 68, 135, 278, 581, 1251, 2749,
\\ not in OEIS:          2, 3, 6, 10, 18, 35, 68, 135, 278, 581, 1251, 2749,
\\
\\ free trees distinct Laplacian poly
\\              1, 1, 1, 2, 3, 6, 11, 23, 47, 106, 232, 548, 1292, 3144, 7717,
\\ not in OEIS:          2, 3, 6, 11, 23, 47, 106, 232, 548, 1292, 3144, 7717,
\\
\\ free trees distinct path lengths poly
\\ n=1
\\   1, 1, 1, 2, 3, 6, 11, 23, 47, 106, 235, 551, 1301, 3159, 7741, 19320,
\\ all trees different, cf van Dam and Haemers
\\ (Graham and Lovasz had as unknown yet)
\\ vector(10,n, vpar_count_free(n))

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        m=Map());
     until((vpar=vpar_free_next(vpar))=='none,
           \\ my(poly=vpar_matchpoly(vpar));
           my(poly=vpar_path_lengths_poly(vpar));
           \\ my(poly=vpar_perfect_dompoly(vpar));
           my(count); mapisdefined(m,poly,&count);
           count++;
           mapput(m,poly,count));
     m=Mat(m);  \\ m[i,1] = poly, m[i,2]=how many such
     my(polys=m[,1]~,
        counts=m[,2]~,
        unique=sum(i=1,#counts, counts[i]==1),
        dups=sum(i=1,#counts, counts[i]!=1),
        pairs=sum(i=1,#counts, counts[i]==2),
        triples=sum(i=1,#counts, counts[i]==3));
     \\ print("n="n" counts "counts" total "vecsum(counts));
     \\ print("n="n" polys "polys);
     \\ print("n="n" all real roots "apply(test_poly_is_all_real_roots,polys));
     \\ print1(unique,", ");
     \\ print1(dups,", ");
     \\ print1(pairs,", ");
     print1(#counts,", ");
     \\ print("n="n"  "unique*1.0/vpar_count_free(n));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_maximal_matchpoly() matrix

triangle_to_matrix(vec) =
{
  my(l=List([]),
     pos=1,
     w=1);
  for(w=1,#vec,
     \\ print("w="w" take "pos" .. "pos+w-1);
     listput(l,vec[pos..pos+w-1]);
     pos += w;
     if(pos==#vec+1,break));
  my(m=matrix(#l,#l));
  for(i=1,#l,
     for(j=1,#l[i],
        m[i,j]=l[i][j];
        m[j,i]=l[i][j]));
  m;
}
check_equal(triangle_to_matrix([9]), Mat(9));
check_equal(triangle_to_matrix([1,2,3]), [1,2; 2,3]);

{
  my(forest=1);
  for(n=2,2,
     my(vpar=vpar_free_first(n),
       count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(poly=vpar_maximal_matchpoly(vpar));
           print(vpar"  "poly);
           forvec(vec=vector(n*(n+1)/2,i,[-n+1,n+1]),
                 my(m=triangle_to_matrix(vec));
                 matsize(m)==[n,n] || error();
                 print("  "m"  "charpoly(m));
                 if(charpoly(m)==poly,
                    print(m))));
  );
  quit;
}
quit


\\------------------------------------------------------------------------------
\\ vpar_domsets_count() Total

{
  \\ functional digraphs total domsets
  \\ not in OEIS: 1,2,19,232,4037,89136,
  \\
  \\ rootless
  \\ not in OEIS: 1,1,10,117,1992,43425,

  my(rootless=0);
  for(n=0,8,
     my(count=0);
     forvec(vpar=vector(n,i,[rootless,n]),
           forvec(flags=vector(n,i,[0,1]),
                 count += vpar_is_domset_flags(vpar,flags)));
     print1(count",");
  );
  quit;
}
{
  \\ labelled total domsets
  \\ trees
  \\ not in OEIS: 1,1,6,45,576,10025,222048,
  \\
  \\ forests
  \\ not in OEIS: 1,1,7,64,901,16656,384979,
  \\
  \\ root=1
  \\ not in OEIS: 1,1,3,15,144,2005,37008,854315,

  my(forest=1);
  for(n=0,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count += vpar_domsets_count(vpar));
     print1(count",");
  );
  quit;
}
{
  \\ free total domsets
  \\ free trees
  \\ not in OEIS: 1,1,3,5,18,49,172,567,2083,7577,30192,118715,492767,2063407,
  \\
  \\ free forests
  \\ not in OEIS: 1,1,4,9,36,100,378,1227,4632,16623,65013,251190,1021819,

  my(forest=1);
  for(n=0,20,
     my(vpar=vpar_free_first(n),
       count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           count += vpar_domsets_count(vpar));
     print1(count",");
  );
  quit;
}
{
  \\ unlabelled
  \\ trees
  \\ not in OEIS: 1,1,3,10,36,145,570,2438,10337,45672,203545,925418,4247046,

  \\ forests
  \\ not in OEIS: 1,1,4,14,59,234,1014,4337,19274,86335,393976,1813340,

  my(forest=0);
  for(n=0,20,
     my(vpar=vpar_unlabelled_first(n),
       count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count += vpar_domsets_count(vpar));
     print1(count",");
  );
  quit;
}
{
  \\ ordered
  \\ trees
  \\ not in OEIS: 1,1,3,10,45,222,1176,6516,37323,219190,1312740,7987020
  \\
  \\ forests
  \\ not in OEIS: 1,1,4,17,84,442,2442,13953,81796,489182,2972892,

  my(forest=1);
  for(n=0,20,
     my(vpar=vpar_preorder_first(n),
       count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           count += vpar_domsets_count(vpar));
     print1(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_indsets_count() Total

{
  \\ ordered
  \\ trees
  \\ 1,2,3,10,42,198,1001,5304,29070,163438,937365,5462730,
  \\ A007226 a(n+2) = 2*C(3n, n) - C(3n, n+1)
  \\
  \\ forests
  \\ 1,2,7,30,143,728,3876,21318,120175,690690,4032015,23841480,
  \\ A006013 binomial(3*n+1,n)/(n+1)
  \\   pairs of ternary trees

  my(forest=0);
  for(n=0,20,
     my(vpar=vpar_preorder_first(n),
       count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           count += vpar_indsets_count(vpar));
     print1(count",");
  );
  quit;
}
{
  \\ functional digraphs
  \\ not in OEIS: 1,3,22,257,4136,84937,
  \\ vector(8,n,n--; sum(k=0,n, binomial(n,k)*(k+1)^(n-k)*(n+1)^k))
  \\
  \\ rootless
  \\ n=0 1 2  3    4     5      6        7
  \\   1,1,8,90,1424,28900,716292,20972098,
  \\ A295623 a(n) = n! * [x^n] exp(n*x*exp(x)).
  \\ vector(8,n,n--; sum(k=0,n, binomial(n,k)*k^(n-k)*n^k))

  my(rootless=1);
  for(n=0,8,
     my(count=0);
     forvec(vpar=vector(n,i,[rootless,n]),
           forvec(flags=vector(n,i,[0,1]),
                 count += vpar_is_indset_flags(vpar,flags)));
     print1(count",");
  );
  quit;
}
{
  \\ labelled
  \\ trees
  \\ not in OEIS: 1,2,6,45,528,8525,175428,
  \\
  \\ forests
  \\ 1,2,10,89,1156,19897,428002,
  \\ A055779 num "fat" trees
  \\ a(n) = Sum_{k=1..n} binomial(n,k) * k^(n-k) * n^(k-2)  , Vladeta Jovovic
  \\ fat = vertex set V, partition of V
  \\       edges between vertices link the partition tree-like
  \\
  \\ root=1
  \\ not in OEIS: 1,2,3,15,132,1705,29238,627305,


  my(forest=-1);
  for(n=0,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count += vpar_indsets_count(vpar));
     print1(count",");
  );
  quit;
}
{
  \\ free trees
  \\ not in OEIS: 1,2,3,5,17,44,149,463,1629,5597,21096,78367,307226,1213278,4926608,20194351,
  \\
  \\ free forests
  \\ not in OEIS: 1,2,7,19,64,187,626,1977,6776,22942,82014,295322,1105863,4202566,16412311,

  my(forest=0);
  for(n=0,20,
     my(vpar=vpar_free_first(n),
       count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           count += vpar_indsets_count(vpar));
     print1(count",");
  );
  quit;
}
{
  \\ unlabelled
  \\ trees
  \\ not in OEIS: 1,2,3,10,34,129,484,1947,7829,32611,137310,589015,2551751,11192582,

  \\ forests
  \\ not in OEIS: 1,2,7,24,91,341,1370,5504,22914,96457,413673,1792077,7860331,

  my(forest=1);
  for(n=0,20,
     my(vpar=vpar_unlabelled_first(n),
       count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count += vpar_indsets_count(vpar));
     print1(count",");
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_lower_matchnum() vs vpar_indnum()

\\ indnum >= n-2*lower matchnum, since lower matchnum leaves an indset

\\ free trees
\\ lower matchnum == 2n-indnum, so leave a maximum indset
\\ not in OEIS: 1,1,1,2,3,5,7,12,21,39,69,130,
\\
\\ free trees
\\ lower matchnum != 2n-indnum,
\\ not in OEIS: 0,1,2,3,5,9,18,36,74,162,367,857,

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        m=Map());
     until((vpar=vpar_free_next(vpar))=='none,
           my(indnum=vpar_indnum(vpar),
              lower_matchnum=vpar_lower_matchnum(vpar));
           indnum >= n-2*lower_matchnum || error();
           count += (indnum == n-2*lower_matchnum));
     print1(count",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_perfect_dompoly() roots

{
  for(n=1,7,
     print("n="n);
     my(vpar=vpar_free_first(n),
        m=Map());
     until((vpar=vpar_free_next(vpar))=='none,
           my(poly=vpar_perfect_dompoly(vpar));
           mapisdefined(m,poly,&count);
           count++;
           mapput(m,poly,count));
     m=Mat(m);  \\ m[i,1] = poly, m[i,2]=how many such
     my(polys=m[,1]~,
        counts=m[,2]~);
     for(i=1,#polys,
        print(factor(polys[i])));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_preorder_depths_to_postorder() is reversal

\\ preorder forests which are the same in mirror image
\\
\\ 1,2,4,7,13,23,43,78,148,274,526,988,1912,
\\ A036256 a(n) = Sum_{i=0..n} binomial(i,floor(i/2)).
\\ A036256(n) = sum(i=0,n, binomial(i,floor(i/2)));
\\ vector(20,n, A036256(n))
\\ f(n) = sum(h=0,n\2, (n-2*h+1)*vpar_count_ordered(h,1));
\\ vector(20,n, f(n))

{
  my(forest=1);
  for(n=0,20,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(depths=vpar_depths_vector(vpar),
              postorder_depths=vpar_preorder_depths_to_postorder(depths),
              is_rev=(depths==Vecrev(postorder_depths)),
              rev_vpar=vpar_relabel_reverse(vpar),
              is_rel=(vpar==vpar_relabel_preorder(rev_vpar))
             );
           \\ if(n<=6 && is_rev, print(vpar" "depths));
           \\ count += is_rev;
           count += is_rel;
          );
     print1(count",");
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ Pre-Order Depths Vectors Partial Ordering, Siblings Split

siblings_matrix(vpar) =
{
  my(setnum=vpar_siblings_setnum_vector(vpar));
  matrix(#vpar,#vpar,u,v, setnum[u]==setnum[v]);
}

n=4
vpars=Vecrev(vpar_preorder_all(n,1))  \\ forests
sms=apply(siblings_matrix,vpars)
matrix_is_subset(m,sub) =
{
  for(i=1,matsize(m)[1],
     for(j=1,matsize(m)[2],
        if(!(m[i,j] >= sub[i,j]),
           return(0))));
  1;
}

to_str(vec) = concat(apply(n->Str(n),vec));
\\ ord=matrix(#vpars,#vpars,i,j, i!=j && matrix_is_subset(sms[i],sms[j]));
ord=matrix(#vpars,#vpars,i,j, i!=j && vpar_is_siblings_split(vpars[i],vpars[j]));
{
  my(more=0);
  until(!more,
        print("ord "ord);
        more=0;
        for(i=1,#vpars,
           for(j=1,#vpars,
              if(ord[i,j],
                 for(k=1,#vpars,
                    if(ord[j,k] && ord[i,k],
                       print("  transitive "i" "j" "k);
                       ord[i,k]=0;
                       more=1))))));
}
print("ord");
ord
{
  for(i=1,#vpars,
     print("i="i);
     for(j=1,#vpars,
        if(ord[i,j],
           my(bi=vpar_to_balanced_binary(vpars[i]),
              bj=vpar_to_balanced_binary(vpars[j]));
           print("['"to_str(vpars[i])"','"to_str(vpars[j])"'],");
           print("      "test_to_binary(bi));
           print("      "test_to_binary(bj));
           printf("  xor %*d\n", 2*n, test_to_binary(bitxor(bi,bj))))));
}
quit

{
  for(i=1,#vpars,
     print("i="i);
     for(j=1,#vpars,
        if(ord[i,j],
           my(bi=vpar_to_balanced_binary(vpars[i]),
              bj=vpar_to_balanced_binary(vpars[j]));
           print("['"to_str(vpars[i])"','"to_str(vpars[j])"'],"
                 "    # "vpars[j]-vpars[i]" "vecsum(vpars[j]-vpars[i])
                 "  "test_to_binary(bi)" "test_to_binary(bj)" xor "test_to_binary(bitxor(bi,bj))))));
}


\\-----------------------------------------------------------------------------
\\ vpar_preorder_all()

{
  my(forest=1);
  forstep(n=1,5,1,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           print(vpar"  "vpar_depths_vector(vpar)));
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Preorder Sizes One Change

{
  my(forest=1);
  for(n=4,4,
     my(vpars=vpar_postorder_all(n),
        sizes=apply(vpar_subtree_sizes_vector,vpars),
        pre_vpars=apply(vpar_relabel_preorder,vpars),
        pre_sizes=apply(vpar_subtree_sizes_vector,pre_vpars),

        sizes_to_i=((s)->test_vector_search(sizes,s)));

     for(i=1,#vpars,
        my(vpar=vpars[i],
           sizes=sizes[i]);
        print(vpar" "sizes" pre "pre_vpars[i]" sizes "pre_sizes[i]);

        for(pos=1,#sizes,
           for(s=0,1,
              for(o=1,n,
                 my(delta=o*(-1)^s,
                    new_sizes=sizes);
                 new_sizes[pos] += delta;
                 my(j=sizes_to_i(new_sizes));
                 if(!j, next);
                 print(" pos="pos" delta "delta" to "new_sizes);
                 print("  pre "pre_sizes[i]);
                 print("  to  "pre_sizes[j]
                       " diff "pre_sizes[i] - pre_sizes[j]);
                 next(2))));
        print();
        );
     );
  quit;
}
{
  my(forest=1);
  for(n=4,4,
     my(vpars=vpar_preorder_all(n),
        sizes=apply(vpar_subtree_sizes_vector,vpars),
        post_vpars=apply(vpar_relabel_postorder,vpars),
        post_sizes=apply(vpar_subtree_sizes_vector,post_vpars),

        sizes_to_i=((s)->test_vector_search(sizes,s)));

     for(i=1,#vpars,
        my(vpar=vpars[i],
           sizes=sizes[i]);
        print(vpar" "sizes" post "post_vpars[i]" sizes "post_sizes[i]);

        for(pos=1,#sizes,
           for(s=0,1,
              for(o=1,n,
                 my(delta=o*(-1)^s,
                    new_sizes=sizes);
                 new_sizes[pos] += delta;
                 my(j=sizes_to_i(new_sizes));
                 if(!j, next);
                 print(" pos="pos" delta "delta" to "new_sizes);
                 print("  post "post_sizes[i]);
                 print("  to   "post_sizes[j]
                       " diff "post_sizes[i] - post_sizes[j]);
                 next(2))));
        print();
        );
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_make_random_preorder() steps gcd

rand_of_dk(d,k) = (d+2*k)*(d+1);
comp_of_dk(d,k) = (d+2)*k;
comp2_of_dk(d,k) = (d+k+1)*d;
check_equal(rand_of_dk('d,'k), comp_of_dk('d,'k) + comp2_of_dk('d,'k));

{
  for(d=1,10,
     print();
     for(k=1,10,
        my(c=comp2_of_dk(d,k),
           r=rand_of_dk(d,k),
           g=gcd(r,c));
        print("d="d" k="k"  r="r" c="c"  gcd "g" div to "r/g" "c/g)));
}
{
  for(n=1,10,
     for(k=1,n,
        my(d=n-k);
        my(c=comp2_of_dk(d,k),
           r=rand_of_dk(d,k),
           g=gcd(r,c));
        print1(g","));
     print());
}
quit


\\-----------------------------------------------------------------------------
\\ Equal 0,1 to Balanced

\\ M.D. Atkinson and J.R. Sack, "Generating Binary Trees At Random",
\\ Information Processing Letters, volume 41, 1992, pages 21-23
\\ http://www.cs.otago.ac.nz/staffpriv/mike/Papers/RandomGeneration/RandomBinaryTrees.pdf
\\

\\ vec is a vector of 0s and 1s with the same number of 0s as 1s
\\ return them rearranged to balanced binary open/closes
equals_to_balanced(vec) =
{
  my(s=1,
     d=0,
     ret=[],
     end=[]);
  for(i=1,#vec,
     d += if(vec[i],1,-1);
     if(d==0,
        if(vec[s],
           ret=concat(ret,vec[s..i]);
           ,
           for(j=s,i, vec[j]=1-vec[j]);
           ret=concat(ret,[1]);
           end=concat([0, vec[s+1..i-1], end]);
           );
        s=i+1));
  d==0 || error;
  concat(ret,end);
}
check_equal(equals_to_balanced([1,1,0,0]), [1,1,0,0]);
check_equal(equals_to_balanced([0,1,0,1]), [1, 1,0, 0]);
check_equal(equals_to_balanced([0,0,1,1]), [1, 0, 1,0]);
check_equal(equals_to_balanced([0,0,0,1,1,1,1,0]), [1, 1,0, 0, 1,1,0,0]);

all_distinct_perms(vec) =
{
  my(m=Map());
  for(i=1,(#vec)!,
     mapput(m, vecextract(vec,numtoperm(#vec,i)), 1));
  (Mat(m)[,1])~;
}
bits_list(ones,zeros) =
{
  if(ones==0 && zeros==0, [ [] ],
  concat
  (if(ones>0,  apply(elem->concat(1,elem), bits_list(ones-1,zeros)), []),
   if(zeros>0, apply(elem->concat(0,elem), bits_list(ones,zeros-1)), [])));
}
equals_list(n) = bits_list(n,n);

my(n=2); all_distinct_perms(vector(2*n,i,i<=n))
equals_list(2)
check_equal(vector(2,n, vecsort(equals_list(n))), \
            vector(2,n, vecsort(all_distinct_perms(vector(2*n,i,i<=n)))))
check_equal(vector(5,n, binomial(2*n,n)), \
            vector(5,n, #equals_list(n)));


e = equals_list(5);
b = apply(equals_to_balanced,e);
{
  my(m=Map());
  for(i=1,#b,
     my(c); mapisdefined(m,b[i],&c);
     mapput(m,b[i],c+1));
  print(Mat(m)[,2]);
}

quit


\\-----------------------------------------------------------------------------
\\ vpar_make_random_preorder_by_steps()

my_random(n)=random(n);
{
  for(i=1,6,
     my(vpar=vpar_make_random_preorder_by_steps(6));
     print(vpar"  "vpar_is_preorder(vpar));
  );
}
{
  my(forest=1,
     n=6);
  my(vpars=Vecrev(vpar_preorder_all(n,forest)),
     counts=vector(#vpars));
  print("count "#vpars);
  vpars==Set(vpars) || error();
  for(i=1,1000,
     my(vpar=vpar_make_random_preorder_by_steps(n,forest));
     vpar_validate(vpar);
     vpar_is_preorder(vpar) || error(vpar);
     my(pos=setsearch(vpars,vpar));
     pos || error("not found: "vpar);
     counts[pos]++);
  print(counts);
}

limits=List([]);
uptos=List([]);
this_limits=List([]);

my_random(n) =
{
  print(n);
  return(n-1);
  return(random(n));
  return(0);
  listput(this_limits,n);
  my(key=Vec(this_limits),
     pos=setsearch(limits,key));
  if(!pos,
     pos=setsearch(limits,key,1);
     listinsert(limits,key,pos);
     listinsert(uptos,-1,pos));
  my(r = uptos[pos]++);
  if(r>=n,
     error("oops"));
  r;
}
{
  my(n=6,
    forest=1);
  print("C="vpar_INTERNAL_Catalan_number(6));
  for(i=1,1,
     t=1;
     my(vpar=vpar_make_random_preorder_by_steps(n,forest));
     print(vpar);
     print(t);
  );

}
quit


\\-----------------------------------------------------------------------------
\\ Random Preorder

r(n) =
{
  my(p=n,
     q=n,
     m=1,
     ret=vector(2*n));
  while(q,
        if(random((q+p)*(q-p+1)) < (q+1)*(q-p),
           q--; ret[m]=0; m++;
           ,
           p--; ret[m]=1; m++));
  ret;
}
r(n) =
{
  my(p=n,
     q=n,
     upto=0,
     ret=vector(2*n));
  while(q,
        if(ret[upto++] = (random((q+p)*(q-p+1)) >= (q+1)*(q-p)),
           p--, q--));
  ret;
}
r(n) =
{
  my(p=n,
     upto=0,
     ret=vector(2*n));
  forstep(q=n,1,-1,
     while(ret[upto++] = (random((q+p)*(q-p+1)) >= (q+1)*(q-p)), p--));
  my(b=fromdigits(ret,2));
  vpar_is_balanced_binary(b) || error();
  ret=vpar_from_balanced_binary(b);
  ret=vpar_depths_vector(ret);
  ret;
}
r(n) =
{
  my(d=0,
     p=n,
     upto=0,
     ret=vector(2*n),
     depths=vector(n),
     u=0);
  for(i=1,n,
     my(q=n+1-i);
     d==n-p-i+1 || error();
     \\ p==-d+q || error();
     \\ p==-d+1+n-i || error();
     while(ret[upto++] = (random((q+p)*(q-p+1)) >= (q+1)*(q-p)),
           depths[u++]=d;
           d++;
           p--);
     d--);

  \\ for(i=1,n,
  \\    d++;
  \\    while(ret[upto++] = (random((2*n-2*i+d+1)*(2-d)) >= (n-i+2)*(1-d)),
  \\          d--));

  my(b=fromdigits(ret,2));
  vpar_is_balanced_binary(b) || error();
  ret=vpar_from_balanced_binary(b);
  ret=vpar_depths_vector(ret);
  print(ret"  "depths);
  depths==ret || error();
  ret;
}

r(n) =
{
  my(p=n,   \\ how many 1s remaining
     q=n,   \\ how many 0s remaining
     d=0,
     m=1,
     depths=vector(n),
     u=0,
     ret=vector(2*n));
  while(p,
        d==q-p || error();
        q==d+p || error();
        if(random((q+p)*(q-p+1)) >= (q+1)*(q-p),
           p--; ret[m]=1; m++;
           depths[u++]=d;
           d++;
           ,
           q--; ret[m]=0; m++;
           d--;
          ));
  ret;

  my(b=fromdigits(ret,2));
  vpar_is_balanced_binary(b) || error();
  ret=vpar_from_balanced_binary(b);
  ret=vpar_depths_vector(ret);
  print(ret"  "depths);
  depths==ret || error();
  ret;
}

r(n) =
{
  \\ (d+2*x)*(d+1) - (d+x+1)*d == (d+2)*x
  \\
  my(q=n,   \\ how many 0s remaining
     d=-1,
     m=1,
     depths=vector(n),
     u=0,
     ret=vector(2*n));
  for(i=1,n,
     d++;
     my(p=n+1-i);  \\ how many 1s remaining
     d==q-p || error();
     q==d+p || error();
     \\ while(random((q+p)*(q-p+1)) < (q+1)*(q-p),
     my(x=n-i+1);
     x==p || error();
     \\ while(random((d+2*x)*(d+1)) < (d+x+1)*d,
     while(random((d+2*p)*(d+1)) >= (d+2)*p,
           ret[m]=0; m++;
           q--;
           d--);
     ret[m]=1; m++;
     \\ depths[u++]=d;
     depths[i]=d;
  );

  my(b=fromdigits(ret,2));
  vpar_is_balanced_binary(b) || error();
  ret=vpar_from_balanced_binary(b);
  ret=vpar_depths_vector(ret);
  \\ print(ret"  "depths);
  depths==ret || error();
  ret;
}

r=vpar_make_random_preorder_by_steps;
{
  for(i=1,6,
     my(vec=r(6));
     print(vec
           \\ "   "vpar_is_balanced_binary(fromdigits(vec,2))
           "  "vpar_is_preorder_depths_vector(vec,1));
  );
}
{
  my(n=6);
  my(vpars=Vecrev(vpar_preorder_all(n,1)),
     depths=apply(vpar_depths_vector,vpars),
     counts=vector(#depths));
  print("count "#vpars);
  depths==Set(depths) || error();
  for(i=1,1000,
     my(vec=r(n),
        pos=setsearch(depths,vec));
        counts[pos]++);
  print(counts);
}

{
  my(n=1000,
     reps=20);
  my(t1=getabstime());
  for(i=1,reps, r(n));
  my(rtime=getabstime()-t1);

  my(t1=getabstime());
  for(i=1,reps, vpar_make_random_preorder(n,1));
  my(ptime=getabstime()-t1);

  print(rtime);
  print(ptime);
}

quit


\\-----------------------------------------------------------------------------
\\ vpar_transpose() Ordered Unchanged
\\
\\ vpar_is_isomorphic([0,1,0], vpar_transpose([0,1,0]))

{
  \\ labelled rooted forests
  \\ never identical
  \\
  \\ isomorphic (n even)
  \\ 1, 3, 50, 2100,
  \\
  \\ free isomorphic (n even)
  \\ not in OEIS: 1, 3, 50, 2100,
  \\
  \\ ordered isomorphic (n even)
  \\ not in OEIS: 1, 2, 27, 508,

  my(forest=1); \\ trees or forests
  forstep(n=1,9,2,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              flip=vpar_transpose(vpar));
           \\ print(vpar"  "flip);
           \\ count += (flip==vpar);
           \\ count += vpar_is_isomorphic(vpar,flip);
           count += vpar_is_free_isomorphic(vpar,flip);
           \\ count += vpar_is_ordered_isomorphic(vpar,flip);
     );
     print1(count", ");
  );
  quit;
}
{
  \\ ordered forests
  \\ 1,0,1,0,2,0,5,0,14,0,42,
  \\ is left,right binary trees mirror images, so Catalan many

  my(forest=1);
  forstep(n=1,20,1,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(flip=vpar_transpose(vpar));
           \\ flip=vpar_relabel_preorder(flip);
           \\ count += (flip==vpar);
           count += vpar_is_ordered_isomorphic(vpar,flip);
          );
     print1(count",");
     );
}
quit


\\------------------------------------------------------------------------------
\\ Polys All Real Roots

{
  \\ free

  \\ trees, indpoly all real roots
  \\ not in OEIS: 1, 1, 1, 1, 2, 3, 4, 7, 14, 27, 45, 87, 173, 339,
  \\ trees, indpoly not all real roots
  \\ not in OEIS: 0, 0, 0, 1, 1, 3, 7, 16, 33, 79, 190, 464, 1128, 2820,
  \\ n=4,5 stars are not all real

  \\ trees, indompoly not all real roots
  \\ not in OEIS:          1, 1, 3, 6, 16, 31, 75, 178, 438, 1078, 2707, 6809,
  \\ n=4,5 stars are not all real

  \\ trees, dompoly not all real roots
  \\ not in OEIS:          1, 2, 4, 9, 19, 42, 97, 222, 528, 1266, 3096, 7639,
  \\ n=4,5 stars and path-5 are not all real

  \\ trees, indpoly and perfect_dompoly both all real roots
  \\ n=1        4           8
  \\   1, 1, 1, 0, 1, 1, 2, 1, 4, 3, 12, 9, 27, 21, 67, 68,
  \\ not in OEIS:            4, 3, 12, 9, 27, 21, 67, 68,
  \\
  \\ n=1  [0]
  \\ n=2  [0, 1]
  \\ n=3  [0, 1, 1]
  \\ n=5  [0, 1, 2, 1, 1]  bistar 2,3    3---2---1---5
  \\                                             |
  \\                                             4
  \\ n=6  [0, 1, 2, 2, 1, 1]     bistar 3,3
  \\ n=7  [0, 1, 2, 2, 1, 5, 1]    3---2---1---5---6
  \\                                   |   |
  \\                                   4   7
  \\      https://hog.grinvin.org/ViewGraphInfo.action?id=714
  \\ n=7  [0, 1, 2, 1, 4, 1, 1]    3---2---1---4---5
  \\                                      / \
  \\                                     6   7
  \\
  \\ forests
  \\ n=1        4             8
  \\   1, 2, 3, 4, 6, 9, 13, 18, 28, 39, 64, 93, 154, 227,
  \\ not in OEIS: 1, 2, 3, 4, 6, 9, 13, 18, 28, 39, 64, 93, 154, 227,


  my(forest=0);
  for(n=1,20,
     my(count=0,
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(got=
              1
              && test_poly_is_all_real_roots(vpar_indpoly(vpar))
              \\ && test_poly_is_all_real_roots(vpar_dompoly(vpar))
              \\ && test_poly_is_all_real_roots(vpar_minimal_dompoly(vpar))
              \\ && test_poly_is_all_real_roots(vpar_indompoly(vpar))
              \\ && test_poly_is_all_real_roots(vpar_totdompoly(vpar))
              && test_poly_is_all_real_roots(vpar_perfect_dompoly(vpar))
                  );
           count += got;
           if(1 && n<=8 && got,
              print("n="n"  "vpar);
              print(factor(vpar_indpoly(vpar)));
              print(factor(vpar_perfect_dompoly(vpar)));
             );
          );
     if(1, print1(count", "));
     );
  quit;
}
{
  \\ labelled rooted

  \\ trees, indpoly all real roots
  \\ not in OEIS: 1, 2, 9, 48, 600, 4860,
  \\ forests, indpoly all real roots
  \\ not in OEIS: 1, 3, 16, 109, 1191, 13021,

  \\ trees, dompoly all real roots
  \\ not in OEIS: 1, 2, 9, 48, 300, 2700,
  \\ trees, dompoly not all real roots
  \\ not in OEIS:          16, 325, 5076,

  my(forest=0); \\ trees or forests
  for(n=1,10,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              poly=vpar_Laplacian_poly(vpar));
           count += (test_poly_num_real_roots(poly)!=poldegree(poly)));
     print1(count", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_Seidel_adjacency_matrix() cospectral

{
  \\ free trees num sets of cospectals
  \\         n = 12
  \\ not in OEIS: 1, 2, 6, 17, 49, 129, 334,
  \\
  \\ n=12
  \\ [0, 1, 2, 3, 4, 4, 3, 1, 8, 9, 10, 1]
  \\    5---4---3---2---1---8---9---10---11
  \\        |   |       |
  \\        6   7      12
  \\ [0, 1, 2, 3, 4, 4, 1, 7, 8, 9, 7, 1]]
  \\    5---4---3---2---1---7---8---9---10
  \\        |           |   |
  \\        6          12  11

  \\ free forests num sets of cospectals
  \\          n = 2
  \\ not in OEIS: 1, 1, 2, 4, 7, 7, 11, 12, 17, 19, 26, 30, 46, 70,
  \\
  \\ n=2  [0, 1], [0, 0]

  my(forest=1);
  for(n=1,20,
     \\ print("n="n);
     my(vpar=vpar_free_first(n),
        m=Map());
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(M=vpar_Seidel_adjacency_matrix(vpar));
           \\ M=vpar_adjacency_matrix(vpar);
           my(poly=charpoly(M),
              l);
           mapisdefined(m,poly,&l) || (l=List([]));
           listput(l,vpar);
           mapput(m,poly,l));

     m=Mat(m);
     my(vpars_list=if(matsize(m)[1]==0,[],m[,2]));

     if(n==2,
        for(i=1,#vpars_list,
           my(vpars=vpars_list[i]);
           if(#vpars >= 2,
              print("  "vpars))));

     print1(#select(l->#l>=2, vpars_list),", ");  \\ cospectrals
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_is_horizpos_vector()

{
  my(forest=1); \\ trees or forests
  for(n=3,4,
     forvec(vec=vector(n,i,[0,n-1]),
           if(vpar_is_horizpos_vector(vec),
              print(vec)));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_horizpos_vector() increasing

{
  \\ labelled forests
  \\ 1,3,13,71,461,3447,29093,
  \\ A003319 indecomposable permutations
  \\ same as
  \\ vpar_count_min_descendant_increasing()
  \\
  \\ labelled trees
  \\ 1,2,8,42,266,1954,16270,
  \\ A104982 = column 3 of A104980
  \\         = 1/2 * column 0 of A104988
  \\
  \\ root=1 trees, same as forests 1 later
  \\ 1,1,3,13,71,461,3447,

  my(forest=-1);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_horizpos_vector(vpar));
           if(test_vector_is_increasing(vec),
              \\ print(vpar"  "vec);
              count++;
              \\ vpar_dot(vpar);
             ));
     print1(count","));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_weights_vector() increasing

{
  \\ labelled forests
  \\ not in OEIS: 1, 3, 6, 25, 74, 399
  \\
  \\ labelled trees
  \\ not in OEIS: 1, 2, 3, 12, 30, 162
  \\
  \\ root=1 trees
  \\          n = 1  2  3  4  5
  \\              1, 1, 1, 3, 6, 27, 78, 437
  \\ not in OEIS:          3, 6, 27, 78, 437

  my(forest=0);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              weights=vpar_weights_vector(vpar));
           count += test_vector_is_increasing(weights));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_diagdepths_vector() occurring

{
  \\ Preorder
  \\
  \\ forests
  \\ 1, 1, 2, 4, 9, 21, 51, 127, 323, 835, 2188,
  \\ A001006 Motzkin
  \\
  \\ trees, same 1 later
  \\ 1, 1, 1, 2, 4, 9, 21, 51, 127, 323, 835,
  \\

  my(forest=0);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vec=vpar_diagdepths_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
     m=Mat(m);
     if(n==7, for(i=1,matsize(m)[1], print(m[i,1])));
     );
  quit;
}
{
  \\ Unlabelled (Pre-Max)
  \\
  \\ forests
  \\ 1, 1, 2, 4, 9, 19, 42, 94, 214, 494, 1154, 2722,
  \\ not in OEIS: 2, 4, 9, 19, 42, 94, 214, 494, 1154, 2722,
  \\
  \\ trees, same 1 later
  \\ 1, 1, 1, 2, 4, 9, 19, 42, 94, 214, 494, 1154,
  \\

  my(forest=1);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(vec=vpar_diagdepths_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
     );
  quit;
}
{
  \\ trees
  \\ not in OEIS: 1, 2, 6, 32, 200, 1560, 14245,
  \\
  \\ forests
  \\ not in OEIS: 1, 2, 8, 40, 260, 2035, 18585,
  \\
  \\ trees root=1, same as forests 1 previous
  \\ 1, 1, 2, 8, 40, 260, 2035,

  my(forest=-1); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           my(vec=vpar_diagdepths_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
     m=Mat(m);
     \\ if(n==4, for(i=1,matsize(m)[1], print(m[i,1])));
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Balanced Binary Differences

binary(vpar_balanced_binary_next(test_from_binary(111000111000)))
quit

{
  my(prev=0);
  for(b=1,4096,
     if(vpar_is_balanced_binary(b),
        printf("%12s\n", test_to_binary(prev));
        printf("%12s\n", test_to_binary(b));
        printf("%12s  xor\n", test_to_binary(bitxor(b,prev)));
        printf("%12s  incr\n", test_to_binary(b-prev));
        print();
        prev=b));
  quit;
}


\\------------------------------------------------------------------------------
\\ Kreweras Lattice

\\ Ordered forests, edge to new forest made by merging two sets of siblings.
\\ So i,j siblings in F are always siblings in new F.

is_siblings_subset(vpar,sub) =
{
  if(vpar==sub,return(0));
  my(setnum=vpar_siblings_setnum_vector(vpar),
     sub_setnum=vpar_siblings_setnum_vector(sub));
  for(i=1,#vpar,
     for(j=i+1,#vpar,
        if(setnum[i]==setnum[j] && sub_setnum[i]!=sub_setnum[j],
           return(0))));
  1;
}
{
  my(vpar=vpar_make_preorder_depths([0,1,2,3]));
  check_equal(is_siblings_subset(vpar,vpar_make_preorder_depths([0,1,1,2])),1);
  check_equal(is_siblings_subset(vpar,vpar_make_preorder_depths([0,1,2,2])),1);
  check_equal(is_siblings_subset(vpar,vpar_make_preorder_depths([0,1,0,2])),1);
  check_equal(is_siblings_subset(vpar,vpar_make_preorder_depths([0,1,2,1])),1);


  \\     1          1
  \\    / \         |
  \\   2   3        2
  \\       |       / \
  \\       4      3   4

  check_equal(is_siblings_subset(vpar_make_preorder_depths([0,1,1,2]),
                                 vpar_make_preorder_depths([0,1,2,2])),0);
  check_equal(is_siblings_subset(vpar_make_preorder_depths([0,1,2,2]),
                                 vpar_make_preorder_depths([0,1,1,2])),0);
}

to_str(vec) = concat(apply(n->Str(n),vec));

{
  n=4;
  my(vpars=vpar_preorder_all(n,1), \\ forests
     depths=apply(vpar_depths_vector,vpars),
     names=apply(to_str,depths),
     m=matrix(#vpars,#vpars,i,j,
              i<j && is_siblings_subset(vpars[i],vpars[j])));

  my(more);
  until(!more,
        print("untransit");
        more=0;
        for(i=1,#vpars,
           for(j=i+1,#vpars,
              for(k=i+1,#vpars,
                 if(m[i,j] && m[j,k]  && m[i,k],
                    m[i,k]=0; more=1)))));

  my(num_edges=0);
  for(i=1,#vpars,
     for(j=i+1,#vpars,
        if(m[i,j],
           print("['"names[i]"','"names[j]"'],");
           num_edges++)));
  print("# num edges "num_edges);
}
quit


\\------------------------------------------------------------------------------
\\ Isomorphic Halves -- Eccentricity up to Similarity

pending = [ [0] ];  \\ singleton
print("k=0 size "#pending[1]" count "#pending);
{
  for(k=1,6,
     my(seen=Map());
     for(i=1,#pending,
        my(vpar=Vec(pending[i]),
           orbits=vpar_free_orbit_sets(vpar),
           eccentricities=vpar_eccentricities_vector(vpar));
        for(j=1,#orbits,
           my(v=orbits[j][1]);                \\ first of each orbit
           for(k=j,#orbits,
              my(u=orbits[k][1]);             \\ first of each orbit
              if(eccentricities[u]==eccentricities[v],
                 my(new_vpar=vpar_join(vpar,u, vpar_reroot(vpar,v)));
                 new_vpar=vpar_to_free(new_vpar);   \\ canonical
                 new_vpar=Vecsmall(new_vpar);       \\ save memory
                 mapput(seen,new_vpar,0)))));
     pending=Mat(seen)[,1];

     \\ count num vertices of given eccentricity, vertices up to similarity
     \\ 20, 71, 151, 284, 320, 327, 324, 266, 234, 185, 167, 98, 48, 38, 23
     \\
     \\ total distinct vertices up to similarity
     \\ not in OEIS: 1,2,8,80,2624,293248,
     \\
     my(e=vector(2^k-1));
     for(i=1,#pending,
        my(vpar=Vec(pending[i]),
           orbits=vpar_free_orbit_sets(vpar),
           eccentricities=vpar_eccentricities_vector(vpar));
        for(j=1,#orbits,
           my(v=orbits[j][1]);                \\ first of each orbit
           e[eccentricities[v]]++));
     \\ print("k="k"  "e);
     print1(vecsum(e),",");
  );
}
quit


\\------------------------------------------------------------------------------
\\ Binary Tree Rotation Hamiltonian

\\ vector(50,n, vpar_INTERNAL_Catalan_number(n)%2)
\\ select(n->vpar_INTERNAL_Catalan_number(n)%2, [1..1000])
\\ Catalan odd when n = 2^k-1
\\ C(n) = (2*n)!/(n!*(n+1)!).

\\ vector(50,n, valuation(vpar_INTERNAL_Catalan_number(n),2))
\\ A048881 NumOneBits(n+1) - 1

\\ GP-Test  vector(100,n, valuation(n!,2)) == vector(100,n, n - hammingweight(n))
\\ A048881 n - NumOneBits(n)
\\ 2n - Ones(2n) - (n - Ones(n) + n+1 - Ones(n+1))
\\ 2n - Ones(2n) - n + Ones(n) - n-1 + Ones(n+1)
\\ Ones(2n) + Ones(n) - 1 + Ones(n+1)
\\ Ones(n+1) - 1
\\ GP-Test  vector(100,n, valuation(vpar_INTERNAL_Catalan_number(n),2)) == \
\\ GP-Test  vector(100,n, hammingweight(n+1)-1)
\\ GP-Test  vpar_INTERNAL_Catalan_number(8) == 1430
\\ GP-Test  vpar_INTERNAL_Catalan_number(16) == 35357670

\\ Wu and Wang
\\
\\     1111 -- 1112 -- 1212 -- 1211 -- 1231 -- 1234 -- 1214
\\  -- 1114 -- 1113 -- 1123 -- 1124 -- 1134 -- 1131 -- 1121
\\  -- 1111

\\ cf
\\ S.W. Bent "Ranking Trees Generated By Rotations", Proc. 2nd Scandanavian
\\ Workshop On Algorithm Theory, LNCS 447:132{142, 1991.


all(n) =
{
  if(n==1, return([ [1] ]));
  my(below=all(n-1),
     l=List([]),
     rev=0);
  for(i=1,#below,
     my(sizes=Vec(below[i],n),
        t=List([]),
        s=1);
     while(1,
           sizes[n]=s;
           listput(t,sizes);
           if(s==n, break);
           s += sizes[n-s]);
     if(rev,t=Vecrev(t));
     rev=!rev;
     for(i=1,#t, listput(l,t[i])));
  Vec(l);
}
l=all(5)
for(i=1,#l, vpar_is_postorder_sizes_vector(l[i],1) || error(l[i]));
for(i=2,#l, hammingweight(l[i]-l[i-1])==1 || error(l[i]));

quit


\\------------------------------------------------------------------------------
\\ Binary Tree One Rotation
\\
\\ slightly wild effect on Dyck word sometimes/often

\\ 1110101000  [1, 1, 1, 4, 5]  vpar [4, 4, 4, 5, 0]
\\ 1101010010  [1, 1, 1, 4, 1]  vpar [4, 4, 4, 0, 0]
\\   11111010   ham 6

\\     5          4   5
\\     |         /|\
\\     4        1 2 3
\\    /|\
\\   1 2 3
\\
\\     5            4
\\    /            / \
\\   4            1   5
\\    \            \
\\     1            2
\\      \            \
\\       2            3
\\        \
\\         3


{
  my(n=5,
     forest=1);
  my(sizes=vpar_postorder_sizes_first(n,forest),
     count=0);
  until((sizes=vpar_postorder_sizes_next(sizes,forest))=='none,
        print("-----");
        my(vpar=vpar_make_postorder_sizes(sizes),
           b=vpar_to_balanced_binary(vpar));
        for(i=1,#sizes,
           forstep(j=sizes[i]-1,1,-1,
              my(to_sizes=sizes);
              to_sizes[i]=j;
              if(vpar_is_postorder_sizes_vector(to_sizes,1),
                 my(to_vpar=vpar_make_postorder_sizes(to_sizes),
                    to_b=vpar_to_balanced_binary(to_vpar),
                    x=bitxor(b,to_b));
                 print(test_to_binary(b),"  ",sizes"  vpar "vpar);
                 print(test_to_binary(to_b),"  ",to_sizes"  vpar "to_vpar);
                 printf("%*s   ham %d\n",2*n,test_to_binary(x),
                       hammingweight(x));
                 print();
                 count++;
                 break))));
  print("count "count);
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_downwards_flip() Graph

to_str(vec) = concat(apply(n->Str(n),vec));
{
  my(forest=1); \\ trees or forests
  n=4;
  forvec(vec=vpar_forvec_from(n,forest),
        my(vpar=vpar_from_upascend(vec));
        print("  ['",to_str(vpar),"', '"to_str(vpar_downwards_flip(vpar)),"'],"));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_downwards_flip() vs vpar_to_balanced_binary()

{
  \\ Downwards
  \\
  \\ trees
  \\ not in OEIS: 1, 1, 2, 4, 9, 24, 73, 242, 850,
  \\
  \\ forests
  \\ not in OEIS: 1, 1, 2, 5, 14, 42, 135, 463, 1686,

  my(n=4,
     forest=1);
  for(i=1,vpar_count_updown(n,forest),
     my(vpar=vpar_downwards_ith(n,i,forest),
        flipped_vpar=vpar_downwards_flip(vpar),
        b=vpar_to_balanced_binary(vpar),
        fb=vpar_to_balanced_binary(flipped_vpar));
     print(test_to_binary(b)"  "vpar);
     print(test_to_binary(fb)"  "flipped_vpar);
     printf("%*s\n", 2*n, test_to_binary(bitxor(b,fb)));
     print();
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_make_preorder_depths() Up Steps
\\
\\ forests
\\ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
\\ reps
\\ 1, 1, 2, 5, 14, 42, 132, 429, 1430, 4862, 16796,

\\ trees
\\ 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
\\ reps
\\ 1, 1, 1, 2, 5, 14, 42, 132, 429, 1430, 4862,


depths_up_steps(depths) =
{
  my(total=0);
  for(v=2,#depths,
     for(i=depths[v],depths[v-1], total++));
  total;
}

{
  my(forest=0);
  for(n=1,12,
     my(m=0,
        reps=0,
        all=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(depths=vpar_depths_vector(vpar),
              up=depths_up_steps(depths));
           if(up>m, m=up;reps=0);
           if(up==m, reps++);
           all++;
           if(n==6 && up==n-1, print(vpar"  "depths));

           if(n==6,
              print(up"  "n-depths[#depths]-1));
          );
     \\print1(m", ");
     \\print1(reps", ");
     \\print1(all", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ Pallo Tree Structure

vec=[1,2,1,4]     \\ 111 001 00

vec=[1,1,2,4]     \\ 11 011 000
vec=[1,1,3,1]     \\ 11 01001 0

vec=[1,1,1,4]     \\ 11 0101 00

vec=[1,1,1,2]     \\ 10 1011 00

vec=[1,1,2,1]     \\ 10 11001 0
vec=[1,1,1,3]     \\ 10 1101 00
vec=[1,1,2,3]     \\ 10 111 000
vec=[1,1,2,4]     \\ 11 011 000
vec=[1,1,3,4]     \\ 11 101 000

vec=[1,1,2,1]     \\ 10 11 00 10
vec=[1,1,3,1]     \\ 11 01 001 0
vec=[1,2,3,1]     \\ 111 0001 0
vec=[1,2,3,4]     \\ 1111 0000



\\ vec=apply(d->d-1,vec)
check_equal(vpar_is_postorder_sizes_vector(vec,1), 1);
\\ vpar_is_preorder_depths_vector(vec,1)
\\ check_equal(vpar_is_preorder_depths_vector(vec,1), 1);
\\ vpar=vpar_make_preorder_depths(vec)
vpar=vpar_make_postorder_sizes(vec)
test_to_binary(vpar_to_balanced_binary(vpar))
quit


\\------------------------------------------------------------------------------
\\ vpar_preorder_to_binary_tree()

\\      1             2             2
\\       \           / \           / \
\\        2         1   4         3   4
\\       / \         \           /
\\      3   4         3         1
\\
\\   1  2  4        1    4        1    4
\\      |          / \            |
\\      3         2   3           2
\\                                |
\\                                3


vpar=[0,0,2,0]
\\b=vpar_preorder_to_binary_tree(vpar)
\\vpar_dot(b)

rotate_left(vpar,v) =
{
  my(p=vpar[v], t=vpar_next_sibling_of_vertex(vpar,p,1));
  print("t="t);
  if(t,
     \\ prev sibling drops down
     vpar[v]=t;
     vpar[p]=t;
     ,
    );
  vpar;
}

vpar=rotate_left(vpar,3)
\\ check_equal(vpar, [2,0,2,0]);
check_equal(vpar, [0,1,1,0]);  \\ preorder
\\ b=vpar_preorder_to_binary_tree(vpar)
\\vpar_dot(vpar)

vpar=rotate_left(vpar,3)
check_equal(vpar, [3,0,2,0]);
\\ b=vpar_preorder_to_binary_tree(vpar)
vpar_dot(vpar)

quit


\\------------------------------------------------------------------------------
\\ vpar_indomsets_count() max of forests

\\ cf trees
\\ vector(12,n,vpar_indomsets_count_max(n))
\\ n=1  2  3  4  5  6  7
\\   1, 2, 2, 3, 4, 5, 8, 9, 16, 17, 32, 33

\\ forests
\\ 1, 2, 2, 4, 4, 8, 8, 16, 16, 32, 32, 64, 64, 128, 128, 256, 256, 512,
\\ reps
\\ 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 1,
\\ even is forest of path-2 pairs
\\ odd is path-odd and rest forst of path-2 pairs
\\
\\ indomsets count is product over component trees, so max tree product over
\\ partitions

{
  print(vector(12,n,n--; vpar_indomsets_count_max(n)));
  print(vector(12,n,n--; vpar_indomsets_count_max(n,1)));
  quit
}
{
  my(tree_indomsets_count=vector(50,n,vpar_indomsets_count_max(n)));
  for(n=1,20,
     my(m=0,
        reps=0,
        l=List([]));
     forpart(p=n,
             my(got=prod(i=1,#p, tree_indomsets_count[p[i]]));
             if(m<got, m=got; reps=0; l=List([]));
             if(m==got, reps++; listput(l,p)));
     \\ print1(m,", ");
     \\ print1(reps,", ");
     print("n="n);
     for(i=1,#l, print(l[i]));
     );
  quit
}

{
  my(forest=1);
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        m=0,
        reps=0,
        l=List([]));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(got=vpar_indomsets_count(vpar));
           \\ if(n==11 && got==5, print(vpar));
           \\ if(n==13 && got==6, print(vpar));
           if(m<got, m=got; reps=0; l=List([]));
           if(m==got, reps++; listput(l,vpar)));
     print1(m,", ");
     \\ print1(reps,", ");
     \\ print("n="n);
     \\ for(i=1,#l, print(l[i]));

     \\ if(reps==1, print("\nn="n" unique "seen_vpar));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_depths_vector() Differing One Place, Balanced Binary

\\ depths decrease by 1 is flip 10 <-> 01
\\ depths decrease by 2 is flip 100 <-> 001
\\ depths decrease by 3 is flip 1000 <-> 0001

n=5;
forest=1;
vpars=vpar_preorder_all(n,forest);
depths=apply(vpar_depths_vector,vpars);
bins=apply(vpar_to_balanced_binary,vpars);
is01(x,y) = x=bitxor(x,y); (x>>valuation(x,2))==3;
vector_num_diffs(u,v) = sum(i=1,#u, u[i]!=v[i]);
{
  for(i=1,#vpars,
     for(j=i+1,#vpars,
        dd=depths[i]-depths[j];
        if(
           \\ is01(bins[i],bins[j]),
           \\ vecsum(dd)!=1 &&
           vector_num_diffs(depths[i],depths[j])==1,
           bi=vpar_to_balanced_binary(vpars[i]);
           bj=vpar_to_balanced_binary(vpars[j]);
           bx=bitxor(bi,bj);
           w=#binary(bx)-valuation(bx,2);
           w==vecsum(dd)+1 || error();
           w==4 || next;
           print(depths[i]"  "depths[j]"     ",dd,
                 "\n  "
                 test_to_binary(bi),"    ",test_to_binary(bx),
                 "  w=",w,
                 "\n  "
                 test_to_binary(bj)))));
}
quit;


\\------------------------------------------------------------------------------
\\ vpar_subtree_num_childless_vector() occurring

{
  \\ Downwards
  \\
  \\ trees
  \\ not in OEIS: 1, 1, 2, 4, 9, 24, 73, 242, 850,
  \\
  \\ forests
  \\ not in OEIS: 1, 1, 2, 5, 14, 42, 135, 463, 1686,

  my(forest=1);
  for(n=1,12,
     my(m=Map());
     for(i=1,vpar_count_updown(n,forest),
        my(vpar=vpar_downwards_ith(n,i,forest));
        my(vec=vpar_subtree_num_childless_vector(vpar));
        mapput(m,vec,1));
     print1(#m", ");
     );
  quit;
}
{
  \\ Functional Digraphs num childlessness vectors
  \\
  \\ no roots
  \\ not in OEIS: 1, 2, 8, 38, 192, 1052, 6316,
  \\
  \\ forests
  \\ not in OEIS: 2, 4, 11, 42, 197, 1058, 6323,
  my(rootless=0);
  for(n=1,8,
     my(m=Map());
     forvec(vpar=vector(n,i,[rootless,n]),
           my(vec=vpar_subtree_num_childless_vector(vpar));
           \\ if(n==7,print(vec));
           \\ test_vector_is_decreasing(vec) || error(n"  "vec);
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}
{
  \\ Unlabelled (Pre-Max)
  \\
  \\ forests
  \\ 1, 1, 2, 4, 8, 18, 40, 94, 223, 534, 1288, 3147,
  \\ not in OEIS: 1, 2, 4, 8, 18, 40, 94, 223, 534, 1288, 3147,
  \\
  \\ trees
  \\ 1, 1, 2, 4, 8, 16, 34, 73, 166, 386, 923, 2231,
  \\ not in OEIS: 1, 2, 4, 8, 16, 34, 73, 166, 386, 923, 2231,
  \\
  \\ n=7 has first non-decreasing

  my(forest=0);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(vec=vpar_subtree_num_childless_vector(vpar));
           \\ if(n==7,print(vec));
           \\ test_vector_is_decreasing(vec) || error(n"  "vec);
           mapput(m,vec,1));
     print1(#m", ");
     );
  quit;
}
{
  \\ Pre-Order
  \\
  \\ forests
  \\ 1, 1, 2, 5, 13, 36, 103, 302, 906, 2765, 8552, 26757,
  \\ not in OEIS: 1, 2, 5, 13, 36, 103, 302, 906, 2765, 8552, 26757,
  \\
  \\ trees
  \\ 1, 1, 2, 4, 9, 23, 63, 180, 530, 1594, 4871, 15085,
  \\ not in OEIS: 1, 2, 4, 9, 23, 63, 180, 530, 1594, 4871, 15085,

  my(forest=1);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vec=vpar_subtree_num_childless_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
     );
  quit;
}
{
  \\ labelled rooted

  \\ forests
  \\ 1, 1, 4, 15, 66, 335, 1898,
  \\ not in OEIS: 4, 15, 66, 335, 1898,
  \\
  \\ trees
  \\ 1, 1, 4, 15, 66, 335, 1898,
  \\ same

  my(forest=1); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_subtree_num_childless_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_free_automorphism_generators() max

\\ trees
\\ n=1  2  3  4        7  8       11 12       15 16
\\   0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8,
\\ A263997 maybe
\\ reps, free trees
\\ n=1  2  3  4     6     8    10    12     14     16     18     20
\\   1, 1, 1, 1, 1, 3, 1, 4, 1, 6, 1, 8, 2, 12, 2, 16, 3, 27, 3, 40,
\\ not in OEIS: 3, 1, 4, 1, 6, 1, 8, 2, 12, 2, 16, 3, 27, 3, 40
\\
\\ n=1 unique [0]           singleton
\\ n=2 unique [0, 1]        path-2
\\ n=3 unique [0, 1, 1]     path-3
\\ n=4 unique [0, 1, 1, 1]  claw
\\ n=5 unique [0, 1, 1, 1, 1]   star-5
\\
\\ n=7 unique [0, 1, 2, 2, 1, 1, 1]
\\    3      5
\\     \    /         12 automorphisms
\\      2--1-6         3 generators
\\     /    \         bi-star 3,4
\\    4      7
check_equal(vpar_is_free_isomorphic([0, 1, 2, 2, 1, 1, 1], \
                                    vpar_make_bistar(3,4)), 1);
\\
\\ n=9 unique [0, 1, 2, 2, 2, 1, 1, 1, 1]
\\        bi-star 4,5
check_equal(vpar_is_free_isomorphic([0, 1, 2, 2, 2, 1, 1, 1, 1], \
                                    vpar_make_bistar(4,5)), 1);
\\
\\ n=11 unique [0, 1, 2, 2, 2, 1, 6, 6, 1, 1, 1]
\\    *           *
\\     \         /         72 automorphisms
\\      *---*---*-*         5 generators
\\     /   /|\   \
\\    *   * * *   *
\\
\\ n=13  2 of
\\ [0, 1, 2, 3, 3, 3, 1, 7, 7, 7, 1, 1, 1]
\\ [0, 1, 2, 2, 2, 2, 1, 7, 7, 7, 1, 1, 1]
{
  my(forest=1);
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        m=-1,
        seen_vpar,
        reps=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(got=vpar_num_free_automorphism_generators(vpar));
           \\ if(n==11 && got==5, print(vpar));
           \\ if(n==13 && got==6, print(vpar));
           if(m==got, reps++,
              m<got, m=got;reps=1;seen_vpar=vpar));
     print1(m,", ");
     \\ print1(reps,", ");

     \\ if(reps==1, print("\nn="n" unique "seen_vpar));
     );
  quit;
}


\\------------------------------------------------------------------------------

n=4;
vpars=vpar_postorder_all(n,1); \\ forests
vpars=vpar_preorder_all(n,1); \\ forests
vpars=vecsort(vpars);
{
  for(i=1,#vpars,
     print(vpars[i]"  sizes "vpar_subtree_sizes_vector(vpars[i])));
}
quit



\\------------------------------------------------------------------------------
\\ J M Pallo, "Enumerating, Ranking and Unranking Binary Trees", The Computer
\\ Journal, volume 29, number 2, 1986, pages 171-175.
\\ https://academic.oup.com/comjnl/article/29/2/171/460530

\\ ordered oriented binary trees
\\ "external" nodes childless
\\ "internal" nodes with children
\\ Catalan number many such
\\ "size"   |T| = number of internal
\\ "weight"  T~ = number of external  = |T|+1
\\ "product" of T,T'          *
\\                   T'' =   / \
\\                          T   T'
\\ external nodes in pre-order
\\ weight w[i] = size of largest subtree with external node i rightmost
\\ weight sequence = w1,w3,...,w[|T|]
\\   last would be w[T~]=T~, is omitted
\\ weights of product T'' = wT1,wT2,...wTn-1, T~,  wT'1, wT'2, ... wT'|T'|
\\
\\ 1 <= w[i] <= i
\\ Then i' in range i-w[i]+1 < i' < i must have i-w[i] <= i'-w[i']
\\
\\ in-order traversal, label each internal with weight of its left subtree

\\ Joan Lucas, Dominique Roelants van Baronaigien, Frank Ruskey, "On
\\ Rotations and the Generation of Binary Trees", Journal of Algorithms,
\\ volume 15, 1993, pages 343-366.
\\ http://webhome.cs.uvic.ca/~ruskey/Publications/Rotation/Rotation.html

weights_first(n) = vector(n,i,1);
weights_next(vec) =
{
  forstep(i=#vec,1,-1,
     if(vec[i]<i,
        vec[i] = vec[i] + vec[i-vec[i]];
        return(vec),
        vec[i] = 1));
  'none;
}
{
  n=4;
  ldepths0=List([]);
  ldepths=List([]);
  my(vpar=vpar_preorder_first(n));
  until((vpar=vpar_preorder_next(vpar,1))=='none,
        \\ my(vpar=vpar_relabel_reverse(vpar));
        my(vpar=vpar_relabel_postorder(vpar));
        my(depths=vpar_subtree_sizes_vector(vpar));
        print((depths)"   ");
        listput(ldepths0,depths);
        listput(ldepths,vec));
}
print();
{
  n=4;
  lvecs=List([]);
  vec=vpar_postorder_subtree_sizes_first(n);
  for(i=1,43,
     if(vec=='none,break);
     print(vec"   vpar="vpar_make_postorder_subtree_sizes(vec));
     listput(lvecs,vec);
     vec=vpar_postorder_subtree_sizes_next(vec));
  print(vec);
}
lvecs=vecsort(lvecs)
ldepths=vecsort(ldepths)
lvecs==ldepths


vector_partial_ordering(w1,w2) =
{
  if(w1==w2,return(0));
  for(i=1,#w1, w1[i] <= w2[i] || return(0));
  1;
}
weights_str(vec) = concat(apply(n->Str(n),vec));
print();
{
  n=4;
  l=List([]);
  vec=weights_first(n);
  until((vec=weights_next(vec))=='none,
        listput(l,vec));
  l=vecsort(l);

  my(degrees=vector(#l),
     num_edges=0);
  for(i=1,#l,
     for(j=1,#l,
        if(vector_partial_ordering(l[i],l[j]),
           for(k=1,#l,
              if(vector_partial_ordering(l[i],l[k])
                 && vector_partial_ordering(l[k],l[j]), next(2)));
           print("['"weights_str(l[i])"','"weights_str(l[j])"'],"
                 "    # "l[i]" "l[j]"   "l[j]-l[i]);
           num_edges++;
           degrees[i]++;
           degrees[j]++)));

  print("# degrees "degrees);
  print("# num edges "num_edges);


  for(i=1,#l,
     for(j=1,#l,
        if(vector_partial_ordering(l[i],l[j]),
           my(want=1);
           for(k=1,#l,
              if(vector_partial_ordering(l[i],l[k])
                 && vector_partial_ordering(l[k],l[j]), want=0));

           my(diff=l[j]-l[i],
              by_diff = 1);
           if(hammingweight(diff)!=1, by_diff=0);
           for(i=1,#diff, if(diff[i]<0, by_diff=0));
           if(by_diff,
              my(diff1=diff/vecmax(diff));
              \\ print("diff1 "diff1);
              for(q=1,n,
                 my(other = l[i]+q*diff1);
                 if(other==l[j], break);
                 if(setsearch(l,other), by_diff=0)));

           if(by_diff != want,
              print(l[i]" "l[j]"  diff="diff"  want="want" by_diff="by_diff);
              error()))));
}
\\ vector_partial_ordering(w1,w2) =

quit


\\------------------------------------------------------------------------------
\\ Pallo Subtree Sizes
\\
\\                  *                        5         8
\\             /       \                   / | \      / \
\\          *             *               2  3  4    6   7
\\       /    \          / \              |
\\      *      *        *   9             1
\\     / \    / \      / \
\\    1   2  3   *    6   *            "natural" correspondence
\\              / \      / \           postorder labelling
\\             4   5    7   8
\\
\\ v=1  2  3  4  5  6  7  8   (9)
\\   1, 2, 1, 1, 5, 1, 1, 3   (9)

\\ presubtree_sizes_vector(vpar) =
\\ {
\\   my(prev_sibling=vpar_next_sibling_vector(vpar,1),
\\      subtree_sizes=vpar_subtree_num_childless_vector(vpar));
\\   vector(#vpar,v,
\\          my(total);
\\          until(!(v=prev_sibling[v]),
\\                total += subtree_sizes[v]);
\\          total);
\\ }
\\ \\      1         6
\\ \\    / | \      / \
\\ \\   2  4  5    7   8
\\ \\   |
\\ \\   3
\\ \\
\\ vpar=[0,1,2,1,1, 0,6,6];
\\ vpar=vpar_relabel_postorder(vpar)
\\ presubtree_sizes_vector(vpar)

{
  n=4;
  my(vpar=vpar_preorder_first(n));
  until((vpar=vpar_preorder_next(vpar,1))=='none,
        my(vpar=vpar_relabel_postorder(vpar));
        print(vpar"   "vpar_subtree_sizes_vector(vpar)));
  quit;
}

\\ "Lattice" change
\\
\\         A                   A
\\       /   \               /   \
\\      T     B             B     T''
\\           / \           / \
\\         T'   T''       T   T'
\\
\\
\\     A    B   T''         B   T''
\\     |    |              / \
\\     T    T'            A   T'
\\                        |
\\                        T
\\
\\ drop A to child of its next sibling
\\     subtree size B increases
\\
\\ conversely raise A to under its grandparent
\\     subtree size B decreases
\\
\\ first sibling can raise, other than first root
\\ each with next sibling can drop
\\
\\ first sibling = num_siblings_sets - 1
\\ without next sibling = num_siblings_sets
\\ with next sibling = #vpar - num_siblings_sets
\\ total first + with = #vpar - 1


\\------------------------------------------------------------------------------
\\ vpar_preorder_to_i() vs vpar_to_balanced_binary() size

vpar=vpar_make_random(50);
vpar=vpar_relabel_preorder(vpar);
#binary(vpar_preorder_to_i(vpar))
#binary(vpar_to_balanced_binary(vpar))
quit


\\------------------------------------------------------------------------------
\\ Pre-Order Depths Vectors Partial Ordering, Move Down One

vector_partial_ordering(w1,w2) =
{
  if(w1==w2,return(0));
  for(i=1,#w1, w1[i] <= w2[i] || return(0));
  1;
}
to_str(vec) = concat(apply(n->Str(n),vec));
print();

n=4;
l=List([]);
{
  my(vpar=vpar_preorder_first(n));
  until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
        listput(l,vpar_depths_vector(vpar)));
}
l=vecsort(l);
{
  for(i=1,#l,
     for(j=1,#l,
        if(vector_partial_ordering(l[i],l[j]),
           for(k=1,#l,
              if(vector_partial_ordering(l[i],l[k])
                 && vector_partial_ordering(l[k],l[j]), next(2)));
           my(bi=vpar_to_balanced_binary(vpar_make_preorder_depths(l[i])),
              bj=vpar_to_balanced_binary(vpar_make_preorder_depths(l[j])));
           print("['"to_str(l[i])"','"to_str(l[j])"'],"
                "    # "l[j]-l[i]" "vecsum(l[j]-l[i])
                "  "test_to_binary(bi)" "test_to_binary(bj)" "test_to_binary(bitxor(bi,bj))))))
}

{
  \\ number of edges
  \\ 0, 1, 5, 21, 84, 330, 1287, 5005,
  \\ A002054 Binomial coefficient C(2n+1, n-1)
  \\
  for(n=1,8,
     my(l=List([]),
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           listput(l,vpar_depths_vector(vpar)));
     my(num_edges=0);
     for(i=1,#l,
        for(j=1,#l,
           if(vector_partial_ordering(l[i],l[j]),
              for(k=1,#l,
                 if(vector_partial_ordering(l[i],l[k])
                    && vector_partial_ordering(l[k],l[j]), next(2)));
              num_edges++)));
     print1(num_edges", "));
}
quit


\\------------------------------------------------------------------------------
\\ E8

S2 = test_perms_Sn(2);
E8 = test_perms_direct_power(S2,3);
\\ test_perms_minimum_generators(E8)
\\
\\ [Vecsmall([1,2, 3,4, 6,5]),    (5 6)
\\  Vecsmall([1,2, 4,3, 5,6]),    (3 4)
\\  Vecsmall([2,1, 3,4, 5,6])]    (1 2)

{
  \\ num free trees with E8 automorphism group
  \\ n=1  2  3  4  5  6  7  8  9 10 11  12  13   14   15    16    17
  \\   0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 5, 19, 57, 173, 484, 1376, 3770,
  \\ not in OEIS: 2, 5, 19, 57, 173, 484, 1376, 3770,

  \\ n=10
  \\ [0, 1, 2, 3, 3, 1, 6, 6, 1, 1]  fixed [1, 2, 3, 6]
  \\             1
  \\          /  |   \ \       *               *
  \\         2   6    9 10      \             /
  \\        /   / \              *---*---*---*
  \\       3   7   8            /   /\        \
  \\      / \                  *   *  *        *
  \\     4   5
  \\
  \\ [0, 1, 2, 2, 1, 5, 1, 7, 1, 1]  fixed [1, 2]
  \\               1
  \\           /  / \   \ \       *   *   *---*
  \\         2    5  7   9 10      \   \ /
  \\        / \   |  |              *---*
  \\       3   4  6  8             /   / \
  \\                              *   *   *---*

  \\ -------------------
  \\ num free trees with E16 automorphism group
  \\ n=1  2  3  4  5  6  7  8  9 10 11 12 13  14  15   16   17
  \\   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 11, 53, 192, 685,
  \\ not in OEIS: 2, 11, 53, 192, 685,
  \\
  \\ n=13
  \\ [0, 1, 2, 3, 3, 2, 2, 1, 8, 9, 9, 1, 1]  fixed [1, 2, 3, 8, 9]
  \\              1
  \\          /   |   \ \       *                    *
  \\         2    8    12 13     \                  /
  \\        /|\   |               *---*----*---*---*
  \\       3 6 7  9              /   /\    /\       \
  \\      / \    / \            *   *  *  *  *       *
  \\     4   5  10  11
  \\ [0, 1, 2, 3, 3, 2, 2, 1, 8, 1, 10, 1, 1]  fixed [1, 2, 3]
  \\               1
  \\          /   / \  \ \      *   *    *   *---*
  \\         2    8 10  12 13    \   \    \ /
  \\        /|\   |  |            *---*----*
  \\       3 6 7  9 11           /   /    / \
  \\      / \                   *   *    *   *---*
  \\     4   5

  \\ -------------------
  \\ num free forests with E8 automorphism group
  \\ n=1  2  3  4  5  6  7  8  9  10  11  12   13  14  15   16   17
  \\   0, 0, 0, 0, 0, 0, 1, 1, 6, 16, 40, 98, 264,
  \\ not in OEIS: 1, 1, 6, 16, 40, 98, 264,

  \\ n=7  [0, 1, 1, 0, 4, 0, 0]  fixed [1]
  \\        1       4   6  7
  \\       / \      |
  \\      2   3     5
  \\ n=8  [0, 1, 2, 1, 0, 5, 0, 0]  fixed []
  \\        1       5   6  7
  \\       / \      |
  \\      2   4     6
  \\     /
  \\    3

  \\ num free trees with C2 automorphism group
  \\ n=1  2  3  4  5  6  7  8   9  10  11   12   13   14    15    16    17
  \\   0, 1, 1, 1, 2, 3, 3, 8, 14, 28, 51, 116, 236, 529, 1147, 2590, 5810,
  \\ not in OEIS: 8, 14, 28, 51, 116, 236, 529, 1147, 2590, 5810,


  my(verbose=0,
     forest=0,
     S2 = test_perms_Sn(2),
     want = test_perms_direct_power(S2,3));
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           vpar_num_free_automorphisms(vpar)==#want || next;
           my(perms=vpar_free_automorphisms(vpar));
           #test_perms_fixed_points(perms) == 0 || next;
           if(test_perms_is_isomorphic(perms,want),
              if(n<=8,
                  print(vpar"  fixed "test_perms_fixed_points(perms));
                );
              count++));

     print1(count,", ");
     );
  quit;
}
quit


\\------------------------------------------------------------------------------
\\ E8 One Fixed Point

\\ tree
\\ [0, 1, 2, 3, 1, 5, 6, 1, 8, 1, 10, 1, 1]
\\ forest
\\ [0, 1, 1, 0, 4, 0, 0]

vpar=vpar_join([0], 1,vpar_make_path(1))
vpar=vpar_join(vpar, 1,vpar_make_path(1))
vpar=vpar_join(vpar, 1,vpar_make_path(2))
vpar=vpar_join(vpar, 1,vpar_make_path(2))
vpar=vpar_join(vpar, 1,vpar_make_path(3))
vpar=vpar_join(vpar, 1, vpar_make_path(3))
S2 = test_perms_Sn(2);
want = test_perms_direct_power(S2,3);
vpar_num_free_automorphisms(vpar)==#want
perms=vpar_free_automorphisms(vpar);
test_perms_is_isomorphic(perms,want)
#test_perms_fixed_points(perms) == 1
quit


\\------------------------------------------------------------------------------
\\ vpar_weights_vector() distinct

\\ trees, or root=1 trees
\\ 1, 1, 3, 10, 55, 261, 2121, 13476,
\\ not in OEIS: 3, 10, 55, 261, 2121, 13476

{
  my(forest=-1);
  for(n=1,8,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec = vpar_weights_vector(vpar));
           \\ if(n==2,print(vec));
           mapput(m,vec,1));
     print1(#m", "));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_weights_vector() increasing, labelled and rooted

\\ outwards from centroid(s)
\\ leaves weight=n-1 last
\\
\\ n=5  6 of
\\      1         1      1      1         1         1
\\    /| |\      /|\    /|\    /|\       / \       / \
\\   2 3 4 5    2 3 4  2 3 5  2 4 5     2   3     2   3
\\              |      |      |         |   |     |   |
\\              5      4      3         5   4     4   5
\\  [0,1,1,1,1]    [2,3,4,4,4]            [2,3,3,4,4]

{
  my(forest=-1);
  for(n=5,5,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              weights=vpar_weights_vector(vpar));
           if(weights==vecsort(weights),
              print(vpar"  "weights);
              \\ vpar_dot(vpar);
             );
     ));
  quit;
}

{
  \\ forests
  \\ not in OEIS: 1, 3, 6, 25, 74, 399
  \\
  \\ trees
  \\ not in OEIS: 1, 2, 3, 12, 30, 162
  \\
  \\ root=1 trees
  \\          n = 1  2  3  4  5
  \\              1, 1, 1, 3, 6, 27, 78, 437
  \\ not in OEIS:          3, 6, 27, 78, 437

  my(forest=0);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              weights=vpar_weights_vector(vpar));
           count += test_vector_is_increasing(weights));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Total Number of Automorphism Fixed Vertices

{
  \\ free

  \\ trees total fixed vertices
  \\ 1, 0, 1, 1, 5, 10, 31, 72, 201, 509, 1374, 3587,
  \\ A005201

  \\ forests total fixed vertices
  \\ not in OEIS: 1, 0, 2, 3, 11, 23, 69, 168, 460, 1178, 3158, 8273,

  my(verbose=0,
     forest=1);
  for(n=1,12,
     my(total=0,
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(orbit_sizes=vpar_free_orbit_sizes(vpar));
           total += sum(i=1,#orbit_sizes, orbit_sizes[i]==1));
     print1(total,", ");
     );
  quit;
}

{
  \\ unlabelled

  \\ trees total fixed vertices
  \\ 1,2,4,11,28,78,213,598,1670,4723,13356,37986,
  \\ A005200

  \\ forests total fixed vertices
  \\ not in OEIS: 1,2,7,19,58,165,483,1384,4004,11514,33220,95707,
  \\ nothing from superseek
  \\
  \\ >=2 of same component tree are none fixed
  \\ singles of a tree sum their fixed

  my(forest=1);
  for(n=1,12,
     my(total=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(orbit_sizes=vpar_orbit_sizes(vpar));
           total += sum(i=1,#orbit_sizes, orbit_sizes[i]==1));
     print1(total","));
  quit;
}


\\------------------------------------------------------------------------------
\\ Free Trees Automorphisms No Fixed Points

{
  my(vpar=vpar_make_bistar(4,4),
     perms=vpar_free_automorphisms(vpar));
  test_perms_fixed_points(perms)
}

{
  \\ num free trees with no fixed points in automorphisms
  \\
  \\ unicentral would be centre fixed
  \\ bicentral swap centres only by halves same
  \\ all rooted trees

  \\ asymmetric duplicated to bicentral same halves
  \\ n=8
  \\ [0, 1, 2, 3, 4, 1, 6, 7]
  \\     1
  \\    2 6      path-8
  \\   3   7
  \\  4     8
  \\ 5
  \\
  \\ [0, 1, 2, 3, 2, 1, 6, 1]
  \\         1
  \\       /  \ \      *---*---*---*---*---*
  \\      2    6  8            |   |
  \\     / \    \              *   *
  \\    3   5    7
  \\   /
  \\  4
  \\

  my(verbose=0,
     forest=0);
  forstep(n=1,10,1,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
            my(perms=vpar_free_automorphisms(vpar),
              fixed=test_perms_fixed_points(perms));
           if(#fixed==0,
              \\ if(n==4,print(vpar));
              count++));

     print1(count,", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Minimum Generators most in free trees

\\ free trees
\\ free count  1, 1, 1, 2, 3, 6, 11, 23, 47, 106
\\           n=1  2  3  4  5  6   7   8   9   10
\\ min gens    0, 1, 1, 2, 2, 2,  2,  2,  3,
\\ reps        1, 1, 1, 1, 1, 3,  7, 14,  1,
\\
\\ n=4 max 2 reps [0, 1, 1]  need 2 for claw
\\ n=5 max 2 reps [0, 2, 1]  need 2 for star-5
\\ n=6 max 2 reps [0, 3, 3]
\\ [0, 1, 2, 2, 1, 1] = bistar 3,3 one gen in each  C2xC2
\\ [0, 1, 2, 1, 1, 1] = star arms 1,1,1,2  C3
\\ [0, 1, 1, 1, 1, 1] = star-6             S5
\\
\\ n=7 max 2 reps [1, 3, 7]
\\ n=8 max 2 reps [1, 8, 14]
\\
\\ n=9 max 3 reps [3, 14, 29, 1]
\\ [0, 1, 2, 2, 1, 5, 5, 1, 1]
\\           1
\\        /  |  \ \         S2wrS2 x S2
\\     2     5   8 9         16 automorphisms
\\    / \   / \
\\   3  4  6   7
\\
\\ Order 8 groups:
\\   C8,  C4xC2,  C2xC2xC2=E8, D8, Quaternion
\\
\\ n=10 new high 3  [0, 1, 2, 3, 3, 1, 6, 6, 1, 1]
\\
\\           1
\\        /  |  \ \         S2wrS2 x S2
\\     2     6   9 10
\\    /     / \
\\   3     7   9
\\  / \
\\ 4   5
\\
\\ ---------
\\ free forests
\\ forests count  1, 2, 3, 6, 10, 20, 37, 76, 153, 329
\\              n=1  2  3  4   5   6   7   8
\\ min gens       0, 1, 2, 2,  2,  3,  3,
\\ reps           1, 2, 1, 4,  7,  1,  3,
\\
\\ n=3  3 singletons = 2 gens
\\ n=6
\\     *  *   *  *
\\     |  |              16 automorphisms
\\     *  *                3 generators

\\ vector(10,n,vpar_count_free(n,0))

{
  my(verbose=0,
     forest=0);
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        max_gens=-1,
        reps=vector(n+1));
     \\ print();
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(perms=vpar_free_automorphisms(vpar));
           if(verbose, print1(vpar" auts "#perms" (Sn "if(test_perms_is_Sn(perms),n)")"));
           my(num_gens=test_perms_minimum_num_generators(perms));
           if(verbose, print(" num_gens="num_gens));
           \\ if(n==6 && num_gens==2, print(vpar));
           \\ if(n==13 && num_gens==6, print(vpar));
           reps[num_gens+1]++;
           if(num_gens>max_gens,
              \\ print("n="n" new high "num_gens"  "vpar);
              max_gens=num_gens));
     \\ print1(max_gens,", ");
     \\ print1(reps[max_gens+1],", ");
     \\ print(reps);
     \\ forstep(i=#reps,2,-1, if(reps[i]==0, reps=reps[^i]));
     print("n="n" max "max_gens", reps "reps);

     \\ if(reps==1,
     \\    print("\nn="n" unique "seen_vpar));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_protnums_vector() root=1 partition duplicates

part_dups(p) =
{
  my(ret=1,term=1);
  for(i=2,#p,if(p[i]==p[i-1],term++,ret*=term;term=1));
  ret*term;
}
check_equal(part_dups([1,1,2,2,2]), 2*3);
check_equal(part_dups([2,2]), 2);
print("root1_by_partitions");

partitions_undup(n) = \
  vecsum(apply(p->n!/prod(i=1,#p,p[i]!)/part_dups(p), partitions(n,,[2,n+2])));
vector(10,n,partitions_undup(n))

w=[1, 1, 2, 6, 23, 110, 617, 4035, 29927, 249926, 2316317, 23674841, 264329177]

quit


\\------------------------------------------------------------------------------
\\ vpar_protnums_vector() root=1 vector

root1_vector0_by_steps(n,max_k=n) =
{
  \\ a = unrestricted partitioning
  \\ v = forests with vertex 1 prot=0, biggest partition
  \\     make by choose t-1,k-1
  \\ w = v with any additional k after, so root=1 is prot>0

  \\ r+k   binomial(r+k,k)
  \\ r+2k  binomial(r+2k,k) * binomial(r+k,k)
  \\ r+3k  binomial(r+3k,k) * binomial(r+2k,k) * binomial(r+k,k)

  my(zero=vector(n+1,i,0),
     a=vector(n+1,i, (i-1)!),
     v=vector(n+1,i, if(i<2,1, (i-2)!)),
     w=vector(n+1,i, if(i<2,1, (i-2)!)));
  a=v=w=zero;
  a[1]=1;
  v[1]=1;
  w[1]=1;
  for(k=1,max_k,             \\ new term k
     my(new_a=a, new_v=v, new_w=w);
     for(r=0,#a,             \\ remainder, so t=r+extra
        my(b=1);
        forstep(e=k,n-r,k,   \\ e multiple of k terms
           my(t=r+e);        \\ total new size
           new_a[t+1] += a[r+1] * (b *= binomial(t,k)));
        b=1;
        forstep(e=k,min(n-r,k),k,   \\ new v
           my(t=r+e);
           b *= if(e==k, binomial(t-1,k-1), binomial(t-1,k));
           new_v[t+1] += a[r+1] * b;
        );
        forstep(e=2*k,n-r,k,  \\ new w
           my(t=r+e);
           b *= binomial(t-1,k);
           new_w[t+1] += a[r+1] * b;
        );
        b=1;
        forstep(e=k,n-r,k,   \\ extend v or w
           my(t=r+e);
           b *= binomial(t-1,k);
           new_w[t+1] += (v[r+1]+w[r+1]) * b;
        );
     );
     a=new_a; v=new_v; w=new_w);

  \\ print("a="a);
  \\ print("v="v);
  \\ print("w="w);

  \\ print("exp v="vpar_INTERNAL_exponential_transform(v[2..#v]));
  [a,v,w];
}
root1_vector0_by_steps(15)[3]

print("want");
[1, 1, 3, 13, 63, 371, 2433, 18446, 154423, 1445287, 14795283, 165999626, 2015699733]

limit=11;
{
  for(max_k=1,limit,
     print("w="root1_vector0_by_steps(limit-1,max_k)[3], " at max_k="max_k));
}
{
  \\ root=1 <=m
  my(forest=-1);
  for(n=1,limit,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_protnums_vector(vpar));
           n==1 || vec[1]!=0 || error();
           \\ vec[1]==1 || next;
           \\ vec[1]==vecmax(vec) || next;
           mapput(m,vec,1));
     my(counts=vector(limit));
     m=Mat(m);
     for(i=1,matsize(m)[1],
        my(vec=m[i,1]);
        vec=apply(n->n+1,vec);
        vec=Vec(vpar_INTERNAL_histogram(vec));
        my(mm=vecmax(vec));  \\ biggest term
        for(k=1,#counts,
           if(mm<=k, counts[k]++));
        );
     print("n="n"  "counts));
}

quit


\\------------------------------------------------------------------------------
\\ vpar_protnums_vector() root=1 vs forests

{
  \\ root=1 <=m
  for(n=1,8,
     my(mr=Map());
     forvec(vec=vpar_forvec_from(n,1),
           my(vpar=vpar_from_upascend(vec),
              prots=vpar_protnums_vector(vpar));
           if(vpar_roots(vpar)==[1], mapput(mr,prots,1)));

     my(seen=Map(),
        mf=Map());
     forvec(vec=vpar_forvec_from(n,1),
           my(vpar=vpar_from_upascend(vec),
              prots=vpar_protnums_vector(vpar));
           if(mapisdefined(seen,prots),next);
           mapput(seen,prots,1);

           my(hist=Vec(vpar_INTERNAL_histogram(apply(n->n+1,prots))));
           test_vector_is_decreasing(hist) || error();
           my(by_mr=mapisdefined(mr,prots),
              h=prots[1]+1,
              by_mf = (hist==[1]
                       || (h>1 && hist[h]>if(h+1>#hist,0,hist[h+1]))
                      ));
           if(by_mr != by_mf,
              print("prots="prots"  hist="hist" h="h"   mr "by_mr" vs mf "by_mf);
              error());
           if(by_mr, mapput(mf,prots,1)));

     print(mr==mf);
     \\ print("mr "Mat(mr));
     \\ print("mf "Mat(mf));
     mr==mf || error();
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_protnums_vector() root=1 log

v=[1, 1, 3, 13, 63, 371, 2433, 18446, 154423, 1445287, 14795283, 165999626, 2015699733, 26468281906, 372791768716, 5620156035323, 90196928404423, 1537249560753231, 27712940127091959, 527219161951661834];
g=test_func_to_egf(n->if(n==0,1,v[n]), #v+1)
h=log(g)
check_equal(exp(h),g);
test_egf_terms(h,#v)
\\ not in OEIS: 2, 4, 22, 98, 514, 3581, 21565, 203343, 1555913, 18285920
quit


\\------------------------------------------------------------------------------
\\ vpar_protnums_vector() distinct occurring

\\ part k include root=1, choose (n-1,k-1), rest must be <= k-1
\\ part k not include root=1, choose (n-1,k), rest <= k
\\ two parts k<=n/2 top with root=1 choose (n-1,k-1), and rest
\\ single part include root=1 only n==1
\\ n=3  [1, 0, 0]
\\      [2, 1, 0]
\\      [2, 0, 1]
\\ n=4  [1, 0, 0, 0]     1,3
\\      [1, 0, 0, 1]     2,2    \
\\      [1, 0, 1, 0]     2,2    |
\\      [1, 1, 0, 0]     2,2    /
\\      [2, 1, 0, 0]     1,1,2  \
\\      [2, 0, 1, 0]     1,1,2  |
\\      [2, 0, 0, 1]     1,1,2  /
\\      [3, 0, 2, 1]     1,1,1,1  \
\\      [3, 2, 0, 1]     1,1,1,1  |  6 of <=1
\\      [3, 0, 1, 2]     1,1,1,1  |
\\      [3, 1, 2, 0]     1,1,1,1  |
\\      [3, 1, 0, 2]     1,1,1,1  |
\\      [3, 2, 1, 0]     1,1,1,1  /
\\
\\

\\ all with terms <=m
Sle(n,m) =
{
  n>=1 || error("Sle want n>=1");
  m>=1 || error("Sle want m>=1");
  (m>=n) + sum(k=1,min(n-1,m), binomial(n,k)*Sle(n-k,k));
}
\\ root=1 forms with terms <=m
\\ bottom=1 for bottom-most prot, bottom=0 for other
Ple(n,m, bottom=1) =
{
  print("  Ple "n" "m);
  n>=1 || error("want n>=1");
  m>=0 || error("want m>=0");
  if(n==0, return(m>=0));
  if(n==1 && m>=1, return(1));

  sum(k=1,min(n-1,m),
      \\ print("  term n="n" k="k"  not root"" bin "binomial(n-1,k)" * rest "Ple(n-k,k, 0)" = "binomial(n-1,k)*Ple(n-k,k, 0));
      binomial(n-1,k)*(Ple(n-k,k, 0) + (n-k>=2 && n-k<=k)))

  + if(!bottom,
     sum(k=2,min(n-1,m),
         \\ print("  term n="n" k="k"  root here bin "binomial(n-1,k-1)" * rest "Sle(n-k,k-1)" (le "k-1")");
         binomial(n-1,k-1)*Sle(n-k,k-1)));
}
Ple=memoize(Ple);
vector(3,n,Ple(n,n+1));
Ple(1,3); Ple(1,3,0); Ple(2,2); Ple(2,2,0); Ple(3,1); Ple(3,1,0);
print("Ple(2,2) = "Ple(2,2));
print("Ple(2,2,0) = "Ple(2,2,0));
print();
print();
for(m=1,4, print("n="4" m="m);print("  is "Ple(4,m)));

limit=7;
for(n=1,limit,;for(m=1,n,(Ple(n,m)-Ple(n,m-1))));
for(n=1,limit,print1("n="n"     ");for(m=1,n,print1(Ple(n,m)-Ple(n,m-1),", ")); print());
{
  \\ root=1 by <=m
  my(forest=-1);
  for(n=2,limit,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec = vpar_protnums_vector(vpar));
           \\ if(n==4,print(vec));
           \\ if(n<=5 && vec[1]!=vecmax(vec), print(vec));
           mapput(m,vec,1));
     my(counts=vector(n));
     m=Mat(m);
     for(i=1,matsize(m)[1],
        my(vec=m[i,1]);
        vec=apply(n->n+1,vec);
        vec=Vec(vpar_INTERNAL_histogram(vec));
        \\ if(n==4,print(vec));
        counts[vecmax(vec)+1]++);
     print("n="n"  "counts"  total "vecsum(counts)));
}

vector(20,n,Ple(n,n))
[1, 1, 3, 13, 63, 371, 2433, 18446]
quit

\\ root=1 can go in first term, or in a new bigger term other than the last
n=9;
parts=partitions(n,,[2,n+2]);
{
  my(total=0);
  for(i=1,#parts,
     my(p=parts[i]);
     for(i=1,#p-1,
        if(i==1 || p[i]>p[i-1],
           p[i]--;
           total += test_multinomial(p);
           p[i]++)));
  print(total);
}
quit

{
  \\ A226874
  \\ A292222
  \\    multinomials over partitions of length t
  \\
  \\ times length
  \\ 0,
  \\ 1, 0,
  \\ 1, 2, 0,
  \\ 1, 6, 6, 0,
  \\ 1, 14, 24, 24, 0,
  \\ 1, 30, 100, 120, 120, 0,
  \\ 1, 62, 330, 600, 720, 720, 0,
  \\ 1, 126, 1099, 2940, 4200, 5040, 5040, 0,
  \\ 1, 254, 3416, 12992, 23520, 33600, 40320, 40320, 0,
  \\ 1, 510, 10644, 54684, 139608, 211680, 302400, 362880, 362880, 0,

  for(n=2,10,
     for(t=1,n-1,
        my(total=0);
        forpart(p=n-1,
                #p==t || error();
                total += test_multinomial(p)
                               \\ * #Set(p)
                               , ,[t,t]);
        print1(total,", "));
     print());
  quit;
}
vector(10,n, protnums_vectors_count(n))

protnums_vectors_count(n) =
{
  \\ for root=1
  \\ v=1 at last of each set of same terms, so num distinct terms
  my(total=0);
  forpart(p=n-1, total += test_multinomial(p) * #Set(p));
  total;
}
vector(10,n, protnums_vectors_count(n))

{
  \\ labelled

  \\ forests distinct protection numbers vectors
  \\ 1, 3, 10, 47, 246, 1602, 11481, 95503,
  \\
  \\ A005651 sum multinomial coeffs over partitions
  \\ Protection numbers are hierarchical per comment in A005651 by Thomas
  \\ Wieder since how many num=0 must be >= how many num=1, etc.
  \\ Same as vpar_subtree_heights_vectors_count(), partition is how many of
  \\ each protection number, permutations to assign vertices to them.
  \\ Those of prot=0 are childless.  Those prot=1 their parents.
  \\ Those prot=1 parent of prot=2.  Can do other ways, but that makes
  \\ a forest of required prot numbers vector.

  \\ trees distinct protection numbers vectors
  \\ not in OEIS: 1, 2, 9, 46, 245, 1601, 11480,
  \\ A005651 - 1
  \\ forests-1 since can make all partitions except every prot==0
  \\
  \\ n=2 [1,0] [0,1]  forest [0,0]

  \\ root=1 distinct protection numbers vectors
  \\              1, 1, 3, 13, 63, 371, 2433, 18446,
  \\ not in OEIS:       3, 13, 63, 371, 2433, 18446,
  \\
  \\ v=1 root has no parent, so its prot number can only be an element of
  \\ the partition which is > next bigger prot, which is next smaller
  \\ partition term, otherwise not enough to make those bigger

  my(forest=-1);
  for(n=1,8,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec = vpar_protnums_vector(vpar));
           \\ if(n==2,print(vec));
           mapput(m,vec,1));
     print1(#m", "));
  quit;
}
{
  \\ unlabelled

  \\ trees distinct protection numbers vectors
  \\ 1,1,2,4,8,16,35,75,168,382,876,2035,
  \\ not in OEIS: 2,4,8,16,35,75,168,382,876,2035,

  \\ forests distinct protection numbers vectors
  \\ not in OEIS: 1,2,3,6,11,24,49,111,243,556,1270,2960,
  \\

  my(forest=0);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(vec = vpar_protnums_vector(vpar));
           mapput(m,vec,1));
     print1(#m","));
  quit;
}
{
  \\ ordered

  \\ trees distinct protection numbers vectors
  \\ 1, 1, 2, 5, 11, 27, 64, 158, 388, 971, 2435, 6170,
  \\ not in OEIS: 2, 5, 11, 27, 64, 158, 388, 971, 2435, 6170,

  \\ forests distinct protection numbers vectors
  \\ not in OEIS: 1, 2, 4, 9, 20, 47, 111, 269, 657, 1628,
  \\ is cumulative trees

  vector_cumulative = ((v) -> for(i=2,#v,v[i]+=v[i-1]); v);
  print(vector_cumulative([1,1,2,5,11,27,64,158,388,971,2435,6170]));

  my(forest=1);
  for(n=1,12,
     my(m=Map(),
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(vec = vpar_protnums_vector(vpar));
           mapput(m,vec,1));
     print1(#m","));
  quit;
}


\\------------------------------------------------------------------------------
\\ A005651 sum multinomial over partitions

\\ not in OEIS: 1,3,1,10,1,1,47,7,1,1,246,11,1,1,1,1602,126,21,1,1,1,11481,267,36,1,1,1,1,95503,3655,127,71,1,1,1,1,
\\ not in OEIS: 1,1,3,1,1,10,1,1,7,47,1,1,1,11,246,1,1,1,21,126,1602,1,1,1,1,36,267,11481,1,1,1,1,71,127,3655,95503,
\\ not in OEIS: 3,10,1,47,7,1,246,11,1,1,1602,126,21,1,1,11481,267,36,1,1,1,95503,3655,127,71,1,1,1,
\\ not in OEIS: 3,1,10,1,7,47,1,1,11,246,1,1,21,126,1602,1,1,1,36,267,11481,1,1,1,71,127,3655,95503,

\\ Vladimir Kruchinin in A005651, with A005651 = S(n,1)
\\ S(n,m) = sum multinomials over partitions with all terms >= m
\\ new term k>=m onto n-k
\\ (n-k)!/terms!  * n!/(n-k)!/k!  = n!/terms!/k!
\\    m=1
\\ n=1  1,
\\ n=2  3,1,
\\      10,1,1,
\\      47,7,1,1,
\\      246,11,1,1,1,
\\      1602,126,21,1,1,1,
\\ n=7  11481,267,36,1,1,1,1,
\\ n=8  95503,3655,127,71,1,1,1,1,
S_ge(n,m) =
{
  n>=1 || error();
  m>=1 || error();
  m<=n || error();
  1 + sum(k=m,n>>1, binomial(n,k) * S_ge(n-k,k));
}
S_ge=memoize(S_ge);
print("S_ge");
for(n=1,8,for(m=1,n,print1(S_ge(n,m),",")); print());
{
  for(n=1,6,
     for(m=1,n,
        my(count=0);
        forpart(p=n, if(vecmin(Vec(p))>=m, count += test_multinomial(p)));
        check_equal(count,S_ge(n,m))));
}
check_equal(S_ge(8,1), 95503);
check_equal(S_ge(7,1), 11481);
check_equal(S_ge(6,2), 126);
check_equal(S_ge(5,3), 1);
check_equal(S_ge(4,4), 1);
check_equal(1 + 11481*binomial(8,1)+ 126*binomial(8,2) + 1*binomial(8,3) + 1*binomial(8,4), 95503);

\\ biggest term <=m
\\ 1,
\\ 2,3,
\\ 6,9,10,
\\ 24,42,46,47,
\\ 120,210,240,245,246,
\\ 720,1350,1550,1595,1601,1602,
\\ 5040,9450,11060,11410,11473,11480,11481,
\\ 40320,78120,91560,94850,95410,95494,95502,95503,
S_le(n,m) =
{
  n>=1 || error("S_le want n>=1, got "n);
  m>=1 || error();
  sum(k=1,min(n,m), S_eq(n,k));
}
\\ biggest term ==m
S_eq(n,m) =
{
  n>=1 || error();
  m>=1 || error();
  m<=n || error();
  if(n-m < 0, return(0));
  if(n-m == 0, return(1));
  binomial(n,m)*S_le(n-m,m);
}
S_eq=memoize(S_eq);
print("S_le");
for(n=1,8,for(m=1,n,print1(S_le(n,m),",")); print());
{
  for(n=1,8,
     for(m=1,n,
        my(count=0);
        forpart(p=n, if(p[#p]==m, count += test_multinomial(p)));
        check_equal(count,S_eq(n,m));
        \\ print("n="n" m="m"  "count"  "S_eq(n,m));
     ));
}
{
  for(n=1,8,
     for(m=1,n,
        my(count=0);
        forpart(p=n, count += test_multinomial(p), m);
        check_equal(count,S_le(n,m))));
}

S_le_by_le(n,m) =
{
  n>=1 || error();
  m>=1 || error();
  (m>=n) + sum(k=1,min(n-1,m), binomial(n,k)*S_le_by_le(n-k,k));
}
\\ for(n=1,8,for(m=1,n,print1(S_le(n,m),",")); print());
\\ for(n=1,8,for(m=1,n,print1(S_le_by_le(n,m),",")); print());
\\ quit
{
  for(n=1,8,
     for(m=1,n,
        \\ print("n="n" m="m"  "S_le_by_le(n,m)"  "S_le(n,m));
        check_equal(S_le_by_le(n,m), S_le(n,m));
     ));
}

Srm_le(r,m) = S_le(r+m,m);
Srm_le_by_rm(r,m) =
{
  my(n=r+m);
  n>=1 || error();
  m>=1 || error();
  (r+m==m) + sum(k=1,min(r+m-1,m), binomial(r+m,k)*Srm_le(r-k+m-k,k));
}
{
  for(n=1,8,
     for(m=1,n,
        \\ print("n="n" m="m"  "S_le_by_le(n,m)"  "S_le(n,m));
        check_equal(Srm_le_by_rm(n,m), Srm_le(n,m));
     ));
}

Srm_ge(r,m) = S_ge(r+m,m);
Srm_ge_by_rm(r,m) =
{
  my(n=r+m);
  n>=1 || error();
  m>=1 || error();
  1 + sum(k=m,(r+m)>>1, binomial(r+m,k) * Srm_ge(r+m-k-k,k));
}
{
  for(n=1,8,
     for(m=1,n,
        check_equal(Srm_ge_by_rm(n,m), Srm_ge(n,m))));
}

Spm_ge(p,m) = S_ge(p-m,m);
Spm_ge_by_pm(p,m) =
{
  my(n=p-m);
  n>=1 || error();
  m>=1 || error();
  1 + sum(k=m,(p-m)>>1, binomial(p-m,k) * Spm_ge(p-m,k));
}
{
  for(n=1,8,
     for(m=1,n,
        check_equal(Spm_ge_by_pm(n+m,m), Spm_ge(n+m,m))));
}


\\ not in OEIS: 2,6,9,24,42,46,120,210,240,245,720,1350,1550,1595,1601,5040,9450,11060,11410,11473,11480,40320,78120,91560,94850,95410,95494,95502
for(n=1,8,for(m=1,n-1,print1(S_le(n,m),",")); print1());
print();

for(n=1,8,for(m=1,n-1,print1(S_ge(n,m),",")); print1());
print();
for(n=1,8,forstep(m=n-1,1,-1,print1(S_ge(n,m),",")));
print();

\\ by number of terms
\\ not in OEIS: 1,1,2,1,3,6,1,10,12,24,1,15,50,60,120,1,41,180,300,360,720,
{
  for(n=1,6,
     for(m=1,n,
        my(count=0);
        forpart(p=n, count += test_multinomial(p), ,[m,m]);
        print1(count,",")));
}
print();

\\ by biggest term
{
  for(n=1,6,
     for(m=1,n,
        my(count=0);
        forpart(p=n,
                if(vecmin(Vec(p))>=m,
                \\ if(#p<=m,
                   count += test_multinomial(p)));
        print1(count,",")));
}
print();

a_parts_le(n,m) =
{
  my(count=0);
  forpart(p=n, if(vecmax(Vec(p))<=m, count += test_multinomial(p)));
  count;
}


\\ n=1  1
\\ n=2  3     [1,1]   [2] in 2 labellings

\\ n=4  [1,1,1,1] is 4!=24
\\      k=2  [1,1,2]  is a(2)=2 * binomial(4,2)=6 = 12
\\           [2,2]    is a(0)=1 * binomial(4,2)=6 * binomial(2,2)=1 = 6
\\ 4!/1!/1!/2! == 12
\\ 4!/2!/2! == 6
\\ 24+12+6 == 42

a=vector(11,i, 0);
{
  for(k=1,#a,            \\ new term k
     print("k="k"  a="a"  cf "vector(#a,n,a_parts_le(n,k-1)));
     my(new_a=a);
     for(r=0,#a,         \\ remainder, so n=r+extra
        my(b=1);
        for(m=1,#a,      \\ multiple m of new term
           my(n=r+m*k);  \\ m*k extra by new term
           if(n>#a,break);
           b *= binomial(n,k);
           b==prod(i=1,m, binomial(r+i*k,k)) || error();
           print(" m="m" r="r" to n="n"  b="b" times a[r]="if(r==0,1,a[r]));
           new_a[n] += b * if(r==0,1,a[r])));
     a=new_a);
}
a
A005651_samples[2..12]

\\ WRONG
\\ a=vector(5,i, 0);
\\ {
\\   for(n=1,#a,
\\      print("n="n"  a="a);
\\      for(k=1,n,
\\         for(m=1,#a,
\\            my(e=m*k, r=n-e);
\\            if(e>n,break);
\\            print(" k="k" m="m"  binomial "binomial(n,e)"*"m!" times "if(r==0,1,a[r]));
\\            a[n] += binomial(n,e)*m! * if(r==0,1,a[r]))));
\\ }
\\ a

a_vector(n) =
{
  \\ m limit
  \\   t = r+m*k <= n
  \\   m*k <= n-r
  \\   m <= floor (n-r)/k

  my(a=vector(n,i,i!));
  for(k=2,#a,            \\ new term k
     \\ print("k="k"  a="a"  cf "vector(#a,n,a_parts_le(n,k-1)));
     my(new_a=a);
     for(r=0,#a,          \\ remainder, so t=r+extra
        my(b=1);
        for(m=1,(n-r)\k,  \\ multiple m of new term
           my(t=r+m*k);   \\ m*k extra by new term
           new_a[t] += if(r==0,1,a[r]) * (b *= binomial(t,k))));
     a=new_a);
  a;
}
a_vector(11)

a_vector0(n,max_k=n) =
{
  \\ m limit
  \\   t = r+m*k <= n
  \\   m*k <= n-r
  \\   m <= floor (n-r)/k

  \\ repeat bin
  \\ k=2 r=0 e=10 t=10  bin 10,2
  \\ k=2 r=2 e=8 t=10  bin 10,2
  \\ k=2 r=4 e=6 t=10  bin 10,2
  \\ k=2 r=6 e=4 t=10  bin 10,2
  \\ k=2 r=8 e=2 t=10  bin 10,2

  my(a=vector(n+1,i,(i-1)!));
  for(k=2,max_k,             \\ new term k
     my(new_a=a);
     for(r=0,#a,             \\ remainder, so t=r+extra
        my(b=1);
        forstep(e=k,n-r,k,   \\ e multiple of k terms
           my(t=r+e);   \\ m*k extra by new term
           new_a[t+1] += a[r+1] * (b *= binomial(t,k));
           print("k="k" r="r" e="e" t="t"  bin "t","k" for b="b);
        ));
     a=new_a);
  a;
}
a_vector0(11)
quit

print();
vector(10,n,a_parts_le(n,2))
\\ not in OEIS: 1, 3, 9, 42, 210, 1350, 9450, 78120, 703080, 7144200
a_vector0(11,2)[2..11] == vector(10,n,a_parts_le(n,2))
a_vector0(11,3)[2..11] == vector(10,n,a_parts_le(n,3))
a_vector0(11,5)[2..11] == vector(10,n,a_parts_le(n,5))
a_vector0(11,10)[2..11] == vector(10,n,a_parts_le(n,10))

print("sd");
sd_vector(n) =
{
  my(a=vector(n)); a[1]=1;
  my(b=a,
     f=a);  \\ factorials f[n]=n!
  for(n=2,n,
     f[n] = f[n-1]*n;
     b[n] = sumdiv(n,d,
                   denominator(f[n-1] * d / f[d]^(n/d))==1 || error();
                   f[n-1] * d / f[d]^(n/d));
     a[n] = b[n] + sum(k=1,n-1, binomial(n-1,k-1) * b[k] * a[n-k]));
  print("b="b);
  a;
}
sd_vector(10)
A005651_samples[2..12]

\\ WRONG
\\ print();
\\ print("a_vector0_t");
\\ a_vector0_t(n,max_k=n) =
\\ {
\\   \\ m limit
\\   \\   t = r+m*k <= n
\\   \\   m*k <= n-r
\\   \\   m <= floor (n-r)/k
\\
\\   my(a=vector(n+1,i,(i-1)!));
\\   for(k=2,max_k,             \\ new term k
\\      my(new_a=a);
\\      my(b=1);
\\      forstep(t=k,n,k,
\\         b *= binomial(t,k);
\\         forstep(e=k,1000,k,
\\            if(t-e+1<0,break);
\\            new_a[t+1] += a[t-e+1] * b));
\\      a=new_a);
\\   a;
\\ }
\\ a_vector0_t(12)
\\ A005651_samples[1..12]

quit


\\------------------------------------------------------------------------------
\\ vpar_protnum() counts

{
  \\ labelled
  \\ forests
  \\
  \\ [1]
  \\ [1, 2]
  \\ [7, 3, 6]
  \\ [49, 40, 12, 24]
  \\ [491, 425, 200, 60, 120]
  \\ [6181, 5436, 2910, 1200, 360, 720]
  \\
  \\ forests num=0, is >=1 singleton
  \\ not in OEIS: 1, 1, 7, 49, 491, 6181, 94305,
  \\ vector(8,n, vpar_count(n,1) - vpar_count_no_singletons(n));

  my(forest=1);
  for(n=1,8,
     my(counts=vector(n));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           counts[vpar_protnum(vpar)+1]++);
     print1(counts[1],", ");
     \\ print(counts);
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ Protection Number

\\ not in OEIS: 1.6229713847153530495146582031843

{
  \\ k=2
  \\ A000957 Fine's
  \\
  \\ k=3
  \\ A114627
  my(limit = 10,
     L = O(x^limit),
     gR0 = (1-sqrt(1-4*x+L))/2,     \\ Catalan, num ordered rooted
     gR = (k)-> (1-x) * x^(k-2) * gR0^3
                / (1 + x^(k-2) * gR0^3 ));
  print(test_gf_terms(gR0, 10));
  for(k=1,10,
     print("k="k"  "test_gf_terms(gR(k), 15)));
}
quit


\\-----------------------------------------------------------------------------
\\ vpar_is_automorphisms_le() Abelian Count

{
  \\ trees rooted unlabelled
  \\ not in OEIS: 2, 3, 7, 15, 33, 77, 184, 446, 1098, 2737, 6893, 17522,
  \\            n=3  4  5
  \\
  \\ forests rooted unlabelled, one earlier
  \\ root=1 trees same
  \\ not in OEIS: 1, 2, 3, 7, 15, 33, 77, 184, 446, 1098, 2737, 6893,
  \\
  \\ <=2 of any asymmetric
  \\ <=1 of symmetric Abelian
  \\
  \\           n=1  2  3  4  5   6
  \\ Abelian     1, 1, 2, 3, 7, 15
  \\ Asymmetric  1, 1, 1, 2, 3,  6
  \\ Sym Ab      0, 0, 1, 1, 4,  9
  \\ n=5 want 7
  \\   1+ 4              3 ways
  \\   1+ 3 + 1          2 ways
  \\   1+ 2 + 2          1 way with asymmetric 2
  \\   1+ 2 + 1 + 1      1 way with asymmetric 1
  \\   1+ 1 + 1 + 1 + 1  0 ways
  \\
  \\ symmetric Abelians, at least one sibling pair, no nesting
  \\ not in OEIS: 1, 4, 9, 21, 52, 132, 333, 851, 2189

  my(limit=15,
     L=O(x^limit),
     g=x+L);      \\ c*x^n = how many n vertex trees
  for(n=1,limit-1,
     my(a = vpar_count_unlabelled_asymmetric(n),
        s = polcoeff(g,n) - a);
     print("n="n" "polcoeff(g,n)" - "a"  s="s);
     g *= (1 + x^n +L          )^s   \\ 1 Abelian symmetric
        * (1 + x^n + x^(2*n) +L)^a;  \\ 1 or 2 asymmetric
     print("  "Vec(g)));
  print(g);
  print(Vec(g));

  my(forest=0);
  for(n=1,24,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count += vpar_is_automorphisms_le(vpar,2,1));
     print1(count,", ");
     );
  quit;
}
{
  \\ free trees free automorphisms Abelian
  \\ not in OEIS: 1, 1, 1, 1, 2, 3, 5, 11, 23, 49, 110, 249, 580, 1366, 3268,

  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_automorphisms_le(vpar_INTERNAL_for_free_automorphisms(vpar),2,1));
     print1(count,", ");
     );
  quit;
}
{
  \\ trees ordered rooted
  \\ not in OEIS: 1, 1, 2, 4, 12, 33, 98, 303, 958, 3140, 10302, 34417,
  my(forest=0);
  for(n=1,20,
     my(vpar=vpar_preorder_first(n),
        count=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           count += vpar_is_automorphisms_le(vpar,2,1));
     print1(count,", ");
     );
  quit;
}
{
  \\ trees labelled rooted
  \\ not in OEIS: 1, 2, 9, 60, 600, 7380, 109620,

  my(forest=0);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_automorphisms_le(vpar,2,1));
     print1(count,", "));
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_is_automorphisms_le() Supersolvable Count

{
  \\ free trees
  \\ 1, 1, 1, 2, 2, 5, 9, 18, 36, 82, 182, 425, 999, 2410, 5866,
  \\ not in OEIS: 2, 5, 9, 18, 36, 82, 182, 425, 999, 2410, 5866,

  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_automorphisms_le(vpar_INTERNAL_for_free_automorphisms(vpar),3,2));
     print1(count,", ");
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ Group S3 Generators

K = nfsplitting(x^3 + 2);  \\ S3
G=galoisinit(K);
smallgroup=[G.gen,G.orders];

install(group_elts, "GL", "group_elts");
check_equal(group_elts(smallgroup,#G.group), G.group);
check_equal(test_perm_commute(G.gen[1],G.gen[2]), 0);

print("elems ");
{
  my(id=vpar_INTERNAL_identity_perm(#G.group),
     rev=Vecsmall(Vecrev(id)));
  for(i=1,#G.group,
     my(g=G.group[i]);
     for(j=1,#g, printf(" %2d", g[j]));
     printf("   order %d", permorder(g));
     printf("   exponents %3d", fromdigits(test_smallgroup_elem_to_exponents(smallgroup,g)));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}
print("gens");
{
  for(i=1,#G.gen,
     my(g=G.gen[i]);
     for(j=1,#g, printf(" %2d", g[j]));
     print1("    "apply(Vec,test_perm_to_cycles(g)));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}
print("num gens ",#G.gen);
print("orders ",apply(permorder,G.gen));

print("elems by exponents");
{
  my(id=vpar_INTERNAL_identity_perm(#G.group),
     rev=Vecsmall(Vecrev(id)));
  for(i=1,#G.group,
     my(g=G.group[i]);
     printf(" %3d = ",fromdigits(test_smallgroup_elem_to_exponents(smallgroup,g)));
     for(j=1,#g, printf(" %03s",fromdigits(test_smallgroup_elem_to_exponents(smallgroup,G.group[g[j]]))));
     printf("   order %d", permorder(g));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     for(j=1,#G.gen,
        if(g==G.gen[j],
           print1("  gen "j)));
     print());
}

{
  print("Cayley table");
  my(m=test_perms_Cayley_table(G.group));
  for(r=1,matsize(m)[1],
     print("  "m[r,]));
}

whole=test_perms_generators_to_group(G.gen);
{
  for(i=1,#G.gen,
     my(sub=test_perms_generators_to_group(G.gen[1..i]));
     print("size "#sub"   "test_perms_is_normal_subgroup(whole,sub)));
}
print();
{
  for(i=1,#G.gen,
     my(sub=test_perms_generators_to_group(G.gen[i..#G.gen]));
     print("size "#sub"   "test_perms_is_normal_subgroup(whole,sub)));
}

\\ S2 = test_perms_Sn(2);
\\ S2xS2 = test_perms_direct_product(S2,S2);
\\ C4 = test_perms_Cn(4);
\\ {
\\   my(gens=G.gen[1..2],
\\      perms=test_perms_generators_to_group(gens));
\\   print("1,2 orders ",apply(permorder,perms));
\\   check_equal(test_perms_is_isomorphic(perms, S2xS2), 1);
\\ }
\\ {
\\   my(gens=G.gen[2..3],
\\      perms=test_perms_generators_to_group(gens));
\\   print("2,3 orders ",apply(permorder,perms));
\\ }
\\ {
\\   my(gens=[G.gen[1], G.gen[3]],
\\      perms=test_perms_generators_to_group(gens));
\\   print("1,3 orders ",apply(permorder,perms));
\\   check_equal(#perms, 12);
\\ }
quit


\\------------------------------------------------------------------------------
\\ vpar_automorphism_group()

vpar=vpar_make_complete_tree(2,2)
vpar_automorphism_group(vpar)
quit


\\------------------------------------------------------------------------------
\\ Group Action S2xS2 wr S2

S2 = test_perms_Sn(2);
S2xS2 = test_perms_direct_product(S2,S2);
S2pow4 = test_perms_direct_power(S2,4);
S2xS2_wr_S2=test_perms_wreath_product(S2xS2,S2);

gen_to_bitpos = [1,2,3,4,5];
ord = 2^#gen_to_bitpos;
gens=vector(#gen_to_bitpos,i, vectorsmall(ord));
id=vpar_INTERNAL_identity_perm(#gens);
gen_perm = [id,id, id,id, Vecsmall([3,4, 1,2, 5])];
{
  for(b=0,ord-1,
     for(g=1,#gen_to_bitpos,
        my(p=gen_to_bitpos[g]);
        print("b="b" g="g" p="p);
        forstep(k=#gen_to_bitpos,g+1,-1,
           if(bittest(b,k-1),
              print(" k="k" p="p" -> "gen_perm[k][p]);
              p=gen_perm[k][p];
             ));
        print(" xor p="p);
        gens[g][b+1] = bitxor(b, 1<<(p-1)) + 1;
     ));
}
{
  for(g=1,#gen_to_bitpos,
     for(n=1,ord,
        printf(" %2d",gens[g][n]-1));
     print());
}
test_perms_validate(gens);
perms=test_perms_generators_to_group(gens[1..1]);
check_equal(test_perms_is_isomorphic(perms, S2), 1);
perms=test_perms_generators_to_group(gens[1..2]);
check_equal(test_perms_is_isomorphic(perms, S2xS2), 1);
perms=test_perms_generators_to_group(gens[3..4]);
check_equal(test_perms_is_isomorphic(perms, S2xS2), 1);
perms=test_perms_generators_to_group(gens[1..4]);
check_equal(test_perms_is_isomorphic(perms, S2pow4), 1);
perms=test_perms_generators_to_group(gens[5..5]);
check_equal(test_perms_is_isomorphic(perms, S2), 1);

pgens=vector(4,i,i--; vectorsmall(16,n,n--; bitxor(n,1<<i)+1))
test_perms_validate(pgens);
perms=test_perms_generators_to_group(pgens);
check_equal(test_perms_is_isomorphic(perms, S2pow4), 1, "S2pow4");

perms=test_perms_generators_to_group(gens);
print("gens orders ",apply(test_permorder,perms));
check_equal(test_perms_is_group(perms), 1);
check_equal(test_perms_is_isomorphic(perms, S2xS2_wr_S2), 1, "S2xS2_wr_S2");

pgens=concat(pgens, [vectorsmall(16,n,n--; (bitand(n,3)<<2) + (bitand(n,3<<2)>>2) + 1)]);
pgens[5]
test_perms_validate(pgens);
perms=test_perms_generators_to_group(pgens);
print("S2xS2_wr_S2 orders ",vecsort(apply(test_permorder,S2xS2_wr_S2)));
print("pgens orders       ",vecsort(apply(test_permorder,perms)));
check_equal(test_perms_is_group(perms), 1);
check_equal(test_perms_is_isomorphic(perms, S2xS2_wr_S2), 1, "S2xS2_wr_S2");

quit


\\------------------------------------------------------------------------------
\\ vpar_num_free_orbits() total -- free

\\ trees
\\ n=1  2  3
\\   1, 1, 2, 4, 9, 20, 48, 115, 286, 719, 1842, 4766, 12486, 32973,
\\ A000081 num rooted trees
\\
\\ forests
\\ n=1  2  3
\\   1, 2, 5, 11, 26, 59, 140, 331, 804, 1972, 4928, 12456, 31924, 82659,
\\ not in OEIS: 2, 5, 11, 26, 59, 140, 331, 804, 1972, 4928, 12456, 31924,
\\ one rooted component, rest free forest
\\ free forests with one distinguished vertex
\\
\\ forests - trees
\\ [1, 2, 5, 11, 26, 59, 140, 331, 804, 1972, 4928, 12456, 31924, 82659] - \
\\ [1, 1, 2,  4,  9, 20,  48, 115, 286,  719, 1842,  4766, 12486, 32973]
\\ 0, 1, 3, 7, 17, 39, 92, 216, 518, 1253, 3086, 7690, 19438, 49686
\\ not in OEIS: 1, 3, 7, 17, 39, 92, 216, 518, 1253, 3086, 7690, 19438, 49686
\\
\\ n=3  5 total orbits
\\ GP-Test  vpar_count_free(3,1) == 3  /* forests */
\\ * * *      *--* *     *--*--*

vector(10,n, sum(i=1,n, vpar_count_unlabelled(i) * vpar_count_free(n-i,1)))
{
  my(forest=1);
  for(n=1,20,
     my(total=0,
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           total += vpar_num_free_orbits(vpar));
     print1(total,", "));
  quit;
}


\\------------------------------------------------------------------------------
\\ S3 Cayley Tables

S3_orig = test_perms_Sn(3);
S3 = vecsort(S3_orig, (x,y)->lex(Vecrev(x),Vecrev(y)), 4)
check_equal(test_perms_is_isomorphic(S3,S3_orig), 1);
n=#S3;
{
  my(seen=Map(),
     count=0);
  for(i=0,n!-1,
     my(p=numtoperm(n,i));
     if(p[1]!=1, next);
     my(perms=vecextract(S3,p),
        m=test_perms_Cayley_table(perms));
     if(mapisdefined(seen,m), next);
     if(m[2,]!=[2,1,4,3,6,5] && m[2,]!=[2,1,6,5,4,3], next);
     \\ if(m[4,]!=[4,3,2,1,6,5] && m[2,]!=[4,3,6,5,2,1], next);
     \\ if(m[2,2]!=1, next);
     \\ if(m[4,2]!=3, next);
     mapput(seen,m,p);
     print(Vec(p));
     for(i=1,n,
        print1("g"i"    ");
        for(j=1,n,
           printf("%d,",m[i,j]));
        print("   "Vec(perms[i])));

     for(i=1,n,
        my(i_perms=test_perms_generators_to_group([perms[i]]));
        for(j=1,n,
           my(j_perms=test_perms_generators_to_group([perms[i],perms[j]]));
           if(test_perms_is_isomorphic(j_perms,S3)
              && test_perms_is_normal_subgroup(j_perms,i_perms),
              print("  normal gens "i" "j" sizes "#i_perms" "#j_perms))));

     count++;
  );
  print("count "count);
}

p = [1, 2, 4, 3, 5, 6];
perms=vecextract(S3,p);
for(i=1,#perms,print(Vec(perms[i])));

gens=[Vecsmall([2,1,6,5,4,3]), \
      Vecsmall([3,4,5,6,1,2])];
perms=test_perms_generators_to_group(gens);
check_equal(test_perms_is_isomorphic(S3,perms), 1);

gens=[Vecsmall([2,1, 6,5, 4,3]), \
      Vecsmall([4,3, 2,1, 6,5])];
perms=test_perms_generators_to_group(gens);
check_equal(test_perms_is_isomorphic(S3,perms), 1);

quit


\\------------------------------------------------------------------------------
\\ Groups

\\ src/test/in/galois
\\ src/test/32/galois

\\ [2, -1, 1, "S2"]1              T2_1 = x^2+x+1;
\\ [3, 1, 1, "A3"]1               T3_1 = x^3+x^2-2*x-1;
\\ [6, -1, 2, "S3"]1              T3_2 = x^3+2;
\\ [4, -1, 1, "C(4) = 4"]1        T4_1 = x^4+x^3+x^2+x+1;
\\ [4, 1, 2, "E(4) = 2[x]2"]1     T4_2 = x^4+1;
\\ [8, -1, 3, "D(4)"]1            T4_3 = x^4-2;
\\ [12, 1, 4, "A4"]1              T4_4 = x^4+8*x+12;
\\ [24, -1, 5, "S4"]1             T4_5 = x^4+x+1;

p=x^4-x^3-3*x^2+x-1;
p=x;
p = x^2+x+1;   \\ S2
p = x^3+x^2-2*x-1;  \\ A3=C3
p = x^3 + 2;   \\ S3, no group
p = x^4+x^3+x^2+x+1;   \\ C4
p = x^4 + x + 1;     \\ S4
p = x^4-42*x^2+144;  \\ E(4) = 2[x]2
p = x^4 + 1;         \\ E(4) = 2[x]2
p = x^4 + 8*x + 12;  \\ A4
p = x^6 + 108;   \\ D_6(6) = [3]2
p = nfsplitting(x^4+x+1);  \\ S4
\\ polgalois(p)
G=galoisinit(p)
G.group
print("gens ",G.gen);
print("num gens ",#G.gen);
quit



\\------------------------------------------------------------------------------
\\ Group A4 Generators

K = nfsplitting(x^4+8*x+12);  \\ A4
G=galoisinit(K);
smallgroup=[G.gen,G.orders];

install(group_elts, "GL", "group_elts");
check_equal(group_elts(smallgroup,#G.group), G.group);

\\ G.group[2]*G.group[5]
\\ check_equal(G.group[2]*G.group[2], []);

print("elems ");
{
  my(id=vpar_INTERNAL_identity_perm(#G.group),
     rev=Vecsmall(Vecrev(id)));
  for(i=1,#G.group,
     my(g=G.group[i]);
     for(j=1,#g, printf(" %2d", g[j]));
     printf("   order %d", permorder(g));
     printf("   exponents %3d", fromdigits(test_smallgroup_elem_to_exponents(smallgroup,g)));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}
print("gens");
{
  for(i=1,#G.gen,
     my(g=G.gen[i]);
     for(j=1,#g, printf(" %2d", g[j]));
     print1("    "apply(Vec,test_perm_to_cycles(g)));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}
print("num gens ",#G.gen);
print("orders ",apply(permorder,G.gen));

print("elems by exponents");
{
  my(id=vpar_INTERNAL_identity_perm(#G.group),
     rev=Vecsmall(Vecrev(id)));
  for(i=1,#G.group,
     my(g=G.group[i]);
     printf(" %3d  ",fromdigits(test_smallgroup_elem_to_exponents(smallgroup,g)));
     for(j=1,#g, printf(" %03s",fromdigits(test_smallgroup_elem_to_exponents(smallgroup,G.group[g[j]]))));
     printf("   order %d", permorder(g));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}

{
  print("Cayley table");
  my(m=test_perms_Cayley_table(G.group));
  for(r=1,matsize(m)[1],
     print("  "m[r,]));
}

whole=test_perms_generators_to_group(G.gen);
{
  for(i=1,#G.gen,
     my(sub=test_perms_generators_to_group(G.gen[1..i]));
     print("size "#sub"   "test_perms_is_normal_subgroup(whole,sub)));
}
print();
{
  for(i=1,#G.gen,
     my(sub=test_perms_generators_to_group(G.gen[i..#G.gen]));
     print("size "#sub"   "test_perms_is_normal_subgroup(whole,sub)));
}

S2 = test_perms_Sn(2);
S2xS2 = test_perms_direct_product(S2,S2);
C4 = test_perms_Cn(4);
{
  my(gens=G.gen[1..2],
     perms=test_perms_generators_to_group(gens));
  print("1,2 orders ",apply(permorder,perms));
  check_equal(test_perms_is_isomorphic(perms, S2xS2), 1);
}
{
  my(gens=G.gen[2..3],
     perms=test_perms_generators_to_group(gens));
  print("2,3 orders ",apply(permorder,perms));
}
{
  my(gens=[G.gen[1], G.gen[3]],
     perms=test_perms_generators_to_group(gens));
  print("1,3 orders ",apply(permorder,perms));
  check_equal(#perms, 12);
}
quit



\\------------------------------------------------------------------------------
\\ A4 making S2xS2

A4 = test_perms_An(4);
S2 = test_perms_Sn(2);
S2xS2 = test_perms_direct_product(S2,S2);
print("A4");
{
  for(i=1,#A4,
     print(" "apply(Vec,test_perm_to_cycles(A4[i]))));
}

print("sub1");
sub1 = [A4[1],A4[4]];
{
  for(i=1,#sub1,
     print(" "apply(Vec,test_perm_to_cycles(sub1[i]))));
}
check_equal(test_perms_is_isomorphic(sub1,S2), 1);

print("making S2xS2");
{
  for(i=1,#A4,
     my(g=A4[i],
        perms=test_perms_generators_to_group(concat(sub1,[g])));
     if(test_perms_is_isomorphic(perms,S2xS2),
        print(i"  "apply(Vec,test_perm_to_cycles(g))"  "Vec(g))));
}

print("sub2");
sub2 = test_perms_generators_to_group([A4[4],A4[9]]);
{
  for(i=1,#sub2,
     print(" "apply(Vec,test_perm_to_cycles(sub2[i]))));
}
check_equal(test_perms_is_isomorphic(sub2,S2xS2), 1);

test_vector_uniq(vec) =
{
  if(#vec>=2,
     my(ret=vector(#vec),
        upto=1);
     ret[1]=vec[1];
     for(i=2,#vec,
        for(j=1,upto,
           if(vec[i]==ret[j], next(2)));  \\ vec[i] is a duplicate, skip it
        ret[upto++] = vec[i]);  \\ pre-increment store
     Vec(ret,upto)
     ,
     vec);
}

print("making whole");
{
  my(seen=Map());
  for(i=1,#A4,
     my(g=A4[i],
        perms=sub2);
     perms=concat([perms,
                   apply(p->p*g, perms),
                   apply(p->p*g^2, perms)]);
     perms=test_vector_uniq(perms);
     print(perms);
     #perms==#Set(perms) || error();
     test_perms_is_group(perms) || error();
     if(test_perms_is_isomorphic(perms,A4),
        print(i"  "apply(Vec,test_perm_to_cycles(g))"  "Vec(g)
             " commute "test_perm_commute(g,sub1[2]));
        print(" elems");
        for(i=1,#A4,
           print("  "Vec(perms[i])"  "apply(Vec,test_perm_to_cycles(perms[i]))));
        my(m=test_perms_Cayley_table(perms));
        mapput(seen,m,1);
        for(r=1,matsize(m)[1],
           print("  "m[r,]));
       ));
  print(#seen);
}
quit


\\------------------------------------------------------------------------------
\\ Group S4 Generators
\\
\\            1  2   3  4   5  6   7  8   9 10  11 12  13 14 15 16 17,18 19,20
\\ Vecsmall([14,13, 19,20, 21,22, 15,16, 17,18, 24,23,  2,1,  7,8,  9,10, 3,4, 5,6, 12,11]),
\\ Vecsmall([11,12, 6,5, 4,3, 10,9, 8,7, 1,2, 23,24, 18,17, 16,15, 22,21, 20,19, 13,14]),
\\ Vecsmall([7, 8, 23, 24, 1, 2, 5, 6, 19, 20, 18, 17, 9, 10, 4, 3, 22, 21, 13, 14, 11, 12, 16, 15]),
\\ Vecsmall([13, 14, 15, 16, 9, 10, 19, 20, 5, 6, 12, 11, 1, 2, 3, 4, 21, 22, 7, 8, 17, 18, 24, 23])]

K = nfsplitting(x^4+x+1);  \\ S4
G=galoisinit(K);
smallgroup=[G.gen,G.orders]

install(group_elts, "GL", "group_elts");
check_equal(group_elts(smallgroup,#G.group), G.group);
check_equal(vecsort(test_perms_generators_to_group(G.gen)), vecsort(G.group));

\\ G.group
print("elems ");
{
  my(id=vpar_INTERNAL_identity_perm(#G.group),
     rev=Vecsmall(Vecrev(id)));
  for(i=1,#G.group,
     my(g=G.group[i]);
     for(j=1,#g, printf(" %2d", g[j]));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}
print("gens");
{
  for(i=1,#G.gen,
     my(g=G.gen[i]);
     for(j=1,#g, printf(" %2d", g[j]));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}
print("num gens ",#G.gen);
print("orders ",apply(permorder,G.gen));

print("elems by exponents");
{
  my(id=vpar_INTERNAL_identity_perm(#G.group),
     rev=Vecsmall(Vecrev(id)));
  for(i=1,#G.group,
     my(g=G.group[i]);
     printf(" %4d  ",fromdigits(test_smallgroup_elem_to_exponents(smallgroup,g)));
     for(j=1,#g, printf(" %04d",fromdigits(test_smallgroup_elem_to_exponents(smallgroup,G.group[g[j]]))));
     printf("   order %d", permorder(g));
     if(g==id,print1("  id"),
        g==rev,print1("  rev"));
     print());
}

want = Vecsmall([13,14,15,16,17,18,19,20,21,22,23,24, 1,2,3,4,5,6,7,8,9,10,11,12]);
test_is_perm(want) || error();
p = G.gen[1]^-1*want
apply(g->g*p, G.gen)

\\ Vecsmall([13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]), swap
\\ Vecsmall([12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1,  24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13]), rev
\\            1   2   3   4   5   6   7   8   9  10  11  12   13  14
\\ Vecsmall([ 8,  7,  5,  6, 19, 20, 23, 24,  1,  2, 17, 18,  10, 9, 13, 14, 11, 12, 4, 3, 22, 21, 15, 16]),
\\ Vecsmall([14, 13, 19, 20, 5, 6, 15, 16, 9, 10, 11, 12, 2, 1, 7, 8, 17, 18, 3, 4, 21, 22, 23, 24])]

print();
want = Vecsmall([13,14,15,16,17,18,19,20,21,22,23,24, 1,2,3,4,5,6,7,8,9,10,11,12]);
test_is_perm(want) || error();
p = G.gen[4]^-1*want
r=apply(g->g*p, G.gen)
print("orders ",apply(permorder,r));

\\ Vecsmall([14, 13, 19, 20, 5, 6, 15, 16, 9, 10, 11, 12, 2, 1, 7, 8, 17, 18, 3, 4, 21, 22, 23, 24]),
\\ Vecsmall([11, 12, 6, 5, 20, 19, 10, 9, 16, 15, 14, 13, 23, 24, 18, 17, 8, 7, 22, 21, 4, 3, 2, 1]),
\\ Vecsmall([ 7,  8, 23, 24, 11, 12,  5,  6, 22, 21, 15, 16,  9, 10, 4, 3, 19, 20, 13, 14, 1, 2, 17, 18]),
\\ Vecsmall([13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])

print();
want = Vecsmall([2,1, 4,3, 6,5, 8,7, 10,9, 12,11, 14,13, 16,15, 18,17, 20,19, 22,21, 24,23]);
test_is_perm(want) || error();
p = G.gen[4]^-1*want
r=apply(g->g*p, G.gen)
print("orders ",apply(permorder,r));

\\ Vecsmall([1, 2, 8, 7, 18, 17, 4, 3, 22, 21, 24, 23, 13, 14, 20, 19, 6, 5, 16, 15, 10, 9, 12, 11]),
\\ Vecsmall([24, 23, 17, 18, 7, 8, 21, 22, 3, 4, 1, 2, 12, 11, 5, 6, 19, 20, 9, 10, 15, 16, 13, 14]),
\\            1 2  3 4   5  6   7  8  9 10 11 12  13 14 15,16  17,18 19,20 21,22  23,24
\\ Vecsmall([10,9, 3,4, 20,19, 14,13, 2,1, 18,17,  8,7, 24,23, 12,11, 6,5, 21,22, 16,15]),
\\ Vecsmall([2, 1, 4, 3, 6, 5, 8, 7, 10, 9, 12, 11, 14, 13, 16, 15, 18, 17, 20, 19, 22, 21, 24, 23])]

whole=test_perms_generators_to_group(G.gen);
{
  for(i=1,#G.gen,
     my(sub=test_perms_generators_to_group(G.gen[1..i]));
     print("size "#sub"   "test_perms_is_normal_subgroup(whole,sub)));
}
print();
{
  for(i=1,#G.gen,
     my(sub=test_perms_generators_to_group(G.gen[i..#G.gen]));
     print("size "#sub"   "test_perms_is_normal_subgroup(whole,sub)));
}
quit


\\------------------------------------------------------------------------------
\\ S3 not in A4

{
  my(A4 = test_perms_An(4),
     S3 = test_perms_Sn(3));
  forsubset([#A4,#S3],set,
            my(subgroup=vecextract(A4,set));
            if(test_perms_is_group(subgroup)
               && test_perms_is_isomorphic(subgroup,S3),
               print(subgroup)));
  quit;
}


\\------------------------------------------------------------------------------
\\ Delete Leaf to Same

{
  for(n=1,20,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(orbitnum=vpar_free_orbitnum_vector(vpar),
              leaves=vpar_leaves(vpar),
              vpar_trims=vector(#leaves,i, vpar_to_free(vpar_delete_vertex(vpar,leaves[i]))));
           \\ for(i=1,#leaves,
           \\    print(" v="leaves[i]" orbit ",orbitnum[leaves[i]],"  ",vpar_trims[i]));
           for(i=1,#leaves,
              my(u=leaves[i]);
              for(j=i+1,#leaves,
                 my(v=leaves[j]);
                 if(orbitnum[u]!=orbitnum[v] && vpar_trims[i]==vpar_trims[j],
                    print(" "u" "v" different orbit same trim"))))));
  quit;
}


\\------------------------------------------------------------------------------
\\ re-rooting to vertices in same orbit ... not

{
  for(n=1,12,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           my(orbitnum=vpar_orbitnum_vector(vpar));
           for(u=1,#vpar,
              for(v=u+1,#vpar,
                 (vpar_is_isomorphic(vpar_reroot(vpar,u),vpar_reroot(vpar,v))
                 == (orbitnum[u]==orbitnum[v])
                 || error())))));
  quit;
}


\\------------------------------------------------------------------------------
\\ re-rooting to vertices in same free orbit is isomorphic

{
  for(n=1,12,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(orbitnum=vpar_free_orbitnum_vector(vpar));
           for(u=1,#vpar,
              for(v=u+1,#vpar,
                 (vpar_is_isomorphic(vpar_reroot(vpar,u),vpar_reroot(vpar,v))
                 == (orbitnum[u]==orbitnum[v])
                 || error())))));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_free_orbit_sets() partitions occurring

\\ Not all partitions can be orbit sets sizes.
\\ n=2 missing [1, 1]
\\ n=3 missing [1, 1, 1]
\\ ...
\\ n=6 missing [3, 3]  [1, 1, 1, 1, 1, 1]

\\ num occurring
\\ forests
\\ 1, 1, 2, 4, 6, 9, 15, 20, 29, 37, 54, 69, 96, 119,
\\ not in OEIS: 2, 4, 6, 9, 15, 20, 29, 37, 54, 69, 96, 119,
\\
\\ n=7 got [1, 1, 1, 1, 1, 1, 1]  count 1
\\ vpar [0, 1, 2, 3, 1, 5, 1]
\\       1
\\    /  |  \
\\   2   5   7    smallest asymmetric
\\   |   |        asymmetrics are [1,1,...,1]
\\   3   6
\\   |
\\   4

\\ num missing
\\ forests
\\ n=1  2  3  4  5  6  7  8  9 10 11 12 13
\\   0, 1, 1, 1, 1, 2, 0, 2, 1, 5, 2, 8, 5, 16, 10, 28, 20, 45,
\\ not in OEIS: 2, 1, 5, 2, 8, 5, 16, 10, 28, 20, 45,


{
  my(forest=1); \\ trees or forests
  for(n=1,20,
    my(seen=Map(),
       vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(part=Vecsmall(vecsort(apply(length,vpar_free_orbit_sets(vpar)))),
              count=0);
           mapisdefined(seen,part,&count);
           mapput(seen,part,count+1);

           \\ if(part == Vecsmall([1, 1, 1, 1, 1, 1, 1]),
           \\    print("found "vpar));
          );
     my(m=Mat(seen),
        num_got=matsize(m)[1],
        num_missing=numbpart(n)-num_got);
     print1(num_got", ");

     if(0,
        print();
        for(i=1,num_got,
           my(part=m[i,1]);
           vecsum(Vec(part))==n || error();
           print(Vec(part)"  count "m[i,2]));
        print("n="n" missing ",num_missing);
        forpart(part=n,
                if(!mapisdefined(seen,part),
                   print(Vec(part))));
        print());
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_orbit_sets() with same partitions

{
  my(forest=1); \\ trees or forests
  for(n=1,5,
     print("n="n);
     my(seen=Map(),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(part=Vecsmall(vecsort(apply(length,vpar_orbit_sets(vpar)))),
              count=0);
           mapisdefined(seen,part,&count);
           mapput(seen,part,count+1));
     my(m=Mat(seen),
        num_got=matsize(m)[1],
        num_missing=numbpart(n)-num_got);
     for(i=1,num_got,
        my(part=m[i,1]);
        vecsum(Vec(part))==n || error();
        print(Vec(part)"  "m[i,2]));
     print();
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_orbit_sets() partitions occurring

\\ Not all partitions can be orbit sets sizes.
\\ n=5 missing [2, 3]
\\ n=7 missing [2, 5]  [3, 4]
\\ n=8 missing [3, 5]

\\ num occurring
\\ forests
\\ not in OEIS: 1, 2, 3, 5, 6, 11, 13, 21, 28, 38, 51, 73, 93,
\\ trees
\\ not in OEIS: 1, 1, 2, 3, 5, 6, 11, 13, 21, 28, 38, 51, 73,
\\ same one later

\\ num missing
\\ forests
\\ n=1           5     7  8
\\   0, 0, 0, 0, 1, 0, 2, 1, 2, 4, 5, 4, 8, 11, 13, 19, 19,
\\ not in OEIS: 1, 2, 4, 5, 4, 8, 11, 13, 19, 19,


{
  my(forest=1); \\ trees or forests
  for(n=1,20,
    my(seen=Map(),
       vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(part=Vecsmall(vecsort(apply(length,vpar_orbit_sets(vpar)))),
              count=0);
           mapisdefined(seen,part,&count);
           mapput(seen,part,count+1));
     my(m=Mat(seen),
        num_got=matsize(m)[1],
        num_missing=numbpart(n)-num_got);
     print1(num_missing", ");

     if(0,
        print();
        for(i=1,num_got,
           my(part=m[i,1]);
           vecsum(Vec(part))==n || error();
           print(Vec(part)));
        print("n="n" missing ",num_missing);
        forpart(part=n,
                if(!mapisdefined(seen,part),
                   print(Vec(part))));
        print());
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_orbits() total -- unlabelled

\\ trees
\\ 1, 2, 5, 13, 35, 95, 262, 727, 2033, 5714, 16136, 45733,
\\ A000107 rooted trees label one vertex
\\ gf A(x) = B(x)/(1-B(x)), where B(x) gf of rooted unlabelled A000081
\\ in R. Harary, R. W. Robinson, "Isomorphic Factorizations VIII: Bisectable
\\ Trees", Combinatorica 4 (2) (1984) 169-179, eq. (4.12).
\\ http://dx.doi.org/10.1007/BF02579217
\\
\\ forests
\\ 1, 3, 9, 26, 75, 214, 612, 1747, 4995, 14294, 40967, 117560,
\\ A000243 trees with 2 labelled vertices
\\ gf A(x) = B(x)^2/(1-B(x)), where B(x) gf rooted unlabelled trees A000081

{
  my(forest=1); \\ trees or forests
  for(n=1,20,
     my(total=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           total += vpar_num_orbits(vpar));
     print1(total,", "));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_orbitnum_vector() occurring

\\ forests
\\ not in OEIS: 1, 2, 5, 15, 42, 203, 821,
\\
\\ trees
\\              1, 1, 4, 11, 41, 102, 715,
\\ not in OEIS:       4, 11, 41, 102, 715,
\\

{
  my(forest=0); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_orbitnum_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ lexmax

{
  \\ of path
  for(n=1,10,
     my(vpar=vpar_make_path(n),
        seq=test_seq_lexmax_by_search(vpar),
        relabelled=vpar_relabel_seq(vpar,seq));
     print("n="n"  "relabelled);
     print("  "Vec(seq));
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_height_num_components() totals

{
  \\ unlabelled rooted forests total num components of whole forest height
  \\ not in OEIS: 1,3,6,13,26,60,134,325,793,2002,5106,13276,
  for(n=1,16,
     my(total=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,1))=='none,  \\ forests
           total += vpar_height_num_components(vpar));
     print1(total,","));
  quit;
}
{
  \\ labelled forests total num components of whole forest height
  \\ not in OEIS: 1,4,18,140,1420,18162,280210,

  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests
           my(vpar=vpar_from_upascend(vec));
           total += vpar_height_num_components(vpar));
     print1(total",");
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_depths_vector() Palindromes

\\ forests, labelled, palindromic depths, up to ordered
\\ 1, 1, 4, 4, 27, 25, 218,
\\ not in OEIS: 1, 4, 4, 27, 25, 218,
\\
\\ trees odd only so root sole depth=0 is middle, then any depths each end
\\   1, 0, 1, 0, 4, 0, 25,
\\ n=1     3     5      7
{
  my(forest=0);
  for(n=1,10,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              depths=vpar_depths_vector(vpar),
              ord=vpar_relabel_preorder(vpar));
           if(depths==Vecrev(depths) && !mapisdefined(seen,ord),
              mapput(seen,ord,depths);
              count++;
              \\ print(vpar"  "depths);
              ));
     print1(count", "));
  quit;
}

\\ forests, labelled, palindromic depths, up to isomorphism
\\ 1, 1, 3, 3, 12, 10, 54,
\\ not in OEIS: 1, 3, 3, 12, 10, 54,
\\
\\ trees odd only so root sole depth=0 is middle
\\   1, 0, 1, 0, 3, 0, 10,
\\ n=1     3     5       7
{
  my(forest=1);
  for(n=1,10,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              depths=vpar_depths_vector(vpar),
              un=vpar_to_unlabelled(vpar));
           if(depths==Vecrev(depths) && !mapisdefined(seen,un),
              mapput(seen,un,depths);
              count++;
              \\ print(vpar"  "depths);
              ));
     print1(count", "));
  quit;
}

\\ forests, labelled
\\ 1, 1, 4, 9, 68, 193, 2240,
\\ not in OEIS: 4, 9, 68, 193, 2240,
\\
\\ trees odd only so root sole depth=0 is middle
\\   1, 0, 1, 0, 9, 0, 193,
\\ n=1     3     5       7
\\ not in OEIS: 1, 9, 193,
{
  my(forest=0);
  for(n=1,10,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              depths=vpar_depths_vector(vpar));
           if(depths==Vecrev(depths),
              count++;
              \\ print(vpar"  "depths);
              ));
     print1(count", "));
  quit;
}



\\------------------------------------------------------------------------------
\\ Various Polynomials

{
  my(forest=0);
  for(n=6,6,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           print(vpar);
           print("  adj   "Vec(vpar_adjacency_poly(vpar)));
           print("  Sadj  "Vec(vpar_skew_adjacency_poly(vpar)));
           print("  match "Vec(vpar_matchpoly(vpar)));
           print("  Lap   "Vec(vpar_Laplacian_poly(vpar)));
           print("  sub   "Vec(vpar_spanning_freqpoly(vpar)));
           print("  dist  "Vec(vpar_path_lengths_poly(vpar)));
           print("  ddist "Vec(vpar_distance_distribution_polynomial(vpar)));
           print("  cord  "vpar_cordial_poly(vpar));
           print("  ind   "Vec(vpar_indpoly(vpar)));
           print("  dom   "Vec(vpar_dompoly(vpar)));
           print("  indom "Vec(vpar_indompoly(vpar)));
           print()));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Free Trees with All Leaf Semi-Stable

\\ trees
\\   1, 1, 1, 0, 1, 1, 2, 4, 9, 17, 44, 86, 212, 481,
\\ n=0  1  2  3
\\ not in OEIS: 2, 4, 9, 17, 44, 86, 212, 481,

{
  my(verbose=0);
  for(n=0,16,
     if(verbose, print("n="n));
     my(count=0,
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(leaves=vpar_leaves(vpar),
              semis=vector(#leaves,i, test_is_free_semistable_at_vertex(vpar,leaves[i])));
           count += (vecsum(semis)==#leaves);
          );
     print1(count,", ");
     if(verbose, print()));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Rooted Unlabelled Trees with All Childless Semi-Stable

\\ trees
\\   1, 1, 2, 3, 7, 13, 33, 72, 174, 416, 1023,
\\ n=0  1  2  3
\\ not in OEIS: 2, 3, 7, 13, 33, 72, 174, 416, 1023,
\\
\\ forests
\\ same 1 earlier

{
  my(forest=0,
     verbose=0);
  for(n=0,16,
     if(verbose, print("n="n));
     my(count=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(childless=vpar_childless_vertices(vpar),
              semis=vector(#childless,i, test_is_rooted_semistable_at_vertex(vpar,childless[i])));
           count += (vecsum(semis)==#childless);
          );
     print1(count,", ");
     if(verbose, print()));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Free Semi-Stable

\\ free trees total num vertices semi-stable
\\              1, 2, 1, 4, 5, 15, 34, 88, 203, 548,
\\ not in OEIS:             5, 15, 34, 88, 203, 548,

\\ free trees number of semi-stable
\\ not in OEIS: 1, 1, 1, 1, 2, 5, 9, 22, 46, 105,
\\ cf all free  1, 1, 1, 2, 3, 6, 11, 23, 47, 106
\\          n = 1  2  3  4  5  6  7   8   9   10
\\ vector(10,n,vpar_count_free(n))

\\ not semi-stable
\\   path-4,5,6,7
\\   asymmetric 7 = star 1,2,3
\\   path-8,9,10
{
  for(n=1,10,
     my(vpar=vpar_free_first(n),
        total=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(vertices=test_free_semistable_vertices(vpar));
           \\ print(vpar"  "ind);
           \\ total += #vertices;
           \\ total++;
           \\ total += (#vertices!=0);
           if(#vertices==0, print(vpar));
           \\ if(#vertices!=0, print(vpar));
          );
     print1(total", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Free Tree Distinct Automorphism Groups

\\ free trees, n=num vertices
\\   n=0  1  2  3  4  5  6  7   8   9  10
\\     1, 1, 1, 1, 2, 2, 5, 8, 11, 15, 22
\\ not in OEIS: 1, 2, 2, 5, 8, 11, 15, 22

{
  for(n=0,13,
     my(l=List([]),
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(perms=vpar_automorphisms(vpar));
           for(i=1,#l, if(test_perms_is_isomorphic(perms,l[i]), next(2)));
           listput(l,perms));
     print1(#l,", "));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Rooted Tree Distinct Automorphism Groups

\\ rooted trees, n = num vertices
\\   n=0  1  2  3  4  5  6  7   8   9  10
\\     1, 1, 1, 2, 3, 4, 6, 9, 12, 17, 20,
\\ not in OEIS: 2, 3, 4, 6, 9, 12, 17, 20,
\\
\\ n=3  trivial, S2
\\ n=4  trivial, S2, S3
\\ n=5  trivial, S2, S3, S4    4 of
\\ n=6  trivial, S2, S3, S4, S5,  S2xS2
\\        1
\\       / \ \
\\      2   5 6
\\     / \
\\    3  4   S2xS2
\\
\\ n=7  trivial, S2, S3, S4, S5, S6,  S2xS2, S2xS3, S2wrS2
\\        1
\\       /  \
\\      2    3
\\     / \  / \
\\    4  5  6  7
{
  my(forest=0,
     verbose=0);
  for(n=0,10,
     if(verbose, print("n="n));
     my(l=List([]),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(perms=vpar_automorphisms(vpar));
           for(i=1,#l, if(test_perms_is_isomorphic(perms,l[i]), next(2)));
           if(verbose, print(" "vpar));
           listput(l,perms));
     print1(#l,", ");
     if(verbose, print()));
  quit;
}


\\------------------------------------------------------------------------------
\\ permsign()

{
  for(n=0,3,
     for(i=0,n!-1,
        my(perm=numtoperm(n,i));
        print(Vec(perm)" = "apply(Vec,test_perm_to_cycles(perm))"    "permsign(perm))));
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_free_next() Try Forests

{
  for(n=0,8,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
            print(vpar"   centres "vpar_centres(vpar))));
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_free_next() Forests Runs

{
  for(n=0,30,
     my(vpar=vpar_free_first(n),
        count=0);
     while(vpar!='none,
           count++;
           my(want=test_free_next_forest_by_search(vpar),
              got=vpar_free_next_forest(vpar,1));
           if(got != want,
              print(vpar"  at i="i" of n="n);
              if(!test_unlabelled_is_free_primary(vpar),
                 print("  oops, NOT PRIMARY",""));
              print(" got  ",got,
                if(!test_unlabelled_is_free_primary(vpar),"  NOT PRIMARY",""));
              print(" want ",want,
                if(!test_unlabelled_is_free_primary(want),"  NOT PRIMARY",""));
              quit;
             );

           for(extra=1,n+3,
              my(vpar=Vec(vpar,n+extra),
                 want=test_free_next_forest_by_search(vpar),
                 got=vpar_free_next_forest(vpar,1));

              if(got != want,
                 print(vpar"  at i="i" of n="n" extra="extra);
                 if(!test_unlabelled_is_free_primary(vpar),
                    print("  oops, NOT PRIMARY",""));
                 print(" got  ",got,
                 if(!test_unlabelled_is_free_primary(vpar),"  NOT PRIMARY",""));
                 print(" want ",want,
                 if(!test_unlabelled_is_free_primary(want),"  NOT PRIMARY",""));
                 quit;
             ));

          vpar=want));
  print("ok");
  quit;
}

\\ [0, 1, 2, 2, 2, 2, 1, 7, 1, 0, 0, 0, 0, 0, 0]  at i=18392 of n=15
\\  got  [0, 1, 2, 2, 2, 2, 1, 7, 0, 9, 10, 10, 9, 13, 14]
\\  want [0, 1, 2, 2, 2, 2, 1, 7, 0, 9, 10, 10, 10, 9, 14]


\\-----------------------------------------------------------------------------
\\ Num Siblings Sets in Ordered Trees

\\ trees total
\\ 0, 1, 2, 5, 15, 49, 168, 594, 2145, 7865, 29172, 109174, 411502,
\\ A189176
\\ sum(k=0..n, binomial(2*n-k,n-k)*(n^2+n*k-k^2-k) / ((2*n-k)*(2*n-k-1)))
\\ vector(10,n, sum(k=1,n, (3-k)*binomial(2*n-k-1,n-1)))
\\
\\ forests total
\\ 0, 1, 3, 10, 35, 126, 462, 1716, 6435, 24310, 92378, 352716,
\\ A001700 binomial(2n+1,n)

\\ forests by k many siblings sets
\\ A001263 Narayana triangle binomial(n,k-1)*binomial(n-1,k-1)/k
\\ Dyck n-paths with k peaks
\\ binomial(n,k)*binomial(n-1,k-1)/(n-k+1)

{
  my(forest=0);
  for(n=1,12,
     my(counts=vector(n),
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           counts[vpar_num_siblings_sets(vpar)]++);
     print(counts);
     );
  quit;
}
{
  my(forest=0);
  for(n=0,12,
     my(total=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           total += vpar_num_siblings_sets(vpar));
     print1(total", ");
     );
  quit;
}
{
  \\ 2^n ordered forests with consecutive siblings
  my(forest=1,
     func=min);
  for(n=0,12,
     my(count=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(min_sibling=vpar_minmax_sibling_vector(vpar,func),
              is_monotonic = test_vector_is_increasing(min_sibling));
           count += is_monotonic;
          );
     print1(count", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Free Trees Stability Index

\\ free trees total stability index
\\ not in OEIS: 1, 2, 3, 4, 10, 25, 58, 145, 359, 926

{
  for(n=1,10,
     my(vpar=vpar_free_first(n),
        total=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(ind=test_free_stability_index_by_search(vpar));
           \\ print(vpar"  "ind);
           total += ind);
     print1(total", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Unlabelled Premax or Lexmin Differences

\\ n=3 graph only triangle and 1 hanging

\\ vpar_to_lexmin
\\ test_to_lexmax_by_search
{
  my(forest=1,
     n=5,
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
        print(test_to_lexmax_by_search(vpar),","));
  print("count "vpar_count_unlabelled(n,forest));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_minmax_sibling_vector() vpars with monotonic sibling vector

\\ trees min or max sibling monotonic
\\ not in OEIS: 1, 2, 8, 44, 302, 2462, 23132,
\\
\\ forests min or max sibling monotonic
\\ A000262
\\ 1, 3, 13, 73, 501, 4051, 37633,
\\ root=1 same one later, forest underneath
\\ same as vpar_minmax_child_vector() distinct occurring
\\ sets of ordered lists of min child, its parent which is min child, etc

\\ siblings in contiguous blocks
\\

{
  my(forest=1,
     func=min);
  for(n=4,4,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              sibling_vector = vpar_minmax_sibling_vector(vpar,func),
              is_monotonic = test_vector_is_increasing(sibling_vector));
           count += is_monotonic;
           if(is_monotonic,
              print(vpar"  "sibling_vector));
     );
     print1(count", ");
  );
  quit;
}

{
  \\ the monotonics occurring
  \\ for tree >= 1 singleton block
  \\ forest monotonics not occurring in trees
  my(n=4,
     func=min);
  my(m=Map());
  forvec(vec=vpar_forvec_from(n,1),
        my(vpar=vpar_from_upascend(vec),
           sibling_vector = vpar_minmax_sibling_vector(vpar,func));
        if(test_vector_is_increasing(sibling_vector),
           mapput(m,sibling_vector, vpar)));

  forvec(vec=vpar_forvec_from(n,0),
        my(vpar=vpar_from_upascend(vec),
           sibling_vector = vpar_minmax_sibling_vector(vpar,func));
        if(test_vector_is_increasing(sibling_vector),
           if(mapisdefined(m,sibling_vector),
              mapdelete(m,sibling_vector))));

  print("n="n" forest monotonics not occurring in trees");
  m=Mat(m);
  for(i=1,matsize(m)[1],
     print(m[i,1]"  from vpar="m[i,2]));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_minmax_sibling_vector() monotonic - cf chains upwards

\\ A000262 "sets of lists", partition 1..n into ordered lists
\\ 1, 3, 13, 73, 501, 4051, 37633, 394353,

\\ chains up in trees distinct occurring
\\ 1, 2, 9, 53, 378, 3151, 30000, 320785,

\\ if 1 is last in a list then it is a root

\\ n=3 forests, 13 of
\\ [[1], [2], [3]]        singletons [0, 0, 0]
\\ [[2], [3, 1]]      from vpar=[0, 0, 1]
\\ [[1], [3, 2]]      from vpar=[0, 0, 2]
\\ [[1], [2, 3]]      from vpar=[0, 3, 0]
\\ [[2, 1], [3]]
\\ [[3, 1, 2]]
\\ [[2, 1, 3]]
\\ [[1, 2], [3]]
\\ [[3, 2, 1]]
\\ [[1, 2, 3]]
\\ [[1, 3], [2]]
\\ [[2, 3, 1]]
\\ [[1, 3, 2]]

\\ n=3 trees, 9 of
\\ [[2, 1], [3]]            exclude  [[1], [2], [3]]
\\ [[1, 2], [3]]                     [[1], [3, 2]]
\\ [[1, 3], [2]]                     [[1], [2, 3]]
\\ [[3, 1, 2]]                       [[2], [3, 1]]
\\ [[2, 1, 3]]
\\ [[3, 2, 1]]
\\ [[1, 2, 3]]
\\ [[2, 3, 1]]
\\ [[1, 3, 2]]

min_siblings_chains_up(vpar) =
{
  my(min_sibling=vpar_minmax_sibling_vector(vpar,min),
     num_children=vpar_num_children_vector(vpar),
     ret=List([]));
  for(i=1,#vpar,
     if(num_children[i]==0,
        my(v=i, l=List([]));
        while(1,
              listput(l,v);
              min_sibling[v]==v || break;
              (v=vpar[v]) || break);
        listput(ret,Vec(l))));
  Vec(ret);
}
min_siblings_chains_up([0,1,2,3])

{
  my(forest=0);
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              chains = min_siblings_chains_up(vpar));
           if(!mapisdefined(m,chains),
              mapput(m,chains,1);
              \\ print(chains"   from vpar="vpar);
             ));
     print(#m", ");
  );
  quit;
}
{
  \\ chains in forests which are not in trees

  my(n=5, m=Map());
  forvec(vec=vpar_forvec_from(n,1),  \\ forests
        my(vpar=vpar_from_upascend(vec),
           chains = min_siblings_chains_up(vpar));
        mapput(m,chains, vpar));

  forvec(vec=vpar_forvec_from(n,0),  \\ trees
        my(vpar=vpar_from_upascend(vec),
           chains = min_siblings_chains_up(vpar));
        if(mapisdefined(m,chains), mapdelete(m,chains)));

  m=Mat(m);
  for(i=1,matsize(m)[1],
     print(m[i,1]"  from vpar="m[i,2]));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_seq_siblings() already in sequence
\\
\\ Must be siblings consecutive numbers.
\\ Chains up min sibling to min sibling are "sets of list" A000262.
\\ The top-most t in each is sibling of t-1

\\ trees
\\ not in OEIS: 1, 2, 8, 44, 302, 2462, 23132,
\\
\\ forests
\\ 1, 3, 13, 73, 501, 4051, 37633, 394353,
\\ A000262 "sets of lists", partition 1..n into ordered lists
\\
\\ root=1 = forests one later
\\ 1, 1, 3, 13, 73, 501, 4051,

{
  my(forest=0);
  for(n=1,8,
     my(id=vpar_INTERNAL_identity_perm(n),
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq=vpar_seq_siblings(vpar));
           count += (seq==id));
     print1(count", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Free Forests Iteration Samples

\\ vector(10,n, vpar_count_free(n))
\\ vector(10,n, vpar_count_free(n,1))

{
  my(n=8);
  for(n=1,n, print("first n="n"  "vpar_free_first(n)));
  my(vpar=vpar_unlabelled_first(n),
     count=0);
  until((vpar=vpar_unlabelled_next(vpar,1))=='none,
        test_unlabelled_is_primary(vpar) || next;
        my(sizes=vpar_component_sizes(vpar),
           depths=vpar_depths_vector(vpar));
        print(vpar" sizes "sizes);
        \\ print(depths" sizes "sizes);
        count++);
  print("count "count);
  check_equal(count, vpar_count_free(n,1));
}
{
  print("free trees");
  my(n=7);
  my(vpar=vpar_free_first(n),
     count=0);
  until((vpar=vpar_free_next(vpar))=='none,
        my(sizes=vpar_component_sizes(vpar_delete_vertex(vpar,1)));
        print(vpar" sizes "sizes);
        count++);
  print("count "count);
}
quit;


\\------------------------------------------------------------------------------
\\ Free Forest

\\ [0,1,2,2,2,1,1,1]        [0,1,2,2,1,5,5,1]
\\      1                        1
\\     / \ \ \                  /  \   \
\\    2   6 7 8        =>      2    5   8
\\   /|\                      /\    /\
\\  3 4 5                    3  4  6  7
vpar=vpar_make_bistar(4,4)
vpar=vpar_to_free(vpar)
vpar_free_next(vpar)

print()

\\      1                     1
\\     / \ \ \               /  \  \
\\    2   7 8 9     =>      2    5   8
\\   /|\\                  /\    /\   \
\\  3 4 5 6               3  4  6  7   9
vpar=vpar_make_bistar(4,5)
vpar=vpar_to_free(vpar)
vpar_free_next(vpar)
quit




\\------------------------------------------------------------------------------
\\ vpar_minmax_sibling_vector() distinct occurring

\\ determined by sets of siblings same as vpar_next_sibling_vectors_count()

\\ forests distinct sibling_vectors
\\ 1, 2, 5, 15, 52, 203, 877,
\\ A000110 Bell numbers
\\ root=1 same
\\
\\ trees distinct sibling_vectors
\\ 1, 1, 4, 11, 41, 162, 715,
\\ A000262 partition 1..n into ordered lists (ordered within lists, no order
\\ among lists)

{
  my(forest=1,
     func=max);
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              sibling_vector = vpar_minmax_sibling_vector(vpar,func));
           mapput(m,sibling_vector,1));
     print1(#m", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_minmax_child_vector() monotonic

\\ forests min or max child monotonic
\\ 1 2 5 15 52 203 877
\\ A000110 Bell numbers
\\ trees same one later
\\ trees root=1 none

\\ childless first, being min/max child = 0
\\ path upwards is monotonic

{
  my(forest=1,
     func=min);
  for(n=1,6,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              child_vector = vpar_minmax_child_vector(vpar,func),
              is_monotonic = test_vector_is_increasing(child_vector));
           count += is_monotonic;
           if(is_monotonic,
              print(vpar"  "child_vector));
     );
     print1(count", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_minmax_child_vector() occurring

\\ forests distinct child_vectors
\\ 1 3 13 73 501 4051 37633
\\ A000262 partition 1..n into ordered lists (ordered within lists, no order
\\ among lists)
\\ A000262(n) = n! * sum(j=0,n-1, binomial(n-1,j)/(j+1)!);  /* Wilf */
\\ vector(10,n, A000262(n))
\\ a(n) = (2*n-1)*a(n-1) - (n-1)*(n-2)*a(n-2).
\\
\\ trees distinct min or max child vectors
\\ not in OEIS: 1, 2, 9, 53, 378, 3151, 30000,
\\
\\ trees root=1 distinct min or max child vectors
\\              1, 1, 3, 14, 86, 646, 5694
\\ not in OEIS:       3, 14, 86, 646, 5694

{
  my(forest=0,
     func=max);
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              child_vector = vpar_minmax_child_vector(vpar,func));
           mapput(m,child_vector,1));
     print1(#m", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_unlabelled_next_pos()

\\ trees
\\ total of pos
\\ not in OEIS: 3, 14, 53, 175, 547, 1662, 4951, 14634, 42899, 125444,
\\ 0-based
\\ not in OEIS: 2, 10, 40, 138, 447, 1398, 4262, 12834, 38189, 113035,
\\ measured from end, 1-based
\\ not in OEIS: 1, 6, 25, 84, 253, 714, 1939, 5166, 13621, 35873,
\\ measured from end, 0-based
\\ not in OEIS: 2, 12, 47, 153, 450, 1250, 3366, 8911, 23464,
\\
\\ Measure from end is total num trailing singletons in pre-order lex-max
\\ unlabelleds.

\\ forests
\\ total of pos
\\ not in OEIS: 2, 8, 28, 83, 251, 720, 2088, 5985, 17215, 49379, 142019,
\\ 0-based
\\ not in OEIS: 1, 5, 20, 64, 204, 606, 1803, 5267, 15374, 44614, 129534,
\\ measured from end, 1-based
\\ not in OEIS: 1, 4, 12, 31, 78, 192, 477, 1195, 3036, 7801, 20286,
\\ measured from end, 0-based
\\ not in OEIS: 0, 1, 4, 12, 31, 78, 192, 477, 1195, 3036, 7801,
\\
\\ Measure from end is total num trailing singletons in pre-order lex-max
\\ unlabelled forests.  Or likewise total trailing depth=1 for trees.

{
  my(forest=0);
  for(n=1,12,
     my(vpar=vpar_unlabelled_first(n),
        total=0);
  until((vpar=vpar_unlabelled_next(vpar,1))=='none,
        my(v=vpar_unlabelled_next_pos(vpar,forest));
        if(v,
           total += v-1));
     print1(total", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_unlabelled_next_pos()

\\ Changes to vpar[v..#vpar].  Any vertices with unchanged parent, or all of
\\ that vpar[] range different.

\\ trees
\\ any same
\\ 0, 0, 0, 1, 1, 3, 5, 12, 25, 61, 138, 348, 855, 2187, 5604,
\\ not in OEIS: 3, 5, 12, 25, 61, 138, 348, 855, 2187, 5604,
\\ plus 1
\\ 1, 1, 1, 2, 2, 4, 6, 13, 26, 62, 139, 349, 856, 2188, 5605,
\\ not in OEIS: 4, 6, 13, 26, 62, 139, 349, 856, 2188, 5605,
\\
\\ all diff
\\ 0, 0, 1, 2, 7, 16, 42, 102, 260, 657, 1703, 4417, 11630, 30785, 82206,
\\ not in OEIS: 2, 7, 16, 42, 102, 260, 657, 1703, 4417, 11630, 30785, 82206,
\\ plus 1
\\ 1, 1, 2, 3, 8, 17, 43, 103, 261, 658, 1704, 4418, 11631, 30786, 82207,
\\ not in OEIS: 3, 8, 17, 43, 103, 261, 658, 1704, 4418, 11631, 30786, 82207,

\\ forests
\\ any same
\\ 0, 0, 1, 1, 3, 5, 12, 25, 61, 138, 348, 855,
\\ all diff
\\ 0, 1, 2, 7, 16, 42, 102, 260, 657, 1703, 4417, 11630,
\\ one earlier than trees

{
  my(forest=1);
  for(n=1,12,
     my(vpar=vpar_unlabelled_first(n),
        count_any_same=0,
        count_all_diff=0);
     while(1,
           my(next_vpar=vpar_unlabelled_next(vpar,forest),
              v=vpar_unlabelled_next_pos(vpar,forest));
           if(next_vpar=='none, break);
           my(sames=vector(#vpar-v+1,i, vpar[i-1+v]==next_vpar[i-1+v]),
              any_sames=(vecsum(sames)!=0));
           if(any_sames, count_any_same++, count_all_diff++);
           \\ print(sames);
           vpar=next_vpar);
     \\ print1(count_any_same", ");
     print1(count_all_diff", ");
     count_all_diff+count_any_same == vpar_count_unlabelled(n,forest)-1 || error();
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_unlabelled_prev() vs vpar_unlabelled_next() speed
\\ prev takes about 2.5* longer

{
  my(forest=1);
  for(n=0,16,
     my(t1=getabstime());
     my(vpar=vpar_unlabelled_first(n),
        ncount=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none, ncount++);
     my(nn=getabstime()-t1);

     my(t1=getabstime());
     my(vpar=vpar_unlabelled_last(n,forest),
        pcount=0);
     until((vpar=vpar_unlabelled_prev(vpar,forest))=='none, pcount++);
     my(pp=getabstime()-t1);

     ncount==pcount || error();
     print(nn" vs "pp"   "pp/if(nn,nn,1)*1.0"   ("ncount" trees)");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ how much same vpar of unlabelled with last extended

\\ vpar1 >= vpar2,
\\ ext = concat(vpar2,[1]) new extra
\\ assuming all vpar1 and vpar2 equally likely,
\\ mean location of first difference just over half way

vector_first_diff_pos(vec1,vec2) =
{
  my(m=min(#vec1,#vec2));
  for(i=1,m,
     if(vec1[i]!=vec2[i], return(i)));
  m+1;
}
{
  my(l=[]);
  for(n=0,14,
     l=concat(l,vpar_unlabelled_all(n));

     my(total=0,
        count=0);
     for(i=1,#l,
        my(ext=concat(l[i],[1]));
        for(j=1,i,
           total += vector_first_diff_pos(l[j],ext);
           count++));

     print("n="n" mean ",total*1.0/count);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ how many preorders last vertex no prev sibling

\\ with last sibling
\\ trees
\\ 0, 0, 1, 3, 9, 28, 90, 297, 1001, 3432, 11934, 41990, 149226,
\\ forests, one earlier
\\ 0, 1, 3, 9, 28, 90, 297, 1001, 3432, 11934, 41990, 149226,
\\ A000245  3*(2*n)!/((n+2)!*(n-1)!)

\\ without last sibling
\\ trees
\\ 1, 1, 1, 2, 5, 14, 42, 132, 429, 1430, 4862, 16796, 58786, 208012,
\\ forests, one earlier
\\ 1, 1, 2, 5, 14, 42, 132, 429, 1430, 4862, 16796, 58786, 208012,
\\ A000108 Catalan
\\ being n-1 tree or forest with last forced to be child of n-1
\\
\\ limit 1/4 without prev sibling
\\
\\ (4^(n-1) / (sqrt(Pi * (n-1)) * n)) / (4^n / (sqrt(Pi * n) * (n + 1)))
\\ 1/4 * sqrt(Pi * n)/(sqrt(Pi * (n-1))) *  * (n + 1)/n
\\ 1/4 * sqrt(n)/(sqrt((n-1))) * (n + 1)/n
\\ my(n=100000000); 1/4.0 * sqrt(n)/(sqrt((n-1))) * (n + 1)/n

{
  my(forest=1);
  for(n=100000,100000,
     print(vpar_count_ordered(n-1,forest) * 1.0 / vpar_count_ordered(n,forest)));
  quit;
}

{
  my(forest=1);
  for(n=1,14,
     my(count=0);
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           count += (vpar_INTERNAL_preorder_prev_sibling_of_vertex(vpar,#vpar) == 0));
     print1(count", "));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_to_dot()

\\ print1(vpar_dot([2,0,2,3,4, 0,0,0,6,6],1));
\\ print1(vpar_to_dot([2,0,2,3,4, 0,0,0,6,6]));

vpar_to_dot([0,1,1])
vpar_to_dot([0,1,1],0)
vpar=vpar_make_complete_tree(2,3)
vpar=vpar_relabel_random(vpar);
vpar=concat(vpar,[0,0,0]);
vpar[1]=1; \\ self-loop
vpar=[3,5,2,3,4,5,3,3,2,7];  \\ cyclic
vpar=[3,5,2,0,4,5,3,3,2,7];
vpar=[0,1,1,1,1];
vpar=vpar_make_complete_tree(2,3);
print(vpar);
print(vpar_to_dot(vpar,0));
vpar_dot(vpar,0+32)
print(vpar_to_dot(vpar,1));
vpar_dot(vpar,1)
print(vpar_to_dot(vpar,2));
vpar_dot(vpar,2)
quit


\\-----------------------------------------------------------------------------
\\ vpar_seq_downqueue() Extracts and Childless Relabelling

\\ Childless vertice shuffle around the rest variously.
\\
\\ extract [0, 0, 1, 2]
\\ [0, 1, 0, 2], [0, 0, 1, 2], [0, 0, 2, 1], [0, 1, 2, 0]
\\   1  3           1  2          1  2          1  4
\\   2              3  4          4  3          2  3
\\   4

{
  my(vpar=[0, 1, 0, 2],
     seq=vpar_seq_downqueue(vpar));
  check_equal(seq, Vecsmall([1, 3, 2, 4]));
  check_equal(vecextract(vpar,seq),  [0, 0, 1, 2]);
}

is_relabelling_childless(vpar,perm) =
{
  my(num_children=vpar_num_children_vector(vpar));
  for(v=1,#vpar,
     if(num_children[v] && perm[v]!=v, return(0)));
  1;
}

are_relabelling_childless(vec) =
{
  my(vpar=vec[1]);
  for(i=2,#vec,
     my(perm=vpar_isomorphic_perm(vpar,vec[i]));
     if(perm=='none || !is_relabelling_childless(vpar,perm),
        return(0)));
  1;
}
{
  check_equal(vpar_is_isomorphic([0, 0, 1], [0, 1, 0]), 1);
  my(perm=vpar_isomorphic_perm([0, 0, 1], [0, 1, 0]));
  check_equal(perm, Vecsmall([1,3,2]));
  check_equal(is_relabelling_childless([0, 0, 1],perm), 1);
  check_equal(is_relabelling_childless([0, 1, 0],perm), 1);
}
{
  my(forest=1);
  for(n=3,4,
     print("n="n);
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq =vpar_seq_downqueue(vpar),
              vec=Vecsmall(vecextract(vpar,seq)));
           my(l);
           if(!mapisdefined(m,vec,&l), l=List([]));
           listput(l,vpar);
           mapput(m,vec,l));
     m=Mat(m);
     for(i=1,matsize(m)[1],
        my(vpars=m[i,2]);
        print(Vec(m[i,1])" "vpars"  ",are_relabelling_childless(vpars)));
     );
  quit;
}



\\-----------------------------------------------------------------------------
\\ Otter Count Unlabelled

rho = .3383219
b = 2.68112
b^2/3
r = 1/rho  \\ 2.95
r = 2.95576528565199497471481752412319458837549230466359659535

v = vpar_count_unlabelled_vector(1000);
r = vecsum(v[1..#v]) / vecsum(v[1..#v-1]) * 1.0
r = 1 + v[#v]/vecsum(v[1..#v-1]) * 1.0

Otter(n) = b*sqrt(rho)/(2*sqrt(Pi)) * r^n / (n*sqrt(n));
{
  for(n=1,100,
     my(t = vpar_count_unlabelled(n),
        o = Otter(n));
     print(t"  "o"  "t/o));
}

rho^-20
20^-(3/2)
quit;

\\-----------------------------------------------------------------------------
\\ Cyclic Distance Matrix
\\ (WRONG)

a = ['a,'b,'c,'d]
n = 8
m = matrix(n,n,i,j, (i-j)%n)
p = charpoly(m)
polroots(p)
quit


\\-----------------------------------------------------------------------------
\\ vpar_count_unrooted()

\\ vector(10,n, n!/(1<<(n>>1)))
\\ A090932 a(n) = n!  / 2^floor(n/2)

{
  for(n=1,8,
     print("n="n);
     print("  ",n!/(1<<(n>>1)));
     for(j=0,n>>1,
        print("  ",
              n! * (-1)^j
              / ((j! * (n-2*j)!) << j))));
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_num_siblings_vector() total
\\
\\ trees
\\ not in OEIS: 6, 96, 1500, 25920, 504210,
\\ trees / n, mean
\\ 2, 24, 300, 4320, 72030,
\\ A065513

\\ forests
\\ forests / n, mean
\\ 8, 75, 864, 12005, 196608,
\\ A071720 num spanning trees in K(n)-e, complete graph less one edge

\\ root=1
\\ 2, 24, 300, 4320, 72030
\\ A065513
\\ root=1 / n, mean
\\ 2/3, 6, 60, 720, 10290,
\\ not in OEIS: 6, 60, 720, 10290, 172032

{
  my(forest=-1);
  for(n=3,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           \\ print(vpar"  "vecsum(vpar_num_siblings_vector(vpar)/n));
           total += vecsum(vpar_num_siblings_vector(vpar));
     );
     print1(total", ");
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_depths_vector() total cf vpar_subtree_sizes_vector() total
\\
\\ A001864 trees depths total

{
  my(forest=1);
  for(n=1,4,
     my(total_subtree_sizes=0,
        total_depths=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              subtree_sizes=vpar_subtree_sizes_vector(vpar),
              depths=vpar_depths_vector(vpar));
           total_subtree_sizes += vecsum(subtree_sizes);
           total_depths += vecsum(depths);
           \\ print(vpar" "vecsum(subtree_sizes)" "vecsum(depths));
     );
     print(total_subtree_sizes" "total_depths);
  );
  quit;
}
{
  my(forest=0);
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              depths=vpar_depths_vector(vpar));
           total += vecsum(depths);
     );
     print(total", ");
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Labelled Rooted Vpar Differences

\\ num edges
\\ not in OEIS: 2, 33, 528, 9300, 183810,
\\              2, 33, 528, 9300, 183810, 4068456,
\\
\\ vector(6,n, 1/2*( ((n+1)-1)^2 * (n+1)^((n+1)-2) -  ((n+1)-1)! * sum(k=0,(n+1)-2, (n+1)^k/k!)) )
\\ vector(6,n, 1/2*( n^2 * (n+1)^(n-1) -  sum(k=0,n-1, n!/k! * (n+1)^k)) )

\\ vector(6,n, n^2*vpar_count(n,1))
\\ 1, 12, 144, 2000, 32400, 605052
\\ A076728

\\ total subtree sizes
\\ 4, 39, 472, 6900, 118716,
\\ = 2*A000435 normalized total height, so sum(heights)/n
\\             connected endofunctions with no fixed points

A000435(n) = (n-1)! * sum(k=0,n-2, n^k/k!);  \\ for n>=1
check_equal(vector(#A000435_samples,n, A000435(n)), A000435_samples, \
            "A000435 samples");

A076728(n) = (n-1)^2*vpar_count(n-1,1);
A076728(n) = (n-1)^2 * n^(n-2);
check_equal(vector(#A076728_samples,n, A076728(n)), A076728_samples, \
            "A076728 samples");

vector(8,n, A000435(n+1))
vector(8,n, A076728(n+1))
vector(8,n, (A076728(n+1) - A000435(n+1))/2)

{
  \\ num edges by subtree sizes

  my(forest=1);
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              subtree_sizes=vpar_subtree_sizes_vector(vpar));
           \\ count += vecsum(subtree_sizes);
            \\ count += n^2;
           count += (n^2 - vecsum(subtree_sizes))/2;
     );
     print1(count,", ");
  );
  quit;
}

vector_num_diffs(u,v) = sum(i=1,#u, u[i]!=v[i]);
{
  \\ num edges by search

  my(forest=1);
  for(n=2,6,
     my(vpars=List([]));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           listput(vpars,vpar));
     vpars=Vec(vpars);
     my(num_edges=0);
     for(i=1,#vpars,
        for(j=i+1,#vpars,
           if(vector_num_diffs(vpars[i],vpars[j])==1,
              num_edges++)));
     print1(num_edges,", ");
  );
  quit;
}

\\ gmaker pictures.tex, via gmaker devel/misc.pl
{
  my(forest=1);
  for(n=4,4,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           print(vpar",")));
  quit;
}



\\-----------------------------------------------------------------------------
\\ Ordered Forests Preorder Depths Vector Differences

\\ N=4 trees, N=3 forests
\\      *
\\     / \
\\    *---*
\\    |   |
\\    *---*
\\
\\ N=5 trees, N=4 forests
\\ gmaker pictures.tex, via gmaker devel/misc.pl
\\
\\ for(n=3,10,print("n="n" "vpar_INTERNAL_Catalan_number(n)));
\\ n=5 42
\\ n=6 132

{
  my(forest=1);
  for(n=4,4,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(modified_vpar=vpar_relabel_postorder(vpar),
              depths=vpar_depths_vector(modified_vpar));
           print(depths",")));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Automorphism Groups Max Element Order

\\ S2 wr S2 with n terms
\\ n         = 2  3    4    5
\\ max order = 2, 4,   8,   16
\\ size      = 2, 2^3, 2^7, 2^15
\\ size 2^k
\\ elem order 2^floor(log2(k))

{
  my(S2 = test_perms_Sn(2),
     perms=S2);
  for(i=1,10,
     my(m=vecmax(apply(test_permorder,perms)));
     print("i="i" size="#perms" max order "m);
     perms=test_perms_wreath_product(perms,S2));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Well-Covered by vpar_indompoly()
\\
\\ free trees
\\ not in OEIS: 1, 1, 0, 1, 0, 1, 0, 2, 0, 3, 0, 6, 0, 11, 0, 23, 0,
\\ evens = all free trees
\\ 1, 1, 1, 2, 3, 6, 11, 23, 47, 106,

is_well_covered(vpar) =
{
  my(poly=vpar_indompoly(vpar));
  poldegree(poly) == valuation(poly,'x);
}

{
  forstep(n=2,20,2,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += is_well_covered(vpar));
     print1(count,", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ A213920 rooted unlabelled sibling subtrees at most 2 of same size
\\ 0, 1, 1, 2, 3, 7, 15, 34, 79, 190, 457, 1132, 2823, 7126, 18136, 46541,

\\ more 2-groups A248869
\\ 2-groups can have multiple of same size, as long as not isomorphic
\\ those of same size restricted to
my(len=min(#A213920_samples, #A248869_samples)); \
A248869_samples[1..len] - A213920_samples[1..len]

test_is_max_two_same_size_subtrees(vpar) =
{
  my(subtree_sizes=vpar_subtree_sizes_vector(vpar),
     seq=vecsort(subtree_sizes,,1),
     prev_size=vectorsmall(#vpar+1),
     count =vectorsmall(#vpar+1));
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v]);
     if(prev_size[p+1] == subtree_sizes[v],
        count[p+1]++;
        count[p+1]<=2 || return(0);
        ,
        prev_size[p+1] = subtree_sizes[v];
        count[p+1]=1);
     );
  1;
}

{
  my(forest=0);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count += test_is_max_two_same_size_subtrees(vpar));
     print1(count,", ");

     if(n+1<=#A213920_samples,
        count==A213920_samples[n+1] || error("n="n));
     );
  quit;
}

\\-----------------------------------------------------------------------------
\\ vpar_is_automorphisms_solvable() count

\\ trees rooted unlabelled
\\ 1, 1, 2, 4, 9, 19, 46, 110, 273, 684, 1746, 4503, 11758, 30943,
\\ A318851 rooted unlabelled with <= 4 isomorphic sibling subtrees
\\
\\ forests rooted unlabelled, one earlier
\\ root=1 trees same
\\ not in OEIS: 1, 2, 4, 9, 19, 46, 110, 273, 684, 1746, 4503, 11758, 30943,

\\ free trees
\\ not in OEIS: 1, 1, 1, 2, 3, 5, 10, 21, 43, 97, 214, 502, 1185, 2872, 7024,

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_automorphisms_solvable(vpar_INTERNAL_for_free_automorphisms(vpar)));
     print1(count,", ");
     );
  quit;
}
{
  my(forest=-1);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           count += vpar_is_automorphisms_solvable(vpar));
     print1(count,", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_seq_downqueue() Labelled Already

\\ trees
\\ 1,1,2,5,14,42,132,
\\ A000108 Catalan numbers

\\ forests, is one earlier
\\ 1,2,5,14,42,132,

\\ ordered tree labelled row-wise
\\ root 1, then children 2..a
\\ then row a+1..b with choice of how many to each above

num_children_of_1(vpar) = sum(i=1,#vpar, vpar[i]==1);
{
  my(forest=1);
  for(n=1,10,
     my(id=vpar_INTERNAL_identity_perm(n),
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           if(vpar_seq_downqueue(vpar) == id,
              \\ && num_children_of_1(vpar)==1
              \\ if(n<=6, print(vpar"  "num_children_of_1(vpar)));
              count++));
     print1(count",");
  );
}
quit


\\-----------------------------------------------------------------------------
\\ Trees Free Stable Automorphisms
\\
\\ free trees stable
\\ 2,4,8,17,37,85,196,469,1134,2799,6975,17628,44903,115497,
\\ A003426 = num free trees - A002988
\\ A002988 = trimmed trees, forbid limb length >=2
\\         = non-stable free trees

{
  for(n=0,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_free_stable(vpar));
     print1(count","));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Automorphism Commutators

perms_commutator(vec1,vec2=vec1) =
{
  my(m=Map(),
     inv2=apply(p->p^-1,vec2));
  for(i=1,#vec1,
     my(inv1=vec1[i]^-1);
     for(j=1,#vec2,
        mapput(m, vec1[i]*vec2[j]*inv1*inv2[j], 1)));
  Vec(Mat(m)[,1]);
}

\\                S2 nilpotent,2-group
\\                S3 not          S3 supersolvable
\\ S4 solvable    S4 not          S4 not supersolvable   S4 weakly super
\\ S5 not         S5 not
\\ S6 not         S6 not
\\
\\ S>=5 are simple, so not solvable

\\ G0 sub G1 sub G2 sub Gn    G0=1 and Gn=G
\\ solvable either each Gi normal in G
\\              or each quotient Gi+1 / Gi cyclic
\\ super-solvable both
\\
\\ weakly super-solvable, each Gi normal, and cyclic up to Gn-1 where the
\\ last G/Gn-1 = A4 or S4

vpar=[0,1,1]
autos=vpar_automorphisms(vpar);
print("autos "#autos);

sub=autos;
{
  while(#sub>1,
        my(new_sub=perms_commutator(autos,sub));
        if(new_sub==sub,
           print("unchanged at "#sub);
           break);
        sub=new_sub;
        print("sub "#sub
             \\ "  "sub
             ));
}
quit



\\------------------------------------------------------------------------------
\\ vpar_num_automorphism_generators() max of rooted

\\ trees
\\ n=1  2  3  4        7  8       11 12       15 16
\\   0, 0, 1, 2, 2, 2, 3, 4, 4, 4, 5, 6, 6, 6, 7, 8,
\\ A004524 three even, one odd
\\ reps, rooted unlabelled
\\   1, 1, 1, 1, 2, 6, 2, 1, 4, 22, 4, 1,
\\ not in OEIS: 2, 6, 2, 1, 4, 22, 4, 1,
\\
\\ forests, is one earlier
\\ n=1  2  3  4        7  8       11 12       15 16
\\   0, 1, 2, 2, 2, 3, 4, 4, 4, 5, 6,
\\   1, 1, 1, 2, 6, 2, 1, 4, 22, 4, 1,
\\
\\   *
\\   |\\
\\   *  * *     n=3 g=1     n=4 g=2
\\   |\\
\\   *  * *
\\
\\ vector(16,n, n)
\\ T(n) = (n\2)-(n%4==2);
\\ vector(16,n, T(n))
\\ vector(16,n, T(n) <= n/2)
\\ vector(16,n, T(n) <= n\2)
\\ vector(16,n, T(n) == n\2)
\\
\\ F(n) = T(n+1);
\\ vector(16,n, F(n))
\\ vector(16,n, F(n) <= (n+1)/2)
\\ vector(16,n, F(n) <= (n+1)\2)
\\ vector(16,n, F(n) == (n+1)\2)
\\ vector(16,n, F(n) == n\2)

{
  my(forest=1);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        m=0,
        reps=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(got=vpar_num_automorphism_generators(vpar));
           if(m==got, reps++,
              m<got, m=got;reps=1));
     print1(m,", ");
     \\ print1(reps,", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_automorphisms()

vpar_INTERNAL_automorphisms([0,1,1,1],111)
quit


\\-----------------------------------------------------------------------------
\\ Permutations

vpar_automorphism_generators([0,1,1,1,1,1,1])
a = Vecsmall([1,2,3,4,5,7,6])
b = Vecsmall([2,3,4,5,6,7,1])
{
  for(c=2,7,
     my(w = vectorsmall(7,i, if(i<=c, (i%c)+1, i)));
     w = vectorsmall(7);
     for(i=1,7-c, w[i]=i);
     for(i=7-c+1,6, w[i]=i+1);
     w[7] = 7-c+1;
     print("c="c" w="w);
     check_equal(test_is_perm(w),1);
     for(i=0,test_permorder(a)-1,
        for(j=0,test_permorder(b)-1,
           for(k=0,test_permorder(a)-1,
              for(l=0,test_permorder(b)-1,
                 for(m=0,test_permorder(a)-1,
                    for(n=0,test_permorder(b)-1,
                       if(a^i*b^j*a^k*b^l*a^m*b^n == w,
                          print(i" "j" "k" "l" "m" "n);
                          next(7));
     )))))));
}

make_perm_cycle_up(n,w) = vectorsmall(w,i, if(i<=n,(i%n)+1,i));
for(n=2,7, print(make_perm_cycle_up(n,7)))

gens=vector(6,n, make_perm_cycle_up(n+1,7))
check_equal(#test_perms_generators_to_group(gens), 7!);
\\ check_equal(test_perm_generators_is_minimal(gens), 1);
{
  for(i=1,#gens,
     print("i="i" order "test_permorder(gens[i])
           "  "#test_perms_generators_to_group(gens[^i])));
}

quit

\\-----------------------------------------------------------------------------
\\ vpar_num_automorphism_generators() total

\\ unlabelled trees total vpar_num_automorphism_generators()
\\ not in OEIS: 3, 8, 20, 56, 149, 411, 1133, 3160, 8839, 24904, 70379, 199735,
\\
\\ unlabelled trees total vpar_num_automorphisms()
\\ not in OEIS: 3, 10, 41, 188, 1046, 6731, 50718, 436263, 4228173, 45467134,

\\ labelled trees total vpar_num_automorphism_generators()
\\ not in OEIS: 3, 20, 290, 4032, 69818,
\\
\\ labelled trees total vpar_num_automorphisms()
\\ not in OEIS: 2, 12, 96, 1080, 14400, 241920,
\\ successive ratios 1, 2, 6, 8, 45/4, 40/3, 84/5, 115/6,

\\ free trees total vpar_num_automorphism_generators()
\\ not in OEIS: 4, 9, 18, 40, 87, 210, 495, 1229, 3077, 7878,
\\
\\ free trees total vpar_num_free_automorphisms()
\\ not in OEIS: 2, 8, 28, 140, 787, 5387, 41967, 373902, 3702831, 40545383, 484679647, 6286660867, 87853871378,

{
  for(n=0,16,
     my(vpar=vpar_free_first(n),
        total=0);
     until((vpar=vpar_free_next(vpar)) == 'none,
           total += vpar_num_free_automorphism_generators(vpar));
     print1(total", ");
     );
  quit;
}
{
  my(forest=0,
     prev=0);
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           total += vpar_num_automorphism_generators(vpar));
     print1(total", ");
     \\ print1(total/if(prev,prev,1)", ");
     prev=total;
     );
  quit;
}
{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        total=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           total += vpar_num_automorphisms(vpar));
     print1(total", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_num_automorphism_ids() of maximum #vpar
\\ Solely path-n, as >=2 childless (located anywhere) are same id.

{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           count += (vpar_num_automorphism_ids(vpar)==#vpar));
     print(count",");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_seq_downqueue() extracts

{
  my(forest=-1);
  for(n=1,4,
     print("n="n);
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq =vpar_seq_downqueue(vpar),
              vec=Vecsmall(vecextract(vpar,seq)),
              num=0);
           print(vpar"  "seq" vec "Vec(vec));
           test_is_perm(seq) || error();
           mapisdefined(m,vec,&num);
           mapput(m,vec,num+1));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq =vpar_seq_downqueue(vpar),
              vec=Vecsmall(vecextract(vpar,seq)));
           if(mapget(m,vec)==1,
              print(vpar"  unique vec="Vec(vec))));
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Dreadnaut

{
  for(n=1,50,
     vpar = vpar_make_random(150);
     system("rm /tmp/x.dre");
     \\ write("/tmp/x.dre", "d");
     write("/tmp/x.dre", test_to_dreadnaut(vpar));
     write("/tmp/x.dre", test_dreadnaut_roots(vpar));
     write("/tmp/x.dre", "x");
     system("cat /tmp/x.dre");
     system("dreadnaut </tmp/x.dre");
  );
  quit
}


\\-----------------------------------------------------------------------------
\\ vpar_downwards_ith() preorder and postorder

{
  my(forest=1);
  for(n=4,4,
     for(i=1,vpar_count_ordered(n,forest),
        my(vpar=vpar_preorder_ith(n,i,forest));
        print(vpar"  down="vpar_is_downwards(vpar));
     );
  );
  quit;
}
{
  my(forest=0);
  for(n=4,4,
     for(i=1,vpar_count_updown(n,forest),
        my(vpar=vpar_downwards_ith(n,i,forest));
        print(vpar"  pre="vpar_is_preorder(vpar)"  post="vpar_is_postorder(vpar));
     );
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Groups

install(group_order, "lG", "group_order");

G = galoisinit(x^4+1);
G = galoisinit(x^6+108);
\\ {
\\   for(i=1,1000000,
\\      if(G = galoisinit(x^16 + i),
\\         gens = G.gen;
\\         orders = G.orders;
\\         group=[gens,orders];
\\         if(group_order(group) == 4!,
\\            break)));
\\ }
\\ galoisexport(G)
\\ galoisexport(G,1)
print("elts   ",g = G.group);    \\ all elts
print("gens   ",gens = G.gen);
print("orders ",orders = G.orders);
group=[gens,orders];
print("group_order ",group_order(group));
#test_perms_generators_to_group(gens)
for(i=1,#g, print(g[i]));
print("gens");
for(i=1,#gens, print(gens[i]));
quit
print();
G = galoisinit(x^4+1);
g = G.group
group_order(g)
test_perms_generators_to_group(g)
for(i=1,#g, print(g[i]));


n = 3
elems = vector(n!,i,i--; Vecsmall(numtoperm(n,i)))
m=Map();
for(i=1,#elems, mapput(m,elems[i],i));
elem_to_i(elem) = mapget(m,elem);
perm_times_vec(p,vec) = apply(pv->p*pv, vec);
l=List([]);
{
  for(i=1,#elems,
     my(p=apply(elem_to_i, perm_times_vec(elems[i], elems)));
     p=Vecsmall(p);
     print(p"  order ",test_permorder(p));
     check_equal(test_is_perm(p), 1);
     listput(l,p);
  );
}
swap=mapget(m,Vecsmall([2,1,3]))
rot=mapget(m,Vecsmall([2,3,1]))
l=Vec(l);
l=[l[swap],l[rot]]
group=[l, Vecsmall(apply(test_permorder,l))]
test_smallgroup_validate(group);
group_order(group)
install(group_elts, "GL", "group_elts");
group_elts(group,6)
install(group_isabelian, "lG", "group_isabelian");
group_isabelian(group)
install(group_subgroups, "G", "group_subgroups");
subgroups=group_subgroups(group);
for(i=1,#subgroups, print(subgroups[i]));

test_make_perm_swap(n, a,b) =
{
  my(ret=vpar_INTERNAL_identity_perm(n));
  ret[a]=b;
  ret[b]=a;
  ret;
}
check_equal(test_is_perm(test_make_perm_swap(2, 1,2)), 1);
check_equal(test_is_perm(test_make_perm_swap(4, 1,2)), 1);
check_equal(test_make_perm_swap(4, 1,2), Vecsmall([2,1, 3,4]));

test_make_perm_rotate(n, offset=1) = \
  vectorsmall(n,i, ((i+offset-1)%n)+1);
check_equal(test_is_perm(test_make_perm_rotate(0,0)), 1);
check_equal(test_is_perm(test_make_perm_rotate(1,1)), 1);
check_equal(test_is_perm(test_make_perm_rotate(3,1)), 1);
check_equal(test_make_perm_rotate(4,1), Vecsmall([2,3,4,1]));

install(cyclicgroup, "GL", "cyclicgroup");
test_make_group_C(n) =
{
  n>=2 || error();
  my(perm=vectorsmall(n,i, (i%n)+1));
  \\ cyclicgroup(perm,n);
  [ [perm], Vecsmall([n]) ];
}
test_permtonum_1(perm) = my(n=permtonum(Vec(perm))); if(n,n,(#perm)!);
test_make_group_S(n) =
{
  if(n==2, test_make_group_C(n),
     my(order=n!,
        swap_perm=test_make_perm_swap(n, 1,2),
        rotate_perm=test_make_perm_rotate(n));
     [ [vectorsmall(order,i, test_permtonum_1(swap_perm   * Vecsmall(numtoperm(n,i)))),
        vectorsmall(order,i, test_permtonum_1(rotate_perm * Vecsmall(numtoperm(n,i))))],
      Vecsmall([2,n]) ];
     );
}
print("S2");
S2 = test_make_group_S(2)
test_smallgroup_validate(S2);
group_order(S2)
group_elts(S2,2)
\\ print("checkgroup");
\\ install(checkgroup, "GD&", "checkgroup");
\\ elts=0
\\ checkgroup(S2,&elts)
\\ elts
test_perms_generators_to_group(S2[1])

print();
print("S3");
S3 = test_make_group_S(3)
test_smallgroup_validate(S3);
check_equal(group_order(S3), 3!);
group_elts(S3,6)
subgroups=group_subgroups(S3);
for(i=1,#subgroups, print(subgroups[i]));
all = test_perms_generators_to_group(S3[1])
check_equal(test_perms_is_group(all), 1);

print();
print("S4");
S4 = test_make_group_S(4)
test_smallgroup_validate(S4);
apply(test_permorder,S4[1])
check_equal(#test_perms_generators_to_group(S4[1]), 24, \
            "S4 test_perms_generators_to_group()");
check_equal(#group_elts(S4,24), 24, "S4 group_elts() count");
check_equal(group_order(S4), 24, "S4 group_order()");

quit;


\\-----------------------------------------------------------------------------
\\ Groups

print("vpar");
vpar = [0,1,2, 1,4, 1,6, 1,8]
vpar = [0,1,1]
vpar_validate(vpar);
print("gens");
num_autos = vpar_num_automorphisms(vpar)
vpar_num_automorphism_generators(vpar)
gens = vpar_automorphism_generators(vpar)
print("autos");
autos = test_perms_generators_to_group(gens);
autos=concat(vpar_INTERNAL_identity_perm(#autos[1]),autos)
#autos

perm_times_vec(p,vec) = apply(pv->p*pv, vec);
permed_autos = vector(num_autos,i, \
                      perm_times_vec(Vecsmall(numtoperm(#vpar,i-1)),autos));
permed_autos = Set(permed_autos);
print("permed");
for(i=1,#permed_autos, print(permed_autos[i]));

permed_prod(p1,p2) = vector(#p1,i, p1[i]*p2[i]);
l=List([]);
{
  for(i=1,#permed_autos,
     my(v=vector(#autos,j,
                  my(mul=perm_times_vec(permed_autos[i][j], permed_autos[i]),
                     pos=setsearch(permed_autos, mul));
                  if(pos==0,
                     print("not found: "mul);
                    );
                  pos));
     print(v);
     listput(l,v));
}
l=Vec(l);
l=apply(Vecsmall,l);
l
for(i=1,#l, check_equal(test_is_perm(l[i]), 1));
apply(test_permorder,l)
quit


\\-----------------------------------------------------------------------------
\\ galoisinit()

install(group_isA4S4, "lG", "group_isA4S4");
{
  for(n=1,3,
     P = x^4 - 4*x - 1;
     G = galoisinit(P);
     if(G,
        my(group=G.group,
           gens=group);
        print("n="n"  "group);
        print("  gens "vecsort(gens));
        print("  A4S4=",group_isA4S4(group));
        \\ print("  num="test_perms_generators_to_group(gens));
        print("  num="#test_perms_generators_to_group(gens));
       );
     );
  quit;
}



\\-----------------------------------------------------------------------------
\\ Pari Groups

perm = Vecsmall([2,3,4,5,6,7,8,1])
order = test_permorder(perm)
install(cyclicgroup, "GL", "cyclicgroup");
cyclicgroup(perm,order)

install(trivialgroup, "", "trivialgroup");
trivialgroup()

perm1 = Vecsmall([2,3,4,5,1, 6,7,8])
perm2 = Vecsmall([1,2,3,4,5, 7,8,6])
check_equal(test_is_perm(perm1), 1);
check_equal(test_is_perm(perm2), 1);
order1 = test_permorder(perm1)
order2 = test_permorder(perm2)
install(dicyclicgroup, "GGLL", "dicyclicgroup");
group = dicyclicgroup(perm1,perm2,order1,order2)

install(group_isA4S4, "lG", "group_isA4S4");

install(group_domain, "lG", "group_domain");
group_domain(group)

install(group_elts, "GL", "group_elts");
print("group_elts ",group_elts(group,#perm1));

install(group_set, "GL", "group_set");
print("group_set  ",group_set(group,#perm1));

install(groupelts_set, "GL", "groupelts_set");
groupelts_set(group,#perm1)

install(group_order, "lG", "group_order");
group_order(group)

install(group_isabelian, "lG", "group_isabelian");
group_isabelian(group)

print("HNF");
install(group_abelianHNF, "GDG", "group_abelianHNF");
group_abelianHNF(group)

print("SNF");
install(group_abelianSNF, "GDG", "group_abelianSNF");
group_abelianSNF(group)

group_S(n,w=n) =
{
  n>=2 || error();
  my(perm1=vectorsmall(w,i, if(i==1,2, i==2,1, i)));
  check_equal(test_is_perm(perm1), 1);
  check_equal(test_permorder(perm1), 2);
  if(n==2,
     return([ [perm1], Vecsmall([2]) ]));

  my(perm2=vectorsmall(w,i, if(i<=n, (i%n)+1, i)));
  check_equal(test_is_perm(perm2), 1);
  check_equal(test_permorder(perm2), n);
  [ [perm1,perm2], Vecsmall([2,n]) ];
  dicyclicgroup(perm1,perm2,2,n);
}
print("S3   = ", S3   = group_S(3));
test_smallgroup_validate(S3);
check_equal(group_isabelian(S3), 0, "S3 not Abelian");
check_equal(group_isA4S4(S3), 0, "S3 vs group_isA4S4()");

print("S2_3 = ", S2_3 = group_S(2,3));
test_smallgroup_validate(S2_3);
check_equal(group_domain(S3),   3);
check_equal(group_domain(S2_3), 3);
check_equal(group_isabelian(S2_3), 1, "S2_3 is Abelian");

print("S4   = ", S4   = group_S(4));
test_smallgroup_validate(S4);
check_equal(group_isabelian(S4), 0, "S4 not Abelian");
\\ FIXME
\\ check_equal(group_isA4S4(S4), 1, "S4 vs group_isA4S4()");

make_group_C(n,w=n) =
{
  n>=2 || error();
  my(perm=vectorsmall(w,i, if(i<=n, (i%n)+1, i)));
  [ [perm1,perm], Vecsmall([2,n]) ];
  cyclicgroup(perm,n);
}
print("C3   = ", C3   = make_group_C(3));
test_smallgroup_validate(C3);

install(checkgroup, "GD&", "checkgroup");
\\ print("checkgroup");
\\ checkgroup(S3)
\\ print("checkgroup");
\\ checkgroup(S,&elts)
\\ elts

S = group_S(#perm1)
print("group_subgroup_isnormal");
install(group_subgroup_isnormal, "lGG", "group_subgroup_isnormal");
group_subgroup_isnormal(group,S)
group_subgroup_isnormal(S3,S2_3)

print("group_leftcoset");
install(group_leftcoset, "GG", "group_leftcoset");
install(group_rightcoset, "GG", "group_rightcoset");
group_leftcoset(group,perm1)
group_rightcoset(group,perm1)

print("group_perm_normalize");
install(group_perm_normalize, "lGG", "group_perm_normalize");
group_perm_normalize(group,perm1)

print("group_quotient");
install(group_quotient, "GG", "group_quotient");
group_quotient(S,group)
group_quotient(group_S(2,3),group_S(3))
group_quotient(S3, S2_3)

print("vpar");
vpar = [0,1,2, 1,4, 1,6, 1,8]
vpar_validate(vpar);
print("gens");
vpar_num_automorphisms(vpar)
vpar_num_automorphism_generators(vpar)
gens = vpar_automorphism_generators(vpar)
\\ WRONG
gens_to_group(gens) = [ gens, Vecsmall(apply(test_permorder,gens)) ];
group = gens_to_group(gens)
n = #gens[1];
S = group_S(n)
group_subgroup_isnormal(S,group)
group_isabelian(group)
group_order(group)
group_elts(group,n)
#group_elts(group,n)
check_equal(4!, 24)
groupelts_set(group,n)

install(group_subgroups, "G", "group_subgroups");

print("galoisinit");
   P = x^6 + 108
   G = galoisinit(P)
   L = galoissubgroups(G)
print("G.group ", G.group);

install(galois_group, "G", "galois_group");
print("galois_group  g = ", g = galois_group(G));
test_smallgroup_validate(g);
group_subgroups(g)

print("A4 = ", A4 = [ [Vecsmall([2,1, 3,4]), Vecsmall([1,2, 4,3])], Vecsmall([2,2]) ]);
check_equal(group_isA4S4(A4), 1);
test_smallgroup_validate(A4);
print("group_subgroups of A4");
group_subgroups(A4)

print("C4   = ", C4 = make_group_C(4));
test_smallgroup_validate(C4);
print("group_subgroups of C4");
group_subgroups(C4)

print("C3xC3 = ", C33 =   [ [Vecsmall([2,3,1, 4,5,6]), Vecsmall([1,2,3, 5,6,4])], Vecsmall([3,3]) ]);
print("C3 in 6 = ", C33 =   [ [Vecsmall([2,3,1, 4,5,6])], Vecsmall([3]) ]);
test_smallgroup_validate(C33);
print("group_subgroups of C33");
group_subgroups(C33)

print("group_subgroups of S2");
print("S2 = ", S2 = group_S(2));
group_subgroups(S2)

print("group_subgroups of S3");
group_subgroups(S3)

quit


\\-----------------------------------------------------------------------------
\\ vpar_seq_lexmin()

\\        10
\\       / | \
\\     3   4   9
\\    / \     / \
\\   1   2   5   8
\\              / \
\\             6   7
\\
vpar=[3,3,10,10,9,8,8,9,10, 0];     \\ Knuth 2.3.4.4
print("postorder ",Vec(vpar_seq_postorder(vpar)));
print("preorder  ",Vec(vpar_seq_preorder(vpar)));
print("premax    ",Vec(vpar_seq_premax(vpar))" vpar "vpar_to_unlabelled(vpar));
print("lexmin    ",Vec(vpar_seq_lexmin(vpar))" vpar "vpar_to_lexmin(vpar));
print("rows      ",Vec(vpar_seq_rows(vpar)));
vpar_seq_premax(vpar) * vpar_seq_rows(vpar) * vpar_seq_premax(vpar)^-1
vpar_seq_rows(vpar) * vpar_seq_premax(vpar) * vpar_seq_rows(vpar)^-1
vpar_seq_rows(vpar) * vpar_seq_premax(vpar)^-1
vpar_seq_premax(vpar)^-1 * vpar_seq_rows(vpar)
vpar_seq_rows(vpar)^-1 * vpar_seq_premax(vpar)
vpar_seq_premax(vpar) * vpar_seq_rows(vpar)^-1
r=vpar;
r = vpar_relabel_seq(r,vpar_seq_premax(r))
r = vpar_relabel_seq(r,vpar_seq_rows(r))
p = (vpar_seq_rows(r)^-1 * vpar_seq_premax(vpar)^-1)
s = p^-1
r=vpar
pre = vpar_relabel_perm(vpar, vpar_seq_premax(vpar)^-1);
vpar_relabel_perm(vpar, vpar_seq_rows(pre)^-1 * vpar_seq_premax(vpar)^-1)
want=vpar_seq_rows(pre);
want=vpar_seq_rows(pre)^-1 * vpar_seq_premax(vpar)^-1;
print("want "want);
v=[vpar_seq_premax(vpar), vpar_seq_rows(vpar), vpar_seq_preorder(vpar), test_perm_reversal(#vpar)];
v=concat(v,apply(p->p^-1,v));
{
  for(n=1,8,
     forvec(c=vector(n,i,[1,#v]),
           my(got=vpar_INTERNAL_identity_perm(#vpar));
           for(i=1,#c, got *= v[c[i]]);
           \\ print("n="n"  "got" vs "want);
           if(got==want,
              print("c="c))));
}
quit




\\-----------------------------------------------------------------------------
\\ Claw Count Total Cases

\\ claws not at root of tree
\\ n*binomial(n-1,3)*4  claw middle, others, 3+1 choose parent or none
\\ it and n-4 rest = n-3 blobs
\\ as tree (n-3)^(n-3-1) = (n-3)^(n-4)
\\ n-4 non-roots
\\ when claw is root can raise those from n-3 choices to n choices
\\   1/(n-3) root choice is (n-3)^(n-5), raise to 4*n^(n-5) with 4 of root
\\ when claw not root choose n-4 as root and rest...
\\   (n-4) * n^(n-5)
\\ (n-3 + 4) * n^(n-5) * n*binomial(n-1,3)*4
\\ = (n+1) * n^(n-5)

\\ n*binomial(n-1,3)*4  claw middle, others, 3+1 choose parent or none
\\ tree n-3 blobs (n-3)^(n-4)
\\ raise to n^(n-4)
\\ so 4*binomial(n-1,3) * n^(n-3)

\\ n*binomial(n-1,3)*4  claw middle, others, 3+1 choose parent or none
\\ forest n-3 blobs (n-2)^(n-4)
\\ raise to (n+1)^(n-4)
\\ so 4*n*binomial(n-1,3) * (n+1)^(n-4)

vector(8,n, 4*n*binomial(n-1,3))
vector(8,n, 16*binomial(n,4))
quit

\\ non-root claws
vector(8,n, vpar_num_claws_total(n) - n*binomial(n-1,3)*4 * if(n<=4,1, 4*n^(n-5)) )
vector(8,n, if(n<=4,0, 4*binomial(n-1,3) * (n-4) * n^(n-4)) )
quit


\\ claws at root of tree
\\ n*binomial(n-1,3)*4  claw middle, others, 3+1 choose parent or none
\\ rest n-4 forest (n-4+1)^(n-4-1) = (n-3)^(n-5)
\\ but 1-choice 0 as root becomes 4-choice, so n-4+1+3 = n each in power,
\\ and final always-0 becomes 4*
vector(8,n, n*binomial(n-1,3)*4 * if(n<=4,1, 4*n^(n-5)) )

{
  for(n=1,9,
     my(count=0);
     forvec(vec=vpar_forvec_from(n),
           my(vpar=vpar_from_upascend(vec),
              root=vpar_root(vpar),
              degree=vpar_degree_of_vertex(vpar,root));
           count += binomial(degree,3);

           my(children=vpar_children_of_vertex(vpar,root));
           for(i=1,#children,
              my(c=children[i],
                 num_children=vpar_num_children_of_vertex(vpar,c));
              count += binomial(num_children,2));
     );
     print1(count", ");
     );
  quit;
}


\\ claws middle as root of tree
\\ n*binomial(n-1,3)*4  claw middle, others
\\ (n-3)^(n-4) ways tree of it and other vertices
\\ when claw root, others any so n^(n-5)
vector(8,n, n*binomial(n-1,3) * if(n<5,1, (n-3) * n^(n-5)) )

{
  for(n=1,9,
     my(count=0);
     forvec(vec=vpar_forvec_from(n),
           my(vpar=vpar_from_upascend(vec),
              root=vpar_root(vpar),
              degree=vpar_degree_of_vertex(vpar,root));
           count += binomial(degree,3));
     print1(count", ");
     );
  quit;
}

\\-----------------------------------------------------------------------------
\\ Claw Count Total

\\ n*binomial(n-1,3)*4  claw middle, others, 3+1 choose parent or none
\\ (n-3)^(n-4) ways tree of it and other vertices
\\ when claw root, others any so n^(n-5)
\\ when claw not root, others except root  (n-3)*(n-1)*n^(n-6)
\\ 4*binomial(n-1,3) * (n-3) * n^(n-4)
print(vector(8,n, vpar_num_claws_total(n))," = vpar_num_claws_total");
vector(8,n, n*binomial(n-1,3)*4 * if(n<5,1, (n-3) * n^(n-5)) )

n=4;
check_equal(n*binomial(n-1,3)*4, 16);
check_equal(n-3, 1);
check_equal((n-3)^(n-4), 1);

n=5;
check_equal(n*binomial(n-1,3)*4, 80);
check_equal(n*binomial(n-1,3)*4 * 5, 400);
check_equal(n-4, 1);
check_equal(n-3, 2);
check_equal(vpar_count(n-3,-1), 1);  \\ root=1 of 2 vertices
check_equal((n-3)^(n-5), 1);  \\ root=1 of 2 vertices
my(n=6); check_equal(vpar_count(n-3,-1), 3);  \\ root=1 of 3 vertices
my(n=6); check_equal((n-3)^(n-5), 3);  \\ root=1 of 3 vertices
\\ n-3 root=1  (n-3)^(n-5)

check_equal( (n-1)^(n-4), 4);  \\ claw root, others increase to any

\\ non-claw n-4 root
\\ claw under n-4 others
\\ non-claws under n-1 others
check_equal( (n-4) * (n-4) * (n-1)^(n-5), 1);
\\ other root,  (n-4) * (n-3) * n^(n-6)

vector(8,n,n+=3; n*binomial(n-1,3)*4 * ( (n-1)^(n-4) + (n-4) * (n-4) * (n-1)^(n-5) ) )

quit;


\\------------------------------------------------------------------------------
\\ vpar_num_claws() total in forest

\\ n=5
check_equal(vpar_num_claws_total(5), 400);
check_equal(binomial(5,4)*vpar_num_claws_total(4), 80);

\\ n=6
check_equal(vpar_num_claws_total(6), 8640);
check_equal(binomial(6,5)*vpar_num_claws_total(5)*vpar_count(1,1), 2400);
check_equal(binomial(6,4)*vpar_num_claws_total(4)*vpar_count(2,1), 720);
check_equal(8640 + 2400 + 720, 11760, "n=6 total");

\\ n=8
check_equal(vpar_num_claws_total(8), 4587520);
7check_equal(binomial(8,7)*vpar_num_claws_total(7), 1536640);
check_equal(binomial(8,6)*vpar_num_claws_total(6), 241920);
check_equal(binomial(8,5)*vpar_num_claws_total(5), 22400);
check_equal(binomial(8,4)*vpar_num_claws_total(5), 28000);

check_equal(vpar_num_claws_total(8,1), 7348320);
check_equal(7348320 - (4587520 + 1536640 + 241920 + 22400), 959840);

\\ want 16, 480, 11760, 286720, 7348320, 201600000,

\\ choose t vertices for tree containing v=1
\\ its count, times num forests of rest which its count will apply in
\\ rest count, times num trees t which its count will apply in
F(n) =
{
  if(n<4,return(0));
  vpar_num_claws_total(n)
  +
  sum(t=1,n-1,
      binomial(n-1,t-1) * (vpar_num_claws_total(t)*vpar_count(n-t,1)
                           + F(n-t)*vpar_count(t)));

  vpar_num_claws_total(n)
  +
  1/n * sum(t=1,n-1, binomial(n,t) *t * vpar_num_claws_total(t)*vpar_count(n-t,1))
  +
  1/n * sum(t=1,n-1, binomial(n,t) *t * F(n-t)*vpar_count(t));
}
vector(9,n, F(n))


quit



\\------------------------------------------------------------------------------
\\ vpar_num_claws() total

\\ labelled rooted trees = n * trees root=1
\\              0, 0, 0, 16, 400, 8640, 192080,
\\ not in OEIS:          16, 400, 8640, 192080,
\\
\\ labelled rooted forests
\\              0, 0, 0, 16, 480, 11760, 286720, 7348320, 201600000,
\\ not in OEIS:          16, 480, 11760, 286720, 7348320, 201600000,
\\
\\ labelled rooted trees root=1
\\ 0, 0, 0, 4, 80, 1440, 27440, 573440, 13226976,
\\          4, 80, 1440, 27440, 573440, 13226976,
\\ A053508 binomial(n-1,3)*n^(n-4)
\\
\\ B(n) = if(n==0,0, 4*binomial(n-1,3)*n^(n-4));
\\ T(n) = n*B(n);
\\ F(n) = sum(t=1,n, binomial(n,t)*T(t)*vpar_count(n-t));
\\ vector(8,n, F(n))
\\ F(n) = my(v=test_compositions(n)); sum(i=1,#v, test_multinomial(Vec(v[i]))*sum(j=1,#v[i], B(v[i][j])));
{
  my(forest=1); \\ trees or forests
  for(n=1,10,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count+=vpar_num_claws(vpar));
     print(count",");
  );
  quit;
}

\\ unlabelled rooted trees
\\              0, 0, 0, 0, 2, 12, 49, 170, 543, 1671, 5010, 14870, 43735, 128246, 374816, 1094276,
\\ not in OEIS:             2, 12, 49, 170, 543, 1671, 5010, 14870, 43735, 128246, 374816, 1094276,
{
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           count+=vpar_num_claws(vpar));
     print(count",");
     );
  quit;
}

\\ free trees
\\              0, 0, 0, 1, 5, 18, 50, 138, 349, 911, 2308, 6017, 15620, 41296, 109470,
\\ not in OEIS:             5, 18, 50, 138, 349, 911, 2308, 6017, 15620, 41296, 109470,

try(vpar) = count+=vpar_num_claws(vpar)
{
  for(n=1,20,
     count=0;
     total=0;
     read(Str("/tmp/trees-"n".gp"));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Domination Number Etc Different

\\ 4 different  (excl indomnum)
\\        1  2  3  4  5  6  7
\\ n=13  [0, 1, 2, 3, 4, 5, 2, 1, 8, 9, 10, 11, 8]
\\           1
\\          / \
\\         2   8
\\        / \   \ \
\\       3   7   9  12
\\      /         \
\\     4          10
\\    /             \
\\   5              11
\\  /
\\ 6
\\
\\        1  2  3  4  5  6  7
\\ n=13  [0, 1, 2, 3, 4, 2, 6, 6, 1, 9, 10, 11, 1]
\\           1
\\          / \ \
\\         2   9  12
\\        / \   \
\\       3   6   10
\\      /   / \   \
\\     4   7   8   11
\\    /
\\   5

\\ 5 different
\\        1  2  3  4  5  6  7  8  9 10 11  12  13  14 15 16
\\ n=20  [0, 1, 2, 3, 4, 5, 5, 3, 8, 8, 2, 11, 12, 13, 2, 1, 16, 17, 18, 1]
\\
\\        6---5---4---3----2------1---16---17---18---19
\\            |       |   / \     |
\\            7       8   11 15   20
\\                  / |   |
\\                 9 10   12
\\                        |
\\                        13
\\                        |
\\                        14
\\ n=21 many

vector_is_all_different(vec) = #vecsort(vec,,8)==#vec;

try(vpar) =
{
  my(domnum=vpar_domnum(vpar),
     indomnum=vpar_indomnum(vpar),
     totdomnum=vpar_totdomnum(vpar),
     semitotdomnum=vpar_semitotdomnum(vpar),
     perfdomnum=vpar_perfect_domnum(vpar),
     matchnum=vpar_matchnum(vpar));
  if(vector_is_all_different([domnum
                              , indomnum
                              , totdomnum
                              , semitotdomnum
                              , perfdomnum
                              \\ , matchnum
              ]),
     count++;
     print("n="#vpar"  "vpar));
  0;
}

{
  count=0;
  for(n=1,21,
     print("n="n);
     read(Str("/tmp/trees-"n".gp"));
     if(count, print("stop");break());
  );
  quit;
}
{
  for(n=1,10000000,
     try(vpar_make_random(random(100))));
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_indsets_count() factorization

\\ primes not factors: 3137, 3371, 3389, 3613, 3847, 3931, 3967, 4013, 4049, 4139, 4177, 4229

\\ primes not whole count:
\\ not in OEIS: 7, 11, 19, 29, 31, 47, 53, 67, 71, 73, 79, 103, 127, 131, 137, 139, 151, 179, 191, 193, 211,

\\ integers whole count:
\\ not in OEIS: 2, 3, 5, 8, 9, 13, 14, 17, 21, 22, 23, 24, 26, 33, 34, 35, 36, 37, 38, 40, 41,
\\ integers not whole count:
\\ not in OEIS: 4, 6, 7, 10, 11, 12, 15, 16, 18, 19, 20, 25, 27, 28, 29, 30, 31, 32, 39, 42,

\\ {
\\   for(n=1,20,
\\      my(vpar=vpar_free_first(n));
\\      until((vpar=vpar_free_next(vpar))=='none,
\\            my(count=vpar_indsets_count(vpar));
\\            if(count==7,
\\               print(vpar)));
\\      );
\\ }


factor_vec(n) =
{
  my(m=factor(n));
  concat(vector(matsize(m)[1],i, vector(m[i,2],j, m[i,1])));
}
{
  my(seen=vectorsmall(10000),
     seenwhole=vectorsmall(10000));
  for(n=1,16,
     print("n="n);
     my(vpar=vpar_free_first(n),
        m=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(count=vpar_indsets_count(vpar),
              vec=factor_vec(count));
           \\ print(" "vec" = "count);
           \\ for(i=1,#vec,
           \\    my(p=vec[i]);
           \\    if(p<#seen && !seen[p], print("  first "p"   "vpar); seen[p]=1));
           m=max(m,count);
           if(count<#seenwhole, seenwhole[count]=1);
          );
     print(" maximum "m);
     );

  print1("primes not factors: ");
  my(count=0);
  forprime(p=2,#seen,
           if(!seen[p],
              print1(p", ");
              if(count++>20,break())));
  print();

  print1("integers whole count: ");
  my(count=0);
  for(p=1,#seenwhole,
           if(seenwhole[p],
              print1(p", ");
              if(count++>20,break())));
  print();

  print1("integers not whole count: ");
  my(count=0);
  for(p=1,#seenwhole,
           if(!seenwhole[p],
              print1(p", ");
              if(count++>20,break())));
  print();

  print1("primes not whole count: ");
  my(count=0);
  forprime(p=2,#seenwhole,
           if(!seenwhole[p],
              print1(p", ");
              if(count++>20,break())));
  print();

}
quit;


\\------------------------------------------------------------------------------
\\ Nesetril Trees Determined by Maximal Asymmetric Subtrees
\\
\\ Jaroslav Nesetril, "A Congruence Theorem For Asymmetric Trees", Pacific
\\ Journal of Mathematics, volume 37, number 3, March 1971, pages 771-778.
\\ https://projecteuclid.org/euclid.pjm/1102970478
\\
\\ A(T) = list all maximal asymmetric proper subtrees, up to isomorphism.
\\ Maximal by vertex-wise subset.
\\ AG(T) = set all maximal asymmetric proper subtrees, ie. list unduplicated.

\\ Return 1 if sub is an induced free subtree of vpar.
is_free_subtree(vpar,sub) =
{
  if(#sub <= #vpar,
     forvec(flags=vector(#vpar,i,[0,1]),
           vecsum(flags) == #sub || next();
           if(vpar_is_free_isomorphic(sub,vpar_keep_flags(vpar,flags)),
              return(1))));
  0;
}

set_is_subset(subset,superset) = setminus(subset,superset)==[];

\\ Return [ [v1,v2,...], [v1,v2,...] ] vector of vectors of vertex numbers.
\\ Each of the subtrees of those vertices are asymmetric induced subtrees of
\\ vpar and are maximal in the sense not a subset of any other.
maximal_asymmetric_subtree_vertices(vpar) =
{
  my(l=List([]));
  forvec(flags=vector(#vpar,i,[0,1]),
        my(size=vecsum(flags));
        (2<=size && size<#vpar) || next();
        my(sub=vpar_keep_flags(vpar,flags));
        vpar_is_tree(sub) || next();
        vpar_is_free_asymmetric(sub) || next();
        listput(l,test_flags_to_Set(flags)));
  my(i=1);
  while(i<#l,
        for(j=1,#l,
           if(i!=j && set_is_subset(l[i],l[j]),
              listpop(l,i); next(2)));
        i++);
  Vec(l);
}

\\ Return vector of vpar_to_free representatives of the maximal asymmetric
\\ subtrees of vpar.  Sorted and with duplicates included.
maximal_asymmetric_subtrees_list(vpar) =
{
  vecsort(apply(set->vpar_to_free(vpar_keep_vertices(vpar,set)),
               maximal_asymmetric_subtree_vertices(vpar)));
}

\\ Return vector of vpar_to_free representatives of the maximal asymmetric
\\ subtrees of vpar.  Sorted and unduplicated.
maximal_asymmetric_subtrees_set(vpar) =
{
  \\ my(l=List([]));
  \\ forvec(flags=vector(#vpar,i,[0,1]),
  \\       my(size=vecsum(flags));
  \\       (0<size && size<#vpar) || next();
  \\       my(sub=vpar_keep_flags(vpar,flags));
  \\       vpar_is_tree(sub) || next();
  \\       vpar_is_free_asymmetric(sub) || next();
  \\       sub=vpar_to_free(sub);
  \\       for(i=1,#l,
  \\          if(#l[i] < size, break());
  \\             if(if(#l[i] < size, sub==l[i], is_free_subtree(l[i],sub)),
  \\                next(2)));
  \\       listput(l,sub);
  \\       l=vecsort(l,(x,y)->lex(#x,#y),4));
  \\ my(i=1);
  \\ while(i<#l,
  \\       if(is_free_subtree(l[i],l[i+1]),
  \\          listpop(l,i+1),
  \\          i++));
  \\ Vec(l);

  \\ my(l=List(apply(set->vpar_keep_vertices(vpar,set),
  \\                maximal_asymmetric_subtree_vertices(vpar))));
  \\ my(i=1);
  \\ while(i<#l,
  \\       for(j=1,#l,
  \\          if(i!=j && is_free_subtree(l[i],l[j]),
  \\             listpop(l,i); next(2)));
  \\       i++);
  \\ vecsort(apply(vpar_to_free, Vec(l)));

  Set(maximal_asymmetric_subtrees_list(vpar));  \\ unduplicate
}

\\ n=9 pair
\\
\\ common maximal_asymmetric_subtrees_set(),
\\ star 1,2,3 smallest asymmetric
\\ star 1,2,4 second smallest asymmetric
check_equal(vpar_is_free_asymmetric([0, 1, 2, 3, 1, 5, 1]), 1);
check_equal(vpar_is_free_asymmetric([0, 1, 2, 3, 2, 1, 6, 7]), 1);
check_equal(vpar_count_free_asymmetric(7), 1);
check_equal(vpar_count_free_asymmetric(8), 1);


\\      - *
\\     /                        1,3,4
\\    * - *---*---*
\\     \
\\      - *---*---*---*
\\      1  2 3 4 5  6 7
vpar = [0, 1,2,3,4, 2,6,7,8];
vpar=vpar_to_free(vpar)
vpar_validate(vpar);
print("n="#vpar"  "maximal_asymmetric_subtrees_set(vpar));
maximal_asymmetric_subtree_vertices(vpar)

\\      -*  -*
\\     /   /
\\    * - *---*---*
\\     \
\\      - *---*---*
\\      1 2 3 4 5 6 7
vpar = [0,1,2,3,4,5,6, 3,4];
vpar=vpar_to_free(vpar)
vpar_validate(vpar);
print("n="#vpar"  "maximal_asymmetric_subtrees_set(vpar));
maximal_asymmetric_subtree_vertices(vpar)
print();


\\ n=10 (wrong)
\\ [0, 1, 2, 3, 4, 2, 1, 7, 8, 7]
\\    5---4---3---2---1---7---8---9
\\                |       |
\\                6      10
\\
\\ [0, 1, 2, 3, 4, 1, 6, 7, 1, 9]
\\      - 2---3---4---5
\\     /                        2,3,4
\\    1 - 6---7---8
\\     \
\\      - 9---10
\\
\\ maximal asymmetrics ?
\\ [0, 1, 2, 3, 1, 5, 1]            1,2,3 smallest
\\ [0, 1, 2, 3, 4, 1, 6, 7, 1]      1,3,4
{
  my(m=Map());
  for(n=3,14,
     print("n="n);
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           vpar_is_free_asymmetric(vpar) || next();
           my(set=maximal_asymmetric_subtrees_set(vpar),
              count=0);
           mapisdefined(m,set,&count);
           count++;
           mapput(m,set,count);
           print(vpar" -- "set" len "#set" count "count);
          ));
  m=Mat(m);
  print(m[,2]);
}
quit


\\------------------------------------------------------------------------------
\\ vpar_preorder_to_i()

{
  for(forest=0,1,
     n=5-forest;
     for(i=1,vpar_count_ordered(n,forest),
        my(vpar=vpar_preorder_ith(n,i,forest));
        print("i="i"  "vpar_preorder_to_i(vpar,forest)"  "vpar)));
}
quit


\\------------------------------------------------------------------------------
\\ vpar_Laplacian_poly() of paths

\\ 6,18,40,75,126,196,288
\\ A002411 n * n*(n+1)/2

{
  for(n=1,10,
     my(vpar=vpar_make_path(n));
     print("n="n"  W="vpar_Wiener_index(vpar)"  "vpar_Laplacian_poly(vpar)));
}
{
\\  + O(x^12)
  for(n=1,10,
     my(vpar=vpar_make_star(n));
     print("n="n"  W="vpar_Wiener_index(vpar)"  "vpar_Laplacian_poly(vpar)));
}
quit



\\------------------------------------------------------------------------------
\\ vpar_seq_premax() unique vecextracts

\\ trees unique vecextract
\\ 1, 2, 9, 40, 205, 1236, 8659,
\\ A296964 expansion of (x*exp(x)-1)/(1-x).
\\
\\ forests unique vecextract
\\ 1, 3, 10, 41, 206, 1237, 8660,
\\ A002627 a(n) = n*a(n-1) + 1, a(0) = 0.
\\
\\ root=1 unique vecextract, forests 1 later
\\ 1, 1, 3, 10, 41, 206, 1237,

\\ trees max same vecextract
\\ 1, 1, 1, 2, 3, 6,
\\ forests max same vecextract
\\ 1, 1, 2, 3, 6, 12, 30,

{
  my(forest=-1);
  for(n=1,8,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq =vpar_seq_premax(vpar),
              vec=Vecsmall(vecextract(vpar,seq)),
              num=0);
           mapisdefined(m,vec,&num);
           mapput(m,vec,num+1));
     m=Mat(m);
     my(counts=m[,2],
        count_uniques=sum(i=1,#counts, counts[i]==1),
       count_max=vecmax(counts));
     print1(count_uniques", ");
     \\ print1(count_max", ");
     );
  quit;
}
{
  my(forest=-1);
  for(n=1,4,
     print("n="n);
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq =vpar_seq_premax(vpar),
              vec=Vecsmall(vecextract(vpar,seq)),
              num=0);
           mapisdefined(m,vec,&num);
           mapput(m,vec,num+1));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              seq =vpar_seq_premax(vpar),
              vec=Vecsmall(vecextract(vpar,seq)));
           if(mapget(m,vec)==1,
              print(vpar"  "Vec(vec))));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Second Largest Eigenvalue

is_reflexive(vpar) =
{
  my(poly=vpar_adjacency_poly(vpar));
  poly /= x^valuation(poly,'x);
  for(i=-1,5,
     my(f='x+i, qd);
     while(my(qd=divrem(poly,f)); !qd[2], poly /= f));
  \\ print(factor(poly));
  print(polsturm(poly,[2,oo]));
  polsturm(poly,[2,oo]) >= 2;
}

{
  for(n=2,9,
     print("n="n);
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(poly=vpar_adjacency_poly(vpar),
              roots=polrootsreal(poly,[2,oo]),
              order=vecsort(roots,,5),
              second=if(#order>=2,roots[order[2]]));
           print(vpar"  "is_reflexive(vpar)"  "second"  all="roots)));
  quit;
}



\\------------------------------------------------------------------------------
\\ Minimal Asymmetric X9

\\ 1---2
\\ |
\\ 3---4
\\ |
\\ 5
\\ |
\\ 6---7
\\
vpar= [0,1,1,3,3,5,6]
vpar_validate(vpar);
vpar_is_free_asymmetric(vpar)

is_minimal_asymmetric(vpar) =
{
  vpar_is_free_asymmetric(vpar) || return(0);
  forvec(flags=vector(#vpar,i,[0,1]),
        my(set=select(s->s, flags, 1));
        if(#set<#vpar,
           my(sub=vpar_keep_vertices(vpar,set));  \\ subgraph
           if(#sub>=2 && vpar_is_free_asymmetric(sub),
              return(0))));
  1;
}


{
  forvec(flags=vector(#vpar,i,[0,1]),
        my(set=select(s->s, flags, 1),
           sub=vpar_delete_vertices(vpar,set));  \\ subgraph
        if(#sub<2, next());
        if(vpar_is_free_asymmetric(sub),
           print(sub"  set="set" num roots="vpar_num_roots(vpar))));
}
print(vpar_to_sparse6(vpar))

{
  for(n=2,12,
     print("n="n);
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           if(is_minimal_asymmetric(vpar),
              print(vpar)));
     );
  quit;
}

quit


\\------------------------------------------------------------------------------
\\ different domination

vpar = [0, 1, 2, 3, 4, 5, 2, 1, 8, 9, 10, 11, 8];  \\ n=13
vpar = [0, 1, 2, 3, 4, 5, 5, 3, 8, 8, 2, 11, 12, 13, 2, 1, 16, 17, 18, 1];

\\ n=24 min,max diameter
vpar=[0,1,2,3,4,5,2,7,8,9,2,11,12,13,2,15,15,15,1,19,20,21,22,19]
vpar=[0,1,2,3,4,5,6,7,8,5,4,4,3,13,13,1,16,17,18,19,20,21,18,1]

[vpar_domnum(vpar), \
 vpar_indomnum(vpar), \
 vpar_perfect_domnum(vpar), \
 vpar_semitotdomnum(vpar), \
 vpar_totdomnum(vpar), \
 vpar_matchnum(vpar), \
 vpar_indnum(vpar), \
 "", \
vpar_diameter(vpar) ]

[vpar_domnum_and_count(vpar), \
 vpar_indomnum_and_count(vpar), \
 vpar_perfect_domnum_and_count(vpar), \
 vpar_semitotdomnum_and_count(vpar), \
 vpar_totdomnum_and_count(vpar) ]

[vpar_domnum_and_count(vpar), \
vpar_indomnum_and_count(vpar), \
vpar_perfect_domnum_and_count(vpar), \
vpar_semitotdomnum_and_count(vpar), \
vpar_totdomnum_and_count(vpar), \
vpar_matchnum_and_count(vpar), \
vpar_indnum_and_count(vpar) ]

quit


\\------------------------------------------------------------------------------
\\ domnum different

funcs={[
        vpar_domnum,
        vpar_indomnum,
        vpar_totdomnum,
        vpar_semitotdomnum,
        vpar_perfect_domnum
]};

vector_to_classes(vec) =
{
  my(ret=vec,
     upto=0);
  for(i=1,#vec,
     for(j=1,i-1,
        if(vec[i]==vec[j],
           ret[i]=ret[j]; next(2)));
     ret[i] = upto++);
  ret;
}

{
  my(seen=Map());
  for(n=1,20,
     print("n="n);
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(vec=vector(#funcs,i, funcs[i](vpar)),
              classes=vector_to_classes(vec),
              prev);
           if(!mapisdefined(seen,classes,&prev) || prev==n,
              mapput(seen,classes,n);
              if(classes==[1,2,3,4,5],
                 print("c="classes"  "vpar"  n="n);
              ));
          );
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_depths_vector() == vpar
\\
\\ 1, 1, 2, 5, 14, 43, 144,
\\ A047970 antidiagonal sums of nexus numbers A047969
\\ see nex/misc.c more values
\\ (1 - x)* sum x^n/(1 - (n+2)*x)
\\ A047969
\\ nexus numbers (n+1)^(k+1) - n^(k+1)

vector(12,n,n--; 1 + sum(k=1,n-1, (k-1)*k^(n-k-1)))
vector(12,n,n--; 1 + sum(k=1,n-1, k^(n-k)) - sum(k=1,n-1, k^(n-k-1)))

n=5;
for(k=1,5, print((k-1)*k^(n-k-1)));


vpar=[0, 1, 2, 3, 4, 5, 6, 7, 8, 1, 1, 1]
vpar_depths_vector(vpar)
vpar_is_preorder(vpar)

\\ Return the length of an initial path [0,1,2,...] in vpar.
\\ If vpar doesn't begin with any such (so vpar[1]!=0) then return 0.
initial_pathlen(vpar) =
{
  for(k=1,#vpar,
     if(vpar[k]!=k-1,
        return(k-1)));
  #vpar;
}

{
  my(forest=0);
  for(n=5,5,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           \\ vpar_is_preorder(vpar) || next();
           my(depths=vpar_depths_vector(vpar),
              eq=(vpar==depths));
           if(eq,
              print(vpar" pre="vpar_is_preorder(vpar)" k="initial_pathlen(vpar));
              count++);
           vpar_is_equaldepths(vpar) == eq || error();
           );
     print1(count", ");
     );
  quit;
}

{
  my(forest=0);
  for(n=1,12,
     my(count=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar))=='none,
           my(depths=vpar_depths_vector(vpar),
              eq=(vpar==depths));
           if(eq,
              \\ print(vpar);
              count++;
             );
           vpar_is_equaldepths(vpar) == eq || error();
          );
     print(count);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_automorphism_generators() total
\\
\\ Excluding identity perm.

\\ unlabelled trees
\\          n = 1  2  3  4
\\              0, 0, 1, 3, 8, 20, 56, 149, 411, 1133, 3160, 8839, 24904,
\\ not in OEIS:          3, 8, 20, 56, 149, 411, 1133, 3160, 8839, 24904,

\\ n=4   *                           *
\\      /|\     2 generators         |
\\     * * *                         *
\\                                  / \     1 generator
\\                                 *   *

{
  my(forest=0);
  for(n=1,7,
     my(count=0,
        total=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           total += vpar_num_automorphism_generators(vpar);
           count++);
     print("n="n"  "total*1.0/count);  \\ mean
     );
  quit;
}
{
  my(forest=0);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        total=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           total += vpar_num_automorphism_generators(vpar));
     \\ print1(total,", ");
     print("n="n"  "total*1.0/vpar_count_unlabelled(n,forest));  \\ mean
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_preorder_next() and vpar ordering

{
  my(forest=0);
  for(n=2,6,
     my(count=0,
        prev,
        vpar=vpar_preorder_first(n));
     while(prev=vpar; (vpar=vpar_preorder_next(vpar,forest))!='none,
           my(c=lex(vpar,prev));
           print(vpar"  "c);
           count += c;
           \\ if(eq && n==3, print(vpar));
          );
     print(count", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_seq_premax() speed

target = 200;
verbose = 1;
try(n) =
{
  my(reps=5);
  while(reps<1000000,
        if(verbose, print("n="n" reps="reps));
        my(t1=getabstime());
        for(i=1,reps,
           vpar_seq_premax(vpar_make_random(n)));
        my(ms=getabstime()-t1);
        if(verbose, print(" ms="ms));
        ms >= 0 || error("ms="ms);
        if(ms >= target,
           return(ms*1.0/reps));
        if(ms==0,
           reps *=3,
           reps *= ceil(target*1.1/ms));
        );
}

\\ try(10)
\\ try(15)
\\ try(20)
\\ try(50)
\\ vector(20,n,n+=9; try(n))
\\ prev=0;
\\ for(n=10,100, this=try(n)/n; print(this"  "this-prev); prev=this);

m=Map();
try=memoize(try);
f(n) = n=floor(n); try(n)/n;
ploth(x=10,200,f(x))
quit



\\------------------------------------------------------------------------------
\\ vpar_seq_premax() not reversible

\\ vpar=[0, 1, 1, 2] vec=[0, 1, 2, 1] seen 0
\\ vpar=[0, 1, 2, 1] vec=[0, 1, 2, 1] seen 1

\\     1           1
\\    / \         / \
\\   2   3       2   4
\\   |           |
\\   4           3

{
  for(n=1,4,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,0), \\ trees
           my(vpar=vpar_from_upascend(vec));
           my(seq =vpar_seq_premax(vpar));
           my(vec=vecextract(vpar,seq));
           my(seen=mapisdefined(m,vec));
           print("vpar="vpar" vec="vec" seen "seen);
           mapput(m,vec,1);
     ));
  quit;
}

\\-----------------------------------------------------------------------------
\\ Iterate Non-Cyclics

\\ forests sans last
\\ 1, 2, 8, 50, 432, 4802, 65536,
\\ A007334 spanning trees on complete collapse one edge
\\ 2*n^(n-3)
\\ cf all of [0..n] for n-1
\\ 2, 3, 16, 125, 1296, 16807,
\\ A000272 = n^(n-2)

\\ trees sans last
\\ not in OEIS: 1, 2, 7, 40, 325, 3456, 45619,

{
  my(forest=0); \\ trees or forests
  for(n=1,8,
     \\ print("n="n);
     my(m=Map(),
        a=Map());
     forvec(vpar=vector(n,i,[0,n]),
           mapput(a,Vec(vpar,n-1),1);
           if(vpar_is_cyclic(vpar),
              \\ print(" skip "vpar);
              next());
           forest || vpar_is_tree(vpar) || next();
           \\ print(vpar);

           my(sans=Vec(vpar,n-1));
           mapput(m,sans,1);
     );
     m=Mat(m);
     a=Mat(a);
     print1(matsize(a)[1]", ");
     \\ print1(matsize(m)[1] == matsize(a)[1]", ");
     \\ print();
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_seq_premax()

\\         1          id=6
\\       /   \
\\      2     6       id=3,4          id[2]=2 from 5
\\      |\    |                       then new id=3 from 3
\\      3 5   7       ids=2,1,2       id[6]=4 from 7
\\      |     |
\\      4     8       ids=1,1
\\
\\ [0, 1, 2, 3, 2, 1, 6, 7]  vpar
\\  1  2  3  4  5  6  7  8
\\ [6, 3, 2, 1, 1, 4, 2, 1]  ids
\\     *           *
\\ [0, 1, 2, 3, 2, 1, 2, 3]  depths
vpar=[0, 1, 2, 3, 2, 1, 6, 7];

\\         1
\\       /| |\
\\      2 3 4 5
\\      | | | |
\\      6 7 8 9
\\            |
\\            10
\\
vpar=[0,1,1,1,1,2,3,4,5, 9];
seq=vpar_seq_premax(vpar)
vpar_seq_is_ordered(vpar,seq)
depths=vpar_depths_vector(vpar);
vecextract(depths,seq)
quit


\\------------------------------------------------------------------------------
\\ vpar_rowpos_vector() occurring
\\
\\ Same counts as siblingpos

\\ forests
\\ 1, 2, 4, 10, 26, 76, 232,
\\ A000085 involutions
\\
\\ trees
\\ 1, 1, 3, 7, 20, 56, 182,
\\ A238124 ballot sequences =1 largest part

{
  my(forest=1); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_rowpos_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_disjoint_domnum_and_count() most ways

\\ cf similar in sequences.c

\\ n=19
\\ [0, 1, 2, 3, 2, 5, 2, 7, 2, 9, 1, 11, 12, 11, 14, 11, 16, 11, 18]

try(vpar) =
{
  my(count=vpar_disjoint_domnum_and_count(vpar)[2]);
  if(count==max_count, reps++,
     count>max_count, max_count=count; reps=1; max_vpar=vpar);
  0;
}
{
  for(n=19,19,
     max_count=0;
     reps=0;
     max_vpar=[];
     read(Str("/tmp/trees-"n".gp"));
     print("n="n"  "max_count"  reps "reps);
     print("  "max_vpar);
     print("  "vpar_disjoint_domnum_and_count(max_vpar));
  );
  quit;
}

{
  for(n=19,20,
     my(vpar=vpar_free_first(n),
        max_count=0,
        reps=0,
        max_vpar);
     until((vpar=vpar_free_next(vpar))=='none,
           my(count=vpar_disjoint_domnum_and_count(vpar)[2]);
           if(count==max_count, reps++,
              count>max_count, max_count=count; reps=1; max_vpar=vpar));
     print("n="n"  "max_count"  reps "reps);
     print("  "max_vpar);
     );
  quit;
}



\\-----------------------------------------------------------------------------
\\ Free Trees matchnum==2

\\ n-4 trees of matchnum==2
\\ 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,

try(vpar) =
{
  count += (vpar_matchnum(vpar)==2);
  0;
}
{
  for(n=1,31,
     count=0;
     read(Str("/tmp/trees-"n".gp"));
     print1(count,", ");
  );
  quit;
}

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += (vpar_matchnum(vpar)==2));
     print1(count,", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Mowshowitz y,v solutions

\\ y^2 - v^2 + (n-1)*v - (n-2)*y - (n-2) = 0
\\ u+v=n-1  x+y=n-2
\\
\\ All are T1,T2 forms, for some u,v,x,y, probably ...
\\ in which case all cospectral pairs are solutions to the quadratic form.

make_bistar_split_middle(a,b) =
{
  my(vpar=vpar_make_bistar(a,b));
  vpar=Vec(vpar,#vpar+1);         \\ extend to new vertex
  vpar[1] = vpar[a+1] = #vpar;    \\ it as parent of the star centres
  vpar;
}
T1_uv(u,v) = vpar_make_bistar(u+1,v+1);
T2_xy(x,y) = make_bistar_split_middle(x+1,y+1);

{
  for(n=1,26,     \\ num edges
     print();
     for(v=1,n,      \\ v smaller half
        my(u=n-1-v);
        u >= 1 || next();
        u >= v || next();
        my(poly = y^2 - v^2 + (n-1)*v - (n-2)*y - (n-2),
           d=poldisc(poly));
        d>=0 || next();
        issquare(d) || next();
        my(roots=apply(round,polroots(poly)));
        print("n="n" v="v"  "poly"  disc="d"  "roots);
        my(prev='none);
        for(i=1,#roots,
           my(y=roots[i],
              x=n-2-y);
           y>=1 || next;
           x>=1 || next;
           my(vpar1 = T1_uv(u,v),
              vpar2 = T2_xy(x,y));
           if(vpar_is_free_isomorphic(vpar1,vpar2), next());
           if(prev!='none,
              if(vpar_is_free_isomorphic(vpar1,prev), next()));
           print("  y="y);
           print("  vpar1 "vpar1"  "vpar_adjacency_poly(vpar1));
           print("  vpar2 "vpar2"  "vpar_adjacency_poly(vpar2));
           prev = vpar1;
           );
     ));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Cospectral Trees
\\
\\ First pair n=8 from Mowshowitz.

\\ free trees matchnum=2 number of sets of cospectrals
\\ n = 1           5             10                            20             25             30
\\     0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 2, 0, 1, 1, 0, 1,
\\ not in OEIS:             1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 2, 0, 1, 1, 0, 1,

\\ n=26 two pairs
\\ matchnum=2  List([
\\ [0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,19,19,19,19,19,19,19],
\\    bistar 8,17 split middle
\\ [0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1]
\\    bistar 11,15
\\ matchnum=2  List([
\\ [0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,18,18,18,18,18,18,18,18]
\\    bistar 9,16 split middle
\\ [0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1]
\\    bistar 12,14

try(vpar) =
{
  vpar_matchnum(vpar) ==2 || return();

  my(poly=vpar_adjacency_poly(vpar),
     l);
  mapisdefined(m,poly,&l) || (l=List([]));
  listput(l,vpar);
  mapput(m,poly,l);
  0;
}
{
  for(n=26,31,
     m=Map();
     read(Str("/tmp/trees-"n".gp"));
     m=Mat(m);
     my(vpars_list=if(matsize(m)[1]==0,[],m[,2]));

     print("n="n);
     for(i=1,#vpars_list,
        my(vpars=vpars_list[i]);
        if(#vpars >= 2,
           print("matchnum="vpar_matchnum(vpars[1])"  "vpars)));

     \\ print1(#select(l->#l>=2, vpars_list),", ");
  );
  quit;
}

{
  for(n=1,20,
     \\ print("n="n);
     my(vpar=vpar_free_first(n),
        m=Map());
     until((vpar=vpar_free_next(vpar))=='none,
           vpar_matchnum(vpar) ==2 || next();

           my(poly=vpar_adjacency_poly(vpar),
              l);
           mapisdefined(m,poly,&l) || (l=List([]));
           listput(l,vpar);
           mapput(m,poly,l));

     m=Mat(m);
     my(vpars_list=if(matsize(m)[1]==0,[],m[,2]));

     \\ for(i=1,#vpars_list,
     \\    my(vpars=vpars_list[i]);
     \\    if(#vpars >= 2,
     \\       print("matchnum="vpar_matchnum(vpars[1])"  "vpars)));

     print1(#select(l->#l>=2, vpars_list),", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_num_rootings_max()

for(n=3,9,print("n="n"  "vpar_num_rootings_max(n)))
quit


\\------------------------------------------------------------------------------
\\ vpar_is_free_asymmetric() free forests
\\
\\ trees
\\ vector(15,n,n--; vpar_count_free_asymmetric(n))
\\ 1, 1, 0, 0, 0, 0, 0, 1, 1, 3, 6, 15, 29, 67, 139
\\
\\ forests
\\ 1, 0, 0, 0, 0, 0, 1, 2, 4, 9, 21, 44, 96, 206,
\\ A035056

count_free_asymmetric_vector(n) =
{
  my(a=vector(15,n,n--; vpar_count_free_asymmetric(n)));
  my(b=vector(n,i, 1));
  for(n=2,n,
     b[n] = -sum(k=1,n-1, sumdiv(k,d, (-1)^(k/d)*d*a[d]) * b[n-k]) / (n-1));
  b;
}
count_free_asymmetric_vector(15)
vpar_INTERNAL_weigh_transform(vector(15,n, vpar_count_free_asymmetric(n)))

\\ hexagonal(n)=n*(2*n-1);
\\ vpar_INTERNAL_weigh_transform(vector(15,n, hexagonal(n)))

{
  my(forest=1);
  for(n=1,20,
     my(vpar=vpar_unlabelled_first(n),
        l=List([]));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           vpar_is_free_asymmetric(vpar) || next();
           for(i=1,#l,
              if(vpar_is_free_isomorphic(vpar,l[i]), next(2)));
           listput(l,vpar));
     print1(#l,", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_domnum() == vpar_totdomnum()
\\
\\ which implies also
\\ vpar_domnum() == vpar_semitotdomnum() == vpar_totdomnum()
\\
\\ free domnum == matchnum
\\          n = 1  2  3  4  5  6   7
\\              0, 0, 0, 1, 1, 3, 4, 9, 15, 32, 62, 130, 267, 575, 1235,
\\ not in OEIS:                3, 4, 9, 15, 32, 62, 130, 267, 575, 1235,
\\
\\ n=6
\\ [0, 1, 2, 2, 1, 1]  bistar 3,3
\\ [0, 1, 2, 1, 4, 1]  starlike 2,2,1
\\ [0, 1, 2, 1, 1, 1]  bistar 4,2 = starlike 2,1,1,1

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
         count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(totdomnum=vpar_totdomnum(vpar),
              domnum=vpar_domnum(vpar),
             eq=(domnum==totdomnum));
           count += eq;
           if(eq && n<=6, print(vpar));
          );
     print1(count", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ num rooted or free isomorphic

\\ [0, 3, 1, 1, 2]  300  rooted=120 rat 5/2
\\        1
\\       / \         is_free_isomorphic to 300 any
\\      3   4        num_free_isomorphic = 120 unrooted
\\     /             num_free_automorphisms = 2
\\    2                swap 2,1, 3, 5,4
\\   /
\\  5
\\ GP-Test  5*5!/2 == 300
\\ GP-Test  5! == 120


\\ [0, 1, 1, 2]
\\      1            is_free_isomorphic to 48 any
\\     / \           num_free_isomorphic = 24 unrooted
\\    2   3          num_free_automorphisms = 2
\\   /                    swap 2,1,4,3, changes root
\\  4
\\
\\ path 4!, one end /2, and 4* roots
\\ GP-Test  4 * 4!/2 == 48

\\ num_free_isomorphic = how many root=1
\\

num_rerootings_upto_isomorphism(vpar) =
{
  my(l=List([]));
  for(i=1,vpar_num_rootings(vpar),
     my(vpar_rerooted=vpar_reroot_ith(vpar,i));
     for(j=1,#l,
        if(vpar_is_isomorphic(vpar_rerooted,l[j]),
           next(2)));
     listinsert(l,vpar_rerooted, setsearch(l,vpar_rerooted,1)));
  #l;
}
{
  my(forest=0);
  for(n=1,6,
     print("n="n);
     my(keys=List([]),
        lists=List([]));
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           for(i=1,#keys,
              if(vpar_is_free_isomorphic(vpar,keys[i]),
                 listput(lists[i], vpar);
                 next(2)));
           listput(keys, vpar);
           listput(lists, List([vpar]));
     );
     for(i=1,#keys,
        my(vpar=keys[i],
           list=lists[i],
           num_found=#list,
           num_isomorphic=vpar_num_free_isomorphic(vpar),
           num_rootings=vpar_num_rootings(vpar),
           num_rootings_upto=num_rootings_upto_isomorphism(vpar),
           orbit_sizes=vpar_free_orbit_sizes(vpar),
           prod1 = num_isomorphic * num_rootings,
           prod2 = num_isomorphic * num_rootings_upto);
        print(vpar"  found="num_found
              "  num_iso="num_isomorphic
              "  num_auto="vpar_num_free_automorphisms(vpar)
              "  num_rootings="num_rootings
              "  prod "prod1,if(prod1==num_found,""," *** "));
        print("  orbit sizes "orbit_sizes,
              "  num_rootings_upto_iso="num_rootings_upto
              "  prod "prod2,if(prod2==num_found,""," *** "));

        list=vecsort(list);
        if(vpar==[0, 1, 1, 2],
           for(j=1,#list,
              print("  "list[j])));

        );
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ labelled rooted trees with root degree >= 3
\\
\\ all trees
\\ 4, 65, 1026, 17857, 349224,
\\ A331577 labelled > 2 branches at root
\\
\\ root=1
\\ not in OEIS: 1, 13, 171, 2551, 43653,
\\ *4
\\ not in OEIS: 4, 52, 684, 10204, 174612

{
  my(forest=0);
  for(n=4,10,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              root=vpar_root(vpar));
           if(vpar_degree_of_vertex(vpar,root)>=3,
              count++));
     print1(count", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_rootings() total
\\

\\ trees
\\ n = 1  2  3  4
\\     1, 4, 27, 256, 3125, 46656, 823543,
\\ n^n
\\ n * vpar_count(n)
\\
\\ forests
\\ n = 1  2  3  4
\\     1, 5, 40, 437, 6036, 100657, 1965160,
\\ A202477 ways to build endofunctions on partitions into blocks
\\ egf exp(T(x)/(1-T(x)))
\\ exponential transform of n^n
\\ vpar_INTERNAL_exponential_transform(vector(7,n,n^n))

{
  my(forest=1);
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           total += vpar_num_rootings(vpar));
     print1(total,", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_free_automorphism_generators()
\\
\\ maximum number of generators (minimum set of generators)
\\ 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9,

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        m=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(num=vpar_num_free_automorphism_generators(vpar));
           m=max(m,num));
     print1(m,", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_automorphisms() which occur

\\ n=7 unlabelled trees distinct num automorphisms occurring
\\ not in OEIS: 1, 2, 4, 6, 8, 12, 24, 120, 720
\\
\\ n=10 unlabelled trees how many of each num automorphisms
\\ not in OEIS: 1, 2, 3, 5, 6, 8, 9, 14, 18, 23, 38, 64, 83, 98, 113, 228

{
  my(forest=1);
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        m=Map());
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(num=vpar_num_automorphisms(vpar),
              count=0);
           mapisdefined(m,num,&count);
           count++;
           mapput(m,num,count));
     m=Mat(m);
     my(nums=m[,1],
        counts=m[,2]);

     print("n="n"  "Set(nums));

     \\ print("n="n"  "Set(counts)"  "vecsum(counts));
     vecsum(counts) == vpar_count_unlabelled(n,forest)
       || error("want total "vpar_count_unlabelled(n,forest));

     \\ print1(#counts", ");
     );
  quit;
}

{
  my(forest=0);
  for(n=4,4,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           print(vpar
                 "  num iso "vpar_num_ordered_isomorphic(vpar)
                 "  num auto "vpar_num_ordered_automorphisms(vpar)
                 "  cf plain "vpar_num_isomorphic(vpar)
                 " auto "(vpar));
           my(count=0);
           print("  "vpar_num_automorphisms(test_ordered_to_unlabelled(vpar)));
     ));
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_perm_is_ordered() count

{
  my(forest=0);
  for(n=5,5,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           print(vpar
                 "  num iso "vpar_num_ordered_isomorphic(vpar)
                 "  num auto "vpar_num_ordered_automorphisms(vpar));
           my(count=0);
           for(i=0,n!-1,
              my(perm=numtoperm(n,i));
              if(vpar_perm_is_ordered(vpar,perm),
                 \\ print("  "perm" to "vpar_relabelled);
                 count++));
           print("  "count,
                if(count!=vpar_num_ordered_isomorphic(vpar),
                  "  ******", "")));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_ordered_automorphism()

{
  my(forest=0);
  for(n=4,4,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           print(vpar
                 "  num iso "vpar_num_ordered_isomorphic(vpar)
                 "  num auto "vpar_num_ordered_automorphisms(vpar)
                 "  cf plain "vpar_num_isomorphic(vpar)
                 " auto "vpar_num_automorphisms(vpar));
           my(count=0);
           print("  "vpar_num_automorphisms(test_ordered_to_unlabelled(vpar)));
     ));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_automorphism() vs isomorphic
\\ by definition relabelling by all perms gives isomorphic

{
  my(forest=0);
  for(n=4,4,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           print(vpar
                 "  num iso "vpar_num_isomorphic(vpar)
                 "  num auto "vpar_num_automorphisms(vpar));
           my(count=0);
           for(i=0,n!-1,
              my(perm=numtoperm(n,i),
                 vpar_relabelled=vpar_relabel_perm(vpar,perm));
              if(vpar_is_isomorphic(vpar,vpar_relabelled),
                 print("  "perm" to "vpar_relabelled);
                 count++));
           print("  "count"  cf plain "vpar_num_automorphisms(vpar)));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_ordered_isomorphic()

{
  my(forest=0);
  for(n=5,5,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           print(vpar"  num iso "vpar_num_ordered_isomorphic(vpar));
           my(count=0);
           for(i=0,n!-1,
              my(perm=numtoperm(n,i),
                 vpar_relabelled=vpar_relabel_perm(vpar,perm));
              if(vpar_is_ordered_isomorphic(vpar,vpar_relabelled),
                 print("  "perm" to "vpar_relabelled);
                 count++));
           print("  "count"  cf plain "vpar_num_automorphisms(vpar)));
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_adjacency_poly() and vpar_Laplacian_poly()

{matdet([1,2,3;
         4,5,6;
         7,8,6])}
{1*matdet([5,6;
           8,6])}
{-2*matdet([4,6;
           7,6])}
{3*matdet([4,5;
           7,8])}
{matdet([-x,1,1;
         1,-x,1;
         1,1,-x])}^2
j={matdet([-x,1,1,0,0,0;
         1,-x,1,0,0,0;
         1,1,-x,0,0,0;
         0,0,0,-x,1,1;
         0,0,0,1,-x,1;
         0,0,0,1,1,-x])}
m={matdet([-x,1,1,0,1,0;
         1,-x,1,0,0,0;
         1,1,-x,0,0,0;
         0,0,0,-x,1,1;
         1,0,0,1,-x,1;
         0,0,0,1,1,-x])}
p={matdet([-x,1;
         1,-x])}^2
j-p

print();
j={matdet([3-x,1,1,0,0,0;
         1,3-x,1,0,0,0;
         1,1,3-x,0,0,0;
         0,0,0,3-x,1,1;
         0,0,0,1,3-x,1;
         0,0,0,1,1,3-x])}
m={matdet([3-x,1,1,0,1,0;
           1,3-x,1,0,0,0;
           1,1,3-x,0,0,0;
           0,0,0,3-x,1,1;
           1,0,0,1,3-x,1;
           0,0,0,1,1,3-x])}
p={matdet([3-x,1;
           1,3-x])}^2
m-j
j-p

print()
p={matdet(Mat([0-x]))}

vpar=[3,3,10,10,9,8,8,9,10, 0];
vpar_Laplacian_poly(vpar)
\\ poly(vpar)


quit


\\-----------------------------------------------------------------------------
\\ vpar_downwards_ith() root above forest

{
  my(forest=1);
  for(n=4,4,
     my(forest_perm=vector(n+1,i, (i%(n+1))+1));
     print(forest_perm);
     for(i=1,n!,
        my(vpar=vpar_downwards_ith(n,i,forest),
           add=vpar_add_root_above_forest(vpar));
        print(vpar"  "add" "vpar_relabel_perm(add,forest_perm));
     );
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_is_downwards() and vpar_downwards_ith()

{
  my(forest=0);
  for(n=0,5,
     my(l=vector(n!,i,i--;
                 my(perm=numtoperm(n,i));
                 vpar_from_perm_noup(perm)
                ));
     if(!forest, l=select(vpar_is_tree,l));

     my(l_depths=apply(vpar_depths_vector,l),
        keys=vector(#l,i, [-l[i], -l_depths[i] ]),
        perm=vecsort(keys,,1));
     l=vecextract(l,perm);
     l_depths=apply(vpar_depths_vector,l);

     print("vpar issorted "test_vector_is_decreasing(l));
     print("depths issorted "test_vector_is_decreasing(l_depths));

     for(i=1,#l,
        my(vpar=l[i]);
        print(vpar"  "vpar_depths_vector(vpar)"     "vpar_downwards_ith(n,i));
     );

     \\ for(i=1,#l,
     \\    my(vpar=);
     \\    print(vpar"  "vpar_depths_vector(vpar));
     \\ );
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ preorder and downwards
\\
\\ 1, 1, 1, 2, 5, 14, 42, 132, 429, 1430, 4862, 16796, 58786,
\\ A120588

{
  my(forest=0);
  for(n=0,12,
     my(count=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           count += vpar_is_downwards(vpar);
           \\ if(eq && n==3, print(vpar));
          );
     print1(count", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ trees with num automorphism ids == height+1 minimum
\\ so subtree structure determined by height, but various shorter siblings too

\\ ordered trees
\\              1, 1, 1, 2, 5, 14, 40, 118, 352, 1066, 3247, 9967,
\\ not in OEIS:          2, 5, 14, 40, 118, 352, 1066, 3247, 9967,

\\ ordered forests
\\ 1, 1, 2, 5, 14, 40, 118, 352, 1066, 3247, 9967,
\\ same one earlier

{
  my(forest=1);
  for(n=0,12,
     my(count=0,
        vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,forest))=='none,
           my(height=vpar_height(vpar),
              num_ids=vpar_num_automorphism_ids(vpar),
              eq = (height+1 == num_ids));
           count += eq;
           \\ if(eq && n==3, print(vpar));
          );
     print1(count", ");
     );
  quit;
}
quit


\\ h made by tree h-1 of k vertices, rest n-k-1 other children partition
\\ into <= h-1 heights from this tree -- but how big are each?
TH(n,h) =
{
  \\ print("TH "n","h);
  n>=0 || error();
  h>=-1 || error();
  n>h || error();
  if(n==0, h==-1,
     h==-1, n==0,
     h==0, n==1,
     n==h-1, 1,
     1 + sum(k=h,n-2, TH(k,h-1))); \\ * numbpart(n-1-k)));
}
TH=memoize(TH);
T(n) =
{
  sum(h=0,n-1, TH(n,h));
}
vector(12,n, T(n))
TH(3,-1)
TH(3,0)
TH(3,1)
T(3)
quit
\\ n=3
\\ [0, 1, 2]  path-3
\\ [0, 1, 1]  star-3

\\ unlabelled trees
\\          n = 0  1  2  3  4
\\              1, 1, 1, 2, 4, 9, 19, 43, 94, 211, 470, 1058, 2374,
\\ not in OEIS:          2, 4, 9, 19, 43, 94, 211, 470, 1058, 2374,

\\ unlabelled forests
\\              1, 1, 2, 4, 9, 19, 43, 94, 211, 470, 1058, 2374, 5364,
\\ not in OEIS:       2, 4, 9, 19, 43, 94, 211, 470, 1058, 2374, 5364,

{
  my(forest=0);
  for(n=0,12,
     my(count=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(height=vpar_height(vpar),
              num_ids=vpar_num_automorphism_ids(vpar),
              eq=(height+1 == num_ids));
           count += eq;
           if(eq && n==3, print(vpar));
          );
     print1(count", ");
     );
  quit;
}

\\ labelled trees
\\              1, 1, 2, 9, 64, 625, 7416, 106729,
\\ not in OEIS:       2, 9, 64, 625, 7416, 106729,

\\ labelled forests
\\              1, 1, 3, 16, 125, 1236, 15247,
\\ not in OEIS:       3, 16, 125, 1236, 15247,

{
  my(forest=1);
  for(n=0,9,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              height=vpar_height(vpar),
              num_ids=vpar_num_automorphism_ids(vpar));
           count += (height+1 == num_ids));
     print1(count", ");
     );
}
quit


\\-----------------------------------------------------------------------------
\\ vpar_matchpoly() high 1
\\ is perfect matching trees
\\ any less has at least 2 ways to arrange the unmatched

{
  forstep(n=2,20,2,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(poly=vpar_matchpoly(vpar));
           if(pollead(poly)==1,
              count++));
     print1(count,", ");
     );
  quit;
}

\\-----------------------------------------------------------------------------
\\ vpar_has_perfect_matching() near-prefect

\\ h+1 children under h parents >= 1 and solitary >= 0
\\ h+2 under h+1 places >= 1
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h)*h!*stirling(h+2,h+1,2) * n^(h-1))
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h)*h!*stirling(h+2,h+1,2) * (n+1)^h)

\\ Return a vector of sets which are all size n subsets from "set".
make_subsets(set,n) =
{
  my(ret=List([]));
  forvec(v=vector(#set,i,[0,1]),
        if(vecsum(v)==n,
           listput(ret, Set(vecextract(set, test_flags_to_Set(v))))));
  vecsort(Vec(ret),cmp);
}
check_equal(make_subsets([x,y,z],2), [ [x,y], [x,z], [y,z] ]);

set_permutations(set) =
{
  vecsort(vector((#set)!,i, vecextract(set,numtoperm(#set,i))), cmp);
}
check_equal(set_permutations([x,y,z]), [ [x,y,z],[x,z,y], [y,x,z],[y,z,x], [z,x,y],[z,y,x] ]);

make_all(n) =
{
  my(h = n>>1,
     all=[1..n]);
  my(parents_list=make_subsets(all,h));
  for(i=1,#parents_list,
     my(parents=parents_list[i],
        children=setminus(all,parents));
  );
  parents_list;
}
make_all(3)

{
  my(forest=1);
  for(n=3,3,
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           if(vpar_has_perfect_matching_near(vpar,1),
              print("vpar="vpar);
              );
     ));
  quit;
}

\\ my(n=3,h=1); binomial(n,h) == 3
\\ my(n=3,h=1); h!            == 1
\\ my(n=3,h=1); stirling(h+1,h,2) == 1
\\ my(n=3,h=1); stirling(h+1,h,2) + h+1 == 3
\\ my(n=3,h=1); (h+2)*(h+1)/2 == 3
\\ my(n=3,h=1); (h+4)*(h+1)/2 == 5
\\ my(n=3,h=1); n^(h-1)       == 1
\\ my(n=3,h=1); (n+1)^(h-1)   == 1
\\
\\ my(n=3,h=1); n^h           == 3
\\ my(n=3,h=1); (n+1)^h       == 4
\\
\\ n=3 forests
\\ vpar=[0, 0, 1]     1  2
\\                    |
\\                    3
\\ vpar=[0, 0, 2]    1  2
\\                      |
\\                      3
\\ vpar=[0, 3, 0]   1  3
\\                     |
\\                     2
\\ vpar=[0, 1, 0]     1  3
\\                    |
\\                    2
\\ vpar=[3, 0, 0]   2  3
\\                     |
\\                     1
\\ vpar=[2, 0, 0]    2  3
\\                   |
\\                   1
\\ n=3 trees
\\ vpar=[0, 1, 1]      1
\\                    / \
\\                   2   3
\\ vpar=[2, 0, 1]
\\ vpar=[3, 1, 0]
\\ vpar=[0, 1, 2]
\\ vpar=[2, 0, 2]
\\ vpar=[2, 3, 0]
\\ vpar=[0, 3, 1]
\\ vpar=[3, 0, 2]
\\ vpar=[3, 3, 0]



\\------------------------------------------------------------------------------
\\ vpar_semitotdomnum() == vpar_matchnum()
\\
\\ Alister Justin Marcon, thesis "Semitotal Domination in Graphs".

\\ semitotdomnum == matchnum
\\          n = 1  2  3  4  5  6   7
\\              0, 0, 0, 1, 2, 5, 10, 22, 46, 99, 219, 484, 1114, 2560, 6025, 14285,
\\ not in OEIS:             2, 5, 10, 22, 46, 99, 219, 484, 1114, 2560, 6025, 14285,
\\
\\ n=4 [0, 1, 2, 1]  path-4
\\ n=5 [0, 1, 2, 1, 4]  path-5
\\     [0, 1, 2, 1, 1]
\\      *--*--*--*
\\         |
\\         *
\\
\\ totdomnum == matchnum
\\          n = 1  2  3  4  5  6   7
\\              0, 0, 0, 1, 1, 3, 4, 11, 19, 44, 88, 205, 454, 1074, 2489, 5991,
\\ not in OEIS:                3, 4, 11, 19, 44, 88, 205, 454, 1074, 2489, 5991,
\\
\\ domnum == matchnum
\\          n = 1  2  3  4  5  6   7
\\              0, 1, 1, 2, 3, 5, 10, 18, 36, 73, 153, 325, 711, 1573, 3540,
\\ not in OEIS:          2, 3, 5, 10, 18, 36, 73, 153, 325, 711, 1573, 3540,
\\

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
         count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(semitotdomnum=vpar_domnum(vpar),
              matchnum=vpar_matchnum(vpar),
             eq=(semitotdomnum==matchnum));
           count += eq;
           if(0 && eq && n<=5, print(vpar));
          );
     print1(count", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Brouwer
\\ n=31
\\ Figure 1

depths=[0,1,2, 3,3,3, 1,2,3,2,3,2,3, 1,2,3,2,3,2,3, 1,2,2,2,2, 1,2,2,2,2, 1]
vpar=vpar_make_preorder_depths(depths)
#vpar
factor(vpar_adjacency_poly(vpar))
print(vpar_to_sparse6(vpar))
quit


\\------------------------------------------------------------------------------
\\ vector intermediates in loop

foo(n) =
{
  my(work=vector(n));
  for(i=1,#work,
     work[i]=x^10000+1;
     if(i>1, work[i-1]=0);
     print(sizebyte(work)"  "getstack());
     );
}
foo(50);
quit

\\------------------------------------------------------------------------------
\\ vpar_adjacency_poly() from matchpoly

vpar=vpar_make_path(10)
m=vpar_adjacency_matrix(vpar)
charpoly(m)
m-=matid(#vpar)*x
matdet(m)
print("dets");
total=0;
for(i=1,#vpar, my(t=(-1)^(#vpar-i)*m[1,i]*matdet(m[^1,^i])); print(t); total+=t);
print("total "total);
print("nine "charpoly(vpar_adjacency_matrix(vpar_make_path(9))));
print("indpoly   "vpar_indpoly(vpar));
print("dompoly   "vpar_dompoly(vpar));
print("matchpoly "vpar_matchpoly(vpar));
print("matchpoly neg "vpar_matchpoly(vpar,-'x));
print("Laplacian "vpar_Laplacian_poly(vpar));
quit


\\------------------------------------------------------------------------------
\\ free trees determined by spectrum

\\ uniques
\\ not in OEIS: 1, 1, 1, 2, 3, 6, 11, 21, 37, 98, 175, 432, 886, 2333, 5271, 14074,
\\ frac around .7 of all
\\ n=13  0.6810146041506533435818601076
\\ n=14  0.7385248496359607470718581830
\\ n=15  0.6809197778064849502648236662
\\ n=16  0.7284679089026915113871635611
\\ n=17  0.6926936601616319480145592136
\\ n=18  0.7388569998466096700493271009
\\ n=19  0.7354405497633312890188863204
\\
\\ dups
\\              0, 0, 0, 0, 0, 0, 0, 1, 5, 4, 29, 56, 192, 390, 1132, 2405,
\\ not in OEIS:                      1, 5, 4, 29, 56, 192, 390, 1132, 2405,
\\
\\ pairs
\\              0, 0, 0, 0, 0, 0, 0, 1, 5, 4, 27, 49, 162, 349, 960, 2028,
\\ not in OEIS:                      1, 5, 4, 27, 49, 162, 349, 960, 2028,
\\
\\ triples
\\          n = 1  2  3  4  5  6  7  8  9 10 11
\\              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 7, 29, 36, 145, 326,
\\ not in OEIS:                               2, 7, 29, 36, 145, 326,

\\ n=8 cospectral
\\ Brouwer and Haemers, "Spectra of Graphs", page 18, exercise 7
\\ 98.
\\ L. Collatz & U. Sinogowitz, Spektren Endlicher Grafen,
\\ Abh. Math. Sem. Univ. Hamburg 21 (1957) 63-77. (p. 18)
\\
\\ [0, 1, 2, 2, 2, 2, 1, 7]
\\    *---*---*---*
\\  /| |\              star-6 one leg elongated to path-3
\\ * * * *             star 3,1,1,1,1
\\
\\ [0, 1, 2, 2, 2, 1, 1, 1]   bi-star 4,4
\\ x^8 - 7*x^6 + 9*x^4 == x^4*(x^2 + x - 3)*(x^2 - x - 3)

\\ n=11 triples cospectral
\\ 3 of x^11 - 10*x^9 + 32*x^7 - 36*x^5 + 10*x^3
\\ [0, 1, 2, 3, 4, 4, 1, 7, 8, 1, 1]
\\   *---*---*---*---*---*---*
\\  / \         / \
\\ *   *       *   *
\\ [0, 1, 2, 3, 3, 3, 1, 7, 8, 1, 10]
\\   *---*---*---*---*---*
\\  /|\      |
\\ * * *     *
\\           |
\\           *
\\ [0, 1, 2, 3, 3, 2, 1, 7, 8, 8, 1]
\\   *---*---*---*---*---*
\\  /|   |   |           |\
\\ * *   *   *           * *
\\
\\ 3 of x^11 - 10*x^9 + 31*x^7 - 34*x^5 + 10*x^3
\\ [0, 1, 2, 3, 3, 3, 2, 7, 1, 9, 1]
\\ [0, 1, 2, 3, 3, 2, 6, 2, 1, 9, 9]
\\ [0, 1, 2, 3, 3, 2, 1, 7, 8, 1, 1]

try(vpar) =
{
  my(p=charpoly(vpar_adjacency_matrix(vpar)),
     count=0);
  mapisdefined(m,p,&count);
  count++;
  mapput(m,p,count);

  if(p==x^11 - 10*x^9 + 31*x^7 - 34*x^5 + 10*x^3,print(vpar));
  0;
}
{
  for(n=1,11,
     m=Map();
     read(Str("/tmp/trees-"n".gp"));
     m=Mat(m);
     \\ print(m);
     \\ for(i=1,matsize(m)[1],
     \\    if(m[i,2]>1,
     \\       print(m[i,2]" of "m[i,1])));

     my(counts=m[,2],
        unique=sum(i=1,#counts, counts[i]==1),
        dups=sum(i=1,#counts, counts[i]!=1),
        pairs=sum(i=1,#counts, counts[i]==2),
        triples=sum(i=1,#counts, counts[i]==3));
     \\ print1(unique,", ");
     print1(triples,", ");
     \\ print("n="n"  "unique*1.0/vpar_count_free(n));

     if(1 && n==11,
        for(i=1,matsize(m)[1],
           if(m[i,2]>2,
              print(m[i,2]" of "m[i,1]))));
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ gentreeg same free representatives as vpar_free_next(), but different order

n=10;
try(vpar) = listput(gen,vpar); 0;
{
  for(n=1,16,
     gen = List([]);
     read(Str("/tmp/trees-"n".gp"));
     gen=Vec(gen);
     it = vpar_free_all(n);
     check_equal(it!=gen, n>=5);
     it = vecsort(it);
     gen = vecsort(it);
     check_equal(it,gen);
  );
}
quit


\\------------------------------------------------------------------------------
\\ near perfect matching failing at root
\\
\\ trees
\\ n = 1  3    5      7
\\     1, 6, 300, 41160,
\\ A277168 series reversion of x*exp(-x^2)
\\ factor(41160)
\\ 41160 = 8*3*5*7^3

\\       U
\\     / | \
\\    r  s  t
\\
\\ >= 2 of r,s,t near at root
\\ any even subtrees perfect

\\      p
\\      |
\\      U
\\    /   \
\\   r     s
\\   |     |
\\  perf  perf
\\
\\ perfect forest below root
\\ B(n) = n*vpar_count_perfect_matching(n-1,,1)
\\ vector(10,n,n--; B(n))
\\ vector(6,n,n--; B(2*n+1))
\\ 1, 6, 300, 41160, 11022480, 4870182240
\\ A277168 series reversion of x*exp(-x^2)
\\
\\ FB(n) = if(n<0,error(), n<=1,1, sum(t=1,n, binomial(n-1,t-1) * B(t) * FB(n-t)));
\\ vector(10,n,n--; FB(n))
\\ FBge2(n) = FB(n) - B(n);
\\ vector(10,n,n--; FBge2(n))
\\ FBge2(2) == 1
\\ T(n) = if(n%2==0,0, sum(t=0,n, (n-t) * binomial(n,t) * FBge2(t) * vpar_count_perfect_matching(n-t-1)));
\\ vector(10,n,n--; T(n))
\\ vector(5,n,n--; T(2*n+1))
\\ T(3)
\\ my(n=3); for(t=0,n, print("t="t" rest="n-t"  "binomial(n,t)" "FBge2(t)" "vpar_count_perfect_matching(n-t-1)))

\\ NT(m) = if(m%2==0,0,m==1,1, my(n=m-1,h=n\2); m * (h+1) * h!*binomial(n,h) * n^(h-1))
\\ vector(10,n,n--; NT(n))
\\ vector(5,n,n--; NT(2*n+1))

highest_unmatched_parent(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     matched  = vectorsmall(#vpar));
  for(i=1,#seq,
     my(v=seq[i]);
     if(!matched[v],
        my(p=vpar[v]);
        if(p && !matched[p],
           matched[p]=1,
           return(p))));
  0;
}

{
  my(forest=0);  \\ trees or forests
  forstep(n=1,9,2,
         my(count=0);
         forvec(vec=vpar_forvec_from(n,forest),
               my(vpar=vpar_from_upascend(vec));
               if(!vpar_has_perfect_matching_near(vpar,1), next());
               my(v=highest_unmatched_parent(vpar));
               count += (v==0);
         );
         print1(count", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\
\\      p
\\      | \
\\      U   c
\\    / | \
\\
\\ U root of near failing at root
\\ under p of perfect above
\\

highest_unmatched(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     matched  = vectorsmall(#vpar));
  for(i=1,#seq,
     my(v=seq[i], p);
     if(!matched[v],
        if((p=vpar[v]) && !matched[p],
           matched[p]=1,
           return(v))));
  0;
}
highest_unmatched_parent(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     matched  = vectorsmall(#vpar));
  for(i=1,#seq,
     my(v=seq[i]);
     if(!matched[v],
        my(p=vpar[v]);
        if(p && !matched[p],
           matched[p]=1,
           return(p))));
  0;
}
subtree_vertices_below(vpar,v) =
{
  concat(apply(c->vpar_subtree_vertices(vpar,c),
              vpar_children_of_vertex(vpar,v)));
}

{
  my(forest=-1);  \\ trees or forests
  forstep(n=7,7,2,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           if(!vpar_has_perfect_matching_near(vpar,1), next());
           my(v=highest_unmatched_parent(vpar),
              s=subtree_vertices_below(vpar,v));
           s=Set(s);
           mapput(m,s,1);
           \\ if(vpar[v]==0,
           \\    print("root v="v" subtree size "#s));
     );
     m=Mat(m);
     print("total ",matsize(m)[1]);
     my(seen=m[,1]);
     print(seen);

     my(lens=apply(length,seen),
        hist=Vec(vpar_INTERNAL_histogram(lens)));
     print(hist);

     for(t=0,n\2,
        my(b=2*t,
           s=2*t+1);
        print("b="b" hist "hist[s]" choose "));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_has_near_perfect_matching()

\\ near perfect matching among labelled rooted
\\ trees, is n*root1
\\          n = 1  3    5       7
\\ not in OEIS: 1, 9, 600, 102900,
\\ vpar_count(3) == 9  \\ all trees n=3
\\
\\ forests
\\          n = 1   3     5       7
\\ not in OEIS: 1, 15, 1080, 188160,
\\ n=3
\\ vpar_count(3,1) == 16
\\ 3 singletons not near perfect
\\
\\ root=1
\\          n = 1  3    5      7        9
\\ not in OEIS: 1, 3, 120, 14700, 3674160,
\\
\\ GP-Test  vector(12,n,stirling(n+1,n,2)) == vector(12,n,(n+1)*n/2)
\\ stirling(10,9,2) == 45
\\ stirling(9,8,2) == 36
\\ stirling(8,7,2) == 28
\\ stirling(7,6,2) == 21
\\ stirling(6,5,2) == 15
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h) * (h+1)*h/2 * h! * n^(h-1)) + \
\\ vector(5,h,h--; my(n=2*h+1); n*vpar_count_perfect_matching(n-1,,1))
\\ vector(5,h,h--;my(n=2*h+1); (h+1) * h!*binomial(n,h) * (n+1)^(h+1))
\\ vector(6,h,my(n=2*h+1); n*h*vpar_count_perfect_matching(n-1))
\\ 15 / vpar_count_perfect_matching(2,,1) == 15/2
\\ 1080 / vpar_count_perfect_matching(4,,1) == 2*3^2
\\ 188160 / vpar_count_perfect_matching(6,,1) == 2^5
\\ 188160 == 2^6 * 5*4*3 * 7^2
\\ 3674160 / vpar_count_perfect_matching(8,,0)
\\ 3674160 / vpar_count_perfect_matching(8,,-1)
\\
\\        even
\\         |         U under parent of pair cannot push higher
\\         p         p with 2 children
\\      /  | \
\\     p   U  c
\\   /    / \
\\  c  even even
\\
\\ EE(n) = sum(f=0,n\2, binomial(n,2*f) * vpar_count_perfect_matching(2*f) * vpar_count_perfect_matching(n-2*f) );
\\ vector(10,n,n--; EE(n))
\\ vector(6,n,n--; EE(2*n))
\\
\\ 2*t above U
\\ n-2*t choice for U
\\ n-1-2*t below U forest
\\ N(n) = if(n%2, sum(t=0,n\2, binomial(n,2*t) \
\\                             * (n-2*t) \
\\                             * vpar_count_perfect_matching(2*t) \
\\                             * vpar_count_perfect_matching(n-1-2*t,,1)))
\\ N(3)
\\ vector(10,n,n--; N(n))
\\ vector(6,n,n--; N(2*n))

\\ trees 1, 9, 600, 102900,
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h) * (h+1)*h/2 * h! * n^(h-1)) + \
\\ vector(5,h,h--; my(n=2*h+1); n*vpar_count_perfect_matching(n-1,,1))
\\
\\ vector(5,h,h; my(n=2*h+1); binomial(n,h)* h! * (h+1)*h/2 * n^(h-1)) + \
\\ vector(5,h,h; my(n=2*h+1); binomial(n,h)* h! * (h+1) * n^(h-1))
\\
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h)* h! * (h+1)*(h+2)/2 * n^(h-1))
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h)* h! * stirling(h+2,h+1,2) * n^(h-1))
\\ vector(5,h,h--; my(n=2*h+1); vpar_count_perfect_matching(n,1))
\\
\\ h+1 children into h non-empty buckets for unmatch child under some parent
\\ or unmatch under h+1 children
\\ GP-Test  (h+1)*h/2 + h+1 == (h+2)*(h+1)/2

\\ forests 1, 15, 1080, 188160,
\\ vector(5,h,h;h--; my(n=2*h+1); binomial(n,h)* h! * (h+1)*h/2 * (n+1)^(h-1)) + \
\\ vector(5,h,h;h--; my(n=2*h+1); binomial(n,h)* h! * (h+1) * (n+2)^(h-1))
\\ my(n=7,h=3); 188160 / binomial(n,h) / (n+1)^(h-1) / h! == 14
\\ my(n=5,h=2); 1080 / binomial(n,h) / (n+1)^(h-1) / h! == 9
\\ my(n=3,h=1); 15 / binomial(n,h) / (n+1)^(h-1) / h! == 5
\\ vector(3,h, (h+4)*(h+1)/2)
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h)* h! * (h+4)*(h+1)/2 * (n+1)^(h-1))
\\ vector(5,h,h--; my(n=2*h+1); binomial(n,h)* h! * (stirling(h+2,h+1,2)+h+1) * (n+1)^(h-1))
\\
\\ h+1 children into h non-empty buckets for unmatch child under some parent
\\ or one of h+1 as root rest h into h non-empty buckets = 1
\\ or unmatch under on of h+1 children
\\ GP-Test  (h+1)*h/2 + h+1 + h+1 == (h+4)*(h+1)/2
\\
\\ Perfect tree pairs, tree of pairs, parent edge up to either.
\\
\\ Near perfect tree, pairs with one 2-children, or unmatched as root.
\\ Tree of h pairs but parent edge up to any.
\\ Forest of pairs under the separate unmatched,
\\
\\ forest:
\\ Forest of h pairs but parent edge up to any n+1.
\\ Forest of h+1 with separate would be (h+2)^h, but the separate is root so
\\ (h+2)^(h-1), but can go to either of pair so (n+1)^(h-1)


has_near_perfect_matching(vpar) = 2*vpar_matchnum(vpar)==#vpar-1;

{
  my(forest=-1);  \\ trees or forests
  forstep(n=1,10,2,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count += has_near_perfect_matching(vpar));
     print1(count", ");
     );
  quit;
}


\\----------------------------------------------------------------------------
\\ vpar_is_premax()

\\         1          id=6
\\       /   \
\\      2     6       id=3,4          id[2]=2 from 5
\\      |\    |                       then new id=3 from 3
\\      3 5   7       ids=2,1,2       id[6]=4 from 7
\\      |     |
\\      4     8       ids=1,1
\\
\\ [0, 1, 2, 3, 2, 1, 6, 7]  vpar
\\  1  2  3  4  5  6  7  8
\\ [6, 3, 2, 1, 1, 4, 2, 1]  ids
\\     *           *
\\ [0, 1, 2, 3, 2, 1, 2, 3]  depths

vpar_automorphism_ids_vector([0, 1, 2, 3, 2, 1, 6, 7])
vpar_depths_vector([0, 1, 2, 3, 2, 1, 6, 7])

\\         1
\\       /   \
\\      2     5
\\      |     |\
\\      3     6 8
\\      |     |
\\      4     7
\\
\\ [0, 1, 2, 3, 1, 5, 6, 5]   vpar
\\  1  2  3  4  5  6  7  8
\\ [0, 1, 2, 3, 1, 2, 3, 2]   depths

vpar_to_unlabelled([0, 1, 2, 3, 2, 1, 6, 7])
vpar_depths_vector([0, 1, 2, 3, 1, 5, 6, 5])

{
  for(n=8,8,
     my(un=Set(vpar_unlabelled_all(n)));

     my(vpar=vpar_preorder_first(n),
        count=0,
        l=List([]));
     until((vpar=vpar_preorder_next(vpar))=='none,
           my(ids=vpar_automorphism_ids_vector(vpar));

           my(good=1);
           for(v=1,#vpar,
              my(c=vpar_children_of_vertex(vpar,v));
              for(i=1,#c-1,
                 if(ids[c[i]]<ids[c[i+1]],
                    good=0; break(2))));
           count += good;

           print(good" "setsearch(un,vpar)"  "vpar);
           if(good, listput(l,vpar));
          );
     l=Set(l);
     print("n="n"  "count"  unlabelled "vpar_count_unlabelled(n)" ordered "vpar_count_ordered(n));

     my(missed=setminus(un,l));
     print(missed);
     for(i=1,#missed,
        my(vpar=missed[i],
           ids=vpar_automorphism_ids_vector(vpar),
           depths=vpar_depths_vector(vpar));
        print("vpar   "vpar);
        print("ids    "ids);
        print("depths "depths);
        my(good=1);
        for(v=1,#vpar,
           my(c=vpar_children_of_vertex(vpar,v));
           if(#c<2,next());
           for(i=1,#c-1,
              if(ids[c[i]]<ids[c[i+1]],
                 print(" p="v" children "c);
                 print(" ids          "vecextract(ids,c));
                 good=0)));
        );
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_root_of_vertex_vector() occurring

\\ trees
\\ 1, 2, 3, 4, 5, 6, 7,
\\ single root any of n

\\ forests
\\ 1, 3, 10, 41, 196, 1057, 6322, 41393,
\\ A000248 exp(x*exp(x))
\\   = count forests height <=1
\\   = sum(k=0,n, binomial(n,k)*(n-k)^k)
\\ vector(8,n, vpar_count_height_le(n,1,1))
\\ vector(8,n, sum(k=0,n, binomial(n,k)*(n-k)^k))
\\ vector(8,n, sum(k=0,n, binomial(n,k)*k^(n-k)))
\\ k roots, other n-k assigned under any of them

{
  my(forest=1);
  for(n=1,9,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              roots=vpar_root_of_vertex_vector(vpar));
           mapput(m,roots,1));
     print1(#m", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ total diameter

try(vpar) = total += vpar_diameter(vpar);
{
  for(n=8,13,
     total=0;
     read(Str("/tmp/trees-"n".gp"));
     print(total));
}
quit


\\------------------------------------------------------------------------------
\\ vpar_is_free_asymmetric() - free tree counts

\\ asymmetric free trees
\\ n = 0  1  2  3  4  5  6  7  8  9 10  11  12  13
\\     1, 1, 0, 0, 0, 0, 0, 1, 1, 3, 6, 15, 29, 67, 139, 310, 667, 1480, 3244, 7241,
\\ A000220

{
  for(n=0,20,
     my(vpar=vpar_free_first(n),
        num_vpar=0,
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(is_asymmetric=vpar_is_free_asymmetric(vpar));
           num_vpar++;
           count += is_asymmetric;
           if(0 && is_asymmetric && n==8, print(vpar));
          );
     num_vpar==vpar_count_free(n) || error();
     print1(count", ");
     );
  quit;
}

\\ asymmetric rooted unlabelled among free primary
\\          n = 0  1  2  3  4  5  6  7  8  9 10  11  12  13
\\              1, 1, 1, 0, 1, 0, 1, 1, 3, 3, 9, 15, 35, 67, 151, 310, 692, 1480, 3296,
\\ not in OEIS:                            3, 9, 15, 35, 67, 151, 310, 692, 1480, 3296,
\\
\\ vpar_count_free(8) == 23
\\
\\          1
\\         / \
\\        2   6      [0, 1, 2, 3, 4, 1, 6, 7]
\\       /     \
\\      3       7
\\     /         \
\\    4           8
\\   /
\\  5
check_equal(vpar_free_first(8), [0, 1, 2, 3, 4, 1, 6, 7]);
\\
\\          1
\\         / \
\\        2   6      [0, 1, 2, 3, 2, 1, 6, 7]
\\       / \   \
\\      3   5   7
\\     /         \
\\    4           8
\\
\\             1
\\          /  |  \
\\        2    6   8    [0, 1, 2, 3, 2, 1, 6, 1]
\\       / \   |
\\      3   5  7
\\     /
\\    4
\\


\\-----------------------------------------------------------------------------
\\ vpar_automorphism_ids_vector() occurring

\\ But ids in different order between different trees, so not really
\\ comparable.

\\ trees
\\ not in OEIS: 1, 2, 9, 52, 335, 2586,

\\ forests
\\ not in OEIS: 1, 3, 13, 67, 411, 2871,
\\ root=1 same

\\ trees collapsed range
\\ not in OEIS: 1, 2, 9, 40, 235, 1476,

\\ forests collapsed range
\\ 1, 3, 10, 47, 246, 1602, 11481,
\\ A005651 sum multinomial coeffs over partitions

vector_collapse_range(vec) =
{
  my(to=vectorsmall(vecmax(vec)));
  for(i=1,#vec, to[vec[i]]=1);
  my(upto=0);
  for(i=1,#to, if(to[i], to[i]=upto++));
  apply(x->to[x], vec);
}
check_equal(vector_collapse_range([4,3,1]), [3,2,1]);

{
  my(forest=1);
  for(n=1,9,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              ids=vpar_automorphism_ids_vector(vpar));
           ids=vector_collapse_range(ids);
           mapput(m,ids,1));
     print1(#m", ");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_is_asymmetric() - labelled rooted counts

\\ unlabelled asymmetric trees with n!
\\ my(v=[1, 1, 1, 2, 3, 6, 12, 25]); vector(#v,i,v[i]*i!)

\\ asymmetric labelled rooted trees
\\ n = 0  1  2  3
\\     1, 1, 2, 6, 48, 360, 4320,
\\ A228159

\\ asymmetric labelled rooted forests
\\ not in OEIS: 1, 1, 2, 12, 72, 720, 8640, 126000,
\\ vector(8,n,vpar_count_asymmetric(n,1))

\\ asymmetric labelled root=1 trees
\\ not in OEIS: 1, 1, 1, 2, 12, 72, 720, 8640,
\\ vector(8,n,vpar_count_asymmetric(n,-1))

\\ free asymmetric labelled rooted trees [FIXME: is this right?]
\\              0, 1, 2, 6, 52, 420, 5220, 78876
\\ not in OEIS:       2, 6, 52, 420, 5220, 78876

{
  my(forest=-1); \\ trees or forests
  for(n=0,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              is_asymmetric=vpar_is_asymmetric(vpar));
           count += is_asymmetric;
     );
     print1(count", "));
  quit;
}

{
  my(vpar=[0, 1, 2]);
  for(i=0,(#vpar)!-1,
     print(vpar_relabel_perm(vpar,numtoperm(#vpar,i))));
  quit;
}
{
  \\
  \\ [0, 1, 2]        1 <- 2 <- 3
  \\ [0, 3, 1]        1 <- 3 <- 2
  \\ [2, 0, 1]        2 <- 1 <- 3
  \\ [3, 0, 2]        2 <- 3 <- 1
  \\ [3, 1, 0]        3 <- 1 <- 2
  \\ [2, 3, 0]        3 <- 2 <- 1

  for(n=3,3,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              is_asymmetric=vpar_is_asymmetric(vpar));
           count += is_asymmetric;
           if(is_asymmetric,
              print(vpar));
     );
     print(count));
  quit;
}


\\----------------------------------------------------------------------------
\\ vpar_is_root_minmax() average how much to look at

\\ not in OEIS: 1, 5, 33, 291, 3289, 45759, 758795,

is_root_min_total_look(vpar) =
{
  my(seen=vectorsmall(#vpar));
  for(i=1,#vpar,
     my(v=i);
     seen[v]=1;
     while(vpar[v], seen[v]=1; v=vpar[v]);  \\ to v=root
     if(v>i, break()));
  hammingweight(seen);
}

{
  for(n=1,200,
     my(total=0,
       count=0);
     for(rep=1,2*n+100,
        my(vpar=vpar_make_random(n));
        total += is_root_min_total_look(vpar);
        count++);
     \\ print1(total", ");
     print(n"  "total*1.0/count);
  );
  quit;
}

{
  for(n=1,10,
     my(total=0,
       count=0);
     forvec(vec=vpar_forvec_from(n,1),  \\ forests
           my(vpar=vpar_from_upascend(vec));
           total += is_root_min_total_look(vpar);
           count++);
     \\ print1(total", ");
     print(total*1.0/count);
  );
  quit;
}


\\----------------------------------------------------------------------------
\\ vpar_free_next() info

free_primary_info(vpar) =
{
  my(L1=vpar_subtree(vpar,2),
     L2=vpar_delete_subtree(vpar,2),
     L1_height=vpar_height(L1),
     L2_height=vpar_height(L2));
  if(L1_height < L2_height,
     return(Str("heights "L1_height" < "L2_height)));
  if(L1_height > L2_height,
     return(Str("heights "L1_height" > "L2_height" -- not primary")));

  if(#L1 < #L2,
     return(Str("heights "L1_height"="L2_height" sizes "#L1" < "#L2)));
  if(#L1 > #L2,
     return(Str("heights "L1_height"="L2_height" sizes "#L1" > "#L2" -- not primary")));

  my(L1_depths=vpar_depths_vector(L1),
     L2_depths=vpar_depths_vector(L2));
  if(lex(L1_depths,L2_depths) < 0,
     return(Str("heights "L1_height"="L2_height" sizes "#L1"="#L2" lexdepths L1 < L2")));
  if(lex(L1_depths,L2_depths) > 0,
     return(Str("heights "L1_height"="L2_height" sizes "#L1"="#L2" lexdepths L1 > L2 -- not primary")));

  return(Str("heights "L1_height"="L2_height" sizes "#L1"="#L2" lexdepths L1 = L2"));
}

free_primary_L1_height(vpar) = \
  if(#vpar<2, 0, vpar_subtree_height(vpar,2));

free_primary_m(vpar) =
{
  for(v=3,#vpar, if(vpar[v]==1, return(v)));
  #vpar;
}

free_primary_L1_size(vpar) = \
  if(#vpar<=1,0, #vpar==2,1, free_primary_m(vpar) - 2);
free_primary_L2_size(vpar) = #vpar - free_primary_L1_size(vpar);

{
  for(n=12,12,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           print(vpar"  "free_primary_info(vpar));
           count++;
          );
     count==vpar_count_free(n) || error();
     print(count", ");
     );
  quit;
}

\\----------------------------------------------------------------------------
\\ gentreeg where path

try(vpar) =
{
  count++;
  if(vpar_is_free_isomorphic(vpar,path),
     print("i="count" "vpar"  depths "vpar_depths_vector(vpar)));
}
{
  for(n=8,13,
     count=0;
     path=vpar_make_path(n);
     read(Str("/tmp/trees-"n".gp"));
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_asymmetric() - unlabelled rooted counts

\\ asymmetric unlabelled rooted trees
\\ 1, 1, 1, 1, 2, 3, 6, 12, 25, 52, 113, 247, 548, 1226,
\\ A004111 rooted identity trees with n nodes

\\ asymmetric unlabelled rooted forests
\\ 1, 1, 1, 2, 3, 6, 12, 25, 52, 113, 247, 548, 1226,

vector(10,n,n--; vpar_count_asymmetric(n,,1))

{
  my(forest=1);
  for(n=0,20,
     my(vpar=vpar_unlabelled_first(n),
        num_vpar=0,
        count=0);
     until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
           my(is_asymmetric=vpar_is_asymmetric(vpar));
           num_vpar++;
           count += is_asymmetric;
          );
     num_vpar==vpar_count_unlabelled(n,forest) || error();
     print1(count", ");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_next_sibling_vector() occurring

\\ forests
\\ 1, 2, 5, 15, 52, 203, 877,
\\ A000110 Bell numbers
\\
\\ 1, 1, 4, 11, 41, 162, 715, 3425,
\\ A000296 set partitions without singletons

vpar_next_sibling_vectors_count(6)

vector(10,n, vpar_next_sibling_vectors_count(n))
print();
{
  my(forest=0); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_next_sibling_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_siblings_vector() occurring

\\ trees
\\ not in OEIS: 1, 1, 4, 11, 31, 132, 484,
\\
\\ forests
\\ 1, 2, 5, 13, 42, 150, 576, 2266,
\\ A178682 num elements mapped to m is divisible by m
\\
\\ trees root=1, same as forests 1 previous

F(n,k=n) =
{
  if(n<0, error());
  if(n<=1, 1,
     k==1, 1,
     F(n,k-1) + sum(m=1,n\k, binomial(n,m*k) * F(n-m*k,k-1)));
}
vector(10,n,F(n))

F(n,k=n) =
{
  if(n<0, error());
  if(n<=1, 1,
     k==1, 1,
     sum(m=0,n\k, binomial(n,m*k) * F(n-m*k,k-1)));
}
vector(10,n,F(n))

A(n) =
{
  if(n<0, error());
  if(n<=1, return(1));
  my(a=vector(n,i,1));
  for(k=2,n,
     \\ print("k="k" a="a);
     forstep(n=n,1,-1,
            a[n] += sum(m=1,n\k,
                        \\ print(" n="n" m="m" bin "binomial(n,m*k)" * ",if(n-m*k==0,1,a[n-m*k]));
                        binomial(n,m*k) * if(n-m*k==0,1,a[n-m*k]))));
  a;
}
A(10)

vector(10,n,vpar_num_siblings_vectors_count(n))

T(n,k=n) =
{
  if(n<0, error());
  if(k<1, error());
  if(k==1, 1, \\ if(n!=1,0,1),
     n==0, 1,
     sum(m=0, (n-1)\k,
         binomial(n,m*k) * T(n-m*k,k-1)));
}
F(2)
T(2)
check_equal(T(1,1), 1, "T(1,1)");
check_equal(T(2,1), 1, "T(2,1)");
check_equal(T(2,2), 1, "T(2,2)");
vector(10,n,T(n))

C(n,k,forest) =
{
  if(n<0, error());
  if(k<1, error());
  if(n==0, 1,
     k==1, 1,
     sum(m=0, (n-!forest)\k,
         binomial(n,m*k) * C(n-m*k,k-1,forest)));
}
vector(10,n,C(n,n,0))
vector(10,n,C(n,n,1))

print();
{
  my(forest=0); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_num_siblings_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_siblingpos_vectors_count() bits

{
  for(n=1,100,
     \\ print(vpar_siblingpos_vectors_count(n)*1.0/vpar_count(n));
     \\ print(log(vpar_siblingpos_vectors_count(n)));
     print(vpar_siblingpos_vectors_count(n)^(1/n));
  );
}
quit


\\-----------------------------------------------------------------------------
\\ vpar_make_num_children() upwards and downwards

\\ Over all possible compositions, being all vpar_is_num_children_vector(),
\\ what kinds of trees result from vpar_make_num_children().

\\ vpar upwards
\\ not in OEIS: 1, 2, 4, 9, 20, 51, 125, 305
\\ not A002861 connected functions has 329
\\
\\ vpar upwards, num_children increasing
\\ 1, 2, 3, 5, 7, 11, 15, 22,
\\ A000041 partitions
\\ since increasing num_children results in upwards vpar and partitions many
\\ increasing compositions
\\
\\ vpar upwards, num_children not increasing
\\              0, 0, 1, 4, 13, 40, 110, 283
\\ not in OEIS:          4, 13, 40, 110, 283

\\ vpar downwards n= 2 3 4 5  6
\\              1, 1, 2, 4, 5, 13, 27, 61
\\ not in OEIS:       2, 4, 5, 13, 27, 61
\\
\\ vpar downwards, num_children not decreasing
\\           n= 2 3 4 5  6
\\              0, 0, 0, 1, 1, 7, 18, 48, 115
\\ not in OEIS:                7, 18, 48, 115
\\ maybe A250785 arrays

\\ upsorted = downsorted= partitions
\\ 1 2 3 5 7 11 15
\\ A000041

{
  for(n=2,11,
     my(comps=test_compositions(n-1,[0,n-1],n),
        count_upwards=0,
        count_upwards_sort=0,
        count_upwards_mono=0,
        count_upwards_others=0,
        count_downwards=0,
        count_downwards_sort=0,
        count_downwards_mono=0,
        count_downwards_others=0,
        count_nonewards=0);
     for(i=1,#comps,
        my(num_children=comps[i],
           vpar=vpar_make_num_children(num_children),
           is_upwards  =vpar_is_upwards(vpar),
           is_downwards=vpar_is_downwards(vpar),
           num_children_is_upwards  =test_vector_is_increasing(num_children),
           num_children_is_downwards=test_vector_is_decreasing(num_children));

        count_upwards        += is_upwards;
        count_upwards_mono   += (is_upwards && num_children_is_upwards);
        count_upwards_others += (is_upwards && !num_children_is_upwards);
        count_upwards_sort   += num_children_is_upwards;
        count_downwards_sort += num_children_is_downwards;
        count_downwards        += is_downwards;
        count_downwards_mono   += (is_downwards && num_children_is_downwards);
        count_downwards_others += (is_downwards && !num_children_is_downwards);
        count_nonewards += (!is_upwards && !is_downwards);

        if(    (0 && is_upwards && !num_children_is_upwards)
            || (0 && is_downwards && !num_children_is_downwards),
           print("vpar "vpar);
           print("num_children "num_children);
           vpar_view(vpar,1));
        );
     \\ print(count_upwards" "count_downwards" "count_nonewards);
     print1(count_upwards_others", ");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_weights_vector() increasing, unlabelled

\\ unlabelled trees, labelled preorder max depths
\\ n = 1  2  3  4  5  6  7  8  9
\\     1, 1, 1, 2, 2, 4, 4, 8, 8, 16, 16, 32
\\ A158780 repeat powers-of-2, presumably
\\
{
  for(n=1,12,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           my(weights=vpar_weights_vector(vpar));
           if(n==6&&weights==vecsort(weights), vpar_view(vpar));
           count += test_vector_is_increasing(weights));
     print(count);
     );
  quit;
}


\\------------------------------------------------------------------------------

tableau_insert(t,n) =
{
  for(i=1,#t,
     for(j=1,#t[i],
        if(t[i][j]>n,
           my(temp=t[i][j]); t[i][j]=n; n=temp; next(2)));
     t[i]=concat(t[i],[n]); return(t));
  concat(t,[[n]]);
}
perm_to_tableau(perm) =
{
  my(t=[]);
  for(i=1,#perm, t=tableau_insert(t,perm[i]));
  t;
}
tableau_shape(t) = apply(length,t);
perm_is_involution(perm) = perm^-1==perm;
install(perm_cycles, "G", "perm_cycles");
shape_is_winner(s) = #s<2 || s[1]>s[2];
shape_is_tie(s) = #s>=2 && s[1]==s[2];
shape_is_equal(s) = #s<2 || s[1]==s[2];
shape_is_lastone(s) = s[#s]==1;

n=6;
{
  my(count=0);
  for(i=0,n!-1,
     my(perm=Vecsmall(numtoperm(n,i)));
     perm_is_involution(perm) || next();
     my(t=perm_to_tableau(perm),
        cycles=apply(Vec,perm_cycles(perm)),
        shape=tableau_shape(t));
     \\ shape_is_tree(shape) || next;
     \\ shape_is_lastone(shape) || next;
     shape_is_winner(shape) || next;
     \\ printf("%-27s %-27s %s\n", cycles,t,shape);
     printf("%-27s %s\n", cycles,shape);
     count++;
  );
  print("count "count);
}
quit


\\------------------------------------------------------------------------------
\\ vpar_siblingpos_vectors_count()
\\ A238124 MAPLE

\\ h:= proc(l) local n; n:=nops(l);
\\   add(i, i=l)! / mul(mul(1+l[i]-j+add(`if`(l[k]>=j, 1, 0), k=i+1..n), j=1..l[i]), i=1..n)
\\
\\ g:= proc(n, i, l) `if`(n=0, 0,
\\                   `if`(i=1, h([l[], 1$n]),
\\                   add(g(n-i*j, i-1, [l[], i$j]), j=0..n/i)))
\\ a:= n-> g(n, n, []):

h(l) =
{
  my(n=#l);
  \\ print(l);
  vecsum(l)! / prod(i=1,n, prod(j=1,l[i], 1 + l[i] - j + sum(k=i+1,n, if(l[k]>=j, 1, 0))));
}
g(n,i,l) =
{
  if(n==0, return(0));
  if(i==1, return(h(concat(l, vector(n,x,1)))));
  sum(j=0,floor(n/i), g(n-i*j, i-1, concat(l, vector(j,x,i))));
}
a(n) = g(n,n,[]);
a(6)
forpart(v=6,print(Vecrev(v)));

vector(10,n, a(n))

a_by_part(n) =
{
  my(total=0);
  forpart(v=n, my(v=Vecrev(v)); if(v[#v]==1, total+=h(v)));
  total;
}
vector(10,n, a_by_part(n))

a_by_first(n) =
{
  my(total=0);
  forpart(v=n, my(v=Vecrev(v)); if(#v<2 || v[1]>v[2], total+=h(v)));
  total;
}
vector(10,n, a_by_part(n))


check_equal(vector(#A238124_samples,n,n--; a(n)), A238124_samples)
quit


\\------------------------------------------------------------------------------
\\ vpar_is_siblingpos_vector()

{
  my(n=5,
    count_F=0,
    count_T=0);
  forvec(vec=vector(n,i,[0,n]),
        my(F=vpar_is_siblingpos_vector(vec,,1),
           T=vpar_is_siblingpos_vector(vec));
        count_F += F;
        count_T += T;
        if(F||T,
           print(vec"  "F" "T);
           ));
  print("F "count_F"  T "count_T);
}
quit



\\------------------------------------------------------------------------------
\\ vpar_siblingpos_vector() occurring, ballot sequence counts

f(v) =
{
  if(#v==0, return(1));
  for(i=1,#v, v[i]>=0 || return(0));
  for(i=1,#v-1, v[i]>=v[i+1] || return(0));
  if(v[#v]==0, return(f(v[^#v])));
  sum(i=1,#v, my(d=v);d[i]--;f(d));
}
f=memoize(f);
F(n) =
{
  my(total=0);
  forpart(v=n, my(v=Vecrev(v)); total+=f(v));
  total;
}
T_by_filter(n) =
{
  my(total=0);
  forpart(v=n, my(v=Vecrev(v)); if(#v<2||v[1]>v[2], total+=f(v)));
  total;
}

diff(v) =
{
  \\ print("diff v="v);
  #v>=2 || error();
  v[1]==v[2] || error();
  if(v==[0,0], return(1));
  sum(i=2,#v, my(d=v);d[i]--;f(d));
}
diff=memoize(diff);
Diff(n) =
{
  my(total=0);
  for(f=0,n\2, forpart(v=n-2*f,
                       \\ print("f="f" "v);
                       my(v=Vecrev(v)); total+=diff(concat([f,f],v)),
                      f));
  total;
}
\\T(3)
\\ quit
print("F    ",vector(10,n, F(n)));
print("Tf   ",vector(10,n, T_by_filter(n)));
print("Diff ",vector(10,n, Diff(n)));
print("F-Tf ",vector(10,n, F(n)-T_by_filter(n)))
vector(10,n, vpar_siblingpos_vectors_count(n,1)-vpar_siblingpos_vectors_count(n))

forpart(v=6, my(v=Vecrev(v)); print(v"  "f(v)));

\\ quit
\\ {
\\   for(n=1,100,
\\      check_equal(vpar_siblingpos_vectors_count(n), T(n), "T");
\\      check_equal(vpar_siblingpos_vectors_count(n,1), F(n), "F");
\\      );
\\ }

\\ f=first
H(n,f) =
{
  n>=0 || error();
  if(f<0 || f>n,return(0));
  if(n==0, return(f!));
  H(n-1,f-1) + (n-1)*H(n-1,f);
}
for(n=1,8, my(row=0); for(f=0,n, my(h=H(n,f)); printf(" %4d",h); row+=h); print(" = "row));
quit



\\------------------------------------------------------------------------------
\\ vpar_siblingpos_vector() occurring

\\ trees
\\ 1, 1, 3, 7, 20, 56, 182, 589,
\\ A238124 ballot sequences length n, 1 largest part
\\
\\ forests
\\ 1, 2, 4, 10, 26, 76, 232, 764,
\\ A000085 self-inverse permutations
\\ egf exp(x+x^2/2) satifies y^2 = y''*y' - (y')^2
\\ a(n) = a(n-1) + (n-1)*a(n-2)
\\ 1+A001189 permutations order=2
\\
\\ trees root=1, same as forests 1 previous
\\ 1, 1, 2, 4, 10, 26, 76,
\\
\\ forests - trees
\\ [1, 2, 4, 10, 26, 76, 232, 764] - [1, 1, 3, 7, 20, 56, 182, 589]
\\ 0, 1, 1, 3, 6, 20, 50, 175
\\ not A005558
\\ vector(10,n, binomial(n+1, ceil(n/2))*binomial(n, floor(n/2)) - binomial(n+1, ceil((n-1)/2))*binomial(n, floor((n-1)/2)))
\\ vector(10,n,  binomial(n, floor(n/2)) *binomial(n-1,ceil(n/2)-1) \
\\             - binomial(n, ceil(n/2)-1)*binomial(n-1,floor(n/2)-1))
\\ vector(10,n, my(c = ((n+1)>>1)-1); binomial(n, n>>1) *binomial(n-1,c) \
\\              - binomial(n, c)*binomial(n-1,(n>>1)-1))
\\ vector(10,n, vpar_siblingpos_vectors_count(n,1))
\\ vector(10,n, vpar_siblingpos_vectors_count(n,1)-vpar_siblingpos_vectors_count(n))
{
  my(forest=-1); \\ trees or forests
  for(n=1,10,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=vpar_siblingpos_vector(vpar));
           mapput(m,vec,1));
     print1(#m", ");
  );
  quit;
}


\\----------------------------------------------------------------------------
\\ vpar_relabel_random() uniformity

n=5;
vpar=vpar_make_random(n)
m=Map()
{
  for(i=1,n!,
     my(r=vpar_relabel_perm(vpar,numtoperm(n,i)),
        c=0);
     print(r);
     mapisdefined(m,r,&c);
     mapput(m,r,c+1));
}
vpar_INTERNAL_histogram(Mat(m)[,2])
quit

\\-----------------------------------------------------------------------------
\\ buckets and empties

\\ all
\\ A076014 triangle m^n
\\
\\ >=1 empties
\\ A101030 triangle
\\ 0,
\\ 0, 2,
\\ 0, 2, 21,
\\ 0, 2, 45, 232,
\\ 0, 2, 93, 784, 3005,
\\
\\ all buckets >=2
\\ A200091
\\ 0,
\\ 1, 0,
\\ 1, 0, 0,
\\ 1, 6, 0, 0,
\\ 1, 20, 0, 0, 0,
\\ 1, 50, 90, 0, 0, 0,

{
  my(limit=6,
     m=matrix(limit,limit));
  for(n=1,limit,
     for(k=1,n,
        \\ print("n="n" k="k);
        my(count_all=0,
           count_none_empty=0,
           count_ge1_empty=0,
           count_ge1_one=0,
           count_all_getwo=0);
        forvec(vec=vector(n,i,[1,k]),
              \\ print(vec);
              count_all++;
              my(hist=vector(k));
              for(i=1,#vec, hist[vec[i]]++);
              my(non_empties=hammingweight(hist),
                 empties=#hist-non_empties);
              count_ge1_empty += (empties>=1);
              count_none_empty += (empties==0);

              my(ones=sum(i=1,#hist, hist[i]==1));
              count_ge1_one += (ones>=1);

              my(twos=sum(i=1,#hist, hist[i]==2),
                 getwos=sum(i=1,#hist, hist[i]>=2));
              count_all_getwo += (getwos==#hist);

              \\ if(1&&n==3&&k==3,
              \\    print(vec" hist "hist"  empties "empties" non"non_empties));
              );
        check_equal(count_all, k^n, Str("count_all n="n" k="k));
        check_equal(count_ge1_empty, sum(d=1,k, (-1)^(d-1) * binomial(k,d)*(k-d)^n));
        check_equal(count_none_empty, sum(d=0,k, (-1)^d * binomial(k,d)*(k-d)^n));

        \\ Dennis P. Walsh in A200091
        check_equal(count_all_getwo,
          sum(i=0,k, sum(j=0,k,
             (-1)^j * binomial(k,j) * binomial(j,i) * binomial(n,i) * i!
             * (k-j)^(n-i))));
        m[n,k] = count_ge1_empty;
        m[n,k] = count_ge1_one;
        m[n,k] = count_all_getwo;
     );
  );

  for(n=1,limit,
     for(k=1,n, print1(m[n,k]", "));
     print());

  for(n=1,limit,
     for(k=1,n,
        v = binomial(n,1) * sum(d=1,k, (-1)^(d-1) * binomial(k,d)*(k-d)^(n-1));
        print1(v", "));
     print());
}
quit




\\ all 1, 4, 27, 256, 3125
\\ A000312 n^n
\\
\\ >=1 empty 0, 1, 14, 181, 2584, 41973, 776250
\\ A133286 n^n overestimates value of (1/2) Sum_{k>=0} k^n/2^k.

{
  my(limit=7);
  my(count_all=vector(limit));
  my(count_ge1_empty=vector(limit));
  for(n=1,limit,
     forvec(vec=vector(n,i,[1,n]),
           count_all[n]++;
           my(hist=vpar_INTERNAL_histogram(vec),
              non_empties=hammingweight(hist),
              empties=#hist-non_empties);
           count_ge1_empty[n] += (empties>=1);
           ));
  print("all        "count_all);
  print(">=1 empty  "count_ge1_empty);
}
quit


\\----------------------------------------------------------------------------
\\ vpar_eccentricities_vector() occurring formulas

limit = 10;
L = O(x^limit);

\\ A032032 ordered partition into >=2 each
\\ unicentral with v=1 centre
GE2(n) = if(n==0,1, sum(k=2,n, binomial(n,k)*GE2(n-k)));
vector(10,n, GE2(n))
vector(10,n, if(n==1,0, 1 + sum(k=2,n, binomial(n,2*k)*(n-2*k)^k)))

gGE2 = test_func_to_egf(GE2, limit);

check_equal(binomial(n,0), 1);
check_equal(binomial(n,1), n);
check_equal(test_func_to_egf(n->n*GE2(n-1), limit), \
            x*gGE2);

\\ g = 1 + exp(x)*g - g - x*g
check_equal(gGE2, 1 + exp(x+L)*gGE2 - (1+x)*gGE2);
check_equal(gGE2, 1/(2+x-exp(x+L)));

\\ unicentral eccentricities, any centre
U(n) = n*GE2(n-1);
vector(10,n, U(n))
check_equal(test_func_to_egf(U, limit), x/(2+x-exp(x+L)));

\\ bicentral eccentricities, any centre
B(n) = binomial(n,2)*GE2(n-2);
vector(10,n, B(n))
check_equal(test_func_to_egf(B, limit), 1/2*x^2/(2+x-exp(x+L)));

\\ eccentricities vectors
E(n) = U(n) + B(n);
vector(10,n, E(n))
check_equal(test_func_to_egf(E, limit), \
            x*(1+x/2)/(2+x-exp(x+L)));


\\ A008277 Stirling triangle
for(n=1,8,for(k=1,n, print1(stirling(n,k,2)","));print())

\\ k buckets
\\ i extras anywhere
St(n,k) = 1/k! * sum(i=0,k, (-1)^(k-i)*binomial(k,i)*i^n);
matrix(8,8,n,k, St(n,k))


\\ GP-DEFINE  G(n) = if(n==0,1, sum(k=2,n, binomial(n,k)*G(n-k)));
\\ vector(10,n,G(n))
\\ A032032
\\
\\ GP-DEFINE  E(n) = if(n==0,1, if(n>=1,n*G(n-1)) + if(n>=2,(n-1)*n/2*G(n-2)));
\\ vector(10,n,n--; E(n))
\\ A032032
\\
\\ n=13;k=5; binomial(n-2,k-1)*(n-1)/k == binomial(n-1,k)
\\ n=13;k=5; n*binomial(n-1,k) == binomial(n,k)*(n-k)
\\ G(n-1) = sum(k=2,n-1, binomial(n-1,k)*G(n-1-k))
\\ vector(10,n, G(n-1))
\\ vector(10,n, if(n==1,1, sum(k=2,n-1, binomial(n-1,k)*G(n-1-k))))
\\
\\ G(n-2) = sum(k=2,n-2, binomial(n-2,k)*G(n-2-k))
\\        = sum(k=3,n-1, binomial(n-2,k-1)*G(n-2-k+1)))
\\        = sum(k=3,n-1, k/(n-1)*binomial(n-1,k)*G(n-1-k)))
\\ vector(10,n,n++; G(n-2))
\\ vector(10,n,n++; if(n==2,1,sum(k=3,n-1, k/(n-1)*binomial(n-1,k)*G(n-1-k))))
\\
\\ vector(10,n,n--; E(n))
\\ vector(10,n,n--; \
\\    if(n==0,1, \
\\         if(n>=1,n*if(n==1,1, sum(k=2,n-1, binomial(n-1,k)*G(n-1-k)))) \
\\       + if(n>=2,(n-1)*n/2*if(n==2,1,sum(k=3,n-1, k/(n-1)*binomial(n-1,k)*G(n-1-k))))))
\\ vector(10,n,n--; \
\\    if(n<=2,1, \
\\         sum(k=2,n-1, n*binomial(n-1,k)*G(n-1-k)) \
\\       + sum(k=3,n-1, n/2*k*binomial(n-1,k)*G(n-1-k))))
\\ vector(10,n,n--; \
\\    if(n<=2,1, sum(k=2,n-1, n*(1+k/2)*binomial(n-1,k)*G(n-1-k)) \
\\               - n/2*2*binomial(n-1,2)*G(n-1-2)))
\\ vector(10,n,n--; \
\\    if(n<=2,1, sum(k=2,n-1, n*(1+k/2)*binomial(n-1,k)*G(n-1-k)) \
\\               - 3*binomial(n,3)*G(n-3)))
\\ vector(10,n,n--; \
\\    if(n<=2,1, sum(k=2,n-1, 1/2*(k+2)*(n-k) * binomial(n,k)*G(n-k-1)) \
\\               - 3*binomial(n,3)*G(n-3)))

quit

\\----------------------------------------------------------------------------
\\ vpar_eccentricities_vector() increasing, or occurring
\\ decreasing the same

\\ forests with eccentricity increasing
\\ not in OEIS: 1, 3, 6, 30, 93, 493, 2014,
\\
\\ trees with eccentricity increasing
\\ not in OEIS: 1, 2, 3, 12, 45, 186, 847, 4312,
\\
\\ trees root=1 with eccentricity increasing
\\              1, 1, 1, 3, 9, 31, 121, 539, 2699,
\\ not in OEIS:          3, 9, 31, 121, 539, 2699,
\\
\\ trees root=unicentre=1 with eccentricity increasing
\\          n = 0  1  2  3  4  5   6   7    8
\\              0, 1, 0, 1, 1, 3, 13, 55, 237, 1113,
\\ not in OEIS:                3, 13, 55, 237, 1113,


\\ n=3 trees
\\ 2 <- 1 <- 3     ecc(1)=1, ecc(2,3)=2
\\ 3 <- 1 <- 2      same
\\ 2 -> 1 <- 3      same
\\
\\ n=4
\\ 4 <- 2 <- 1 <- 3   ecc(1,2)=2, ecc(3,4)=3   and perms and roots
\\      1
\\     /|\            ecc(1)=1, ecc(2,3,4)=2   and roots
\\    2 3 4
\\
\\ 1 or 1,2 centre, min ecc
\\ n leaf attain ecc from centre for max ecc
\\ with root=1, n at height

\\ trees eccentricities vectors occurring, unicentral v=1
\\ 1  2  3  4  5   6    7    8
\\ 1, 0, 1, 1, 7, 21, 141, 743,
\\ A032032 partition labelled into sets each >=2 and order those sets
\\
\\ trees eccentricities vectors occurring, unicentral
\\ not in OEIS: 1, 0, 3, 4, 35, 126, 987, 5944,
\\
\\ vector(10,n, n*sum(k=0,n-1, binomial(n-1,k)*(n-1-k)^k))
\\
\\ trees eccentricities vectors occurring, bicentral
\\ not in OEIS: 0, 1, 0, 6, 10, 105, 441,
\\
\\ trees eccentricities vectors occurring
\\ not in OEIS: 1, 1, 3, 10, 45, 231, 1428,

\\ partition into k, each >=1
\\ A000670 ordered Bell
\\ vector(10,n, sum(k=1,n, k!*stirling(n,k,2)))
\\
\\ vector(10,n, sum(k=1,n, k!*binomial(n,k)*stirling(n-k,k,2)))
\\ vector(10,n, sum(k=0,n, binomial(n,k)*(n-k-1)^k))
\\ A052506

{
  for(n=1,10,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           \\ if(vpar_centres(vpar)!=[1], next());  \\ 1=centre
           \\#vpar_centres(vpar)==2 || next();

           my(eccentricities = vpar_eccentricities_vector(vpar),
              is_increasing = (eccentricities==vecsort(eccentricities,,0)));

           mapput(m,eccentricities,1);
           count += is_increasing;

           \\ if(n==4 && is_increasing, vpar_view(vpar));
     );
     \\ print(count",");             \\ vpar with increasing
     print(#m,",");  \\ vectors occurring
     \\ if(n==2,
     \\    print("n="n"  "Mat(m)[,1]);
     \\   )
  );
  quit;
}


\\----------------------------------------------------------------------------
\\ vpar_indnum() == n/2 in free trees is perfect matching

\\ not in OEIS: 0, 1, 0, 1, 0, 2, 0, 5, 0, 15, 0, 49, 0, 180, 0, 701,
\\ 1, 1, 2, 5, 15, 49, 180, 701, 2891,
\\ A005751 Number of matched trees with 2n nodes.

try(vpar) =
{
  count += (vpar_indnum(vpar) == #vpar/2);
  0;
}
{
  forstep(n=2,20,2,
     count=0;
     total=0;
     read(Str("/tmp/trees-"n".gp"));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_critical_inddiff()

set_is_subset(sub,super) = setintersect(sub,super) == sub;

\\ Butenko & Trukhanov, critical indset is subset of some maximum indset
try(vpar) =
{
  my(diff=vpar_critical_inddiff(vpar),
     sets=vpar_indsets(vpar),
     indnum_sets=vecsort(vpar_indnum_sets(vpar)),
     indnum=#indnum_sets[1],
     critical_sets=vecsort(select(set->vpar_critical_setdiff(vpar,set)>=diff,
                                  sets)),
     critical_indnum = vecmax(apply(length,critical_sets)),
     maximals_count = vpar_indomsets_count(vpar),
     maximals_sets=vecsort(select(set->vpar_is_indomset(vpar,set),
                                  sets)));

           \\ if(maximals_count==#critical_sets, next());
           \\ if(maximals_count==#indnum_sets, next());
  if(diff, next());

  print("indsets "#sets
        "  diff "diff
        "  maximals "maximals_count
        "  criticals "#critical_sets" maxsize "critical_indnum
        "  indnums "#indnum_sets" size "indnum
        "  delta "critical_indnum-indnum);

  for(i=1,#critical_sets,
     my(found);
     for(j=1,#indnum_sets,
        if(set_is_subset(critical_sets[i],indnum_sets[j]),
           found++));
     found || error());

  if(#critical_sets > #indnum_sets,
     print("  vpar "vpar);
     print("  maximals_sets "maximals_sets);
     print("  critical_sets "critical_sets);
     print("  indnum_sets   "indnum_sets);
    );
}
{
  for(n=1,10,
     read(Str("/tmp/trees-"n".gp"));
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_cycles() total

\\ forest vpars
\\ 1, 7, 62, 696, 9564, 156115, 2957072,
\\ A216534 total num cycles in functional digraphs
\\
\\ tree vpars ==1 root
\\ not in OEIS: 0, 2, 21, 248, 3480, 57384, 1092805,
\\ tree vpars <=1 root
\\ not in OEIS: 1, 7, 59, 638, 8529, 136344, 2540691,
\\
\\ tree vpars, one root=1, is forest vpars n-1 as root above forest
\\ 0, 1, 7, 62, 696, 9564, 156115, 2957072,
\\ A216534 as above

\\ n=2
\\ [0,0]
\\ [1,0] = 1    tree
\\ [2,0]
\\ [0,1]
\\ [1,1] = 1
\\ [2,1] = 1
\\ [0,2] = 1    tree root=1
\\ [1,2] = 2
\\ [2,2] = 1
\\ GP-Test  vpar_count(2,1) == 3

{
  my(forest=-1);
  for(n=1,10,
     my(total=0);
     forvec(vpar=vector(n,v,[0,n]),
           if(forest==-1 && vpar[1]!=0, next());
           if(forest<=0 && vpar_num_roots(vpar)!=1, next());
           total += vpar_num_cycles(vpar));
     print(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ Konig-Egervary indum + matchnum = n

\\ always in tree ...

{
  forstep(n=0,16,1,
     my(depths=vpar_preorder_depths_first(n),
        count=0,
       count_non=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              indnum=vpar_indnum(vpar),
              matchnum=vpar_matchnum(vpar),
              is_KE = (indnum+matchnum==n));
           count += is_KE;
           count_non += !is_KE;
          );
     \\print(count",");
     print(count_non",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar step to next acyclic

diff_pos(x,y) =
{
  for(i=1,#x, if(x[i]!=y[i], return(i))); 0;
}
{
  my(forest=1);
  for(n=1,8,
     my(prev='none,
        m=99);
     forvec(vpar=vector(n,i,[0,n]),
           if(vpar_is_cyclic(vpar),next());
           forest || vpar_is_tree(vpar) || next();
           my(pos);
           if(prev!='none,
              pos=diff_pos(vpar,prev);
              if(pos < m,
                 print("new low pos "pos));
              m=min(m,pos);
             );
           \\ print(vpar"  ",if(prev=='none,"",pos));
           prev=vpar;
     );
     print("min diff_pos "m);
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ leaves at same depths parity


\\ free trees with leaves all at even distances from each other
\\              1, 0, 1, 1, 2, 2, 5, 6, 13, 20, 41, 70, 144, 266, 545, 1072, 2210, 4491,
\\ A304867                        5, 6, 13, 20, 41, 70, 144, 266, 545, 1072, 2210, 4491,
\\ proportion out of all free trees
\\ 1.0000  1
\\ 0.0000  0
\\ 1.0000  1
\\ 0.5000  1/2
\\ 0.6667  2/3
\\ 0.3333  1/3
\\ 0.4545  5/11
\\ 0.2609  6/23
\\ 0.2766  13/47
\\ 0.1887  10/53
\\ 0.1745  41/235
\\ 0.1270  70/551
\\ 0.1107  144/1301
\\ 0.0842  266/3159
\\ 0.0704  545/7741
try(vpar) =
{
  my(depths = vpar_depths_vector(vpar),
     leaves=vpar_leaves(vpar));
  my(parity=-1);
  for(i=1,#leaves,
     if(parity==-1, parity=depths[leaves[i]]%2);
     if(depths[leaves[i]]%2!=parity, return(0)));
  count++;
}
{
  for(n=1,20,
     count=0;
     total=0;
     read(Str("/tmp/trees-"n".gp"));
     \\print(count",");
     my(total=vpar_count_free(n));
     printf("%.4f  %s\n", count/total*1.0, count/total);
  );
  quit;
}

\\ unlabelled rooted trees all even parity depths
\\              1, 1, 0, 1, 1, 3, 4, 10, 17, 38, 73, 158, 324, 700, 1483, 3224, 6979,
\\ A337089                     3, 4, 10, 17, 38, 73, 158, 324, 700, 1483, 3224, 6979,
\\
\\ unlabelled rooted trees all odd parity depths
\\              1, 0, 0, 1, 1, 2, 3, 7, 12, 26, 51, 110, 226, 488, 1038, 2259,
\\ not in OEIS:                2, 3, 7, 12, 26, 51, 110, 226, 488, 1038, 2259,
\\
\\ unlabelled rooted trees all same parity depths
\\              1, 1, 0, 2, 2, 5, 7, 17, 29, 64, 124,
\\ not in OEIS:          2, 2, 5, 7, 17, 29, 64, 124,
{
  my(forest=0);
  for(n=0,16,
     my(vpar=vpar_unlabelled_first(n),
        count=0);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(depths = vpar_depths_vector(vpar),
              leaves=vpar_leaves(vpar));
           my(parity=0);
           for(i=1,#leaves,
              if(parity==-1, parity=depths[leaves[i]]%2);
              if(depths[leaves[i]]%2!=parity, next(2)));
           count++);
     print(count",");
     );
  quit;
}

\\ labelled rooted trees all even depths
\\              1, 0, 6, 12, 200, 1470, 20832,
\\ not in OEIS:       6, 12, 200, 1470, 20832,
\\
\\ labelled rooted trees all odd depths
\\              0, 0, 3, 4, 125, 726, 13027,
\\ not in OEIS:       3, 4, 125, 726, 13027,
\\
\\ labelled rooted trees all same parity depths
\\              1, 0, 9, 16, 325, 2196, 33859,
\\ not in OEIS:       9, 16, 325, 2196, 33859,
{
  my(forest=0); \\ trees or forests
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              depths=vpar_depths_vector(vpar),
              leaves=vpar_leaves(vpar));
           my(parity=-1);
           \\ print(depths" "leaves" "parity);
           for(i=1,#leaves,
              if(parity==-1, parity=depths[leaves[i]]%2);
              if(depths[leaves[i]]%2!=parity, next(2)));
           count++;
     );
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_degrees_vectors_count() binomials

m=8;
C(m) =
{
  print1("sum");
  for(c=0,floor(m/2)-1, print1(" ("2*m-3-2*c","m-1")"));
  print();
}

F(n) =
{
  for(m=2,n, print1("("n","m") * "); C(m));
}
F(8);
\\ (8,2) * sum (1,1)
\\ (8,3) * sum (3,2)
\\ (8,4) * sum (5,3) (3,3)
\\ (8,5) * sum (7,4) (5,4)
\\ (8,6) * sum (9,5) (7,5) (5,5)
\\ (8,7) * sum (11,6) (9,6) (7,6)
\\ (8,8) * sum (13,7) (11,7) (9,7) (7,7)

binomial(13,7) + binomial(11,7) + binomial(9,7) + binomial(7,7)
13*12/6/5*11*10/4/3*9*8/2/1 + 11*10/4/3*9*8/2/1 + 9*8/2/1 + 1
(13*12/6/5*11*10/4/3 + 11*10/4/3 + 1)*9*8/2/1 + 1
((13*12/6/5 + 1)*11*10/4/3 + 1)*9*8/2/1 + 1
((13*12 + 6*5)/6/5*11*10/4/3 + 1)*9*8/2/1 + 1
((13*12 + 6*5)*11*10 + 6*5*4*3)*9*8 /6/5/4/3/2/1 + 1
13*12/6/5    \\ not an integer

\\ not a linear recurrence
\\ recurrence_guess(vector(50,m,C(m)));

\\ vector(10,n, my(total=0); forstep(k=n,2*n,2, total+=binomial(k,n)); total)
\\ vector(10,n, my(total=0); forstep(k=n+1,2*n,2, total+=binomial(k,n)); total)
\\ not in OEIS: 1, 7, 11, 86, 148, 1163, 2083, 16414, 30086, 237590
\\ not in OEIS: 2, 3, 24, 40, 314, 553, 4352, 7896, 62292, 115126
\\
\\ vector(10,n, my(total=0); forstep(k=n,2*n,1, total+=binomial(k,n)); total)
\\ A001700 binomial(2n+1, n+1)
\\ A088218 total leaves in ordered trees n

quit


\\------------------------------------------------------------------------------
\\ vpar_num_ordered_isomorphic()

\\ sizes occurring

\\ not in OEIS: 1, 2, 3, 6, 4, 12, 24, 5, 20, 30, 60, 120, 6, 30, 60, 120, 180, 360, 720, 7, 42, 105, 140, 210, 420, 630, 840, 1260, 2520, 5040

\\ 6, 30, 60, 120, 180, 360, 720
\\ A260633 new records for prime factorizations

\\ A210237 ,1,1,2,1,3,6,1,4,6,12,24,1,5,10,20,30,60,120,1,6,15,20,30,60,90,120,180,360,720,1,7,21,35,42,105,140,210,420,630,840,1260,2520,5040,1,8,28,56,70,168,280,336,420,560,840,1120,1680,2520,3360,5040,6720,
\\ 1,6,15,20,30,60,90,120,180,360,720 is more than occur from ordered trees
\\ omits values of M(n) seen by a smaller n

test_multinomial(v) = vecsum(Vec(v))! / prod(i=1,#v,v[i]!);
forpart(v=6, print(test_multinomial(v)));
\\ n=7
\\ 1 7 21 35 42 105 140 210 210 420 630 840 1260 2520 5040
\\ A036038 multinomials of partitions in lexicographic order


{
  my(forest=0); \\ trees or forests
  for(n=1,8,
     my(m=Map());
     my(sizes=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              pre=vpar_relabel_preorder(vpar),
              size=vpar_num_ordered_isomorphic(vpar));
           mapput(sizes,size,1);
           my(data);
           if(mapisdefined(m,pre,&data),
              if(size != data[1],
                 error());
              data[2]++;
              ,
              data=[size,1]);
           mapput(m,pre,data));
     m=Mat(m);
     for(i=1,matsize(m)[1],
        my(data=m[i,2]);
        if(data[1]!=data[2],
           error()));

     sizes=Mat(sizes);
     print(vecsort(Vec(sizes[,1])));

  );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_ordered_isomorphic_perm() uniqueness

{
  my(forest=0); \\ trees or forests
  for(n=1,5,
     my(total=0,
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar1=vpar_from_upascend(vec));
           forvec(vec=vpar_forvec_from(n,forest),
                 my(vpar2=vpar_from_upascend(vec));
                 if(lex(vpar1,vpar2)>=0, next());
                 my(iso_perm=vpar_ordered_isomorphic_perm(vpar1,vpar2));
                 if(iso_perm=='none, next());
                 iso_perm=Vec(iso_perm);
                 vpar1==vpar_relabel_perm(vpar2,iso_perm) || error();

                 my(seen=0);
                 for(i=1,n!,
                    my(perm=numtoperm(n,i));
                    if(perm==iso_perm,next());
                    if(vpar1==vpar_relabel_perm(vpar2,perm),
                       if(seen++==1,
                          print(vpar1" "vpar2" perm "iso_perm));
                       print("  also perm "perm
                            " "vpar_seq_is_ordered(vpar2,Vecsmall(perm)^-1))));
                 )));
  quit;
}

\\------------------------------------------------------------------------------
\\ childpos total (sibling position 1-based)

\\ trees total
\\ not in OEIS: 1, 4, 30, 304, 3875, 59616,
\\ = root1 * n, being n choices of root
\\
\\ trees root=1 total
\\ not in OEIS: 1, 2, 10, 76, 775, 9936, 153664,
\\ mean total/count
\\ 1.0000  1          1
\\ 2.0000  2          4
\\ 3.3333  10/3       10
\\ 4.7500  19/4       19
\\ 6.2000  31/5       31
\\ 7.6667  23/3       46
\\ 9.1429  64/7       64
\\ 10.6250  85/8      85
\\ 12.1111  109/9
\\ 1, 4, 10, 19, 31, 46, 64, 85, 109
\\ centre triangular 3n(n-1)/2 + 1
\\ mean total/count/n
\\ 1.0000  1
\\ 1.0000  1
\\ 1.1111  10/9
\\ 1.1875  19/16
\\ 1.2400  31/25
\\ 1.2778  23/18
\\ 1.3061  64/49
\\
\\ forests total
\\ not in OEIS: 1, 7, 60, 650, 8640, 136857,
\\ C(n) = 3*(n-1)*(n)/2+1;
\\ T1(n) = C(n)/n*vpar_count(n,-1);
\\ T1(n) = (3*(n-1)*n/2 + 1)*n^(n-3);
\\ T(n) = n*T1(n);
\\ T(n) = (3*(n-1)*n/2+1)*vpar_count(n,-1);
\\ F(n) = T1(n+1) - vpar_count(n,1);
\\ F(n) = (3*(n+1)*n/2 + 1)*(n+1)^(n-2) - (n+1)^(n-1);
\\ F(n) = (3*(n+1)*n/2 + 1 - n-1)*(n+1)^(n-2);
\\ F(n) = (3*n+1)*n/2 * (n+1)^(n-2);
\\ F(n) = 1/2*(3*n+1)*n/(n+1) * vpar_count(n,1);
\\ my(n=1000); F(n) / vpar_count(n,1) / n * 1.0  \\ -> 1.5
\\ my(n=1000); 1/2*(3*n+1)*n/(n+1) / n*1.0       \\ -> 1.5
\\ my(n=1000); T(n) / vpar_count(n) / n * 1.0  \\ -> 1.5
\\ my(n=1000); T1(n) / vpar_count(n,-1) / n * 1.0  \\ -> 1.5
\\ vector(10,n,C(n))
\\ vector(10,n,T1(n))
\\ vector(10,n,T(n))
\\ vector(10,n,F(n))
\\ T(3) == 30
\\ T(2) == 4
\\ T(1) == 1
\\ 1 <-- 2   = 1+1 = 2
\\ 2 <-- 1   = 1+1 = 2
\\ 1     2   = 1+2 = 3  total 7
\\ binomial(2,1) == 2
\\ vector(10,n,T1(n+1)-T1(n))
\\ vector(10,n,T1(n)-n*vpar_count(n,-1))
\\ 0, 0, 1, 12, 150, 2160, 36015, 688128, 14880348, 360000000
\\ A053507 binomial(n-1,2)*n^(n-3)
\\ vector(10,n,T(n)-n*vpar_count(n))
\\ not in OEIS: 3, 48, 750, 12960, 252105, 5505024, 133923132, 3600000000
\\ vector(10,n,F(n)-n*vpar_count(n,1))  \\ same as T1
\\ 1, 12, 150, 2160, 36015, 688128, 14880348, 360000000, 9646149645

\\ counting first position as 1
childpos_vector(vpar) = apply(pos->pos+1, vpar_siblingpos_vector(vpar));
{
  my(forest=1); \\ trees or forests
  for(n=1,3,
     my(total=0,
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              vec=childpos_vector(vpar));
           print(vpar" vec="vec"  comps="vpar_compnum_vector(vpar)" total "vecsum(vec));
           total += vecsum(vec);
           count++;
     );
     print(total",");
     \\ printf("%.4f  %s\n", n*total/count*1.0, n*total/count);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_preorder_depths_first(), vpar_preorder_depths_next()

{
  \\ p=5 q=2        q        p
  vpar     = [0, 1, 2, 3, 2, 1, 1, 1];
  print("vpar "vpar);
  print("got  "vpar_unlabelled_next(vpar));
  print("         q        p");
  print("want ",[0, 1, 2, 3, 1, 5, 6, 1]);
  print();
}
{
  \\ q=2,p=4     q     p
  vpar     = [0, 1, 2, 2, 1, 1, 1, 1];
  print("vpar "vpar);
  print("got  "vpar_unlabelled_next(vpar));
  print("         q     p");
  print("want ",[0, 1, 2, 1, 4, 1, 6, 1]);
  print();
}
\\quit

\\ p=5 q=2     q        p
\\ got  [0, 1, 2, 3, 4, 1, 1, 1]
\\ want [0, 1, 2, 3, 1, 5, 6, 1]

\\quit


\\       1      depths [0, 1, 2, 3, 2]       1
\\       |      vpar   [0, 1, 2, 3, 2]      / \
\\       2                 q        p      2   5
\\      / \     next qp=[2, 5]             |
\\     3   5    next [0, 1, 2, 3, 1]       3
\\     |        vpar [0, 1, 2, 3, 1]       |
\\     4                                   4

preorder_depths_next_qp(depths) =
{
  my(p=0,q=0);
  forstep(v=#depths,1,-1, if(depths[v]!=1, p=v;break()));
  forstep(v=p-1,1,-1, if(depths[v]<depths[p], q=v;break()));
  [q,p];
}

{
  for(n=3,8,
     print("n="n);
     my(depths=vpar_preorder_depths_first(n),
        count=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths));
           \\ vpar_preorder_depths_is_free_primary(depths) || error();
           count++;

           my(p,q); [q,p] = preorder_depths_next_qp(depths);
           print(depths"  vpar "vpar
                 "  next q="q",p="p
                 " q anc "vpar_is_ancestor(vpar,p,q));
          );
     print(count);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_relabel_minmax()

\\ min vpar is row-wise so its depths vector is min too

test_minmax_depths_perm_by_search(vpar,func) =
{
  my(best_perm=numtoperm(#vpar,0),
     best_depths=vpar_depths_vector(vpar),
     want_lex=func(1,-1));
  for(i=1,(#vpar)!-1,
     my(this_perm=numtoperm(#vpar,i),
        this_vpar   = vpar_relabel_perm(vpar,this_perm),
        this_depths = vpar_depths_vector(this_vpar));
     if(lex(this_depths,best_depths) == want_lex,
        best_perm   = this_perm;
        best_depths = this_depths));
  Vecsmall(best_perm);
}

{
  for(n=1,7,
     my(depths=vpar_preorder_depths_first(n),
        count=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              depths=vpar_depths_vector(vpar),

              mindepths_perm = test_minmax_depths_perm_by_search(vpar,min),
              mindepths_vpar = vpar_relabel_perm(vpar,mindepths_perm),
              mindepths_depths = vpar_depths_vector(mindepths_vpar),

              minpar_vpar  = vpar_relabel_minmax(vpar,min),
              minpar_depths = vpar_depths_vector(minpar_vpar),

              diff_depths=if(mindepths_depths==minpar_depths, "", " diff"),
              diff_vpar=if(mindepths_vpar==minpar_vpar, "", " diff"));

           \\ print(depths" ->  "mindepths_depths"  "minpar_depths,diff_depths);
           \\ if(mindepths_depths!=minpar_depths,
           \\    error("different depths from "vpar));

           print(vpar" ->  "mindepths_vpar"  "minpar_vpar,diff_vpar);
           if(mindepths_vpar!=minpar_vpar,
              error("different vpar from ",vpar));
          );
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_span_count()

\\ trees low = paths
\\ 2, 4, 7, 11, 16, 22, 29, 37,
\\ A000124 triangular + 1
\\ reps
\\ 1, 2, 9, 48, 300, 2160,
\\ A074143 a(1) = 1; a(n) = n * sum {a(k) | k < n}.
\\ n=4 [0, 1, 1, 2]  path-4
\\     [2, 0, 1, 2]  path-4
\\
\\ GP-Test  48+16 == vpar_count(4)
\\ paths n! ways, choose root, not also reversals
\\ GP-Test  4*4!/2 == 48
\\ GP-Test  vector(10,n, if(n<=1,1, n*n!/2)) == \
\\ GP-Test    [1, 2, 9, 48, 300, 2160, 17640, 161280, 1632960, 18144000]
\\ A074143 a(1) = 1; a(n) = n * sum {a(k) | k < n}.
\\ vpar_INTERNAL_exponential_transform([1, 2, 9, 48, 300, 2160, 17640, 161280, 1632960, 18144000])

\\ trees high
\\ 2, 4, 7, 12, 21, 38, 71,
\\ A005126 2^n + n + 1
\\ reps
\\ n          = 1  2  3   4   5   6   7
\\ not in OEIS: 1, 2, 9, 16, 25, 36, 49,
\\ n^2 except at n=2
\\ n=2 [0,1] and [2,0]
\\ n=4 stars  4*4=16 of

{
  my(forest=0, \\ trees or forests
     root=0);
  for(n=1,8,
     my(lo=-1,
        lo_reps=0,
        lo_list=List([]),
        hi=-1,
        hi_reps=0,
        hi_list=List([]),
        total=0,
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              c=vpar_span_count(vpar,root));
           if(lo==-1 || c<lo, lo=c;lo_reps=1;lo_list=List([vpar]),
              c==lo, lo_reps++;if(lo_reps<=10,listput(lo_list,vpar)));
           if(c>hi, hi=c;hi_reps=1;hi_list=List([vpar]),
              c==hi, hi_reps++;listput(hi_list,vpar));
           count++;
           total += c;
     );
     \\ print(lo_reps",")
     print(lo_reps" "lo_list)
     \\ print("n="n" low "lo" reps "lo_reps"  high "hi" reps "hi_reps"  total "total);
     \\ printf("%.4f  %s\n", total/count*1.0, total/count);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_spanfreqs() total

\\ total subtrees of free trees (reckoning vertices distinct)
\\ not in OEIS: 1, 3, 6, 21, 52, 165, 466, 1489, 4564, 15339, 50395, 174975, 609334, 2187642, 7917251,

\\ total non-empty subtrees of labelled rooted trees
\\ 1, 6, 54, 656, 10100, 189252, 4195870,
\\ A292633 n-th term of the n-th binomial transform of trees A000169
\\ b[n] = sum(k=0,n, (-1)^(n-k) * binomial(n,k) * a[k]
\\
\\ mean
\\ 1.0000  1
\\ 3.0000  3
\\ 6.0000  6
\\ 10.2500  41/4
\\ 16.1600  404/25
\\ 24.3380  5257/216
\\
\\ total non-empty subtrees of labelled rooted trees root=1
\\ not in OEIS: 1, 3, 18, 164, 2020, 31542, 599410,
\\ divide n
\\ mean
\\ 1.0000  1
\\ 3.0000  3
\\ 6.0000  6
\\ 10.2500  41/4
\\ 16.1600  404/25
\\ 24.3380  5257/216
\\ 35.6643  85630/2401
\\
\\ total non-empty subtrees of labelled rooted forests
\\ not in OEIS: 1, 8, 81, 1044, 16585, 315678,
\\ mean
\\ 1.0000  1
\\ 2.6667  8/3
\\ 5.0625  81/16
\\ 8.3520  1044/125
\\ 12.7971  16585/1296
\\ 18.7825  315678/16807

\\ root spanning
\\ trees total
\\ 1, 4, 30, 332, 4880, 89742, 1986124,
\\ A207833 T(T(x)) labelled rooted trees
\\ Paul D. Hanna
\\    a(n)=if(n==0||n==1,1, n^(n-1) - sum(k=1,n-1, (-1)^(n-k)*binomial(n,k) * k^(n-k) * a(k)))
\\ vector(10,n,a(n))
\\ trees mean
\\ 1.0000  1
\\ 3.0000  3
\\ 6.0000  6
\\ 10.2500  41/4
\\ 16.1600  404/25
\\ 24.3380  5257/216
\\ 35.6643  85630/2401
\\
\\ trees root=1 total
\\ n = 1  2   3   4    5      6       7
\\     1, 2, 10, 83, 976, 14957, 283732,
\\ A101878 -LambertW(LambertW(-x))/x
\\         sum(k=0,n, binomial(n,k) * (k+1)^k * (n+1)^(n-k-1))
\\ Vaclav Kotesovec
\\        a(n) -> exp(1+1/e) / sqrt(1-1/e) * exp(n/e) * n^(n-1)
\\ my(n=7); exp(1+exp(-1)+n*exp(-1)) / sqrt(1-exp(-1)) * n^(n-1)
\\ my(n=7,e=exp(1)); exp(1+1/e) / sqrt(1-1/e) * exp(n/e) * n^(n-1) / vpar_count(n,-1)
\\ mean
\\ 1.0000  1
\\ 2.0000  2
\\ 3.3333  10/3
\\ 5.1875  83/16
\\ 7.8080  976/125
\\ 11.5409  14957/1296
\\ 16.8818  283732/16807
\\
\\ forests total
\\ not in OEIS: 1, 6, 51, 588, 8705, 158838,
\\ mean
\\ 1.0000  1
\\ 2.0000  2
\\ 3.1875  51/16
\\ 4.7040  588/125
\\ 6.7168  8705/1296
\\ 9.4507  158838/16807

{
  my(forest=-1, \\ trees or forests
     root=1);
  for(n=1,8,
     my(count=0,
        total=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec));
           count++;
           total += vecsum(vpar_spanfreqs(vpar,root)));
      print(total",");
     \\ printf("%.4f  %s\n", total/count*1.0, total/count);
  );
  quit;
}

try(vpar) =
{
  count++;
  total += vecsum(vpar_spanfreqs(vpar));
}
{
  for(n=1,20,
     count=0;
     total=0;
     read(Str("/tmp/trees-"n".gp"));
     \\ print(total",");
     printf("%.4f  %s\n", total/count*1.0, total/count);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_spanning_freqpoly() same

\\ classes
\\ not in OEIS: 1, 4, 5, 12, 32, 62, 139, 298, 649,

\\ n=16 two equivalence classes of 3
\\ same [16, 15, 19, 29, 46, 68, 92, 113, 124, 120, 101, 72, 42, 19, 6, 1]
\\   [0, 1, 2, 3, 4, 5, 3, 7, 3, 2, 10, 1, 12, 13, 14, 1]   not bone
\\   [0, 1, 2, 3, 4, 5, 3, 7, 2, 2, 1, 11, 12, 13, 1, 15]
\\   [0, 1, 2, 3, 4, 3, 2, 7, 8, 2, 10, 1, 12, 13, 14, 12]
\\
\\ same [16, 15, 20, 27, 38, 52, 68, 85, 101, 108, 100, 78, 49, 23, 7, 1]
\\   [0, 1, 2, 3, 4, 5, 5, 3, 8, 3, 1, 11, 12, 13, 11, 1]
\\   [0, 1, 2, 3, 4, 5, 5, 3, 8, 1, 10, 11, 12, 10, 1, 1]
\\   [0, 1, 2, 3, 4, 4, 2, 1, 8, 9, 10, 9, 9, 8, 14, 15]


try(vpar) =
{
  count++;
  my(poly = vpar_spanning_freqpoly(vpar),
     v=[]);
  mapisdefined(m,poly,&v);
  v = concat(v,[vpar]);
  \\ print(v);
  mapput(m,poly,v);
}
{
  for(n=19,20,
     m=Map();
     count=0;
     read(Str("/tmp/trees-"n".gp"));
     m=Mat(m);
     my(len=matsize(m)[1]);
     \\ print("n="n" distinct "len" of "count" trees");
     my(max_same=0,
        printed=0,
        classes=0);
     for(i=1,len,
        my(v=m[i,2]);
        max_same=max(max_same,#v);
        \\ print(v);
        if(#v >= 2, classes++);
        if(#v >= 4 || (n==9 && #v>=2),
           if(printed++==5,
              print("and more");
              ,
              printed<=5,
              print("same "Vecrev(m[i,1]\'x));
              for(i=1,#v, print("  "v[i]));
             )));
     \\ print("classes "classes" max same "max_same);
     \\ print(classes",");
     print(max_same",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_bandwidth()
\\
\\ low = 1 path only
\\ high = floor(n/2) star and star arm length=2
\\
\\ total trees, multiply n from root=1
\\ not in OEIS: 0, 2, 9, 80, 950, 13428,
\\          n = 1  2  3   4    5      6
\\ mean trees
\\ 0.0000  0
\\ 1.0000  1
\\ 1.0000  1
\\ 1.2500  5/4
\\ 1.5200  38/25
\\ 1.7269  373/216
\\
\\ total trees root=1
\\ not in OEIS: 0, 1, 3, 20, 190, 2238, 31311,
\\          n = 1  2  3   4    5      6     7
\\ total trees root=1
\\ 0.0000  0
\\ 1.0000  1
\\ 1.0000  1
\\ 1.2500  5/4
\\ 1.5200  38/25
\\ 1.7269  373/216
\\ 1.8630  639/343
\\
\\ total forests
\\ not in OEIS: 0, 2, 15, 140, 1700, 25128,
\\ mean forests
\\ 0.0000  0
\\ 0.6667  2/3
\\ 0.9375  15/16
\\ 1.1200  28/25
\\ 1.3117  425/324
\\ 1.4951  25128/16807

{
  my(forest=0); \\ trees or forests
  for(n=1,8,
     my(lo=n+1,
        lo_reps=0,
        hi=0,
        hi_reps=0,
        total=0,
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              b=vpar_bandwidth(vpar));
           if(b<lo, lo=b;lo_reps=1,
              b==lo, lo_reps++);
           if(b>hi, hi=b;hi_reps=1,
              b==hi, hi_reps++);
           count++;
           total += b;
     );
     \\ print("n="n" low "lo" reps "lo_reps"  high "hi" reps "hi_reps"  total "total);
     printf("%.4f  %s\n", total/count*1.0, total/count);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_num_singletons() >= 1

{
  my(forest=1); \\ trees or forests
  for(n=4,4,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(f);
           for(i=1,#vec-1, if(vec[i]==0,f=i;break()));
           f || next();
           for(i=1,f-1, if(vec[i]==0 || vec[i]==f, next()));
           for(i=f+1,#vec-1, if(vec[i]==f, next()));
           print(vec);
           count++);
     print(count));
}
{
  my(forest=1); \\ trees or forests
  for(n=4,4,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              singletons = vpar_num_singletons(vpar));
           if(singletons >= 1,
              print(vec);
              count++);
     );
     print(count));
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_num_singletons() total

\\ total singletons in all forests of n vertices
\\ 0, 1, 2, 9, 64, 625, 7776, 117649,
\\ A152917 = A000169 with initial 0
\\ A000169 = n^(n-1) trees

\\ num forests with >=1 singleton
\\              0, 1, 1, 7, 49, 491, 6181, 94305,
\\ not in OEIS:          7, 49, 491, 6181, 94305,
\\
\\ code one 0 entry, rest not equal to it
\\ first 0 at f, before not 0,f so n-1
\\               rest not f so n
\\ vector(10,n, sum(f=1,n-1, (n-1)^(f-1)*(n)^(n-1-f)))

\\ num forests with 0 singletons
\\ 1, 0, 2, 9, 76, 805, 10626,
\\ A105785 without isolated vertices
\\ Sum_{k=0..n} (-1)^(n-k)*binomial(n, k)*(k+1)^(k-1)
\\ Sum_{j=1..n-1} binomial(n-1,j) (j+1)^j a(n-1-j)

\\ num forests with =1 singleton
\\ 0, 1, 0, 6, 36, 380, 4830, 74382,
\\ A277474 egf with Lambert W
\\ 5*76 == 380
\\ n*forests_no_singletons(n-1), by choose the singleton rest none

\\ gS(x) = exp(gT(x)-x) = exp(gT(x))*exp(-x)
\\ gS'(x) = exactly 1 singleton
\\        = exp(gT(x))*exp'(-x) + exp'(gT(x))*exp(-x)
\\        = exp(gT(x))*-exp(-x) + exp'(gT(x))*exp(-x)
\\ total = suminf(s=0, s * exp'(s)(gT(x)-x))
\\ exp(-x)' == -exp(-x)
\\
\\ total = sum(s=0,n, s*binomial(n,s)*S(n-s))
\\       = sum(s=0,n, s*binomial(n,s)*sum(k=0,n-s, (-1)^(n-s-k)*binomial(n-s,k)*vpar_count(k,1)))
\\ k+1 up to n-s
\\ n-s up to n
\\       = sum(s=0,n, sum(k=0,n-s, s*(-1)^(n-s-k)*binomial(n,k)*vpar_count(k,1)))
\\ gtotal = x*exp(x) * gS(x)
\\        = x*exp(x) * exp(gT(x)) * exp(-x)
\\        = x*exp(gT(x))
\\        = x*gF(x)

\\ forests (n+1)^(n-1) having S(n) singletons
\\ S(n) = n*S(n-1)         \\ new not singleton
\\      + n * (n^(n-2)     \\ new singleton * count forests n-1
\\             + S(n-1))   \\ new singleton those in rest
\\
\\ p=1+x+x^2 + O(x^4)
\\ q=1+5*x+6*x^2 + O(x^4)
\\ p'
\\ q'
\\ (p*q)' == p*q' + q*p'

Sp(n) = if(n==0,0, vpar_count(n));
S(n) = if(n==0,0, n==1,1, Sp(n-1)-1 + vpar_count(n-1,1) + Sp(n-1));
vector(10,n,S(n))
vector(10,n,Sp(n))
vector(10,n, sum(s=0,n, s*binomial(n,s)*vpar_count_no_singletons(n-s)))

\\ n=3 degrees
\\     [0, 0, 0],   \ new singleton in count plus total of n-1
\\     [0, 1, 1],   /
\\     [1, 0, 1],   \
\\     [1, 1, 0],   /
\\     [1, 1, 2],
\\     [1, 2, 1],
\\     [2, 1, 1]
\\ n=2 degrees
\\     [0, 0]
\\     [1, 1]
\\     [1, 1]
\\ vpar_count(2,1) == 3
\\ vpar_count(3,1) == 16

{
  my(forest=1); \\ trees or forests
  for(n=0,8,
     my(total=0,
        count_any=0,
        count_one=0,
        count_none=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              singletons = vpar_num_singletons(vpar));
           total += singletons;
           count_any += (singletons!=0);
           count_one += (singletons==1);
           count_none += (singletons==0);
     );
     \\print(total",");
     print(count_one",");
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_degrees_vector() forests count recurrence

\\ vecsum(non-zeros) - 2*num_nonzeros <= -2
\\
\\ s+d-2 + (n-1)*-1 <= -2
\\ d <= n-s-1
\\ remaining s + -1*n <= -2 is
F(n,s) =
{
  if(n==0,
     s%2==0 && s<=-2,
     sum(d=1,n-s-1, F(n-1,s+d-2)));
}
F = memoize(F);
F(3,0)
vector(10,n,n--; 1 + sum(nz=1,n, binomial(n,nz) * F(nz,0)))

vector(10,n,n--; vpar_degrees_vectors_count(n,1))


F_non_zeros(m) = sum(c=0,floor(m/2)-1, binomial(2*m-3-2*c, m-1));
vector(10,m,m++; F_non_zeros(m))
\\
\\ 1, 3, 11, 40, 148, 553, 2083, 7896, 30086, 115126
\\ A014301 internal nodes even outdegree in ordered rooted trees with n edges


quit


\\------------------------------------------------------------------------------
\\ vpar_degrees_vector() forests count recurrence

\\ want vecsum(non-zeros) <= 2*num_nonzeros - 2
\\      vecsum(non-zeros) - 2*num_nonzeros <= -2
\\ s = sum non-zeros
F(n,nz,s) =
{
  \\ print("F("n","nz","s")");
  if(nz==0, s==0||error());
  \\ if(nz!=0, s!=0||error());

  if(n==0,
     if(nz==0, return(1));
     if(s%2,
        \\ print(" odd, none");
        return(0));
     if(s<=2*nz-2, return(1));
     \\ print(" too big");
     return(0));

  F(n-1,nz,s)     \\ first degree=0
  + sum(d=1,n+nz-1,  \\ other first degree
        my(t=F(n-1,nz+1,s+d));
        \\ print(" d="d" term "t);
        t);
}
F = memoize(F);
check_equal(F(0,0,0), 1);

check_equal(F(1,0,0), 1);   \\ [0] only

check_equal(F(2,0,0), 2);   \\ 0,0 or 1,1

\\ check_equal(F(2,2,2), 1);
\\ check_equal(F(0,2,3), 0);    \\ 1,2 or 2,1
\\ check_equal(F(2,2,4), 0);
\\ check_equal(F(2,2,5), 0);
\\ check_equal(F(2,2,6), 0);
\\ check_equal(F(2,2,7), 0);

check_equal(F(3,0,0), 7);
\\check_equal(F(4,0,0), 0);
\\ check_equal(F(3,1,1), 0);   \\ 1,_,_,_

\\check_equal(F(2,1,1), 0);
vector(10,n,n--; F(n,0,0))
quit


\\------------------------------------------------------------------------------
\\ vpar_degrees_vector()

\\ d>=3 attached is sd increase by d-1
\\ d==1 is sd decrease
\\ joining all d>=3 is all their d-1
\\ sum d-2 cf 2n-2 becomes <= -2

\\ forests sorted degrees vectors
\\              1, 1, 2, 3, 6, 10, 18, 29, 48
\\ not in OEIS:          3, 6, 10, 18, 29, 48

is_forest_degrees_vector(vec) =
{
  \\ if(#vec==0, return(1));
  \\ if(!( vecsum(vec) <= 2*#vec - 2), return(0));
  vec = select(d->d>0, vec);
  if(#vec==0, return(1));
  if(!( vecsum(vec) <= 2*#vec - 2), return(0));
  1;
}

{
  my(forest=1); \\ trees or forests
  for(n=0,8,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              degrees=vpar_degrees_vector(vpar));
           degrees=vecsort(degrees);
           is_forest_degrees_vector(degrees) || error(degrees);
           if(mapisdefined(seen,degrees),next());
           mapput(seen,degrees,1);
           count++);
     my(l=Vec(Mat(seen)[,1]));
     l=vecsort(l);
     \\ print(Vec(l));
     \\ if(n==3,print(l));
     \\ for(i=1,#l, print(l[i]));
     print(count"  "vpar_num_children_vectors_count(n,-1)"  "vpar_num_children_vectors_count(n)"  "vpar_num_children_vectors_count(n,1));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ equi independence
\\ 3,6,11,23,47
\\ maximals = maximums only on corona of singletons ...

is_maximal_indset(vpar,set) =
{
  if(!vpar_is_indset(vpar,set), return(0));
  for(v=1,#vpar,
     if(setsearch(set,v),next());
     my(new_set=setunion(set,[v]));
     if(vpar_is_indset(vpar,new_set), return(0)));
  1;
}
maximal_indsets(vpar) = \
  select(set->is_maximal_indset(vpar,set), vpar_indsets(vpar));
maximal_indsets_count(vpar) = #maximal_indsets(vpar);

try(vpar) =
{
  my(indnum_and_count = vpar_indnum_and_count(vpar),
     maximals = maximal_indsets_count(vpar));

  if(maximals == indnum_and_count[2],
     count++;
     if(#vpar==10,
        \\ print(vpar,"     ",vpar_matchnum_and_count(vpar)," ",vpar_maximal_matchings_count(vpar));
         vpar_view(vpar)
       );
    );
}
{
  for(n=1,22,
     count=0;
     read(Str("/tmp/trees-"n".gp"));
     print("n="n" count "count);
     \\ print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ equi domination
\\ 3,6,11,23,47
\\ minimals = minimums only on corona of singletons ...

try(vpar) =
{
  my(domnum_and_count = vpar_domnum_and_count(vpar),
     minimals = vpar_minimal_domsets_count(vpar));

  if(#vpar==3,
     print(vpar"   "domnum_and_count" "minimals);
     print(vpar_domnum_sets(vpar));
     print(vpar_minimal_domsets(vpar));
    );

  if(minimals == domnum_and_count[2],
     count++;
     if(7 <= #vpar && #vpar<=7,
        \\ print(vpar,"     ",vpar_matchnum_and_count(vpar)," ",vpar_maximal_matchings_count(vpar));
        \\ vpar_view(vpar)
       );
    );
}
{
  for(n=1,22,
     count=0;
     read(Str("/tmp/trees-"n".gp"));
     print("n="n" count "count);
     \\ print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_count_height_le()

vector(7,n,vpar_count(n,1))
\\ vpar_count_height_le = memoize(vpar_count_height_le);
print(vpar_count_height_le(6,3,1)" - "vpar_count_height_le(6,2,1)" = "vpar_count_height_le(6,3,1) - vpar_count_height_le(6,2,1));
print("got "vpar_count_height(6,3,1));
quit
for(n=0,8, print1("n="n"  "); for(h=-1,n, print1(vpar_count_height(n,h),", "));print());

x(n,h) =
{
  if(n<0 || h<-1, return(0));
  if(h==-1,return(n==0));
  if(n==0,return(1));

  h=min(h+1,n);
  my(m=matrix(2,n,v,i, 1),
     v=1,p=2);

  for(j=1,h,
     print("col "m[p,]" "v" "p);
     for(i=j,n,
        m[v,i] = sum(t=1,i,
                     \\ print(" t="t" "t * binomial(i-1,t-1)" "t-1" "i-t);
                     t * binomial(i-1,t-1)
                     * if(t<=1,1,m[p,t-1])
                     * if(t==i,1,if(i<=t,1,m[v,i-t]))));
     v=3-v; p=3-p;
     m[v,j]=m[p,j]);
  m[p,n];
}
\\x(7,4)
quit


\\------------------------------------------------------------------------------
\\ vpar_height() histogram

\\ trees height = h
\\ A034855 triangle n,d, number of labelled rooted trees with n>=2 nodes and
\\ height d>=1
\\ 1,
\\ 0,2,
\\ 0,3,6,
\\ 0,4,36,24,
\\ 0,5,200,300,120,
\\ 0,6,1170,3360,2520,720,
\\ A034855

\\ forests
\\ 1,
\\ 1,2,
\\ 1,9,6,
\\ 1,40,60,24,
\\ 1,195,560,420,120,
\\ 1,1056,5550,6240,3240,720,
\\ A235595 Triangle read by rows: the triangle in A034855, with the n-th row normalized by dividing it by n.

\\ forests height = 1, column of table
\\ 2,9,40,195,1056
\\ A235596 Second column of triangle in A235595
\\ Bell number n, ways to partition n into non-empty subsets

\\ trees height <= h
\\ 1,
\\ 0,2,
\\ 0,3,9,
\\ 0,4,40,64,
\\ 0,5,205,505,625,
\\ 0,6,1176,4536,7056,7776,
\\ 0,7,7399,46249,89929,112609,117649,
\\ A236396

\\ forests height <= h
\\ 1,
\\ 1,3,
\\ 1,10,16,
\\ 1,41,101,125,
\\ 1,196,756,1176,1296,
\\ 1,1057,6607,12847,16087,16807,
\\ A210725

A210725(n,h) =     \\ forests <= h
{
  if(n<0,error("n=",n));
  if(h<0,error("h=",h));
  if(min(n,h)==0, return(1));
  sum(j=1,n, binomial(n-1,j-1)*j*A210725(j-1,h-1)*A210725(n-j,h));
}
\\ for(n=1,7, for(h=0,n-1, print1(A210725(n,h),", "));print());

A235595(n,h) =    \\ forests == h
{
  if(n<0,error("n=",n));
  if(h<0,error("h=",h));
  if(min(n,h)==0, return(1));
  A210725(n,h) - A210725(n,h-1);
}
\\for(n=1,7, for(h=0,n-1, print1(A235595(n,h),", "));print());

A034855(n,h) =    \\ trees height = h
{
  if(h==0, n==1,
     h>=n, 0,
     A235595(n-1,h-1) * n);
}
for(n=0,6, for(h=0,max(0,n-1), print1(A034855(n,h),", "));print());

\\ A034855(1,0)
\\ A034855(2,1)

\\ forests all components height = h
E(n,h) = A034855(n,h) + sum(t=1,n-1, binomial(n-1,t-1)* A034855(t,h) * E(n-t,h));
for(n=1,7, print1("n="n"  ");for(h=0,n-1, print1(E(n,h),", "));print());

check_equal(binomial(2,0), 1);
check_equal(A034855(1,1), 0);
check_equal(A034855(2,1), 2);
check_equal(E(2,2), 0);
Eany(n) = sum(h=0,n,E(n,h));
vector(7,n,Eany(n))

\\ T(n,h) = if(min(n,h)==0, 1, \
\\   sum(j=1,n, binomial(n-1,j-1)*j*T(j-1,h-1)*T(n-j,h)));
quit

b(n,h) = if(min(n,h)==0, 1, \
  sum(j=1,n, binomial(n-1, j-1)*j*b(j-1, h-1)*b(n-j, h)));
for(n=1,7, for(k=0,n, print1(b(n,k),", "));print());

T(n,k) = b(n-1,k-1) - b(n-1,k-2);
\\ for(n=1,7, for(k=0,n, print1(T(n,k),", "));print());

\\ for(n=1,7, for(k=0,n-1, print1(binomial(n,k) * (n-k)^(n-k-1)*k^k,", "));print());

\\ Riordan
\\ S(n,h) =
\\ {
\\   if(n==1,1, 1/(n-1) * sum(k=0,n, k*binomial(n,k) * S(n-k,h) * S(k,h-1)));
\\ }
\\ for(n=1,7, for(k=0,n-1, print1(S(n,h)));print());

{
  \\ counts of <= height h
  for(n=1,8,
     my(histogram=vector(n));
     forvec(vec=vpar_forvec_from(n,0), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           for(h=vpar_height(vpar)+1,#histogram, histogram[h]++));
     for(i=1,#histogram, print1(histogram[i]",")); print();
     \\ for(i=1,#histogram-1, print1(histogram[i]/histogram[i+1]", ")); print();
     );
  quit;
}
{
  for(n=1,8,
     my(histogram=vector(n));
     forvec(vec=vpar_forvec_from(n,1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           histogram[vpar_height(vpar)+1]++);
     for(i=1,#histogram, print1(histogram[i]",")); print();
     \\ for(i=1,#histogram-1, print1(histogram[i]/histogram[i+1]", ")); print();
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_component_heights() increasing

\\ labelled forests of increasing component heights
\\ not in OEIS: 1, 3, 13, 99, 961, 12063, 183793,

\\ labelled forests of component heights all same
\\ not in OEIS: 1, 3, 10, 77, 686, 8467, 127100,

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests
           my(vpar=vpar_from_upascend(vec),
              component_heights = vpar_component_heights(vpar));
           count += (component_heights == vecsort(component_heights));
           \\ count += (vecmin(component_heights) == vecmax(component_heights));

           \\ if(#vpar==4 && vecmin(component_heights) == vecmax(component_heights),
           \\    print(vpar));
           );
     print(count",");
  );
}
quit;


\\------------------------------------------------------------------------------
\\ vpar_component_sizes() occurring

\\ 2^n
{
  for(n=0,6,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,1), \\ forests
           my(vpar=vpar_from_upascend(vec),
              component_sizes=vpar_component_sizes(vpar));
           if(mapisdefined(seen,component_sizes),next());
           mapput(seen,component_sizes,1);
           count++);
     my(l=Mat(seen)[,1]);
     l=vecsort(l);
     \\ for(i=1,#l, print(l[i]));
     print(count",");
     check_equal(#l,2^max(0,n-1));
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_component_heights() occurring

\\ number of vectors occurring
\\     1, 2, 5, 12, 27, 58, 121,
\\ n = 1  2  3   4   5
\\ A000325  2^n - n

\\ C(5) = h=0 s=1 + C(4)    12
\\        h=1 s=2 + C(3)     5
\\        h=2 s=3 + C(2)     2
\\        h=3 s=4 + C(1)     1
\\        h=4 s=5            1  path
\\        h=1 e=1 s=3 + C(2)     2
\\        h=2 e=1 s=4 + C(1)     1
\\        h=3 e=1 s=3            1 single
\\        h=1 e=2 s=4 + C(1)     1
\\        h=2 e=2 s=5            1 single
\\        h=1 e=3 s=5            1 single
\\ 12+5+2+1+1 + 2+1+1 + 1+1 + 1 == 28

{
  for(n=1,5,
     my(count=0,
        seen=Map());
     forvec(vec=vpar_forvec_from(n,1), \\ forests
           my(vpar=vpar_from_upascend(vec),
              component_heights=vpar_component_heights(vpar));
           if(mapisdefined(seen,component_heights),next());
           mapput(seen,component_heights,1);
           count++);
     my(l=Mat(seen)[,1]);
     l=vecsort(l);
     for(i=1,#l, print(l[i]));
     print(count",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_height_num_vertices() totals or how many with certain num

\\ labelled forests height count 1
\\ not in OEIS: 1, 2, 12, 84, 860, 11070,
\\ labelled trees height count 1
\\ not in OEIS: 1, 2, 6, 48, 420, 5160, 77490,
\\ labelled root=1 height count 1   is 1 later than forest
\\              1, 1, 2, 12, 84, 860, 11070, 170562
\\ not in OEIS:       2, 12, 84, 860, 11070, 170562

\\ labelled forests height count = 2
\\ not in OEIS: 0, 1, 3, 36, 330, 4200, 66885,
\\ labelled trees height count = 2
\\ not in OEIS: 0, 0, 3, 12, 180, 1980, 29400,
\\ labelled root=1 height count = 2  is 1 later than forest
\\ not in OEIS: 0, 0, 1, 3, 36, 330, 4200, 66885,

\\ labelled forests height count >=2
\\ not in OEIS: 0, 1, 4, 41, 436, 5737,
\\ labelled trees height count >=2
\\ not in OEIS: 0, 0, 3, 16, 205, 2616, 40159,
\\ labelled root=1 height count >=2
\\ not in OEIS: 0, 0, 1, 4, 41, 436, 5737,

\\ labelled forests total vertices at height
\\ not in OEIS: 1, 4, 21, 172, 1845, 24366, 384811,
\\ labelled trees total vertices at height
\\ not in OEIS: 1, 2, 12, 84, 860, 11070, 170562,
\\ labelled root=1 total vertices at height  is 1 later than forest
\\ not in OEIS: 1, 1, 4, 21, 172, 1845, 24366, 384811

\\ transform to make forests out of trees would be one or more tree of the
\\ same height, then rest anything shorter

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           \\ count += (vpar_height_num_vertices(vpar) == 2);
           count += vpar_height_num_vertices(vpar);
     );
     print(count",");
  );
  quit;
}

\\ unlabelled height count 1
\\              1, 1, 1, 2, 4, 9, 21, 51, 126, 318, 814, 2111
\\ not in OEIS:          2, 4, 9, 21, 51, 126, 318, 814, 2111

\\ unlabelled height count 2
\\              0, 0, 1, 1, 3, 6, 14, 33, 82, 204, 520, 1337
\\ not in OEIS:             3, 6, 14, 33, 82, 204, 520, 1337

\\ unlabelled height count >=2
\\              0, 0, 1, 2, 5, 11, 27, 64, 160, 401, 1028, 2655
\\ not in OEIS:          2, 5, 11, 27, 64, 160, 401, 1028, 2655

\\ unlabelled total num vertices at height
\\              1, 1, 3, 7, 17, 39, 97, 234, 590, 1489, 3837, 9950
\\ not in OEIS:       3, 7, 17, 39, 97, 234, 590, 1489, 3837, 9950

{
  for(n=1,12,
     my(depths=vpar_preorder_depths_first(n),
        count=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths));
           \\ count += (vpar_height_num_vertices(vpar) >= 2);
           count += vpar_height_num_vertices(vpar);
          );
     print(count);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ eccentricity histogram palindromic

{
  \\ how many, up to free Isomorphism
  \\ n          = 2     4     6  7  8  9 10 11  12  13  14  15
  \\ not in OEIS: 1, 0, 1, 0, 1, 1, 3, 2, 7, 4, 22, 15, 78, 77,

  my(rel=Map(),
     l=List([]));
  for(n=2,16,
     my(count=0,
        depths=vpar_preorder_depths_first(n));
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              eccs=vpar_eccentricities_vector(vpar),
              hist=Vec(vpar_INTERNAL_histogram(eccs)));
           hist=hist[vecmin(eccs),vecmax(eccs)];
           if(hist==Vecrev(hist),
              for(i=1,#l, if(vpar_is_free_isomorphic(l[i],vpar),next(2)));
              listput(l,vpar);

              my(centres=vpar_centres(vpar),
                 num_centres=#centres);
              \\ if(#centres==2,next());
              \\ print("n="n"  "hist"  centres "num_centres
              \\       " radius "vpar_radius(vpar)" diam "vpar_diameter(vpar));
              \\ print("    "vpar);
             \\ vpar_view(vpar,1);
              count++;
             ));
     print(count",");
     \\ print();
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_indnum_sets() maximum independent sets, vertex in common

\\ never =1 vertex common ... ?

\\ free tree fewest vertices common to its indnum sets,
\\ is 0 if n even, 2 if n odd
\\              0, 1, 0, 1, 1, 2, 3, 5, 10, 15, 39, 49, 160, 180, 702, 701,3177,
\\ not in OEIS:                2, 3, 5, 10, 15, 39, 49, 160, 180, 702, 701,3177,
\\ even terms are =0 vertices common
\\ 0, 1, 1, 2, 5, 15, 49, 180, 701, 2891,
\\          2, 5, 15, 49, 180, 701, 2891,
\\ A005751 matched trees with 2n vertices, ie. has a perfect matching
\\
\\ odd terms are =2 vertices common
\\ not in OEIS: 0, 0, 1, 0, 1, 0, 3, 0, 10, 0, 39, 0, 160, 0, 702,
\\              0, 1, 1, 3, 10, 39, 160, 702, 3177,
\\                       3, 10, 39, 160, 702, 3177,
\\ A005750 planted matched trees n nodes

\\ free tree most vertices common to its indnum sets
\\ indnum max is n-1 for star-n
\\ num trees with this many in common
\\              1, 1, 1, 2, 3, 5, 8, 11, 17, 26, 43, 70, 119, 199, 340, 574,
\\ not in OEIS:          2, 3, 5, 8, 11, 17, 26, 43, 70, 119, 199, 340, 574,

\\ Indsets (all such) never have a vertex in common because at a leaf can
\\ choose the leaf or its attachment and continue from there.

try(vpar) =
{
  my(sets=vpar_indnum_sets(vpar),
     cset=test_setintersect_sets(sets),
     c=#cset);
  if(c>max_common, max_common=c; max_common_reps++,
     c==max_common, max_common_reps++);
  if(#sets>1,
     if(c<min_common, min_common=c; min_common_reps++,
        c==min_common, min_common_reps++));
  if(c==0, no_common_reps++);
  if(c==1, one_common_reps++);
  if(c==2, two_common_reps++);

  \\ if(#vpar <= 6, print(vpar"  "sets));
  \\ if(#vpar <= 9 && c==1, print(vpar" common "cset" sets "sets"  "has_one_star_tail(vpar)));
  \\ if(#vpar == 9 && c==1, print(vpar" common "cset" sets "sets); vpar_view(vpar));
}
{
  for(n=1,30,
     max_common = 0;
     max_common_reps = 0;
     min_common = n+1;
     min_common_reps = 0;
     no_common_reps = 0;
     one_common_reps = 0;
     two_common_reps = 0;
     read(Str("/tmp/trees-"n".gp"));
     if(min_common==n+1, min_common='none);

     \\ print("n="n" max_common "max_common" reps "max_common_reps"   min_common "min_common" reps "min_common_reps);
     \\ print(one_common_reps",");
     print(two_common_reps",");
     \\ print(min_common_reps",");
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_indomnum_sets(), vertex in common

\\ free trees with its indomnum sets having =1 vertex common
\\              1, 0, 1, 1, 1, 2, 2, 6, 9, 20, 41, 84, 199, 420, 974, 2216,
\\ not in OEIS:                      6, 9, 20, 41, 84, 199, 420, 974, 2216,
\\
\\ Something where second from a leaf is forced present, and rest have
\\ choices ...

\\ free tree fewest vertices common to its indomnum sets, is 0 common
\\          n = 1
\\              none, 0, none, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
\\ num trees with none in common
\\              0, 1, 0, 1, 1, 2, 3, 6, 9, 20, 36, 76, 147, 328, 675, 1501,
\\ not in OEIS:                2, 3, 6, 9, 20, 36, 76, 147, 328, 675, 1501,

\\ free tree most vertices common to its indomnum sets
\\          n = 1           5     6     8    10    11    12    13
\\              1, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8,
\\ num trees with this many in common
\\              1, 1, 1, 2, 1, 2, 4, 7, 16, 18, 27, 39, 74, 126, 247, 386,
\\ not in OEIS:                   4, 7, 16, 18, 27, 39, 74, 126, 247, 386,

\\ Indomsets (all such) never have a vertex in common because at a leaf can
\\ choose the leaf or its attachment and continue from there.

has_one_star_tail(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar),
     count=0);
  for(v=1,#vpar,
     if(degrees[v]<3,next());
     my(neighbours=vpar_neighbours_of_vertex(vpar,v),
        non_one=0);
     for(i=1,#neighbours,
        non_one += degrees[neighbours[i]]!=1;
        if(non_one>=2, next(2)));
     count++;
     if(count>=2,return(0)));
  count==1;
}

try(vpar) =
{
  my(sets=vpar_indomnum_sets(vpar),
     cset=test_setintersect_sets(sets),
     c=#cset);
  if(c>max_common, max_common=c; max_common_reps++,
     c==max_common, max_common_reps++);
  if(#sets>1,
     if(c<min_common, min_common=c; min_common_reps++,
        c==min_common, min_common_reps++));
  if(c==0, no_common_reps++);
  if(c==1, one_common_reps++);

  \\ if(#vpar <= 6, print(vpar"  "sets));
  if(#vpar <= 9 && c==1, print(vpar" common "cset" sets "sets"  "has_one_star_tail(vpar)));
  \\ if(#vpar == 9 && c==1, print(vpar" common "cset" sets "sets); vpar_view(vpar));
}
{
  for(n=1,30,
     max_common = 0;
     max_common_reps = 0;
     min_common = n+1;
     min_common_reps = 0;
     no_common_reps = 0;
     one_common_reps = 0;
     read(Str("/tmp/trees-"n".gp"));
     if(min_common==n+1, min_common='none);

     \\ print("n="n" max_common "max_common" reps "max_common_reps"   min_common "min_common" reps "min_common_reps);
     \\ print("  one_common reps "one_common_reps);

     \\ print(one_common_reps",");
     print(one_common_reps",");
     \\ print(min_common",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_totdomnum_quantities()

{
  \\ with_dom <  with_undom                             [0, 1, 2, 3, 2, 5]
  \\ with_dom == with_undom                             path-4
  \\ with_dom >  with_undom                             path-2
  \\
  \\ with_dom <               without_dom               [0,1,2,1,4] path-5 mid
  \\ with_dom ==              without_dom               path-3
  \\ with_dom >               without_dom               path-4
  \\
  \\ with_dom <                     without_undom       [0, 1, 2, 3, 2, 5, 2, 7]
  \\ with_dom ==                    without_undom       [0, 1, 2, 3, 2, 5]
  \\ with_dom >                     without_undom       path-4
  \\
  \\            with_undom <  without_dom               path-6
  \\            with_undom == without_dom               path-5
  \\            with_undom >  without_dom               path-4
  \\
  \\            with_undom <        without_undom       path-11 middle
  \\            with_undom ==       without_undom       path-6
  \\            with_undom >        without_undom       singleton
  \\
  \\                 without_dom <  without_undom       [0, 1, 2, 3, 2, 5]
  \\                 without_dom == without_undom       path-4
  \\                 without_dom >  without_undom       path-5
  \\

  my(rel=Map());
  for(n=1,16,
     my(depths=vpar_preorder_depths_first(n));
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              quantities=vpar_totdomnum_quantities(vpar)[^1]);

           for(i=1,#quantities,
              if(quantities[i]<0,next());
              for(j=1,#quantities,
                 if(i==j||quantities[j]<0,next());
                 my(key=[i,j,"<"]);
                 if(!mapisdefined(rel,key)
                    && quantities[i]<quantities[j],
                    mapput(rel,key,1);
                    print(vpar" rel "key));
                 key=[i,j,"=="];
                 if(i<j && !mapisdefined(rel,key)
                    && quantities[i]==quantities[j],
                    mapput(rel,key,1);
                    print(vpar" rel "key))));
           );
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_indomnum_quantities()

{
  \\ with , without_dom , without_undom
  \\
  \\ with <  without_dom                    star-3
  \\ with == without_dom                    path-2
  \\ with >  without_dom                    path-3
  \\
  \\ with <                 without_undom   *--*--*--*--*
  \\                                                  \-*
  \\ with ==                without_undom   path-5
  \\ with >                 without_undom   initial
  \\
  \\         without_dom <  without_undom   top star-4
  \\         without_dom == without_undom   path-3
  \\         without_dom >  without_undom   path-4
  \\
  \\ each either way
  my(rel=Map());
  for(n=1,16,
     my(depths=vpar_preorder_depths_first(n),
        seen=Map());
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              quantities=vpar_indomnum_quantities(vpar)[^1]);

           for(i=1,#quantities,
              if(quantities[i]<0,next());
              for(j=1,#quantities,
                 if(i==j||quantities[j]<0,next());
                 my(key=[i,j,"<"]);
                 if(!mapisdefined(rel,key)
                    && quantities[i]<quantities[j],
                    mapput(rel,key,1);
                    print(vpar" rel "key));
                 key=[i,j,"=="];
                 if(i<j && !mapisdefined(rel,key)
                    && quantities[i]==quantities[j],
                    mapput(rel,key,1);
                    print(vpar" rel "key))));

           quantities=select(s->s>=0,quantities);
           mapput(seen,vecsort(quantities,,1),1);
           );
     seen=Vec(Mat(seen)[,1]);
     seen=apply(v->concat(apply(x->Str(x),Vec(v))), seen);
     seen=vecsort(seen);
     print("n="n" "seen);
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_disjoint_domnum_quantities() different
\\
\\ with_dom = with_undom or without_undom+1, when with_undom exists

vector_is_all_different(v) =
{
  my(m=Map());
  for(i=1,#v, if(mapisdefined(m,v[i]),return(0)); mapput(m,v[i],1));
  1;
}
vector_range(v) =
{
  v=select(x->x>=0,v);
  if(#v, [vecmin(v),vecmax(v)], 'none);
  if(#v, vecmax(v)-vecmin(v), 'none);
}
{
  for(n=1,16,
     my(depths=vpar_preorder_depths_first(n),
        seen=Map());
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
             quantities=vpar_disjoint_domnum_quantities(vpar)[^6]);
           if(mapisdefined(seen,quantities),next());
           mapput(seen,quantities,1);
           print("n="n" "vpar" quantities "quantities" range "vector_range(quantities[1..2]));
           );
     );
  quit;
}
{
  for(n=1,16,
     my(depths=vpar_preorder_depths_first(n),
        end=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
             quantities=vpar_disjoint_domnum_quantities(vpar)[^6]);
           if(\\ vecmin(quantities) >= 1
              \\ &&
              vector_is_all_different(quantities)
              ,
              print("n="n" "vpar" quantities "quantities);
              end=1));
     if(end,break());
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_disjoint_domnum() five quantities

\\ v_dom
\\ recurrence_guess([-1, 3, 6, 14, 27, 54, 110, 219, 438, 878, 1755, 3510, 7022, 14043, 28086, 56174, 112347])

\\ without_undom_undom
\\ A294627
\\ 12,28,56,108,220,440,876,1756,3512,7020
\\ recurrence_guess([0, -1, -1, 12, 28, 56, 108, 220, 440, 876, 1756, 3512, 7020, 14044, 28088, 56172, 112348])

{
for(h=0,9,
   print(factor(vpar_disjoint_domnum_and_count_quantities(vpar_make_complete_tree(2,h))[5,2]),","));
}

quit


\\------------------------------------------------------------------------------
\\ vpar_disjoint_domnum() range

\\ low
\\ n = 2  3  4  5  6  7  8
\\     2, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11,
\\ A004523 floor(2*n/3)
\\ trees count, labelled
\\ not in OEIS: 2, 9, 64, 300, 4320, 86730
\\ trees count, unlabelled (but rooted)
\\ not in OEIS: 1, 2, 4, 3, 8, 28, 9, 41, 175, 36, 211, 1100,

\\ high = n when entirely pairs, maybe
\\ 2 3 4 5 6 7
\\ trees count, labelled
\\ not in OEIS: 2, 9, 64, 325, 3456, 30919
\\ trees count, unlabelled (but rooted)
\\ 1, 2, 4, 6, 12, 20, 40, 72, 144, 274, 550, 1082,
\\ 2*
\\ A007562 planted trees non-root non-leaf at even dist from root degree 2

{
  for(n=8,8,
     my(depths=vpar_preorder_depths_first(n));
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              disjoint_domnum = vpar_disjoint_domnum(vpar));
           if(disjoint_domnum == n,
              vpar_view(vpar,1));
          );
     );
  quit;
}
{
  for(n=2,16,
     my(depths=vpar_preorder_depths_first(n),
        lo=n+1,
        hi=0,
        lo_count = 0,
        hi_count = 0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              disjoint_domnum = vpar_disjoint_domnum(vpar));
           if(disjoint_domnum < lo,  lo = disjoint_domnum; lo_count = 1,
              disjoint_domnum == lo,                       lo_count++);
           if(disjoint_domnum > hi,  hi = disjoint_domnum; hi_count = 1,
              disjoint_domnum == hi,                       hi_count++);
          );
     print(hi_count",");
     );
  quit;
}
{
  for(n=2,8,
     my(lo=n+1,
        hi=0,
        lo_count = 0,
        hi_count = 0);
     forvec(vec=vpar_forvec_from(n,0), \\ trees
           my(vpar=vpar_from_upascend(vec),
              disjoint_domnum = vpar_disjoint_domnum(vpar));
           if(disjoint_domnum < lo,  lo = disjoint_domnum; lo_count = 1,
              disjoint_domnum == lo,                       lo_count++);
           if(disjoint_domnum > hi,  hi = disjoint_domnum; hi_count = 1,
              disjoint_domnum == hi,                       hi_count++);
           );
     print(lo_count);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_2height() == height

\\ trees
\\ count height == 2height
\\              0, 3, 16, 205, 2616, 40159,
\\ not in OEIS:    3, 16, 205, 2616, 40159,
\\ count height != 2height
\\ not in OEIS: 2, 6, 48, 420, 5160,
\\
\\ root=1
\\ count height == 2height
\\ not in OEIS: 0, 1, 4, 41, 436, 5737,
\\ count height != 2height
\\ not in OEIS: 1, 2, 12, 84, 860, 11070,
\\ per forests vpar_height_num_vertices() == 1

{
  for(n=2,8,
     my(count_eq = 0,
        count_ne = 0,
        total = 0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              height = vpar_height(vpar),
              height2 = vpar_2height(vpar),
              eq = (height==height2));
           count_eq += eq;
           count_ne += !eq;
           total++;
     );
     print(count_eq",");
  );
  quit;
}


\\ unlabelled count height == 2height
\\              0, 0, 1, 2, 5, 11, 27, 64, 160, 401, 1028, 2655,
\\ not in OEIS:          2, 5, 11, 27, 64, 160, 401, 1028, 2655,
\\
\\ unlabelled count height != 2height
\\              1, 1, 1, 2, 4, 9, 21, 51, 126, 318, 814, 2111,
\\ not in OEIS:          2, 4, 9, 21, 51, 126, 318, 814, 2111,

{
  for(n=1,12,
     my(depths=vpar_preorder_depths_first(n),
        count_eq = 0,
        count_ne = 0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              height = vpar_height(vpar),
              height2 = vpar_2height(vpar),
              eq = (height==height2));
           count_eq += eq;
           count_ne += !eq;
          );
     print(count_eq",");
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_2height() total

\\ unlabelled total 2height
\\ not in OEIS: 0, 2, 6, 18, 49, 137, 374, 1035, 2857, 7937, 22081

\\ unlabelled total height
\\ 0 1 3 8 22 58 158 425 1161 3175 8751 24192
\\ A001853


{
  for(n=1,12,
     my(depths=vpar_preorder_depths_first(n),
        total=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths));
           total += vpar_2height(vpar));
     print(total);
     );
  quit;
}

\\ forests
\\          n = 2  3    4
\\ not in OEIS: 0, 9, 148, 2195, 36366,
\\
\\ trees
\\ not in OEIS: 0, 9, 100, 1365, 20946, 372211,
\\
\\ trees root=1
\\ not in OEIS: 0, 3, 25, 273, 3491, 53173,
{
  for(n=2,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           total += vpar_2height(vpar));
     print(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_perfect_domnum() maximum

\\ maximum perfect domnum
\\ n= 0 1 2 3 4 5 6
\\    0 1 1 1 2 2 3
\\
\\ number of vpar attaining that
\\ not in OEIS: 1, 1, 2, 9, 48, 600, 2160
\\
\\ number of free trees attaining that
\\          n = 0  1  2  3  4  5  6
\\ not in OEIS: 1, 1, 2, 2, 2, 3, 2
\\
\\ n=0 to 4 path-n
\\                              *
\\ n=5 path and                 |
\\ [0, 1, 1, 1, 2]        *--*--*--*
\\ GP-Test  vpar_perfect_domnum_and_count([0, 1, 1, 1, 2]) == [2,1]
\\
\\ n=6 perfect domnum = 3 in 2 ways
\\ [0, 1, 1, 1, 2, 3]           *
\\         1                    |
\\       / |  \           *--*--*--*--*
\\     2   3   4
\\     |   |
\\     5   6
\\ GP-Test  vpar_perfect_domnum_and_count([0, 1, 1, 1, 2, 3]) == [3,2]
\\
\\ n=7 max=3 count 6 trees
\\ [0, 1, 1, 1, 1, 2, 3]      *   *
\\                             \ /
\\                        *--*--*--*--*
\\ GP-Test  vpar_perfect_domnum_and_count([0, 1, 1, 1, 1, 2, 3]) == [3,1]
\\
\\ [0, 3, 1, 1, 1, 2, 2]    *     *
\\                          |     |
\\                       *--*--*--*--*
\\ GP-Test  vpar_perfect_domnum_and_count([0, 3, 1, 1, 1, 2, 2]) == [3,1]
\\
\\ [2, 0, 2, 1, 1, 2, 3]
\\ GP-Test  vpar_perfect_domnum_and_count([2, 0, 2, 1, 1, 2, 3]) == [3,1]
\\
\\ [0, 1, 4, 1, 1, 2, 3]       *
\\                             |
\\                       *--*--*--*--*--*
\\ GP-Test  vpar_perfect_domnum_and_count([0, 1, 4, 1, 1, 2, 3]) == [3,2]
\\
\\ [2, 0, 2, 2, 1, 3, 4]  subdivided star-4
\\ GP-Test  vpar_perfect_domnum_and_count([2, 0, 2, 2, 1, 3, 4]) == [3,3]
\\
\\ [0, 3, 4, 5, 1, 1, 2]  path-7
\\ GP-Test  vpar_perfect_domnum_and_count([0, 3, 4, 5, 1, 1, 2]) == [3,3]

{
  for(n=0,7,
     my(m=0, count=0,
        seen=List([]));
     forvec(vec=vpar_forvec_from(n,0),  \\ trees any
           my(vpar=vpar_from_upascend(vec),
              perfect_domnum = vpar_perfect_domnum(vpar));
           if(perfect_domnum>=m,
              for(i=1,#seen,
                 if(vpar_is_free_isomorphic(vpar,seen[i]), next(2)));
              listput(seen,vpar));

           if(perfect_domnum>m, m=perfect_domnum;count=1;seen=List([vpar]),
              perfect_domnum==m, count++);
     );
     \\ print(m);
     \\ print(count);

     print("n="n" max="m" count "count);
     count==#seen || error();
     for(i=1,#seen,print(seen[i]));
     print();
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_matchpoly() and vpar_skew_adjacency_matrix()

vpar=[3,3,10,10,9,8,8,9,10, 0]   \\ Knuth 2.3.4.4
p = vpar_matchpoly(vpar)
charpoly(vpar_adjacency_matrix(vpar))
subst(p,'x,-1/x^2)*x^#vpar

charpoly(vpar_skew_adjacency_matrix(vpar))
subst(p,'x,1/x^2)*x^#vpar
quit


\\------------------------------------------------------------------------------
\\ vpar_make_random() root=1

{
  for(i=1,1,
     my(vpar=vpar_make_random(2,-1));
     print(vpar);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_num_children_vector() of root=1

\\ A088218 total leaves in rooted trees
\\ 1 1 1 3 10 35 126 462 1716
\\ a(n) = sum( i=0, n, binomial(n+i-2, i));
\\ vector(10,n,n--; binomial(2*n-3,n-1))

\\ vpar_num_children_vectors_count(6,-1)
\\ vpar_num_children_vectors_count(6,0)
\\ vpar_num_children_vectors_count(5,1)
\\ n=6 root=1 126
\\     all    252
{
  for(n=3,3,
     print("root=1");
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,-1), \\ root=0
           my(vpar=vpar_from_upascend(vec),
              num_children = vpar_num_children_vector(vpar));
           mapput(m,num_children,1));
     print(#m);
     my(r=Mat(m)[,1]);
     for(i=1,#r,print(r[i]));

     print("not root=1");
     my(seen=Map());
     forvec(vec=vpar_forvec_from(n,0),  \\ trees any
           my(vpar=vpar_from_upascend(vec),
              num_children = vpar_num_children_vector(vpar));
           if(mapisdefined(seen,num_children),next());
           mapput(seen,num_children,1);
           if(!mapisdefined(m,num_children) && vpar_root(vpar)!=1,
              print(num_children)));
     print(matsize(Mat(seen))[1]);;
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ minimal domset always invertible

\\ Every v in minimal domset has at least one absent neighbour, otherwise
\\ would not be minimal.
\\
\\ Sometimes the complement is not minimal.
\\ So vpar_minimal_domsets_count() = 2*min&min + min&notmin

\\ inv 0 not 1 minimal inv 0 not 1
\\ inv 4 not 0 minimal inv 4 not 0
\\ inv 18 not 0 minimal inv 18 not 0
\\ inv 224 not 0 minimal inv 224 not 0
\\ inv 2450 not 0 minimal inv 1850 not 600
\\ inv 46152 not 0 minimal inv 35352 not 10800

{
  for(n=1,8,
     my(count_inv=0, count_not=0, count_minimal_inv=0, count_minimal_not=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           forvec(flags=vector(#vpar,i,[0,1]),
                 vpar_is_minimal_domset_flags(vpar,flags) || next();
                 my(inv_flags=apply(f->!f,flags),
                    is_minimal_inv=vpar_is_minimal_domset_flags(vpar,inv_flags),
                    is_inv=vpar_is_domset_flags(vpar, inv_flags));
                 count_inv += is_inv;
                 count_not += !is_inv;
                 count_minimal_inv += is_minimal_inv;
                 count_minimal_not += !is_minimal_inv;
                 if(!is_inv,
                    print(vpar" "flags" "is_inv));
                 ));
     print("inv "count_inv" not "count_not
           " minimal inv "count_minimal_inv" not "count_minimal_not);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ invertible domset not always minimal

\\ vpars up to isomorphism with an invertible but not minimals domset
\\
\\ min 0 not 0 vpars 0
\\ min 4 not 0 vpars 0
\\ min 18 not 0 vpars 0
\\ min 224 not 0 vpars 0
\\
\\ n=5 path, 01101 invertible but not minimal
\\ [0, 3, 1, 1, 2] flags [0, 1, 1, 1, 0] 0
\\ min 2450 not 600 vpars 1
\\
\\ n=6 path 101101 invertible but not minimal
\\ and
\\     5--1--3--2--6     *--.--*--.--*
\\        |                 |
\\        4                 *
\\ [0, 3, 1, 1, 1, 2] flags [0, 1, 1, 1, 1, 0] 0
\\ [0, 3, 4, 1, 1, 2] flags [0, 0, 1, 1, 1, 1] 0
\\ min 46152 not 10800 vpars 2
\\
\\ n=7
\\ [0, 3, 1, 1, 1, 1, 2] flags [0, 1, 1, 1, 1, 1, 0] 0
\\ [0, 3, 1, 1, 1, 2, 2] flags [0, 1, 1, 1, 1, 0, 0] 0
\\ [0, 3, 4, 1, 1, 1, 2] flags [0, 0, 1, 1, 1, 1, 1] 0
\\ [0, 1, 4, 1, 1, 2, 3] flags [0, 0, 1, 1, 1, 1, 0] 0
\\ [2, 0, 2, 2, 1, 3, 4] flags [0, 1, 0, 1, 1, 1, 0] 0
\\ [0, 3, 4, 5, 1, 1, 2] flags [0, 0, 0, 1, 1, 1, 1] 0
\\ min 882098 not 373380 vpars 6
\\
\\ [0, 3, 1, 1, 1, 1, 1, 2] flags [0, 1, 1, 1, 1, 1, 1, 0] 0
\\ [0, 3, 1, 1, 1, 1, 2, 2] flags [0, 1, 1, 1, 1, 1, 0, 0] 0
\\ [0, 3, 4, 1, 1, 1, 1, 2] flags [0, 0, 1, 1, 1, 1, 1, 1] 0
\\ [0, 1, 4, 1, 1, 1, 2, 3] flags [0, 0, 1, 1, 1, 1, 1, 0] 0
\\ [0, 3, 4, 1, 1, 1, 2, 2] flags [0, 0, 1, 1, 1, 1, 1, 1] 0
\\ [2, 0, 4, 2, 1, 1, 2, 3] flags [0, 0, 1, 1, 1, 1, 1, 0] 0

{
  for(n=1,8,
     my(count_min=0, count_not=0);
     my(seen=List([]));
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           forvec(flags=vector(#vpar,i,[0,1]),
                 vpar_is_domset_flags(vpar,flags) || next();
                 vpar_is_domset_flags(vpar, apply(f->!f,flags)) || next();
                 my(is_min=vpar_is_minimal_domset_flags(vpar,flags));
                 count_min += is_min;
                 count_not += !is_min;
                 if(!is_min,
                    my(s=0);
                    for(i=1,#seen,
                       if(vpar_is_free_isomorphic(vpar,seen[i]), s=1;break()));
                    if(!s,
                       listput(seen,vpar);
                       print(vpar" flags "flags" "is_min);
                      ));
                 ));
     print("min "count_min" not "count_not" vpars "#seen);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Domatic Number

\\ trees always 2 by alternate vertices
\\ various count of invertible dominating sets by flipping alternation at non-leaf

\\ Ore, complement of minimal domset is a domset

\\ disjoint domination number, size of two disjoint domsets
\\ global dominating set dominates graph and graph complement

classes_are_domsets(vpar,classes) =
{
  \\ print("classes ",classes);
  for(try_c=vecmin(classes),vecmax(classes),
     my(flags=apply(c->c==try_c, classes));
     \\ print(flags);
     if(!vpar_is_domset_flags(vpar,flags), return(0)));
  1;
}

domatic_number_by_search(vpar) =
{
  my(ret=1);
  for(k=2,4,
     my(found=0);
     forvec(classes=vector(#vpar,i,[1,k]),
           if(vecmin(classes)==1 && vecmax(classes)==k
                && classes_are_domsets(vpar,classes),
              found=1; break()));
     if(!found, return(ret));
     ret=k);
  error();
}

domatic_count_by_search(vpar) =
{
  my(count=0);
  forvec(classes=vector(#vpar,i,[1,2]),
        if(vecmin(classes)==1 && vecmax(classes)==2
           && classes_are_domsets(vpar,classes),
           \\ print(classes);
           count++));
  count;
}
{
  \\ domatic count total
  \\ trees
  \\ not in OEIS:   4, 18, 224, 3050
  \\ not in OEIS:   2, 9, 112, 1525, 28476, 627739
  \\ trees indsets
  \\ not in OEIS: 2, 6, 45, 528, 8525, 175428
  \\ trees domsets
  \\ not in OEIS: 1, 6, 45, 576, 10025, 222048
  \\ trees minimal domsets
  \\ not in OEIS: 1, 4, 18, 224, 2450, 46152

  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           \\ total += domatic_count_by_search(vpar);
           \\ total += vpar_disjoint_domsets_count(vpar);
           total += 2*vpar_disjoint_domsets_count(vpar) - vpar_minimal_domsets_count(vpar);
           \\ total += vpar_minimal_domsets_count(vpar);
     );
     print(total));
  quit;
}
{
  \\ domatic count maximum
  \\ 2,2,4,6,10
  for(n=1,8,
     print("n="n);
     my(m=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           m=max(m,domatic_count_by_search(vpar)));
     print(" "m));
  quit;
}
{
  for(n=1,8,
     print("n="n);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           if(domatic_number_by_search(vpar)==1,
              print(vpar);
              break())));
  quit;
}
{
  for(i=1,1,
     my(vpar=vpar_make_random(6));
     \\ print(domatic_number_by_search(vpar));
     print(domatic_count_by_search(vpar));
     vpar_view(vpar);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_domset_flags() how early exit

\\ always at least n/2 since find at most 2 dom at each v
\\ initial dominated=0
\\ n=1 1.000000000000000000000000000 [1]
\\ n=2 1.666666666666666666666666667 [2, 4]
\\ n=3 2.644444444444444444444444444 [0, 16, 29]
\\ n=4 3.555555555555555555555555556 [0, 32, 192, 352]
\\ n=5 4.512418952618453865336658354 [0, 0, 840, 3208, 5977]
\\ n=6 5.473843493298746217034154777 [0, 0, 1728, 21504, 68640, 130176]
\\ n=7 6.443805020062021285223499863 [0, 0, 0, 94080, 622160, 1799600, 3464365]
\\                                              .09    .6       1.7      3.4

\\ initial dominated=flags
\\ n=1 1.000000000000000000000000000 [1]
\\ n=2 1.333333333333333333333333333 [4, 2]
\\ n=3 1.800000000000000000000000000 [21, 12, 12]
\\ n=4 2.472222222222222222222222222 [160, 128, 144, 144]
\\ n=5 3.234413965087281795511221945 [1625, 1600, 2040, 2320, 2440]
\\ n=6 4.054042369217466493731085171 [20736, 23328, 33696, 42528, 48960, 52800]
\\ n=7 4.913736736449670203613421279 [319333, 393764, 629748, 872004, 1092308, 1271844, 1401204]
\\                                      .3      .4      .6      .9      1.1      1.3      1.4

domset_flags_exit_location(vpar,flags) =
{
  \\ "count" is how many undominated vertices.
  \\ When v or p is newly dominated the count decreases

  my(dominated=Vecsmall(flags,#vpar),  \\ all of flags itself dominated
     count=#vpar-hammingweight(dominated));
  for(v=1,#vpar,
     my(p=vpar[v]);
     if(((flags[v] || (p && flags[p])) && (count-=(dominated[v]++==1))==0)
        || (flags[v] && p              && (count-=(dominated[p]++==1))==0),
        return(v)));
  0;
}

{
  \\ 2,2,4,6,10
  for(n=1,8,
     my(num=0,den=0,
        histogram=vector(n));
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           forvec(flags=vector(#vpar,i,[0,1]),
                 my(v=domset_flags_exit_location(vpar,flags));
                 if(v>0, num += v; den++; histogram[v]++);
                 ));
     print("n="n" "num*1.0/den" "histogram);
  );
  quit;
}



\\------------------------------------------------------------------------------

\\ Kubica, "An Efficient Method of Examining All Trees, Combinatorics,
\\ Probabilty and Computing, volume 5, 1996, pages 403-413.

\\ 1.5113
\\ rho = 0.3383219
\\ radius of convergence of gf of rooted trees counts
n=5000;
v=vpar_count_unlabelled_vector(n);
vecsum(v) * 1.0 / v[n]
1 - v[n] / vecsum(v) * 1.0
quit



\\------------------------------------------------------------------------------
\\ subdivision of bistar vpar_minimal_domsets_count

\\ T1 1/27 power bigger than bistar
{
  for(n=1,200,
     my(vpar=vpar_subdivide(vpar_make_bistar(ceil(n/2),floor(n/2))),
        got=vpar_minimal_domsets_count(vpar),
        pow=(12161.0^(1/27.0))^#vpar,
        bigger=if(got>pow," bigger",""));
     printf("n=%d sub n=%d   %d %.2f%s\n",
            n, #vpar, got, pow, bigger);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_minmax_descendants_vector() distinct vectors occurring

\\ forests
\\ 1, 2, 5, 15, 52, 203, 877,
\\ A000110 Bell numbers

\\ trees
\\ 1, 2, 5, 15, 52, 203,
\\ A000110 Bell numbers

\\ root=1
\\ 1, 1, 2, 5, 15, 52, 203, 877,
\\ A000110 Bell numbers

{
  for(n=1,8,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              min_descendant = vpar_minmax_descendants_vector(vpar,min),
              count=0);
           mapisdefined(m,min_descendant,&count);
           mapput(m,min_descendant,count+1);
     );
     m=Mat(m);
     \\ for(i=1,matsize(m)[1], print(m[i,1]" count "m[i,2]));
     print(matsize(m)[1],",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar up steps child->parent

\\ forests total upwards to parent
\\ 0, 1, 12, 150, 2160, 36015, 688128,
\\ A053507 a(n) = binomial(n-1,2)*n^(n-3).
\\
\\ trees total upwards to parent
\\ 0, 1, 9, 96, 1250, 19440,
\\ A081131 n^(n-2)*C(n,2) = (n-1)/2 * n^(n-1)
\\
\\ root=1 trees total upwards to parent
\\ 0, 0, 1, 12, 150, 2160, 36015, 688128,
\\ same as forests 1 later
\\
\\ totals downwards same, so half each total num edges (vpar_num_edges())

{
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           total += sum(v=1,#vpar, vpar[v] && v>vpar[v]);
           );
     print(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_weights_vector() total, labelled rooted

\\ forests
\\ not in OEIS: 0, 6, 81, 1264, 21625, 422016,
\\
\\ trees
\\ not in OEIS: 0, 4, 45, 640, 10325, 193536,
\\
\\ root=1 trees
\\ not in OEIS: 0, 2, 15, 160, 2065, 32256, 586579,

{
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              weights=vpar_weights_vector(vpar));
           total += vecsum(weights));
     print(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_diameter_num_endpoints() total

\\ forests
\\ not in OEIS: 1, 6, 33, 280, 3085, 42456, 676795,
\\
\\ trees
\\ not in OEIS: 1, 4, 18, 144, 1600, 20340,
\\
\\ trees root=1
\\ not in OEIS: 1, 2, 6, 36, 320, 3390, 44352,

{
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           total += vpar_diameter_num_endpoints(vpar));
     print(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_diameter_num_vertices() total

\\ forests
\\ not in OEIS: 1, 6, 42, 428, 5440, 83352,
\\
\\ trees
\\ not in OEIS: 1, 4, 27, 256, 3125, 44496,
\\
\\ trees root=1
\\ not in OEIS: 1, 2, 9, 64, 625, 7416, 107569,

{
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           total += vpar_diameter_num_vertices(vpar));
     print(total",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_automorphism_generators()

test_permorder(Vecsmall([1,2,3,4]))
numtoperm(4,0)
test_permorder(Vecsmall(numtoperm(4,0)))

vpar=[0,1,2];
vpar_automorphism_generators(vpar)
vpar=vpar_make_star(5)
g=vpar_automorphism_generators(vpar)
print("generators");
for(i=1,#g, print(g[i]));
apply(test_permorder,g)

perms_all_products(v) =
{
  if(#v==0,return(v));
  my(ret=List([ vpar_INTERNAL_identity_perm(#v[1]) ]),
     more=1);
  while(more,
        more=0;
        for(i=1,#ret,
           my(perm=ret[i]);
           for(j=1,#v,
              my(new_perm=perm*v[j], pos);
              if(pos=setsearch(ret,new_perm,1),
                 listinsert(ret,new_perm,pos);
                 more=1))));
  Vec(ret);
}
print("all products");
v=perms_all_products(g);
for(i=1,#v,print(v[i]));
#v
4!

\\ total automorphism orders of labelled rooted, maybe
\\ not in OEIS: 1, 2, 6, 30, 238, 2408,
{
  for(n=0,5,
     forvec(vec=vpar_forvec_from(n,1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
              g=vpar_automorphism_generators(vpar),
              all=perms_all_products(g),
              ord=max(1,#all));
           total += ord;
           if(0 && n<=4,
              print(vpar" order "#all));
           );
     print(total",");
     );
}

\\ total automorphism orders of unlabelled rooted, maybe
\\          n = 0  1  2  3
\\              1, 1, 1, 3, 10, 41, 188, 1046, 6731, 50718,
\\ not in OEIS:          3, 10, 41, 188, 1046, 6731, 50718,
{
  for(n=0,10,
     my(depths=vpar_preorder_depths_first(n),
        total=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(vpar=vpar_make_preorder_depths(depths),
              g=vpar_automorphism_generators(vpar),
              all=perms_all_products(g),
              ord=max(1,#all));
           total += ord;
          );
     print(total",");
     );
}

quit


\\------------------------------------------------------------------------------
\\ Number of Vertices Which are Min Descendant

\\ forests
\\ not in OEIS: 0, 1, 5, 36, 350, 4320, 64827, 1146880,

\\ trees
\\ 0, 1, 3, 18, 160, 1875, 27216,
\\ 1/2 of
\\ A055541 total leaves in labelled trees   (offset=1)
\\       = rooted labelled trees with root degree=1
\\ A061302 n*(n-1)^(n-2)                    (offset=0)

{
  for(n=0,10,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
              min_descendant=vpar_minmax_descendants_vector(vpar,min));
           total += sum(v=1,#vpar, v==min_descendant[v]);
           );
     print(total",");
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_automorphism_ids_vector()
\\ distinct classes in all vertices is path-n with n! permutations

vector_is_distinct(v) =
{
  my(seen=vectorsmall(#v));
  for(i=1,#v, if(seen[v[i]]++>1, return(0)));
  1;
}

{
  \\       1 2 3 4 5 6 7
  my(vpar=[0,1,1,3,1,5,6]);
  vpar=vpar_make_path(7);
  vpar_validate(vpar);
  my(ids=vpar_automorphism_ids_vector(vpar));
  print("ids "ids);
  print("distinct "vector_is_distinct(ids));
  \\ vpar_view(vpar);
}

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              ids=vpar_automorphism_ids_vector(vpar),
              is_distinct = vector_is_distinct(ids));
           count += is_distinct;
     );
     print(count));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_upwards() and vpar_is_downwards()

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_upwards(vpar);
           \\ count += (vpar_is_upwards(vpar) && vpar_is_downwards(vpar));
     );
     print(count));
  quit;
}


\\------------------------------------------------------------------------------
\\ transforms_pari.txt
\\ taking a[1] as the n=0 term

read("transforms_pari.txt");
vpar_INTERNAL_Euler_transform([1,2,3,4,5,6,7,8])
trv_euler([0, 1,2,3,4,5,6,7,8])

vpar_INTERNAL_Euler_transform([1,1,1,1,1,1,1])
trv_euler([0,1,1,1,1,1,1,1])


vpar_INTERNAL_exponential_transform([1,2,3,4,5,6,7])
trv_exp([0, 1,2,3,4,5,6,7])


opv_mul_egf([1,2,3,4], [5,6,7,8])
quit


\\------------------------------------------------------------------------------

non_leastchildmonk_first(vpar) =
{
  my(min_child=vpar_minmax_child_vector(vpar,min),
    first=#vpar+1);
  for(v=1,#vpar,
     if(vpar[v]==0 && min_child[v] && min_child[min_child[v]],
        first=min(first,
                  max(v, max(min_child[v], min_child[min_child[v]])))));
  \\ first-1;
  if(first==#vpar+1, 0, first);
}
{
  for(n=3,10,
     my(total = 0,
        count = 0);
     forvec(vec=vpar_forvec_from(n,1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
              first=non_leastchildmonk_first(vpar));
           if(first,
              count++;
              total+=first));
     print(total/count /n *1.0);
     );
}
quit


\\------------------------------------------------------------------------------
\\ trees total parents ie. not-childless vertices

\\ A060226 a(n) = n^n - n*(n-1)^(n-1)
\\ 0, 2, 15, 148, 1845, 27906,

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count += vpar_num_childful(vpar));
     print(count",");             \\ vpar with increasing
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ partition max product

\\ 1 2 3 4 6 9 12 18 27 36 54 81
\\ A000792  a(n) = max{ (n-i)*a(i) : i < n}; a(0) = 1
\\ biggest product made by extending partition

\\ partition number in lexicographic order attaining maximum
\\            n=0 1 2 3 4 5 6 7  8
\\              1, 1, 1, 1, 1, 3, 4, 4, 10, 12, 14, 27, 34, 39, 70, 84, 101, 163, 199, 235, 364
\\ not in OEIS:                         10, 12, 14, 27, 34, 39, 70, 84, 101, 163, 199, 235, 364
\\
\\ counting first partition as 0
\\              0, 0, 0, 0, 0, 2, 3, 3, 9, 11, 13, 26, 33, 38, 69, 83, 100, 162, 198, 234, 363
\\ not in OEIS:                            11, 13, 26, 33, 38, 69, 83, 100, 162, 198, 234, 363

\\ A000792_samples=[1,1,2,3,4,6,9,12,18,27,36,54,81,108,162,243,324,486,729,972,1458,2187,2916,4374,6561,8748,13122,19683,26244,39366,59049,78732,118098,177147,236196,354294,531441,708588,1062882,1594323,2125764,3188646,4782969];

vector_product(v) = prod(i=1,#v, v[i]);
{
  for(n=0,30,
     my(pos=1,m=1,mp,mpos=1,reps=1, l=List([]));
     forpart(p=n,
             my(t=vector_product(p));
             if(t>m, m=t; mp=p; mpos=pos; reps=1; l=List([p]),
               t==m, reps++; listput(l,p));
             pos++);
     l=apply(Vec,Vec(l));
     \\ print("n="n" "m" at "Vec(mp)" pos="pos);
     \\ print(m);
     \\ print(mpos);
     \\ nt(reps);
     print("n="n" reps="reps" "l);

     if(n+1<=#A000792_samples,
        m==A000792_samples[n+1] || error());
  );
}

\\ for(n=8,10, print(2+((n-2)%3)))
\\ for(n=8,10, print((n-2)\3))
\\ for(n=8,10, print(A000792_by_power(n)))
\\ for(n=8,10, print(factor(A000792_by_power(n))))

A000792_by_power(n) = if(n<2,1, 3^((n-2)\3) * (2+((n-2)%3)));
check_equal(vector(#A000792_samples,n,n--; A000792_by_power(n)), \
            A000792_samples);
quit


\\------------------------------------------------------------------------------
\\ vpar_nest() occurring

\\ forests distinct nests sorted
\\ 1, 2, 4, 9, 20, 48,
\\ A000081 unlabelled rooted trees
\\
\\ trees or trees root=1 distinct nests sorted
\\ same 1 later

\\ forests distinct nests unsorted
\\ 1, 2, 5, 14, 42, 132,
\\ A000108 Catalan numbers
\\
\\ trees or trees root=1 distinct nests sorted
\\ same 1 later

{
  for(n=1,8,
     my(m=Map());
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              nest = vpar_nest(vpar,0));  \\ sorted
           \\ print(vpar"  "nest);
           mapput(m,nest,1));
      \\ print(count",");             \\ vpar with increasing
     print(#m,",");  \\ vectors occurring
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ vpar_is_hereditaryleastsingle()

\\ forests hereditaryleastsingle
\\ not in OEIS: 1, 3, 10, 53, 326, 2587, 23570,
\\ initially close to forest all leastchildmonk, but then smaller
\\
\\ trees is hereditaryleastsingle
\\ not in OEIS: 1, 2, 3, 16, 75, 576, 4655,
\\
\\ trees root=1 is hereditaryleastsingle
\\ n=           1  2  3  4
\\              1, 1, 1, 4, 15, 96, 665, 6028
\\ not in OEIS:          4, 15, 96, 665, 6028

forest=-1;
vector(10,n,n--; vpar_count_hereditaryleastsingle(n,-1))

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count+=vpar_is_hereditaryleastsingle(vpar));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_root_minmax()

\\ forests hereditaryleastsingle
\\
\\ 1, 2, 7, 38, 291, 2932, 36961,
\\ A001858 unrooted labelled forests
\\ by when minimum is root is one-to-one with unrooted
\\
\\ trees is count of root=1 trees n^(n-2)
\\ trees root=1 is same
\\ 1, 1, 3, 16, 125, 1296, 16807,
\\ A000272
\\

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count+=vpar_is_root_minmax(vpar,min));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_leastchildmonk()

\\ forests all components are leastchildmonk
\\ not in OEIS: 1, 3, 10, 53, 386, 3907, 51290,

\\ trees is leastchildmonk
\\ not in OEIS: 1, 2, 3, 16, 135, 1536, 21875,

\\ trees root=1 is leastchildmonk
\\           n= 1  2  3  4
\\              1, 1, 1, 4, 27, 256, 3125, 46656,
\\ n^n

vector(10,n,n--; vpar_count_leastchildmonk(n,1))

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count+=vpar_is_leastchildmonk(vpar));
     print(count",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_leastchildmonk() and  vpar_is_root_minmax()

\\ forests all components are leastchildmonk and all have min as root
\\ not in OEIS: 1, 2, 5, 18, 93, 704, 7403,
\\ typo in Chunwei Song's thesis, think 704 not 104
\\ not in OEIS: 1, 2, 5, 18, 93, 104

\\ trees is leastchildmonk

\\ trees root=1 is leastchildmonk
\\ n= 1  2  3  4
\\    1, 1, 1, 4, 27, 256, 3125, 46656,
\\ n^n

vpar_count_leastchildmonk_and_rootmin(3,1)
vector(10,n,n--; vpar_count_leastchildmonk_and_rootmin(n,1))
{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_root_minmax(vpar,min)
                    && vpar_is_leastchildmonk(vpar));
     print(count",");
  );
  quit;
}


\\ 3!/1/1/1 * (1-2)^(1-2)  / 3!

\\------------------------------------------------------------------------------
\\ vpar_limb_height() count

\\ forests total height count
\\ not in OEIS: 1, 3, 19, 169, 1901, 25561, 409375,
\\
\\ trees total height count
\\ not in OEIS: 1, 2, 12, 84, 920, 11550, 182112,
\\
\\ trees root=1 total height count
\\              1, 1, 4, 21, 184, 1925, 26016,
\\ not in OEIS:       4, 21, 184, 1925, 26016,

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           count += vpar_limb_height_and_count(vpar)[2];
           );
     print(count",");
  );
}
quit;


\\------------------------------------------------------------------------------
\\ vpar_limb_height() == vpar_limb_length()

\\ forests  height==length
\\ not in OEIS: 1, 3, 13, 89, 786, 10657, 159769,
\\
\\ trees  height==length
\\ not in OEIS: 1, 2, 6, 40, 325, 5016, 71869,
\\
\\ root=1 trees  height==length
\\              1, 1, 2, 10, 65, 836, 10267,
\\ not in OEIS:       2, 10, 65, 836, 10267,

\\ forests  height < length
\\              0, 0, 3, 36, 510, 6150,
\\ not in OEIS:       3, 36, 510, 6150,
\\
\\ trees  height < length
\\              0, 0, 3, 24, 300, 2760, 45780,
\\ not in OEIS:       3, 24, 300, 2760, 45780,
\\
\\ root=1 trees  height < length
\\              0, 0, 1, 6, 60, 460, 6540, 83874,
\\ not in OEIS:          6, 60, 460, 6540, 83874,

\\ Tree longer length is limb > h going above the root.
\\ Length l, tree n-l vertices height <=l-1 root degree>=2.
\\ Labelled unrooted with unique limb length, root as any along it.

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,-1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           count += (vpar_limb_height(vpar) < vpar_limb_length(vpar));
           );
     print(count",");
  );
}
quit;


\\------------------------------------------------------------------------------
\\ vpar_component_sizes() total

\\ forests total count components
\\ 1, 4, 24, 200, 2160, 28812,
\\ A089946 2*(n+1) * (n+2)^(n-1)
\\
\\ n-1 positions 0 to n code for forest, plus 1 always root
\\          existing    new 0 in each of k-1 ways
\\ C(n,k) = (n+1)*C(n,k-1) + 1*(n+1)^(k-1)
\\ B(n) = C(n,n-1) + (n+1)^n
\\  0-4   1 out of 5
\\  0-4  5*1 + 5^1 = 10      2*5^1
\\  0-4  5*10 + 5^2 = 75     (2+1)*5^2
\\  0    75 + 5^3 = 200      (2+1)*5^2
\\
\\ n=5
\\  0-5   1                  1*6^0
\\  0-5  6*1 + 6^1 = 12      2*6^1
\\  0-5  6*12 + 6^2 = 108    3*6^2
\\  0-5  6*108 + 6^3 = 864   4*6^3
\\  0    864 + 6^4           4*6^3 + 6^4
\\ vector(6,n,n--; 2*n*(n+1)^(n-2))
\\ vector(16,n, 2*n*(n+1)^(n-2)/vpar_count(n,1)*1.0)
\\  2*n*(n+1)^(n-2) / (n+1)^(n-1)
\\  2*n / (n+1) == 2 - 2/(n+1)

\\ labelled forests total max component size
\\ not in OEIS: 1, 5, 40, 413, 5376, 83797, 1533610,

{
  my(n=5, total=0);
  forvec(\\ vec=vpar_forvec_from(n,1),
        \\ vec=[[0,4], [0,4], [0,4], [0,4], [0,0]],
        vec=[[0,5],[0,5],[0,5],[0,5], [0,0]],
        total+=sum(i=1,#vec, vec[i]==0));
  print(total);
  quit;
}


{
  for(n=1,8,
     my(total_max=0, total_count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
              component_sizes = vpar_component_sizes(vpar));
           total_max += vecmax(component_sizes);
           total_count += #component_sizes;
           );
     \\ print(total_max",");
     \\ print(total_count",");
     \\ print(total_count/vpar_count(n,1)*1.0);  \\ mean num components -> 2
  );
}
quit;


\\------------------------------------------------------------------------------
\\ vpar_limb_height() total

\\ trees sum limb heights
\\              0, 2, 15, 136, 1465, 19416,
\\ not in OEIS:    2, 15, 136, 1465, 19416,
\\
\\ trees sum limb height attain
\\ not in OEIS: 1, 2, 12, 84, 920, 11550,

\\ forests sum limb heights
\\              0, 2, 21, 220, 2675, 38286,
\\ not in OEIS:    2, 21, 220, 2675, 38286,
\\
\\ forests sum limb height attain
\\ not in OEIS: 1, 2, 12, 84, 920, 11550, 182112,

\\ A117887 labelled trees on <= n nodes

\\ n with limb height h count c, further n+h count +1
\\ S(n,h) = S(n-h,h)+1
\\ +1 vertex extending limb
\\ C(n,h) = C(n-1,h-1)

\\ partition limbs and rest tree, limbs under tree >=2 each leaf, >=0 other

{
  for(n=1,8,
     my(total=0, total_count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
              v = vpar_limb_height_and_count(vpar));
           total += v[1];
           total_count += v[2]);
     \\ print(total",");
     print(total_count",");
  );
}

A060946(n) = trace(matrix(n,n,i,j, i^(j-1)));
vector(9,n, A060946(n))

quit;


\\------------------------------------------------------------------------------
\\ limb vertex count

\\ trees
\\ not in OEIS: 1, 4, 24, 204, 2300, 32550,
\\          n = 1  2   3
\\
\\ 1->2  2->1 both all = 4

\\ forests
\\ not in OEIS: 1, 6, 45, 436, 5305, 78486,


limb_vertex_count(vpar) =
{
  my(num_children = vpar_num_children_vector(vpar),
     limb = vector(#vpar));
  for(i=1,#vpar,
     if(num_children[i]==0,
        my(v=i);
        until(v==0||num_children[v]>=2,
              limb[v]=1;
              v=vpar[v])));
  vecsum(limb);
}
check_equal(limb_vertex_count([]), 0, "empty");
check_equal(limb_vertex_count([0]), 1, "singleton");
check_equal(limb_vertex_count([0,1]), 2);

{
  for(n=1,8,
     my(total=0);
     forvec(vec=vpar_forvec_from(n,0), \\ trees or forests
           my(vpar=vpar_from_upascend(vec));
           total += limb_vertex_count(vpar));
     print(total",");
  );
}
quit

\\------------------------------------------------------------------------------
\\ vpar_limb_height() counts

\\ limb_height = 1
\\ 2,3,16,145,1536,19579, 290816
\\ A052318 labelled rooted trimmed n nodes

\\ limb_height = 2
\\ not in OEIS: 6, 24, 240, 3000, 46830, 836976

\\ limb_height = 3
\\ not in OEIS: 24, 120, 1800, 28560, 539280

{
  for(n=1,8,
     my(count=vector(n+1));
     forvec(vec=vpar_forvec_from(n,0), \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
             h=vpar_limb_height(vpar));
           count[h+1]++);
     print(count);
  );
  quit;
}


\\------------------------------------------------------------------------------

test_bandwidth_by_search(vpar) =
{
  my(b=#vpar);
  for(p=1,(#vpar)!,
     b=min(b,vpar_dilation(vpar_relabel_perm(vpar,numtoperm(#vpar,p)))));
  b;
}

vpar=vpar_make_path(6);
vpar=vpar_make_random(6);
vpar=[0, 1, 1, 2, 3, 4];
vpar=[0, 0, 1, 1, 3, 4, 6];
\\ vpar_view(vpar);
s=vpar_bandwidth(vpar)
p=test_bandwidth_by_search(vpar)
\\if(s!=p,vpar_view(vpar));

\\perm=[3, 1, 2, 4, 5, 6];
\\ vpar=vpar_relabel_perm(vpar,perm);
\\vpar_view(vpar);
\\ vpar_dilation(vpar)
quit



\\------------------------------------------------------------------------------
\\ vpar_row_widths_vector() occurring

\\ forests distinct row_widths
\\ 1, 2, 4, 8, 16, 32,
\\ 2^n
\\ trees or trees root=1 same but 1 later

\\ forests increasing row_widths
\\ not in OEIS: 1, 3, 13, 85, 651, 6691, 76518,
\\
\\ trees increasing row_widths
\\ 1, 2, 6, 24, 120, 720, 5040,
\\ factorial
\\ trees root=1 increasing row_widths same but 1 later

{
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              row_widths = vpar_row_widths_vector(vpar));
           \\ vpar_is_row_widths_vector(row_widths) || error();
           mapput(m,row_widths,1);
           count += (row_widths==vecsort(row_widths,,4)));
     \\ print(count",");             \\ vpar with increasing
     print(#m,",");  \\ vectors occurring
     if(n<=5,
        print("n="n"  "Mat(m)[,1]);
       )
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_subtree_heights_vector() occurring

\\ labelled forests distinct subtree_heights
\\ trees root=1 distinct subtree_heights same but 1 later
\\ 1, 3, 10, 47, 246, 1602, 11481,
\\ A005651 sum multinomial over partitions
\\
\\ labelled trees distinct subtree_heights
\\ not in OEIS: 1, 2, 9, 40, 235, 1476, 11214,

\\ forests subtree_heights decreasing, or increasing
\\ not in OEIS: 1, 2, 5, 16, 61, 268,
\\ trees subtree_heights decreasing same but 1 later
\\ trees root=1 subtree_heights decreasing same but 1 later
\\ trees root=1 subtree_heights increasing none
\\ = seq same recurrence
\\

{
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              subtree_heights = vpar_subtree_heights_vector(vpar));
           vpar_is_subtree_heights_vector(subtree_heights) || error();
           mapput(m,subtree_heights,1);
           count += (subtree_heights==vecsort(subtree_heights,,4)));
     print(count",");
     \\ print(#m,",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_minmax_child_vector() occurring

\\ forests min or max child monotonic
\\ 1 2 5 15 52 203 877
\\ A000110 Bell numbers
\\ trees same 1 later, trees root=1 none

\\ forests distinct child_vectors
\\ 1 3 13 73 501 4051 37633
\\ A000262 partition 1..n into ordered lists (ordered within lists, no order
\\ among lists)
\\ A000262(n) = n! * sum(j=0,n-1, binomial(n-1,j)/(j+1)!);  /* Wilf */
\\ vector(10,n, A000262(n))
\\ a(n) = (2*n-1)*a(n-1) - (n-1)*(n-2)*a(n-2).
\\
\\ trees distinct child_vectors
\\ not in OEIS: 1, 2, 9, 53, 378, 3151, 30000
\\
\\ trees root=1 distinct child_vectors
\\              1, 1, 3, 14, 86, 646, 5694
\\ not in OEIS:       3, 14, 86, 646, 5694

{
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              child_vector = vpar_minmax_child_vector(vpar,max));
           mapput(m,child_vector,1);
           count += (child_vector==vecsort(child_vector,,4)));
     \\ print(count);
     print(#m",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_modulo()

\\ 3-partite trees
\\              1  2  3   4    5     6      7     8
\\ not in OEIS: 1, 2, 9, 32, 225, 2304, 19600

\\ 3-partite forests
\\              1  2   3   4    5     6      7    8
\\ not in OEIS: 1, 3, 16, 75, 576, 6125, 57600

\\ 3-partite trees, root=1
\\              1  2  3  4   5    6     7    8
\\              1, 1, 3, 8, 45, 384, 2800
\\ not in OEIS:       3, 8, 45, 384, 2800


check_equal(vpar_is_modulo([],3), 1);
check_equal(vpar_is_modulo([0,1],3), 1);
check_equal(vpar_is_modulo([2,0],3), 1);
check_equal(vpar_is_modulo([0,1,2],3), 1);
check_equal(vpar_is_modulo([0,1,1],3), 1);
check_equal(vpar_is_modulo([0,1,1,1],3), 0);
{
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           count += vpar_is_modulo(vpar,3));
     print(count));
  quit;
}


\\------------------------------------------------------------------------------
\\ isomorphisms table

{
  my(n=4,
     num=n^(n-2),
     vpars=vector(num),
     upto=0);
  forvec(vec=vpar_forvec_from(n,-1), \\ root=1 trees
        vpars[upto++]=vpar_from_upascend(vec));
  for(i=1,#vpars,
     for(j=1,#vpars,
        printf("%d",vpar_is_isomorphic(vpars[i],vpars[j])));
     printf("  ");
     for(j=1,#vpars,
        printf("%d",vpar_is_isomorphic(vpars[i],vpars[j],1)));
     printf("\n");
  );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_is_depths_vector() count

\\ tree, any root
\\ A052882
\\ 1 2 9 52 375 3246

vector(20,n, n*sum(k=1,n-1, k! * stirling(n-1,k,2)))

{
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec));
           mapput(m,vpar_depths_vector(vpar),1));
     print(#m,","));
  quit;
}

\\ 1 3 13 75 541 4683 47293 545835

check_equal(stirling(4,2,2),7);

\\ Wilf "Gfology"
\\ stirling2(n,k) ways to partition into k non-empty depth levels
\\ k!*stirling2(n,k) to order those depths
\\ sum over all depths is ordered Bell numbers (Fubini)
\\ A000670
f(n,k) = k! * stirling(n,k,2);
check_equal(f(1,1),1);
check_equal(f(2,1),1);
vector(20,n,ordered_Bell(n))
quit

{
  for(n=1,8,
     my(count=0);
     forvec(vec=vector(n,i,[0,n-1]),
           count += vpar_is_depths_vector(vec));
     print(count));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_is_preorder() count

\\ forests or trees both
\\ 1, 1, 2, 5, 14, 42, 132, 429,
\\ A000108 Catalan numbers
\\
\\ 1, 15, 18, 7, 1
\\ A080936 T(n,k) = num of Dyck paths of semilength n and height k (1<=k<=n)

\\ Catalan number, sum of Catalan triangle column n
catalan(n) = binomial(2*n,n)/(n+1);

{
  for(n=1,8,
     my(count=0,
        by_height=vector(n));
     forvec(vec=vpar_forvec_from(n,0), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              h=vpar_height(vpar));
           if(vpar_is_preorder(vpar),
              count++;
              by_height[h+1]++);
     );
     print(by_height,"  total "count);
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_INTERNAL_seq_bucket_sort() speed

vpar_INTERNAL_seq_bucket_sort(keys,down=0) =
{
  my(seq=vectorsmall(#keys));
  if(#keys,
     my(chain   = seq,
        base    = vecmin(keys)-1,
        buckets = vectorsmall(vecmax(keys)-base));

     \\ make linked lists in buckets
     forstep(v=#keys,1,-1,
        my(b=keys[v]-base);
        chain[v]=buckets[b]; buckets[b]=v);

     \\ linked lists into seq
     my(upto=0);             \\ ready for pre-increment
     forstep(b=if(down,#buckets,1),if(down,1,#buckets),if(down,-1,1),
        my(v=buckets[b]);
        while(v,
              seq[upto++]=v;  \\ pre-increment
              v=chain[v])));
  seq;
}

n=3 000 000;
v=vector(n,i,random(n\2));
#
vpar_INTERNAL_seq_bucket_sort(v,1); print("bucket");
vecsort(-v,,1); print("vecsort");
vpar_INTERNAL_seq_bucket_sort(v,1); print("bucket");
vecsort(-v,,1); print("vecsort");
vecsort(v,,1); print("vecsort");
quit;




\\------------------------------------------------------------------------------
\\ random tree cf "genrang -t", and others

\\ random list, at each pair p,v with p in tree and v not, edge v->p
\\ p,v,p,p,p, v,
\\ random walk between vertices of complete-n, when destination not in tree
\\ add edge

ran(n) =
{
  my(vpar=vector(n),
     degrees=vectorsmall(n),
     ne=0,
     p=random(n)+1);
  (1<=p && p<=n) || error();

  while(ne<n-1,
        my(v);
        until(v!=p, v=random(n)+1);
        (1<=v && v<=n && v!=p) || error();
        (1<=p && p<=n) || error();
        if(degrees[v]==0,
           vpar[v]==0 || error();
           vpar[v]=p;
           degrees[v]++;
           degrees[p]++;
           ne++;
           p=v;
          ,
           p=v;
          ));
  \\ vpar_validate(vpar);
  vpar;
}
\\ for(i=1,20,print(ran(5)));

\\ http://www.cs.cmu.edu/~15859n/RelatedWork/AldousRandomTrees.pdf
\\ Algorithm 2
\\ Vertices 1..n.
\\ For i = 2 to n, connect i to min(random,i-1), where random 1..n.
\\ Random relabel 1..n.
make_random_Aldous(n,perm=1) =
{
  my(vpar=vector(n,i, min(random(n)+1,i-1)));
  if(perm,vpar=vpar_relabel_random(vpar));
  vpar;
}

{
  \\ number of trees A000055 1,1,2,3,6,11,23,47,106
  \\                      n= 2 3 4 5 6  7  8
  my(n=6,
     free=1,
     m=Map());
  for(i=1,100000,
     \\ my(vpar=ran(n));
     \\ my(vpar=vpar_make_random(n));
     \\ vpar=vpar_reroot(vpar,1);
     my(vpar=make_random_Aldous(n,0));
     mapput(m,vpar, if(mapisdefined(m,vpar), mapget(m,vpar), 0) + 1);
  );
  m=Mat(m);
  print("rows "matsize(m)[1]);
  \\ print("sorted ",vecsort(m[,2]));

  my(vpars=List([]),
     counts=List([]));
  for(i=1,matsize(m)[1], \\ rows
     my(vpar=m[i,1],c=m[i,2]);
     if(free,
        vpar=vpar_add_roots_above_centres(vpar));
     \\ print(vpar" count "c);
     if(free,
        for(j=1,#vpars,
           \\ print("  compare "vpars[j]);
           if(vpar_is_isomorphic(vpars[j],vpar),
              \\ print("  duplicate of "j);
              counts[j]+=c;
              next(2)));
       );
     \\ print("    new");
     listput(vpars,vpar);
     listput(counts,c);
  );
  vpars=Vec(vpars);
  counts=Vec(counts);
  print("distinct "#counts);
  counts=Vec(counts);
  print(vecsort(counts));

  my(perm=vecsort(counts,,1));
  vpars=vecextract(vpars,perm);
  counts=vecextract(counts,perm);

  for(i=1,#vpars,
     my(vpar=vpars[i],
        count=counts[i],
       diameter=vpar_diameter(vpar));
     print(count"  "vpar"  diam="diameter);
  );
}
quit

\\------------------------------------------------------------------------------
\\ count attach


vector(8,n, prod(v=2,n, (v-1)*(n-v+1)))   \\ (n!)^2
vector(8,n, ((n-1)!)^2)
quit



\\------------------------------------------------------------------------------
\\ vecextract() inverse

v=['a,'b,'c]
seq=Vecsmall([3,1,2])
vecextract(v,seq)
vecextract(v,seq^-1)

v=Vecsmall([100,200,300])
vecextract(v,seq)
v*seq
quit


\\------------------------------------------------------------------------------
\\ vpar_make_random()

setrand(1);
for(i=1,10, vpar=vpar_make_random(3); vpar_validate(vpar));
quit


\\------------------------------------------------------------------------------
\\ vpar_from_downsuccessive() sample

vec=[0,3,3,3];
vpar=vpar_from_downsuccessive(vec)
vpar_view(vpar);
quit;

\\------------------------------------------------------------------------------
\\ vpar_seq_successive_paths() set of extracts

mangle(vec) =
{
  my(seen=vectorsmall(#vec));
  vec[#vec]==0 || error();
  for(i=1,#v,
     if(vec[i]==1 || (vec[i] && seen[vec[i]]),
        vec[#vec]=vec[i]; vec[i]=0; break());
     if(vec[i]==0,break());
     seen[vec[i]]=1);
  vec;
}
{
  my(l=List([]));
  my(m=List([]));
  for(n=4,4,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ trees or forests
           listput(m,mangle(vec));
           my(vpar=vpar_from_upascend(vec));
           my(seq =vpar_seq_successive_paths(vpar));
           my(vec=vecextract(vpar,seq));
           listput(l,vec)));
  l=vecsort(l);
  m=vecsort(m);
  for(i=1,#l,
     print(l[i]"  "m[i],if(l[i]!=m[i],"  *****","")));
  quit;
}


\\------------------------------------------------------------------------------
\\ vpar_seq_successive_paths() show

{
  for(n=4,4,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ trees
           my(vpar=vpar_from_upascend(vec));
           my(seq =vpar_seq_successive_paths(vpar));
           \\ seq =vpar_seq_downsuccessive(vpar);
           my(vec=vecextract(vpar,seq));
           print(vec)));
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_seq_downsuccessive()

{
  for(n=4,4,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1), \\ trees
           my(vpar=vpar_from_upascend(vec));
           my(seq =vpar_seq_downsuccessive(vpar));
           my(vec =vecextract(vpar,seq));
           my(seq_again =vpar_INTERNAL_seqfrom_downsuccessive(vec));
           print(seq" "vec" "seq_again);
           if(seq!=seq_again,error())));
  quit;
}

\\------------------------------------------------------------------------------
\\ vpar_INTERNAL_seqfrom_downsuccessive()

\\        3
\\      / | \
\\    4   5   6
\\           / \
\\          1   2
vpar=[6,6,0,3,3,3];
\\vpar=[2,0];  \\ 1 --> 2
seq=vpar_seq_downsuccessive(vpar)
vec=vecextract(vpar,seq)
vpar_INTERNAL_seqfrom_downsuccessive(vec)
quit

\\------------------------------------------------------------------------------
\\ vpar_seq_successive_paths()

\\        3
\\      / | \
\\    4   5   6
\\           / \
\\          1   2
vpar=[6,6,0,3,3,3];

vpar=[0,0];
vpar=[0,1];
print("vpar "vpar);
seq=vpar_seq_successive_paths(vpar)
vec=vecextract(vpar,seq)
vpar_from_successive_paths(vec)
quit

\\------------------------------------------------------------------------------
\\ seq max descendant

\\        3
\\      / | \
\\    4   5   6
\\           / \
\\          1   2
vpar=[6,6,0,3,3,3];

\\        2
\\      /   \
\\    4       1
\\            |
\\            3
vpar=[2,0,1,2];

try_seq_other(vpar) =
{
  my(subtree_heights = vpar_subtree_heights_vector(vpar),
     min_descendant  = vpar_minmax_descendants_vector(vpar,min),
     max_descendant  = vpar_minmax_descendants_vector(vpar,max),
     depths          = vpar_depths_vector(vpar),
     keys);

  \\ max descendant, then increasing subtree height
  keys = vector(#vpar,v, [max_descendant[v],subtree_heights[v]]);
  vecsort(keys,,1); \\ flag=1 for seq

  \\ max descendant, then decreasing depth
  keys = vector(#vpar,v, [max_descendant[v],-depths[v]]);
  vecsort(keys,,1); \\ flag=1 for seq

  keys = vector(#vpar,v, [subtree_heights[v], min_descendant[v]]);
  vecsort(keys,,1); \\ flag=1 for seq
}

print("upascend       ",vpar_seq_upascend(vpar));
print("try            ",try_seq_other(vpar));
print("subtree height ",vpar_seq_subtree_height(vpar));
print("max descendant ",vpar_seq_minmax_descendant(vpar,max));
print("upascend ahesd ",vpar_seq_upascend(vpar,1));
print("min descendant ",vpar_seq_minmax_descendant(vpar,min));

{
  for(n=1,10,
     my(count=0);
     forvec(vec=vpar_forvec_from(n), \\ trees
           my(vpar=vpar_from_upascend(vec));
           my(seq_upascend =vpar_seq_upascend(vpar,1));
           my(seq_other    =try_seq_other(vpar));
           if(seq_upascend != seq_other,
              vpar_view(vpar);
              print("");
              print(vpar);
              print(seq_upascend);
              print(seq_other);
              quit())));
}
quit

\\------------------------------------------------------------------------------
\\ free tree primary depths list

\\ 1, 2, 3, 6, 11, 23, 47, 106,
\\ A000055 free trees

{
  \\ 1,2,4,9,20,48
  \\ A000081 unlabelled rooted n nodes

  for(n=5,5,
     my(depths=vpar_preorder_depths_first(n),
        count=0,
        count_primary=0);
     while(1,
           if(depths=='none,break());
           my(vpar=vpar_make_preorder_depths(depths),
              is_preorder=vpar_is_preorder(vpar),
              is_depths=vpar_is_preorder_depths_vector(depths),
              is_primary=vpar_preorder_depths_is_free_primary(depths),
              again=(vpar_depths_vector(vpar)==depths));
           print(depths" dep="is_depths" vpar="vpar" pre="is_preorder" "again" prim="is_primary);
           count++;
           count_primary+=is_primary;
           depths = vpar_preorder_depths_next(depths));
     \\ print(count",");
     print(count_primary",");
     );
  quit;
}

\\------------------------------------------------------------------------------
\\ depths list

{
  \\ 1,2,4,9,20,48
  \\ A000081 unlabelled rooted n nodes

  for(n=3,8,
     my(depths=vpar_preorder_depths_first(n),
        count=0,
        count_primary=0);
     while(1,
           if(depths=='none,break());
           my(vpar=vpar_make_preorder_depths(depths),
              is_preorder=vpar_is_preorder(vpar),
              is_depths=vpar_is_preorder_depths_vector(depths),
              is_primary=vpar_preorder_depths_is_free_primary(depths),
              again=(vpar_depths_vector(vpar)==depths));
           print(depths" dep="is_depths" vpar="vpar" pre="is_preorder" "again" prim="is_primary);
           count++;
           count_primary+=is_primary;
           depths = vpar_preorder_depths_next(depths));
     \\ print(count",");
     print(count_primary",");
     );
  quit;
}
{
  \\ root=1 trees, or all forests
  \\ 10, 47, 246, 1602, 11481
  \\ A005651 sum multinomials over partitions
  \\
  \\ trees
  \\ 40, 235, 1476
  \\ A327827 sum multinomials over partitions incl 1
  my(n=7,
     seen=Map());
  forvec(vec=vpar_forvec_from(n,1),
        my(orig_vec=vec,
           vpar=vpar_from_upascend(vec),

           depths =vpar_depths_vector(vpar),
           seen_str=if(mapisdefined(seen,depths)," seen",""));
        \\ print(orig_vec" vpar="vpar" depths="depths,seen_str);
        mapput(seen,depths,1);
  );
  print("distinct "matsize(Mat(seen))[1]);
  quit
}


\\-----------------------------------------------------------------------------

vpar1= [0, 1, 2];
vpar2=[0, 3, 1];
vpar_validate(vpar1);
vpar_validate(vpar2);
vpar_is_isomorphic(vpar1,vpar2)
quit


\\-----------------------------------------------------------------------------
\\ uniqueness of vecextract

\\ postorder
\\ vpar_view([0, 1, 2, 1])
\\ vpar_view([0, 1, 1, 2])
\\
\\      1                1
\\     / \              / \
\\    2   4            2   3
\\    |                |
\\    3                4
\\
\\  [3, 2, 4, 1]    [4, 2, 3, 1]
\\  [2, 1, 1, 0]    [2, 1, 1, 0]
{
  my(max_n=4);
  for(n=1,max_n,
     my(seen=Map());
     forvec(can=vpar_forvec_from(n), \\ trees
           my(vpar=vpar_from_upascend(can));

           my(seq=vpar_seq_dandelion(vpar,1),
              vec=vecextract(vpar,seq),
              str=if(mapisdefined(seen,vec)," seen",""));
           mapput(seen,vec,1);
           print("vpar=",vpar," seq=",Vec(seq),"  ",vec,str)));
}
quit


\\-----------------------------------------------------------------------------

\\ vpar_view([2,3,0,1])
\\  3 <- 2 <- 1 <- 4
\\ seq1  [3, 2, 1, 4] downsuccessive
\\ seq2  [2, 1, 3, 4] dd

seq_dd(vpar) =
{
  my(min_descendant = vpar_minmax_descendants_vector(vpar,min),
     depths         = vpar_depths_vector(vpar),
     keys = vector(#vpar,v, [min_descendant[v],depths[v]]));
  vecsort(keys,,1); \\ flag=1 for seq
}

{
  for(n=4,4,
     forvec(vec=vpar_forvec_from(n,0),  \\ trees
           my(vpar=vpar_from_upascend(vec));

           my(seq1 = vpar_seq_downsuccessive(vpar),
              seq2 = seq_dd(vpar));
           if(seq1 != seq2,
              print("vpar "vpar);
              print("  seq1  "Vec(seq1));
              print("  seq2  "Vec(seq2));
             );
     ));
}
quit


\\-----------------------------------------------------------------------------

\\     2        vpar 2,0,1,2
\\    / \       upascend 3, 1, 4, 2
\\   1   4
\\   |
\\   3
\\
\\     2        vpar [2, 0, 1, 1]
\\     |        upascend 3, 4, 1, 2
\\     1
\\    / \
\\   3   4

seq_hd(vpar) =
{
  my(heights    = vpar_subtree_heights_vector(vpar),
     descendant = vpar_minmax_descendants_vector(vpar,max),
     keys       = vector(#vpar,v, [descendant[v],heights[v]]));
  vecsort(keys,,1); \\ flag=1 for seq
}

{
  for(n=4,4,
     forvec(vec=vpar_forvec_from(n,0),  \\ trees
           my(vpar=vpar_from_upascend(vec));

           my(seq1 = vpar_seq_upascend(vpar),
              seq2 = seq_hd(vpar));
           if(seq1 != seq2,
              print("vpar "vpar);
              print("  seq1  "Vec(seq1));
              print("  seq2  "Vec(seq2));
             );
     ));
}
quit


\\-----------------------------------------------------------------------------
\\ sequence is idseq

\\ subtree_height
\\ idseq
\\ 1, 1, 2, 5, 16, 61, 268
\\ not in OEIS: 2, 5, 16, 61, 268
\\
\\ idvec
\\ 1, 1, 2, 6, 22, 92, 428
\\ A229741 a(0)=1; thereafter a(n) = n! + Sum_{i=0..n-1} a(i)*a(n-1-i).

\\ subtree_sizes
\\ idseq all trees
\\ these are root=#vpar since it is last in seq
\\ 1, 1, 2, 5, 16, 58, 246
\\ not in OEIS: 2, 5, 16, 58, 246
\\
\\ idvec all trees
\\ 1, 1, 2, 6, 22, 94, 448
\\ not in OEIS: 2, 6, 22, 94, 448

\\ upascend, factorial
\\ idseq all trees
\\ these are root=#vpar since it is last in seq
\\ 1, 2, 6, 24, 120, 720
\\
\\ idvec all trees
\\ 1, 1, 2, 6, 24, 120

\\ upqueue,
\\ idseq all trees
\\ these are root=#vpar since it is last in seq
\\ 1, 1, 2, 5, 15, 52, 203
\\ A000110 Bell
\\
\\ idvec all trees
\\ 1, 1, 2, 6, 22, 92, 428
\\ A229741 same as subtree_height

\\ blob,
\\ idseq n^(n-1) all trees
\\ idvec
\\ 1, 2, 9, 64, 625, 7776, 117649

\\ minancestor (reverse)
\\ idseq n^(n-1) all trees
\\ idvec
\\ 1, 2, 9, 64, 625, 7776, 117649


{
  my(max_n=9,
     count_idseq=vector(max_n),
     count_idvec=vector(max_n),
     seq_func,from_func);
  seq_func  = vpar_seq_subtree_height; from_func = vpar_from_subtree_height;
  seq_func  = vpar_seq_minancestor;    from_func = vpar_from_minancestor;
  seq_func  = vpar_seq_blob;           from_func = vpar_from_blob;
  seq_func  = vpar_seq_upqueue;        from_func = vpar_from_upqueue;
  seq_func  = vpar_seq_upascend;       from_func = vpar_from_upascend;
  seq_func  = vpar_seq_subtree_size;   from_func = vpar_from_subtree_size;

  for(n=1,7,
     my(idseq=vpar_INTERNAL_identity_perm(n));
     forvec(vec=vpar_forvec_from(n,0),  \\ trees
           my(vpar=from_func(vec));
           \\ vpar=vpar_reroot(vpar,#vpar);

           my(seq = seq_func(vpar));
           count_idseq[n]+=(seq==idseq);
           count_idvec[n]+=(vec==vpar);
           \\ if(seq==idseq,
           \\    print("vpar "vpar);
           \\    print("seq  "Vec(seq));
           \\   );
           \\ if(vec==vpar,
           \\    print("vpar "vpar);
           \\    print(" seq "Vec(seq));
           \\   );
     ));
  print("unchanged");
  print(count_idseq);
  print(count_idvec);
}
quit

\\-----------------------------------------------------------------------------
\\ bucket sort of subtree sizes

{
  my(n=8,
     seen=Map());
  forvec(vec=vpar_forvec_from(n,1),  \\ forests
        my(vpar=vpar_from_upascend(vec),
           seq=vpar_seq_subtree_size(vpar),
           vec=vecextract(vpar,seq),
           vpar_again=vpar_from_subtree_size(vec),
           seen_str=if(mapisdefined(seen,vpar_again)," seen",""));
        \\ print(vpar" vec="vec" vpar_again="vpar_again,seen_str,
        \\      if(vpar_is_cyclic(vpar_again)," cyclic",""));
        if(mapisdefined(seen,vpar_again), error());
        if(vpar!=vpar_again,
           print(vpar" vec="vec" vpar_again="vpar_again,seen_str,
                 if(vpar_is_cyclic(vpar_again)," cyclic",""));
           error());
        if(vpar_is_cyclic(vpar_again), error());
        mapput(seen,vpar_again,1);
  );
  quit
}
vpar=[3, 0, 0, 2, 1]
vpar_from_subtree_size([2, 1, 3, 0, 0])
quit

{
  my(n=6,
    seen=Map());
  forvec(vec=vpar_forvec_from(n,1),  \\ forests
        my(vpar=vpar_from_upqueue(vec),
           seq =vpar_seq_subtree_size(vpar),
           sizes_vec =vecextract(vpar,seq),
           seen_str=if(mapisdefined(seen,sizes_vec)," seen",""));
        mapput(seen,sizes_vec,1);
        print("vec="vec" vpar="vpar"  sizes vec="vec,seen_str,
              seen_str,
              if(vec==sizes_vec,"","  different"));
  );
  quit
}


\\-----------------------------------------------------------------------------
\\ vpar_seq_minancestor()
\\ vpar_from_minancestor()

{
  my(n=2,
     seen=Map());
  forvec(vec=vpar_forvec_from(n,1),  \\ forests
        my(orig_vec=vec,
           vpar=vpar_from_upascend(vec),

           above=vpar_minmax_ancestors_vector(vpar,min),
           above_flags=vector(#above,v,above[v]==v),
           seq =vpar_seq_minancestor(vpar),
           vec =vecextract(vpar,seq),

           vec_above=vpar_minmax_ancestors_vector(vec,min),
           vec_above_flags=vector(#above,v,vec_above[v]==v),

           vpar_again=vpar_from_minancestor(vec),
           seen_str=if(mapisdefined(seen,vec)," seen",""));
        print(orig_vec" vpar="vpar" above="above_flags" seq="Vec(seq)"   vec="vec,seen_str" vec_above="vec_above_flags,
             if(above_flags==vec_above_flags,"","  different"));
  );
  quit
}

{
  \\ min,max above "stable"s not the same
  my(n=4);
  forvec(vec=vpar_forvec_from(n,1),  \\ forests
        my(vpar=vpar_from_upascend(vec),

           min_above=vpar_minmax_ancestors_vector(vpar,min),
           max_above=vpar_minmax_ancestors_vector(vpar,max),
           min_above_flags=vector(#min_above,v,min_above[v]==v),
           max_above_flags=vector(#max_above,v,max_above[v]==v));

        print(min_above"  "max_above" "min_above==max_above"    "min_above_flags" "max_above_flags" "min_above_flags==max_above_flags);
  );
  quit
}

{
  my(n=5,
    seen=Map());
  forvec(vec=vpar_forvec_from(n,1),  \\ forests
        my(vpar=vpar_from_upascend(vec),
           seq =vpar_seq_minancestor(vpar),
           vec =vecextract(vpar,seq),
           seen_str=if(mapisdefined(seen,vec)," seen",""));
        mapput(seen,vec,1);
        print("vpar="vpar" vec="vec,seen_str);
  );
quit
}


\\-----------------------------------------------------------------------------
\\ Blob Codes

{
  my(n=4);
  forvec(vec=vector(n,i, if(i==1,[0,n], i==n,[0,0], [0,n])),
        my(vpar=vpar_from_blob(vec));
        print(" vec="vec
              " vpar="vpar
              " roots "vpar_roots(vpar));
  );
}
quit

\\-----------------------------------------------------------------------------
\\ Blob Codes
{
  \\ -1 trees root=1
  \\  0 trees any root
  \\  1 forests any roots
  my(vtype = 1);

  for(n=4,4,
     print("n="n);
     forvec(vec=vpar_forvec_from(n,vtype),
           my(vpar=vpar_from_upascend(vec));
           vpar=vpar_reroot(vpar,3);

           my(seq=vpar_seq_blob(vpar),
              blobvec=vecextract(vpar,seq));
           print(" vec="vpar
                 " vpar="vpar
                 " seq="Vec(seq)
                 " blobvec="blobvec
                 " roots "vpar_roots(vpar)));
     );
}
quit

\\-----------------------------------------------------------------------------
\\ Number of 0,1 Cordial Labellings

vpar_cordial_poly(vpar_make_star(1))
vpar_cordial_poly(vpar_make_star(2))
print("star");
vpar_cordial_poly(vpar_make_star(5))
print("path");
vpar_cordial_poly(vpar_make_path(5))

vecsum(Vec(numerator(vpar_cordial_poly(vpar_make_random(10)))))


vector(11,n,vpar_cordial_count(vpar_make_path(n)))
\\ 2, 4, 4, 12, 12, 40, 40, 140, 140, 504, 504, 1848, 1848, 6864, 6864
\\ 2*A128014 binomial(2*n,n)
\\ *---*---*  +/-1 returning to 0

vector(11,n,vpar_cordial_count(vpar_make_star(n)))
\\ 2, 4, 4, 12, 12, 40, 40, 140, 140, 504, 504, 1848, 1848, 6864, 6864
\\ 2*A128014 binomial(2*n,n)

vector(11,n,vpar_cordial_count(vector(n,v,0)))
\\ 2, 4, 4, 12, 12, 40, 40, 140, 140, 504, 504, 1848, 1848, 6864, 6864
\\ 2*A128014 binomial(2*n,n)
quit


\\-----------------------------------------------------------------------------

\\vpar_from_blob([6,1,2,3,4,5,0])
\\ vpar_from_blob([2,3,4,5,6,0])
 vpar_from_blob([1,7,2,5,9,6,8,8,0])
print("want");
[0,7,2,5,9,1,8,6,8]
quit;

\\-----------------------------------------------------------------------------
\\ Cordial Labellings
\\
\\ trees root=1
\\ count 1, 1, 2, 12, 51, 810, 5452, 143360
\\ n =   1  2  3   4   5
\\ not in OEIS: 12, 51, 810, 5452, 143360
\\ count -1,0:   1, 1, 2, 8, 51, 486, 5452, 81920
\\ count +1,0:   1, 0, 2, 4, 51, 324, 5452, 61440
\\
\\ trees any root
\\ count 1, 2, 6, 48, 255, 4860, 38164
\\ n =   1  2  3   4    5
\\ not in OEIS: 48, 255, 4860, 38164
\\
\\ forests
\\ count 1, 3, 13, 85, 744, 9121
\\ n =   1  2   3   4    5
\\ not in OEIS: 13, 85, 744, 9121

\\ GP-Test  5*51 == 255

{
  \\ -1 trees root=1
  \\  0 trees any root
  \\  1 forests any roots
  my(vtype = -1);

  my(l=List([]));
  for(n=1,7-vtype,
     print("n="n);
     my(count=0);
     forvec(vec=vpar_forvec_from(n,vtype),
           my(vpar=vpar_from_upascend(vec));
           count+=(vpar_cordial_delta(vpar)==!(n%2)));
     print("  cordial "count);
     listput(l,count);
     );
  print(Vec(l));
}
quit


\\-----------------------------------------------------------------------------
\\ Number of rooted trees up to isomorphism.
\\ And vpar_is_isomorphic() an equivalence relation.
\\
\\ 1,1,2,3,5,7

\\ vtype=-1
\\ A000081 unlabelled rooted trees with n nodes
\\ A000081 0,1,1,2,4,9,20,48,115,286,719,1842
\\     n = 0 1 2 3 4
\\ [1, 1, 2, 4, 9, 20]
\\
\\ vtype=0
\\ same number of classes (must n* representatives)
\\
\\ vtype=1
\\ 1,2,4,9,20
\\ same since each n+1 tree = forest of subtrees at children of the root.

{
  \\ -1 trees root=1
  \\  0 trees any root
  \\  1 forests any roots
  my(vtype = 1);

  my(num_classes_list=List([]));
  for(n=1,6,
     print("n="n);

     my(vpar_list=List([]));
     forvec(vec=vpar_forvec_from(n,vtype),
           my(vpar=vpar_from_upascend(vec));
           \\ vpar_roots(vpar)==[1] || next();
           if(vtype==-1,
              vpar_roots(vpar)==[1] || print("oops vec="vec" vpar="vpar));
           \\ if(n==3,vpar_view(vpar));
           listput(vpar_list,vpar));
     vpar_list=Vec(vpar_list);
     print("  num trees ",#vpar_list);

     my(m=matrix(#vpar_list,#vpar_list,i,j,
                 progress(Str("  matrix j="j));
                 \\ print(vpar_list[i]," ",vpar_list[j]);
                 vpar_is_isomorphic(vpar_list[i],vpar_list[j])));
     print("  matdet ",matdet(m));

     for(i=1,#vpar_list,      \\ reflexive
        if(!m[i,i],print("oops")));

     m==m~ || print("oops");
     for(i=1,#vpar_list,      \\ symmetric
        for(j=1,#vpar_list,
           if(m[i,j] != m[j,i], print("oops"))));

     if(n<=4,
        print("  check transitive");
        for(i=1,#vpar_list,
           progress(Str("    i="i));
           for(j=1,#vpar_list,
              if(!m[i,j], next());
              for(k=1,#vpar_list,
                 if(m[i,j] && m[j,k],
                    if(!m[i,k], print("oops"))))));
       );

     \\ print matrix
     if(n<=4,
        for(i=1,#vpar_list,
           print1("  ");
           for(j=1,#vpar_list,
              print1(m[i,j]));
           print()));

     print("  isomorphism class representatives");
     my(rep=vector(#vpar_list),
        rep_upto=0,
        class=vector(#vpar_list));
     for(i=1,#vpar_list,
        for(j=1,rep_upto,
           if(m[i,rep[j]],  \\ i isomorphic to rep[j]
              if(n<=4,
                 print("  "i" class "j"  "vpar_list[i]" == "vpar_list[rep[j]]));
              class[i]=j;
              next(2)));    \\ next i

        \\ new class
        class[i]=rep_upto++; \\ pre-increment
        rep[rep_upto]=i;
        if(n<=4,
           print("  "i" new class " rep_upto"  "vpar_list[i]);
           \\ vpar_view(vpar_list[i]);
          );
     );

     print("  num classes "rep_upto);
     listput(num_classes_list,rep_upto);
     );

  print(Vec(num_classes_list));
}
quit

\\-----------------------------------------------------------------------------
\\ vpar_seq_rows() uniqueness
\\
\\ 1,1,2,6,24,120

{
  for(n=1,6,
     print("n="n);

     my(m=Map());
     forvec(vec=vpar_forvec_from(n,-1),  \\ trees root=1
           my(vpar=vpar_from_upascend(vec),
              seq=vpar_seq_rows(vpar),
              rowsvec=vecextract(vpar,vpar_seq_rows(vpar)),
              count=0);
           mapisdefined(m,rowsvec,&count);
           mapput(m,rowsvec,count+1));
     my(v=Mat(m)[,1]);
     v=vecsort(v);
     v=vecsort(v, (x,y)->lex([mapget(m,x),x],[mapget(m,y),y]));
     print("  ",#v," distinct");
     for(i=1,#v,
        my(rowsvec=v[i],
           count=mapget(m,rowsvec));
        print("  "Vec(rowsvec)," count "count));

     v=Mat(m)[,2];
     my(hist=vector(vecmax(v)));
     for(i=1,#v, hist[v[i]]++);
     for(i=1,#hist, if(hist[i],print("  count "i" occurs "hist[i])));
     );
}
quit

\\-----------------------------------------------------------------------------
\\ vpar_seq_rows() duplications
\\

{
  for(n=7,7,
     print("n="n);

     my(m=Map());
     forvec(vec=vpar_forvec_from(n,-1),  \\ trees root=1
           my(vpar=vpar_from_upascend(vec),
              seq=vpar_seq_rows(vpar),
              rowsvec=vecextract(vpar,vpar_seq_rows(vpar)),
              count=0);
           if(!mapisdefined(m,rowsvec),mapput(m,rowsvec,[]));
           my(v=mapget(m,rowsvec));
           mapput(m,rowsvec,concat(v,[vpar])));
     my(v=Mat(m)[,1]);
     v=vecsort(v);
     v=vecsort(v, (x,y)->lex([#mapget(m,x),x],[#mapget(m,y),y]));
     for(i=1,#v,
        my(rowsvec=v[i],
           vpar_list=mapget(m,rowsvec));
        if(#vpar_list<2,next());
        print("rowsvec "Vec(rowsvec));
        for(j=1,#vpar_list,
           my(vpar=vpar_list[j],
              height=vpar_height(vpar));
           print("   vpar "vpar"  h="height));

        my(iso=matrix(#vpar_list,#vpar_list),
           fiso=iso,
           all_fiso=1);
        for(i=1,#vpar_list,
           for(j=1,#vpar_list,
              my(is=vpar_is_isomorphic(vpar_list[i],vpar_list[j]));
              iso[i,j]=is;
              print1(" ",is));
           print1("     ");
           for(j=1,#vpar_list,
              my(is=vpar_is_isomorphic(vpar_list[i],vpar_list[j],1));
              fiso[i,j]=is;
              all_fiso = all_fiso&&is;
              print1(" ",is));
           print());
        if(iso==fiso,
           print("  same isomorphisms"),
           print("  different",
                if(all_fiso,"",", not free iso")));
     );
  print());
}
quit


\\-----------------------------------------------------------------------------
\\ number of subtree ids vs height

vpar_automorphism_ids_vector([0, 1, 1, 2, 3, 3])
vpar_automorphism_ids_vector([0,1,1,1,2,3,3])
vpar_automorphism_ids_vector([0,1,4,1,2,3,3])

{
  my(num_classes_list=List([]));
  for(n=8,9,
     print("n="n);

     my(m=Map());
     forvec(vec=vpar_forvec_from(n,-1),  \\ trees root=1
           my(vpar=vpar_from_upascend(vec),
              height=vpar_height(vpar),
              num_ids=vpar_num_automorphism_ids(vpar),
              elem=[height,num_ids]);
           if(!mapisdefined(m,elem),
              mapput(m,elem,0);
              my(delta = elem[2]-elem[1]);
              if(delta>1,
                 print("delta "delta" "elem" vpar "vpar);
                 vpar_view(vpar));
             );
           mapput(m,elem,mapget(m,elem)+1);
           );
     my(v=vecsort(Mat(m)[,1]));
     for(i=1,#v,
        my(elem=v[i],
           delta = elem[2]-elem[1]);
        print(elem," delta "delta"  count ",mapget(m,elem)));
     );
}
quit


\\-----------------------------------------------------------------------------
\\              4      3
\\             /      /
\\            1      2
\\
\\ seq [1, 2, 3, 4]  ahead=0
\\ seq [1, 4, 2, 3]  ahead=1
\\ extract [4, 3, 0, 0]
\\ extract [4, 0, 3, 0]

vpar=[4,3,0,0]
vpar_validate(vpar);
vpar_seq_upascend(vpar,0)
vpar_seq_upascend(vpar,1)
vecextract(vpar,vpar_seq_upascend(vpar,0))
vecextract(vpar,vpar_seq_upascend(vpar,1))

\\ vpar [4, 3, 0, 0] ahead=0
\\ vpar [4, 0, 0, 3] ahead=1
print("vec");
vec=[4,3,0,0]
vpar_from_upascend(vec,0)
vpar_from_upascend(vec,1)

\\ vpar [4, 0, 0, 3] ahead=0
\\ vpar [4, 3, 0, 0] ahead=1
print("vec");
vec=[4,0,3,0]
vpar_from_upascend(vec,0)
vpar_from_upascend(vec,1)
quit;


\\-----------------------------------------------------------------------------
\\              4
\\             /
\\            1
\\           / \
\\          2   3

vpar=[4,1,1,0]
vpar_validate(vpar);
\\vpar_view(vpar);
vpar_seq_upascend(vpar,1)

vec=[1, 1, 4, 0]
vpar_from_upascend(vec,1)
quit



\\-----------------------------------------------------------------------------

vpar=[3,3,10,10,9,8,8,9,10, 0];
\\ vpar_view(vpar)
vpar_seq_uplexqueue(vpar)

vpar_view(vpar_relabel_perm(vpar,vpar_seq_uplexqueue(vpar)^-1))
quit
vpar=vpar_relabel_random(vpar);
vpar_relabel_perm(vpar,vpar_seq_uplexqueue(vpar)^-1)
vpar=vpar_relabel_random(vpar);
vpar_relabel_perm(vpar,vpar_seq_uplexqueue(vpar)^-1)



\\-----------------------------------------------------------------------------

vpar=[3,3,10,10,9,8,8,9,10, 0];
\\ vpar_view(vpar)
vpar_seq_preorder(vpar,1)

quit


\\-----------------------------------------------------------------------------
\\ vpar_bandwidth() of star

{
  for(n=1,10,
     my(vpar=vpar_make_star(n));
     print("n="n"  ",vpar_dilation(vpar),"  ",vpar_bandwidth(vpar)));
}
quit


\\-----------------------------------------------------------------------------

vpar_subtree_vertices(vpar_make_path(2),1)

vpar_subtree_vertices(vpar_make_path(10),4)
[4,5,6,7,8,9,10]
quit

     \\ print(c" to "p);
        \\ print("old chain c "chain[c]);
        \\ print("old tail "chain[tail[c]]);
        \\ print("new chain p "chain[p]);
  \\ print("chain "chain);


\\-----------------------------------------------------------------------------
\\ vpar_INTERNAL_seq_upwards() same as vpar_seq_upascend()


\\ fromleaf == postorder
\\ not in OEIS: 1, 2, 9, 64, 585, 6576, 88375, 1386512

\\ fromleaf == canondown rev
\\ not in OEIS: 1, 2, 9, 40, 245, 1836, 16135,  162352

\\ postorder == canondown rev
\\ not in OEIS: 1, 2, 9, 40, 245, 1896, 17521,  186880

\\ cf total
\\ vector(8,n, n^(n-1))
\\              1, 2, 9, 64, 625, 7776, 117649, 2097152

{
  my(max_n=7,
     names=["fromleaf","postorder","canondown rev"],
     counts=matrix(#names,#names,i,j,vector(max_n)));
  for(n=1,max_n,
     forvec(can=vpar_forvec_from(n),
           my(vpars=[vpar_from_upascend(can),
                     vpar_from_upascend(can),
                     vpar_from_canondown(Vecrev(can))]);
           for(i=1,#names,
              for(j=1,#names,
                 counts[i,j][n]+=(vpars[i]==vpars[j])))));
  for(i=1,#names,
     for(j=i+1,#names,
        print(names[i]" == "names[j]);
        print("  "counts[i,j])));
}
quit

\\-----------------------------------------------------------------------------
\\ vpar_seq_upqueue()

{
  my(n=5);
  forvec(can=vector(n,i,if(i==n,[0,0],[1,n])),
        my(vpar=vpar_from_upascend(can));
        vpar_validate(vpar);
        print("vpar "vpar);
        my(seq=vpar_INTERNAL_seq_upwards(vpar));
        my(seq2=vpar_seq_upqueue(vpar));
        print("  "Vec(seq));
        print("  "Vec(seq2)"  "if(seq!=seq2,"different"));
  );
}
quit

\\-----------------------------------------------------------------------------
\\ trees with root=1
\\ second last of seq postorder is the root

{
  my(n=5);
  forvec(can=vector(n,i,if(i==n,[0,0],[1,n])),
        my(vpar=vpar_from_upascend(can));
        if(vpar_roots(vpar)==[1],
           print(can"   vpar="vpar)));
}
quit

\\-----------------------------------------------------------------------------
\\ vpar_from_upascend()

vpar_INTERNAL_seq_upwards([2,0])
quit
vpar_from_upascend([])
vpar_from_upascend([0])
vpar_from_upascend([0,0])    \\ forest two isolated
vpar_from_upascend([0,1])    \\ path-2 [0,1]
vpar_from_upascend([0,2])    \\ path-2 [1,0]
vpar_from_upascend([1,0])
vpar_from_upascend([1,1])
quit


\\-----------------------------------------------------------------------------
\\ vpar_INTERNAL_seq_upwards() uniqueness

{
  my(m=Map());
  forvec(can=vpar_forvec_from(6),
        my(vpar=vpar_from_upascend(can));
        my(post=vecextract(vpar,vpar_INTERNAL_seq_upwards(vpar)));
        if(mapisdefined(m,post),
           print("repeat "post),
           mapput(m,post,1)));
}
quit

\\-----------------------------------------------------------------------------

Vecsmall([9,10,11])*Vecsmall([1,3,2])
Vecsmall([2,3,4,1])^2
Vecsmall([2,3,4,1])^-1
\\ Vecsmall([9,10,11])^Vecsmall([2,3,4,1])
quit

\\-----------------------------------------------------------------------------

vpar=[3,3,10,10,9,8,8,9,10, 0]
vecextract(vpar,vpar_seq_childless_paths(vpar))
vpar_from_canondown([0, 10, 3, 3, 10, 10, 9, 9, 8, 8])
Vecsmall(vpar)*Vecsmall(vpar_seq_childless_paths(vpar))
vpar_seq_childless_paths(vpar)
v2=vpar_relabel_perm(vpar,vpar_seq_childless_paths(vpar))
vpar_seq_childless_paths(v2)
v2=vpar_relabel_perm(vpar,Vecsmall(vpar_seq_childless_paths(vpar))^-1)
vpar_seq_childless_paths(v2)
quit

\\-----------------------------------------------------------------------------
\\ no apply or select on Vecsmall

apply(n->n==2, Vecsmall([1,2,3]));
select(n->n==2, Vecsmall([1,2,3]));
quit;

\\-----------------------------------------------------------------------------
vpar=vpar_make_star(5);
vpar=[4, 3, 4, 0];
vpar
vpar_matchnum_and_count(vpar)
quit


\\-----------------------------------------------------------------------------
{
  my(n=4,count_cyclic=0,count_acyclic=0,count_acyclic_trees=0);
  forvec(vpar=vector(n,i,[0,n]),
        my(cyclic=vpar_is_cyclic(vpar));
        my(num_roots=vpar_num_roots(vpar));
        count_cyclic+=cyclic;
        count_acyclic+=!cyclic;
        count_acyclic_trees += !cyclic && num_roots==1;
        if (!cyclic,
            print("vpar "vpar" cyclic "cyclic"  num_roots "num_roots);
           );
  );
  print("cyclic  "count_cyclic);
  print("acyclic "count_acyclic);
  print("acyclic trees "count_acyclic_trees);
}
\\ n=5
\\ GP-Test  6^5 == 7776
\\ GP-Test  6^4 == 1296    \\ acyclics
\\
\\ n=4
\\ 500 cyclics
\\ GP-Test  5^3 == 125     \\ acyclics
\\ GP-Test  4^3 == 64      \\ acyclic trees
\\
\\ n=3
\\ GP-Test  4^2 == 16      \\ acyclic forests
\\ GP-Test  3^2 == 9       \\ acyclic trees

quit

\\-----------------------------------------------------------------------------

canonical_to_number(vec) =
{
  if(#vec==0,0,
     #vec==1,1,
     fromdigits(vec,#vec));
}
number_to_canonical(canum) =
{
  for(n=1,canum,
     if(canum%n==0,
        return(digits(canum,n))));
  error("oops");
}

to_canonical(vpar) = vecextract(vpar,vpar_seq_upascend(vpar));
vector(10,n, canonical_to_number(to_canonical(vpar_make_star(n)))/n)
\\ 0, 1, 4, 21, 156, 1555, 19608, 299593, 5380840, 111111111
\\ A060072 a(n) = (n^(n-1) - 1)/(n-1)

vector(10,n, canonical_to_number(to_canonical(vpar_make_star(n))))
\\ 1, 2, 12, 84, 780, 9330, 137256, 2396744, 48427560, 1111111110
\\ A226238 (n^n - n)/(n - 1)

vector(10,n, canonical_to_number(to_canonical(vpar_make_path(n)))/n)
\\ A051846 base n ascending digits
\\ 0, 1, 7, 57, 586, 7465, 114381, 2054353, 42374116, 987654321

vector(10,n, canonical_to_number(to_canonical(vpar_make_path(n))))
\\ A062813 Sum i*n^i, i = 0..n-1.
\\ 1, 2, 21, 228, 2930, 44790, 800667, 16434824, 381367044, 9876543210

print("complete_tree 2");
vector(5,n, canonical_to_number(to_canonical(vpar_make_complete_tree(2,n)))/n)
\\ not in OEIS: 90832, 31215547406949360, 909901382005270846922394177520983267761419008

vector(5,n, canonical_to_number(to_canonical(vpar_make_complete_tree(2,n))))
\\ not in OEIS: 272496, 124862189627797440, 4549506910026354234611970887604916338807095040

vector(10,n, canonical_to_number(to_canonical(vpar_make_binomial_tree(n)))/n)
\\ not in OEIS: 4, 29, 206, 2011, 24634, 367593, 6474664, 131575191

vector(10,n, canonical_to_number(to_canonical(vpar_make_binomial_tree(n))))
\\ not in OEIS: 12, 116, 1030, 12066, 172438, 2940744, 58271976, 1315751910



vpar=[3,3,10,10,9,8,8,9,10,0];
vpar=[6,6,0,3,3,3];
vpar=[2,0,2,2,2];
vpar=[3,3,0,3,3];
vpar=[4,4,4,0,4];
vpar=[0,1,1,1,1];
vpar=[5,5,5,5,0];
print("vpar "vpar);
vec=to_canonical(vpar);
print("vec  "vec);
\\vec=[0, 1, 1, 1, 1]
vpar_from_upascend(vec)

{
  forvec(can=[[1,3],[1,3],[1,3]],
        my(vec=concat(can,[0]));
        my(vpar=vpar_from_upascend(vec));
        vpar_validate(vpar);
        print("vec  ",vec"  vpar "vpar" root ",vpar_root(vpar));
        print("back " to_canonical(vpar));
        print());
}
quit;
{
  forvec(can=[[1,4],[1,4],[1,4],[1,4]],
        my(vec=concat(can,[0]));
        print("vec  ",vec);
        my(vpar=vpar_from_upascend(vec));
        print("back " to_canonical(vpar));
        print());
}
quit;

xvec_to_V(xvec) =
{
  my(n=#xvec,
     V_list=List([]));
  for(i=1,n-1,
     my(xs=Set(concat(xvec[i..n-1],Vec(V_list))));print(xs);
     for(V=1,n,
        if(!setsearch(xs,V),
           listput(V_list,V);
           break())));
  Vec(V_list);
}

xvec_to_V([3,3,10,10,9,8,8,9,10,0])
\\         1,2, 3, 4,5,6,7,8, 9      edge to x


xvec_to_V([6,6,0,3,3,3,4])
xvec_to_V([6,6,0,3,3,3])

\\
print("seven"); \\ [0, 1, 2, 2, 4, 3, 5];
vpar=vpar_make_random(7)
vpar=vector(7,i,random(7)+1)
print(xvec_to_V(vpar));print(vpar);
print("vpar_seq_upascend");
print(vpar_seq_upascend(vpar));
print(vpar_INTERNAL_seq_upwards(vpar));
quit;


\\print(numtoperm(2,random(2!)));
n=8
{
for(i=0,5, vpar=vpar_make_random(n);
   print(vpar);
   \\ print(Vecrev(vpar_INTERNAL_seq_upwards(vpar)));
);
}
vpar_validate(vpar);
\\for(i=0,5, print(numtoperm(n,random(n!))));
Vecsmall([8, 3, 7, 4, 2, 6, 5, 1])^-1
v=[8, 3, 7, 4, 2, 6, 5, 1]
u=vector(#v);
for(i=1,#v,u[v[i]]=i);
print(u);
vpar_INTERNAL_seq_upwards([0, 1, 2, 3, 1, 1, 3, 1])
quit

vpar=vpar_make_star(4);
print(vpar);
vpar=vpar_star_replacement(vpar,4,0);
print(vpar);
vpar=vpar_star_replacement(vpar,4,0);
print(vpar);
vpar_view(vpar);
quit

vpar=vpar_make_path(3);
vpar=vpar_make_star(5);
print(vpar);
vpar=vpar_star_replacement(vpar,5,2);
print(vpar);
vpar_view(vpar);
quit

vpar_view(vpar_make_bistar(4,5));
quit

vpar=vpar_make_complete_tree(2,4);
print(vpar_children_vector(vpar));
print("depths   ",vpar_depths_vector(vpar));
print();
\\ vpar_view(vpar);
\\ vpar_view(vpar_make_star(6));

vpar=vpar_make_path(1);
\\ print(vpar);
\\ vpar_view(vpar);
vpar=vpar_star_replacement(vpar,5);
print(vpar);
\\ vpar_view(vpar);
vpar=vpar_star_replacement(vpar,5);
print(vpar);
\\ vpar_view(vpar);
vpar_traverse_postorder(vpar,v->print("visit "v));
print("num_children    ",vpar_num_children_vector(vpar));
print("list_postorder ",vpar_INTERNAL_seq_upwards(vpar));
print("diameter ",vpar_diameter(vpar));
print("depths   ",vpar_depths_vector(vpar));

\\             1 2 3 4
vpar_validate([0,3,4,1,1]);
vpar_validate([4,1,2,3]);

quit
{
  vpar=vpar_make_path(1);
  for(i=1,7,
     print("diameter ",vpar_diameter(vpar));
     \\ vpar_view(vpar);
     vpar=vpar_star_replacement(vpar,5);
     );
}

\\-----------------------------------------------------------------------------
install(getpid, "l", "getpid");
system(Str("memstat -p ",getpid()));
quit

\\-----------------------------------------------------------------------------
